
#ifndef _FATTR_HXX
#define _FATTR_HXX

#include <tools/fsys.hxx>
#include <string.h>

#if defined (WNT) || defined (OS2)
#include <sys/types.h>
#include <sys/stat.h>
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#endif

#define FA_NORMAL	0x00000000L
#ifndef UNX
#define	FA_RDONLY	00444
#else
#define	FA_RDONLY	S_IREAD 
#endif
#define FA_HIDDEN	0x00000002L
#define	FA_SYSTEM	0x00000004L
#ifndef UNX
#define	FA_WRITE	S_IREAD | S_IWRITE 
#else
#define	FA_WRITE	00777
#endif
#define	FA_SUBDIR	0x00000010L
#define	FA_ARCH		0x00000020L

#if defined(WNT)
#define chmod	_chmod
#endif

struct FileAttribute_Impl;

class FileAttribute : public FileStat
{
	DirEntry			theEntry;
	char				*pFileName;
	FileAttribute_Impl	*pImpl;

protected:
	FileAttribute_Impl* GetAttributes();
	void				SetAttributes( FileAttribute_Impl * );
public:
			FileAttribute();
			FileAttribute( const DirEntry& rDirEntry );
			FileAttribute( const FileAttribute &rFattr );
			~FileAttribute();

	void				SetName( const DirEntry &rEntry );
	void				SetName( const String &rName );
	DirEntry			GetDirEntry();

	BOOL				Attrib( ULONG nAttr );
	BOOL				Touch();
	BOOL				Delete();
	BOOL				Touch( String aRefFile );
	BOOL				Touch( Date &rModDate, Time &rModTime );
	BOOL				Touch( Date &rModDate, Time &rModTime, Date &rAccDate, Time &rAccTime );
	ULONG				GetFlags();

	void				LowerName();
	void				LowerName(char*);
	static void			Error();
};

struct FileCloner_Impl;

class FileCloner
{
	DirEntry			aSource;
	DirEntry			aTarget;
	ULONG				nBytesTotal;
	ULONG				nBytesCopied;
	Link				aProgressLink;
	USHORT				nBlockSize;
	ULONG				nProgress;
	BOOL				bUpdateCopy;
	BOOL				bDiffCopy; 	
	BOOL				bCopied;	
	FileCloner_Impl*	pImp;

private:
	FSysError			DoCopy_Impl( const DirEntry &rSource,
					const DirEntry &rTarget, BOOL bTop );
	BOOL				DoCompare_Impl( const DirEntry &rSource,
					const DirEntry &rTarget );

protected:
	virtual BOOL		Progress();
	virtual ErrCode		Error( ErrCode eErr,
				   const DirEntry *pSource, const DirEntry *pTarget );
	void				Begin();
	void				End();
public:
			FileCloner();
			FileCloner( const DirEntry &rSource,
			            const DirEntry &rTarget );
			FileCloner( const FileCloner &rCloner );

	FileCloner&			operator = ( const FileCloner &rCloner );

	void				SetBlockSize( USHORT nBytes ) { nBlockSize = nBytes; }
	USHORT				GetBlockSize() const { return nBlockSize; }

	ULONG				GetBytesTotal() const { return nBytesTotal; }
	ULONG				GetBytesCopied() const { return nBytesCopied; }

	void				SetSource( const DirEntry &rSource ) { aSource = rSource; }
	void				SetTarget( const DirEntry &rTarget ) { aTarget = rTarget; }
	const DirEntry&		GetSource() const { return aSource; }
	const DirEntry&		GetTarget() const { return aTarget; }

	void				SetUpdateCopy( BOOL bUpdate = TRUE ){bUpdateCopy =
													bUpdate; }
	void				SetDiffCopy( BOOL bDiff = TRUE ){bDiffCopy =
													bDiff; }
	FSysError			Execute( FSysAction nActions = FSYS_ACTION_STANDARD );

	BOOL				Compare();

	const DirEntry*		GetErrorSource() const;
	const DirEntry*		GetErrorTarget() const;
	ErrCode				GetError() const;
	BOOL				Copied(){ return bCopied;}

	void				SetProgressHdl( const Link& rLink ) { aProgressLink = rLink; }
	const Link&			GetProgressHdl() const { return aProgressLink; }
	void				SetErrorHdl( const Link& rLink );
	const Link&			GetErrorHdl() const;~FileCloner();
};

#endif
