

#ifndef _MK_CREATE_HXX
#define _MK_CREATE_HXX

#include <tools/string.hxx>

#ifndef _SSTRING_HXX
#include <tools/sstring.hxx>
#endif

#include <tools/list.hxx>

#ifndef _PRJ_HXX
#include <tools/prj.hxx>
#endif

class SvStream;
class SourceDirectoryList;

class CodedDependency : public ByteString
{
private:
	USHORT nOSType;							

public:

	CodedDependency(
		const ByteString &rCodedIdentifier, 
		USHORT nOperatingSystems			
	) :
	ByteString( rCodedIdentifier ),
	nOSType( nOperatingSystems )
	{
	}

	USHORT GetOperatingSystem()
	{
		return nOSType;
	}

	void SetOperatingSystem( USHORT nOperatingSystems )
	{
		nOSType = nOperatingSystems;
	}

	BOOL TryToMerge(
		const ByteString &rCodedIdentifier, 
		USHORT nOperatingSystems			
	)
	{
		if ( rCodedIdentifier != *this )
			return FALSE;
		nOSType |= nOperatingSystems;
		return TRUE;
	}
};

class Dependency : public ByteString
{
private:
	USHORT nOSType;							

public:

	Dependency(
		const ByteString &rDirectoryName, 	
		USHORT nOperatingSystems			
	) :
	ByteString( rDirectoryName ),
	nOSType( nOperatingSystems )
	{
	}

	USHORT GetOperatingSystem()
	{
		return nOSType;
	}
};

class SourceDirectory : public ByteString
{
private:
	SourceDirectory *pParent;				
	SourceDirectoryList *pSubDirectories;	
	USHORT nOSType;							
	USHORT nDepth;							

	SByteStringList *pDependencies;			

	SByteStringList *pCodedDependencies;   	
	SByteStringList *pCodedIdentifier;     	

	Dependency *ResolvesDependency(
		CodedDependency *pCodedDependency 	
	);

	static USHORT SourceDirectory::GetOSType(
		const ByteString &sDependExt 		
	);

	BOOL RemoveDirectoryTreeAndAllDependencies();

public:

	SourceDirectory(
		const ByteString &rDirectoryName, 			
		USHORT nOperatingSystem,					
		SourceDirectory *pParentDirectory = NULL 	
	);
	~SourceDirectory();

	ByteString GetFullPath();

	SourceDirectoryList *GetSubDirectories() { return pSubDirectories; }

	USHORT GetOperatingSystems() { return nOSType; }

	SourceDirectory *GetDirectory(
		const ByteString &rDirectoryName,	
		USHORT nOperatingSystem				
	);

	SourceDirectory *InsertFull(
		const ByteString &rDirectoryName,	
		USHORT nOperatingSystem				
	)
	{
		return GetDirectory( rDirectoryName, nOperatingSystem );
	}

	SourceDirectory *Insert(
		const ByteString &rDirectoryName,	
		USHORT nOperatingSystem				
	);

	SourceDirectory *GetRootDirectory();

	SourceDirectory *GetSubDirectory(
		const ByteString &rDirectoryPath,	
		USHORT nOperatingSystem				
	);

	CodedDependency *AddCodedDependency(
		const ByteString &rCodedIdentifier, 
		USHORT nOperatingSystems			
	);

	SByteStringList *GetCodedDependencies()
	{
		return pCodedDependencies;
	}

	CodedDependency *AddCodedIdentifier(
		const ByteString &rCodedIdentifier, 
		USHORT nOperatingSystems			
	);

	SByteStringList *GetCodedIdentifier()
	{
		return pCodedIdentifier;
	}

	void ResolveDependencies();

	ByteString GetTarget();

	ByteString GetSubDirsTarget();

	static SourceDirectory *CreateRootDirectory(
		const ByteString &rRoot, 	
		const ByteString &rVersion,	
		BOOL bAll = FALSE			
	);

	BOOL CreateRecursiveMakefile(
		BOOL bAllChilds = FALSE 	
	);
};

class SourceDirectoryList : public SByteStringList
{
public:

	SourceDirectoryList()
	{
	}
	~SourceDirectoryList();

	SourceDirectory *Search(
		const ByteString &rDirectoryName	
	);

	ULONG InsertSorted(
		SourceDirectory *pDirectory 	
	)
	{
		return PutString(( ByteString * ) pDirectory );
	}
};

#endif
