

#ifndef _DBHELPER_DBCHARSET_HXX_
#define _DBHELPER_DBCHARSET_HXX_

#ifndef _COMPHELPER_STLTYPES_HXX_
#include <comphelper/stl_types.hxx>
#endif
#ifndef _RTL_TEXTENC_H 
#include <rtl/textenc.h>
#endif
#include <rtl/ustring.hxx>

namespace dbtools
{

	class OCharsetMap
	{
	protected:
		DECLARE_STL_VECTOR(rtl_TextEncoding, TextEncVector);
		TextEncVector	m_aEncodings;
		DECLARE_STL_VECTOR(::rtl::OUString, StringVector);
		StringVector	m_aIanaNames;

		StringVector	m_aNames;

	#ifdef _DEBUG
		sal_Int32		m_nLivingIterators;			
	#endif

	public:
		class CharsetIterator;
		friend class OCharsetMap::CharsetIterator;
		typedef CharsetIterator iterator;
		typedef CharsetIterator const_iterator;

		OCharsetMap();
		~OCharsetMap();

		struct Logical { };
		struct IANA { };

		CharsetIterator	find(const rtl_TextEncoding _eEncoding) const;

		CharsetIterator	find(const ::rtl::OUString& _rIanaName, const IANA&) const;

		CharsetIterator	find(const ::rtl::OUString& _rLogicalName, const Logical&) const;

		sal_Int32	size() const { return m_aNames.size(); }

		CharsetIterator	begin() const;

		CharsetIterator	end() const;
	};

	class CharsetIteratorDerefHelper
	{
		friend class OCharsetMap::CharsetIterator;

		rtl_TextEncoding	m_eEncoding;
		::rtl::OUString		m_aIanaName;
		::rtl::OUString		m_aName;

	public:
		CharsetIteratorDerefHelper(const CharsetIteratorDerefHelper& _rSource);

		rtl_TextEncoding	getEncoding() const	{ return m_eEncoding; }
		::rtl::OUString		getIanaName() const	{ return m_aIanaName; }
		::rtl::OUString		getName() const		{ return m_aName; }

	protected:
		CharsetIteratorDerefHelper();
		CharsetIteratorDerefHelper(const rtl_TextEncoding _eEncoding, const ::rtl::OUString& _rIanaName, const ::rtl::OUString& _rName);

	};

	class OCharsetMap::CharsetIterator
	{
		friend class OCharsetMap;

		friend bool operator==(const CharsetIterator& lhs, const CharsetIterator& rhs);
		friend bool operator!=(const CharsetIterator& lhs, const CharsetIterator& rhs) { return !(lhs == rhs); }

		friend sal_Int32 operator-(const CharsetIterator& lhs, const CharsetIterator& rhs);

	protected:
		const OCharsetMap*	m_pContainer;
		sal_Int32			m_nPosition;

	public:
		CharsetIterator(const CharsetIterator& _rSource);
		~CharsetIterator();

		CharsetIteratorDerefHelper operator*() const;

		const CharsetIterator&	operator++();

		const CharsetIterator	operator++(int) { CharsetIterator hold(*this); ++*this; return hold; }

		const CharsetIterator&	operator--();

		const CharsetIterator	operator--(int) { CharsetIterator hold(*this); --*this; return hold; }

	protected:
		CharsetIterator(const OCharsetMap* _pContainer, sal_Int32 _nInitialPos = 0);
	};

}	

#endif 

