

#ifndef _DBHELPER_DBEXCEPTION_HXX_
#define _DBHELPER_DBEXCEPTION_HXX_

#ifndef _COM_SUN_STAR_SDBC_SQLEXCEPTION_HPP_
#include <com/sun/star/sdbc/SQLException.hpp>
#endif

namespace com
{
	namespace sun
	{
		namespace star
		{
			namespace sdb
			{
	class SQLContext;
	struct SQLErrorEvent;
			}
			namespace sdbc
			{
	class SQLWarning;
			}				 
		}
	}
}

namespace dbtools
{

class SQLExceptionInfo
{
public:
	enum TYPE { SQL_EXCEPTION, SQL_WARNING, SQL_CONTEXT, UNDEFINED };

private:
	::com::sun::star::uno::Any	m_aContent;
	TYPE			m_eType;	

public:
	SQLExceptionInfo();
	SQLExceptionInfo(const ::com::sun::star::sdbc::SQLException& _rError);
	SQLExceptionInfo(const ::com::sun::star::sdbc::SQLWarning& _rError);
	SQLExceptionInfo(const ::com::sun::star::sdb::SQLContext& _rError);

	SQLExceptionInfo(const SQLExceptionInfo& _rCopySource);

	SQLExceptionInfo(const ::com::sun::star::sdb::SQLErrorEvent& _rError);

	SQLExceptionInfo(const ::com::sun::star::uno::Any& _rError);

	const SQLExceptionInfo& operator=(const ::com::sun::star::sdbc::SQLException& _rError);
	const SQLExceptionInfo& operator=(const ::com::sun::star::sdbc::SQLWarning& _rError);
	const SQLExceptionInfo& operator=(const ::com::sun::star::sdb::SQLContext& _rError);

	sal_Bool	isKindOf(TYPE _eType) const;

	sal_Bool	isValid() const { return m_eType != UNDEFINED; }
	TYPE		getType() const { return m_eType; }

	operator const ::com::sun::star::sdbc::SQLException*	() const;
	operator const ::com::sun::star::sdbc::SQLWarning*	() const;
	operator const ::com::sun::star::sdb::SQLContext*		() const;

	::com::sun::star::uno::Any get() const { return m_aContent; }

protected:
	void implDetermineType();
};

class SQLExceptionIteratorHelper
{
public:

	enum NODES_INCLUDED { NI_EXCEPTIONS, NI_WARNINGS, NI_CONTEXTINFOS };

protected:
	const ::com::sun::star::sdbc::SQLException*	m_pCurrent;
	SQLExceptionInfo::TYPE			m_eCurrentType;

	NODES_INCLUDED					m_eMask;

public:
	SQLExceptionIteratorHelper(const ::com::sun::star::sdbc::SQLException* _pStart, NODES_INCLUDED _eMask = NI_EXCEPTIONS);
	SQLExceptionIteratorHelper(const ::com::sun::star::sdbc::SQLWarning* _pStart, NODES_INCLUDED _eMask = NI_EXCEPTIONS);
	SQLExceptionIteratorHelper(const ::com::sun::star::sdb::SQLContext* _pStart, NODES_INCLUDED _eMask = NI_EXCEPTIONS);

	SQLExceptionIteratorHelper(const SQLExceptionInfo& _rStart, NODES_INCLUDED _eMask = NI_EXCEPTIONS);

	sal_Bool									hasMoreElements() const { return (m_pCurrent != NULL); }
	const ::com::sun::star::sdbc::SQLException*	next();
	void										next(SQLExceptionInfo& _rOutInfo);
};

void throwFunctionSequenceException(const ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >& _Context,
		const ::com::sun::star::uno::Any& _Next = ::com::sun::star::uno::Any()) throw ( ::com::sun::star::sdbc::SQLException );

void throwInvalidIndexException(const ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >& _Context,
		const ::com::sun::star::uno::Any& _Next = ::com::sun::star::uno::Any()) throw ( ::com::sun::star::sdbc::SQLException );

void throwGenericSQLException(const ::rtl::OUString& _rMsg, const ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >& _rxSource)
	throw (::com::sun::star::sdbc::SQLException);

void throwGenericSQLException(
			const ::rtl::OUString& _rMsg,
			const ::com::sun::star::uno::Reference< ::com::sun::star::uno::XInterface >& _rxSource,
			const ::com::sun::star::uno::Any& _rNextException
			)
	throw (::com::sun::star::sdbc::SQLException);

}	

#endif _DBHELPER_DBEXCEPTION_HXX_

