
#ifndef _CONNECTIVITY_PARSE_SQLITERATOR_HXX_
#define _CONNECTIVITY_PARSE_SQLITERATOR_HXX_

#ifndef _CONNECTIVITY_SQLNODE_HXX
#include "connectivity/sqlnode.hxx"
#endif
#ifndef _COM_SUN_STAR_SDBCX_XTABLESSUPPLIER_HPP_
#include <com/sun/star/sdbcx/XTablesSupplier.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBCX_XCOLUMNSSUPPLIER_HPP_
#include <com/sun/star/sdbcx/XColumnsSupplier.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_XDATABASEMETADATA_HPP_
#include <com/sun/star/sdbc/XDatabaseMetaData.hpp>
#endif
#ifndef _COM_SUN_STAR_SDBC_SQLWARNING_HPP_
#include <com/sun/star/sdbc/SQLWarning.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif
#ifndef _MAP_
#include <map>
#endif
#ifndef _CONNECTIVITY_COMMONTOOLS_HXX_
#include "connectivity/CommonTools.hxx"
#endif
#ifndef _VOS_REF_HXX_
#include <vos/ref.hxx>
#endif
#ifndef _CPPUHELPER_WEAK_HXX_
#include <cppuhelper/weak.hxx>
#endif

namespace connectivity
{

	class OSQLParseNode;

	enum OSQLStatementType {
		SQL_STATEMENT_UNKNOWN,
		SQL_STATEMENT_SELECT,
		SQL_STATEMENT_INSERT,
		SQL_STATEMENT_UPDATE,
		SQL_STATEMENT_DELETE,
		SQL_STATEMENT_ODBC_CALL,
		SQL_STATEMENT_SELECT_COUNT
	};

	enum OSQLPredicateType {
		SQL_PRED_EQUAL = 'a',		
		SQL_PRED_NOTEQUAL,			
		SQL_PRED_LESS,				
		SQL_PRED_LESSOREQUAL,
		SQL_PRED_GREATER,
		SQL_PRED_GREATEROREQUAL,
		SQL_PRED_LIKE,
		SQL_PRED_ISNULL,
		SQL_PRED_ISNOTNULL,
		SQL_PRED_NOTLIKE
	};

	struct OSQLParseIteratorErrorInfo
	{
		sal_uInt16		nErrorCode;		
		::rtl::OUString		aExpression;	
	};

	#define RET_CONTINUE	1	
	#define RET_HANDLED		2	
	#define	RET_BREAK		3	

	class OSQLParseTreeIterator
	{
	public:

	private:
		::com::sun::star::sdbc::SQLWarning	m_aWarning;			
		const OSQLParseNode*				m_pParseTree;		
		const OSQLParser*					m_pParser;			
		OSQLStatementType					m_eStatementType;	
		OSQLTables							m_aTables;			
		::vos::ORef<OSQLColumns>			m_aSelectColumns;	
		::vos::ORef<OSQLColumns>			m_aParameters;		
		::comphelper::UStringMixEqual		m_aCaseEqual;

		::com::sun::star::uno::Reference< ::com::sun::star::container::XNameAccess>		m_xTables;
		::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData>	m_xDatabaseMetaData;

		void appendWarning(const ::rtl::OUString& _sErrMsg); 

		void				traverseOneTableName(const OSQLParseNode * pTableName, const ::rtl::OUString & rTableRange);
		void				traverseORCriteria(OSQLParseNode * pSearchCondition);
		void				traverseANDCriteria(OSQLParseNode * pSearchCondition);
		void				traverseOnePredicate(
									OSQLParseNode * pColumnRef,
									OSQLPredicateType ePredicateType,
									::rtl::OUString& aValue,
									sal_Bool bCompareNull,
									OSQLParseNode * pParameter);
		OSQLParseNode *		getTableRef(OSQLParseNode *pTableRef,::rtl::OUString& aTableRange);
		OSQLParseNode *		getQualified_join(OSQLParseNode *pTableRef,::rtl::OUString& aTableRange);
		void				getSelect_statement(OSQLParseNode *pSelect);
		::rtl::OUString		getUniqueColumnName(const ::rtl::OUString & rColumnName)	const;
	protected:
		void setSelectColumnName(const ::rtl::OUString & rColumnName,const ::rtl::OUString & rColumnAlias, const ::rtl::OUString & rTableRange,sal_Bool bFkt=sal_False);
		void appendColumns(const ::rtl::OUString& _rTableAlias,const OSQLTable& _rTable);
	public:
		OSQLParseTreeIterator();
		OSQLParseTreeIterator(const OSQLParseTreeIterator & rIter);
		OSQLParseTreeIterator(	const ::com::sun::star::uno::Reference< ::com::sun::star::container::XNameAccess>& _xTableSupplier ,
					const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XDatabaseMetaData>& _xDatabaseMetaData,
					const OSQLParseNode* pRoot,
					const OSQLParser* _pParser = NULL);
		~OSQLParseTreeIterator();

		void dispose();
		sal_Bool isCaseSensitive() const { return m_aCaseEqual.isCaseSensitive(); }

		void setParseTree(const OSQLParseNode * pNewParseTree);
		void setParser(const OSQLParser* _pParser) { m_pParser = _pParser; }
		const OSQLParseNode * getParseTree() const { return m_pParseTree; };

		const OSQLParseNode* getWhereTree() const;
		const OSQLParseNode* getOrderTree() const;
		const OSQLParseNode* getGroupByTree() const;
		const OSQLParseNode* getHavingTree() const;

		::com::sun::star::sdbc::SQLWarning	getWarning() const { return m_aWarning; }

		OSQLStatementType getStatementType() const { return m_eStatementType; }

		void traverseTableNames();
		virtual void setTableName(const ::rtl::OUString & rTableName, const ::rtl::OUString & rDBName, const ::rtl::OUString& rOwner,
					  const ::rtl::OUString & rTableRange);

		void traverseSelectColumnNames(const OSQLParseNode* pSelectNode);

		void traverseOrderByColumnNames(const OSQLParseNode* pSelectNode);
		virtual void setOrderByColumnName(const ::rtl::OUString & rColumnName, const ::rtl::OUString & rTableRange, sal_Bool bAscending);

		void traverseSelectionCriteria(const OSQLParseNode* pSelectNode);
		virtual void setORCriteriaPre();
		virtual void setORCriteriaPost();
		virtual void setANDCriteriaPre();
		virtual void setANDCriteriaPost();
		virtual void setPredicate(const ::rtl::OUString & rColumnName,
					  const ::rtl::OUString & rTableRange,
					  OSQLPredicateType ePredicateType,
					  const ::rtl::OUString & rValue,
					  const ::rtl::OUString & rParameterName);

		void traverseAssignments();
		virtual void setAssign(const ::rtl::OUString & rColumnName,
				   const ::rtl::OUString & rValue, sal_Bool bsetNull,
				   const ::rtl::OUString & rParameterName);

		void traverseAll();

		const OSQLTables& getTables() const { return m_aTables;};

		::vos::ORef<OSQLColumns> getSelectColumns() const { return m_aSelectColumns;}
		::vos::ORef<OSQLColumns> getParameters()	const { return m_aParameters; }

		::rtl::OUString getColumnAlias(const OSQLParseNode* pDerivedColumn) const;

		void getColumnRange(const OSQLParseNode* pColumnRef,::rtl::OUString &rColumnName,::rtl::OUString &rTableRange) const;

		sal_Bool getColumnTableRange(const OSQLParseNode* pNode, ::rtl::OUString &rTableRange) const;

		sal_Bool isTableNode(const OSQLParseNode* _pTableNode) const;
	};
}

#endif 

