
#ifndef _CONNECTIVITY_SQLPARSE_HXX
#define _CONNECTIVITY_SQLPARSE_HXX

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif
#ifndef _OSL_MUTEX_HXX_ 
#include <osl/mutex.hxx>
#endif
#ifndef _CONNECTIVITY_SQLNODE_HXX
#include <connectivity/sqlnode.hxx>
#endif
#ifndef YYBISON
#ifndef FLEX_SCANNER
#include <connectivity/sqlbison.hxx>
#endif
#endif
#ifndef _COM_SUN_STAR_I18N_XCHARACTERCLASSIFICATION_HPP_
#include <com/sun/star/i18n/XCharacterClassification.hpp>
#endif
#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif
#ifndef _COM_SUN_STAR_I18N_XLOCALEDATA_HPP_
#include <com/sun/star/i18n/XLocaleData.hpp>
#endif

namespace com
{
	namespace sun
	{
		namespace star
		{
			namespace beans
			{
	class XPropertySet;
			}
			namespace util
			{
	class XNumberFormatter;
			}
			namespace lang
			{
	struct Locale;
			}
		}
	}
}
namespace connectivity
{
	class OSQLScanner;

	class OParseContext
	{
	public:
		enum	ErrorCode
		{
			ERROR_NONE		= 0,
			ERROR_GENERAL,					
			ERROR_GENERAL_HINT,				
			ERROR_VALUE_NO_LIKE,			
			ERROR_FIELD_NO_LIKE,			
			ERROR_INVALID_COMPARE,			
			ERROR_INVALID_INT_COMPARE,		
			ERROR_INVALID_STRING_COMPARE,	
			ERROR_INVALID_DATE_COMPARE,		
			ERROR_INVALID_REAL_COMPARE,		
			ERROR_INVALID_TABLE,			
			ERROR_INVALID_COLUMN			
		};

		enum	InternationalKeyCode
		{
			KEY_NONE = 0,
			KEY_LIKE = SQL_TOKEN_LIKE,
			KEY_NOT = SQL_TOKEN_NOT,
			KEY_NULL = SQL_TOKEN_NULL,
			KEY_TRUE = SQL_TOKEN_TRUE,
			KEY_FALSE = SQL_TOKEN_FALSE,
			KEY_IS = SQL_TOKEN_IS,
			KEY_BETWEEN = SQL_TOKEN_BETWEEN,
			KEY_OR = SQL_TOKEN_OR,
			KEY_AND = SQL_TOKEN_AND,
			KEY_AVG = SQL_TOKEN_AVG,
			KEY_COUNT = SQL_TOKEN_COUNT,
			KEY_MAX = SQL_TOKEN_MAX,
			KEY_MIN = SQL_TOKEN_MIN,
			KEY_SUM = SQL_TOKEN_SUM
		};

	public:
		OParseContext();

		virtual ~OParseContext();

		virtual ::rtl::OUString getErrorMessage(ErrorCode _eCodes) const;

		virtual ::rtl::OString getIntlKeywordAscii(InternationalKeyCode _eKey) const;

		virtual InternationalKeyCode getIntlKeyCode(const ::rtl::OString& rToken) const;

		static const ::com::sun::star::lang::Locale& getDefaultLocale();
	};

	class OSQLParser
	{
		friend class OSQLParseNode;
		friend class OSQLInternalNode;
		friend struct OSQLParseNode::SQLParseNodeParameter;

	private:

		static sal_uInt32			s_nRuleIDs[OSQLParseNode::rule_count + 1];
		static OParseContext		s_aDefaultContext;

		static OSQLScanner*			s_pScanner;
		static OSQLParseNodes*		s_pGarbageCollector;
		static sal_Int32			s_nRefCount;

		const OParseContext*		m_pContext;
		OSQLParseNode*				m_pParseTree;	
		::com::sun::star::lang::Locale*	m_pLocale;		
		::rtl::OUString						m_sFieldName;	
		::rtl::OUString						m_sErrorMessage;
		::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet >
						m_xField;		
		::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter >
						m_xFormatter;	
		sal_Int32					m_nFormatKey;	
		::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >	m_xServiceFactory;
		::com::sun::star::uno::Reference< ::com::sun::star::i18n::XCharacterClassification> m_xCharClass;
		static ::com::sun::star::uno::Reference< ::com::sun::star::i18n::XLocaleData>		s_xLocaleData;
		::com::sun::star::uno::Reference< ::com::sun::star::i18n::XLocaleData>		xDummy; 

		::rtl::OUString stringToDouble(const ::rtl::OUString& _rValue,sal_Int16 _nScale);

		static ::osl::Mutex& getMutex();

	public:

		OSQLParser(const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& _xServiceFactory,const OParseContext* _pContext = NULL);
		~OSQLParser();

		OSQLParseNode* parseTree(::rtl::OUString& rErrorMessage,
		   const ::rtl::OUString& rStatement,
		   sal_Bool bInternational = sal_False);

		OSQLParseNode* predicateTree(::rtl::OUString& rErrorMessage, const ::rtl::OUString& rStatement,
		   const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
		   const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & xField);

		const OParseContext& getContext() const {return *m_pContext;}

		static ::rtl::OString TokenIDToStr(sal_uInt32 nTokenID, const OParseContext* pContext = NULL);

		static ::rtl::OUString RuleIDToStr(sal_uInt32 nRuleID);

		static sal_uInt32 StrToRuleID(const ::rtl::OString & rValue);

		static sal_uInt32 RuleID(OSQLParseNode::Rule eRule);

		void error(sal_Char *fmt);
		int SQLlex();
#ifdef YYBISON
		void setParseTree(OSQLParseNode * pNewParseTree);

		sal_Bool inPredicateCheck() const {return m_xField.is();}
		const ::rtl::OUString& getFieldName() const {return m_sFieldName;}

		void reduceLiteral(OSQLParseNode*& pLiteral, sal_Bool bAppendBlank);

		sal_Int16 buildNode(OSQLParseNode*& pAppend,OSQLParseNode* pLiteral,OSQLParseNode*& pCompare);

		sal_Int16 buildNode_STR_NUM(OSQLParseNode*& pAppend,OSQLParseNode*& pLiteral,OSQLParseNode*& pCompare);
		sal_Int16 buildNode_Date(const double& fValue, sal_Int16 nType, OSQLParseNode*& pAppend,OSQLParseNode* pLiteral,OSQLParseNode*& pCompare);

		sal_Int16 buildComparsionRule(OSQLParseNode*& pAppend,OSQLParseNode* pLiteral);

		sal_Int16 buildComparsionRule(OSQLParseNode*& pAppend,OSQLParseNode* pLiteral,OSQLParseNode*& pCompare);

		sal_Int16 buildLikeRule(OSQLParseNode*& pAppend,OSQLParseNode*& pLiteral,const OSQLParseNode* pEscape);
		sal_Int16 buildStringNodes(OSQLParseNode*& pLiteral);
#else
#endif
	};
}

#endif 
