/*____________________________________________________________________________
	pgpKeys.h
	
	Copyright(C) 1996,1997 Pretty Good Privacy, Inc.
	All rights reserved.
	
	Public definitions for PGP KeyDB Library

	$Id: pgpKeys.h,v 1.1 2000/07/28 15:40:22 mhu Exp $
____________________________________________________________________________*/
#ifndef Included_pgpKeys_h	/* [ */
#define Included_pgpKeys_h



#include "pgpPubTypes.h"
#include "pgpHash.h"
#include "pgpEncode.h"



/* Key ordering */
enum PGPKeyOrdering_
{
	kPGPAnyOrdering = 1,
	kPGPUserIDOrdering,
	kPGPReverseUserIDOrdering,
	kPGPKeyIDOrdering,
	kPGPReverseKeyIDOrdering,
	kPGPValidityOrdering,
	kPGPReverseValidityOrdering,
	kPGPTrustOrdering,
	kPGPReverseTrustOrdering,
	kPGPEncryptKeySizeOrdering,
	kPGPReverseEncryptKeySizeOrdering,
	kPGPSigKeySizeOrdering,
	kPGPReverseSigKeySizeOrdering,
	kPGPCreationOrdering,
	kPGPReverseCreationOrdering,
	kPGPExpirationOrdering,
	kPGPReverseExpirationOrdering,
	
	PGP_ENUM_FORCE( PGPKeyOrdering_ )
} ;
PGPENUM_TYPEDEF( PGPKeyOrdering_, PGPKeyOrdering );



/* Key properties */

enum PGPKeyPropName_
{	/* String properties */
	kPGPKeyPropUNUSED_1 = 10,	/* feel free to use this */
	kPGPKeyPropFingerprint,
	kPGPKeyPropPreferredAlgorithms,
	/* Number properties */
	kPGPKeyPropAlgID = 20,
	kPGPKeyPropBits,	
	kPGPKeyPropTrust,			/* old trust model only */
	kPGPKeyPropValidity,    /* both trust models */
	/* Time properties */
	kPGPKeyPropCreation = 30,
	kPGPKeyPropExpiration,
	/* PGPBoolean properties */
	kPGPKeyPropIsSecret = 40,
	kPGPKeyPropIsAxiomatic,
	kPGPKeyPropIsRevoked,
	kPGPKeyPropIsDisabled,
	kPGPKeyPropIsNotCorrupt,
	kPGPKeyPropIsExpired,
	kPGPKeyPropNeedsPassphrase,
	kPGPKeyPropCanSign,
	kPGPKeyPropCanEncrypt,
	kPGPKeyPropHasUnverifiedRevocation,
	
	PGP_ENUM_FORCE( PGPKeyPropName_ )
} ;
PGPENUM_TYPEDEF( PGPKeyPropName_, PGPKeyPropName );

/* User ID properties */

enum PGPUserIDPropName_
{	/* String properties */
	kPGPUserIDPropName = 50,
	/* Number properties */
	kPGPUserIDPropValidity = 60,/* both trust models */
	kPGPUserIDPropConfidence,	/* new trust model only */
	/* Time properties */
	/* PGPBoolean properties */
	
	PGP_ENUM_FORCE( PGPUserIDPropName_ )
} ;
PGPENUM_TYPEDEF( PGPUserIDPropName_, PGPUserIDPropName );

/* Signature properties */

enum PGPSigPropName_
{
	/* String properties */
	kPGPSigPropKeyID = 90,
	/* Number properties */
	kPGPSigPropAlgID = 100,
	kPGPSigPropTrustLevel,
	kPGPSigPropTrustValue,
	/* Time properties */
	kPGPSigPropCreation = 110,
	kPGPSigPropExpiration,
	/* PGPBoolean properties */
	kPGPSigPropIsRevoked = 120,
	kPGPSigPropIsNotCorrupt,
	kPGPSigPropIsTried,
	kPGPSigPropIsVerified,
	kPGPSigPropIsMySig,
	kPGPSigPropIsExportable,
	kPGPSigPropHasUnverifiedRevocation,
	
	PGP_ENUM_FORCE( PGPSigPropName_ )
} ;
PGPENUM_TYPEDEF( PGPSigPropName_, PGPSigPropName );
/*
 * Note on kPGPSigPropIsMySig.  This is a convenience property for
 * determining whether the certification was made by one of the
 * caller's own private keys.  This can only return true if the
 * signing key is in the same base keyset as the certification. If the
 * signing key is( suspected to be) in a different base keyset, call
 * PGPGetSigCertifierKey( certset, signerset, &key) followed by
 * PGPGetKeyBoolean( key, kPGPKeyPropIsSecret, &secret).
 */

enum	/* PGPKeyRingOpenFlags */
{
	kPGPKeyRingOpenFlags_Reserved			= (1UL << 0 ),
	kPGPKeyRingOpenFlags_Mutable			= (1UL << 1 ),
	kPGPKeyRingOpenFlags_Create				= (1UL << 2 ),

	/* The following flags are only used by PGPOpenKeyRing */
	kPGPKeyRingOpenFlags_Private			= (1UL << 8 ),
	kPGPKeyRingOpenFlags_Trusted			= (1UL << 9 )
} ;
typedef PGPUInt32		PGPKeyRingOpenFlags;


/*
 * Used by filtering functions to specify type of match.
 */

enum PGPMatchCriterion_
{
	kPGPMatchDefault = 1,
	kPGPMatchEqual = 1,			 /* searched val == supplied val */
	kPGPMatchGreaterOrEqual = 2, /* searched val >= supplied val */
	kPGPMatchLessOrEqual = 3, /* searched val <= supplied val */
	kPGPMatchSubString = 4,	 /* searched val is contained in supplied val */

	PGP_ENUM_FORCE( PGPMatchCriterion_ )
} ;
PGPENUM_TYPEDEF( PGPMatchCriterion_, PGPMatchCriterion );


/* This is the value of the expiration time which means "never expires" */
#define kPGPExpirationTime_Never	( (PGPTime)0 )


/* Public entry points */

PGP_BEGIN_C_DECLARATIONS
#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif


/* Functions for setting up key filters. */

PGPError 	PGPAPI PGPIncFilterRefCount ( PGPFilterRef filter );

PGPError	PGPAPI PGPFreeFilter ( PGPFilterRef filter );

PGPError 	PGPAPI PGPNewKeyIDFilter( PGPContextRef context, 
						PGPKeyID const * keyID, PGPFilterRef *outFilter);

PGPError	PGPAPI PGPNewSubKeyIDFilter( PGPContextRef		context,
						PGPKeyID const *	subKeyID,  
						PGPFilterRef *		outFilter);

PGPError 	PGPAPI PGPNewKeyEncryptAlgorithmFilter( PGPContextRef context, 
						PGPPublicKeyAlgorithm encryptAlgorithm,
						PGPFilterRef *outFilter );

PGPError 	PGPAPI PGPNewKeyFingerPrintFilter( PGPContextRef context, 
						void const *fingerPrint, 
						PGPSize fingerPrintLength, PGPFilterRef *outFilter );

PGPError 	PGPAPI PGPNewKeyCreationTimeFilter( PGPContextRef context, 
						PGPTime creationTime, PGPMatchCriterion match, 
						PGPFilterRef *outFilter );

PGPError 	PGPAPI PGPNewKeyExpirationTimeFilter( PGPContextRef context, 
						PGPTime expirationTime, 
						PGPMatchCriterion match, PGPFilterRef *outFilter );

PGPError 	PGPAPI PGPNewKeyRevokedFilter( PGPContextRef context,
						PGPBoolean revoked, PGPFilterRef *outFilter );

PGPError	PGPAPI PGPNewKeyDisabledFilter( PGPContextRef context, 
						PGPBoolean			disabled, 
						PGPFilterRef *		outFilter);

PGPError 	PGPAPI PGPNewKeySigAlgorithmFilter( PGPContextRef context,
						PGPPublicKeyAlgorithm sigAlgorithm, 
						PGPFilterRef *outFilter );

PGPError 	PGPAPI PGPNewKeyEncryptKeySizeFilter( PGPContextRef context,
						PGPUInt32 keySize, PGPMatchCriterion match,
						PGPFilterRef *outFilter );

PGPError 	PGPAPI PGPNewKeySigKeySizeFilter( PGPContextRef context,
						PGPUInt32 keySize, PGPMatchCriterion match,
						PGPFilterRef *outFilter );

PGPError	PGPAPI PGPNewUserIDStringFilter( PGPContextRef context,
						char const *		userIDString, 
						PGPMatchCriterion	match,
						PGPFilterRef *		outFilter);

PGPError 	PGPAPI PGPNewUserIDEmailFilter( PGPContextRef context,
						char const *		emailString, 
						PGPMatchCriterion	match,
						PGPFilterRef *		outFilter);

PGPError 	PGPAPI PGPNewUserIDNameFilter( PGPContextRef context,
						char const *		nameString, 
						PGPMatchCriterion	match,
						PGPFilterRef *		outFilter);

PGPError 	PGPAPI PGPNewSigKeyIDFilter( PGPContextRef context,
						PGPKeyID const * keyID, PGPFilterRef *outFilter );


/* freeing outfilter will call PGPFreeFilter on filter */
PGPError 	PGPAPI PGPNegateFilter( PGPFilterRef filter,
						PGPFilterRef *outFilter);

/* freeing outfilter will call PGPFreeFilter on filter1, filter2 */
PGPError 	PGPAPI PGPIntersectFilters( PGPFilterRef filter1,
						PGPFilterRef filter2, PGPFilterRef *outFilter);

/* freeing outfilter will call PGPFreeFilter on filter1, filter2 */
PGPError 	PGPAPI PGPUnionFilters( PGPFilterRef filter1,
						PGPFilterRef filter2, PGPFilterRef *outFilter);


PGPError 	PGPAPI PGPFilterKeySet( PGPKeySetRef origSet,
						PGPFilterRef filter, PGPKeySetRef *resultSet);


/* Keyserver filter functions */

PGPError	PGPAPI PGPLDAPQueryFromFilter( PGPFilterRef filter,
						char **queryOut );

PGPError 	PGPAPI PGPHKSQueryFromFilter( PGPFilterRef	filter,
						char **queryOut );

/* KeySet manipulations */


/* Creates a new memory-based KeyDB and returns its root set */
PGPError  	PGPAPI PGPNewKeySet(PGPContextRef context, PGPKeySetRef *keySet);


/* Used for building arbitrary subsets of an existing KeyDB */
PGPError  	PGPAPI PGPNewEmptyKeySet(PGPKeySetRef origSet, PGPKeySetRef *newSet);
PGPError  	PGPAPI PGPNewSingletonKeySet(PGPKeyRef key, PGPKeySetRef *keySet);
PGPError	PGPAPI PGPUnionKeySets(PGPKeySetRef set1, PGPKeySetRef set2,
								PGPKeySetRef *newSet);

PGPError  	PGPAPI PGPOpenDefaultKeyRings( PGPContextRef context,
						PGPKeyRingOpenFlags openFlags, PGPKeySetRef *keySet);

PGPError  	PGPAPI PGPOpenKeyRingPair( PGPContextRef context,
						PGPKeyRingOpenFlags openFlags, 
						PGPFileSpecRef pubFileRef, PGPFileSpecRef secFileRef,
						PGPKeySetRef *keySet);

PGPError  	PGPAPI PGPOpenKeyRing (PGPContextRef context,
						PGPKeyRingOpenFlags openFlags, PGPFileSpecRef fileRef,
						PGPKeySetRef *keySet);

PGPError 	PGPAPI PGPCommitKeyRingChanges (PGPKeySetRef keys);

PGPError 	PGPAPI PGPPropagateTrust (PGPKeySetRef keys);

PGPError 	PGPAPI PGPRevertKeyRingChanges (PGPKeySetRef keys);

PGPError 	PGPAPI PGPCheckKeyRingSigs (PGPKeySetRef keysToCheck,
						PGPKeySetRef keysSigning, PGPBoolean checkAll,
						PGPEventHandlerProcPtr progress,
						PGPUserValue userValue);

PGPError 	PGPAPI PGPReloadKeyRings (PGPKeySetRef keys);

PGPError 	PGPAPI PGPGetKeyByKeyID (PGPKeySetRef keys,
						PGPKeyID const * keyID,
						PGPPublicKeyAlgorithm pubKeyAlgorithm,
						PGPKeyRef *outRef);

PGPBoolean 	PGPAPI PGPKeySetIsMember(PGPKeyRef key, PGPKeySetRef set);

PGPError 	PGPAPI PGPCountKeys( PGPKeySetRef keys, PGPUInt32 *numKeys);

PGPError 	PGPAPI PGPIncKeySetRefCount( PGPKeySetRef keys);

PGPError 	PGPAPI PGPFreeKeySet( PGPKeySetRef keys);

PGPBoolean 	PGPAPI PGPKeySetIsMutable( PGPKeySetRef keys);

PGPBoolean 	PGPAPI PGPKeySetNeedsCommit( PGPKeySetRef keys);


PGPError 	PGPAPI PGPAddKeys( PGPKeySetRef keysToAdd, PGPKeySetRef set );

PGPError 	PGPAPI PGPRemoveKeys( PGPKeySetRef keysToRemove, PGPKeySetRef set );



/* Key manipulation functions */

PGPError 	PGPAPI PGPDisableKey( PGPKeyRef key);

PGPError 	PGPAPI PGPEnableKey( PGPKeyRef key);

PGPError 	PGPAPI PGPChangePassphrase( PGPKeyRef key, char const *oldphrase,
						char const *newphrase);

PGPError 	PGPAPI PGPChangeSubKeyPassphrase( PGPSubKeyRef subkey,
						   char const *oldphrase, char const *newphrase);

PGPError 	PGPAPI PGPRemoveSubKey( PGPSubKeyRef subkey);

PGPError 	PGPAPI PGPRemoveUserID( PGPUserIDRef userID);

PGPError 	PGPAPI PGPSetPrimaryUserID( PGPUserIDRef userid);

PGPError 	PGPAPI PGPGetSigCertifierKey( PGPSigRef cert, PGPKeySetRef allkeys,
						PGPKeyRef *certkey);

PGPError 	PGPAPI PGPRemoveSig( PGPSigRef cert);

PGPError	PGPAPI PGPCountAdditionalRecipientRequests( PGPKeyRef basekey,
					PGPUInt32 * numARKeys);
					
PGPError 	PGPAPI PGPGetIndexedAdditionalRecipientRequest(
						PGPKeyRef basekey, PGPKeySetRef allkeys, PGPUInt32 nth,
						PGPKeyRef *arkey, PGPByte *arclass );
					
PGPError 	PGPAPI PGPGetIndexedAdditionalRecipientRequestKeyID(
						PGPKeyRef basekey, PGPUInt32 nth, PGPKeyID *arkeyid,
						PGPByte *arclass );

PGPBoolean	PGPAPI PGPPassphraseIsValid( PGPKeyRef key, const char *passphrase );

/* Wrapper functions */
PGPError	PGPAPI PGPGenerateKey( PGPContextRef context, PGPKeyRef *key,
						PGPOptionListRef firstOption, ...);
PGPError	PGPAPI PGPGenerateSubKey( PGPContextRef context,
						PGPSubKeyRef *subkey,
						PGPOptionListRef firstOption, ...);
PGPUInt32	PGPAPI PGPGetKeyEntropyNeeded( PGPContextRef context,
						PGPOptionListRef firstOption, ...);
PGPError	PGPAPI PGPExportKeySet( PGPKeySetRef keys,
						PGPOptionListRef firstOption, ...);
PGPError	PGPAPI PGPImportKeySet( PGPContextRef context,
						PGPKeySetRef *keys,
						PGPOptionListRef firstOption, ...);
PGPError	PGPAPI PGPSignUserID( PGPUserIDRef userID,
						PGPKeyRef certifyingKey,
						PGPOptionListRef firstOption, ...);
PGPError	PGPAPI PGPAddUserID( PGPKeyRef key, char const *userID,
						PGPOptionListRef firstOption, ...);
PGPError	PGPAPI PGPRevokeSig( PGPSigRef cert, PGPKeySetRef allkeys,
						PGPOptionListRef firstOption, ...);
PGPError	PGPAPI PGPRevokeKey( PGPKeyRef key,
						PGPOptionListRef firstOption, ...);
PGPError	PGPAPI PGPRevokeSubKey( PGPSubKeyRef subkey,
						PGPOptionListRef firstOption, ...);


/*
 * Trust values for PGPSetKeyTrust and kPGPKeyPropTrust property:
 * 
 * kPGPKeyTrust_Undefined	(do not pass to PGPSetKeyTrust)
 * kPGPKeyTrust_Unknown		(unknown)
 * kPGPKeyTrust_Never		(never)
 * kPGPKeyTrust_Marginal	(sometimes)
 * kPGPKeyTrust_Complete	(always)
 * kPGPKeyTrust_Ultimate	(do not pass to PGPSetKeyTrust)
 */

 /* old trust model */
PGPError 	PGPAPI PGPSetKeyTrust( PGPKeyRef key, PGPUInt32 trust);

PGPError 	PGPAPI PGPSetKeyAxiomatic( PGPKeyRef key,
						PGPBoolean checkPassphrase, char const *passphrase);

PGPError 	PGPAPI PGPUnsetKeyAxiomatic( PGPKeyRef key);

/*  Get property functions */

PGPError 	PGPAPI PGPGetKeyBoolean( PGPKeyRef key, PGPKeyPropName propname,
						PGPBoolean *prop);

PGPError 	PGPAPI PGPGetKeyNumber( PGPKeyRef key, PGPKeyPropName propname,
						PGPInt32 *prop);

/* 'buffer' is NOT null-terminated */
/* returns kPGPError_BufferTooSmall if buffer is too small, but
fill buffer to capacity */
PGPError 	PGPAPI PGPGetKeyPropertyBuffer( PGPKeyRef key,
						PGPKeyPropName propname,
						PGPSize bufferSize, void *data, PGPSize *fullSize);

PGPError 	PGPAPI PGPGetKeyTime( PGPKeyRef key, PGPKeyPropName propname,
						PGPTime *prop);

PGPError 	PGPAPI PGPGetSubKeyBoolean( PGPSubKeyRef subkey,
						PGPKeyPropName propname, PGPBoolean *prop);

PGPError 	PGPAPI PGPGetSubKeyNumber( PGPSubKeyRef subkey,
						PGPKeyPropName propname, PGPInt32 *prop);

/* returns kPGPError_BufferTooSmall if buffer is too small, but
fill buffer to capacity */
PGPError 	PGPAPI PGPGetSubKeyPropertyBuffer( PGPSubKeyRef subkey,
						PGPKeyPropName propname,
						PGPSize bufferSize, void *prop, PGPSize *fullSize);

PGPError 	PGPAPI PGPGetSubKeyTime( PGPSubKeyRef subkey,
						PGPKeyPropName propname, PGPTime *prop);

PGPError 	PGPAPI PGPGetUserIDNumber( PGPUserIDRef userID,
						PGPUserIDPropName propname, PGPInt32 *prop);

/* 'string' is always NULL-terminated */
/* returns kPGPError_BufferTooSmall if buffer is too small, but
fill buffer to capacity */
PGPError 	PGPAPI PGPGetUserIDStringBuffer( PGPUserIDRef userID,
						PGPUserIDPropName propname,
						PGPSize bufferSize,
						char *		string,
						PGPSize *	fullSize);

PGPError 	PGPAPI PGPGetSigBoolean( PGPSigRef cert,
						PGPSigPropName propname, PGPBoolean *prop);

PGPError 	PGPAPI PGPGetSigNumber( PGPSigRef cert, PGPSigPropName propname,
						PGPInt32 *prop);
						
PGPError 	PGPAPI PGPGetKeyIDOfCertifier( PGPSigRef sig, PGPKeyID *outID );
	
PGPError 	PGPAPI PGPGetSigTime( PGPSigRef cert, PGPSigPropName propname,
						PGPTime *prop);

PGPError 	PGPAPI PGPGetHashAlgUsed( PGPKeyRef key, PGPHashAlgorithm *hashAlg);


/* Convenience property functions */

PGPError  	PGPAPI PGPGetPrimaryUserID( PGPKeyRef key, PGPUserIDRef *outRef );

/* 'string' is always a C string and
	*fullSize includes the '\0' terminator */
/* returns kPGPError_BufferTooSmall if buffer is too small, but
fill buffer to capacity */
PGPError 	PGPAPI PGPGetPrimaryUserIDNameBuffer( PGPKeyRef key, 
					PGPSize bufferSize,
					char *	string, PGPSize *fullSize );

PGPError 	PGPAPI PGPGetPrimaryUserIDValidity(PGPKeyRef key,
					PGPValidity *validity);


PGPInt32 	PGPAPI PGPCompareKeys(PGPKeyRef a, PGPKeyRef b,
						PGPKeyOrdering order);

PGPInt32 	PGPAPI PGPCompareUserIDStrings(char const *a, char const *b);

PGPError  	PGPAPI PGPOrderKeySet( PGPKeySetRef src,
						PGPKeyOrdering order, PGPKeyListRef *outRef );

PGPError 	PGPAPI PGPIncKeyListRefCount( PGPKeyListRef keys);

PGPError 	PGPAPI PGPFreeKeyList( PGPKeyListRef keys);


/* Key iteration functions */



PGPError 	PGPAPI PGPNewKeyIter (PGPKeyListRef keys, PGPKeyIterRef *outRef);

PGPError 	PGPAPI PGPCopyKeyIter (PGPKeyIterRef orig, PGPKeyIterRef *outRef);
	
PGPError 	PGPAPI PGPFreeKeyIter (PGPKeyIterRef iter);

PGPInt32 	PGPAPI PGPKeyIterIndex (PGPKeyIterRef iter);

PGPError 	PGPAPI PGPKeyIterRewind (PGPKeyIterRef iter);

PGPInt32 	PGPAPI PGPKeyIterSeek (PGPKeyIterRef iter, PGPKeyRef key);

PGPError 	PGPAPI PGPKeyIterMove (PGPKeyIterRef iter, PGPInt32 relOffset,
						PGPKeyRef *outRef);

PGPError 	PGPAPI PGPKeyIterNext (PGPKeyIterRef iter, PGPKeyRef *outRef);

PGPError 	PGPAPI PGPKeyIterPrev (PGPKeyIterRef iter, PGPKeyRef *outRef);

PGPError 	PGPAPI PGPKeyIterKey (PGPKeyIterRef iter, PGPKeyRef *outRef);

PGPError  	PGPAPI PGPKeyIterSubKey (PGPKeyIterRef iter,
						PGPSubKeyRef *outRef );

PGPError  	PGPAPI PGPKeyIterUserID (PGPKeyIterRef iter,
						PGPUserIDRef *outRef );
PGPError  	PGPAPI PGPKeyIterSig (PGPKeyIterRef iter,
						PGPSigRef *outRef );

PGPError  	PGPAPI PGPKeyIterNextSubKey (PGPKeyIterRef iter,
						PGPSubKeyRef *outRef );

PGPError  	PGPAPI PGPKeyIterPrevSubKey (PGPKeyIterRef iter,
						PGPSubKeyRef *outRef);

PGPError 	PGPAPI PGPKeyIterRewindSubKey (PGPKeyIterRef iter);

PGPError  	PGPAPI PGPKeyIterNextUserID (PGPKeyIterRef iter,
						PGPUserIDRef *outRef);

PGPError  	PGPAPI PGPKeyIterPrevUserID (PGPKeyIterRef iter,
						PGPUserIDRef *outRef);

PGPError 	PGPAPI PGPKeyIterRewindUserID (PGPKeyIterRef iter);

PGPError  	PGPAPI PGPKeyIterNextUIDSig (PGPKeyIterRef iter,
						PGPSigRef *outRef);

PGPError  	PGPAPI PGPKeyIterPrevUIDSig (PGPKeyIterRef iter,
						PGPSigRef *outRef);

PGPError 	PGPAPI PGPKeyIterRewindUIDSig (PGPKeyIterRef iter);


/* Get/set default private key */

PGPError 	PGPAPI PGPGetDefaultPrivateKey( PGPKeySetRef keyset,
						PGPKeyRef *outRef );

PGPError 	PGPAPI PGPSetDefaultPrivateKey( PGPKeyRef key);



/* Get/set user value */

PGPError 	PGPAPI PGPSetKeyUserVal( PGPKeyRef key, PGPUserValue userValue);

PGPError 	PGPAPI PGPSetUserIDUserVal( PGPUserIDRef userid,
						PGPUserValue userValue);

PGPError 	PGPAPI PGPSetSubKeyUserVal( PGPSubKeyRef subkey,
						PGPUserValue userValue);

PGPError 	PGPAPI PGPSetSigUserVal( PGPSigRef cert,
						PGPUserValue userValue);

PGPError 	PGPAPI PGPGetKeyUserVal( PGPKeyRef key,
						PGPUserValue *userValue);

PGPError 	PGPAPI PGPGetUserIDUserVal( PGPUserIDRef userid,
						PGPUserValue *userValue);

PGPError 	PGPAPI PGPGetSubKeyUserVal( PGPSubKeyRef subkey,
						PGPUserValue *userValue);

PGPError 	PGPAPI PGPGetSigUserVal( PGPSigRef cert,
						PGPUserValue *userValue);


/*____________________________________________________________________________
	Key IDs
____________________________________________________________________________*/
PGPError 	PGPAPI PGPGetKeyIDFromString(
				const char *string, PGPKeyID *id );
					
PGPError 	PGPAPI PGPGetKeyIDFromKey( PGPKeyRef key, PGPKeyID *id );

PGPError 	PGPAPI PGPGetKeyIDFromSubKey( PGPSubKeyRef key, PGPKeyID * id );

enum PGPKeyIDStringType_
{
	kPGPKeyIDString_Abbreviated = 2,
	kPGPKeyIDString_Full = 3,
	PGP_ENUM_FORCE( PGPKeyIDStringType_ )
};
PGPENUM_TYPEDEF( PGPKeyIDStringType_, PGPKeyIDStringType );


#define kPGPMaxKeyIDStringSize		( 127 + 1 )
PGPError 	PGPAPI PGPGetKeyIDString( PGPKeyID const * ref,
					PGPKeyIDStringType type,
					char outString[ kPGPMaxKeyIDStringSize ] );


/* outputs opaque string of bytes for storage of maximum size as keyID*/
/* do NOT attempt to parse the output; it is opaque to you */
#define kPGPMaxExportedKeyIDSize		( sizeof( PGPKeyID ) )
PGPError 	PGPAPI PGPExportKeyID( PGPKeyID const * keyID,
					PGPByte exportedData[ kPGPMaxExportedKeyIDSize ],
					PGPSize *exportedLength );

/* must be in format output by PGPExportKeyID */
PGPError 	PGPAPI PGPImportKeyID(  void const * data, PGPKeyID * id );

/* returns 0 if equal, -1 if key1 < key2, 1 if key1 > key2 */
PGPInt32 	PGPAPI PGPCompareKeyIDs( PGPKeyID const * key, PGPKeyID const * key2);




/*____________________________________________________________________________
	Getting contexts back from key related items.  If the key is invalid,
	you get kPGPInvalidRef back.
____________________________________________________________________________*/

PGPContextRef	PGPAPI PGPGetKeyListContext( PGPKeyListRef ref );
PGPContextRef	PGPAPI PGPGetKeySetContext( PGPKeySetRef ref );
PGPContextRef	PGPAPI PGPGetKeyIterContext( PGPKeyIterRef ref );
PGPContextRef	PGPAPI PGPGetKeyContext( PGPKeyRef ref );
PGPContextRef	PGPAPI PGPGetSubKeyContext( PGPSubKeyRef ref );
PGPContextRef	PGPAPI PGPGetUserIDContext( PGPUserIDRef ref );





#if PRAGMA_IMPORT_SUPPORTED
#pragma import reset
#endif
PGP_END_C_DECLARATIONS


#endif /* ] Included_pgpKeys_h */

/*__Editor_settings____

	Local Variables:
	tab-width: 4
	End:
	vi: ts=4 sw=4
	vim: si
_____________________*/
