

#ifndef _B3D_B3DGEOM_HXX
#define _B3D_B3DGEOM_HXX

#ifndef _B3D_B3DENTITY_HXX
#include <goodies/b3dentty.hxx>
#endif

#ifndef _B3D_VOLUM_HXX
#include <goodies/b3dvolum.hxx>
#endif

#define	B3D_CREATE_DEFAULT_X			(0x0001)
#define	B3D_CREATE_DEFAULT_Y			(0x0002)
#define	B3D_CREATE_DEFAULT_Z			(0x0004)
#define	B3D_CREATE_DEFAULT_ALL			(B3D_CREATE_DEFAULT_X|B3D_CREATE_DEFAULT_Y|B3D_CREATE_DEFAULT_Z)

#define	B3D_INDEX_MODE_FILLED					(0)
#define	B3D_INDEX_MODE_LINE						(1)
#define	B3D_INDEX_MODE_POINT					(2)
#define	B3D_INDEX_MODE_UNUSED					(3)

class GeometryIndexValue
{
private:
	unsigned					nIndex		: 30;
	unsigned					nMode		:  2;

public:
	GeometryIndexValue() : nIndex(0), nMode(B3D_INDEX_MODE_FILLED) {}
	GeometryIndexValue(UINT32 nInd) : nIndex(nInd), nMode(B3D_INDEX_MODE_FILLED) {}
	GeometryIndexValue(UINT32 nInd, UINT8 nMod) : nIndex(nInd), nMode(nMod) {}

	UINT32 GetIndex() { return ((UINT32)nIndex); }
	void SetIndex(UINT32 nNew) { nIndex = nNew; }

	UINT8 GetMode() { return ((UINT8)nMode); }
	void SetMode(UINT8 nNew) { nMode = nNew; }

protected:
};

BASE3D_DECL_BUCKET(GeometryIndexValue, Bucket)

class B3dComplexPolygon;
class Matrix4D;

class B3dGeometry
{
private:

	B3dComplexPolygon*			pComplexPolygon;

	B3dEntityBucket				aEntityBucket;

	GeometryIndexValueBucket	aIndexBucket;

	void Reset();

	B3dEntity& GetFreeEntity();

	unsigned					bHintIsComplex		: 1;
	unsigned					bOutline			: 1;

public:

	B3dGeometry();

	void StartDescription();
	void EndDescription();

	void StartObject(BOOL bHintIsComplex=TRUE, BOOL bOutl=FALSE);
	void EndObject();

	void AddEdge(const Vector3D& rPoint);
	void AddEdge(const Vector3D& rPoint,
		const Vector3D& rNormal);
	void AddEdge(const Vector3D& rPoint,
		const Vector3D& rNormal,
		const Vector3D& rTexture);

	void Erase();

	void Empty();

	void operator=(const B3dGeometry& rObj);

	B3dEntityBucket& GetEntityBucket() { return aEntityBucket; }
	GeometryIndexValueBucket& GetIndexBucket() { return aIndexBucket; }

	void Transform(const Matrix4D&);

	INT32 CheckHit(const Vector3D &rFront, const Vector3D &rBack, USHORT nTol);

	B3dVolume GetBoundVolume();

	Vector3D GetCenter();

	void CreateDefaultNormalsSphere();
	void RemoveNormals();

	void CreateDefaultTexture(UINT16 nCreateWhat=B3D_CREATE_DEFAULT_ALL,
		BOOL bUseSphere=TRUE);
	void RemoveTexture();

	void CreateCube(const B3dVolume& rVolume);
	void CreateSphere(const B3dVolume& rVolume, double nX, double nY);

	void InvertNormals();

protected:

	friend class B3dComplexPolygon;
	void StartComplexPrimitive();
	void EndComplexPrimitive();
	void AddComplexVertex(B3dEntity& rNew, BOOL bIsVisible);

	void StartPolygon();
	void EndPolygon();

	INT32 CheckSinglePolygonHit(UINT32 nLow, UINT32 nHigh, const Vector3D& rFront, const Vector3D& rBack);

	BOOL GetCutPoint(UINT32 nLow, Vector3D& rCut, const Vector3D& rFront, const Vector3D& rBack);

	BOOL IsInside(UINT32 nLow, UINT32 nHigh, const Vector3D& rPnt);

	Vector3D CalcNormal(UINT32 nLow, UINT32 nHigh);
};

#endif          
