

#ifndef _B3D_B3DTRANS_HXX
#define _B3D_B3DTRANS_HXX

#ifndef _B3D_HMATRIX_HXX
#include <goodies/hmatrix.hxx>
#endif

#define ZBUFFER_DEPTH_RANGE			(256.0 * 256.0 * 256.0)

class Base3D;
class B3dVolume;

enum Base3DRatio
{
	Base3DRatioGrow = 1,
	Base3DRatioShrink,
	Base3DRatioMiddle
};

enum Base3DProjectionType
{
	Base3DProjectionTypeParallel = 1,
	Base3DProjectionTypePerspective
};

class B3dTransformationSet
{
private:

	Matrix4D				aObjectTrans;
	Matrix4D				aInvObjectTrans;

	Matrix4D				aOrientation;
	Matrix4D				aInvOrientation;

	Matrix4D				aProjection;
	Matrix4D				aInvProjection;

	Matrix4D				aTexture;

	Matrix4D				aObjectToDevice;

	Matrix4D				aInvTransObjectToEye;

	Matrix4D				aMatFromWorldToView;
	Matrix4D				aInvMatFromWorldToView;

	Vector3D				aScale;
	Vector3D				aTranslate;

	double					fLeftBound;
	double					fRightBound;
	double					fBottomBound;
	double					fTopBound;

	double					fNearBound;
	double					fFarBound;

	double					fRatio;

	Rectangle				aViewportRectangle;
	Rectangle				aVisibleRectangle;

	Rectangle				aSetBound;

	Base3DRatio				eRatio;

	unsigned				bPerspective				: 1;
	unsigned				bWorldToViewValid			: 1;
	unsigned				bInvTransObjectToEyeValid	: 1;
	unsigned				bObjectToDeviceValid		: 1;
	unsigned				bProjectionValid			: 1;

public:
	B3dTransformationSet();

	void Reset();

	void SetObjectTrans(Matrix4D& rObj);
	const Matrix4D& GetObjectTrans() { return aObjectTrans; }
	const Matrix4D& GetInvObjectTrans() { return aInvObjectTrans; }

#ifndef ICC
	void SetOrientation(Vector3D& aVRP = Vector3D(0.0,0.0,1.0),
		Vector3D& aVPN = Vector3D(0.0,0.0,1.0),
		Vector3D& aVUP = Vector3D(0.0,1.0,0.0));
#else
	void SetOrientation(Vector3D aVRP = Vector3D(0.0,0.0,1.0),
		Vector3D aVPN = Vector3D(0.0,0.0,1.0),
		Vector3D aVUP = Vector3D(0.0,1.0,0.0));
#endif
	void SetOrientation(Matrix4D& mOrient);
	const Matrix4D& GetOrientation() { return aOrientation; }
	const Matrix4D& GetInvOrientation() { return aInvOrientation; }

	void SetProjection(Matrix4D& mProject);
	const Matrix4D& GetProjection();
	const Matrix4D& GetInvProjection();

	void SetTexture(Matrix4D& rTxt);
	const Matrix4D& GetTexture() { return aTexture; }

	double GetRatio() { return fRatio; }
	void SetRatio(double fNew=1.0);
	Base3DRatio GetRatioMode() { return eRatio; }
	void SetRatioMode(Base3DRatio eNew=Base3DRatioGrow);

	void SetDeviceRectangle(double fL=-1.0, double fR=1.0, double fB=-1.0, double fT=1.0,
		BOOL bBroadCastChange=TRUE);
	void SetDeviceVolume(const B3dVolume& rVol, BOOL bBroadCastChange=TRUE);
	void GetDeviceRectangle(double &fL, double &fR, double& fB, double& fT);
	B3dVolume GetDeviceVolume();
	double GetDeviceRectangleWidth() const { return fRightBound - fLeftBound; }
	double GetDeviceRectangleHeight() const { return fTopBound - fBottomBound; }
	void SetFrontClippingPlane(double fF=0.0);
	double GetFrontClippingPlane() { return fNearBound; }
	void SetBackClippingPlane(double fB=1.0);
	double GetBackClippingPlane() { return fFarBound; }
	void SetPerspective(BOOL bNew);
	BOOL GetPerspective() { return bPerspective; }
	void SetViewportRectangle(Rectangle& rRect, Rectangle& rVisible);
	void SetViewportRectangle(Rectangle& rRect) { SetViewportRectangle(rRect, rRect); }
	const Rectangle& GetViewportRectangle() { return aViewportRectangle; }
	void CalcViewport();

	Matrix4D GetMatFromObjectToView();

	const Matrix4D& GetInvTransObjectToEye();

	const Matrix4D& GetObjectToDevice();

	const Matrix4D& GetMatFromWorldToView();
	const Matrix4D& GetInvMatFromWorldToView();

	const Rectangle& GetLogicalViewportBounds();
	const Vector3D& GetScale();
	const Vector3D& GetTranslate();

	const Vector3D WorldToEyeCoor(const Vector3D& rVec);
	const Vector3D EyeToWorldCoor(const Vector3D& rVec);
	const Vector3D EyeToViewCoor(const Vector3D& rVec);
	const Vector3D ViewToEyeCoor(const Vector3D& rVec);
	const Vector3D WorldToViewCoor(const Vector3D& rVec);
	const Vector3D ViewToWorldCoor(const Vector3D& rVec);
	const Vector3D DeviceToViewCoor(const Vector3D& rVec);
	const Vector3D ViewToDeviceCoor(const Vector3D& rVec);
	const Vector3D ObjectToWorldCoor(const Vector3D& rVec);
	const Vector3D WorldToObjectCoor(const Vector3D& rVec);
	const Vector3D ObjectToViewCoor(const Vector3D& rVec);
	const Vector3D ViewToObjectCoor(const Vector3D& rVec);
	const Vector3D ObjectToEyeCoor(const Vector3D& rVec);
	const Vector3D EyeToObjectCoor(const Vector3D& rVec);
	const Vector3D DeviceToEyeCoor(const Vector3D& rVec);
	const Vector3D EyeToDeviceCoor(const Vector3D& rVec);

	const Vector3D InvTransObjectToEye(const Vector3D& rVec);
	const Vector3D TransTextureCoor(const Vector3D& rVec);

protected:
	void PostSetObjectTrans();
	void PostSetOrientation();
	void PostSetProjection();
	void PostSetTexture();
	void PostSetViewport();

	void CalcMatObjectToDevice();
	void CalcMatFromWorldToView();
	void CalcMatInvTransObjectToEye();

	virtual void DeviceRectangleChange();
};

class B3dViewport : public B3dTransformationSet
{
private:
	Vector3D				aVRP;			
	Vector3D				aVPN;			
	Vector3D				aVUV;			

public:
	B3dViewport();

	void SetVRP(const Vector3D& rNewVRP);
	void SetVPN(const Vector3D& rNewVPN);
	void SetVUV(const Vector3D& rNewVUV);
	void SetViewportValues(
		const Vector3D& rNewVRP,
		const Vector3D& rNewVPN,
		const Vector3D& rNewVUV);

	const Vector3D&	GetVRP() const	{ return aVRP; }
	const Vector3D&	GetVPN() const	{ return aVPN; }
	const Vector3D&	GetVUV() const	{ return aVUV; }

protected:
	void CalcOrientation();
};

class B3dCamera : public B3dViewport
{
private:
	Vector3D				aPosition;
	Vector3D				aCorrectedPosition;
	Vector3D				aLookAt;
	double					fFocalLength;
	double					fBankAngle;

	unsigned				bUseFocalLength			: 1;

public:
	B3dCamera(const Vector3D& rPos = Vector3D(0.0, 0.0, 1.0),
		const Vector3D& rLkAt = Vector3D(0.0, 0.0, 0.0),
		double fFocLen = 35.0, double fBnkAng = 0.0,
		BOOL bUseFocLen = FALSE);

	void SetPosition(const Vector3D& rNewPos);
	const Vector3D& GetPosition() const { return aPosition; }
	void SetLookAt(const Vector3D& rNewLookAt);
	const Vector3D& GetLookAt() const { return aLookAt; }
	void SetPositionAndLookAt(const Vector3D& rNewPos, const Vector3D& rNewLookAt);

	void SetFocalLength(double fLen);
	double GetFocalLength() const { return fFocalLength; }

	void SetBankAngle(double fAngle);
	double GetBankAngle() const { return fBankAngle; }

	void SetUseFocalLength(BOOL bNew);
	BOOL GetUseFocalLength() const { return (BOOL)bUseFocalLength; }

protected:
	void CalcNewViewportValues();
	BOOL CalcFocalLength();

	virtual void DeviceRectangleChange();
};

#endif          
