

#ifndef _B3D_BASE3D_HXX
#define _B3D_BASE3D_HXX

#ifndef _B3D_B3DLIGHT_HXX
#include <goodies/b3dlight.hxx>
#endif

#ifndef _B3D_HMATRIX_HXX
#include <goodies/hmatrix.hxx>
#endif

#ifndef _B3D_B3DENTITY_HXX
#include <goodies/b3dentty.hxx>
#endif

#ifndef _B3D_B3DCOMPO_HXX
#include <goodies/b3dcompo.hxx>
#endif

#ifndef _INC_FLOAT
#include <float.h>
#endif

#ifndef _SV_OUTDEV3D_HXX
#include <vcl/outdev3d.hxx>
#endif

#ifndef _SV_TIMER_HXX
#include <vcl/timer.hxx>
#endif

#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif

class B3dTexture;
class OutputDevice;
class Bitmap;
class BitmapEx;
class TextureAttributes;
class B3dTransformationSet;

SV_DECL_PTRARR_DEL(B3dTextureStore, B3dTexture*, 0, 4);

class B3dGlobalData
{
private:

	Container				maTextureStore;
	AutoTimer				maTimer;
	vos::OMutex				maMutex;
	 DECL_LINK(TimerHdl, AutoTimer*);

public:
	B3dGlobalData();
	virtual ~B3dGlobalData();

	B3dTexture* ObtainTexture(TextureAttributes& rAtt);
	void InsertTexture(B3dTexture* pNew);
	void DeleteTexture(B3dTexture* pOld);
	void DeleteAllTextures();
};

#define	BASE3D_TYPE_DEFAULT			0x0000
#define	BASE3D_TYPE_OPENGL			0x0001
#define	BASE3D_TYPE_PRINTER			0x0002

enum Base3DObjectMode
{

	Base3DPoints = 0,
	Base3DLines,
	Base3DLineLoop,
	Base3DLineStrip,
	Base3DTriangles,
	Base3DTriangleStrip,
	Base3DTriangleFan,
	Base3DQuads,
	Base3DQuadStrip,
	Base3DPolygon,

	Base3DComplexPolygon,

	Base3DComplexPolygonCut
};

enum Base3DRenderMode
{
	Base3DRenderNone = 0,
	Base3DRenderPoint,
	Base3DRenderLine,
	Base3DRenderFill
};

enum Base3DCullMode
{
	Base3DCullNone = 0,
	Base3DCullFront,
	Base3DCullBack
};

enum Base3DShadeModel
{
	Base3DSmooth = 0,
	Base3DFlat,
	Base3DPhong
};

enum Base3DPolygonOffset
{
	Base3DPolygonOffsetFill = 1,
	Base3DPolygonOffsetLine,
	Base3DPolygonOffsetPoint
};

#define	SMALL_DVALUE					(0.0000001)

class Base3D : public OutDev3D
{
private:

	OutputDevice*			pDevice;

	Base3DObjectMode		eObjectMode;

	Color					aCurrentColor;

	B3dComplexPolygon		aComplexPolygon;

	Base3DRenderMode		eRenderModeFront;
	Base3DRenderMode		eRenderModeBack;

	Base3DShadeModel		eShadeModel;

	Base3DCullMode			eCullMode;

	double					fPointSize;
	double					fLineWidth;

	Rectangle				aScissorRectangle;

	B3dTexture*				pActiveTexture;

	B3dTransformationSet*	pTransformationSet;

	B3dLightGroup*			pLightGroup;

	B3dMaterial				aMaterialFront;
	B3dMaterial				aMaterialBack;

	UINT8					nDisplayQuality;

	unsigned				bEdgeFlag					: 1;

	unsigned				bContextIsValid				: 1;

	unsigned				bPolyOffsetFill				: 1;
	unsigned				bPolyOffsetLine				: 1;
	unsigned				bPolyOffsetPoint			: 1;

	unsigned				bScissorRegionActive		: 1;

	unsigned				bDitherActive				: 1;

	unsigned				bTransparentParts			: 1;

	unsigned				bForceFlat					: 1;

protected:

	Base3D(OutputDevice* pOutDev);
	virtual ~Base3D();

	void SetContextIsValid(BOOL bNew=TRUE)
		{ bContextIsValid=bNew; }

	static Base3D* CreateScreenRenderer(OutputDevice* pDev);

public:

	virtual UINT16 GetBase3DType()=0;

	OutputDevice* GetOutputDevice()
		{ return pDevice; }

	void SetTransparentPartsContainedHint(BOOL bNew) { bTransparentParts = bNew; }
	BOOL GetTransparentPartsContainedHint() { return bTransparentParts; }

private:
	B3dGlobalData& GetGlobalData();
public:

	static Base3D* Create(OutputDevice* pOutDev, BOOL bForcePrinter=FALSE);
#if SUPD > 375
	virtual void Destroy(OutputDevice *pOutDev);
#else
	void Destroy(OutputDevice *pOutDev);
#endif

	BOOL IsContextValid()
		{ return bContextIsValid; }
#ifdef DBG_UTIL
	void SetContextValid(BOOL bNew=TRUE) { bContextIsValid = bNew; }
#endif

	virtual void SetTransformationSet(B3dTransformationSet* pSet);
	B3dTransformationSet* GetTransformationSet() { return pTransformationSet; }

	virtual void SetLightGroup(B3dLightGroup* pSet, BOOL bSetGlobal=TRUE);
	B3dLightGroup* GetLightGroup() { return pLightGroup; }

	virtual void StartScene() = 0;
	virtual void EndScene() = 0;

	virtual void SetScissorRegionPixel(const Rectangle& rRect, BOOL bActivate=TRUE);
	void SetScissorRegion(const Rectangle& rRect, BOOL bActivate=TRUE);
	virtual void ActivateScissorRegion(BOOL bNew);
	BOOL IsScissorRegionActive() { return bScissorRegionActive; }
	const Rectangle& GetScissorRegionPixel() { return aScissorRectangle; }

	BOOL GetDither() { return bDitherActive; }
	virtual void SetDither(BOOL bNew);

	BOOL GetForceFlat() { return bForceFlat; }
	void SetForceFlat(BOOL bNew) { bForceFlat = bNew; }

	virtual void SetColor(Color aNew);
	virtual Color GetColor();

	virtual void SetMaterial(Color aNew,
		Base3DMaterialValue=Base3DMaterialAmbient,
		Base3DMaterialMode=Base3DMaterialFrontAndBack);
	Color GetMaterial(Base3DMaterialValue=Base3DMaterialAmbient,
		Base3DMaterialMode=Base3DMaterialFrontAndBack);
	virtual void SetShininess(UINT16 nExponent,
		Base3DMaterialMode=Base3DMaterialFrontAndBack);
	UINT16 GetShininess(Base3DMaterialMode=Base3DMaterialFrontAndBack);
	void ResetMaterial(Base3DMaterialMode=Base3DMaterialFrontAndBack);

	B3dTexture* ObtainTexture(TextureAttributes& rAtt);
	B3dTexture* ObtainTexture(TextureAttributes& rAtt, BitmapEx& rBitmapEx);
	void DeleteTexture(TextureAttributes& rAtt);
	void DeleteAllTextures();
private:
	virtual B3dTexture* CreateTexture(TextureAttributes& rAtt, BitmapEx& rBitmapEx);
protected:
	virtual void DestroyTexture(B3dTexture*);
public:
	virtual void SetActiveTexture(B3dTexture* pTex=NULL);
	B3dTexture* GetActiveTexture() { return pActiveTexture; }
	BOOL IsTextureActive() { return (pActiveTexture != NULL) ? TRUE : FALSE; }

	virtual void SetDisplayQuality(UINT8 nNew);
	UINT8 GetDisplayQuality();

	virtual void SetPolygonOffset(
		Base3DPolygonOffset eNew=Base3DPolygonOffsetLine, BOOL bNew=FALSE);
	BOOL GetPolygonOffset(Base3DPolygonOffset eNew=Base3DPolygonOffsetLine);

	virtual void SetRenderMode(Base3DRenderMode eNew,
		Base3DMaterialMode=Base3DMaterialFrontAndBack);
	Base3DRenderMode GetRenderMode(
		Base3DMaterialMode=Base3DMaterialFrontAndBack);

	virtual void SetShadeModel(Base3DShadeModel eNew);
	Base3DShadeModel GetShadeModel();

	virtual void SetCullMode(Base3DCullMode eNew);
	Base3DCullMode GetCullMode();

	BOOL GetEdgeFlag();
	virtual void SetEdgeFlag(BOOL bNew=TRUE);

	double GetPointSize();
	virtual void SetPointSize(double fNew=1.0);
	double GetLineWidth();
	virtual void SetLineWidth(double fNew=1.0);

	virtual Base3DObjectMode GetObjectMode();

	void AddVertex(Vector3D& rVertex);
	void AddVertex(Vector3D& rVertex,
		Vector3D& rNormal);
	void AddVertex(Vector3D& rVertex,
		Vector3D& rNormal, Vector3D& rTexPos);
	void AddVertex(B3dEntity& rEnt);
	void PostAddVertex(B3dEntity& rEnt);

	virtual B3dEntity& GetFreeEntity();

	void StartPrimitive(Base3DObjectMode eMode);
	void EndPrimitive();

	virtual void DrawPolygonGeometry(B3dGeometry& rGeometry, BOOL bOutline=FALSE);

#ifdef DBG_UTIL

	void Test(const Rectangle& rBound, const Rectangle& rVisible);
#endif

protected:

	virtual void ImplPostAddVertex(B3dEntity& rEnt) = 0;

	B3dMaterial& GetMaterialObject(Base3DMaterialMode=Base3DMaterialFront);

	virtual void ImplStartPrimitive() = 0;
	virtual void ImplEndPrimitive() = 0;

	virtual B3dEntity& ImplGetFreeEntity() = 0;

	void HandleComplexPolygon();
};

#endif          
