
#ifndef _MAILSRV_HXX
#define _MAILSRV_HXX

#ifdef OS2
#include <tools/solar.h>
#if 0 // _SOLAR__PRIVATE
#ifndef _DLL_
#define _DLL_ 1			
#endif
#endif
#endif

#ifndef _APP_HXX 
#include <vcl/svapp.hxx>
#endif
#ifndef _SFXHINT_HXX 
#include <svtools/hint.hxx>
#endif
#include <goodies/mailenum.hxx>
#include <goodies/mailbook.hxx>

#define MAIL_LIBRARY()	pImpl->pLibrary
#define MAIL_RES(nId)	ResId( nId, GetSystem().GetResMgr() )

class Link;
class DirEntry;
class Window;
class ResMgr;
class MailServer;
class Mail;
class IncomingMail;
class OutgoingMail;
class MessageRefList;
class MailFolderManager;
class SvStringsDtor;
class INetCoreMIMEMessage;
class INetCoreMailer;

struct MailServer_Impl;
struct MessageRef;

void ExtractMailFromMessage( const INetCoreMIMEMessage&, INetCoreMailer*,
				 FASTBOOL, OutgoingMail&, FASTBOOL& );

typedef BOOL (*GetEmailNames)( SvStringsDtor& rList );
typedef MailFolderManager* (*GetFolderManager)( const String& rName );

struct MailReceiver {
	String      		aUserName;
	MailReceiverRole  	eRole;
};

struct MailAttachFile {
	String      		aFile;
	String      		aName;
};

class MailHint : public SfxHint
{
	MailServer*	_pServer;
	const Mail*	_pMail;
	MailAction	_eAction;

public:
	TYPEINFO();
		MailHint( MailServer* pServer, MailAction eAction,
			  const Mail* pMail = 0 )
	:	_pServer( pServer ),
		_eAction( eAction ),
		_pMail( pMail )
	{}

	MailServer*	GetServer() const { return _pServer; }
	const Mail*	GetMail() const	{ return _pMail; }
	MailAction	GetAction() const { return _eAction; }
};

class MailServer
{
friend class MailSystem;

private:
	const MailSystem&	rSystem;

	MailServer( const MailServer& );
	MailServer& operator=( const MailServer& );

#if 0 // _SOLAR__PRIVATE
	void				PrepareMailList();
#endif

protected:
	FASTBOOL			bLogin;
	FASTBOOL			bLoadMails;
	MailState			eState;
	MailDriver			eDriver;
	MailServer_Impl*	pImpl;
	MessageRefList* 	pRefList;
	MailFolderManager*	pFolderManager;
	String				sUser;
	GetEmailNames		fnGetEmailNames;
	GetFolderManager	fnGetFolderManager;

	MailServer( const MailSystem& rSys );

	void				LoadLocalMails();
	const MailSystem& 	GetSystem() const { return rSystem; }
	void 				SetStatusText( const String& rTxt )
				{ GetpApp()->ShowStatusText( rTxt ); }
	void 				HideStatusText()
				{ GetpApp()->HideStatusText(); }
	void 				SetProgress( ULONG nCur, ULONG nMax,
						 USHORT nResId, BOOL bHide );

public:
	virtual 					~MailServer();
	virtual MailState			Send( OutgoingMail& rMail );

	virtual String				GetAddresses( Window* pParent,
								  const String& rOldAddr );

	virtual MailAddressBook		GetDirectory();
	virtual MailAddressBook		GetMailList();
	virtual MailAddressBook		GetPrivateList();

	virtual void				LoadMailList( MessageRefList& rList,
								  BOOL bAll );
	virtual void				LoadMailAttributes(
						MessageRef* pRef, MessageRef* pNew,
						OutgoingMail& rMail, BOOL bAttach );
	virtual BOOL				MarkMailAsRead( MessageRef* pRef );
	virtual BOOL				RemoveMail( MessageRef* pRef );
	virtual BOOL				DeleteFromServer( SvStringsDtor* pIdList );
	virtual void				ClearMailReference( MessageRef* );
	virtual BOOL				HasNewMail();

	USHORT				GetMailCount();
	void				UpdateMailList( BOOL bAll = FALSE );
	IncomingMail*		GetMail( USHORT nIndex,
					 BOOL bAttach = TRUE, BOOL bLast = FALSE );
	IncomingMail*		NextMail( const IncomingMail* pCurMail );
	IncomingMail*		PrevMail( const IncomingMail* pCurMail );
	BOOL				HasNextMail( const IncomingMail* pCurMail );
	BOOL				HasPrevMail( const IncomingMail* pCurMail );
	ULONG				GetPos( const IncomingMail& rMail ) const;
	void 				ReadMail( USHORT nIndex, BOOL bLast );
	const String&		GetLastId() const;
	void				SetLastId( const String& rId );

	ResMgr*				GetResMgr();
	MailState			GetState() 	{ return eState; }
	MailDriver			GetDriver() { return eDriver; }
	const String&		GetUser()	{ return sUser; }
	BOOL				IsEqual( MailDriver eCmpDriver,
					 const String& rCmpName,
					 const String& rCmpPath ) const;

	void				SetEmailNamesFunc( GetEmailNames pFunc )
				{ fnGetEmailNames = pFunc; }
	void				SetFolderFunc( GetFolderManager pFunc )
				{ fnGetFolderManager = pFunc; }
	BOOL				GetEmailAddresses( SvStringsDtor& rNameList );

	void				SetHintLink( const Link &rLink );
	void 				Hint( MailAction eAction, const Mail *pMail = 0 );

	static IncomingMail*	CreateMail( Mail* pCopy = 0 );
	static String			GetContentType4Ext( const String& rExt );
	static String			GetExt4ContentType( const String& rType );
	static FASTBOOL			GetTempFile( DirEntry& rFile,
							 const String& rMask );
};

#endif

