

#ifndef _B2D_MATRIX3D_HXX
#define _B2D_MATRIX3D_HXX

#ifndef _B2D_POINT3D_HXX
#include <goodies/point3d.hxx>
#endif

class Matrix3D
{
private:

	BOOL Ludcmp(UINT16 nIndex[], INT16& nParity);
	void Lubksb(UINT16 nIndex[], Point3D& aVec);

protected:
	Point3D						M[3];

public:

	Matrix3D() { M[0][0] = M[1][1] = 1.0;
	 M[0][2] = M[1][2] = 0.0; }

	Point3D& operator[](int nPos) { return M[nPos]; }
	const Point3D& operator[](int nPos) const { return M[nPos]; }

	Point3D GetColumnVector(int nCol) const
		{ return Point3D(M[0][nCol], M[1][nCol], M[2][nCol]); }

	void Identity(void);

	BOOL Invert();
	BOOL IsInvertible();

	void Correct();

	void Normalize();

	double Determinant();

	double Trace();

	void Transpose();

	void Rotate(double fAngle);
	void Rotate(double fSin, double fCos );

	void Translate(double fX, double fY);
	void Translate(const Vector2D& aTrans);
	void TranslateX(double fValue);
	void TranslateY(double fValue);

	void Scale(double fX, double fY);
	void Scale(const Vector2D& aScale);
	void ScaleX(double fFactor);
	void ScaleY(double fFactor);

	void ShearX(double fSx);
	void ShearY(double fSy);

	Matrix3D&	operator+=	(const Matrix3D&);
	Matrix3D&	operator-=	(const Matrix3D&);
	Matrix3D	operator+ 	(const Matrix3D&) const;
	Matrix3D	operator-	(const Matrix3D&) const;

	BOOL		operator==	(const Matrix3D&) const;
	BOOL		operator!=	(const Matrix3D&) const;

	Matrix3D&	operator*=	(double);
	Matrix3D	operator*	(double) const;
	Matrix3D&	operator/=	(double);
	Matrix3D	operator/	(double) const;

	Matrix3D&	operator*=	(const Matrix3D&);
	Matrix3D	operator*	(const Matrix3D&) const;

	friend Point3D	operator*	(const Matrix3D&, const Point3D&);
	friend Point3D&	operator*=	(Point3D& rPnt, const Matrix3D& rMat)
		{ return (rPnt = rMat * rPnt); }

	friend Vector2D		operator*	(const Matrix3D&, const Vector2D&);
	friend Vector2D&	operator*=	(Vector2D& rVec, const Matrix3D& rMat)
		{ return (rVec = rMat * rVec); }

	void RotateAndNormalize(Vector2D& rVec) const;

	friend SvStream& operator>>(SvStream& rIStream, Matrix3D&);
	friend SvStream& operator<<(SvStream& rOStream, const Matrix3D&);

	BOOL DecomposeAndCorrect(Vector2D& rScale, double& rShear, double& rRotate, Vector2D& rTranslate) const;
};

#endif          
