

#ifndef _B3D_POINT4D_HXX
#define _B3D_POINT4D_HXX

#ifndef _SVX_VECTOR3D_HXX 
#include <goodies/vector3d.hxx>
#endif

#ifndef _SVX_POINT3D_HXX
#include <goodies/point3d.hxx>
#endif

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

class Point4D
{
protected:

	double V[4];

	void	ImplHomogenize();

public:
	Point4D(double fX = 0.0, double fY = 0.0, double fZ = 0.0, double fW = 1.0)
		{ V[0] = fX; V[1] = fY; V[2] = fZ; V[3] = fW; }
	Point4D(const Point& rPnt, double fZ = 0.0, double fW = 1.0);
	Point4D(const Vector3D& rVec, double fW = 1.0);

	const double& X() const { return V[0]; }
	const double& Y() const { return V[1]; }
	const double& Z() const { return V[2]; }
	const double& W() const { return V[3]; }

	double& X() { return V[0]; }
	double& Y() { return V[1]; }
	double& Z() { return V[2]; }
	double& W() { return V[3]; }

	const double& operator[] (int nPos) const { return V[nPos]; }
	double& operator[] (int nPos) { return V[nPos]; }

	void	Homogenize() { if(V[3] != 1.0) ImplHomogenize(); }
	void	Min(const Point4D& rVec);
	void	Max(const Point4D& rVec);
	void	Abs();
	const Vector3D& GetVector3D() const;

	void CalcInBetween(Point4D& rOld1, Point4D& rOld2, double t);
	void CalcMiddle(Point4D& rOld1, Point4D& rOld2);
	void CalcMiddle(Point4D& rOld1, Point4D& rOld2, Point4D& rOld3);

	Point4D&	operator+=	(const Point4D&);
	Point4D&	operator-=	(const Point4D&);
	Point4D		operator+ 	(const Point4D&) const;
	Point4D		operator-	(const Point4D&) const;
	Point4D		operator-	(void) const;
	Point4D&	operator*=	(const Point4D&);
	Point4D&	operator/=	(const Point4D&);
	Point4D		operator* 	(const Point4D&) const;
	Point4D		operator/	(const Point4D&) const;

	Point4D&	operator+=	(const Vector3D&);
	Point4D&	operator-=	(const Vector3D&);
	Point4D		operator+ 	(const Vector3D&) const;
	Point4D		operator-	(const Vector3D&) const;

	Point4D&	operator*=	(double);
	Point4D		operator*	(double) const;
	Point4D&	operator/=	(double);
	Point4D		operator/	(double) const;

	BOOL		operator==	(const Point4D&) const;
	BOOL		operator!=	(const Point4D&) const;

	friend SvStream& operator>>(SvStream& rIStream, Point4D&);
	friend SvStream& operator<<(SvStream& rOStream, const Point4D&);
};

#endif          
