
#ifndef _VDEVCACHE_HXX
#define _VDEVCACHE_HXX

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _SV_GEN_HXX
#include <tools/gen.hxx>
#endif

#ifndef _SV_VIRDEV_HXX
#include <vcl/virdev.hxx>
#endif

class ImpVDCache;

class VDevCacheEntry
{

	Point						maPosition;
	Size						maSize;

	VDevCacheEntry*				mpParent;
	VDevCacheEntry*				mpLeft;
	VDevCacheEntry*				mpRight;

	VDevCacheEntry*				mpNext;
	VDevCacheEntry*				mpPrev;

public:
	VDevCacheEntry(const Point& rPos, const Size& rSize, VDevCacheEntry* pParent)
	:	maPosition(rPos), maSize(rSize), mpParent(pParent),
		mpLeft(0L), mpRight(0L), mpNext(0L), mpPrev(0L)
	{}
	~VDevCacheEntry()
	{ if(mpLeft) delete mpLeft; if(mpRight) delete mpRight;	}

	VDevCacheEntry* GetParent() const { return mpParent; }
	VDevCacheEntry* GetLeft() const { return mpLeft; }
	VDevCacheEntry* GetRight() const { return mpRight; }
	const Point& GetPosition() const { return maPosition; }
	const Size& GetSize() const { return maSize; }

	void ImpSetLeftRight(VDevCacheEntry* pNewL, VDevCacheEntry* pNewR) 
		{ mpLeft = pNewL; mpRight = pNewR; }
	void ImpSetParent(VDevCacheEntry* pNew) { mpParent = pNew; }

	BOOL IsSplit() const { return (mpLeft != 0L); }
	void SplitHorizontal(sal_uInt32 nLength);
	void SplitVertical(sal_uInt32 nLength);
	void Melt();

	VDevCacheEntry* GetNext() const { return mpNext; }
	VDevCacheEntry* GetPrev() const { return mpPrev; }
	void SetNext(VDevCacheEntry* pNew) { mpNext = pNew; }
	void SetPrev(VDevCacheEntry* pNew) { mpPrev = pNew; }
	BOOL IsInFreeList() const { return (mpNext || mpPrev); }
};

class VDevCache
{
	ImpVDCache*					mpImp;

public:
	VDevCache(sal_uInt32 nInitialEdgeLen = 256);
	~VDevCache();

	VDevCacheEntry* Allocate(const Point& rSrcPt,
		const Size& rSrcSize, const OutputDevice& rSrcDev);

	void Copy(VDevCacheEntry* pEntry, const Point& rDstPt, OutputDevice& rDstDev);

	void CopyPart(VDevCacheEntry* pEntry, const Point& rDstPt, 
		const Size& rDstSize, const Point& rSrcPntOffset, OutputDevice& rDstDev);

	void Free(VDevCacheEntry* pEntry);

	const VirtualDevice& GetVDev() const;
};

#endif 
