

#ifndef _INETDNS_HXX
#define _INETDNS_HXX "$Revision: 1.2 $"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _INET_MACROS_HXX
#include <inet/macros.hxx>
#endif

class INetCoreDNSHostEntry;

#ifdef _USE_NAMESPACE
namespace inet 
{
#endif
	class INetDNSCache_Impl;
	class INetDNSResolver_Impl;
#ifdef _USE_NAMESPACE
}
#endif 

enum INetCoreDNSResolverStatus
{
	INETCOREDNS_RESOLVER_NOTFOUND = -2,
	INETCOREDNS_RESOLVER_ERROR    = -1,
	INETCOREDNS_RESOLVER_START    =  0,
	INETCOREDNS_RESOLVER_SUCCESS  =  1,
	INETCOREDNS_RESOLVER_EXPIRED  =  2,
	INETCOREDNS_RESOLVER_NODATA   =  3
};

typedef sal_Bool INetCoreDNSCallback (
	sal_Int32             nStatus,
	INetCoreDNSHostEntry *pHostEntry,
	void                 *pData);

class INetCoreDNSResolver
{
public:
	INetCoreDNSResolver (void);
	virtual ~INetCoreDNSResolver (void);

	virtual sal_Bool GetHostName (
		INetCoreDNSHostEntry *pHostEntry);

	virtual sal_Bool GetHostByName (
		INetCoreDNSHostEntry *pHostEntry,
		INetCoreDNSCallback  *pfnCallback,
		void                 *pData);

	virtual sal_Bool GetHostByAddress (
		INetCoreDNSHostEntry *pHostEntry,
		INetCoreDNSCallback  *pfnCallback,
		void                 *pData);

	virtual sal_Bool GetMailExchangerByName (
		INetCoreDNSHostEntry *pHostEntry,
		INetCoreDNSCallback  *pfnCallback,
		void                 *pData);

private:

	NAMESPACE_INET(INetDNSResolver_Impl) *m_pImpl;

	INetCoreDNSResolver (const INetCoreDNSResolver&);
	INetCoreDNSResolver& operator= (const INetCoreDNSResolver&);
};

class INetCoreDNSHostEntry
{
	friend class NAMESPACE_INET(INetDNSCache_Impl);
	friend class NAMESPACE_INET(INetDNSResolver_Impl);

public:
	INetCoreDNSHostEntry (
		const rtl::OUString &rDomainName, sal_uInt16 nPort = 0);

	INetCoreDNSHostEntry (
		const INetCoreDNSHostEntry& rHostEntry);
	INetCoreDNSHostEntry& operator = (
		const INetCoreDNSHostEntry& rHostEntry);

	~INetCoreDNSHostEntry (void);

	const rtl::OUString& GetDomainName        (void) const;
	const rtl::OUString& GetCanonicalName     (void) const;
	const rtl::OUString& GetDottedDecimalName (void) const;
	sal_uInt16           GetPort              (void) const;

private:

	rtl::OUString m_aDotDecName;  
	rtl::OUString m_aCName;       
	rtl::OUString m_aName;        
	sal_uInt16    m_nPort;        
};

inline const rtl::OUString&
INetCoreDNSHostEntry::GetDomainName (void) const
{
	return m_aName;
}

inline const rtl::OUString&
INetCoreDNSHostEntry::GetCanonicalName (void) const
{
	return m_aCName;
}

inline const rtl::OUString&
INetCoreDNSHostEntry::GetDottedDecimalName (void) const
{
	return m_aDotDecName;
}

inline sal_uInt16 INetCoreDNSHostEntry::GetPort (void) const
{
	return m_nPort;
}

#endif 

