

#ifndef _INETCOREMAIL_HXX
#define _INETCOREMAIL_HXX "$Revision: 1.2 $"

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _INETCOREMSG_HXX
#include <inet/inetmsg.hxx>
#endif

#ifndef _INETSMTP_HXX
#include <inet/inetsmtp.hxx>
#endif

#ifndef _INETPOP3_HXX
#include <inet/inetpop3.hxx>
#endif

#ifndef _INETNNTP_HXX
#include <inet/inetnntp.hxx>
#endif

class DateTime;
class List;

typedef INetCorePOP3MailListEntry INetCoreMailInfoListEntry;

#ifdef _USE_NAMESPACE
namespace inet {
#endif

class INetCoreMailer;

typedef sal_Bool INetCoreMailerCallback (
	INetCoreMailer *pMailer,
	sal_Int32       nReplyCode,
	const sal_Char *pReplyText,
	void           *pData);

class INetCoreMailer
{

	INetCoreMailer (const INetCoreMailer&);
	INetCoreMailer& operator= (const INetCoreMailer&);

protected:

	INetCoreMailer (void);

public:

	virtual ~INetCoreMailer (void);

	virtual INetCoreMIMEMessage *CreateINetCoreMIMEMessage (void) const = 0;
	virtual INetCoreNewsMessage *CreateINetCoreNewsMessage (void) const = 0;

	virtual sal_Bool SendOpenConnection (
		const rtl::OUString    &rHost, sal_uInt16 nPort,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool SendIsOpen (void) const = 0;

	virtual sal_Bool SendCloseConnection (
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual void SendAbortConnection (void) = 0;

	virtual sal_Bool SendMail (
		INetCoreMIMEMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_uInt32 SendGetTransferCount (void) const = 0;
	virtual sal_Bool  SendSetTransferCallback (
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool RecvOpenConnection (
		const rtl::OUString &rHost, sal_uInt16 nPort,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool RecvIsOpen (void) const = 0;

	virtual sal_Bool RecvCloseConnection (
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual void RecvAbortConnection (void) = 0;

	virtual sal_Bool RecvLogin (
		const rtl::OUString &rUsername,
		const rtl::OUString &rPassword,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool RecvLoginSecure ( 
		const rtl::OUString &rUsername,
		const rtl::OUString &rPassword,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool RecvIsLoggedIn (void) const = 0;

	virtual sal_Bool RecvUniqueIdList (
		List& rUniqueIdList,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool RecvMailList (
		List& rMailInfoList,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool RecvMailListEntry (
		INetCoreMailInfoListEntry& rEntry,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool RecvMailHead (
		sal_uInt32 nIndex, INetCoreMIMEMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool RecvMail (
		sal_uInt32 nIndex, INetCoreMIMEMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool RecvDeleteMail (
		sal_uInt32 nIndex,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_uInt32 RecvGetTransferCount (void) const = 0;
	virtual sal_Bool  RecvSetTransferCallback (
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool NewsOpenConnection (
		const rtl::OUString &rHost, sal_uInt16 nPort,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool NewsIsOpen (void) const = 0;

	virtual sal_Bool NewsCloseConnection (
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual void NewsAbortConnection (void) = 0;

	virtual sal_Bool NewsModeReader (
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool NewsAuthenticate (
		const rtl::OUString &rUsername,
		const rtl::OUString &rPassword,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool NewsAuthenticatePassword (
		const rtl::OUString &rPassword,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool NewsGroupList (
		List& rGroupList,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool NewsNewGroupsList (
		const DateTime& rSinceDateTime,
		List& rNewGroupsList,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool NewsNewNewsList (
		const rtl::OUString &rGroups,
		const DateTime& rSinceDateTime,
		List& rNewNewsList,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool NewsSelectGroup (
		INetCoreNNTPGroupListEntry& rGroupListEntry,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool NewsHasOverview (void) const = 0;
	virtual sal_Bool NewsArticleHeadOverview (
		sal_uInt32 nFirstArticleNumber,
		sal_uInt32 nLastArticleNumber,
		INetCoreNewsMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool NewsArticleHead (
		sal_uInt32 nArticleNumber,
		INetCoreNewsMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool NewsArticleHead (
		const rtl::OUString &rMessageID,
		INetCoreNewsMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool NewsArticle (
		sal_uInt32 nArticleNumber,
		INetCoreNewsMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;
	virtual sal_Bool NewsArticle (
		const rtl::OUString &rMessageID,
		INetCoreNewsMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_Bool NewsIsPostingAllowed (void) const = 0;
	virtual sal_Bool NewsPostArticle (
		INetCoreNewsMessage& rMessage,
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual sal_uInt32 NewsGetTransferCount (void) const = 0;
	virtual sal_Bool  NewsSetTransferCallback (
		INetCoreMailerCallback *pfnCallback, void *pData) = 0;

	virtual void Dummy1 (void) = 0;
	virtual void Dummy2 (void) = 0;
	virtual void Dummy3 (void) = 0;
};

#ifdef _USE_NAMESPACE
}
#endif

extern "C" {
	NAMESPACE_INET(INetCoreMailer)* SAL_CALL NewINetCoreMailer (void);
}

#endif 

