

#ifndef _LINGUISTIC_IPRCACHE_HXX_
#define _LINGUISTIC_IPRCACHE_HXX_

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

#include <uno/lbnames.h>			
#include <cppuhelper/implbase2.hxx>	

#ifndef _COM_SUN_STAR_DOCUMENT_XEVENTLISTENER_HPP_
#include <com/sun/star/document/XEventListener.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_XPROPERTYCHANGELISTENER_HPP_
#include <com/sun/star/beans/XPropertyChangeListener.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif

#include <com/sun/star/linguistic2/XDictionaryListEventListener.hpp>
#include <com/sun/star/linguistic2/XDictionaryList.hpp>

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

namespace linguistic
{

class IPRCachedWord;

class Flushable
{
public:
	virtual void	Flush() = 0;
};

class FlushListener : 
	public cppu::WeakImplHelper2
	<
		::com::sun::star::linguistic2::XDictionaryListEventListener,
		::com::sun::star::beans::XPropertyChangeListener
	>
{
	::com::sun::star::uno::Reference<
		::com::sun::star::linguistic2::XDictionaryList >	xDicList;
	::com::sun::star::uno::Reference<
		::com::sun::star::beans::XPropertySet >				xPropSet;
	Flushable				   							   *pFlushObj;

	FlushListener(const FlushListener &);
	FlushListener & operator = (const FlushListener &);

public:
	FlushListener( Flushable *pFO );
	virtual ~FlushListener();

	inline void	SetFlushObj( Flushable *pFO)	{ pFlushObj = pFO; }

	void		SetDicList( 
		::com::sun::star::uno::Reference< 
			::com::sun::star::linguistic2::XDictionaryList > &rDL );
	void		SetPropSet( 
		::com::sun::star::uno::Reference< 
			::com::sun::star::beans::XPropertySet > &rPS );

	virtual void SAL_CALL 
		disposing( const ::com::sun::star::lang::EventObject& rSource ) 
			throw(::com::sun::star::uno::RuntimeException);
	 virtual void SAL_CALL 
		processDictionaryListEvent( 
	const ::com::sun::star::linguistic2::DictionaryListEvent& rDicListEvent ) 
			throw(::com::sun::star::uno::RuntimeException);
	 virtual void SAL_CALL 
		propertyChange( const ::com::sun::star::beans::PropertyChangeEvent& rEvt ) 
			throw(::com::sun::star::uno::RuntimeException);
};

class IPRSpellCache :
	public Flushable
{
	::com::sun::star::uno::Reference<
		::com::sun::star::linguistic2::XDictionaryListEventListener >
			xFlushLstnr;
	FlushListener	   *pFlushLstnr;

	IPRCachedWord	**ppHash;
	IPRCachedWord	 *pFirst;
	IPRCachedWord	 *pLast;
	IPRCachedWord	 *pRun;
	IPRCachedWord	 *pInput;
	ULONG	 		  nIndex;
	ULONG 			  nCount;
	ULONG	 		  nInputPos;
	ULONG 			  nInputValue;
	ULONG 			  nTblSize;
#ifdef DBG_STATISTIC
	ULONG nMaxInput;
	ULONG nMax;
	ULONG nFound;
	ULONG nLost;
#endif

	IPRSpellCache(const IPRSpellCache &);
	IPRSpellCache & operator = (const IPRSpellCache &);

public:
	IPRSpellCache( ULONG nSize );
	virtual ~IPRSpellCache();

	virtual void	Flush();

	void	AddWord( const String& rWord, INT16 nLang );
	BOOL	CheckWord( const String& rWord, INT16 nLang, BOOL bAllLang );
};

}	

#endif

