

#ifndef _SCH_DLL_HXX
#define _SCH_DLL_HXX

#include <sfx2/sfxdefs.hxx>

#ifndef _SFXMODULE_HXX
#include <sfx2/module.hxx>
#endif

#ifndef _IPOBJ_HXX
#include <so3/ipobj.hxx>
#endif

#include <sch/chattr.hxx>	

#define extern_c extern "C"

class SvFactory;
class SchDragServer;
class SchMemChart;
class SfxMedium;
class SfxFilter;
class SvInPlaceObjectRef;
class OutputDevice;
class Window;
class XOutdevItemPool;
class SvStream;
class SfxTabDialog;
class ChartModel;

class SchDLL
{
public:

	SchDLL();
	~SchDLL();

	static void     Init();
	static void     Exit();
	static void     LibInit();
	static void     LibExit();

	static ULONG DetectFilter( SfxMedium& rMedium, const SfxFilter **pFilter,
				   SfxFilterFlags nMust, SfxFilterFlags nDont );

	static void     Update( SvInPlaceObjectRef aIPObj, SchMemChart* pData, OutputDevice* pOut=NULL);
	static void     Update( SvInPlaceObjectRef aIPObj, SchMemChart* pData,
				const SfxItemSet& rAttr, OutputDevice* pOut = NULL );

	static ModalDialog* CreateAutoPilotDlg( Window* pParent, SchMemChart* pData,
								const SfxItemSet& rInAttrs,
								SfxItemSet& rOutAttrs,
								BOOL bPrePage = FALSE );
	static void ChangeChartData( ModalDialog* pDlg, SchMemChart* pData );

	static SfxTabDialog* CreateOptionsDlg( Window* pParent, const SfxItemSet& rInAttrs, XOutdevItemPool* pItemPool );
	static SchMemChart*  GetChartData( SvInPlaceObjectRef aIPObj );

	static SchMemChart* NewMemChart();
	static SchMemChart* NewMemChart( short nCols, short nRows );
	static SchMemChart* NewMemChart( const SchMemChart &rMemChart );
	static ChartModel*  GetModel( SvInPlaceObjectRef aIPObj );

	static void SetTransparentBackground( SvInPlaceObjectRef aIPObj, BOOL bTransp = TRUE );
	static void GetDefaultForColumnText( const SchMemChart& rMemChart, sal_Int32 nCol, String& aResult );
	static void GetDefaultForRowText( const SchMemChart& rMemChart, sal_Int32 nRow, String& aResult );

	static void ConvertChartRangeForWriter( SchMemChart& rMemChart,
								BOOL bOldToNew );
	static void ConvertChartRangeForCalc( SchMemChart& rMemChart,
								BOOL bOldToNew );

	static void MemChartResetTranslation( SchMemChart& rMemChart,
								long *pTable,long nCnt );
	static void MemChartUpdateTranslation( SchMemChart& rMemChart,
								   long *pTable,long nCnt );

	static void MemChartInsertCols( SchMemChart& rMemChart,
						short nAtCol, short nCount);
	static void MemChartRemoveCols( SchMemChart& rMemChart,
						short nAtCol, short nCount);

	static void MemChartInsertRows( SchMemChart& rMemChart,
						short nAtRow, short nCount);
	static void MemChartRemoveRows( SchMemChart& rMemChart,
						short nAtRow, short nCount);

	static void MemChartSwapCols( SchMemChart& rMemChart,
					  int nAtCol1, int nAtCol2);
	static void MemChartSwapRows( SchMemChart& rMemChart,
					  int nAtRow1,int nAtRow2);

};

extern_c void __LOADONCALLAPI SchUpdate( SvInPlaceObjectRef aIPObj, SchMemChart* pData, OutputDevice* pOut = NULL );
extern_c void __LOADONCALLAPI SchUpdateAttr( SvInPlaceObjectRef aIPObj, SchMemChart* pData,
								 const SfxItemSet& rAttr, OutputDevice* pOut = NULL );
extern_c ModalDialog* __LOADONCALLAPI SchCreateDiagramAutoPilotDlg( Window* pParent, SchMemChart* pData,
														const SfxItemSet& rInAttrs,
														SfxItemSet& rOutAttrs,
														BOOL bPrePage = FALSE );
extern_c void __LOADONCALLAPI SchChangeChartData( ModalDialog* pDlg, SchMemChart* pData );
extern_c SfxTabDialog* __LOADONCALLAPI SchCreateOptionsDlg( Window* pParent,
												const SfxItemSet& rInAttrs,
												XOutdevItemPool* pItemPool );
extern_c SchMemChart* __LOADONCALLAPI SchGetChartData( SvInPlaceObjectRef aIPObj );
extern_c SchMemChart* __LOADONCALLAPI SchNewMemChartNone();
extern_c SchMemChart* __LOADONCALLAPI SchNewMemChartXY( short nCols, short nRows );
extern_c SchMemChart* __LOADONCALLAPI SchNewMemChartCopy( const SchMemChart &rMemChart );
extern_c ChartModel*  __LOADONCALLAPI SchGetModel( SvInPlaceObjectRef aIPObj );
extern_c void __LOADONCALLAPI SetTransparentBackground( SvInPlaceObjectRef aIPObj, BOOL bTransp = TRUE );
extern_c void __LOADONCALLAPI SchGetDefaultForColumnText( const SchMemChart& rMemChart, sal_Int32 nCol, String& aResult );
extern_c void __LOADONCALLAPI SchGetDefaultForRowText( const SchMemChart& rMemChart, sal_Int32 nRow, String& aResult );

#endif          

