

#ifndef _HASHTBL_HXX
#define _HASHTBL_HXX

#include <tools/solar.h>

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif

#ifndef _DEBUG_HXX 
#include <tools/debug.hxx>
#endif

class HashItem;

class HashTable
{
	ULONG     m_lSize;
	ULONG     m_lElem;
	HashItem *m_pData;
	double    m_dMaxLoadFactor;
	double    m_dGrowFactor;
	BOOL      m_bOwner;

	ULONG Hash(ByteString const& Key) const;
	ULONG DHash(ByteString const& Key, ULONG lHash) const;
	ULONG Probe(ULONG lPos) const;

	HashItem* FindPos(ByteString const& Key) const;
	void      SmartGrow();
	double    CalcLoadFactor() const;

#ifdef DBG_UTIL
private:
	struct
	{
		ULONG m_lSingleHash;
		ULONG m_lDoubleHash;
		ULONG m_lProbe;
	}
		m_aStatistic;
#endif

protected:
	friend class HashTableIterator;

	virtual void OnDeleteObject(void* pObject);

	void* GetObjectAt(ULONG lPos) const;

public:
	static double m_defMaxLoadFactor;
	static double m_defDefGrowFactor;

public:
	HashTable
	(
		ULONG	lSize,
		BOOL	bOwner,
		double	dMaxLoadFactor = HashTable::m_defMaxLoadFactor ,
		double	dGrowFactor = HashTable::m_defDefGrowFactor 
	);

	~HashTable();

	BOOL  IsFull() const;
	ULONG GetSize() const { return m_lSize; }

	void* Find   (ByteString const& Key) const;
	BOOL  Insert (ByteString const& Key, void* pObject);
	void* Delete (ByteString const& Key);
};

class HashTableIterator
{
	ULONG			 m_lAt;
	HashTable const& m_aTable;

	void* FindValidObject(BOOL bForward);

protected:
	void* GetFirst(); 
	void* GetNext();
	void* GetLast();
	void* GetPrev();

public:
	HashTableIterator(HashTable const&);
};

#define DECLARE_HASHTABLE_INTERN(ClassName,Owner,KeyType,ObjType)       \
	class ClassName : public HashTable                                  \
	{                                                                   \
	public:                                                             \
	    ClassName														\
		(																\
			ULONG	lSize,												\
			double	dMaxLoadFactor = HashTable::m_defMaxLoadFactor,		\
			double	dGrowFactor = HashTable::m_defDefGrowFactor			\
		)																\
	    : HashTable(lSize,Owner,dMaxLoadFactor,dGrowFactor) {}          \
	                                                                    \
	    ObjType  Find (KeyType const& Key) const                        \
	    { return (ObjType) HashTable::Find(ByteString(Key)); }              \
	                                                                    \
	    BOOL Insert (KeyType const& Key, ObjType Object)                \
	    { return HashTable::Insert(ByteString(Key), (void*) Object); }      \
	                                                                    \
	    ObjType  Delete (KeyType const&Key)                             \
	    { return (ObjType) HashTable::Delete (ByteString(Key)); }           \
	};

#define DECLARE_HASHTABLE(ClassName,KeyType,ObjType)                 \
	DECLARE_HASHTABLE_INTERN(ClassName,FALSE,KeyType,ObjType)

#define DECLARE_HASHTABLE_OWNER(ClassName,KeyType,ObjType)           \
	DECLARE_HASHTABLE_INTERN(ClassName##2,TRUE,KeyType,ObjType)      \
	class ClassName : public ClassName##2                            \
	{                                                                \
	protected:                                                       \
	    virtual void OnDeleteObject(void* pObject);                  \
	public:                                                          \
	    ClassName													 \
		(															 \
			ULONG	lSize,											 \
			double	dMaxLoadFactor = HashTable::m_defMaxLoadFactor,	 \
			double	dGrowFactor = HashTable::m_defDefGrowFactor		 \
		)															 \
	    : ClassName##2(lSize,dMaxLoadFactor,dGrowFactor) {}			 \
	    ~ClassName();                                                \
	};

#define IMPLEMENT_HASHTABLE_OWNER(ClassName,KeyType,ObjType)         \
	void ClassName::OnDeleteObject(void* pObject)                    \
	{ delete (ObjType) pObject; }                                    \
	                                                                 \
	ClassName::~ClassName()                                          \
	{                                                                \
	    for (ULONG i=0; i<GetSize(); i++)                            \
	    {                                                            \
	        void *pObject = GetObjectAt(i);                          \
	        if (pObject != NULL)                                     \
	            OnDeleteObject(pObject);                             \
	    }                                                            \
	}

#define DECLARE_HASHTABLE_ITERATOR(ClassName,ObjType)				\
	class ClassName : public HashTableIterator						\
	{																\
	public:															\
		ClassName(HashTable const& aTable)							\
		: HashTableIterator(aTable)	{}								\
														\
		ObjType GetFirst()		 									\
			{ return (ObjType)HashTableIterator::GetFirst(); }		\
		ObjType GetNext()		 									\
			{ return (ObjType)HashTableIterator::GetNext();  }		\
		ObjType GetLast()		 									\
			{ return (ObjType)HashTableIterator::GetLast();  }		\
		ObjType GetPrev()		 									\
			{ return (ObjType)HashTableIterator::GetPrev();  }		\
	};

#endif 

