

#ifndef __SETUP_OS_HXX
#define __SETUP_OS_HXX

#include <com/sun/star/setup/OSType.hpp>
#include <setup2/sifsys.hxx>

class Window;

class OS
{
	static ULONG GetFreeDriveSize(USHORT nDrive);

public:
	static const char* GetAllFilesWildCard();

	static ByteString GetGUIPath();
	static ByteString GetSystemPath();
	static ByteString GetSystemFontPath();
	static ByteString GetIEDir();
	static ByteString GetIEPluginDir( BOOL bPlugin = TRUE );
	static ByteString GetNSPluginDir();
	static ByteString GetNS6PluginDir();

	static ULONG  GetDriveSize(SiDirEntry const&);
	static ULONG  GetClusterSize(SiDirEntry const&);
	static void   SetDateTime (ByteString const& rFilename, Date const&, Time const&);
	static BOOL	  MakeWritable(ByteString const& rFilename);

	static BOOL   InstallFont (ByteString const& rFontFile, ByteString const& rFontName);
	static BOOL   UnInstallFont(ByteString const& rFontFile, ByteString const& rFontName);
	static BOOL   IsFontInstalled(ByteString const& rFontFile,ByteString const& rFontName);

	static com::sun::star::setup::OSType GetType();
	static BOOL   IsConfigurationValid(Window*);
};

#if defined WIN || defined WNT
	inline com::sun::star::setup::OSType OS::GetType() { return com::sun::star::setup::OSType_WIN; }
#elif defined OS2
	inline com::sun::star::setup::OSType OS::GetType() { return com::sun::star::setup::OSType_OS2; }
#elif defined MAC
	inline com::sun::star::setup::OSType OS::GetType() { return com::sun::star::setup::OSType_MAC; }
#elif defined UNX
	inline com::sun::star::setup::OSType OS::GetType() { return com::sun::star::setup::OSType_UNIX_SOLS; }
#else
	inline com::sun::star::setup::OSType OS::GetType() { return com::sun::star::setup::OSType_INVALID; }
#endif

#if defined (WIN) || defined (WNT)

class WinOS
{
public:
	static UINT32		GetFreeMemory();
	static BOOL  		ExecDDECommand(ByteString const& aReceiver, ByteString const& aCommand);

	static BOOL 		CreateShortcut( const UniString& rAbsObject, const UniString& rAbsObjectPath,
				const UniString& rAbsShortcut, const UniString& rDescription,
				const UniString& rParameter );

	static UniString		SHGetUserProgramFilesFolder();
	static UniString		SHGetUserProgramFolder();
	static UniString		SHGetFavoritesFolderName();
	static UniString		SHGetPersonalFolderName();
	static UniString    	SHGetTemplateFolderName();

	static UniString 		SHGetAutostartFolderName();
	static UniString		SHGetSystemDesktopFolderName();

	static BOOL				SHDoShowExtension();

	static long  		Register(ByteString const& rKey, ByteString const& rSubKey,
				ByteString const& rData, ByteString const& rVal, BOOL bHex, BOOL bExpand_SZ = FALSE );
	static long			Deregister(ByteString const& rKey, ByteString const& rSubKey,
				BOOL bKeyOnly, ByteString const& rName,	BOOL bDeleteAllSubKeys);
	static BOOL			IsRegistered(ByteString const& rKey, ByteString const& rSubKey,
				ByteString const& rName, BOOL bHasValue);
	static long			ClearRegEdit(ByteString const& rSubKey,
				ByteString const& rStart, ByteString const& rEnd);
	static ByteString	GetRegValue(ByteString const& rKey, ByteString const& rSubKey,
				ByteString const& rName);

	static BOOL	  		BindBinary(ByteString aBinName);
	static BOOL   		IsSourceFileNewerVersion
	(
		ByteString const& aSourceFile,
		BOOL          bPacked,
		ByteString const& aDestFile
	);

	static ByteString	GetOSVersion();
	static void 		InstallShare();
	static void			Shutdown( BOOL bLogoutOnly );
	static void			PropagateEnvironmentChange();
};

#endif 

#ifdef OS2

#define OS2_VER_211 	211
#define OS2_VER_WARP3	230
#define OS2_VER_WARP4	240

class Os2OS
{
public:
	static USHORT	GetOSVersion();
	static ByteString	GetBootDrive();
	static void		Shutdown();

	static BOOL DoesTempFileExists();
	static void KillTempFile();

	static ByteString GetFavoritesFolderName();  

	static ByteString ReadOS2Ini(ByteString const& rAppName, ByteString const& rKey);
	static BOOL   WriteOS2Ini(ByteString const& rAppName,
			ByteString const& rKey, ByteString const& rValue);

	static ULONG  RegisterClass(ByteString const& rClass, ByteString const& rDLL);
	static ULONG  DeregisterClass(ByteString const& rClass);
	static ULONG  RegisterTemplateIcon(const ByteString& aClassID, const ByteString& aTemplateID,
						   const ByteString& aIconPath);

	static BOOL   ReplaceModule(const char* pDest, const char* pSource);

	static BOOL   DeleteTemplate(ByteString const& aTemplateID);

	static BOOL CreateFolder
	(
		ByteString const& aFolderID,    
		ByteString const& aFolderName,  
		ByteString const& aOpenIcon,
		ByteString const& aCloseIcon
	);

	static BOOL CreateFolderItem
	(
		ByteString const& aFolderID,    
		ByteString const& anItemID,     
		ByteString const& anItemName,   
		ByteString const& aClassName,   
		ByteString const& aCreater,
		ByteString const& aFile,
		ByteString const& aFilterList
	);

	static BOOL CreateFolderItem
	(
		ByteString const& aFolderID,    
		ByteString const& anItemID,     
		ByteString const& anItemName,   
		ByteString const& aReference
	);

	static BOOL DeleteFolder(ByteString const& rFolderID);
	static BOOL IsFolderInstalled(ByteString const& rFolderID);

	static BOOL DeleteFolderItem(ByteString const& rFolderItemID);
	static BOOL IsFolderItemInstalled(ByteString const& rFolderID);

	static ByteString GetAutostartFolderID()
	{ return "WP_START"; }

	static BOOL SetFileTypeAndIcon
	(
		ByteString const& aFile,
		ByteString const& aFileType,
		ByteString const& anIcon
	);

	static BOOL JoinEAs(ByteString const& aFile, ByteString const& anEAFile);
	static BOOL IsFATFileSystem(const ByteString& rPath);

	static BOOL AddDestPathToConfigSys(BOOL bModifyCopy, ByteString const &rDestPath);
	static BOOL RemoveDestPathFromConfigSys(ByteString const &rDestPath);
};

#endif 

#ifdef UNX

struct passwd;

class UnixOS
{
	static passwd*	GetPasswdEntry( const ByteString& rUserName );
public:
	static BOOL  	IsRoot();

	static void		SetRights(ByteString const& aFile, int nRights);
	static void		ChangeOwnerRecursive( const ByteString aPath, const ByteString& rUserName );

	static BOOL		CreateSoftLink( ByteString const& aSource, ByteString const& aDest );
	static BOOL		CreateHardLink( ByteString const& aSource, ByteString const& aDest );
	static BOOL		ConvertShellScript( SiDirEntry const& aDest, SiDirEntry const& aSource,
							ByteString const& rWorkPath, ByteString const& rProgPath );

	static BOOL		GetUserHomeDir( const ByteString& rUserName, SiDirEntry& rEntry );
	static SiDirEntry GetHomeDir(void);
	static SiDirEntry GetProfile(void);

	static ByteString	InsertLine( SiDirEntry const& aNewProfile, ByteString const& aLine );

	static char* 	p_get_localhost();
	static char* 	p_get_exechost();
	static BOOL	 	bSOfficeRemoteApplication();
	static ByteString&	rGetExecHostAction();
};

#endif

#ifdef MAC

class MacOS
{
public:
	static BOOL     AddApplication(ByteString const& rAppName);
	static ByteString   SearchApplication(ByteString Creator );
	static BOOL     RemoveApplication(ByteString const& rAppName, ByteString Creator );

	static BOOL		IsFloppy(ByteString const& rPath);
	static void		EjectDisk(ByteString const& rPath);

	static void		Shutdown();
	static ByteString	GetSystemVolumeName();
	static BOOL		IsKeyPressed(int nKey);
};

#endif

#endif 

