

#ifndef SETUP_PARSER_HXX
#define SETUP_PARSER_HXX

#if SUPD < 364

#else
#ifndef _STREAM_HXX 
#include <tools/stream.hxx>
#endif
#endif

#include <setup2/lexem.hxx>

class SiScanner;

class SiParser
{
	BOOL 	   m_bHaveApp;

	SiScanner *m_pScanner;
	BOOL       m_bSyntaxError;

	BOOL IsDeclarator(SiLexem const&) const;
	BOOL ParseDeclaration();
	BOOL ParsePropertyList();
	BOOL ParseProperty();
	BOOL ParseValueList();

protected:
	virtual void OnDeclaration    (SiLexemType,SiIdentifier const&)		{};
	virtual void OnDeclarationEnd ()                 					{};
	virtual void OnProperty       (ByteString const&, int nLanguage)	{};
	virtual void OnSingleValue    (sal_Int32)  							{};
	virtual void OnSingleValue    (sal_uInt32)  		 				{};
	virtual void OnSingleValue    (ByteString const&)    			   	{};
	virtual void OnSingleValue    (SiIdentifier const&)					{};
	virtual void OnListValue      (sal_Int32)		  					{};
	virtual void OnListValue      (ByteString const&)					{};
	virtual void OnListValue      (SiIdentifier const&)					{};

	enum ExpectedLexem
	{
		EXP_DECLARATOR,
		EXP_END,
		EXP_POPEN,
		EXP_PCLOSE,
		EXP_IDENTIFIER,
		EXP_ASSIGN,
		EXP_VALUE,          
		EXP_SINGLEVALUE,    
		EXP_INTEGER,
		EXP_SEMICOLON
	};

	void SyntaxError(SiLexem const&, ExpectedLexem);

	virtual void DisplayError(
		int nLine,
		ByteString const& ErrorType,
		ByteString const& Message) const;

	virtual void ResetError();

public:
	SiParser(SvStream &);
	~SiParser();

	void		NoApplication()		   				{ m_bHaveApp = FALSE; }
	BOOL		IsApplication() const  				{ return m_bHaveApp; }

	BOOL		ParseScript();
	void		Recover();
	BOOL		Continue();
	BOOL		IsSyntaxError() const				{ return m_bSyntaxError; }
	int 		GetLine() const;
};

#endif 

