
#ifndef _SFX_DINFDLG_HXX
#define _SFX_DINFDLG_HXX

#ifndef _SV_EDIT_HXX 
#include <vcl/edit.hxx>
#endif
#ifndef _SV_FIELD_HXX 
#include <vcl/field.hxx>
#endif
#ifndef _STDCTRL_HXX 
#include <svtools/stdctrl.hxx>
#endif
#ifndef _SFXSTRITEM_HXX 
#include <svtools/stritem.hxx>
#endif
#ifndef _SVEDIT_HXX 
#include <svtools/svmedit.hxx>
#endif

#include <sfx2/tabdlg.hxx>
#include <sfx2/docinf.hxx>

class SfxDocumentInfoItem : public SfxStringItem
{
private:
	SfxDocumentInfo         aDocInfo;
	FASTBOOL                bHasTemplate;
	FASTBOOL                bOwnFormat;

public:
	TYPEINFO();
	SfxDocumentInfoItem();
	SfxDocumentInfoItem( const String &rFileName, const SfxDocumentInfo &, BOOL );
	SfxDocumentInfoItem( const String &rFileName, const SfxDocumentInfo & );
	SfxDocumentInfoItem( const SfxDocumentInfoItem& );
	virtual ~SfxDocumentInfoItem();

	BOOL                    IsOwnFormat() const;

	void                    SetTemplate( BOOL b ) { bHasTemplate = b; }
	FASTBOOL                HasTemplate() const { return bHasTemplate; }

	SfxDocumentInfo&        operator()() { return aDocInfo; }
	const SfxDocumentInfo&  operator()() const { return aDocInfo; }
	SfxDocumentInfo&        GetDocInfo() { return aDocInfo; }
	const SfxDocumentInfo&  GetDocInfo() const { return aDocInfo; }

	virtual SfxPoolItem*    Clone( SfxItemPool* pPool = NULL ) const;
	virtual int             operator==( const SfxPoolItem& ) const;

};

class SfxDocumentPage : public SfxTabPage
{
private:
	FixedImage	aBmp1;
	Edit		aNameED;

	FixedLine 	aLine1FL;
	FixedText 	aTypeFT;
	FixedText 	aShowTypeFT;
	CheckBox	aReadOnlyCB;
	FixedText 	aFileFt;
	FixedInfo 	aFileValFt;
	FixedText 	aSizeFT;
	FixedText 	aShowSizeFT;

	FixedLine 	aLine2FL;
	FixedText 	aCreateFt;
	FixedInfo 	aCreateValFt;
	FixedText 	aTimeLogFt;
	FixedInfo 	aTimeLogValFt;
	FixedText 	aChangeFt;
	FixedInfo 	aChangeValFt;
	FixedText 	aDocNoFt;
	FixedInfo 	aDocNoValFt;
	FixedText 	aPrintFt;
	FixedInfo 	aPrintValFt;
	PushButton  aDeleteBtn;
	CheckBox	aUseUserDataCB;

	FixedLine 	aLine3FL;
	FixedText 	aTemplFt;
	FixedInfo 	aTemplValFt;

	String      aUnknownSize;
	BOOL		bEnableUseUserData	: 1,
	bHandleDelete		: 1;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( DeleteHdl, PushButton * );
#endif

protected:
	SfxDocumentPage( Window* pParent, const SfxItemSet& );

	virtual BOOL		FillItemSet( SfxItemSet& );
	virtual void		Reset( const SfxItemSet& );

public:
	static SfxTabPage*	Create( Window* pParent, const SfxItemSet& );

	void				EnableUseUserData();
};

class SfxDocumentDescPage : public SfxTabPage
{
private:
	FixedText				aTitleFt;
	Edit					aTitleEd;
	FixedText				aThemaFt;
	Edit					aThemaEd;
	FixedText				aKeywordsFt;
	Edit					aKeywordsEd;
	FixedText				aCommentFt;
	MultiLineEdit			aCommentEd;
	SfxDocumentInfoItem*	pInfoItem;

protected:
	SfxDocumentDescPage( Window* pParent, const SfxItemSet& );

	virtual BOOL        	FillItemSet( SfxItemSet& );
	virtual void        	Reset( const SfxItemSet& );

public:
	static SfxTabPage*		Create( Window* pParent, const SfxItemSet& );
};

class SfxDocumentUserPage : public SfxTabPage
{
private:
#if SUPD < 638
	String					aShortcuts;
#endif
	BOOL					bLabelModified;

	FixedText				aInfo1Ft;
	Edit					aInfo1Ed;
	FixedText				aInfo2Ft;
	Edit					aInfo2Ed;
	FixedText				aInfo3Ft;
	Edit					aInfo3Ed;
	FixedText				aInfo4Ft;
	Edit					aInfo4Ed;
	PushButton				aEditLabelBtn;
	SfxDocumentInfoItem*	pInfoItem;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( EditLabelHdl, PushButton * );

	String				GetLabelText_Impl( FixedText* pLabel );
	void				SetLabelText_Impl( FixedText* pLabel, const String& rNewLabel );
#endif

protected:
	SfxDocumentUserPage( Window* pParent, const SfxItemSet& );

	virtual BOOL		FillItemSet( SfxItemSet& );
	virtual void		Reset( const SfxItemSet& );

public:
	static SfxTabPage*	Create( Window* pParent, const SfxItemSet& );
};

class TargetList;

class SfxInternetPage : public SfxTabPage
{
private:
	CheckBox				aReloadEnabled;
	NumericField			aReloadDelay;
	FixedText				aFTSeconds;
	FixedText				aFTURL;
	Edit					aReloadURL;
	FixedText				aFTTarget;
	ComboBox				aTargets;

	String					aOldTarget;
	SfxDocumentInfoItem*	pInfoItem;

	DECL_LINK( ClickHdl, Control* );

protected:
	SfxInternetPage( Window* pParent, const SfxItemSet& );

	virtual BOOL			FillItemSet( SfxItemSet& );
	virtual void			Reset( const SfxItemSet& );

public:
	static SfxTabPage*	Create( Window* pParent, const SfxItemSet& );
};

class SfxDocumentInfoDialog : public SfxTabDialog
{
protected:
	virtual void	PageCreated( USHORT nId, SfxTabPage& rPage );

public:
	SfxDocumentInfoDialog(	Window* pParent, const SfxItemSet& );
};

#endif

