
#ifndef _SFXDISPATCH_HXX
#define _SFXDISPATCH_HXX

#include <stdarg.h>

#define _SVSTDARR_USHORTS
#include <svtools/svstdarr.hxx>		

#include <sfx2/bindings.hxx>
#include <sfx2/app.hxx>

class SfxSlotServer;
class SfxShell;
class SfxRequest;
class SfxShellStack_Impl;
class SfxHintPoster;
class SfxViewFrame;
class SfxBindings;
class SfxItemSet;

struct SfxDispatcher_Impl;
struct SfxPlugInInfo_Impl;

namespace com
{
	namespace sun
	{
	    namespace star
	    {
	        namespace frame
	        {
	            class XDispatch;
	        }
	    }
	}
}

#define SFX_SHELL_POP_UNTIL 	4
#define SFX_SHELL_POP_DELETE    2
#define SFX_SHELL_PUSH			1

typedef SfxPoolItem* SfxPoolItemPtr;
SV_DECL_PTRARR_DEL( SfxItemPtrArray, SfxPoolItemPtr, 4, 4 );

typedef SfxItemPtrArray SfxItemArray_Impl;

class SfxExecuteItem : public SfxItemPtrArray, public SfxPoolItem
{
	USHORT nSlot;
	SfxCallMode eCall;
	USHORT nModifier;
public:
	USHORT                   GetSlot() const { return nSlot; }
	USHORT                   GetModifier() const { return nModifier; }
	void                     SetModifier( USHORT nModifierP ) { nModifier = nModifierP; }
	SfxCallMode              GetCallMode() const { return eCall; }
	void                     SetCallMode( SfxCallMode eMode ) { eCall = eMode; }
	virtual int 			 operator==( const SfxPoolItem& ) const;

	virtual SfxPoolItem*	 Clone( SfxItemPool *pPool = 0 ) const;
				 SfxExecuteItem(
					 USHORT nWhich, USHORT nSlot, SfxCallMode eMode,
					 const SfxPoolItem *pArg1, ... );
				 SfxExecuteItem(
					 USHORT nWhich, USHORT nSlot, SfxCallMode eMode );
				 SfxExecuteItem( const SfxExecuteItem& );
};

class SfxDispatcher
{
	SfxDispatcher_Impl* 			pImp;
	BOOL							bFlushed;

private:

	BOOL				CheckVirtualStack( const SfxShell& rShell, BOOL bDeep );

#ifndef _SFX_HXX

friend class SfxApplication;
friend class SfxViewFrame;

	DECL_LINK( EventHdl_Impl, void * );
	DECL_LINK( PostMsgHandler, SfxRequest * );

	int 				Call_Impl( SfxShell& rShell, const SfxSlot &rSlot, SfxRequest &rReq, BOOL bRecord );
	ULONG               _Update_Impl( BOOL,BOOL,BOOL,SfxMenuBarManager*,SfxWorkWindow*);
	void				CollectTools_Impl(SfxWorkWindow*);

protected:
friend class SfxBindings;
friend class SfxStateCache;
friend class SfxPopupMenuManager;
friend class SfxHelp;

	BOOL				_TryIntercept_Impl( USHORT nId, SfxSlotServer &rServer, BOOL bModal );
	BOOL				_FindServer( USHORT nId, SfxSlotServer &rServer, BOOL bModal );
	BOOL				_FillState( const SfxSlotServer &rServer,
						SfxItemSet &rState, const SfxSlot *pRealSlot );
	const SfxPoolItem*	_Execute( const SfxSlotServer &rServer );
	void				_Execute( SfxShell &rShell, const SfxSlot &rSlot,
					  SfxRequest &rReq,
					  SfxCallMode eCall = SFX_CALLMODE_STANDARD);
	const SfxPoolItem*	_Execute( USHORT nSlot, SfxCallMode eCall,
					  va_list pArgs, const SfxPoolItem *pArg1 );

#endif
protected:
	void				FlushImpl();

public:
			SfxDispatcher( SfxDispatcher* pParent );
			SfxDispatcher( SfxViewFrame *pFrame = 0 );

	void                Construct_Impl( SfxDispatcher* pParent );
				~SfxDispatcher();

	long                UpdateObjectMenus_Impl(SfxMenuBarManager*);

	const SfxPoolItem*  Execute( const SfxExecuteItem& rItem );
	virtual	USHORT		ExecuteFunction( USHORT nSID, SfxPoolItem** ppArgs=0, USHORT nMode=0 );
	USHORT              ExecuteFunction( USHORT nSID, const SfxItemSet& rArgs , USHORT nMode=0 );

	virtual void		SetExecuteMode( USHORT );

	const SfxPoolItem*	Execute( USHORT nSlot,
					 SfxCallMode nCall = SFX_CALLMODE_SLOT,
					 const SfxPoolItem **pArgs = 0,
					 USHORT nModi = 0,
					 const SfxPoolItem **pInternalArgs = 0);

	const SfxPoolItem*	Execute( USHORT nSlot,
					 SfxCallMode nCall,
					 const SfxPoolItem *pArg1, ... );

	const SfxPoolItem*	Execute( USHORT nSlot,
					 SfxCallMode nCall,
					 const SfxItemSet &rArgs );

	USHORT				GetSlotId( const String& rCommand );

	BOOL				IsActive( const SfxShell& rShell );
	BOOL				IsOnTop( const SfxShell& rShell );
	USHORT				GetShellLevel( const SfxShell &rShell );
	SfxBindings*		GetBindings() const;

	void				Push( SfxShell& rShell );
	void				Pop( SfxShell& rShell, USHORT nMode = 0 );

	SfxShell*			GetShell(USHORT nIdx) const;
	SfxViewFrame*       GetFrame() const;

	void				ExecutePopup( const ResId &rId,
				  Window *pWin = 0, const Point *pPosPixel = 0 );
	static void 		ExecutePopup( USHORT nConfigId = 0,
				  Window *pWin = 0, const Point *pPosPixel = 0 );
	static void 		ExecutePopup( USHORT nConfigId,
						  Window *pWin, const Point *pPosPixel,
						  const SfxPoolItem *pArg1, ...  );

	void				EnterAction( const String& rName );
	void				LeaveAction();

	BOOL				IsAppDispatcher() const;
	BOOL				IsFlushed() const;
	void				Flush();
	void				Lock( BOOL bLock );
	BOOL				IsLocked( USHORT nSID = 0 ) const;
	void				SetSlotFilter( BOOL bEnable = FALSE,
						   USHORT nCount = 0, const USHORT *pSIDs = 0 );

	void				HideUI( BOOL bHide = TRUE );
	void				ShowObjectBar(USHORT nId, SfxShell *pShell=0) const;
	USHORT				GetObjectBarId( USHORT nPos ) const;

	SfxItemState		QueryState( USHORT nSID, const SfxPoolItem* &rpState );

	BOOL                IsAllowed( USHORT nSlot ) const;
	::com::sun::star::frame::XDispatch*          GetDispatchInterface( const String& );
	void				SetDisableFlags( ULONG nFlags );
	ULONG				GetDisableFlags() const;

#if 0 // _SOLAR__PRIVATE
	BOOL				HasSlot_Impl( USHORT );
	void 				SetMenu_Impl();
	long                Update_Impl( BOOL bForce = FALSE ); 
	BOOL                IsUpdated_Impl() const;
	void				DebugOutput_Impl() const;
	void				ResetObjectBars_Impl();
	int 				GetShellAndSlot_Impl( USHORT nSlot, SfxShell **ppShell, const SfxSlot **ppSlot,
								  BOOL bOwnShellsOnly, BOOL bModal );
	void				LockUI_Impl( BOOL bLock = TRUE );
	void				SetReadOnly_Impl( BOOL  bOn );
	BOOL				GetReadOnly_Impl() const;
	BOOL				IsSlotEnabledByFilter_Impl( USHORT nSID ) const;
	void				SetQuietMode_Impl( BOOL bOn );
	void				SetModalMode_Impl( BOOL bOn );
	BOOL				IsReadOnlyShell_Impl( USHORT nShell ) const;
	void				RemoveShell_Impl( SfxShell& rShell );
	void				InsertShell_Impl( SfxShell& rShell, USHORT nPos );
	void				DoParentActivate_Impl();
	void				DoParentDeactivate_Impl();
	void				DoActivate_Impl( BOOL bMDI );
	void				DoDeactivate_Impl( BOOL bMDI );
	void				InvalidateBindings_Impl(BOOL);
	USHORT				GetNextToolBox_Impl( USHORT nPos, USHORT nType, String *pStr );
#endif
};

inline BOOL SfxDispatcher::IsFlushed() const

{
	 return bFlushed;
}

inline void SfxDispatcher::Flush()

{
	if ( !bFlushed ) FlushImpl();
}

inline void SfxDispatcher::Push( SfxShell& rShell )

{
	Pop( rShell, SFX_SHELL_PUSH );
}

inline BOOL SfxDispatcher::IsActive( const SfxShell& rShell )

{
	return CheckVirtualStack( rShell, TRUE );
}

inline BOOL SfxDispatcher::IsOnTop( const SfxShell& rShell )

{
	return CheckVirtualStack( rShell, FALSE );
}

#if SUPD<606
#define SFX_DISPATCHER() SfxGetpApp()->GetDispatcher()
#endif

#endif

