
#ifndef _SFX_OBJFAC_HXX
#define _SFX_OBJFAC_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _RTTI_HXX 
#include <tools/rtti.hxx>
#endif
#ifndef _FACTORY_HXX 
#include <so3/factory.hxx>
#endif

#ifndef _VOS_MODULE_HXX_
#include <vos/module.hxx>
#endif
#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#include <sfx2/objsh.hxx>
#include <sfx2/sfxdefs.hxx>

class SfxObjectFactoryArr_Impl;
class SvFactory;
class SfxMedium;
class SfxFilter;
class SfxViewFactory;
struct SfxObjectFactory_Impl;
class SfxFilterContainer;
class SfxFactoryFilterContainer;
class SfxMenuBarManager;
class SfxBindings;
class SfxAcceleratorManager;

typedef SfxObjectShell *(*SfxObjectCtor)(SfxObjectCreateMode);

typedef void (*SfxVoidFunc)();

#define SDT_SDT_DOCFACTPRIO     10      
#define SDT_SW_DOCFACTPRIO      20      
#define SDT_SWW_DOCFACTPRIO     21    	
#define SDT_SC_DOCFACTPRIO      30      
#define SDT_SI_DOCFACTPRIO      40      
#define SDT_SD_DOCFACTPRIO      41      
#define SDT_SCH_DOCFACTPRIO     50      
#define SDT_SMA_DOCFACTPRIO     60      
#define SDT_SIM_DOCFACTPRIO     70      
#define SDT_FRM_DOCFACTPRIO    100      
#define SDT_MSG_DOCFACTPRIO    110      
#define SDT_SDB_DOCFACTPRIO    200      

class SfxObjectFactory: public SvFactory
{
private:
	USHORT					nId;			
	SfxObjectCtor			fnCreate;		
	const char* 			pShortName;		
	SfxObjectFactory_Impl*	pImpl;			
	SfxObjectShellFlags     nFlags;

public:
		TYPEINFO();

	SfxObjectFactory( const SvGlobalName &rName, const String &rClassName, CreateInstanceType funcCIT );
	~SfxObjectFactory();

	void			Construct( USHORT nId, SfxObjectCtor fnCreateFnc,
				   SfxObjectShellFlags, const char *pShortName = 0 );
	SfxObjectShell* CreateObject(SfxObjectCreateMode = SFX_CREATE_MODE_STANDARD) const;

	SfxObjectShellFlags GetFlags() { return nFlags; }
	USHORT			GetOrdinal() const { return nId; }
	const char* 	GetShortName() const { return pShortName; }
#ifdef ENABLE_INIMANAGER
	SfxIniManager*	GetIniManager() const;
#endif
	void			RegisterMenuBar( const ResId &rId );
	void			RegisterPluginMenuBar( const ResId &rId );
	void			RegisterAccel( const ResId &rId );
	void			SetDocumentTypeNameResource( const ResId& rId );
	String			GetDocumentTypeName() const;
	const ResId*	GetMenuBarId() const;
	const ResId*	GetPluginMenuBarId() const;
	const ResId*	GetAccelId() const;
	void			RegisterHelpFile( const String& );
	const String&	GetHelpFile() const;
	void			SetExplorerImageId( USHORT nImageId );
	USHORT			GetExplorerImageId() const;

	void			RegisterHelpPIFile( const String& );
	const String&	GetHelpPIFile() const;
	SfxFactoryFilterContainer *GetFilterContainer( BOOL bForceLoad = TRUE) const;

	void			SetCreateNewSlotId( USHORT nId );
	USHORT			GetCreateNewSlotId() const;

	void			RegisterViewFactory(SfxViewFactory &rFactory);
	USHORT			GetViewFactoryCount() const;
	SfxViewFactory& GetViewFactory(USHORT i = 0) const;

	void            RegisterInitFactory( SfxVoidFunc pFunc );
	void     		DoInitFactory();
	void        	RegisterFilter( const String &rName,
						const String &rWildcard,
						SfxFilterFlags nFlags,
						ULONG lFormat,
						const String &rMacType,
						const String &rTypeName,
						USHORT nIconId,
						const String &rMimeType,
						const String &rUserData);
	USHORT      	 GetFilterCount() const;
	const SfxFilter* GetFilter(USHORT i) const;
	const SfxFilter* GetTemplateFilter() const;

	static const SfxObjectFactory*	GetFactory( const String& rFactoryURL );
	const String&				GetStandardTemplate() const;
	static void					SetStandardTemplate( const String& rFactoryURL, const String& rTemplateName );

	void					SetDocumentServiceName( const rtl::OUString& rServiceName );
	const rtl::OUString&	GetDocumentServiceName() const;

	DECL_LINK( InitFactoryHdl, void* );

	SfxModule*                  GetModule() const;
	 static const SfxObjectFactory&     GetDefaultFactory();
	static BOOL                        HasObjectFactories();

#if 0 // _SOLAR__PRIVATE
	static void                        RemoveAll_Impl();
	static SfxObjectFactoryArr_Impl&   GetObjFacArray_Impl();
	static void                        RegisterObjectFactory_Impl(SfxObjectFactory &rObjFac);
	static USHORT                      GetObjectFactoryCount_Impl();
	static const SfxObjectFactory&     GetObjectFactory_Impl(USHORT i);
	SfxMenuBarManager*                 GetMenuBar_Impl();
	SfxAcceleratorManager*             GetAccMgr_Impl();
	void                               SetModule_Impl( SfxModule* );
#endif

private:

	SfxObjectFactory(const SfxObjectFactory&);
	const SfxObjectFactory& operator=(const SfxObjectFactory &);
};

#define SFX_DECL_OBJECTFACTORY(Class)                                       \
private:																	\
	static SfxObjectFactory*	pObjectFactory;								\
protected:																	\
	virtual BOOL                Close();                   					\
public:                                                                     \
								SO2_DECL_BASIC_CLASS(Class)                 \
	virtual BOOL				DoClose();									\
	static SfxObjectShell*		CreateObject(SfxObjectCreateMode eMode = SFX_CREATE_MODE_STANDARD); \
	static void                 InitFactory();                              \
	static SfxObjectFactory&  	Factory() { return *(SfxObjectFactory*)ClassFactory(); } \
	static void                 RegisterFactory( USHORT nPrio = USHRT_MAX );\
	virtual BOOL                DoInitNew( SvStorage * );                   \
	virtual void                ModifyChanged();                            \
	virtual SfxObjectFactory&   GetFactory() const

#define SFX_DECL_OBJECTFACTORY_DLL(Class,FactoryPtr)						\
private:                                                                    \
	static SfxObjectFactory*	pObjectFactory;                             \
protected:																	\
	virtual BOOL                Close();                   					\
public:                                                                     \
					SO2_DECL_BASIC_CLASS_DLL(Class,FactoryPtr)  \
	virtual BOOL  				DoClose();									\
	static SfxObjectShell*		CreateObject(SfxObjectCreateMode eMode = SFX_CREATE_MODE_STANDARD); \
	static void                 InitFactory();                              \
	static SfxObjectFactory&  	Factory() { return *(SfxObjectFactory*)ClassFactory(); } \
	static void                 RegisterFactory( USHORT nPrio = USHRT_MAX );\
	virtual BOOL                DoInitNew( SvStorage * );                   \
	virtual void                ModifyChanged();                            \
	virtual SfxObjectFactory&   GetFactory() const

#define SFX_DECL_SIMPLE_OBJECTFACTORY(Class)                                \
private:																	\
	static SfxObjectFactory*	pObjectFactory;								\
public:                                                                     \
	static SfxObjectShell*      CreateObject(SfxObjectCreateMode eMode = SFX_CREATE_MODE_STANDARD); \
	static void                 InitFactory();                              \
	static SfxObjectFactory&    Factory();                                  \
	static void                 RegisterFactory( USHORT nPrio = USHRT_MAX );\
	virtual SfxObjectFactory&   GetFactory() const

#define SFX_DECL_SIMPLE_OBJECTFACTORY_DLL(Class ) 					\
private:															\
	static SfxObjectFactory*	pObjectFactory;								\
	static SfxObjectFactory **  GetFactoryAdress();					\
public:																\
	static SfxObjectShell*      CreateObject( SfxObjectCreateMode eMode = SFX_CREATE_MODE_STANDARD);  \
	static void                 InitFactory();			 			\
	static SfxObjectFactory&    Factory();                          \
	static void                 RegisterFactory( USHORT nPrio = USHRT_MAX ); \
	virtual SfxObjectFactory&   GetFactory() const

#define SFX_IMPL_OBJECTFACTORY( Class, nFlags, ShortName, aGlobName ) \
																\
		SfxObjectFactory* Class::pObjectFactory = 0;						\
																\
	    SO2_IMPL_BASIC_CLASS1( Class, SfxObjectFactory, SfxInPlaceObject, aGlobName ) \
																\
		SfxObjectShell* __EXPORT Class::CreateObject(SfxObjectCreateMode eMode) \
	    { return new Class(eMode); }                						\
																\
	    void Class::RegisterFactory( USHORT nPrio )                         \
	    {                                                                   \
	        pObjectFactory = &Factory();                                     \
			pObjectFactory->Construct(   									 \
	nPrio,														\
	&Class::CreateObject, \
	nFlags,											 \
	            #ShortName );                                               \
			pObjectFactory->RegisterInitFactory( &InitFactory	);           \
	        pObjectFactory->Register();                  \
	    }                                                                   \
																\
		BOOL __EXPORT Class::DoInitNew( SvStorage *pStor )					\
	    { return SfxObjectShell::DoInitNew(pStor); }                        \
																\
		BOOL __EXPORT Class::DoClose()										\
		{ return SfxInPlaceObject::DoClose(); }								\
																\
		BOOL __EXPORT Class::Close()										\
	    {   SvObjectRef aRef(this);											\
			SfxInPlaceObject::Close();                             			\
			return SfxObjectShell::Close(); }								\
																\
		void __EXPORT Class::ModifyChanged()								\
	    { SfxObjectShell::ModifyChanged(); }                                \
																\
		SfxObjectFactory& __EXPORT Class::GetFactory() const				\
	    {   return *(SfxObjectFactory*)ClassFactory(); }                     \
	                                                                        \
	    void __EXPORT Class::InitFactory()

extern "C"
{
	typedef void  (__LOADONCALLAPI *So2VoidFunc_TYPE) ();
}

#define SFX_IMPL_MODULE_LIB( LibName, LibString )					\
	void * GetFunc##LibName( const char * pFuncName );				\
	static ::vos::OModule* pLibHandle##LibName = NULL; 				\
	BOOL __EXPORT LoadLib##LibName()   								\
	{																\
		if( pLibHandle##LibName == NULL )							\
		{															\
			pLibHandle##LibName = new ::vos::OModule();				\
			So2VoidFunc_TYPE fFunc;									\
			if (!pLibHandle##LibName->load( LibString )) 			\
	return FALSE;										\
			fFunc = (So2VoidFunc_TYPE)GetFunc##LibName( "Init" #LibName "Dll");\
			if( fFunc )												\
	(*fFunc)();											\
		}															\
		return pLibHandle##LibName->isLoaded();						\
	}																\
	void __EXPORT FreeLib##LibName()								\
	{																\
		if( pLibHandle##LibName && pLibHandle##LibName->isLoaded() )\
		{															\
			So2VoidFunc_TYPE fFunc;									\
			fFunc = (So2VoidFunc_TYPE)GetFunc##LibName("DeInit" #LibName "Dll");\
			if( fFunc )												\
	(*fFunc)();											\
		}															\
	}																\
	void* __EXPORT GetFunc##LibName( const char * pFuncName )		\
	{																\
		if( LoadLib##LibName() )									\
	        return pLibHandle##LibName->getSymbol( ::rtl::OUString::createFromAscii(pFuncName) );\
		return NULL;												\
	}

#define SFX_IMPL_MODULE_DLL( LibName )	  									\
																\
		extern "C" void __LOADONCALLAPI Init##LibName##Dll()                \
		{																	\
			LibName##DLL::Init();											\
		}																	\
		extern "C" void __LOADONCALLAPI DeInit##LibName##Dll()              \
		{																	\
			LibName##DLL::Exit();											\
		}

extern "C"
{
	typedef void* (__LOADONCALLAPI *So2CreateExtern_TYPE) ();
	typedef void* (__LOADONCALLAPI *SfxCreateExtern_TYPE) (int);
}

#define SFX_IMPL_OBJECTFACTORY_GEN( Class, LibType )						\
																\
		SfxObjectShell* __EXPORT Class::CreateObject(SfxObjectCreateMode eMode) \
		{																	\
			SfxObjectShell* pDoc = CreateObj##Class##LibType((int)eMode); \
			return pDoc;													\
		}																	\
		SfxObjectFactory& __EXPORT Class::GetFactory() const				\
		{ 																	\
	        return Factory();                                               \
		}

#define SFX_IMPL_OBJECTFACTORY_MOD( ClassName, nFlags, ShortName, aGlobName, LibName, LibString ) \
	    SfxObjectFactory* ClassName::pObjectFactory = 0;                        \
		extern "C" ClassName * __LOADONCALLAPI Create##ClassName##Lib()			  \
		{																		  \
			So2CreateExtern_TYPE fCreate = (So2CreateExtern_TYPE)                 \
	GetFunc##LibName( "Create" #ClassName "Dll");					  \
			if( fCreate )														  \
	return (ClassName *)(*fCreate)();								  \
			return NULL;														  \
		}																		  \
		SotFactory* __EXPORT ClassName::ClassFactory()                                     \
		{                                                                         \
	        if( !pObjectFactory )                                                     \
			{                                                                     \
	            pObjectFactory = new SfxObjectFactory( aGlobName,                \
	                String::CreateFromAscii( RTL_CONSTASCII_STRINGPARAM(#ClassName) ), ClassName::CreateInstance );  \
	            pObjectFactory->PutSuperClass( SfxInPlaceObject::ClassFactory() );  \
			}                                                                     \
	        return pObjectFactory;                                                    \
		}                                                                         \
		void* __EXPORT ClassName::CreateInstance( SotObject ** ppObj )            \
		{                                                                         \
			ClassName * p = Create##ClassName##Lib();                         	  \
			SfxInPlaceObject* pSuper1 = p;                 						  \
			SotObject* pBasicObj = pSuper1;                    					  \
			if( ppObj )                                                           \
	    		*ppObj = pBasicObj;                                               \
			return p;                                                             \
		}																		  \
		void __EXPORT ClassName::RegisterFactory( USHORT nPrio )			\
		{																	\
			Factory().Construct(											\
	nPrio,														\
	&ClassName::CreateObject, nFlags | SFXOBJECTSHELL_HASMENU,  \
	#ShortName );												\
	        Factory().RegisterInitFactory( &InitFactory );                  \
	        Factory().Register();                                           \
		}																	\
		ClassName* __EXPORT CreateObj##ClassName##Lib(int nMode)				\
		{																	\
			SfxCreateExtern_TYPE fCreate = (SfxCreateExtern_TYPE)			\
	GetFunc##LibName( "CreateObj" #ClassName "Dll");            \
			if( fCreate )													\
	return (ClassName*)(*fCreate)(nMode);						\
			return NULL;													\
		}																	\
																\
		SFX_IMPL_OBJECTFACTORY_GEN( ClassName, Lib )						\
																\
		void __EXPORT ClassName::InitFactory()

#define SFX_IMPL_OBJECTFACTORY_LOD( Class, ShortName, aGlobName, LibName ) \
		SO2_IMPL_BASIC_CLASS1_DLL( Class, SfxObjectFactory, SfxInPlaceObject, aGlobName ) \
		extern "C" Class* __LOADONCALLAPI Create##Class##Dll()     			\
		{																	\
			return new Class();												\
		}																	\
		extern "C" Class* __LOADONCALLAPI CreateObj##Class##Dll(int nMode)  \
		{																	\
			SfxObjectCreateMode eMode = (SfxObjectCreateMode) nMode;		\
			return new Class(eMode);										\
		}																	\
																\
		BOOL Class::DoInitNew( SvStorage *pStor )							\
		{ return SfxObjectShell::DoInitNew(pStor); }						\
																\
		BOOL Class::DoClose()												\
		{ return SfxInPlaceObject::DoClose(); }								\
																\
		BOOL Class::Close()													\
	    {   SvObjectRef aRef(this);											\
			SfxInPlaceObject::Close();                             			\
			return SfxObjectShell::Close(); }								\
																\
		void Class::ModifyChanged()											\
		{ SfxObjectShell::ModifyChanged(); }								\
																\
		SFX_IMPL_OBJECTFACTORY_GEN( Class, Dll )

#define SFX_IMPL_OBJECTFACTORY_LIB( Class, nFlags, ShortName, aGlobName, LibName,LibString ) \
		SFX_IMPL_MODULE_LIB( LibName, LibString )																									\
		SFX_IMPL_OBJECTFACTORY_MOD( Class, nFlags, ShortName, aGlobName, LibName,LibString )

#define SFX_IMPL_OBJECTFACTORY_DLL( Class, ShortName, aGlobName, LibName ) \
		SFX_IMPL_MODULE_DLL( LibName )											\
		SFX_IMPL_OBJECTFACTORY_LOD( Class, ShortName, aGlobName, LibName )

#define SFX_IMPL_SIMPLE_OBJECTFACTORY( Class, nFlags, ShortName )		\
																\
		SfxObjectFactory* Class::pObjectFactory = 0;						\
																\
		SfxObjectShell* __EXPORT Class::CreateObject(SfxObjectCreateMode eMode) \
	    { return new Class(eMode); }                						\
																\
	    SfxObjectFactory& __EXPORT Class::Factory()                                     \
		{                                                                         \
	        if( !pObjectFactory )                                                     \
			{                                                                     \
	            pObjectFactory = new SfxObjectFactory( SvGlobalName(),                \
	                                    String(), 0 );  \
			}                                                                     \
	        return *pObjectFactory;                                                    \
		}                                                                         \
	    void Class::RegisterFactory( USHORT nPrio )                         \
	    {                                                                   \
	        Factory().Construct(                                      \
	nPrio,														\
	&Class::CreateObject, nFlags,								\
	            #ShortName );                                               \
	        Factory().RegisterInitFactory( &InitFactory   );          \
	    }                                                                   \
																\
		SfxObjectFactory& __EXPORT Class::GetFactory() const				\
	    {   return Factory(); }                                             \
																\
		void __EXPORT Class::InitFactory()

#define SFX_IMPL_SIMPLE_OBJECTFACTORY_LOD( Class, LibName, FactoryPtr )		\
	SfxObjectFactory** Class::GetFactoryAdress()                            \
	{ return &FactoryPtr->p##Class##Factory; }                              \
	extern "C" Class* __LOADONCALLAPI CreateObj##Class##Dll(int nMode)      \
	{																		\
		SfxObjectCreateMode eMode = (SfxObjectCreateMode) nMode;			\
		return new Class(eMode);											\
	}																		\
	SfxObjectShell*	 Class::CreateObject(SfxObjectCreateMode eMode)			\
	{																		\
		return new Class( eMode );											\
	}																		\
	SfxObjectFactory&  Class::Factory()                                     \
	    { return **GetFactoryAdress(); }                                     \
	SfxObjectFactory&  Class::GetFactory() const							\
	    { return Factory(); }

#define SFX_IMPL_SIMPLE_OBJECTFACTORY_LIB( Class, nFlags, ShortName, LibName, FactoryPtr ) \
	SfxObjectFactory* Class::pObjectFactory = 0;                        \
	SfxObjectFactory& __EXPORT Class::Factory()                                     \
	{                                                                         \
	    if( !pObjectFactory )                                                     \
	    {                                                                     \
	        pObjectFactory = new SfxObjectFactory( SvGlobalName(),                \
	                                String(), 0 );  \
	    }                                                                     \
	    return *pObjectFactory;                                                    \
	}                                                                         \
	void __EXPORT Class::RegisterFactory( USHORT nPrio )					  \
	{																		  \
	        Factory().Construct(                                        \
	nPrio,														  \
	&Class::CreateObject, nFlags,								  \
	#ShortName );												  \
	        Factory().RegisterInitFactory( &InitFactory    );                    \
	}																		  \
																  \
	SfxObjectShell*	__EXPORT Class::CreateObject(SfxObjectCreateMode eMode)	  \
	{																		  \
		SfxCreateExtern_TYPE fCreate = (SfxCreateExtern_TYPE)			\
			GetFunc##LibName( "CreateObj" #Class "Dll");            \
		if( fCreate )														  \
			return (Class*)(*fCreate)(eMode);								  \
		return 0;															  \
	}																		  \
																  \
	SfxObjectFactory& __EXPORT Class::GetFactory() const 					  \
	    { return Factory(); }                                                \
																  \
	void __EXPORT Class::InitFactory()

#define SFX_FILTER_REGISTRATION(aName, aWild, ePurpose, lClipboardFormat, \
	pMacType, pOS2Type, nOS2DocIcon, pUserData) \
	((SfxObjectFactory&)Factory()).RegisterFilter( \
		aName, aWild, ePurpose, lClipboardFormat, pMacType, pOS2Type, nOS2DocIcon, String(), pUserData )

#define SFX_OWN_FILTER_REGISTRATION(									\
	pDetectFilter, aName, aWild,										\
	ePurpose, lClipboardFormat,											\
	pMacType, pOS2Type, nOS2DocIcon, aMimeType, pUserData)				\
	{																	\
		SfxObjectFactory& rFact = (SfxObjectFactory&)Factory();			\
	    rFact.RegisterFilter(											\
	            aName, aWild, ePurpose, lClipboardFormat, pMacType,		\
	      pOS2Type, nOS2DocIcon, aMimeType, pUserData );			\
		rFact.GetFilterContainer()->SetDetectFilter( pDetectFilter );	\
 	}

#define SFX_SIMPLE_FILTER_REGISTRATION(								\
	aName, aWild,													\
	ePurpose, lClipboardFormat,										\
	pMacType, pOS2Type, nOS2DocIcon, aMimeType, pUserData)			\
	{																\
		SfxObjectFactory& rFact = (SfxObjectFactory&)Factory();		\
	     rFact.RegisterFilter(										\
	            aName, aWild, ePurpose, lClipboardFormat, pMacType,	\
	      pOS2Type, nOS2DocIcon, aMimeType, pUserData );		\
	}

#define SFX_APP_FILTER_REGISTRATION( aName )								\
	{																		\
		if( !SFX_APP()->GetFilterMatcher().GetContainer( aName ) )			\
		{																	\
			SfxFilterContainer* pCont = new SfxFilterContainer( aName );	\
			SFX_APP()->GetFilterMatcher().AddContainer(	pCont );			\
			pCont->LoadFilters(												\
	aName, 	TRUE, 												\
	            SFX_FILTER_MAPTOAPPPLUG|SFX_FILTER_PLUGIN  );				\
		}															   		\
	}

#ifdef _SFX_HXX
#define SFX_DECL_DOCUMENTFACTORY(Class) \
		SFX_DECL_OBJECTFACTORY(Class)

#define SFX_DECL_DOCUMENTFACTORY_DLL(Class,ClassFactory) \
		SFX_DECL_OBJECTFACTORY_DLL(Class,ClassFactory)

#define SFX_DECL_SIMPLE_DOCUMENTFACTORY_DLL(Class) \
		SFX_DECL_SIMPLE_OBJECTFACTORY_DLL(Class)
#endif

#endif 

