
#ifndef _SFXOBJFACE_HXX
#define _SFXOBJFACE_HXX

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _RESID_HXX 
#include <vcl/resid.hxx>
#endif
#include <sfx2/msg.hxx>

struct SfxFormalArgument;
struct SfxTypeLibImpl;
struct SfxInterface_Impl;
class  SfxConfigManager;
class  SfxConfigItem;
class  SfxObjectUIArr_Impl ;
class  SfxModule;
class  SvStream;

class SfxInterface
{
friend class SfxIFConfig_Impl;
friend class SfxSlotPool;

	const char* 			pName;			
	const SfxInterface*     pGenoType;
	SfxSlot*                pSlots;
	USHORT                  nCount;
	const SfxTypeLibImpl*	pTypeLibInfo;
	SfxIFConfig_Impl*		pConfig;		
	USHORT					nClassId;		
	ResId					aNameResId; 	
	SfxInterface_Impl*		pImpData;

	SfxSlot*                operator[]( USHORT nPos ) const;

#ifdef _PROXYSHELL_VERWENDEN
protected:
#if 0 // _SOLAR__PRIVATE
	void					SetCount_Impl( USHORT n )
				{ nCount = n; }
#endif
#endif

public:
				SfxInterface( SfxModule *pMod,
							  const char *pClass,
							  const ResId& rResId,
							  USHORT nClassId,
							  const SfxInterface* pGeno,
							  const SfxTypeLibImpl * pLibInfo,
							  SfxSlot &rMessages, USHORT nMsgCount );
				SfxInterface( SfxModule *pMod,
							  const char *pClass,
							  const ResId& rResId,
							  USHORT nClassId,
							  const SfxInterface* pGeno,
							  const SfxTypeLibImpl * pLibInfo);
				~SfxInterface();

	void                    SetSlotMap( SfxSlot& rMessages, USHORT nMsgCount );
	void                    Init( );
	inline USHORT           Count() const;

	const SfxSlot*          GetRealSlot( const SfxSlot * ) const;
	const SfxSlot*          GetRealSlot( USHORT nSlotId ) const;
	virtual const SfxSlot*	GetSlot( USHORT nSlotId ) const;
	const SfxSlot*			GetSlot( const String& rCommand ) const;

	const char*             GetClassName() const { return pName; }
	int 					HasName() const { return 0 != aNameResId.GetId(); }
	String                  GetName() const
				{ return String(aNameResId); }
	ResMgr* 				GetResMgr() const
				{ return aNameResId.GetResMgr(); }

	const SfxTypeLibImpl*	GetTypeLibInfo() const
				{ return pTypeLibInfo; }
	const SfxInterface* 	GetGenoType() const { return pGenoType; }

	const SfxInterface*		GetRealInterfaceForSlot( const SfxSlot* ) const;

	void					RegisterObjectBar( USHORT, const ResId&, const String* pST=0 );
	void					RegisterObjectBar( USHORT, const ResId&, ULONG nFeature, const String* pST=0 );
	void					TransferObjectBar( USHORT, USHORT, SfxInterface*, const String* pST=0 );
	void					RegisterChildWindow( USHORT, BOOL bContext, const String* pST=0 );
	void					RegisterChildWindow( USHORT, BOOL bContext, ULONG nFeature, const String* pST=0 );
	void                    ReleaseChildWindow( USHORT nId);
	void					RegisterStatusBar( const ResId& );
	void                    ReleaseObjectBar( USHORT nId);
	void                    SetObjectBarName (const String& rName, USHORT nId);
	const ResId&			GetObjectBarResId( USHORT nNo ) const;
	USHORT                  GetObjectBarPos( USHORT nNo ) const;
	ULONG                   GetObjectBarFeature( USHORT nNo ) const;
	USHORT					GetObjectBarCount() const;
	void                    SetObjectBarPos( USHORT nPos, USHORT nId );
	const String*           GetObjectBarName( USHORT nNo ) const;
	BOOL					IsObjectBarVisible( USHORT nNo) const;
	void					SetObjectBarVisible( BOOL, USHORT);
	BOOL					StoreObjectBar (USHORT, SvStream&);
	int 					LoadObjectBar (USHORT, SvStream&);
	static USHORT			RegisterUserDefToolBox(USHORT, const String*,
							SfxConfigManager *pCfgMgr);
	static void				ReleaseUserDefToolBox(USHORT nId, SfxConfigManager*);
	BOOL					HasObjectBar(USHORT nId) const;

	ULONG					GetChildWindowFeature( USHORT nNo ) const;
	ULONG					GetChildWindowId( USHORT nNo ) const;
	USHORT					GetChildWindowCount() const;

	void					RegisterPopupMenu( const ResId& );
	const ResId&			GetPopupMenuResId() const;
	const ResId&			GetStatusBarResId() const;

	void					RegisterObjectMenu( USHORT nPos, const ResId& );
	const ResId&			GetObjectMenuResId( USHORT nNo ) const;
	USHORT					GetObjectMenuPos( USHORT nNo ) const;
	USHORT					GetObjectMenuCount() const;
	void                    ClearObjectMenus();
	void                    RemoveObjectMenu( USHORT nPos );

	BOOL                    IsDefault();
	void                    UseDefault();
	void                    LoadConfig();
	USHORT					GetConfigId() const;
	void					ReInitialize(SfxConfigManager*);
	void					SaveConfig();
	SfxModule*				GetModule() const;

	USHORT					GetInterfaceId() const
				{ return nClassId; }

#if 0 // _SOLAR__PRIVATE
	int                     ContainsSlot_Impl( const SfxSlot *pSlot ) const
				{ return pSlot >= pSlots && pSlot < pSlots + Count(); }
	ResMgr* 				GetResManager_Impl() const
				{ return aNameResId.GetResMgr(); }
	SfxObjectUIArr_Impl*	GetObjectBarArr_Impl() const;
	SfxConfigItem*       	GetConfig_Impl()
				{ return (SfxConfigItem*) pConfig; }
	USHORT					GetClassId_Impl() const
				{ return nClassId; }
#endif
};

#ifdef _PROXYSHELL_VERWENDEN
#if 0 // _SOLAR__PRIVATE
class SfxBindingsProxyInterface_Impl : public SfxInterface
{
	SfxBindings*			pBindings;
public:
				SfxBindingsProxyInterface_Impl( SfxBindings* pBind );

	const SfxSlot*          GetRealSlot( const SfxSlot * ) const;
	const SfxSlot*          GetRealSlot( USHORT nSlotId ) const;
	const SfxSlot*          GetSlot( USHORT nSlotId ) const;
	void					SwitchBindings( SfxBindings* pNew );
};
#endif

#endif

class SfxComponentViewShell;
class SfxComponentProxyInterface_Impl : public SfxInterface
{
	SfxComponentViewShell*	pView;
public:
		SfxComponentProxyInterface_Impl( SfxComponentViewShell* );
#if SUPD>381
	virtual
#endif
	const SfxSlot*	GetSlot( USHORT nSlotId ) const;
};

inline USHORT SfxInterface::Count() const
{
	return nCount;
}

inline SfxSlot* SfxInterface::operator[]( USHORT nPos ) const
{
	return nPos < nCount? pSlots+nPos: 0;
}
#endif
