
#ifndef _SFX_PRINTER_HXX
#define _SFX_PRINTER_HXX

#ifndef _PRINT_HXX 
#include <vcl/print.hxx>
#endif

class SfxFont;
class SfxTabPage;
class SfxItemSet;

struct SfxPrinter_Impl;

#define SFX_RANGE_NOTSET	((USHORT)0xFFFF)

class SfxFontSizeInfo
{
private:
	static USHORT			pStaticSizes[];
	Size*					pSizes;
	USHORT					nSizes;
	BOOL					bScalable;

public:
	SfxFontSizeInfo( const SfxFont& rFont, const OutputDevice& rDevice );
	~SfxFontSizeInfo();

	BOOL					HasSize(const Size &rSize) const;
	BOOL					IsScalable() const { return bScalable; }

	USHORT					SizeCount() const { return nSizes; }
	const Size& 			GetSize( USHORT nNo ) const
				{ return pSizes[nNo]; }
};

class SfxFont
{
private:
	String					aName;
	FontFamily				eFamily;
	FontPitch				ePitch;
	CharSet 				eCharSet;

	SfxFont&				operator=(const SfxFont& rFont); 

public:
	SfxFont( const FontFamily eFam,
	const String& aName,
	const FontPitch eFontPitch = PITCH_DONTKNOW,
	const CharSet eFontCharSet = RTL_TEXTENCODING_DONTKNOW );

	inline const String&	GetName() const { return aName; }
	inline FontFamily		GetFamily() const { return eFamily; }
	inline FontPitch		GetPitch() const { return ePitch; }
	inline CharSet			GetCharSet() const { return eCharSet; }
};

class SfxPrinter : public Printer
{
private:
	JobSetup				aOrigJobSetup;
	SfxItemSet*				pOptions;
	SfxPrinter_Impl*		pImpl;
	BOOL					bKnown;

	void                    UpdateFonts_Impl();

public:
				SfxPrinter( SfxItemSet *pTheOptions );
				SfxPrinter( SfxItemSet *pTheOptions,
							const String &rPrinterName );
				SfxPrinter( SfxItemSet *pTheOptions,
							const JobSetup &rTheOrigJobSetup );
				SfxPrinter( SfxItemSet *pTheOptions,
							const String &rPrinterName,
							const JobSetup &rTheOrigJobSetup );
				SfxPrinter( const SfxPrinter &rPrinter );
				~SfxPrinter();

	SfxPrinter*				Clone() const;

	static SfxPrinter*		Create( SvStream &rStream, SfxItemSet *pOptions );
	SvStream&				Store( SvStream &rStream ) const;

	const JobSetup& 		GetOrigJobSetup() const	{ return aOrigJobSetup; }
	void					SetOrigJobSetup( const JobSetup &rNewJobSetup );

	const SfxItemSet&		GetOptions() const { return *pOptions; }
	void					SetOptions( const SfxItemSet &rNewOptions );

	void					EnableRange( USHORT nRange );
	void					DisableRange( USHORT nRange );
	BOOL					IsRangeEnabled( USHORT nRange ) const;

	BOOL					IsKnown() const { return bKnown; }
	BOOL					IsOriginal() const { return bKnown; }

	USHORT					GetFontCount();
	const SfxFont*			GetFont( USHORT nNo ) const;
	const SfxFont*          GetFontByName( const String &rFontName );
	 BOOL                    InitJob( Window* pUIParent, BOOL bDocumentContainsTransparentObjects );
};

#endif
