
#ifndef _SFXDIR_HXX
#define _SFXDIR_HXX

#include <tools/string.hxx>
#include <svtools/smplhint.hxx>

DECL_OBJHINT(SfxDirEntryHint, String);

#if 0

#ifndef _DEBUG_HXX 
#include <tools/debug.hxx>
#endif
#ifndef _TIMER_HXX 
#include <vcl/timer.hxx>
#endif
#ifndef _SFXBRDCST_HXX 
#include <svtools/brdcst.hxx>
#endif
#ifndef _REF_HXX 
#include <tools/ref.hxx>
#endif
#ifndef _SFXSMPLHINT_HXX 
#include <svtools/smplhint.hxx>
#endif
#ifndef _DATETIME_HXX 
#include <tools/datetime.hxx>
#endif
#ifndef _SFXLSTNER_HXX 
#include <svtools/lstner.hxx>
#endif

class SvStream;
class SvStringsDtor;
class SfxExtDir;
class SfxExtDirEntry;
class SfxExtDirEntryArr_Impl;
class SfxExtDirDummyArr_Impl;
struct SfxExtDirEntry_Impl;

#ifndef FSYS_ACCESS_FLOPPY

#if SUPD<365
#define FSysAccess  BOOL
#else
typedef int FSysAccess;
#endif
#define FSYS_ACCESS_FLOPPY          1
#define FSYS_ACCESS_CACHED          2
#endif

DECL_OBJHINT(SfxDirEntryHint, DirEntry);
DECL_PTRHINT(SfxExtDirEntryHint, SfxExtDirEntry);

class SfxExtDirSink_Impl: public SfxListener

{
	SfxExtDir*                  _pDir;

protected:
	void                        Notify( SfxBroadcaster& rBC, const SfxHint& rHint );
public:
					SfxExtDirSink_Impl(): _pDir(0) {}

	void						SetDir( SfxExtDir *pDir ) { _pDir = pDir; }
};

enum SfxExtDirEntryState
{
	SFX_EXTDIR_UNKNOWN,
	SFX_EXTDIR_UNCHANGED,	
	SFX_EXTDIR_UPDATED,     
	SFX_EXTDIR_INSERTED, 	
	SFX_EXTDIR_MODIFIED		
};

class SfxExtDirEntry: public SvRefBase, public DirEntry

{
friend class SfxExtDir;

	SfxExtDir*					_pDir;      
	ULONG						_nPos;		
	ULONG						_nId;		
	String						_aTitle;	
	ULONG						_nSize; 	
	DateTime					_aCreated;	
	DateTime					_aModified; 
	DateTime					_aAccessed; 
	SvStringsDtor*				_pProps;	
	DirEntryKind				_eKind; 	
	USHORT						_nMaxSize;	
	SfxExtDirEntryState 		_eState;	
#if SUPD>358
	void*						_pUser;		
#endif

#if 0 // _SOLAR__PRIVATE
private:
	void						SetId_Impl( ULONG nId )
					{
						DBG_ASSERT( !_nId, "SfxExtDir: already has id" );
						_nId = nId;
					}
	void						SetState_Impl( SfxExtDirEntryState eState )
					{ _eState = eState; }
	void						SetDir_Impl( SfxExtDir *pDir );
	BOOL                        IsKnown_Impl() const;
	void                        DetectTitle_Impl();
	ULONG						GetPos_Impl() const { return _nPos; }
	BOOL						Update_Impl( const FileStat &rStat );
#endif

protected:
	virtual void     			Renamed( const DirEntry& rDestDir );

public:
					SfxExtDirEntry();
					SfxExtDirEntry( SfxExtDir &rParent, const String &rTitle,
									const String &rExt, DirEntryKind eKind,
									USHORT nPos = USHRT_MAX, BOOL bUseTilde = FALSE );
					SfxExtDirEntry( const DirEntry &rPath );
					SfxExtDirEntry( const DirEntry &rPath, const FileStat &rStat );
					SfxExtDirEntry( SfxExtDir &rParent, ULONG nId,
									SvStream &rStream, ULONG nSize, DirEntryKind eKind );
	virtual 					~SfxExtDirEntry();

#if SUPD>358
	void						SetUserData( void* pUser ) { _pUser = pUser; }
	void*						GetUserData() const { return _pUser; }
#endif

	SfxExtDir*					GetDir() const { return _pDir; }

	inline SfxExtDirEntryState 	GetState() const;
	BOOL						Store( SvStream &rStream, USHORT bMaxSize = 0 );
	ULONG						GetId() const { return _nId; }

	const String&				GetTitle() const { return _aTitle; }
	ErrCode 					SetTitle( const String &rTitle );

	ULONG						GetSize() const { return _nSize; }
	DirEntryKind				GetKind() const;

	const DateTime& 			GetCreated() const { return _aCreated; }
	const DateTime& 			GetModified() const { return _aModified; }
	const DateTime& 			GetAccessed() const { return _aAccessed; }

	const String*				GetProperty( USHORT nPos ) const;
	void						SetProperty( const String& rProp, USHORT nPos );

	ErrCode						Kill() const;

	BOOL						UpdateStat();	
	BOOL						Update();		

};

SV_DECL_IMPL_REF(SfxExtDirEntry)

inline SfxExtDirEntryState SfxExtDirEntry::GetState() const

{
	return _eState;
}

class SfxExtDirFilter

{
public:
	virtual BOOL	IsVisible( const DirEntry &rEntry ) const = 0;

	SfxExtDirFilter()
		{
		}

	virtual			~SfxExtDirFilter();
};

class SfxExtDir: public SfxExtDirEntry, public SfxBroadcaster

{
	void*					_pUserData; 		
	DirEntry				_aIndex;			
	ULONG					_nEndOfRecords; 	
	ULONG					_nLastKnownRevision;
	ULONG					_nLastUsedId;		
	Dir*					_pUpdater;			
	SvStream*				_pStream;			
	AutoTimer*				_pTimer;			
	SfxExtDirFilter*		_pFilter;			
	SfxExtDirEntryArr_Impl* _pEntries;			
	SfxExtDirDummyArr_Impl* _pDummies;			
	SfxExtDirSink_Impl		_aSink; 			
	USHORT					_nRecordsInFile;	
	USHORT					_nFreeRecords;		
	USHORT					_nLocks;			
	BOOL					_bDirty;			

public:
#if SUPD<364
				SfxExtDir( const String &rPath );
#endif
				SfxExtDir( const DirEntry &rPath );
				SfxExtDir( const DirEntry &rPath, const FileStat &rStat );
				SfxExtDir( SfxExtDir &rParent, ULONG nId,
						   SvStream &rStream, ULONG nOffset, DirEntryKind eKind );
				SfxExtDir( SfxExtDir &rParent, const String &rTitle,
						   const String &rExt, DirEntryKind eKind,
						   USHORT nPos = USHRT_MAX );
				~SfxExtDir();

	void					SetAutoUpdateTimeout( ULONG nMilliSecs );
	ErrCode 				Commit( BOOL bAll = FALSE );
	ErrCode 				Lock( BOOL bReScan = FALSE, ULONG nTimeoutMilliSecs = 30*1000L );
	ErrCode 				Lock( BOOL bReScan, ULONG nTimeoutMilliSecs, BOOL bAll );
	ErrCode 				Unlock( BOOL bCommit = FALSE );
	ErrCode 				Update( BOOL bReScan = FALSE );
	ErrCode 				Update( BOOL bReScan, BOOL bAll );
	inline BOOL             UpdateActive() const;

	USHORT					Count() const;
	SfxExtDirEntry* 		GetObject( USHORT n ) const;

	SfxExtDirEntry* 		FindById( ULONG nId, USHORT *pPos = 0 ) const;
	SfxExtDirEntry* 		FindByName( const String &rName, USHORT *pPos = 0 ) const;

	ErrCode 				Insert( SfxExtDirEntry *pEntry, USHORT nPos = USHRT_MAX );
	ErrCode 				Remove( USHORT nPos );
	void    				RemoveOnly( USHORT nPos );

	void					SetFilter( SfxExtDirFilter *pFilter );

	static BOOL				IsIndexed( const DirEntry &rPath );
	BOOL             		IsIndexed() const;
	ErrCode 				RemoveIndex();

	ErrCode 				CreateIndex( BOOL bLock = FALSE );

	void*					GetUserData() { return _pUserData; }
	const void* 			GetUserData() const { return _pUserData; }
	void					SetUserData( void* pUserData ) { _pUserData = pUserData; }

	void                    EnableAppUpdate( BOOL bEnable );

#if 0 // _SOLAR__PRIVATE
	DECL_LINK				(AutoUpdate_Impl, void*);
	ErrCode 				Update_Impl( BOOL bAll = FALSE );
	void 					Clear_Impl();
	static void             Test_Impl();
#endif
};

inline BOOL SfxExtDir::UpdateActive() const

{
	return _pUpdater != NULL;
}

SV_DECL_IMPL_REF(SfxExtDir)

#endif 

#endif 

