
#ifndef _SFXTABDLG_HXX
#define _SFXTABDLG_HXX

#ifndef _SV_TABDLG_HXX 
#include <vcl/tabdlg.hxx>
#endif
#ifndef _SV_BUTTON_HXX 
#include <vcl/button.hxx>
#endif
#ifndef _SV_TABCTRL_HXX 
#include <vcl/tabctrl.hxx>
#endif
#ifndef _SV_TABPAGE_HXX 
#include <vcl/tabpage.hxx>
#endif
#ifndef _SFXITEMPOOL_HXX 
#include <svtools/itempool.hxx>
#endif
#ifndef _SFXITEMSET_HXX 
#include <svtools/itemset.hxx>
#endif

class SfxPoolItem;
class SfxTabDialog;
class SfxViewFrame;
class SfxTabPage;
class SfxBindings;

typedef SfxTabPage* (*CreateTabPage)(Window *pParent, const SfxItemSet &rAttrSet);
typedef USHORT*		(*GetTabPageRanges)(); 

struct TabDlg_Impl;
struct TabPageImpl;
class SfxUs_Impl;

#define ID_TABCONTROL	1
#define RET_USER 		100
#define RET_USER_CANCEL	101

class SfxTabDialogItem: public SfxSetItem
{
public:
	TYPEINFO();
	                        SfxTabDialogItem( USHORT nId, const SfxItemSet& rItemSet );
	                        SfxTabDialogItem(const SfxTabDialogItem& rAttr, SfxItemPool* pItemPool=NULL);
	virtual SfxPoolItem*    Clone(SfxItemPool* pToPool) const;
	virtual SfxPoolItem*    Create(SvStream& rStream, USHORT nVersion) const;
};

class SfxTabDialog : public TabDialog
{
private:
friend class SfxTabPage;
friend class SfxTabDialogController;

	SfxViewFrame*	pFrame;

	TabControl		aTabCtrl;
	OKButton		aOKBtn;
	PushButton*		pUserBtn;
	CancelButton	aCancelBtn;
	HelpButton		aHelpBtn;
	PushButton		aResetBtn;
	PushButton		aBaseFmtBtn;

	const SfxItemSet*	pSet;
	SfxItemSet*			pOutSet;
	TabDlg_Impl*		pImpl;
	USHORT*				pRanges;
	USHORT				nResId;
	USHORT				nAppPageId;
	BOOL				bItemsReset;
	BOOL				bFmt;

#if 0 // _SOLAR__PRIVATE
	DECL_LINK( ActivatePageHdl, TabControl * );
	DECL_LINK( DeactivatePageHdl, TabControl * );
	DECL_LINK( OkHdl, Button * );
	DECL_LINK( ResetHdl, Button * );
	DECL_LINK( BaseFmtHdl, Button * );
	DECL_LINK( UserHdl, Button * );
	DECL_LINK( CancelHdl, Button * );
	void			Init_Impl(BOOL, const String *);
#endif

protected:
	virtual short				Ok();

	virtual SfxItemSet*			CreateInputItemSet( USHORT nId );

	virtual const SfxItemSet*	GetRefreshedSet();
	virtual void				PageCreated( USHORT nId, SfxTabPage &rPage );
#if SUPD>=636
	virtual long                Notify( NotifyEvent& rNEvt );
#endif

	SfxItemSet*		pExampleSet;
	SfxItemSet*		GetInputSetImpl();
	SfxTabPage*		GetTabPage( USHORT nPageId ) const;

	BOOL			IsInOK() const;

	BOOL			PrepareLeaveCurrentPage();

public:
	SfxTabDialog( Window* pParent, const ResId &rResId, USHORT nSetId, SfxBindings& rBindings,
	  BOOL bEditFmt = FALSE, const String *pUserButtonText = 0 );
	SfxTabDialog( Window* pParent, const ResId &rResId, const SfxItemSet * = 0,
	  BOOL bEditFmt = FALSE, const String *pUserButtonText = 0 );
	SfxTabDialog( SfxViewFrame *pViewFrame, Window* pParent, const ResId &rResId,
	  const SfxItemSet * = 0, BOOL bEditFmt = FALSE,
	  const String *pUserButtonText = 0 );
	~SfxTabDialog();

	void				AddTabPage( USHORT nId,
						CreateTabPage pCreateFunc,		
						GetTabPageRanges pRangesFunc,	
						BOOL bItemsOnDemand = FALSE);
	void				AddTabPage( USHORT nId,
						const String &rRiderText,
						CreateTabPage pCreateFunc,		
						GetTabPageRanges pRangesFunc,	
						BOOL bItemsOnDemand = FALSE,
						USHORT nPos = TAB_APPEND);
	void				AddTabPage( USHORT nId,
						const Bitmap &rRiderBitmap,
						CreateTabPage pCreateFunc,		
						GetTabPageRanges pRangesFunc,	
						BOOL bItemsOnDemand = FALSE,
						USHORT nPos = TAB_APPEND);
	void				RemoveTabPage( USHORT nId );

	void				SetCurPageId( USHORT nId ) { nAppPageId = nId; }
	USHORT				GetCurPageId() const
				{ return aTabCtrl.GetCurPageId(); }
	void				ShowPage( USHORT nId );

	const USHORT*		GetInputRanges( const SfxItemPool& );
	void				SetInputSet( const SfxItemSet* pInSet );
	const SfxItemSet*	GetOutputItemSet() const { return pOutSet; }
	const SfxItemSet*	GetOutputItemSet( USHORT nId ) const;
	int					FillOutputItemSet();
	const BOOL			IsFormat() const { return bFmt; }

	const OKButton& 	GetOKButton() const { return aOKBtn; }
	OKButton&			GetOKButton() { return aOKBtn; }
	const CancelButton& GetCancelButton() const { return aCancelBtn; }
	CancelButton&		GetCancelButton() { return aCancelBtn; }
	const HelpButton&	GetHelpButton() const { return aHelpBtn; }
	HelpButton&			GetHelpButton() { return aHelpBtn; }
	const PushButton& 	GetResetButton() const { return aResetBtn; }
	PushButton&			GetResetButton() { return aResetBtn; }

	const PushButton*	GetApplyButton() const;
	PushButton*			GetApplyButton();

	const PushButton*	GetUserButton() const { return pUserBtn; }
	PushButton*			GetUserButton() { return pUserBtn; }
	void				RemoveResetButton();

	short				Execute();
	void				Start( BOOL bShow = TRUE );

	const SfxItemSet*	GetExampleSet() const { return pExampleSet; }
	SfxViewFrame*		GetViewFrame() const { return pFrame; }

	void				EnableApplyButton(BOOL bEnable = TRUE);
	BOOL				IsApplyButtonEnabled() const;
	void				SetApplyHandler(const Link& _rHdl);
	Link				GetApplyHandler() const;

#if 0 // _SOLAR__PRIVATE
	void				Start_Impl();
	BOOL				OK_Impl() { return PrepareLeaveCurrentPage(); }
#endif
};

class SfxTabPage: public TabPage
{
friend class SfxTabDialog;

private:
	const SfxItemSet*	pSet;
	String				aUserString;
	BOOL				bHasExchangeSupport;
	SfxTabDialog*		pTabDlg;
	TabPageImpl*		pImpl;

	void				SetTabDialog( SfxTabDialog* pNew ) { pTabDlg = pNew; }
	void				SetInputSet( const SfxItemSet* pNew ) { pSet = pNew; }

protected:
	SfxTabPage( Window *pParent, const ResId &, const SfxItemSet &rAttrSet );
	SfxTabPage( Window *pParent, WinBits nStyle, const SfxItemSet &rAttrSet );

	USHORT				GetSlot( USHORT nWhich ) const
				{ return pSet->GetPool()->GetSlotId( nWhich ); }
	USHORT				GetWhich( USHORT nSlot ) const
				{ return pSet->GetPool()->GetWhich( nSlot ); }
	const SfxPoolItem* 	GetOldItem( const SfxItemSet& rSet, USHORT nSlot );
	const SfxPoolItem* 	GetExchangeItem( const SfxItemSet& rSet, USHORT nSlot );
	SfxTabDialog*		GetTabDialog() const { return pTabDlg; }

public:
	virtual             ~SfxTabPage();

	const SfxItemSet&	GetItemSet() const { return *pSet; }

	virtual BOOL		FillItemSet( SfxItemSet& ) = 0;
	virtual void		Reset( const SfxItemSet& ) = 0;

	BOOL				HasExchangeSupport() const
				{ return bHasExchangeSupport; }
	void				SetExchangeSupport( BOOL bNew = TRUE )
				{ bHasExchangeSupport = bNew; }

	enum {
		KEEP_PAGE = 0x0000,		

		LEAVE_PAGE = 0x0001,

		REFRESH_SET = 0x0002
	};

	virtual void		ActivatePage( const SfxItemSet& );
	virtual int			DeactivatePage( SfxItemSet* pSet = 0 );
	void				SetUserData(const String& rString)
				{ aUserString = rString; }
	String				GetUserData() { return aUserString; }
	virtual void		FillUserData();
	virtual BOOL		IsReadOnly() const;

	static const SfxPoolItem* GetItem( const SfxItemSet& rSet, USHORT nSlot );
};

#endif

