
#ifndef _SFXVIEWSH_HXX
#define _SFXVIEWSH_HXX

#ifndef _COM_SUN_STAR_FRAME_XCONTROLLER_HPP_
#include <com/sun/star/frame/XController.hpp>
#endif
#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif
#ifndef _SFXLSTNER_HXX 
#include <svtools/lstner.hxx>
#endif

#include <sfx2/shell.hxx>

#define SFX_VIEWSH_INCLUDES_CLIENTSH_HXX	1
#include <sfx2/clientsh.hxx>

class SfxBaseController;
class Size;
class Fraction;
class Window;
class KeyEvent;
class WorkWindow;
class SvBorder;
class SdrView;
class SfxFrame;
class SfxMedium;
class SfxModule;
class SfxViewFrame;
class SfxItemPool;
class SfxTabPage;
class SfxPrintMonitor;
class SfxFrameSetDescriptor;
class PrintDialog;
class SfxPrinter;
class SfxProgress;
class SfxInPlaceClient;
class SfxMenuBarManager;
class SvEmbeddedObject;
class SvInPlaceObject;
class SvInPlaceClient;
class SvInPlaceClientMemberList;
class SfxInPlaceClient;
class SfxAcceleratorManager;
class SvStringsDtor;
class SfxFrameItem;
class Dialog;

#define SFX_PRINTER_PRINTER			 1 	
#define SFX_PRINTER_JOBSETUP    	 2
#define SFX_PRINTER_OPTIONS      	 4
#define SFX_PRINTER_CHG_ORIENTATION  8
#define SFX_PRINTER_CHG_SIZE 		16
#define SFX_PRINTER_ALL         	31

#define SFX_PRINTER_CHG_ORIENTATION_FLAG  3
#define SFX_PRINTER_CHG_SIZE_FLAG 		  4

enum SfxScrollingMode
{
	SCROLLING_NO,
	SCROLLING_YES,
	SCROLLING_AUTO,
	SCROLLING_DEFAULT
};

#define DEFAULT_MARGIN_WIDTH 8
#define DEFAULT_MARGIN_HEIGHT 12

#define	SFX_VIEW_MAXIMIZE_FIRST		 0x0001 
#define	SFX_VIEW_OPTIMIZE_EACH	     0x0002	
#define	SFX_VIEW_DISABLE_ACCELS		 0x0004	
#define	SFX_VIEW_OBJECTSIZE_EMBEDDED 0x0008	
#define SFX_VIEW_HAS_PRINTOPTIONS    0x0010 
#define SFX_VIEW_CAN_PRINT			 0x0020 
#define SFX_VIEW_NO_SHOW			 0x0040 
#define SFX_VIEW_IMPLEMENTED_AS_FRAMESET 0x0080 
#define SFX_VIEW_NO_NEWWINDOW		0x0100		

#define SFX_DECL_VIEWFACTORY(Class) \
private: \
	static SfxViewFactory *pFactory; \
public: \
	static SfxViewShell  *CreateInstance(SfxViewFrame *pFrame, SfxViewShell *pOldView); \
	static void 		  RegisterFactory( USHORT nPrio = USHRT_MAX ); \
	static SfxViewFactory&Factory() { return *pFactory; } \
	static void           InitFactory()

#define SFX_IMPL_VIEWFACTORY(Class, rResId) \
	SfxViewFactory* Class::pFactory; \
	SfxViewShell* __EXPORT Class::CreateInstance(SfxViewFrame *pFrame, SfxViewShell *pOldView) \
	{ return new Class(pFrame, pOldView); } \
	void Class::RegisterFactory( USHORT nPrio ) \
	{ \
		pFactory = new SfxViewFactory(&CreateInstance,&InitFactory,nPrio,rResId);\
		InitFactory(); \
	} \
	void Class::InitFactory()

#define SFX_VIEW_REGISTRATION(DocClass) \
			DocClass::Factory().RegisterViewFactory( Factory() )

#define SFX_REGISTER_VIEW(ViewShellClass) \
			Factory().RegisterViewFactory( ViewShellClass::Factory() )

class SfxViewShell: public SfxShell, public SfxListener
{
#ifdef _SFXVIEWSH_HXX
friend class SfxViewFrame;
friend class SfxTopViewFrame;
friend class SfxPlugInFrame;
friend class SfxInPlaceFrame;
friend class SfxInternalFrame;
friend class SfxInternalObject; 
friend class SfxExternalTopViewFrame_Impl;
friend class SfxOfficeDocController;
friend class SfxBaseController;
#endif

	struct SfxViewShell_Impl*	pImp;

	SvInPlaceClientMemberList   aIPClientList;
	SfxViewFrame*				pFrame;
	SfxShell*					pSubShell;
	Window*						pWindow;
	BOOL                        bMaximizeFirst;
	BOOL						bOptimizeEach;
	BOOL						bNoNewWindow;

protected:
	virtual void                Activate(BOOL IsMDIActivate);
	virtual void                Deactivate(BOOL IsMDIActivate);

	virtual Size                GetOptimalSizePixel() const;

	virtual void				InnerResizePixel( const Point &rOfs, const Size &rSize );
	virtual void                OuterResizePixel( const Point &rOfs, const Size &rSize );
	virtual void                SetZoomFactor( const Fraction &rZoomX, const Fraction &rZoomY );

	virtual void				Move();

	virtual void                SFX_NOTIFY( SfxBroadcaster& rBC,
							const TypeId& rBCType,
							const SfxHint& rHint,
							const TypeId& rHintType );
public:

	static SfxViewShell*        GetFirst( const TypeId* pType = 0, BOOL bOnlyVisible = TRUE );
	static SfxViewShell*        GetNext( const SfxViewShell& rPrev,
							 const TypeId* pType = 0, BOOL bOnlyVisible = TRUE );
	static SfxViewShell*        Current();
						TYPEINFO();
					SFX_DECL_INTERFACE(SFX_INTERFACE_SFXVIEWSH);
						SfxViewShell( SfxViewFrame *pFrame, USHORT nFlags = 0 );
	virtual                     ~SfxViewShell();

	virtual SfxInPlaceClient*	CreateIPClient( WorkWindow * pTop, WorkWindow * pDoc, Window * pDraw );
	SfxInPlaceClient *          GetIPClient() const;
	SfxInPlaceClient *          GetUIActiveClient() const;
	SfxInPlaceClient* 			FindIPClient( SvEmbeddedObject *pObj, Window *pObjParentWin ) const;
	ErrCode						DoVerb( SfxInPlaceClient *pIPClient, long nVerb );
	virtual ErrCode 			DoVerb(long nVerb);
	virtual	void				UIActivate( SvInPlaceObject *pIPObj );
	virtual	void				UIDeactivate( SvInPlaceObject *pIPObj );
	virtual ErrCode             DirectLoad( SfxFrame* pFrame, SfxMedium& rMedium );
	virtual void				JumpToMark( const String& rMark );

	void						VisAreaChanged(const Rectangle& rRect);

	BOOL                        IsMaximizeFirst() const { return bMaximizeFirst; }
	BOOL                        IsOptimizeEach() const { return bOptimizeEach; }
	HACK(inline) BOOL           UseObjectSize() const;
	SfxScrollingMode			GetScrollingMode() const;
	void						SetScrollingMode( SfxScrollingMode eMode );

	virtual USHORT              PrepareClose( BOOL bUI = TRUE, BOOL bForBrowsing = FALSE );
	virtual String              GetSelectionText( BOOL bCompleteWords = FALSE );
	virtual BOOL				HasSelection( BOOL bText = TRUE ) const;
	virtual SdrView*			GetDrawView() const;
	void                        SetSubShell( SfxShell *pShell );
	SfxShell*                   GetSubShell() const { return pSubShell; }
	void						AddSubShell( SfxShell& rShell );
	void						RemoveSubShell( SfxShell *pShell=NULL );
	SfxShell*					GetSubShell( USHORT );

	void						GotFocus() const;
	inline void					LostFocus() const;
	virtual void				ShowCursor( FASTBOOL bOn = TRUE );
	virtual FASTBOOL			KeyInput( const KeyEvent &rKeyEvent );

	Window*                     GetWindow() const { return pWindow; }
	void                        SetWindow( Window *pViewPort );
	virtual void				AdjustPosSizePixel( const Point &rOfs, const Size &rSize );
	const SvBorder&             GetBorderPixel() const;
	void                        SetBorderPixel( const SvBorder &rBorder );
	void						InvalidateBorder();
	inline SfxViewFrame*        GetViewFrame() const;
	void						AdjustVisArea(const Rectangle& rRect);

	virtual void				PreparePrint( PrintDialog *pPrintDialog = 0 );
	virtual ErrCode				DoPrint( SfxPrinter *pPrinter, PrintDialog *pPrintDialog, BOOL bSilent );
	virtual USHORT              Print( SfxProgress &rProgress, PrintDialog *pPrintDialog = 0 );
	virtual SfxPrinter*         GetPrinter( BOOL bCreate = FALSE );
	virtual USHORT              SetPrinter( SfxPrinter *pNewPrinter, USHORT nDiffFlags = SFX_PRINTER_ALL );
	virtual SfxTabPage*         CreatePrintOptionsPage( Window *pParent, const SfxItemSet &rOptions );
	virtual PrintDialog*        CreatePrintDialog( Window *pParent );
	void						LockPrinter( BOOL bLock = TRUE );
	BOOL						IsPrinterLocked() const;

	virtual void                WriteUserData( String&, BOOL bBrowse = FALSE );
	virtual void                ReadUserData( const String&, BOOL bBrowse = FALSE );
	virtual void                WriteUserDataSequence ( ::com::sun::star::uno::Sequence < ::com::sun::star::beans::PropertyValue >&, sal_Bool bBrowse = sal_False );
	virtual void                ReadUserDataSequence ( const ::com::sun::star::uno::Sequence < ::com::sun::star::beans::PropertyValue >&, sal_Bool bBrowse = sal_False );
	virtual void				QueryObjAreaPixel( Rectangle& rRect ) const;

	virtual SfxObjectShell*		GetObjectShell();

	virtual void				MarginChanged();
	const Size&					GetMargin() const;
	void						SetMargin( const Size& );
	void 						DisconnectAllClients();
	virtual SfxFrame*     		GetSmartSelf( SfxFrame* pSelf, SfxMedium& rMedium );
	BOOL						NewWindowAllowed() const			{ return !bNoNewWindow; }
	void						SetNewWindowAllowed( BOOL bSet )	{ bNoNewWindow = !bSet; }
	 void                        SetController( SfxBaseController* pController );
	::com::sun::star::uno::Reference< ::com::sun::star::frame::XController >
					GetController();
	 void                        AddModelessDialog( Dialog* pDialog, USHORT nSlotId );
	BOOL                        HasModelessDialog( USHORT nSlotId );
	void                        RemoveModelessDialog( USHORT nSlotId );

#if 0 // _SOLAR__PRIVATE
	FASTBOOL					GlobalKeyInput_Impl( const KeyEvent &rKeyEvent );
	BOOL                        IsImplementedAsFrameset_Impl() const;

	void 						NewIPClient_Impl( SfxInPlaceClient *pIPClient )
					{ aIPClientList.Insert(pIPClient); }
	void 						IPClientGone_Impl( SfxInPlaceClient *pIPClient )
					{ aIPClientList.Remove(pIPClient); }
	SvInPlaceClientMemberList*	GetIPClientList_Impl( BOOL bCreate = TRUE ) const
					{ return &((SfxViewShell*)this)->aIPClientList; }
	void						DisconnectClients_Impl(SvInPlaceClient *pIP);
	void						DiscardClients_Impl();
	BOOL						PlugInsActive() const;

	SfxPrinter*					SetPrinter_Impl( SfxPrinter *pNewPrinter );
	BOOL						IsShowView_Impl() const;

	void                        ExecPrint_Impl(SfxRequest &);
	void                        ExecMisc_Impl(SfxRequest &);
	void						GetState_Impl(SfxItemSet&);
	SfxFrameSetDescriptor* 		GetFrameSet_Impl() const;
	void						SetFrameSet_Impl(SfxFrameSetDescriptor*);
	void 						CheckIPClient_Impl( SvInPlaceClient*, const Rectangle& );
	void						PushSubShells_Impl( BOOL bPush=TRUE );
	SfxMenuBarManager*			GetMenuBar_Impl( BOOL bPlugin = FALSE );
	 void                        ReleaseMenuBar_Impl();
	SfxAcceleratorManager*		GetAccMgr_Impl();
#endif
};

inline void	SfxViewShell::LostFocus() const

{
}

inline SfxViewFrame* SfxViewShell::GetViewFrame() const

{
	return pFrame;
}

#endif 

