

#ifndef _EMBOBJ_HXX
#define _EMBOBJ_HXX

#include <so3/persist.hxx>
#include <so3/pseudo.hxx>
#include <so3/protocol.hxx>
#include <so3/svstor.hxx>
#include <com/sun/star/uno/Reference.hxx>
#ifndef _MAPMOD_HXX 
#include <vcl/mapmod.hxx>
#endif
#include <svtools/ownlist.hxx>

class Printer;

#define ASPECT_CONTENT      1
#define ASPECT_THUMBNAIL    2 
#define ASPECT_ICON         4
#define ASPECT_DOCPRINT     8
#define ASPECT_ALL      (ASPECT_CONTENT | ASPECT_THUMBNAIL | ASPECT_ICON | ASPECT_DOCPRINT)

struct TransferableObjectDescriptor;
class ImpOleObject;
class ImpViewObject;
class SvEmbeddedClient;
class OutputDevice;
class Region;
class Menu;
class GDIMetaFile;
class JobSetup;
class SotStorage;
class SvStorage;
namespace com { namespace sun { namespace star { namespace datatransfer { class XTransferable; } } } }

class SvEmbeddedObject : virtual public SvPersist,
			 public SvPseudoObject
{
friend ImpOleObject;
friend SvEmbeddedClient;
friend SvEditObjectProtocol;
friend ImplSvEditObjectProtocol;
friend SvPersist; 
friend class SwWW6WrGrf; 
	BOOL                    bAutoSave:1,   
				bAutoHatch:1;  
	Rectangle               aVisArea;      
	MapUnit                 nMapUnit;
	String                  aDocName;      

	void			DrawHatch( OutputDevice * pDev, const Point & rPos,
				   const Size & rSize );
protected:
	SvEditObjectProtocol    aProt;           
		~SvEmbeddedObject();
	virtual BOOL    Load( SvStorage * );
	virtual BOOL    Save();
	virtual BOOL    SaveAs( SvStorage * pNewStg );
	virtual void    LoadContent( SvStream & rStm, BOOL bOwner );
	virtual void    SaveContent( SvStream & rStm, BOOL bOwner );
	virtual void    Connect( BOOL bConnect );
	virtual void    Open( BOOL bOpen );
	virtual void    Embed( BOOL bEmbedded );
	virtual void    PlugIn( BOOL bPlugIn );
	virtual ErrCode Verb( long nVerbPos, SvEmbeddedClient * pCallerClient,
			Window * pWin, const Rectangle * pWorkAreaPixel );

	virtual BOOL	Close();
	void            SetMapUnit( MapUnit nUnit )
		{ nMapUnit = nUnit; }
	virtual void    Draw( OutputDevice *, const JobSetup & rSetup,
			  USHORT nAspect = ASPECT_CONTENT );
	virtual void    DocumentNameChanged( const String & rDocName );

public:
		SvEmbeddedObject();
		SO2_DECL_STANDARD_CLASS_DLL(SvEmbeddedObject,SOAPP)
	virtual USHORT  FuzzyLock( BOOL bLock, BOOL bIntern, BOOL bClose );

	virtual void    FillClass( SvGlobalName * pClassName,
				   ULONG * pFormat,
				   String * pAppName,
				   String * pFullTypeName,
				   String * pShortTypeName,
	                           long nFileFormat = SOFFICE_FILEFORMAT_CURRENT ) const;
	void			FillTransferableObjectDescriptor( TransferableObjectDescriptor& rDesc ) const;
	SvGlobalName    GetClassName() const
		{ return SvPseudoObject::GetClassName(); }
	String			GetFullTypeName() const
		{ return SvPseudoObject::GetFullTypeName(); }

	void            SetAutoSave( BOOL bAutoSaveP )
		{ bAutoSave = bAutoSaveP; }
	BOOL            IsAutoSave() const { return bAutoSave; }
	virtual void    SetModified( BOOL = TRUE );

	void            SetAutoHatch( BOOL bAutoHatchP )
		{ bAutoHatch = bAutoHatchP; }
	BOOL            IsAutoHatch() const { return bAutoHatch; }

	SvEmbeddedClient * GetClient() const { return aProt.GetClient(); }

	static BOOL InsertStarObject
	( const String &rObjName, const String& rStgName,
	  SvStream& rSrc, SvPersist* pPersist );

	static BOOL LoadStarObjectPicture( SvStream& rSrc, GDIMetaFile& rMTF );

	static BOOL     MakeContentStream( SvStorage * pStor, const GDIMetaFile & );
	static BOOL     MakeContentStream( SotStorage * pStor, const GDIMetaFile & );

	virtual ULONG	GetMiscStatus() const;

	ErrCode         DoConnect( SvEmbeddedClient * );
	ErrCode         DoEmbed( BOOL bEmbedded );
	ErrCode         DoPlugIn( BOOL bPLugIn );
	ErrCode         DoOpen( BOOL bOpen );
	ErrCode         DoVerb( long nVerbPos = 0L );
	ErrCode         DoVerb( long nVerbPos, SvEmbeddedClient * pCallerClient,
				Window * pWin, const Rectangle * pWorkAreaPixel );

	const String &  GetDocumentName() const { return aDocName; }
	void			SetDocumentName( const String & rName );

	SvEditObjectProtocol & GetProtocol() { return aProt; }
	const SvEditObjectProtocol & GetProtocol() const { return aProt; }

	void			SendViewChanged();
	void			ViewChanged( USHORT nAspect);

	virtual ::com::sun::star::uno::Reference<
			::com::sun::star::datatransfer::XTransferable >
		CreateTransferableSnapshot();

	GDIMetaFile& GetGDIMetaFile( GDIMetaFile& );
	virtual BOOL SetData( const String& rData );

	MapUnit         GetMapUnit() const { return nMapUnit; }

	virtual Rectangle GetVisArea( USHORT nAspect ) const;
	const Rectangle & GetVisArea() const;
	virtual void    SetVisArea( const Rectangle & rVisArea );

	const Rectangle & SetGetVisArea( const Rectangle & rVisArea );
	void            SetVisAreaSize( const Size & rVisSize );

	void            DoDraw( OutputDevice *, const Point & rObjPos,
				const Fraction & rXScale,
				const Fraction & rScaleY,
				const JobSetup & rSetup,
				USHORT nAspect = ASPECT_CONTENT );
	void            DoDraw( OutputDevice *, const Point & rObjPos,
				const Size & rSize,
				const JobSetup & rSetup,
				USHORT nAspect = ASPECT_CONTENT );

	virtual	Printer *		GetDocumentPrinter();

	virtual	void			OnDocumentPrinterChanged( Printer * pNewPrinter );

	virtual UINT32			GetViewAspect() const;

	static BOOL ConvertToOle2( SvStream& rStm, UINT32 nLen, const GDIMetaFile*,
					const SotStorageRef & rDest );
};
#ifndef SO2_DECL_SVEMBEDDEDOBJECT_DEFINED
#define SO2_DECL_SVEMBEDDEDOBJECT_DEFINED
SO2_DECL_REF(SvEmbeddedObject)
#endif
SO2_IMPL_REF(SvEmbeddedObject)

class SvEmbeddedInfoObject : public SvInfoObject
{
	Rectangle       aVisArea;
	UINT32			nViewAspect;
protected:
		~SvEmbeddedInfoObject();
	virtual void	Assign( const SvInfoObject * pObj );
public:
		SV_DECL_PERSIST1(SvEmbeddedObject,SvInfoObject,2)
		SvEmbeddedInfoObject();
		SvEmbeddedInfoObject( SvEmbeddedObject * );
		SvEmbeddedInfoObject( SvEmbeddedObject *, const String & rObjName );
		SvEmbeddedInfoObject( const String& rObjName,
				   	  const SvGlobalName& rClassName );

	virtual void    SetObj( SvPersist * );

	SvEmbeddedObject * GetEmbed() const { return &SvEmbeddedObjectRef( GetObj() ); }
	BOOL            IsLink() const;
	const Rectangle & GetVisArea() const;
	const Rectangle & GetInfoVisArea() const { return aVisArea; }
	void SetInfoVisArea( const Rectangle& r) { aVisArea = r; }

	UINT32 GetInfoViewAspect() const { return nViewAspect; }
	void SetInfoViewAspect( UINT32 nAspect ) { nViewAspect = nAspect; }
	UINT32			GetViewAspect() const;
};

#endif 
