

#ifndef _FACTORY_HXX
#define _FACTORY_HXX

#ifndef _LIST_HXX 
#include <tools/list.hxx>
#endif
#include <so3/so2ref.hxx>
#ifndef _SFXBRDCST_HXX 
#include <svtools/brdcst.hxx>
#endif
#ifndef _GLOBNAME_HXX 
#include <tools/globname.hxx>
#endif
#ifndef _SOT_FACTORY_HXX
#include <sot/factory.hxx>
#endif

class SvObject;
class SvStorage;
class SvFactory;
class ImpFactory;
class SvClassManager;
struct IClassFactory;

class PlugInManager; 

#if ((SUPD >= 535) && defined(_USE_NAMESPACE))
namespace inet {
	class INetWrapper;
}
#else
class INetWrapper;
#endif

typedef SvGlobalName SvUniqueName;

class SvFactory : public SotFactory

{
friend ImpFactory;
friend class SvFactory_Impl;
	SvFactory_Impl *	pImp;
	ULONG               nRegisterId;

	void                Revoke();
protected:
	virtual             ~SvFactory();
public:
			TYPEINFO();

	static  void        IncAliveCount();
	static  void        DecAliveCount( BOOL bImmediate = TRUE );
	static  UINT32      GetAliveCount();
	static  void        SetAliveHdl( const Link & rAliveHdl );

	static  void        InsertInReleaseList( SvObject * pReleaseObj );
	static  void        ClearReleaseList();

	static  BOOL        			Init();
	static  void        			ClearDemandObjects();
	static  void        			DeInit();
	static  SvClassManager & 		GetInfoClassManager();

	static  String      			GetConfigFileName();

#if ((SUPD >= 535) && defined(_USE_NAMESPACE))
	static  inet::INetWrapper * 	GetINetWrapper();
#else
	static  INetWrapper * 			GetINetWrapper();
#endif

	static  PlugInManager * 		GetPlugInManager(); 

	static  void					SetDefaultPlugInFactory( SvFactory * );
	static  SvFactory * 			GetDefaultPlugInFactory();

	static  void					SetDefaultAppletFactory( SvFactory * );
	static  SvFactory * 			GetDefaultAppletFactory();

	static SvObjectRef  Create( const SvGlobalName & rSvClassName,
					const SvGlobalName & pOleClassName );
	SvObjectRef         Create( const SvGlobalName & rSvClassName ) const;
	SvObjectRef         CreateAndLoad( SvStorage *, BOOL bLink = FALSE ) const;
	SvObjectRef  		CreateAndLoad( SvStream &, ULONG * nLen = NULL ) const;
	SvObjectRef         CreateAndInit( const SvGlobalName & rSvClassName,
						   SvStorage * ) const;
	SvObjectRef         CreateAndInit( const String & rFileName,
						   SvStorage *, BOOL bLink = FALSE ) const;
	SvFactory( const SvGlobalName &,
		   const String & rClassName, CreateInstanceType );

	void                Register();
	BOOL                IsRegistered() const { return nRegisterId != 0; }

	static void			SetAutoTreatAs( const SvGlobalName & rOldClass,
							const String & rOldFullUserTypeName,
							const SvGlobalName & rNewClass );
	static SvGlobalName	GetAutoTreatAs( const SvGlobalName & rClass );

	static void			SetAutoConvertTo( const SvGlobalName & rOldClass,
							const String & rOldFullUserTypeName,
							const SvGlobalName & rNewClass );
	static SvGlobalName	GetAutoConvertTo( const SvGlobalName & rClass );
	static SvGlobalName GetSvClass31( const SvGlobalName & rClass );
	static SvGlobalName GetSvClass( INT32 nFileFormat, const SvGlobalName & rClass );
	static BOOL			IsIntern31( const SvGlobalName & rClass );
	static BOOL			IsIntern( const SvGlobalName & rClass,
				   	  long *pFileFormat	);
	static SvGlobalName GetServerName( long nStorageFormat );

private:
			SvFactory( const SvFactory & );
	SvFactory &         operator = ( const SvFactory & );
};

class SvOleFactory : public SvFactory
{
	SvGlobalNameList aOleIfaceList;
public:
	TYPEINFO();
		SvOleFactory( const SvGlobalName &,
				  const String & rClassName, CreateInstanceType );
	void		AddOleInterfaceName( const SvGlobalName & rName )
	{
		aOleIfaceList.Append( rName );
	}
	SvGlobalName GetOleInterfaceName( ULONG nIdx )
	{
		return aOleIfaceList.GetObject( nIdx );
	}
	BOOL		IsEntry( const SvGlobalName & rName )
	{
		return aOleIfaceList.IsEntry( rName );
	}
};

#define SV_OLE_FACTORY(ClassName)									\
class ClassName : public SvOleFactory                      			\
{                                                                   \
public:                                                             \
 		TYPEINFO();                                                 \
		ClassName( const SvGlobalName & rName,						\
		  		const String & rClassName,              \
		  		CreateInstanceType pCreateFuncP )       \
			: SvOleFactory( rName, rClassName, pCreateFuncP )

class SvOneInstanceFactory : public SvFactory
{
	SotObject * pObj;
	ULONG		nRegId;
public:
	TYPEINFO();
			SvOneInstanceFactory( const SvGlobalName &,
						const String & rClassName,
						CreateInstanceType );
	virtual void *	CreateInstance( SotObject ** ppObj = NULL ) const;
	void 			SetObject( SotObject * );
	void 			RemoveObject();
	SotObject *  	GetObject() const { return pObj; }
};

#endif 
