
#ifndef _INSDLG_HXX
#define _INSDLG_HXX

#ifndef _GLOBNAME_HXX 
#include <tools/globname.hxx>
#endif
#ifndef _OWNCONT_HXX
#include <svtools/ownlist.hxx>
#endif
#ifndef _SO2REF_HXX
#include <so3/so2ref.hxx>
#endif

class SvAppletObject;

#ifndef SO2_DECL_SVINPLACEOBJECT_DEFINED
#define SO2_DECL_SVINPLACEOBJECT_DEFINED
SO2_DECL_REF(SvInPlaceObject)
#endif
#ifndef SV_DECL_SVBASELINK2_DEFINED
#define SV_DECL_SVBASELINK2_DEFINED
SV_DECL_REF(SvBaseLink2)
#endif

class SvObjectServer
{
private:
	SvGlobalName    aClassName;
	String          aHumanName;

public:
	SvObjectServer( const SvGlobalName & rClassP, const String & rHumanP ) :
		aClassName( rClassP ),
		aHumanName( rHumanP ) {}

	const SvGlobalName &    GetClassName() const { return aClassName; }
	const String &          GetHumanName() const { return aHumanName; }
};

class SvObjectServerList
{
	PRV_SV_DECL_OWNER_LIST(SvObjectServerList,SvObjectServer)
	const SvObjectServer *  Get( const String & rHumanName ) const;
	const SvObjectServer *  Get( const SvGlobalName & ) const;
	void                    Remove( const SvGlobalName & );

	void					FillInsertObjects();
};

class SvStorage;
class INetURLObject;

class SvDialog

{
protected:
	ULONG			nHelpId;

public:
		SvDialog() : nHelpId( 0 ) {}

	void			SetHelpId( ULONG nNewId )	{ nHelpId = nNewId; }
};

class SvInsertOleObjectDialog : public SvDialog

{
private:
	String				_aFileName;
	BOOL				_bCreateNew	: 1,
			_bLink 		: 1;

public:
			SvInsertOleObjectDialog() : _bCreateNew(FALSE) {}

	static void			FillObjectServerList( SvObjectServerList* );

	SvInPlaceObjectRef	Execute( Window* pParent, SvStorage* pStor,
					 const SvObjectServerList* pServers = NULL );
	const String&		GetFileName() const { return _aFileName; }
	BOOL				IsCreateNew() const { return _bCreateNew; }
	BOOL				IsLinked() const	{ return _bLink; }
};

class SvInsertPlugInDialog : public SvDialog

{
private:
	INetURLObject*		pURL;
	String				aCommands;

public:
			SvInsertPlugInDialog() : pURL( NULL ) {}
			~SvInsertPlugInDialog();

	SvInPlaceObjectRef	Execute( Window* pParent, SvStorage* pStor );

	INetURLObject*		GetURL() const { return pURL; }
	const String&		GetCommands() const { return aCommands; }
};

class SvInsertAppletDialog : public SvDialog

{
private:
	INetURLObject*		pURL;
	String				aClass;
	String				aCommands;

public:
			SvInsertAppletDialog() : pURL( NULL ) {}
			~SvInsertAppletDialog();

#if SUPD<=358
	SvInPlaceObjectRef	Execute( Window* pParent, SvStorage* pStor );
	SvInPlaceObjectRef	Execute( Window* pParent, SvStorage* pStor,
					 SvAppletObject * );
#else
	SvInPlaceObjectRef	Execute( Window* pParent, SvStorage* pStor,
					 SvAppletObject * = 0);
#endif

	INetURLObject*		GetURL() const { return pURL; }
	const String&		GetClass() const { return aClass; }
	const String&		GetCommands() const { return aCommands; }
};

#endif 

