

#ifndef _IPWIN_HXX
#define _IPWIN_HXX

#ifndef _GEN_HXX 
#include <tools/gen.hxx>
#endif

#ifndef _WINDOW_HXX 
#include <vcl/window.hxx>
#endif
#include <so3/so2ref.hxx>
#include <so3/iface.hxx>

class SvResizeHelper
{
	Size        aBorder;
	Rectangle   aOuter;
	short       nGrab; 
	Point       aSelPos;
	BOOL		bResizeable;
public:
	SvResizeHelper();

	void		SetResizeable( BOOL b ) { bResizeable = b; }
	short       GetGrab() const { return nGrab; }
	void        SetBorderPixel( const Size & rBorderP )
	{ aBorder = rBorderP; }
	const Size & GetBorderPixel() const { return aBorder; }
	const Rectangle & GetOuterRectPixel() const
	{ return aOuter; }
	void        SetOuterRectPixel( const Rectangle & rRect )
	{ aOuter = rRect; }
	Rectangle   GetInnerRectPixel() const
	{
		Rectangle aRect( aOuter );
		aRect.Top()    += aBorder.Height();
		aRect.Left()   += aBorder.Width();
		aRect.Bottom() -= aBorder.Height();
		aRect.Right()  -= aBorder.Width();
		return aRect;
	}

	void        FillHandleRectsPixel( Rectangle aRects[ 8 ] ) const;
	void        FillMoveRectsPixel( Rectangle aRects[ 4 ] ) const;
	void        Draw( OutputDevice * );
	void        InvalidateBorder( Window * );
	BOOL        SelectBegin( Window *, const Point & rPos );
	void        SelectBegin( Window *, short nGrab );
	short       SelectMove( Window * pWin, const Point & rPos );
	Point       GetTrackPosPixel( const Rectangle & rRect ) const;
	Rectangle	GetTrackRectPixel( const Point & rTrackPos ) const;
	void        ValidateRect( Rectangle & rValidate ) const;
	BOOL        SelectRelease( Window *, const Point & rPos, Rectangle & rOutPosSize );
	void        Release( Window * pWin );
};

class SvResizeWindow : public Window
{
	Pointer         aOldPointer;
	short           nMoveGrab;  
	SvBorder        aBorder;
	Point           aPosCorrection;
	Window *        pObjWin;
protected:
	SvResizeHelper  aResizer;
	void            AdjustObjWin();
public:
			SvResizeWindow( Window * pParent );

	void    SetObjWin( Window * pNewWin );
	Window * GetObjWin() const
			{ return pObjWin; }

	void    SetHatchBorderPixel( const Size & rSize )
			{
	 aResizer.SetBorderPixel( rSize );
	 AdjustObjWin();
			}
	const Size & GetHatchBorderPixel() const
			{ return aResizer.GetBorderPixel(); }

	void    SetBorderPixel( const SvBorder & rNewBorder );
	const SvBorder & GetBorderPixel() const
			{ return aBorder; }

	SvBorder GetAllBorderPixel() const
			{
	SvBorder aB( aBorder );
	return aB += GetHatchBorderPixel();
			}

	void    SetPosCorrectionPixel( const Point & rNewCorr )
			{ aPosCorrection = rNewCorr; }
	const Point & GetPosCorrectionPixel() const
			{ return aPosCorrection; }

	Rectangle CalcInnerRectPixel( const Point & rPos, const Size & rSize ) const;
	Rectangle CalcOuterRectPixel( const Point & rPos, const Size & rSize ) const;
	Rectangle GetInnerRectPixel() const;

	void    SetInnerPosSizePixel( const Point &, const Size & );

	void    TerminateResizing() { aResizer.Release( this ); }

	void    SelectMouse( const Point & rPos );
	void    MouseButtonUp( const MouseEvent & rEvt );
	void    MouseMove( const MouseEvent & rEvt );
	void    MouseButtonDown( const MouseEvent & rEvt );
	void    KeyInput( const KeyEvent & rEvt );
	void    Resize();
	void    Paint( const Rectangle & );
	virtual void	QueryObjAreaPixel( Rectangle & );
	virtual void    RequestObjAreaPixel( const Rectangle & );
};

class SvInPlaceEnvironment;
class SvInPlaceWindow : public SvResizeWindow
{
	SvInPlaceEnvironment *    pIPEnv;
public:
		SvInPlaceWindow( Window * pParent, SvInPlaceEnvironment * );
	void            KeyInput( const KeyEvent & rEvt );
	virtual void	QueryObjAreaPixel( Rectangle & );
	virtual void    RequestObjAreaPixel( const Rectangle & );
};

class SvInPlaceClipWindow : public Window
{
	SvResizeWindow * pResizeWin;
	Rectangle        aMaxClip;
public:
			SvInPlaceClipWindow( Window * pParent );

	void    SetResizeWin( SvResizeWindow * pNewWin )
			{ pResizeWin = pNewWin; }
	SvResizeWindow * GetResizeWin() const
			{ return pResizeWin; }

	void    SetRectsPixel( const Rectangle & rObjRect,
			   const Rectangle & rInPlaceWinMaxRect );
	void    SetBorderPixel( const SvBorder & rNewBorder );
};

#endif 
