
#ifndef _PSEUDO_HXX
#define _PSEUDO_HXX

#ifndef _UNQID_HXX 
#include <tools/unqid.hxx>
#endif
#ifndef _IFACE_HXX
#include <so3/iface.hxx>
#endif
#ifndef _OWNLIST_HXX
#include <svtools/ownlist.hxx>
#endif

class  ImpOleObject;
class  SvBaseLink;
class  SvEmbeddedClient;
class  SvLinkName;
class  Menu;

class SvVerb

{
	long            nId;
	String          aName;
	UniqueItemId    aMenuId;
	BOOL    		bOnMenu : 1,
		bConst  : 1;
public:
		SvVerb( long nIdP, const String & rNameP,
				BOOL bConstP = FALSE, BOOL bOnMenuP = TRUE );
		SvVerb( const SvVerb & );
	SvVerb &        operator = ( const SvVerb & );
		~SvVerb();

	const String &  GetName() const { return aName; }
	long            GetId() const { return nId; }
	USHORT          GetMenuId() const { return (USHORT)aMenuId.GetId(); }
	BOOL            IsConst() const { return bConst; }
	BOOL            IsOnMenu() const { return bOnMenu; }
};

class SvVerbList
{
	PRV_SV_DECL_OWNER_LIST(SvVerbList,SvVerb)
};

#define SVVERB_SHOW     	-1
#define SVVERB_OPEN     	-2
#define SVVERB_HIDE     	-3
#define SVVERB_UIACTIVATE	-4
#define SVVERB_IPACTIVATE   -5

#define SVVERB_PROPS		1

#define SVOBJ_MISCSTATUS_SERVERRESIZE 			1
#define SVOBJ_MISCSTATUS_NOTREPLACE   			4
#define SVOBJ_MISCSTATUS_CANTLINKINSIDE			16
#define SVOBJ_MISCSTATUS_LINK         			64
#define SVOBJ_MISCSTATUS_INSIDEOUT    			128
#define SVOBJ_MISCSTATUS_ACTIVATEWHENVISIBLE    256
#define SVOBJ_MISCSTATUS_NOTRESIZEABLE			512
#define SVOBJ_MISCSTATUS_ALWAYSACTIVATE			1024
#define SVOBJ_MISCSTATUS_RESIZEONPRINTERCHANGE	2048
#define SVOBJ_MISCSTATUS_SPECIALOBJECT          4096

class SvPseudoObject : virtual public SvObject

{
friend ImpOleObject;
	SvVerbList *	pVerbs;          	
	BOOL			bDeleteVerbs;		
protected:
		~SvPseudoObject();

	virtual ErrCode Verb( long nVerbPos, SvEmbeddedClient * pCallerClient,
			Window * pWin, const Rectangle * pWorkAreaPixel );
	virtual BOOL	Close();
public:
	SvPseudoObject();

	SO2_DECL_STANDARD_CLASS_DLL( SvPseudoObject, SOAPP )

	virtual void    FillClass( SvGlobalName * pClassName,
				   ULONG * pFormat,
				   String * pAppName,
				   String * pFullTypeName,
				   String * pShortTypeName,
	                           long nFileFormat = SOFFICE_FILEFORMAT_CURRENT ) const;
	SvGlobalName    GetClassName() const;
	String			GetAppName() const;
	String			GetFullTypeName() const;
	String			GetShortTypeName() const;

	void            SetVerbList( SvVerbList * pVerbsP );
	virtual const SvVerbList & GetVerbList() const;
	const SvVerb *  GetVerb( USHORT nMenuId ) const;
	ErrCode         DoVerb( long nVerbPos, SvEmbeddedClient * pCallerClient,
				Window * pWin, const Rectangle * pWorkAreaPixel );
	void            AppendVerbs( Menu & rMenu );

	virtual ULONG	GetMiscStatus() const;

};

#ifndef SO2_DECL_SVPSEUDOOBJECT_DEFINED
#define SO2_DECL_SVPSEUDOOBJECT_DEFINED
SO2_DECL_REF(SvPseudoObject)
#endif
SO2_IMPL_REF( SvPseudoObject )

#endif

