

#ifndef _STGCACHE_HXX
#define _STGCACHE_HXX

#ifndef _TOOLS_SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _TOOLS_STREAM_HXX
#include <tools/stream.hxx>
#endif

#ifndef _STGELEM_HXX
#include <sot/stgelem.hxx>
#endif

class StgIo;
class StgPage;
class StgDirEntry;
class StorageBase;

class StgCache {
	StgPage* pCur;							
	StgPage* pElem1;						
	ULONG nError;							
	INT32 nPages;							
	USHORT nRef;							
	void * pLRUCache;						
	short nPageSize;						
	void Erase( StgPage* );					
	void InsertToLRU( StgPage* );			
	void InsertToOrdered( StgPage* );		
	StgPage* Create( INT32 );				
protected:
	SvStream* pStrm;						
	BOOL  bMyStream;						
	BOOL  bFile;							
	INT32 Page2Pos( INT32 );				
	INT32 Pos2Page( INT32 );				
public:
	StgCache();
	~StgCache();
	void  IncRef()						{ nRef++;			}
	USHORT DecRef()						{ return --nRef;	}
	void  SetPhysPageSize( short );
	INT32 GetPhysPages()				{ return nPages;	}
	short GetPhysPageSize()				{ return nPageSize; }
	SvStream* GetStrm()	  				{ return pStrm;  	}
	void  SetStrm( SvStream*, BOOL );
	BOOL  IsWritable()					{ return pStrm->IsWritable(); }
	BOOL  Good()						{ return BOOL( nError == SVSTREAM_OK ); }
	BOOL  Bad()                         { return BOOL( nError != SVSTREAM_OK ); }
	ULONG GetError()					{ return nError; 	}
	void  MoveError( StorageBase& );
	void  SetError( ULONG );
	void  ResetError();
	BOOL  Open( const String& rName, StreamMode );
	void  Close();
	BOOL  Read( INT32 nPage, void* pBuf, INT32 nPages );
	BOOL  Write( INT32 nPage, void* pBuf, INT32 nPages );
	BOOL  SetSize( INT32 nPages );
	StgPage* Find( INT32 );					
	StgPage* Get( INT32, BOOL );			
	StgPage* Copy( INT32, INT32=STG_FREE );	
	BOOL Commit( StgDirEntry* = NULL );		
	void Revert( StgDirEntry* = NULL );		
	void Clear();							
};

class StgPage {
	friend class StgCache;
	StgCache* pCache;						
	StgPage *pNext1, *pLast1;				
	StgPage *pNext2, *pLast2;				
	StgDirEntry* pOwner;					
	INT32	nPage;							
	void*	pData;							
	short	nData;							
	BOOL	bDirty;							
	StgPage( StgCache*, short );
	~StgPage();
public:
	void  SetDirty()					{ bDirty = TRUE; 	 		}
	INT32 GetPage()						{ return nPage;				}
	void* GetData()						{ return pData; 	 		}
	short GetSize()						{ return nData; 	 		}
	void  SetOwner( StgDirEntry* p )	{ pOwner = p;				}

	INT32 GetPage( short nOff )
	{
		if( ( nOff >= (short) ( nData / sizeof( INT32 ) ) ) || nOff < 0 )
			return -1;
		INT32 n = ((INT32*) pData )[ nOff ];
#ifdef __BIGENDIAN
		return SWAPLONG(n);
#else
		return n;
#endif
	}
	void  SetPage( short, INT32 );		
};

#endif
