

#ifndef _STGSTRMS_HXX
#define _STGSTRMS_HXX

#ifndef _TOOLS_STREAM_HXX
#include <tools/stream.hxx>
#endif

class StgIo;
class StgStrm;
class StgPage;
class StgDirEntry;

class StgFAT
{										
	StgStrm& rStrm;						
	INT32 nMaxPage;						
	short nPageSize;					
	short nEntries;						
	short nOffset;						
	INT32 nLimit;						
	BOOL  bPhys;						
	StgPage* GetPhysPage( INT32 nPage );
	BOOL  MakeChain( INT32 nStart, INT32 nPages );
	BOOL  InitNew( INT32 nPage1 );
public:
	StgFAT( StgStrm& rStrm, BOOL bMark );
	INT32 FindBlock( INT32& nPages );
	INT32 GetNextPage( INT32 nPg );
	INT32 AllocPages( INT32 nStart, INT32 nPages );
	BOOL  FreePages( INT32 nStart, BOOL bAll );
	INT32 GetMaxPage() { return nMaxPage; }
	void  SetLimit( INT32 n ) { nLimit = n; }
};

class StgStrm {							
protected:
	StgIo& rIo;							
	StgFAT* pFat;						
	StgDirEntry* pEntry;				
	INT32 nStart;						
	INT32 nSize;						
	INT32 nPos;							
	INT32 nPage;						
	short nOffset;						
	short nPageSize;					
	BOOL  Copy( INT32 nFrom, INT32 nBytes );
	StgStrm( StgIo& );
public:
	~StgStrm();
	StgIo&  GetIo()		{ return rIo;	 }
	INT32   GetPos()	{ return nPos;   }
	INT32	GetStart()	{ return nStart; }
	INT32	GetSize()	{ return nSize;	 }
	INT32   GetPage()	{ return nPage;  }
	short   GetPageSize() { return nPageSize; }
	INT32	GetPages();
	short	GetOffset()	{ return nOffset;}
	void    SetEntry( StgDirEntry& );
	virtual BOOL SetSize( INT32 );
	virtual BOOL Pos2Page( INT32 nBytePos );
	virtual INT32 Read( void*, INT32 ) 		  { return 0; }
	virtual INT32 Write( const void*, INT32 ) { return 0; }
	virtual StgPage* GetPhysPage( INT32 nBytePos, BOOL bForce = FALSE );
	virtual BOOL IsSmallStrm() { return FALSE; }
};

class StgFATStrm : public StgStrm {		
	virtual BOOL Pos2Page( INT32 nBytePos );
	BOOL  SetPage( short, INT32 );
public:
	StgFATStrm( StgIo& );
	INT32 GetPage( short, BOOL, USHORT *pnMasterAlloc = 0);
	virtual BOOL SetSize( INT32 );
	virtual StgPage* GetPhysPage( INT32 nBytePos, BOOL bForce = FALSE );
};

class StgDataStrm : public StgStrm		
{
	short nIncr;						
	void Init( INT32 nBgn, INT32 nLen );
public:
	StgDataStrm( StgIo&, INT32 nBgn, INT32 nLen=-1 );
	StgDataStrm( StgIo&, StgDirEntry* );
	void* GetPtr( INT32 nPos, BOOL bForce, BOOL bDirty );
	void SetIncrement( short n ) { nIncr = n ; }
	virtual BOOL SetSize( INT32 );
	virtual INT32 Read( void*, INT32 );
	virtual INT32 Write( const void*, INT32 );
};

class StgSmallStrm : public StgStrm		
{
	StgStrm* pData;						
	void Init( INT32 nBgn, INT32 nLen );
public:
	StgSmallStrm( StgIo&, INT32 nBgn, INT32 nLen );
	StgSmallStrm( StgIo&, StgDirEntry* );
	virtual INT32 Read( void*, INT32 );
	virtual INT32 Write( const void*, INT32 );
	virtual BOOL IsSmallStrm() { return TRUE; }
};

class StgTmpStrm : public SvMemoryStream
{
	String aName;
	SvFileStream* pStrm;
	virtual ULONG GetData( void* pData, ULONG nSize );
	virtual ULONG PutData( const void* pData, ULONG nSize );
	virtual ULONG SeekPos( ULONG nPos );
	virtual void FlushData();

public:
	StgTmpStrm( ULONG=16 );
   ~StgTmpStrm();
	BOOL Copy( StgTmpStrm& );
	void SetSize( ULONG );
	ULONG GetSize();
};

#endif
