

#ifndef _SOT_STORAGE_HXX
#define _SOT_STORAGE_HXX

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif

#ifndef _SOT_OBJECT_HXX
#include <sot/object.hxx>
#endif
#ifndef _SOT_FACTORY_HXX
#include <sot/factory.hxx>
#endif
#ifndef _TOOLS_STREAM_HXX
#include <tools/stream.hxx>
#endif
#ifndef _TOOLS_ERRCODE_HXX
#include <tools/errcode.hxx>
#endif

#define STORAGE_FAILIFTHERE		0x02
#define STORAGE_TRANSACTED		0x04
#define STORAGE_PRIORITY		0x08
#define STORAGE_DELETEONRELEASE	0x10
#define STORAGE_CONVERT			0x20
#define STORAGE_UNPACKED_MODE   0x40
#define STORAGE_CREATE_UNPACKED 0x44
typedef short StorageMode;

class SotStorage;
class BaseStorageStream;
class SotStorageStream : virtual public SotObject, public SvStream
{
friend SotStorage;
friend class ImpStream;
	BaseStorageStream * pOwnStm;
protected:
	virtual ULONG       GetData( void* pData, ULONG nSize );
	virtual ULONG       PutData( const void* pData, ULONG nSize );
	virtual ULONG       SeekPos( ULONG nPos );
	virtual void        FlushData();
	                    ~SotStorageStream();
public:
	                    SotStorageStream( const String &,
	                                 StreamMode = STREAM_STD_READWRITE,
	                                 StorageMode = 0 );
	                    SotStorageStream( BaseStorageStream *pStm );
	                    SotStorageStream();
	                    SO2_DECL_BASIC_CLASS_DLL(SotStorageStream,SOTDATA())
			SO2_DECL_INVARIANT()

	virtual void 		SyncSvStream();
	void 				SyncSysStream() { SvStream::SyncSysStream(); }
	 virtual void        ResetError();
	 virtual void        SetSize( ULONG nNewSize );
	UINT32				GetSize() const;
	BOOL				CopyTo( SotStorageStream * pDestStm );
	virtual BOOL        Commit();
	virtual BOOL        Revert();
	BOOL                SetProperty( const String& rName, const ::com::sun::star::uno::Any& rValue );
	BOOL                GetProperty( const String& rName, ::com::sun::star::uno::Any& rValue );
};

#ifndef SOT_DECL_SOTSTORAGESTREAM_DEFINED
#define SOT_DECL_SOTSTORAGESTREAM_DEFINED
SO2_DECL_REF(SotStorageStream)
#endif
SO2_IMPL_REF(SotStorageStream)

namespace ucb
{
	class Content;
};

class  SvStorageInfoList;
class  BaseStorage;
class SotStorage : virtual public SotObject
{
friend class SotStorageStream;
friend class SvStorage;
	BaseStorage *   pTmpStg;   
	BaseStorage *   pOwnStg;   
	SvStream *  pStorStm;  
	ULONG       nError;
	String      aName;      
	BOOL        bIsRoot:1,  
	            bDelStm:1;
	ByteString  aKey;           
	long 		nVersion;
protected:
	                    ~SotStorage();
   void 				CreateStorage( BOOL bUCBStorage, StreamMode, StorageMode );
public:
	                    SotStorage( const String &,
	                               StreamMode = STREAM_STD_READWRITE,
	                               StorageMode = 0 );
	                    SotStorage( BOOL bUCBStorage, const String &,
	                               StreamMode = STREAM_STD_READWRITE,
	                               StorageMode = 0 );
	                    SotStorage( const ::ucb::Content& rContent, const String &,
	                               StreamMode = STREAM_STD_READWRITE,
	                               StorageMode = 0 );
	                    SotStorage( BaseStorage * );
	                    SotStorage( SvStream & rStm );
	                    SotStorage( BOOL bUCBStorage, SvStream & rStm );
	                    SotStorage( SvStream * pStm, BOOL bDelete );
	                    SotStorage();
	                    SO2_DECL_BASIC_CLASS_DLL(SotStorage,SOTDATA())
			SO2_DECL_INVARIANT()
	 SvMemoryStream *    CreateMemoryStream();
	const SvStream *	GetSvStream();
	 static BOOL         IsStorageFile( const String & rFileName );
	static BOOL         IsStorageFile( SvStream* pStream );
	 virtual const String & GetName() const;

	virtual BOOL        Validate();
	 void                SetKey( const ByteString& rKey );
	const ByteString &  GetKey() const { return aKey; }

	void				SetVersion( long nVers )
			{
				nVersion = nVers;
			}
	long				GetVersion() const
			{
				return nVersion;
			}
	 ULONG               GetErrorCode() const { return nError; }
	ULONG               GetError() const { return ERRCODE_TOERROR(nError); }
	void                SetError( ULONG nErrorCode )
	                    {
	                        if( nError == SVSTREAM_OK )
	                            nError = nErrorCode;
	                    }
	virtual void        ResetError();
	 BOOL                IsRoot() const              { return bIsRoot; }
	void                SignAsRoot( BOOL b = TRUE ) { bIsRoot = b; }
	void				SetDeleteStream( BOOL bDelete ) { bDelStm = bDelete; }
	 virtual void        SetClass( const SvGlobalName & rClass,
	                              ULONG bOriginalClipFormat,
	                              const String & rUserTypeName );
	virtual void        SetConvertClass( const SvGlobalName & rConvertClass,
	                                     ULONG bOriginalClipFormat,
	                                     const String & rUserTypeName );
	virtual SvGlobalName GetClassName();
	virtual ULONG       GetFormat();
	virtual String      GetUserName();
	virtual BOOL        ShouldConvert();
	void                SetName( const String& rName );
	 virtual void        FillInfoList( SvStorageInfoList * ) const;
	virtual BOOL        CopyTo( SotStorage * pDestStg );
	virtual BOOL        Commit();
	virtual BOOL        Revert();
	 SotStorageStream *  OpenSotStream( const String & rEleName,
	                                StreamMode = STREAM_STD_READWRITE,
	                                StorageMode = 0 );
	SotStorage *		OpenSotStorage( const String & rEleName,
	                                StreamMode = STREAM_STD_READWRITE,
	                                StorageMode = STORAGE_TRANSACTED );
	SotStorage *		OpenUCBStorage( const String & rEleName,
	                                StreamMode = STREAM_STD_READWRITE,
	                                StorageMode = STORAGE_TRANSACTED );
	SotStorage *        OpenOLEStorage( const String & rEleName,
	                                StreamMode = STREAM_STD_READWRITE,
	                                StorageMode = STORAGE_TRANSACTED );
	 virtual BOOL        IsStream( const String & rEleName ) const;
	virtual BOOL        IsStorage( const String & rEleName ) const;
	virtual BOOL        IsContained( const String & rEleName ) const;
	 virtual BOOL        Remove( const String & rEleName );
	 virtual BOOL        Rename( const String & rEleName,
	                            const String & rNewName );
	virtual BOOL        CopyTo( const String & rEleName, SotStorage * pDest,
	                            const String & rNewName );
	virtual BOOL        MoveTo( const String & rEleName, SotStorage * pDest,
	                            const String & rNewName );

	SvStream*			GetTargetSvStream() const;
	BOOL                SetProperty( const String& rName, const ::com::sun::star::uno::Any& rValue );
	BOOL                GetProperty( const String& rName, ::com::sun::star::uno::Any& rValue );
	BOOL				IsOLEStorage() const;
	static BOOL         IsOLEStorage( const String & rFileName );
	static BOOL         IsOLEStorage( SvStream* pStream );
};

#ifndef SOT_DECL_SOTSTORAGE_DEFINED
#define SOT_DECL_SOTSTORAGE_DEFINED
SO2_DECL_REF(SotStorage)
#endif
SO2_IMPL_REF(SotStorage)

#endif 
