
#ifndef _SVTOOLS_BROOKER_HXX_
#define _SVTOOLS_BROOKER_HXX_

#include <tools/bcst.hxx>
#include <svtools/communi.hxx>

class FilterMessagePack
{
	BroadcastMessage &maMsg;
	BroadcastFilterFlag mnFilterFlags;
public:
	FilterMessagePack( BroadcastMessage &aMsg, BroadcastFilterFlag nFilterFlags )
		: maMsg( aMsg )
		, mnFilterFlags( nFilterFlags )
	{}

	BroadcastMessage& GetMsg() { return maMsg; }
	BroadcastFilterFlag GetFlags() { return mnFilterFlags; }
};

class TypedCommunicationLinkList;
SV_DECL_PTRARR_SORT( ClientLists, TypedCommunicationLinkList*, 1, 10 )

class InformationBrooker
{
	LinkList aMessageArrivedLnkList;
	LinkList aExtraCommandLnkList;
	LinkList aFilterMessageLnkList;
	CommunicationManager *mpManager;

	ClientLists *mpClientLists;

	DECL_LINK( ManagerOpen, CommunicationLink* );
	DECL_LINK( ManagerClose, CommunicationLink* );
	DECL_LINK( ManagerData, CommunicationLink* );

	void ImplDisconnectAll();
	void ImplRemoveLink( TypedCommunicationLinkList *pLinkList, CommunicationLink* pCL );
	void ImplHandleListeningLink( BroadcastCommand aCmd, BroadcastCategory nType, CommunicationLink* pCL );

	virtual void OpenConnection( CommunicationLink* pCL ){;}
	virtual void CloseConnection( CommunicationLink* pCL ){;}
	virtual void ReceiveData( CommunicationLink* pCL ){;}	

public:
	InformationBrooker();
	virtual ~InformationBrooker();

	void Broadcast( ByteString aMsg, BroadcastCategory nCategory, ClientID aCI = BCST_CLIENT_BROOKER, BroadcastFilterFlag nFlags = BCST_FILTER_NOFILTER );

	USHORT GetCommunicationLinkCount() { return mpManager->GetCommunicationLinkCount(); }
	CommunicationLinkRef GetCommunicationLink( USHORT nNr ) { return mpManager->GetCommunicationLink( nNr ); }

	virtual BOOL ExtraCommand( SvStream *pData );

	void AddExtraCommandHdl( Link aLink ) { aExtraCommandLnkList.Insert( new Link( aLink ), LIST_APPEND ); }
	void RemoveExtraCommandHdl( Link aLink );

	virtual void MessageArrived( const BroadcastMessage &aMsg );

	void AddMessageArrivedHdl( Link aLink ) { aMessageArrivedLnkList.Insert( new Link( aLink ), LIST_APPEND ); }
	void RemoveMessageArrivedHdl( Link aLink );

	virtual BOOL FilterMessage( BroadcastMessage &aMsg, BroadcastFilterFlag &nFilterFlags );

	void AddFilterMessageHdl( Link aLink ) { aFilterMessageLnkList.Insert( new Link( aLink ), LIST_APPEND ); }
	void RemoveFilterMessageHdl( Link aLink );

};

#endif

