
#ifndef _SVX_BRWBOX_HXX
#define _SVX_BRWBOX_HXX

#include <limits.h>

#ifndef _SV_SCRBAR_HXX 
#include <vcl/scrbar.hxx>
#endif
#ifndef _SV_CTRL_HXX 
#include <vcl/ctrl.hxx>
#endif
#ifndef _EVENT_HXX 

#endif

#ifndef _HEADBAR_HXX
#include <svtools/headbar.hxx>
#endif
#ifndef _TRANSFER_HXX
#include <svtools/transfer.hxx>
#endif

class BrowserColumn;
class BrowserColumns;
class BrowserDataWin;
class MultiSelection;
class BrowserHeader;

#define BROWSER_INVALIDID           USHRT_MAX
#define BROWSER_ENDOFSELECTION      -1

typedef ULONG BrowserMode;

#define BROWSER_COLUMNSELECTION      0x0001
#define BROWSER_MULTISELECTION       0x0002
#define BROWSER_THUMBDRAGGING        0x0004
#define BROWSER_KEEPHIGHLIGHT        0x0008
#define BROWSER_KEEPSELECTION        BROWSER_KEEPHIGHLIGHT	
#define BROWSER_HLINES               0x0010
#define BROWSER_VLINES               0x0020
#define BROWSER_HLINESFULL           BROWSER_HLINES 
#define BROWSER_VLINESFULL           BROWSER_VLINES 
#define BROWSER_HLINESDOTS           0x0000 
#define BROWSER_VLINESDOTS           0x0000 

#define BROWSER_HIDESELECT           0x0100	
#define BROWSER_HIDECURSOR           0x0200

#define BROWSER_NO_HSCROLL           0x0400
#define BROWSER_NO_SCROLLBACK        0x0800

#define BROWSER_AUTO_VSCROLL	   	 0x1000
#define BROWSER_AUTO_HSCROLL	   	 0x2000

#define BROWSER_TRACKING_TIPS 	  	 0x4000

#define BROWSER_unused            	 0x8000

#define BROWSER_HIGHLIGHT_NONE       0x0100 
#define BROWSER_HIGHLIGHT_TOGGLE 0x00000000 
#define BROWSER_HIGHLIGHT_AUTO   0x00010000
#define BROWSER_HIGHLIGHT_MANU   0x00020000

#define BROWSER_HEADERBAR_NEW	 0x00040000
#define BROWSER_AUTOSIZE_LASTCOL 0x00080000
#define BROWSER_OWN_DATACHANGED	 0x00100000

#define BROWSER_CURSOR_WO_FOCUS  0x00200000

typedef int BrowserColumnMode;
#define BROWSER_COLUMN_TITLEABBREVATION   1
#define BROWSER_COLUMN_STANDARD         	BROWSER_COLUMN_TITLEABBREVATION

#define BROWSER_NONE                      0
#define BROWSER_SELECT                  720
#define BROWSER_EXPANDSELECTION         721
#define BROWSER_ENHANCESELECTION        722
#define BROWSER_SELECTALL               723
#define BROWSER_SELECTDOWN              724
#define BROWSER_SELECTUP                725
#define BROWSER_CURSORDOWN              731
#define BROWSER_CURSORUP                732
#define BROWSER_CURSORLEFT              733
#define BROWSER_CURSORRIGHT             734
#define BROWSER_CURSORPAGEDOWN          735
#define BROWSER_CURSORPAGEUP            736
#define BROWSER_CURSORPAGERIGHT         735
#define BROWSER_CURSORPAGELEFT          736
#define BROWSER_CURSORENDOFFILE         741
#define BROWSER_CURSORTOPOFFILE         742
#define BROWSER_CURSORENDOFSCREEN       743
#define BROWSER_CURSORTOPOFSCREEN       744
#define BROWSER_CURSORHOME              745
#define BROWSER_CURSOREND               746
#define BROWSER_SCROLLDOWN              751
#define BROWSER_SCROLLUP                752
#define BROWSER_SELECTHOME				753
#define BROWSER_SELECTEND				754

class BrowseEvent
{
	Window*             pWin;
	long                nRow;
	Rectangle           aRect;
	USHORT              nCol;
	USHORT              nColId;

public:
			BrowseEvent();
			BrowseEvent( Window* pWindow,
						 long nAbsRow,
						 USHORT nColumn, USHORT nColumnId,
						 const Rectangle& rRect );

	Window*             GetWindow() const { return pWin; }
	long                GetRow() const { return nRow; }
	USHORT              GetColumn() const { return nCol; }
	USHORT              GetColumnId() const { return nColId; }
	const Rectangle&    GetRect() const { return aRect; }
};

class BrowserMouseEvent: public MouseEvent, public BrowseEvent
{
public:
	BrowserMouseEvent();
	BrowserMouseEvent( BrowserDataWin* pWin, const MouseEvent& rEvt );
	BrowserMouseEvent( Window* pWin, const MouseEvent& rEvt,
		   long nAbsRow, USHORT nColumn, USHORT nColumnId,
		   const Rectangle& rRect );
};

class BrowserAcceptDropEvent : public AcceptDropEvent, public BrowseEvent
{
public:
	BrowserAcceptDropEvent();
	BrowserAcceptDropEvent( BrowserDataWin* pWin, const AcceptDropEvent& rEvt );
};

class BrowserExecuteDropEvent : public ExecuteDropEvent, public BrowseEvent
{
public:
	BrowserExecuteDropEvent();
	BrowserExecuteDropEvent( BrowserDataWin* pWin, const ExecuteDropEvent& rEvt );
};

class BrowseBox
		:public Control
#if SUPD>626 || FS_PRIV_DEBUG
		,public DragSourceHelper
#endif
#if SUPD>627 || FS_PRIV_DEBUG
		,public DropTargetHelper
#endif
{
	friend class BrowserDataWin;
	friend class BrowserColumn;
	friend class StringColumn;
	friend class BitmapColumn;
	friend class HandleColumn;

#ifdef DBG_MI
	friend void DoLog_Impl( const BrowseBox *pThis, const char *pWhat, const char *pWho );
#endif

#ifdef DBG_UTIL
	friend const char* BrowseBoxCheckInvariants( const void * pVoid );
#endif

	Window*         pDataWin;       
	ScrollBar*      pVScroll;       
	ScrollBar       aHScroll;       

	long            nDataRowHeight; 
	USHORT          nTitleLines;    
	ULONG           nControlAreaWidth; 
	BOOL            bThumbDragging; 
	BOOL            bColumnCursor;  
	BOOL            bMultiSelection;
	BOOL            bKeepHighlight; 

	BOOL            bHLines;        
	BOOL            bVLines;        
	BOOL            bHDots;         
	BOOL            bVDots;         
	Color			aLineColor;		
	BOOL            bBootstrapped;  
	BOOL            bHasBitmapHandle; 
	long            nTopRow;        
	long            nCurRow;        
	long            nRowCount;      
	USHORT          nFirstCol;      
	USHORT          nCurColId;      

	BOOL			bSelecting;
	BOOL            bDrag;
	BOOL            bHit;
	BOOL            bRubber;
	Point           a1stPoint;
	Point           a2ndPoint;

	long            nResizeX;       
	long            nMinResizeX;    
	long            nDragX;         
	USHORT          nResizeCol;     
	BOOL            bResizing;      

	BOOL            bSelect;        
	BOOL            bSelectionIsVisible; 
	BOOL            bScrolling;     
	BOOL            bNotToggleSel;  
	BOOL            bHasFocus;      
	BOOL            bHideSelect;    
	BOOL            bHideCursor;    
	Range			aSelRange;		

	BrowserColumns* pCols;          
	union
	{
		MultiSelection* pSel;       
		long            nSel;       
	}               uRow;
	MultiSelection* pColSel;        

	BOOL			m_bFocusOnlyCursor;	
	Color			m_aCursorColor;		
	BrowserMode		m_nCurrentMode;		

private:
#if 0 // _SOLAR__PRIVATE
	void            Construct(BrowserMode nMode);
	void            ExpandRowSelection( const BrowserMouseEvent& rEvt );
	void            ToggleSelection( BOOL bForce = FALSE );

	void            UpdateScrollbars();
	void            AutoSizeLastColumn();

	long            ImpGetDataRowHeight() const;
	Rectangle       ImplFieldRectPixel( long nRow, USHORT nColId ) const;
	USHORT          FrozenColCount() const;

	void            ColumnInserted( USHORT nPos );

	DECL_LINK( 		ScrollHdl, ScrollBar * );
	DECL_LINK( 		EndScrollHdl, ScrollBar * );
	DECL_LINK( 		StartDragHdl, HeaderBar * );

	long            GetFrozenWidth() const;
#endif

	BOOL            GoToRow(long nRow, BOOL bRowColMove, BOOL bDoNotModifySelection = FALSE );

	BOOL            GoToColumnId( USHORT nColId, BOOL bMakeVisible, BOOL bRowColMove = FALSE);
	void            SelectColumnPos( USHORT nCol, BOOL bSelect, BOOL bMakeVisible);
	void            SelectColumnId( USHORT nColId, BOOL bSelect, BOOL bMakeVisible)
			{ SelectColumnPos( GetColumnPos(nColId), bSelect, bMakeVisible); }

	void			ImplPaintData(OutputDevice& _rOut, const Rectangle& _rRect, BOOL _bForeignDevice, BOOL _bDrawSelections);

	BOOL			PaintCursorIfHiddenOnce() const { return !m_bFocusOnlyCursor && !HasFocus(); }

protected:
	USHORT          ColCount() const;

	virtual long    GetRowCount() const;

	void 			SetRealRowCount( const String &rRealRowCount );

	virtual BOOL    SeekRow( long nRow ) = 0;
	virtual void    DrawCursor();
	virtual void	PaintRow( OutputDevice &rDev, const Rectangle &rRect );
	virtual void    PaintData( Window& rWin, const Rectangle& rRect );
	virtual void    PaintField( OutputDevice& rDev, const Rectangle& rRect,
					USHORT nColumnId ) const = 0;

	virtual void    VisibleRowsChanged( long nNewTopRow, USHORT nNumRows);

	USHORT          GetVisibleRows()
			{ return (USHORT)((pDataWin->GetOutputSizePixel().Height() - 1 )/ GetDataRowHeight() + 1); }
	long            GetTopRow() { return nTopRow; }
	USHORT			GetFirstVisibleColNumber() const { return nFirstCol; }

	void			DoShowCursor( const char *pWhoLog );
	void			DoHideCursor( const char *pWhoLog );
	short			GetCursorHideCount() const;

	virtual BrowserHeader*	CreateHeaderBar( BrowseBox* pParent );

	void			SetHeaderBar( BrowserHeader* );

	long			CalcReverseZoom(long nVal);

	HeaderBar*		GetHeaderBar() const;

#if SUPD>627 || FS_PRIV_DEBUG
	inline const DataFlavorExVector&
		GetDataFlavors() const;

private:
	void*			implGetDataFlavors() const;

#endif

public:
		BrowseBox( Window* pParent, WinBits nBits = 0,
				   BrowserMode nMode = 0 );
		BrowseBox( Window* pParent, const ResId& rId,
				   BrowserMode nMode = 0 );
		~BrowseBox();

	virtual void	StateChanged( StateChangedType nStateChange );
	virtual void    MouseButtonDown( const MouseEvent& rEvt );
	virtual void    MouseMove( const MouseEvent& rEvt );
	virtual void    MouseButtonUp( const MouseEvent& rEvt );
	virtual void    KeyInput( const KeyEvent& rEvt );
	virtual void    LoseFocus();
	virtual void    GetFocus();
	virtual void    Resize();
	virtual void    Paint( const Rectangle& rRect );
	virtual void	Draw( OutputDevice* pDev, const Point& rPos, const Size& rSize, ULONG nFlags );
	virtual void    Command( const CommandEvent& rEvt );
#if SUPD>626 || FS_PRIV_DEBUG
	virtual void	StartDrag( sal_Int8 _nAction, const Point& _rPosPixel );
#endif
#if SUPD>627 || FS_PRIV_DEBUG
	virtual sal_Int8 AcceptDrop( const AcceptDropEvent& rEvt );		
	virtual sal_Int8 ExecuteDrop( const ExecuteDropEvent& rEvt );	

	virtual sal_Int8 AcceptDrop( const BrowserAcceptDropEvent& rEvt );
	virtual sal_Int8 ExecuteDrop( const BrowserExecuteDropEvent& rEvt );
#endif

	virtual void    MouseButtonDown( const BrowserMouseEvent& rEvt );
	virtual void    MouseMove( const BrowserMouseEvent& rEvt );
	virtual void    MouseButtonUp( const BrowserMouseEvent& rEvt );
	virtual void    StartScroll();
	virtual void    EndScroll();
	virtual void    Select();
	virtual void    DoubleClick( const BrowserMouseEvent& rEvt );
	virtual BOOL	IsCursorMoveAllowed( long nNewRow, USHORT nNewColId ) const;
	virtual void    CursorMoved();
	virtual void    ColumnMoved( USHORT nColId );
	virtual void    ColumnResized( USHORT nColId );
	virtual long    QueryColumnResize( USHORT nColId, long nWidth );

	void            SetUpdateMode( BOOL bUpdate );
	BOOL            GetUpdateMode() const;
	void            Invalidate();
	void            Invalidate( const Rectangle& rRect )
			{ Control::Invalidate( rRect ); }

	void            SetMapMode( const MapMode& rNewMapMode );
	void            SetFont( const Font& rNewFont );
	const Font&     GetFont() const { return pDataWin->GetFont(); }
	void            SetTitleFont( const Font& rNewFont )
			{ Control::SetFont( rNewFont ); }
	const Font&     GetTitleFont() const { return Control::GetFont(); }

	void			SetLineColor(const Color& rColor) {aLineColor = rColor;}
	const Color&	GetLineColor() const {return aLineColor;}

	void            InsertHandleColumn( ULONG nWidth, BOOL bBitmap = FALSE );
	void			InsertDataColumn( USHORT nItemId, const Image& rImage,
						long nSize, HeaderBarItemBits nBits = HIB_STDSTYLE,
						USHORT nPos = HEADERBAR_APPEND );
	void			InsertDataColumn( USHORT nItemId, const XubString& rText,
						long nSize, HeaderBarItemBits nBits = HIB_STDSTYLE,
						USHORT nPos = HEADERBAR_APPEND );
	void			InsertDataColumn( USHORT nItemId,
						const Image& rImage, const XubString& rText,
						long nSize, HeaderBarItemBits nBits = HIB_STDSTYLE,
						USHORT nPos = HEADERBAR_APPEND,
							const String* pHelpText = 0 );
	void            SetColumnTitle( USHORT nColumnId, const String &rTitle );
	void            SetColumnMode( USHORT nColumnId, BrowserColumnMode nFlags );
	void            SetColumnWidth( USHORT nColumnId, ULONG nWidth );
	void            SetColumnPos( USHORT nColumnId, USHORT nPos );
	void            FreezeColumn( USHORT nColumnId, BOOL bFreeze = TRUE );
	void            UnfreezeColumns();
	void            RemoveColumn( USHORT nColumnId );
	void            RemoveColumns();

	void            SetDataRowHeight( long nPixel );
	long            GetDataRowHeight() const;
	void            SetTitleLines( USHORT nLines );
	USHORT          GetTitleLines() const { return nTitleLines; }
	virtual long	GetTitleHeight() const;

	String          GetColumnTitle( USHORT nColumnId ) const;
	BrowserColumnMode GetColumnMode( USHORT nColumnId ) const;
	Rectangle       GetFieldRect( USHORT nColumnId ) const;
	ULONG           GetColumnWidth( USHORT nColumnId ) const;
	USHORT          GetColumnId( USHORT nPos ) const;
	USHORT          GetColumnPos( USHORT nColumnId ) const;
	BOOL            IsFrozen( USHORT nColumnId ) const;

	void            ToTop();
	void            ToBottom();
	void            ResetScroll();
	long            ScrollColumns( long nColumns );
	long            ScrollRows( long nRows );
	long            ScrollPages( long nPagesY );
	BOOL			MakeFieldVisible( long nRow, USHORT nColId, BOOL bComplete = FALSE );

	long            GetCurRow() const { return nCurRow; }
	USHORT          GetCurColumnId() const { return nCurColId; }
	BOOL            GoToRow( long nRow );
	BOOL			GoToRowAndDoNotModifySelection( long nRow );
	BOOL            GoToColumnId( USHORT nColId );
	BOOL            GoToRowColumnId( long nRow, USHORT nColId );

	void            SetNoSelection();
	void            SelectAll();
	void            SelectRow( long nRow, BOOL bSelect = TRUE, BOOL bExpand = TRUE );
	void            SelectColumnPos( USHORT nCol, BOOL bSelect = TRUE )
			{ SelectColumnPos( nCol, bSelect, TRUE); }
	void            SelectColumnId( USHORT nColId, BOOL bSelect = TRUE )
			{ SelectColumnPos( GetColumnPos(nColId), bSelect, TRUE); }
	long            GetSelectRowCount() const;
	USHORT          GetSelectColumnCount() const;
	BOOL            IsRowSelected( long nRow ) const;
	BOOL            IsColumnSelected( USHORT nColumnId ) const;
	long            FirstSelectedRow( BOOL bInverse = FALSE );
	long            LastSelectedRow( BOOL bInverse = FALSE );
	long            PrevSelectedRow();
	long            NextSelectedRow();
	const MultiSelection* GetColumnSelection() const { return pColSel; }
	const MultiSelection* GetSelection() const
		{ return bMultiSelection ? uRow.pSel : 0; }
	void			SetSelection( const MultiSelection &rSelection );
	BOOL			IsResizing() const { return bResizing; }

	Window&         GetEventWindow() const;
	Window&         GetDataWindow() const { return *pDataWin; }
	Rectangle       GetRowRectPixel( long nRow,
						 BOOL bRelToBrowser = TRUE ) const;
	Rectangle       GetFieldRectPixel( long nRow, USHORT nColId,
						   BOOL bRelToBrowser = TRUE  ) const;
	BOOL            IsFieldVisible( long nRow, USHORT nColId,
						BOOL bComplete = FALSE ) const;
	long            GetRowAtYPosPixel( long nY,
							BOOL bRelToBrowser = TRUE  ) const;
	USHORT          GetColumnAtXPosPixel( long nX,
							  BOOL bRelToBrowser = TRUE  ) const;

	void            Clear();
	void            RowRemoved( long nRow, long nNumRows = 1, BOOL bDoPaint = TRUE );
	void            RowModified( long nRow, USHORT nColId = USHRT_MAX );
	void            RowInserted( long nRow, long nNumRows = 1, BOOL bDoPaint = TRUE, BOOL bKeepSelection = FALSE );

	void            ReserveControlArea( USHORT nWidth = USHRT_MAX );
	Rectangle       GetControlArea() const;
	BOOL 			ProcessKey( const KeyEvent& rEvt );
	void            Dispatch( USHORT nId );
	void            SetMode( BrowserMode nMode = 0 );
	BrowserMode		GetMode( ) const { return m_nCurrentMode; }
	BOOL			IsInCommandEvent() const;

	void			SetCursorColor(const Color& _rCol);
	Color			GetCursorColor() const { return m_aCursorColor; }
	void			ResetSelecting() { bSelecting = FALSE; }
};

#if SUPD>627 || FS_PRIV_DEBUG

inline const DataFlavorExVector& BrowseBox::GetDataFlavors() const
{
	return *reinterpret_cast<DataFlavorExVector*>(implGetDataFlavors());
}
#endif

#endif

