

#ifndef _SVTOOLS_CMDPARSE_HXX
#define _SVTOOLS_CMDPARSE_HXX

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _LIST_HXX 
#include <tools/list.hxx>
#endif

namespace svt
{

#ifndef STRING_LIST
#define STRING_LIST
DECLARE_LIST( StringList, String * )
#endif

class CommandParser
{

	const char aDelimiterChar;
	const char aAssignChar;
	const char aQuoteChar;

	const BOOL bOptionsOnly;

	BOOL bAnyOptionAllowed;

	StringList aParameterStringList;

	StringList aOptionNameList;
	StringList aOptionStringList;

	String aInvalidOptions;

	static const String aEmptyString;

	void AddParameter(const String & rParameterString);

  public:

	CommandParser(const String & rString = String(),
	  const char aNewDelimiterChar = '/',
	  const char aNewAssignChar = ':',
	  const BOOL bNewOptionsOnly = FALSE,
	  const char aNewQuoteChar = '"');
	CommandParser(const CommandParser & rCommandParser);
	~CommandParser();

	const CommandParser & operator=(const CommandParser &rCommandParser);

	BOOL operator==(const CommandParser &rCommandParser) const;
	BOOL operator!=(const CommandParser &rCommandParser) const;

	void SetOption(const UINT32 nOptionNumber, const String & rOptionString);
	void SetOption(const String & rOptionName, const String & rOptionString);

	void Clear();

	void Parse(const String & rString);

	void Parse(int argc, char * * argv);

	const UINT32	OptionCount() const { return aOptionStringList.Count(); };
	const String &	OptionName(const UINT32 nOptionNumber) const;
	String			OptionString(const UINT32 nOptionNumber) const;
	String			OptionString(const String & rOptionName) const;
	const BOOL		OptionBool(const UINT32 nOptionNumber) const;
	const BOOL		OptionBool(const String & rOptionName) const;
	const INT32		OptionInt(const UINT32 nOptionNumber) const;
	const INT32		OptionInt(const String & rOptionName) const;
	const UINT32	OptionUInt(const UINT32 nOptionNumber) const;
	const UINT32	OptionUInt(const String & rOptionName) const;

	BOOL		   Exists(const String & rOptionName) const;

	const UINT32 ParameterCount() const { return aParameterStringList.Count(); };
	const String & ParameterString(const UINT32 nParameterNumber) const;
	const BOOL     ParameterBool(const UINT32 nParameterNumber) const;
	const INT32    ParameterInt(const UINT32 nParameterNumber) const;
	const UINT32   ParameterUInt(const UINT32 nParameterNumber) const;

	const BOOL IsInvalid() const { return aInvalidOptions.Len() != 0; }

	const String & InvalidOptions() const { return aInvalidOptions; }

	void Compose(String & rString, const String & rLeaveOutOptions) const;

};

inline BOOL CommandParser::operator!=(const CommandParser &rCommandParser) const
{
	return ! (*this == rCommandParser);
}

}	

#endif

