

#ifndef _COMMUNI_HXX
#define _COMMUNI_HXX

#ifndef _SVARRAY_HXX 
#include <svtools/svarray.hxx>
#endif
#ifndef _VOS_THREAD_HXX_ 
#include <vos/thread.hxx>
#endif
#ifndef _VOS_MUTEX_HXX_ 
#include <vos/mutex.hxx>
#endif

#ifndef _SIMPLECM_HXX
#include <tools/simplecm.hxx>
#endif

class SvStream;
class SvMemoryStream;

class CommunicationManagerServerAcceptThread;
SV_DECL_PTRARR_SORT( CommunicationLinkList, CommunicationLink*, 1, 10);

class MultiCommunicationManager : public CommunicationManager
{
public:
	MultiCommunicationManager( BOOL bUseMultiChannel = FALSE );
	virtual ~MultiCommunicationManager();
	virtual BOOL StopCommunication();		
	virtual BOOL IsLinkValid( CommunicationLink* pCL );
	virtual USHORT GetCommunicationLinkCount();
	virtual CommunicationLinkRef GetCommunicationLink( USHORT nNr );
protected:
	virtual void CallConnectionOpened( CommunicationLink* pCL );
	virtual void CallConnectionClosed( CommunicationLink* pCL );
	CommunicationLinkList *ActiveLinks;
	CommunicationLinkList *InactiveLinks;		

	virtual void DestroyingLink( CommunicationLink *pCL );	
};

class CommunicationManagerServer : public MultiCommunicationManager
{
public:
	CommunicationManagerServer( BOOL bUseMultiChannel = FALSE ):MultiCommunicationManager( bUseMultiChannel ){;}
};

class CommunicationManagerClient : public MultiCommunicationManager, public ICommunicationManagerClient
{
public:
	CommunicationManagerClient( BOOL bUseMultiChannel = FALSE );
};

class CommunicationLinkViaSocket : public SimpleCommunicationLinkViaSocket, public NAMESPACE_VOS(OThread)
{
public:
	CommunicationLinkViaSocket( CommunicationManager *pMan, NAMESPACE_VOS(OStreamSocket) *pSocket );
	virtual ~CommunicationLinkViaSocket();

	virtual BOOL IsCommunicationError();
	virtual BOOL DoTransferDataStream( SvStream *pDataStream, CMProtocol nProtocol = CM_PROTOCOL_OLDSTYLE );

	virtual long ConnectionClosed( void* = NULL );
	virtual long DataReceived( void* = NULL );

protected:
	virtual void SAL_CALL run();

	virtual BOOL ShutdownCommunication();	
	ULONG nConnectionClosedEventId;
	ULONG nDataReceivedEventId;
	NAMESPACE_VOS(OMutex) aMConnectionClosed;	
	NAMESPACE_VOS(OMutex) aMDataReceived;		
	virtual void WaitForShutdown();
};

class CommunicationManagerServerViaSocket : public CommunicationManagerServer
{
	friend CommunicationManagerServerAcceptThread;
public:
	CommunicationManagerServerViaSocket( ULONG nPort, USHORT nMaxCon, BOOL bUseMultiChannel = FALSE );
	virtual ~CommunicationManagerServerViaSocket();

	virtual BOOL StartCommunication();
	virtual BOOL StopCommunication();

protected:
	ULONG nPortToListen;
	USHORT nMaxConnections;

private:
	CommunicationManagerServerAcceptThread *pAcceptThread;
	void AddConnection( CommunicationLink *pNewConnection );
};

class CommunicationManagerServerAcceptThread: public NAMESPACE_VOS(OThread)
{
public:
	CommunicationManagerServerAcceptThread( CommunicationManagerServerViaSocket* pServer, ULONG nPort, USHORT nMaxCon = CM_UNLIMITED_CONNECTIONS );
	virtual ~CommunicationManagerServerAcceptThread();
	CommunicationLinkRef GetNewConnection(){ CommunicationLinkRef xTemp = xNewConnection; xNewConnection.Clear(); return xTemp; }

protected:
	virtual void SAL_CALL run();

private:
	NAMESPACE_VOS(OAcceptorSocket) *pAcceptorSocket;
	ULONG nPortToListen;
	USHORT nMaxConnections;
	CommunicationManagerServerViaSocket* pMyServer;
	ULONG nAddConnectionEventId;
	NAMESPACE_VOS(OMutex) aMAddConnection;	
	void CallInfoMsg( InfoString aMsg ){ pMyServer->CallInfoMsg( aMsg ); }
	CM_InfoType GetInfoType(){ return pMyServer->GetInfoType(); }

	CommunicationLinkRef xNewConnection;
	DECL_LINK( AddConnection, void* );
};

class CommunicationManagerClientViaSocket : public CommunicationManagerClient, CommonSocketFunctions
{
public:
	CommunicationManagerClientViaSocket( ByteString aHost, ULONG nPort, BOOL bUseMultiChannel = FALSE );
	CommunicationManagerClientViaSocket( BOOL bUseMultiChannel = FALSE );
	virtual ~CommunicationManagerClientViaSocket();

	virtual BOOL StartCommunication(){ return StartCommunication( aHostToTalk, nPortToTalk );}
	virtual BOOL StartCommunication( ByteString aHost, ULONG nPort ){ return DoStartCommunication( this, (ICommunicationManagerClient*) this, aHost, nPort  );}

private:
	ByteString aHostToTalk;
	ULONG nPortToTalk;
protected:
	virtual CommunicationLink *CreateCommunicationLink( CommunicationManager *pCM, NAMESPACE_VOS(OConnectorSocket) *pCS ){ return new CommunicationLinkViaSocket( pCM, pCS ); }
};

#endif
