

#ifndef _SFXFILEREC_HXX
#define _SFXFILEREC_HXX

#include <svtools/svarray.hxx>

#ifndef _DEBUG_HXX 
#include <tools/debug.hxx>
#endif

#include <tools/stream.hxx>

SV_DECL_VARARR( SfxUINT32s, UINT32, 8, 8 );

#define SFX_BOOL_DONTCARE				BOOL(2) 	

#define SFX_REC_PRETAG_EXT				BYTE(0x00)	
#define SFX_REC_PRETAG_EOR				BYTE(0xFF)	

#define SFX_REC_TYPE_NONE				BYTE(0x00)	
#define SFX_REC_TYPE_FIRST				BYTE(0x01)
#define SFX_REC_TYPE_SINGLE 			BYTE(0x01)	
#define SFX_REC_TYPE_FIXSIZE			BYTE(0x02)	
#define SFX_REC_TYPE_VARSIZE_RELOC      BYTE(0x03)	
#define SFX_REC_TYPE_VARSIZE            BYTE(0x04)	
#define SFX_REC_TYPE_MIXTAGS_RELOC  	BYTE(0x07)	
#define SFX_REC_TYPE_MIXTAGS			BYTE(0x08)	
#define SFX_REC_TYPE_LAST				BYTE(0x08)
#define SFX_REC_TYPE_MINI					0x100	
#define SFX_REC_TYPE_DRAWENG				0x400	
#define SFX_REC_TYPE_EOR					0xF00	

#define SFX_REC_HEADERSIZE_MINI 	4	
#define SFX_REC_HEADERSIZE_SINGLE	4	
#define SFX_REC_HEADERSIZE_MULTI	6	

#ifndef DBG
#ifdef DBG_UTIL
#define DBG(x) x
#else
#define DBG(x)
#endif
#endif

class SfxMiniRecordWriter

{
protected:
	SvStream*		_pStream;	
	UINT32			_nStartPos; 
	FASTBOOL		_bHeaderOk; 
	BYTE			_nPreTag;	

public:
	inline			SfxMiniRecordWriter( SvStream *pStream,
							 BYTE nTag );
	inline			SfxMiniRecordWriter( SvStream *pStream, BYTE nTag,
							 UINT32 nSize );

	inline			~SfxMiniRecordWriter();

	inline SvStream& operator*() const;

	inline void		Reset();

	UINT32			Close( FASTBOOL bSeekToEndOfRec = TRUE );

private:
			SfxMiniRecordWriter( const SfxMiniRecordWriter& );
	SfxMiniRecordWriter& operator=(const SfxMiniRecordWriter&);
};

class SfxMiniRecordReader

{
protected:
	SvStream*			_pStream;	
	UINT32				_nEofRec;	
	FASTBOOL			_bSkipped;	
	BYTE				_nPreTag;	
				SfxMiniRecordReader() {}
   void 				Construct_Impl( SvStream *pStream, BYTE nTag )
			{
				_pStream = pStream;
				_bSkipped = FALSE;
				_nPreTag = nTag;
			}
	inline FASTBOOL 	SetHeader_Impl( UINT32 nHeader );

	void				SetInvalid_Impl( UINT32 nRecordStartPos )
			{
				_nPreTag = SFX_REC_PRETAG_EOR;
				_pStream->Seek( nRecordStartPos );
			}

public:
	static USHORT   	ScanRecordType( SvStream *pStream );
				SfxMiniRecordReader( SvStream *pStream );
			SfxMiniRecordReader( SvStream *pStream, BYTE nTag );
	inline				~SfxMiniRecordReader();

	inline BYTE 		GetTag() const;
	inline FASTBOOL 	IsValid() const;

	inline SvStream&	operator*() const;

	inline void 		Skip();

private:
				SfxMiniRecordReader( const SfxMiniRecordReader& );
	SfxMiniRecordReader& operator=(const SfxMiniRecordReader&);
};

class SfxSingleRecordWriter: public SfxMiniRecordWriter

{
protected:
		SfxSingleRecordWriter( BYTE nRecordType,
							   SvStream *pStream,
							   UINT16 nTag, BYTE nCurVer );

public:
		SfxSingleRecordWriter( SvStream *pStream,
							   UINT16 nTag, BYTE nCurVer );
		SfxSingleRecordWriter( SvStream *pStream,
							   UINT16 nTag, BYTE nCurVer,
							   UINT32 nSize );

	inline void		Reset();

	UINT32			Close( FASTBOOL bSeekToEndOfRec = TRUE );
};

class SfxSingleRecordReader: public SfxMiniRecordReader

{
protected:
	UINT16				_nRecordTag;	
	BYTE				_nRecordVer;	
	BYTE				_nRecordType;	
				SfxSingleRecordReader() {}
	void				Construct_Impl( SvStream *pStream )
			{
				SfxMiniRecordReader::Construct_Impl(
						pStream, SFX_REC_PRETAG_EXT );
			}
	FASTBOOL			FindHeader_Impl( UINT16 nTypes, UINT16 nTag );
	FASTBOOL            ReadHeader_Impl( USHORT nTypes );

public:
			SfxSingleRecordReader( SvStream *pStream );
			SfxSingleRecordReader( SvStream *pStream, USHORT nTag );

	inline UINT16		GetTag() const;

	inline BYTE 		GetVersion() const;
	inline FASTBOOL 	HasVersion( USHORT nVersion ) const;
};

class SfxMultiFixRecordWriter: public SfxSingleRecordWriter

{
protected:
	UINT32			_nContentStartPos;	
	UINT32			_nContentSize;		
	UINT16			_nContentCount; 	
			SfxMultiFixRecordWriter( BYTE nRecordType,
								 SvStream *pStream,
								 UINT16 nTag, BYTE nCurVer,
								 UINT32 nContentSize );

public:
		SfxMultiFixRecordWriter( SvStream *pStream,
								 UINT16 nTag, BYTE nCurVer,
								 UINT32 nContentSize );
	inline			~SfxMultiFixRecordWriter();

	inline void 	NewContent();

	inline void		Reset();

	UINT32			Close( FASTBOOL bSeekToEndOfRec = TRUE );
};

class SfxMultiVarRecordWriter: public SfxMultiFixRecordWriter

{
protected:
	SfxUINT32s  		_aContentOfs;
	USHORT              _nContentVer;   
				SfxMultiVarRecordWriter( BYTE nRecordType,
									 SvStream *pStream,
									 USHORT nRecordTag,
									 BYTE nRecordVer );

	void                FlushContent_Impl();

public:
			SfxMultiVarRecordWriter( SvStream *pStream,
									 USHORT nRecordTag,
									 BYTE nRecordVer );
			~SfxMultiVarRecordWriter();

	void				NewContent();

	virtual UINT32		Close( FASTBOOL bSeekToEndOfRec = TRUE );
};

class SfxMultiMixRecordWriter: public SfxMultiVarRecordWriter

{
public:
	inline				SfxMultiMixRecordWriter( SvStream *pStream,
									 USHORT nRecordTag,
									 BYTE nRecordVer );

	void				NewContent( USHORT nTag, BYTE nVersion );

	void                NewContent()
			{ DBG_ERROR( "NewContent() only allowed with args" ); }
};

class SfxMultiRecordReader: public SfxSingleRecordReader

{
	UINT32				_nStartPos;		
	UINT32* 			_pContentOfs;	
	UINT32				_nContentSize;	
	UINT16				_nContentCount; 
	UINT16				_nContentNo;	
	UINT16				_nContentTag;	
	BYTE				_nContentVer;	

	FASTBOOL			ReadHeader_Impl();

public:
			SfxMultiRecordReader( SvStream *pStream );
			SfxMultiRecordReader( SvStream *pStream, UINT16 nTag );
			~SfxMultiRecordReader();

	FASTBOOL			GetContent();
	inline UINT16		GetContentTag();
	inline BYTE 		GetContentVersion() const;
	inline FASTBOOL 	HasContentVersion( USHORT nVersion ) const;

	inline UINT32		ContentCount() const;
};

inline SfxMiniRecordWriter::SfxMiniRecordWriter
(
	SvStream*       pStream,        
	BYTE            nTag            
)

:   _pStream( pStream ),
	_nPreTag( nTag ),
	_nStartPos( pStream->Tell() ),
	_bHeaderOk(FALSE)
{
	DBG_ASSERT( _nPreTag != 0xFF, "invalid Tag" );
	DBG( DbgOutf( "SfxFileRec: writing record to %ul", pStream->Tell() ) );

	pStream->SeekRel( + SFX_REC_HEADERSIZE_MINI );
}

inline SfxMiniRecordWriter::SfxMiniRecordWriter
(
	SvStream*		pStream,		
	BYTE			nTag,			
	UINT32			nSize			
)

:   _pStream( pStream ),

	_bHeaderOk(SFX_BOOL_DONTCARE)
{
	DBG_ASSERT( nTag != 0 && nTag != 0xFF, "invalid Tag" );
	DBG(_nStartPos = pStream->Tell());
	DBG( DbgOutf( "SfxFileRec: writing record to %ul", _nStartPos ) );

	*pStream << ( ( nTag << 24 ) | nSize );
}

inline SfxMiniRecordWriter::~SfxMiniRecordWriter()

{

	if ( !_bHeaderOk DBG(||TRUE) )
		Close();
}

inline SvStream& SfxMiniRecordWriter::operator*() const

{
	DBG_ASSERT( !_bHeaderOk, "getting Stream of closed record" );
	return *_pStream;
}

inline void	SfxMiniRecordWriter::Reset()
{
	_pStream->Seek( _nStartPos + SFX_REC_HEADERSIZE_MINI );
	_bHeaderOk = FALSE;
}

inline SfxMiniRecordReader::~SfxMiniRecordReader()

{

	if ( !_bSkipped )
		Skip();
}

inline void SfxMiniRecordReader::Skip()

{
	_pStream->Seek(_nEofRec);
	_bSkipped = TRUE;
}

inline BYTE SfxMiniRecordReader::GetTag() const

{
	return _nPreTag;
}

inline FASTBOOL SfxMiniRecordReader::IsValid() const

{
	return _nPreTag != SFX_REC_PRETAG_EOR;
}

inline SvStream& SfxMiniRecordReader::operator*() const

{
	DBG_ASSERT( _pStream->Tell() < _nEofRec, "read behind record" );
	return *_pStream;
}

inline UINT32 SfxSingleRecordWriter::Close( FASTBOOL bSeekToEndOfRec )

{
	UINT32 nRet = 0;

	if ( !_bHeaderOk )
	{

		UINT32 nEndPos = SfxMiniRecordWriter::Close( bSeekToEndOfRec );

		if ( !bSeekToEndOfRec )
			_pStream->SeekRel( SFX_REC_HEADERSIZE_SINGLE );
		nRet = nEndPos;
	}
#ifdef DBG_UTIL
	else

		SfxMiniRecordWriter::Close( bSeekToEndOfRec );
#endif

	return nRet;
}

inline void	SfxSingleRecordWriter::Reset()
{
	_pStream->Seek( _nStartPos + SFX_REC_HEADERSIZE_MINI +
					 SFX_REC_HEADERSIZE_SINGLE );
	_bHeaderOk = FALSE;
}

inline UINT16 SfxSingleRecordReader::GetTag() const

{
	return _nRecordTag;
}

inline BYTE SfxSingleRecordReader::GetVersion() const

{
	return _nRecordVer;
}

inline FASTBOOL SfxSingleRecordReader::HasVersion( USHORT nVersion ) const

{
	return _nRecordVer >= nVersion;
}

inline SfxMultiFixRecordWriter::~SfxMultiFixRecordWriter()

{

	if ( !_bHeaderOk )
		Close();
}

inline void SfxMultiFixRecordWriter::NewContent()

{
	ULONG nOldStartPos;

	DBG(nOldStartPos = _nContentStartPos;)
	_nContentStartPos = _pStream->Tell();

#ifdef DBG_UTIL

	if ( _nContentCount )
	{

		DBG_ASSERT( _nContentStartPos - nOldStartPos == _nContentSize,
		"wrong content size detected" );
	}
#endif

	++_nContentCount;
}

inline SfxMultiMixRecordWriter::SfxMultiMixRecordWriter
(
	SvStream*		pStream,	
	USHORT			nRecordTag, 
	BYTE			nRecordVer	
)

:	SfxMultiVarRecordWriter( SFX_REC_TYPE_MIXTAGS,
				 pStream, nRecordTag, nRecordVer )
{
}

inline void	SfxMultiFixRecordWriter::Reset()
{
	_pStream->Seek( _nStartPos + SFX_REC_HEADERSIZE_MINI +
					 SFX_REC_HEADERSIZE_SINGLE +
					 SFX_REC_HEADERSIZE_MULTI );
	_bHeaderOk = FALSE;
}

inline UINT16 SfxMultiRecordReader::GetContentTag()

{
	return _nContentTag;
}

inline BYTE SfxMultiRecordReader::GetContentVersion() const

{
	return _nContentVer;
}

inline FASTBOOL SfxMultiRecordReader::HasContentVersion( USHORT nVersion ) const

{
	return _nContentVer >= nVersion;
}

inline UINT32 SfxMultiRecordReader::ContentCount() const

{
	return _nContentCount;
}

#endif

