
#ifndef _SVT_FILEVIEW_HXX
#define	_SVT_FILEVIEW_HXX

#ifndef _COM_SUN_STAR_UNO_SEQUENCE_H_
#include <com/sun/star/uno/Sequence.h>
#endif

#include <vcl/ctrl.hxx>
#include <vcl/image.hxx>
#include <vcl/fixed.hxx>
#include <vcl/button.hxx>
#include <vcl/dialog.hxx>

#define FILEVIEW_ONLYFOLDER			0x0001
#define FILEVIEW_MULTISELECTION		0x0002

#define FILEVIEW_SHOW_TITLE			0x0010
#define FILEVIEW_SHOW_SIZE			0x0020
#define FILEVIEW_SHOW_DATE			0x0040
#define FILEVIEW_SHOW_ALL			0x0070

class ViewTabListBox_Impl;
class SvtFileView_Impl;
class SvLBoxEntry;
class HeaderBar;

class SvtFileView : public Control
{
private:
	SvtFileView_Impl*       mpImp;

	void					OpenFolder( const ::com::sun::star::uno::Sequence< ::rtl::OUString >& aContents );

	DECL_LINK( HeaderSelect_Impl, HeaderBar * );

public:
	SvtFileView( Window* pParent, const ResId& rResId, sal_Bool bOnlyFolder, sal_Bool bMultiSelection );
	SvtFileView( Window* pParent, const ResId& rResId, sal_Int8 nFlags );
	~SvtFileView();
	 const String&           GetViewURL() const;
	String					GetURL( SvLBoxEntry* pEntry ) const;
	String					GetCurrentURL() const;

	void					CreateNewFolder( const String& rNewFolder );
	sal_Bool				HasPreviousLevel( String& rParentURL ) const;
	sal_Bool				PreviousLevel( String& rNewURL );

	void					SetHelpId( sal_uInt32 nHelpId );
	void					SetSizePixel( const Size& rNewSize );
	void					SetPosSizePixel( const Point& rNewPos, const Size& rNewSize );
	void					Initialize( const String& rURL, const String& rFilter );
	void					Initialize( const String& rURL,
							const ::com::sun::star::uno::Sequence< ::rtl::OUString >& aContents );

	void					ExecuteFilter( const String& rFilter );
	void					SetNoSelection();
	void					SetFocusInView();
	void					ResetCursor();

	void					SetSelectHdl( const Link& rHdl );
	void					SetDoubleClickHdl( const Link& rHdl );
	void                    SetOpenDoneHdl( const Link& rHdl );

	ULONG           		GetSelectionCount() const;
	SvLBoxEntry*			FirstSelected() const;
	SvLBoxEntry*			NextSelected( SvLBoxEntry* pEntry ) const;
	void					EnableAutoResize();
	void					SetFocus();
	 void                    EnableContextMenu( sal_Bool bEnable );
	void                    EnableDelete( sal_Bool bEnable );
};

struct SvtContentEntry
{
	sal_Bool	mbIsFolder;
	UniString	maURL;

	SvtContentEntry( const UniString& rURL, sal_Bool bIsFolder ) :
		mbIsFolder( bIsFolder ), maURL( rURL ) {}
};

namespace svtools {

enum QueryDeleteResult_Impl
{
	QUERYDELETE_YES = 0,
	QUERYDELETE_NO,
	QUERYDELETE_ALL,
	QUERYDELETE_CANCEL
};

class QueryDeleteDlg_Impl : public ModalDialog
{
	FixedText               _aEntryLabel;
	FixedText               _aEntry;
	FixedText               _aQueryMsg;

	PushButton              _aYesButton;
	PushButton              _aAllButton;
	PushButton              _aNoButton;
	CancelButton            _aCancelButton;
	 QueryDeleteResult_Impl  _eResult;

private:

	DECL_STATIC_LINK( QueryDeleteDlg_Impl, ClickLink, PushButton* );

public:
					QueryDeleteDlg_Impl( Window* pParent,
									 const String& rName );
	 void                    EnableAllButton() { _aAllButton.Enable( sal_True ); }
	QueryDeleteResult_Impl  GetResult() const { return _eResult; }
};

}

#endif 

