

#ifndef _HTMLOUT_HXX
#define _HTMLOUT_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#ifndef _RTL_TEXTENC_H
#include <rtl/textenc.h>
#endif
#ifndef _SFXMACITEM_HXX 
#include <svtools/macitem.hxx>
#endif

class Color;
class ImageMap;
class String;
class SvStream;
class SvxMacroTableDtor;
class SvNumberFormatter;
class SvDataObject;

struct HTMLOutEvent
{
	const sal_Char *pBasicName;
	const sal_Char *pJavaName;
	USHORT nEvent;
};

class HTMLOutFuncs
{
public:
#if defined(MAC) || defined(UNX)
	static const sal_Char sNewLine;		
#else
	static const sal_Char __FAR_DATA sNewLine[];	
#endif

	static void ConvertStringToHTML( const String& sSrc, ByteString& rDest,
			rtl_TextEncoding eDestEnc
#if SUPD > 637
		   	= RTL_TEXTENCODING_MS_1252
#endif
			,
			String *pNonConvertableChars
#if SUPD > 637
		   	= 0
#endif
					   );
#if SUPD < 638
	static void ConvertStringToHTML( const String& sSrc, ByteString& rDest,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252);
#endif

	static SvStream& Out_AsciiTag( SvStream&, const sal_Char* pStr,
					   BOOL bOn = TRUE,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252);
	static SvStream& Out_Char( SvStream&, sal_Unicode cChar,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252,
			String *pNonConvertableChars = 0 );
	static SvStream& Out_String( SvStream&, const String&,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252,
			String *pNonConvertableChars = 0 );
	static SvStream& Out_Hex( SvStream&, ULONG nHex, BYTE nLen,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252 );
	static SvStream& Out_Color( SvStream&, const Color&,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252 );
	static SvStream& Out_ImageMap( SvStream&, const ImageMap&, const String&,
					   const HTMLOutEvent *pEventTable,
					   BOOL bOutStarBasic,
					   const sal_Char *pDelim = 0,
					   const sal_Char *pIndentArea = 0,
					   const sal_Char *pIndentMap = 0,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252,
			String *pNonConvertableChars = 0 );

	static SvStream& OutScript( SvStream& rStrm,
					const String& rSource,
					const String& rLanguage,
					ScriptType eScriptType,
					const String& rSrc,
					const String *pSBLibrary = 0,
					const String *pSBModule = 0,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252,
			String *pNonConvertableChars = 0 );

	static SvStream& Out_Events( SvStream&, const SvxMacroTableDtor&,
					 const HTMLOutEvent*, BOOL bOutStarBasic,
			rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252,
			String *pNonConvertableChars = 0 );

	static ByteString&	CreateTableDataOptionsValNum( ByteString& aStrTD,
	BOOL bValue, double fVal, ULONG nFormat,
	SvNumberFormatter& rFormatter,
	            rtl_TextEncoding eDestEnc = RTL_TEXTENCODING_MS_1252,
	            String *pNonConvertableChars = 0 );

	static BOOL PrivateURLToInternalImg( String& rURL );
};

#endif

