

#ifndef _HTMLTOKN_H
#define _HTMLTOKN_H

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _SOLAR_h
#include <tools/solar.h>
#endif

class String;

sal_Unicode GetHTMLCharName( const String& rName );

int GetHTMLToken( const String& rName );

int GetHTMLOption( const String& rName );

ULONG GetHTMLColor( const String& rName );

const int HTML_TOKEN_START		= 0x100;
const int HTML_TOKEN_ONOFF		= 0x200;
const int HTML_TOKEN_MICROSOFT	= 0x1000;

enum HTML_TOKEN_IDS
{
	HTML_TEXTTOKEN						= HTML_TOKEN_START,
	HTML_SINGLECHAR,
	HTML_NEWPARA,
	HTML_TABCHAR,
	HTML_RAWDATA,
	HTML_LINEFEEDCHAR,

	HTML_AREA, 
	HTML_BASE, 
	HTML_COMMENT,
	HTML_DOCTYPE,
	HTML_EMBED, 
	HTML_FIGUREOVERLAY, 
	HTML_HORZRULE,						
	HTML_HORZTAB, 
	HTML_IMAGE,							
	HTML_INPUT,							
	HTML_ISINDEX, 
	HTML_LINEBREAK,						
	HTML_LINK, 
	HTML_META, 
	HTML_NEXTID, 
	HTML_OF, 
	HTML_OPTION,						
	HTML_PARAM, 
	HTML_RANGE, 
	HTML_SPACER, 
	HTML_WBR, 

	HTML_NONBREAKSPACE,
	HTML_SOFTHYPH,

	HTML_ABBREVIATION_ON = HTML_TOKEN_ONOFF, 
	HTML_ABBREVIATION_OFF, 
	HTML_ABOVE_ON, 
	HTML_ABOVE_OFF, 
	HTML_ACRONYM_ON, 
	HTML_ACRONYM_OFF, 
	HTML_ADDRESS_ON,
	HTML_ADDRESS_OFF,
	HTML_ANCHOR_ON,
	HTML_ANCHOR_OFF,
	HTML_APPLET_ON, 
	HTML_APPLET_OFF, 
	HTML_ARRAY_ON, 
	HTML_ARRAY_OFF, 
	HTML_AUTHOR_ON, 
	HTML_AUTHOR_OFF, 
	HTML_BANNER_ON, 
	HTML_BANNER_OFF, 
	HTML_BAR_ON, 
	HTML_BAR_OFF, 
	HTML_BASEFONT_ON, 
	HTML_BASEFONT_OFF, 
	HTML_BELOW_ON, 
	HTML_BELOW_OFF, 
	HTML_BIGPRINT_ON, 
	HTML_BIGPRINT_OFF, 
	HTML_BLINK_ON, 
	HTML_BLINK_OFF, 
	HTML_BLOCKQUOTE30_ON, 
	HTML_BLOCKQUOTE30_OFF, 
	HTML_BLOCKQUOTE_ON,
	HTML_BLOCKQUOTE_OFF,
	HTML_BODY_ON,
	HTML_BODY_OFF,
	HTML_BOLDTEXT_ON, 
	HTML_BOLDTEXT_OFF, 
	HTML_BOLD_ON,
	HTML_BOLD_OFF,
	HTML_BOX_ON, 
	HTML_BOX_OFF, 
	HTML_CAPTION_ON, 
	HTML_CAPTION_OFF, 
	HTML_CENTER_ON, 
	HTML_CENTER_OFF,  
	HTML_CITIATION_ON,
	HTML_CITIATION_OFF,
	HTML_CODE_ON,
	HTML_CODE_OFF,
	HTML_COL_ON, 
	HTML_COL_OFF, 
	HTML_COLGROUP_ON, 
	HTML_COLGROUP_OFF, 
	HTML_CREDIT_ON, 
	HTML_CREDIT_OFF, 
	HTML_DD_ON,
	HTML_DD_OFF,
	HTML_DEFLIST_ON,
	HTML_DEFLIST_OFF,
	HTML_DELETEDTEXT_ON, 
	HTML_DELETEDTEXT_OFF, 
	HTML_DIRLIST_ON,
	HTML_DIRLIST_OFF,
	HTML_DIVISION_ON, 
	HTML_DIVISION_OFF, 
	HTML_DOT_ON, 
	HTML_DOT_OFF, 
	HTML_DOUBLEDOT_ON, 
	HTML_DOUBLEDOT_OFF, 
	HTML_DT_ON,
	HTML_DT_OFF,
	HTML_EMPHASIS_ON,
	HTML_EMPHASIS_OFF,
	HTML_FIGURE_ON, 
	HTML_FIGURE_OFF, 
	HTML_FONT_ON, 
	HTML_FONT_OFF, 
	HTML_FOOTNOTE_ON, 
	HTML_FOOTNOTE_OFF, 
	HTML_FORM_ON,
	HTML_FORM_OFF,
	HTML_FRAME_ON, 
	HTML_FRAME_OFF, 
	HTML_FRAMESET_ON, 
	HTML_FRAMESET_OFF, 
	HTML_HAT_ON, 
	HTML_HAT_OFF, 
	HTML_HEAD1_ON,
	HTML_HEAD1_OFF,
	HTML_HEAD2_ON,
	HTML_HEAD2_OFF,
	HTML_HEAD3_ON,
	HTML_HEAD3_OFF,
	HTML_HEAD4_ON,
	HTML_HEAD4_OFF,
	HTML_HEAD5_ON,
	HTML_HEAD5_OFF,
	HTML_HEAD6_ON,
	HTML_HEAD6_OFF,
	HTML_HEAD_ON,
	HTML_HEAD_OFF,
	HTML_HTML_ON,
	HTML_HTML_OFF,
	HTML_IFRAME_ON, 
	HTML_IFRAME_OFF, 
	HTML_ILAYER_ON,
	HTML_ILAYER_OFF,
	HTML_INSERTEDTEXT_ON, 
	HTML_INSERTEDTEXT_OFF, 
	HTML_ITALIC_ON,
	HTML_ITALIC_OFF,
	HTML_ITEM_ON, 
	HTML_ITEM_OFF, 
	HTML_KEYBOARD_ON,
	HTML_KEYBOARD_OFF,
	HTML_LAYER_ON,
	HTML_LAYER_OFF,
	HTML_LANGUAGE_ON, 
	HTML_LANGUAGE_OFF, 
	HTML_LISTHEADER_ON, 
	HTML_LISTHEADER_OFF, 
	HTML_LI_ON,
	HTML_LI_OFF,
	HTML_MAP_ON, 
	HTML_MAP_OFF, 
	HTML_MATH_ON, 
	HTML_MATH_OFF, 
	HTML_MENULIST_ON,
	HTML_MENULIST_OFF,
	HTML_MULTICOL_ON, 
	HTML_MULTICOL_OFF, 
	HTML_NOBR_ON, 
	HTML_NOBR_OFF, 
	HTML_NOEMBED_ON, 
	HTML_NOEMBED_OFF, 
	HTML_NOFRAMES_ON, 
	HTML_NOFRAMES_OFF, 
	HTML_NOSCRIPT_ON, 
	HTML_NOSCRIPT_OFF, 
	HTML_NOTE_ON, 
	HTML_NOTE_OFF, 
	HTML_ORDERLIST_ON,
	HTML_ORDERLIST_OFF,
	HTML_PARABREAK_ON,
	HTML_PARABREAK_OFF,
	HTML_PERSON_ON, 
	HTML_PERSON_OFF, 
	HTML_PLAINTEXT_ON, 
	HTML_PLAINTEXT_OFF, 
	HTML_PREFORMTXT_ON,
	HTML_PREFORMTXT_OFF,
	HTML_ROOT_ON, 
	HTML_ROOT_OFF, 
	HTML_ROW_ON, 
	HTML_ROW_OFF, 
	HTML_SAMPLE_ON,
	HTML_SAMPLE_OFF,
	HTML_SCRIPT_ON,	
	HTML_SCRIPT_OFF,	
	HTML_SELECT_ON,
	HTML_SELECT_OFF,
	HTML_SHORTQUOTE_ON, 
	HTML_SHORTQUOTE_OFF, 
	HTML_SMALLPRINT_ON, 
	HTML_SMALLPRINT_OFF, 
	HTML_SPAN_ON, 
	HTML_SPAN_OFF, 
	HTML_SQUAREROOT_ON, 
	HTML_SQUAREROOT_OFF, 
	HTML_STRIKETHROUGH_ON, 
	HTML_STRIKETHROUGH_OFF, 
	HTML_STRONG_ON,
	HTML_STRONG_OFF,
	HTML_STYLE_ON, 
	HTML_STYLE_OFF, 
	HTML_SUBSCRIPT_ON, 
	HTML_SUBSCRIPT_OFF, 
	HTML_SUPERSCRIPT_ON, 
	HTML_SUPERSCRIPT_OFF, 
	HTML_TABLE_ON, 
	HTML_TABLE_OFF, 
	HTML_TABLEDATA_ON, 
	HTML_TABLEDATA_OFF, 
	HTML_TABLEHEADER_ON, 
	HTML_TABLEHEADER_OFF, 
	HTML_TABLEROW_ON, 
	HTML_TABLEROW_OFF, 
	HTML_TBODY_ON, 
	HTML_TBODY_OFF, 
	HTML_TELETYPE_ON,
	HTML_TELETYPE_OFF,
	HTML_TEXTAREA_ON,
	HTML_TEXTAREA_OFF,
	HTML_TEXTFLOW_ON, 
	HTML_TEXTFLOW_OFF, 
	HTML_TEXT_ON, 
	HTML_TEXT_OFF, 
	HTML_TFOOT_ON, 
	HTML_TFOOT_OFF, 
	HTML_THEAD_ON, 
	HTML_THEAD_OFF, 
	HTML_TILDE_ON, 
	HTML_TILDE_OFF, 
	HTML_TITLE_ON,
	HTML_TITLE_OFF,
	HTML_UNDERLINE_ON,
	HTML_UNDERLINE_OFF,
	HTML_UNORDERLIST_ON,
	HTML_UNORDERLIST_OFF,
	HTML_VARIABLE_ON,
	HTML_VARIABLE_OFF,
	HTML_VECTOR_ON, 
	HTML_VECTOR_OFF, 

	HTML_XMP_ON,
	HTML_XMP_OFF,
	HTML_LISTING_ON,
	HTML_LISTING_OFF,

	HTML_DEFINSTANCE_ON,
	HTML_DEFINSTANCE_OFF,
	HTML_STRIKE_ON,
	HTML_STRIKE_OFF,

	HTML_UNKNOWNCONTROL_ON,
	HTML_UNKNOWNCONTROL_OFF,

	HTML_BGSOUND = HTML_TOKEN_MICROSOFT|HTML_TOKEN_START,

	HTML_COMMENT2_ON = HTML_TOKEN_MICROSOFT|HTML_TOKEN_ONOFF, 
	HTML_COMMENT2_OFF, 
	HTML_MARQUEE_ON,
	HTML_MARQUEE_OFF,
	HTML_PLAINTEXT2_ON, 
	HTML_PLAINTEXT2_OFF, 

	HTML_SDFIELD_ON,
	HTML_SDFIELD_OFF
};

const int HTML_OPTION_START	= 0x100;

enum HTML_OPTION_IDS
{
HTML_OPTION_BOOL_START			= HTML_OPTION_START,

	HTML_O_BOX					= HTML_OPTION_BOOL_START,
	HTML_O_CHECKED,
	HTML_O_COMPACT,
	HTML_O_CONTINUE,
	HTML_O_CONTROLS, 
	HTML_O_DECLARE, 
	HTML_O_DISABLED,
	HTML_O_FOLDED, 
	HTML_O_ISMAP,
	HTML_O_MAYSCRIPT, 
	HTML_O_MULTIPLE,
	HTML_O_NOFLOW,
	HTML_O_NOHREF, 
	HTML_O_NORESIZE, 
	HTML_O_NOSHADE, 
	HTML_O_NOWRAP,
	HTML_O_PLAIN,
	HTML_O_SDFIXED,
	HTML_O_SELECTED,
	HTML_O_SHAPES, 
HTML_OPTION_BOOL_END,

HTML_OPTION_STRING_START		= HTML_OPTION_BOOL_END,
	HTML_O_ABOVE				= HTML_OPTION_STRING_START,
	HTML_O_ACCEPT,
	HTML_O_ACCESSKEY,
	HTML_O_ADD_DATE, 
	HTML_O_ALT,
	HTML_O_AXES,
	HTML_O_AXIS,
	HTML_O_BELOW,
	HTML_O_CHAR, 
	HTML_O_CLASS,
	HTML_O_CLIP,
	HTML_O_CODE, 
	HTML_O_CODETYPE,
	HTML_O_COLSPEC,
	HTML_O_CONTENT,
	HTML_O_COORDS, 
	HTML_O_DP,
	HTML_O_ENCTYPE,
	HTML_O_ERROR,
	HTML_O_FACE, 
	HTML_O_FRAMEBORDER, 
	HTML_O_HTTPEQUIV,
	HTML_O_LANGUAGE, 
	HTML_O_LAST_MODIFIED, 
	HTML_O_LAST_VISIT, 
	HTML_O_MD,
	HTML_O_N,
	HTML_O_NAME,
	HTML_O_NOTATION,
	HTML_O_PROMPT,
	HTML_O_SHAPE,
	HTML_O_STANDBY,
	HTML_O_STYLE, 
	HTML_O_TITLE,
	HTML_O_VALUE,
	HTML_O_SDVAL, 
	HTML_O_SDNUM, 
	HTML_O_SDLIBRARY,
	HTML_O_SDMODULE,
HTML_OPTION_STRING_END,

HTML_OPTION_SGMLID_START		= HTML_OPTION_STRING_END,
	HTML_O_FROM					= HTML_OPTION_SGMLID_START,
	HTML_O_ID,
	HTML_O_TARGET, 
	HTML_O_TO,
	HTML_O_UNTIL,
HTML_OPTION_SGMLID_END,

HTML_OPTION_URI_START			= HTML_OPTION_SGMLID_END,
	HTML_O_ACTION				= HTML_OPTION_URI_START,
	HTML_O_BACKGROUND,
	HTML_O_CLASSID,
	HTML_O_CODEBASE, 
	HTML_O_DATA,
	HTML_O_DYNSRC, 
	HTML_O_DYNSYNC, 
	HTML_O_IMAGEMAP,
	HTML_O_HREF,
	HTML_O_LOWSRC, 
	HTML_O_SCRIPT,
	HTML_O_SRC,
	HTML_O_USEMAP, 
HTML_OPTION_URI_END,

HTML_OPTION_ENTITY_START		= HTML_OPTION_URI_END,
	HTML_O_DINGBAT				= HTML_OPTION_ENTITY_START,
	HTML_O_SYM,
HTML_OPTION_ENTITY_END,

HTML_OPTION_COLOR_START			= HTML_OPTION_ENTITY_END,
	HTML_O_ALINK				= HTML_OPTION_COLOR_START,
	HTML_O_BGCOLOR,
	HTML_O_BORDERCOLOR, 
	HTML_O_BORDERCOLORLIGHT, 
	HTML_O_BORDERCOLORDARK, 
	HTML_O_COLOR,
	HTML_O_LINK,
	HTML_O_TEXT,
	HTML_O_VLINK,
HTML_OPTION_COLOR_END,

HTML_OPTION_NUMBER_START		= HTML_OPTION_COLOR_END,
	HTML_O_BORDER				= HTML_OPTION_NUMBER_START,
	HTML_O_CELLSPACING, 
	HTML_O_CELLPADDING, 
	HTML_O_CHAROFF, 
	HTML_O_COLSPAN,
	HTML_O_FRAMESPACING, 
	HTML_O_GUTTER, 
	HTML_O_INDENT,
	HTML_O_HEIGHT,
	HTML_O_HSPACE,			
	HTML_O_LEFT,
	HTML_O_LEFTMARGIN, 
	HTML_O_LOOP, 
	HTML_O_MARGINWIDTH, 
	HTML_O_MARGINHEIGHT, 
	HTML_O_MAX,
	HTML_O_MAXLENGTH,
	HTML_O_MIN,
	HTML_O_PAGEX,
	HTML_O_PAGEY,
	HTML_O_POINTSIZE,
	HTML_O_ROWSPAN,
	HTML_O_SCROLLAMOUNT, 
	HTML_O_SCROLLDELAY, 
	HTML_O_SEQNUM,
	HTML_O_SKIP,
	HTML_O_SPAN, 
	HTML_O_TABINDEX,
	HTML_O_TOP,
	HTML_O_TOPMARGIN, 
	HTML_O_VSPACE,			
	HTML_O_WEIGHT,
	HTML_O_WIDTH,
	HTML_O_X,
	HTML_O_Y,
	HTML_O_ZINDEX,
HTML_OPTION_NUMBER_END,

HTML_OPTION_ENUM_START			= HTML_OPTION_NUMBER_END,
	HTML_O_BEHAVIOR				= HTML_OPTION_ENUM_START, 
	HTML_O_BGPROPERTIES, 
	HTML_O_CLEAR,
	HTML_O_DIR,
	HTML_O_DIRECTION, 
	HTML_O_FORMAT,
	HTML_O_FRAME, 
	HTML_O_LANG,
	HTML_O_METHOD,
	HTML_O_PALETTE,
	HTML_O_REL,
	HTML_O_REV,
	HTML_O_RULES, 
	HTML_O_SCROLLING, 
	HTML_O_SDREADONLY,
	HTML_O_SUBTYPE,
	HTML_O_TYPE,
	HTML_O_VALIGN,
	HTML_O_VALUETYPE,
	HTML_O_VISIBILITY,
	HTML_O_WRAP,
HTML_OPTION_ENUM_END,

HTML_OPTION_SCRIPT_START		= HTML_OPTION_ENUM_END,
	HTML_O_ONABORT				= HTML_OPTION_SCRIPT_START, 
	HTML_O_ONBLUR,		
	HTML_O_ONCHANGE,	
	HTML_O_ONCLICK,		
	HTML_O_ONERROR,		
	HTML_O_ONFOCUS,		
	HTML_O_ONLOAD,		
	HTML_O_ONMOUSEOUT,	
	HTML_O_ONMOUSEOVER,	
	HTML_O_ONRESET,		
	HTML_O_ONSELECT,	
	HTML_O_ONSUBMIT,	
	HTML_O_ONUNLOAD,	

	HTML_O_SDONABORT, 		
	HTML_O_SDONBLUR,		
	HTML_O_SDONCHANGE, 		
	HTML_O_SDONCLICK, 		
	HTML_O_SDONERROR, 		
	HTML_O_SDONFOCUS, 		
	HTML_O_SDONLOAD, 		
	HTML_O_SDONMOUSEOUT,	
	HTML_O_SDONMOUSEOVER, 	
	HTML_O_SDONRESET,		
	HTML_O_SDONSELECT, 		
	HTML_O_SDONSUBMIT, 		
	HTML_O_SDONUNLOAD, 		
HTML_OPTION_SCRIPT_END,

HTML_OPTION_CONTEXT_START		= HTML_OPTION_SCRIPT_END,
	HTML_O_ALIGN				= HTML_OPTION_CONTEXT_START,
	HTML_O_COLS, 
	HTML_O_ROWS, 
	HTML_O_SIZE,
	HTML_O_START,
	HTML_O_UNITS,
HTML_OPTION_CONTEXT_END,

HTML_O_UNKNOWN 					= HTML_OPTION_CONTEXT_END,
HTML_OPTION_END
};

#endif

