

#ifndef _IMAP_HXX
#define _IMAP_HXX

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _STREAM_HXX 
#include <tools/stream.hxx>
#endif

class Point;
class Rectangle;
class Size;
class Fraction;
class IMapObject;

class ImageMap
{
	List				maList;
	String				aName;

protected:

	void				ImpWriteImageMap( SvStream& rOStm ) const ;
	void				ImpReadImageMap( SvStream& rIStm, USHORT nCount );

	void				ImpWriteCERN( SvStream& rOStm ) const;
	void				ImpWriteNCSA( SvStream& rOStm ) const;
	ULONG				ImpReadCERN( SvStream& rOStm );
	ULONG				ImpReadNCSA( SvStream& rOStm );

	void				ImpReadCERNLine( const ByteString& rLine );
	Point				ImpReadCERNCoords( const char** ppStr );
	long				ImpReadCERNRadius( const char** ppStr );
	String				ImpReadCERNURL( const char** ppStr );

	void				ImpReadNCSALine( const ByteString& rLine );
	String				ImpReadNCSAURL( const char** ppStr );
	Point				ImpReadNCSACoords( const char** ppStr );

	ULONG				ImpDetectFormat( SvStream& rIStm );

public:

	TYPEINFO();
				ImageMap() {};
			ImageMap( const String& rName );
			ImageMap( const ImageMap& rImageMap );
				~ImageMap();

	ImageMap&			operator=( const ImageMap& rImageMap );

	BOOL				operator==( const ImageMap& rImageMap );
	BOOL				operator!=( const ImageMap& rImageMap );

	void				InsertIMapObject( const IMapObject& rIMapObject );

	IMapObject*			GetFirstIMapObject() { return (IMapObject*) maList.First(); }
	IMapObject*			GetNextIMapObject() { return (IMapObject*) maList.Next(); }
	IMapObject*			GetLastIMapObject() { return (IMapObject*) maList.Last(); }
	IMapObject*			GetPrevIMapObject() { return (IMapObject*) maList.Prev(); }
	IMapObject*			GetIMapObject( USHORT nPos ) const { return (IMapObject*) maList.GetObject( nPos ); }

	IMapObject*			GetHitIMapObject( const Size& rOriginalSize,
							  const Size& rDisplaySize,
							  const Point& rRelHitPoint,
							  ULONG nFlags = 0 );

	UINT16				GetIMapObjectCount() const { return (UINT16) maList.Count(); }

	void				ClearImageMap();

	UINT16				GetVersion() const;

	const String&		GetName() const { return aName; }
	void				SetName( const String& rName ) { aName = rName; }

	Rectangle			GetBoundRect() const;

	void				Scale( const Fraction& rFractX, const Fraction& rFracY );

	friend SvStream&	operator<<( SvStream& rOStm, const ImageMap& rImageMap );
	friend SvStream&	operator>>( SvStream& rIStm, ImageMap& rImageMap );

	void				Write( SvStream& rOStm, ULONG nFormat ) const;
	ULONG				Read( SvStream& rIStm, ULONG nFormat );
};

#if 0 // _SOLAR__PRIVATE

class IMapCompat
{
	SvStream*		pRWStm;
	ULONG			nCompatPos;
	ULONG			nTotalSize;
	USHORT			nStmMode;
			IMapCompat() {}
		IMapCompat( const IMapCompat& rCompat ) {}
	IMapCompat&		operator=( const IMapCompat& rCompat ) { return *this; }
	BOOL			operator==( const IMapCompat& rCompat ) { return FALSE; }

public:
			IMapCompat( SvStream& rStm, const USHORT nStreamMode );
		~IMapCompat();
};

#endif 
#endif 
