
#ifndef _SFXITEMSET_HXX
#define _SFXITEMSET_HXX

#if STLPORT_VERSION<321
#include <stdarg.h>
#else
#include <cstdarg> 	
#endif

#ifndef _SFXPOOLITEM_HXX 
#include <svtools/poolitem.hxx>
#endif
#ifndef _RTTI_HXX 
#include <tools/rtti.hxx>
#endif
#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

class SfxItemPool;
class SfxPoolItem;
class SvStream;

typedef SfxPoolItem const** SfxItemArray;

#if SUPD<=383

#if defined MAC || ( defined SOLARIS && defined X86 ) || ( defined SINIX && defined GCC ) || defined SCO
#define USHORT_ARG	int
#else
#define USHORT_ARG	USHORT
#endif

#else

#define USHORT_ARG int

#endif

#ifdef MI_HATS_REPARIERT
#ifndef DBG
#ifdef DBG_UTILx
#define DBG(s) s
#define _pChildCount(THIS)  (  *(USHORT**)SfxPointerServer::GetServer()->GetPointer(THIS) )
#define _pChildCountCtor	( (*(USHORT**)SfxPointerServer::GetServer()->CreatePointer(this)) = new USHORT )
#define _pChildCountDtor	( SfxPointerServer::GetServer()->ReleasePointer(this) )
#else
#define DBG(s)
#endif
#endif
#else
#ifdef DBG
#undef DBG
#endif
#define DBG(s)
#endif

#define SFX_ITEMSET_GET( rSet, pItem, ItemType, nSlotId, bDeep ) \
	const ItemType *pItem = (const ItemType*) \
				(rSet).GetItem( nSlotId, bDeep, TYPE(ItemType) )

class SfxItemSet
{
friend class SfxItemIter;

	SfxItemPool*				_pPool; 		
	const SfxItemSet*           _pParent;       
	SfxItemArray                _aItems;        
	USHORT*                     _pWhichRanges;  
	USHORT                      _nCount;        

#ifndef _SFXITEMS_HXX

friend class SfxItemPoolCache;
friend class SfxAllItemSet;
friend const char *DbgCheckItemSet( const void* );

private:
	void						InitRanges_Impl(const USHORT *nWhichPairTable);
	void						InitRanges_Impl(va_list pWhich, USHORT n1, USHORT n2, USHORT n3);
	void						InitRanges_Impl(USHORT nWh1, USHORT nWh2);

public:
	SfxItemArray				GetItems_Impl() const { return _aItems; }

#endif

private:
	const SfxItemSet&			operator=(const SfxItemSet &);	 

protected:

	virtual void				Changed( const SfxPoolItem& rOld, const SfxPoolItem& rNew );

	int 						PutDirect(const SfxPoolItem &rItem);

public:
					SfxItemSet( const SfxItemSet& );
#if SUPD<367
					SfxItemSet( SfxItemPool& );
#endif

#if SUPD<367 || defined(SFX_ITEMSET_NO_DEFAULT_CTOR)
					SfxItemSet( SfxItemPool&, BOOL bTotalPoolRanges );
#else
					SfxItemSet( SfxItemPool&, BOOL bTotalPoolRanges = FALSE );
#endif
					SfxItemSet( SfxItemPool&, USHORT nWhich1, USHORT nWhich2 );
					SfxItemSet( SfxItemPool&, USHORT_ARG nWh1, USHORT_ARG nWh2, USHORT_ARG nNull, ... );
					SfxItemSet( SfxItemPool&, const USHORT* nWhichPairTable );
	virtual                     ~SfxItemSet();

	virtual SfxItemSet *		Clone(BOOL bItems = TRUE, SfxItemPool *pToPool = 0) const;

	USHORT						Count() const { return _nCount; }
	USHORT						TotalCount() const;
#if SUPD >= 565
	virtual	const SfxPoolItem&	Get( USHORT nWhich, BOOL bSrchInParent = TRUE ) const;
#else
	const SfxPoolItem&			Get( USHORT nWhich, BOOL bSrchInParent = TRUE ) const;
#endif
	const SfxPoolItem*  		GetItem( USHORT nWhich, BOOL bSrchInParent = TRUE,
							 TypeId aItemType = 0 ) const;

	USHORT						GetWhichByPos(USHORT nPos) const;

	SfxItemState				GetItemState(	USHORT nWhich,
									BOOL bSrchInParent = TRUE,
									const SfxPoolItem **ppItem = 0 ) const;

	virtual void				DisableItem(USHORT nWhich);
	virtual void				InvalidateItem( USHORT nWhich );
	virtual USHORT              ClearItem( USHORT nWhich = 0);
	virtual void				ClearInvalidItems( BOOL bHardDefault = FALSE );
			void 				InvalidateAllItems(); HACK(via nWhich = 0)

	inline void 				SetParent( const SfxItemSet* pNew );

	virtual const SfxPoolItem*	Put( const SfxPoolItem&, USHORT nWhich );
	const SfxPoolItem*			Put( const SfxPoolItem& rItem )
					{ return Put(rItem, rItem.Which()); }
	virtual int 				Put( const SfxItemSet&,
						 BOOL bInvalidAsDefault = TRUE );
	void						PutExtended( const SfxItemSet&,
								 SfxItemState eDontCareAs = SFX_ITEM_UNKNOWN,
								 SfxItemState eDefaultAs = SFX_ITEM_UNKNOWN );

	virtual int 				Set( const SfxItemSet&, BOOL bDeep = TRUE );

	virtual void				Intersect( const SfxItemSet& rSet );
	virtual void				MergeValues( const SfxItemSet& rSet, BOOL bOverwriteDefaults = FALSE );
	virtual void				Differentiate( const SfxItemSet& rSet );
	virtual void				MergeValue( const SfxPoolItem& rItem, BOOL bOverwriteDefaults = FALSE  );

	SfxItemPool*				GetPool() const { return _pPool; }
	const USHORT*				GetRanges() const { return _pWhichRanges; }
	void						SetRanges( const USHORT *pRanges );
	void						MergeRange( USHORT nFrom, USHORT nTo );
	const SfxItemSet*			GetParent() const { return _pParent; }

	virtual SvStream &			Load( SvStream &, FASTBOOL bDirect = FALSE,
						  const SfxItemPool *pRefPool = 0 );
	virtual SvStream &			Store( SvStream &, FASTBOOL bDirect = FALSE ) const;

	virtual int                 operator==(const SfxItemSet &) const;
};

inline void SfxItemSet::SetParent( const SfxItemSet* pNew )
{
	DBG( if (_pParent) --*_pChildCount(_pParent) );
	_pParent = pNew;
	DBG( if (_pParent) ++*_pChildCount(_pParent) );
}

class SfxAllItemSet: public SfxItemSet

{
	SfxVoidItem 				aDefault;
	USHORT 						nFree;

public:
					SfxAllItemSet( SfxItemPool &rPool );
					SfxAllItemSet( const SfxItemSet & );
					SfxAllItemSet( const SfxAllItemSet & );

	virtual SfxItemSet *		Clone( BOOL bItems = TRUE, SfxItemPool *pToPool = 0 ) const;
	virtual const SfxPoolItem*	Put( const SfxPoolItem&, USHORT nWhich );
	const SfxPoolItem*	        Put( const SfxPoolItem& rItem )
	{ return Put(rItem, rItem.Which()); }
	virtual int 				Put( const SfxItemSet&,
						 BOOL bInvalidAsDefault = TRUE );

};

#endif 

