

#ifndef _ICNVW_HXX
#define _ICNVW_HXX

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _SV_CTRL_HXX
#include <vcl/ctrl.hxx>
#endif
#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif
#ifndef _SV_IMAGE_HXX
#include <vcl/image.hxx>
#endif
#ifndef _SV_SELENG_HXX
#include <vcl/seleng.hxx>
#endif

#pragma hdrstop

class SvPtrarr;
class ResId;
class Point;
class SvxIconChoiceCtrl_Impl;
class Image;

#define ICNVIEW_FLAG_POS_LOCKED		0x0001
#define ICNVIEW_FLAG_SELECTED		0x0002
#define ICNVIEW_FLAG_FOCUSED		0x0004
#define ICNVIEW_FLAG_IN_USE			0x0008
#define ICNVIEW_FLAG_CURSORED	   	0x0010 
#define ICNVIEW_FLAG_POS_MOVED   	0x0020 
#define ICNVIEW_FLAG_DROP_TARGET	0x0040 
#define ICNVIEW_FLAG_BLOCK_EMPHASIS	0x0080 
#define ICNVIEW_FLAG_USER1			0x0100
#define ICNVIEW_FLAG_USER2			0x0200
#define ICNVIEW_FLAG_PRED_SET		0x0400 

enum SvxIconChoiceCtrlTextMode
{
	IcnShowTextFull = 1,		
	IcnShowTextShort,      		
	IcnShowTextSmart,			
	IcnShowTextDontKnow			
};

enum SvxIconChoiceCtrlPositionMode
{
	IcnViewPositionModeFree = 0,			    
	IcnViewPositionModeAutoArrange = 1,         
	IcnViewPositionModeAutoAdjust = 2,   		
	IcnViewPositionModeLast = IcnViewPositionModeAutoAdjust
};

class SvxIconChoiceCtrlEntry
{
	Image aImage;
	String aText;
	void*  pUserData;

	friend class SvxIconChoiceCtrl_Impl;
	friend class IcnCursor_Impl;
	friend class EntryList_Impl;
	friend class IcnGridMap_Impl;

	Rectangle 				aRect;				
	Rectangle 				aGridRect;			
	ULONG					nPos;

	SvxIconChoiceCtrlEntry*			pblink;		
	SvxIconChoiceCtrlEntry*			pflink;     

	SvxIconChoiceCtrlTextMode 		eTextMode;
	USHORT 					nX,nY;		
	USHORT					nFlags;

	void					ClearFlags( USHORT nMask ) { nFlags &= (~nMask); }
	void					SetFlags( USHORT nMask ) { nFlags |= nMask; }
	void					AssignFlags( USHORT _nFlags ) { nFlags = _nFlags; }

	void					SetBacklink( SvxIconChoiceCtrlEntry* pA )
				{
					pA->pflink->pblink = this;  	
					this->pflink = pA->pflink; 		
					this->pblink = pA;  			
					pA->pflink = this;  			
				}

	void					Unlink()
				{
					this->pblink->pflink = this->pflink;
					this->pflink->pblink = this->pblink;
					this->pflink = 0;
					this->pblink = 0;
				}

public:
				SvxIconChoiceCtrlEntry( USHORT nFlags = 0 );
				SvxIconChoiceCtrlEntry( const String& rText, const Image& rImage, USHORT nFlags = 0 );
				~SvxIconChoiceCtrlEntry () {}

	void					SetImage ( const Image& rImage ) { aImage = rImage; }
	Image					GetImage () const { return aImage; }
	void					SetText ( const String& rText ) { aText = rText; }
	String					GetText () const { return aText; }
	void					SetUserData ( void* _pUserData ) { pUserData = _pUserData; }
	void*					GetUserData () { return pUserData; }
	 const Rectangle &       GetBoundRect() const { return aRect; }

	void					SetFocus ( BOOL bSet )
	                              { nFlags = ( bSet ? nFlags | ICNVIEW_FLAG_FOCUSED : nFlags & ~ICNVIEW_FLAG_FOCUSED ); }

	SvxIconChoiceCtrlTextMode 		GetTextMode() const { return eTextMode; }
	USHORT					GetFlags() const { return nFlags; }
	BOOL					IsSelected() const { return (BOOL)((nFlags & ICNVIEW_FLAG_SELECTED) !=0); }
	BOOL					IsFocused() const { return (BOOL)((nFlags & ICNVIEW_FLAG_FOCUSED) !=0); }
	BOOL					IsInUse() const { return (BOOL)((nFlags & ICNVIEW_FLAG_IN_USE) !=0); }
	BOOL					IsCursored() const { return (BOOL)((nFlags & ICNVIEW_FLAG_CURSORED) !=0); }
	BOOL					IsDropTarget() const { return (BOOL)((nFlags & ICNVIEW_FLAG_DROP_TARGET) !=0); }
	BOOL					IsBlockingEmphasis() const { return (BOOL)((nFlags & ICNVIEW_FLAG_BLOCK_EMPHASIS) !=0); }
	BOOL					WasMoved() const { return (BOOL)((nFlags & ICNVIEW_FLAG_POS_MOVED) !=0); }
	void					SetMoved( BOOL bMoved );
	BOOL					IsPosLocked() const { return (BOOL)((nFlags & ICNVIEW_FLAG_POS_LOCKED) !=0); }
	void					LockPos( BOOL bLock );

	SvxIconChoiceCtrlEntry*			GetSuccessor() const { return pflink; }
	SvxIconChoiceCtrlEntry*			GetPredecessor() const { return pblink; }
};

enum SvxIconChoiceCtrlColumnAlign
{
	IcnViewAlignLeft = 1,
	IcnViewAlignRight,
	IcnViewAlignCenter
};

class SvxIconChoiceCtrlColumnInfo
{
	String					aColText;
	Image					aColImage;
	long					nWidth;
	SvxIconChoiceCtrlColumnAlign	eAlignment;
	USHORT 					nSubItem;

public:
				SvxIconChoiceCtrlColumnInfo( USHORT nSub, long nWd,
					SvxIconChoiceCtrlColumnAlign eAlign ) : nSubItem( nSub ),
					nWidth( nWd ), eAlignment( eAlign ) {}
				SvxIconChoiceCtrlColumnInfo( const SvxIconChoiceCtrlColumnInfo& );

	void					SetText( const String& rText ) { aColText = rText; }
	void					SetImage( const Image& rImg ) { aColImage = rImg; }
	void					SetWidth( long nWd ) { nWidth = nWd; }
	void					SetAlignment( SvxIconChoiceCtrlColumnAlign eAlign ) { eAlignment = eAlign; }
	void					SetSubItem( USHORT nSub) { nSubItem = nSub; }

	const String& 			GetText() const { return aColText; }
	const Image&			GetImage() const { return aColImage; }
	long					GetWidth() const { return nWidth; }
	SvxIconChoiceCtrlColumnAlign	GetAlignment() const { return eAlignment; }
	USHORT					GetSubItem() const { return nSubItem; }
};

#ifdef MAC
#undef WB_ICON
#undef WB_SMALLICON
#undef WB_DETAILS
#endif
#define WB_ICON					WB_RECTSTYLE
#define WB_SMALLICON       		WB_SMALLSTYLE
#define WB_DETAILS				WB_VCENTER
#define WB_NOHSCROLL			WB_SPIN
#define WB_NOVSCROLL			WB_DRAG
#define WB_NOSELECTION			WB_REPEAT
#define WB_NODRAGSELECTION		WB_PATHELLIPSIS
#define WB_SMART_ARRANGE		WB_PASSWORD
#define WB_ALIGN_TOP			WB_TOP
#define WB_ALIGN_LEFT			WB_LEFT
#define WB_NOCOLUMNHEADER		WB_CENTER
#define WB_HIGHLIGHTFRAME		WB_INFO
#define WB_NOASYNCSELECTHDL		WB_NOLABEL

class SvtIconChoiceCtrl : public Control
{
	friend class SvxIconChoiceCtrl_Impl;

	Link				_aClickIconHdl;
	Link				_aDocRectChangedHdl;
	Link				_aVisRectChangedHdl;
	KeyEvent*			_pCurKeyEvent;
	SvxIconChoiceCtrl_Impl*		_pImp;

	BOOL				bAutoFontColor;

protected:

	virtual void    	KeyInput( const KeyEvent& rKEvt );
	virtual BOOL		EditedEntry( SvxIconChoiceCtrlEntry*, const XubString& rNewText, BOOL bCancelled );
	virtual void		DocumentRectChanged();
	virtual void		VisibleRectChanged();
	virtual BOOL		EditingEntry( SvxIconChoiceCtrlEntry* pEntry );
	virtual void    	Command( const CommandEvent& rCEvt );
	virtual void    	Paint( const Rectangle& rRect );
	virtual void    	MouseButtonDown( const MouseEvent& rMEvt );
	virtual void    	MouseButtonUp( const MouseEvent& rMEvt );
	virtual void    	MouseMove( const MouseEvent& rMEvt );
	virtual void    	Resize();
	virtual void    	GetFocus();
	virtual void    	LoseFocus();
	virtual void		ClickIcon();
	virtual void 		StateChanged( StateChangedType nType );
	virtual void 		DataChanged( const DataChangedEvent& rDCEvt );
	virtual void		RequestHelp( const HelpEvent& rHEvt );
	virtual void		DrawEntryImage(
				SvxIconChoiceCtrlEntry* pEntry,
				const Point& rPos,
				OutputDevice& rDev );

	virtual String		GetEntryText(
				SvxIconChoiceCtrlEntry* pEntry,
				BOOL bInplaceEdit );

public:
				SvtIconChoiceCtrl( Window* pParent, WinBits nWinStyle = WB_ICON | WB_BORDER );
			SvtIconChoiceCtrl( Window* pParent, const ResId& rResId );
	virtual 			~SvtIconChoiceCtrl();

	void				SetStyle( WinBits nWinStyle );
	WinBits				GetStyle() const;

	BOOL				SetChoiceWithCursor ( BOOL bDo = TRUE );

	void 				SetUpdateMode( BOOL bUpdateMode );
	void				SetFont( const Font& rFont );
	void				SetPointFont( const Font& rFont );

	void            	SetClickHdl( const Link& rLink ) { _aClickIconHdl = rLink; }
	const Link&     	GetClickHdl() const { return _aClickIconHdl; }

	void				SetBackground( const Wallpaper& );

	void				ArrangeIcons();

	SvxIconChoiceCtrlEntry*	InsertEntry( ULONG nPos = LIST_APPEND,
						 const Point* pPos = 0,
						 USHORT nFlags = 0 );
	SvxIconChoiceCtrlEntry*	InsertEntry( const String& rText, const Image& rImage,
						 ULONG nPos = LIST_APPEND,
						 const Point* pPos = 0,
						 USHORT nFlags = 0 );
	void				RemoveEntry( SvxIconChoiceCtrlEntry* );

	BOOL				DoKeyInput( const KeyEvent& rKEvt );

	BOOL				IsEntryEditing() const;
	void				Clear();

	ULONG				GetEntryCount() const;
	SvxIconChoiceCtrlEntry*		GetEntry( ULONG nPos ) const;
	ULONG				GetEntryListPos( SvxIconChoiceCtrlEntry* ) const;
	void				SetCursor( SvxIconChoiceCtrlEntry* pEntry );

	void				InvalidateEntry( SvxIconChoiceCtrlEntry* );

	SvxIconChoiceCtrlEntry*	GetEntry( const Point& rPosPixel, BOOL bHit = FALSE ) const;

	SvxIconChoiceCtrlEntry*	GetNextEntry( const Point& rPosPixel, SvxIconChoiceCtrlEntry* pCurEntry, BOOL  ) const;

	SvxIconChoiceCtrlEntry*	GetPrevEntry( const Point& rPosPixel, SvxIconChoiceCtrlEntry* pCurEntry, BOOL  ) const;

	SvxIconChoiceCtrlEntry*	GetSelectedEntry( ULONG& ) const;

	void				SetEntryTextMode( SvxIconChoiceCtrlTextMode, SvxIconChoiceCtrlEntry* pEntry = 0 );
	SvxIconChoiceCtrlTextMode	GetEntryTextMode( const SvxIconChoiceCtrlEntry* pEntry = 0 ) const;

	void				Flush();

	virtual BOOL		HasBackground() const;
	virtual BOOL		HasFont() const;
	virtual BOOL		HasFontTextColor() const;
	virtual BOOL		HasFontFillColor() const;

	void				SetFontColorToBackground ( BOOL bDo = TRUE ) { bAutoFontColor = bDo; }
	BOOL				AutoFontColor () { return bAutoFontColor; }

	Point				GetLogicPos( const Point& rPosPixel ) const;
	Point		   		GetPixelPos( const Point& rPosLogic ) const;
};

#endif 

