

#ifndef INCLUDED_SVTOOLS_ONDEMAND_HXX
#define INCLUDED_SVTOOLS_ONDEMAND_HXX

#ifndef INCLUDED_SVTOOLS_SYSLOCALE_HXX
#include <svtools/syslocale.hxx>
#endif

#ifndef _LANG_HXX
#include <tools/lang.hxx>
#endif

#ifndef _UNOTOOLS_LOCALEDATAWRAPPER_HXX
#include <unotools/localedatawrapper.hxx>
#endif
#ifndef _UNOTOOLS_CALENDARWRAPPER_HXX
#include <unotools/calendarwrapper.hxx>
#endif
#ifndef _UNOTOOLS_COLLATORWRAPPER_HXX
#include <unotools/collatorwrapper.hxx>
#endif
#ifndef _COM_SUN_STAR_I18N_COLLATOROPTIONS_HPP_
#include <com/sun/star/i18n/CollatorOptions.hpp>
#endif
#ifndef _UNOTOOLS_TRANSLITERATIONWRAPPER_HXX
#include <unotools/transliterationwrapper.hxx>
#endif
#ifndef _COM_SUN_STAR_I18N_TRANSLITERATIONMODULES_HPP_
#include <com/sun/star/i18n/TransliterationModules.hpp>
#endif
#ifndef _COM_SUN_STAR_UNO_REFERENCE_HXX_
#include <com/sun/star/uno/Reference.hxx>
#endif
#ifndef _COM_SUN_STAR_LANG_XMULTISERVICEFACTORY_HPP_
#include <com/sun/star/lang/XMultiServiceFactory.hpp>
#endif

class OnDemandLocaleDataWrapper
{
	    ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > xSMgr;
	        SvtSysLocale        aSysLocale;
	        LanguageType        eCurrentLanguage;
	        LanguageType        eLastAnyLanguage;
	const   LocaleDataWrapper*  pSystem;
	const   LocaleDataWrapper*  pEnglish;
	        LocaleDataWrapper*  pAny;
	const   LocaleDataWrapper*  pCurrent;

public:
	                            OnDemandLocaleDataWrapper()
	                                : pEnglish(0)
	                                , pAny(0)
	                                , eLastAnyLanguage( LANGUAGE_DONTKNOW )
	                                {
	                                    pCurrent = pSystem = aSysLocale.GetLocaleDataPtr();
	                                    eCurrentLanguage = LANGUAGE_SYSTEM;
	                                }
	                            OnDemandLocaleDataWrapper(
	                                const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& rxSMgr,
	                                ::com::sun::star::lang::Locale& rLocale,
	                                LanguageType eLang
	                                )
	                                : pEnglish(0)
	                                , pAny(0)
	                                , pCurrent(0)
	                                {
	                                    pSystem = aSysLocale.GetLocaleDataPtr();
	                                    init( rxSMgr, rLocale, eLang );
	                                }
	                            ~OnDemandLocaleDataWrapper()
	                                {
	                                    delete pEnglish;
	                                    delete pAny;
	                                }
	         sal_Bool            is() const      { return pCurrent != NULL; }
	         void                init(
	                                const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& rxSMgr,
	                                ::com::sun::star::lang::Locale& rLocale,
	                                LanguageType eLang
	                                )
	                                {
	                                    xSMgr = rxSMgr;
	                                    changeLocale( rLocale, eLang );
	                                }
	         void                changeLocale( ::com::sun::star::lang::Locale& rLocale, LanguageType eLang )
	                                {
	                                    switch ( eLang )
	                                    {
	                                        case LANGUAGE_SYSTEM :
	                                            pCurrent = pSystem;
	                                        break;
	                                        case LANGUAGE_ENGLISH_US :
	                                            if ( !pEnglish )
	                                                pEnglish = new LocaleDataWrapper( xSMgr, rLocale );
	                                            pCurrent = pEnglish;
	                                        break;
	                                        default:
	                                            if ( !pAny )
	                                            {
	                                                pAny = new LocaleDataWrapper( xSMgr, rLocale );
	                                                eLastAnyLanguage = eLang;
	                                            }
	                                            else if ( eLastAnyLanguage != eLang )
	                                            {
	                                                pAny->setLocale( rLocale );
	                                                eLastAnyLanguage = eLang;
	                                            }
	                                            pCurrent = pAny;
	                                    }
	                                    eCurrentLanguage = eLang;
	                                }
	         LanguageType        getCurrentLanguage() const
	                                { return eCurrentLanguage; }
	         LocaleDataWrapper*  getAnyLocale()
	                                {
	                                    if ( !pAny )
	                                    {
	                                        pAny = new LocaleDataWrapper( xSMgr, pCurrent->getLocale() );
	                                        eLastAnyLanguage = eCurrentLanguage;
	                                    }
	                                    else if ( pCurrent != pAny )
	                                    {
	                                        pAny->setLocale( pCurrent->getLocale() );
	                                        eLastAnyLanguage = eCurrentLanguage;
	                                    }
	                                    return pAny;
	                                }
	 const   LocaleDataWrapper*  get() const         { return pCurrent; }
	const   LocaleDataWrapper*  operator->() const  { return get(); }
	const   LocaleDataWrapper&  operator*() const   { return *get(); }
};

class OnDemandCalendarWrapper
{
	    ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > xSMgr;
	        ::com::sun::star::lang::Locale  aLocale;
	mutable CalendarWrapper*    pPtr;
	mutable sal_Bool            bValid;

public:
	                            OnDemandCalendarWrapper()
	                                : pPtr(0)
	                                , bValid( sal_False )
	                                {}
	                            OnDemandCalendarWrapper(
	                                const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& rxSMgr,
	                                ::com::sun::star::lang::Locale& rLocale
	                                )
	                                : bValid( sal_False )
	                                {
	                                    init( rxSMgr, rLocale );
	                                }
	                            ~OnDemandCalendarWrapper()
	                                {
	                                    delete pPtr;
	                                }
	         sal_Bool            is() const      { return pPtr != NULL; }
	         void                init(
	                                const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& rxSMgr,
	                                ::com::sun::star::lang::Locale& rLocale
	                                )
	                                {
	                                    xSMgr = rxSMgr;
	                                    changeLocale( rLocale );
	                                    if ( pPtr )
	                                    {
	                                        delete pPtr;
	                                        pPtr = NULL;
	                                    }
	                                }
	         void                changeLocale( ::com::sun::star::lang::Locale& rLocale )
	                                {
	                                    bValid = sal_False;
	                                    aLocale = rLocale;
	                                }
	         CalendarWrapper*    get() const
	                                {
	                                    if ( !bValid )
	                                    {
	                                        if ( !pPtr )
	                                            pPtr = new CalendarWrapper( xSMgr );
	                                        pPtr->loadDefaultCalendar( aLocale );
	                                        bValid = sal_True;
	                                    }
	                                    return pPtr;
	                                }
	         CalendarWrapper*    operator->()    { return get(); }
	        CalendarWrapper&    operator*()     { return *get(); }
};

class OnDemandCollatorWrapper
{
	    ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > xSMgr;
	        ::com::sun::star::lang::Locale  aLocale;
	mutable CollatorWrapper*    pPtr;
	mutable sal_Bool            bValid;

public:
	                            OnDemandCollatorWrapper()
	                                : pPtr(0)
	                                , bValid( sal_False )
	                                {}
	                            OnDemandCollatorWrapper(
	                                const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& rxSMgr,
	                                ::com::sun::star::lang::Locale& rLocale
	                                )
	                                : bValid( sal_False )
	                                {
	                                    init( rxSMgr, rLocale );
	                                }
	                            ~OnDemandCollatorWrapper()
	                                {
	                                    delete pPtr;
	                                }
	         sal_Bool            is() const      { return pPtr != NULL; }
	         void                init(
	                                const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& rxSMgr,
	                                ::com::sun::star::lang::Locale& rLocale
	                                )
	                                {
	                                    xSMgr = rxSMgr;
	                                    changeLocale( rLocale );
	                                    if ( pPtr )
	                                    {
	                                        delete pPtr;
	                                        pPtr = NULL;
	                                    }
	                                }
	         void                changeLocale( ::com::sun::star::lang::Locale& rLocale )
	                                {
	                                    bValid = sal_False;
	                                    aLocale = rLocale;
	                                }
	 const   CollatorWrapper*    get() const
	                                {
	                                    if ( !bValid )
	                                    {
	                                        if ( !pPtr )
	                                            pPtr = new CollatorWrapper( xSMgr );
	                                        pPtr->loadDefaultCollator( aLocale, ::com::sun::star::i18n::CollatorOptions::CollatorOptions_IGNORE_CASE );
	                                        bValid = sal_True;
	                                    }
	                                    return pPtr;
	                                }
	 const   CollatorWrapper*    operator->() const  { return get(); }
	const   CollatorWrapper&    operator*() const   { return *get(); }
};

class OnDemandTransliterationWrapper
{
	    ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory > xSMgr;
	        LanguageType        eLanguage;
	        ::com::sun::star::i18n::TransliterationModules  nType;
	mutable ::utl::TransliterationWrapper*  pPtr;
	mutable sal_Bool            bValid;

public:
	                            OnDemandTransliterationWrapper()
	                                : pPtr(0)
	                                , bValid( sal_False )
	                                , eLanguage( LANGUAGE_SYSTEM )
	                                {}
	                            OnDemandTransliterationWrapper(
	                                const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& rxSMgr,
	                                LanguageType eLang,
	                                ::com::sun::star::i18n::TransliterationModules nTypeP
	                                )
	                                : bValid( sal_False )
	                                {
	                                    init( rxSMgr, eLang, nTypeP );
	                                }
	                            ~OnDemandTransliterationWrapper()
	                                {
	                                    delete pPtr;
	                                }
	         sal_Bool            is() const      { return pPtr != NULL; }
	         void                init(
	                                const ::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >& rxSMgr,
	                                LanguageType eLang,
	                                ::com::sun::star::i18n::TransliterationModules nTypeP
	                                )
	                                {
	                                    xSMgr = rxSMgr;
	                                    nType = nTypeP;
	                                    changeLocale( eLang );
	                                    if ( pPtr )
	                                    {
	                                        delete pPtr;
	                                        pPtr = NULL;
	                                    }
	                                }
	         void                changeLocale( LanguageType eLang )
	                                {
	                                    bValid = sal_False;
	                                    eLanguage = eLang;
	                                }
	 const   ::utl::TransliterationWrapper*  get() const
	                                {
	                                    if ( !bValid )
	                                    {
	                                        if ( !pPtr )
	                                            pPtr = new ::utl::TransliterationWrapper( xSMgr, nType );
	                                        pPtr->loadModuleIfNeeded( eLanguage );
	                                        bValid = sal_True;
	                                    }
	                                    return pPtr;
	                                }
	 const   ::utl::TransliterationWrapper*  operator->() const  { return get(); }
	const   ::utl::TransliterationWrapper&  operator*() const   { return *get(); }
};

#endif 

