

#ifndef _PARHTML_HXX
#define _PARHTML_HXX

#ifndef _LANG_HXX
#include <tools/lang.hxx>
#endif
#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _SVARRAY_HXX
#include <svtools/svarray.hxx>
#endif
#ifndef _SVPARSER_HXX
#include <svtools/svparser.hxx>
#endif

class Color;
class SvNumberFormatter;
class SvULongs;

#if defined(MAC)
#define HTMLFONTSZ1_DFLT 9
#define HTMLFONTSZ2_DFLT 10
#define HTMLFONTSZ3_DFLT 12
#define HTMLFONTSZ4_DFLT 14
#define HTMLFONTSZ5_DFLT 18
#define HTMLFONTSZ6_DFLT 20
#define HTMLFONTSZ7_DFLT 32
#else
#define HTMLFONTSZ1_DFLT 7
#define HTMLFONTSZ2_DFLT 10
#define HTMLFONTSZ3_DFLT 12
#define HTMLFONTSZ4_DFLT 14
#define HTMLFONTSZ5_DFLT 18
#define HTMLFONTSZ6_DFLT 24
#define HTMLFONTSZ7_DFLT 36
#endif

enum HTMLTableFrame { HTML_TF_VOID, HTML_TF_ABOVE, HTML_TF_BELOW,
	HTML_TF_HSIDES, HTML_TF_LHS, HTML_TF_RHS, HTML_TF_VSIDES, HTML_TF_BOX };

enum HTMLTableRules { HTML_TR_NONE, HTML_TR_GROUPS, HTML_TR_ROWS,
	HTML_TR_COLS, HTML_TR_ALL };

enum HTMLInputType
{
	HTML_IT_TEXT =		0x01,
	HTML_IT_PASSWORD =	0x02,
	HTML_IT_CHECKBOX =	0x03,
	HTML_IT_RADIO =		0x04,
	HTML_IT_RANGE =		0x05,
	HTML_IT_SCRIBBLE =	0x06,
	HTML_IT_FILE =		0x07,
	HTML_IT_HIDDEN =	0x08,
	HTML_IT_SUBMIT =	0x09,
	HTML_IT_IMAGE =		0x0a,
	HTML_IT_RESET =		0x0b,
	HTML_IT_BUTTON = 	0x0c
};

enum HTMLScriptLanguage
{
	HTML_SL_STARBASIC,
	HTML_SL_JAVASCRIPT,
	HTML_SL_UNKNOWN
};

struct HTMLOptionEnum
{
	const sal_Char *pName;	
	sal_uInt16 nValue;		
};

class HTMLOption
{
	String aValue;			
	String aToken;			
	sal_uInt16 nToken;			

public:

	HTMLOption( sal_uInt16 nTyp, const String& rToken, const String& rValue );

	sal_uInt16 GetToken() const { return nToken; }	
	const String& GetTokenString() const { return aToken; } 

	const String& GetString() const { return aValue; }	

	sal_uInt32 GetNumber() const;                          	
	sal_Int32 GetSNumber() const;                          	
	void GetNumbers( SvULongs &rLongs, 					
		 sal_Bool bSpaceDelim=sal_False ) const;
	void GetColor( Color& ) const;						

	sal_uInt16 GetEnum( const HTMLOptionEnum *pOptEnums,
			sal_uInt16 nDflt=0 ) const;
	BOOL GetEnum( sal_uInt16 &rEnum, const HTMLOptionEnum *pOptEnums ) const;

	HTMLInputType GetInputType() const;				   	
	HTMLTableFrame GetTableFrame() const;				
	HTMLTableRules GetTableRules() const;				

};

typedef HTMLOption* HTMLOptionPtr;
SV_DECL_PTRARR(HTMLOptions,HTMLOptionPtr,16,16);

class HTMLParser : public SvParser
{
	BOOL bNewDoc 		: 1;		
	BOOL bIsInHeader 	: 1;		
	BOOL bIsInBody 		: 1;		
	BOOL bReadListing	: 1;		
	BOOL bReadXMP		: 1;		
	BOOL bReadPRE		: 1;		
	BOOL bReadTextArea	: 1;		
	BOOL bReadScript	: 1;		
	BOOL bReadStyle		: 1;		
	BOOL bEndTokenFound : 1;		

	BOOL bPre_IgnoreNewPara : 1;	
	BOOL bReadNextChar : 1;			
	BOOL bReadComment : 1;			

	sal_uInt32 nPre_LinePos;			

	HTMLOptions *pOptions;			
	String aEndToken;

protected:
	String sSaveToken;				

	int ScanText( const sal_Unicode cBreak = 0U );

	int _GetNextRawToken();

	virtual int _GetNextToken();

	virtual ~HTMLParser();

	void FinishHeader( BOOL bBody ) { bIsInHeader = FALSE; bIsInBody = bBody; }

public:
	HTMLParser( SvStream& rIn, int bReadNewDoc = TRUE );

	virtual SvParserState CallParser();   

	BOOL IsNewDoc() const 		{ return bNewDoc; }
	BOOL IsInHeader() const 	{ return bIsInHeader; }
	BOOL IsInBody() const 		{ return bIsInBody; }
	BOOL IsValidSyntax() const 	{ return TRUE; }
	BOOL IsReadListing() const 	{ return bReadListing; }
	BOOL IsReadXMP() const 		{ return bReadXMP; }
	BOOL IsReadPRE() const 		{ return bReadPRE; }
	BOOL IsReadScript() const 	{ return bReadScript; }
	BOOL IsReadStyle() const 	{ return bReadStyle; }

	void SetReadNextChar() 		{ bReadNextChar = TRUE; }

	inline void StartPRE( BOOL bRestart=FALSE );
	void FinishPRE() { bReadPRE = FALSE; }
	int FilterPRE( int nToken );

	inline void StartListing( BOOL bRestart=FALSE );
	void FinishListing() { bReadListing = FALSE; }
	int FilterListing( int nToken );

	inline void StartXMP( BOOL bRestart=FALSE );
	void FinishXMP() { bReadXMP = FALSE; }
	int FilterXMP( int nToken );

	void FinishTextArea() { bReadTextArea = FALSE; }

	void FinishPREListingXMP() { bReadPRE = bReadListing = bReadXMP = FALSE; }

	int FilterToken( int nToken );

	void EndScanScript() { bReadScript = FALSE; }

	void ReadRawData( const sal_Char *pEndToken ) { aEndToken.AssignAscii(pEndToken); }

	void UnescapeToken();

	const HTMLOptions *GetOptions( sal_uInt16 *pNoConvertToken=0 ) const;

	virtual void Continue( int nToken );

	static FASTBOOL IsHTMLFormat( const sal_Char* pHeader,
		  BOOL bSwitchToUCS2 = FALSE,
		  rtl_TextEncoding eEnc=RTL_TEXTENCODING_DONTKNOW );

	BOOL ParseScriptOptions( String& rLangString, HTMLScriptLanguage& rLang,
				 String& rSrc, String& rLibrary, String& rModule );

	static void RemoveSGMLComment( String &rString, BOOL bFull );

	static BOOL InternalImgToPrivateURL( String& rURL );
};

inline void HTMLParser::StartPRE( BOOL bRestart )
{
	bReadPRE = TRUE;
	bPre_IgnoreNewPara = !bRestart;
	nPre_LinePos = 0UL;
}

inline void HTMLParser::StartListing( BOOL bRestart )
{
	bReadListing = TRUE;
	bPre_IgnoreNewPara = !bRestart;
	nPre_LinePos = 0UL;
}

inline void HTMLParser::StartXMP( BOOL bRestart )
{
	bReadXMP = TRUE;
	bPre_IgnoreNewPara = !bRestart;
	nPre_LinePos = 0UL;
}

#endif
