

#ifndef _SBX_SBXOBJECT_HXX
#define _SBX_SBXOBJECT_HXX

#include <svtools/sbxvar.hxx>

#ifndef _SFXLSTNER_HXX
#include <svtools/lstner.hxx>
#endif

#ifndef __SBX_SBXOBJECT_HXX
#define __SBX_SBXOBJECT_HXX

class SbxProperty;
class SvDispatch;

class SbxObject : public SbxVariable, public SfxListener
{
	SbxArray* FindVar( SbxVariable*, USHORT& );

	SbxArray* VCPtrFindVar( SbxVariable*, USHORT& );
protected:
	SbxArrayRef  pMethods;			
	SbxArrayRef  pProps;			
	SbxArrayRef  pObjs;				
	SbxProperty* pDfltProp;			
	String       aClassName;        
	virtual BOOL LoadData( SvStream&, USHORT );
	virtual BOOL StoreData( SvStream& ) const;
	virtual ~SbxObject();
	virtual void SFX_NOTIFY( SfxBroadcaster& rBC, const TypeId& rBCType,
				 const SfxHint& rHint, const TypeId& rHintType );
public:
	SBX_DECL_PERSIST_NODATA(SBXCR_SBX,SBXID_OBJECT,1);
	TYPEINFO();
	SbxObject( const String& rClassname );
	SbxObject( const SbxObject& );
	SbxObject& operator=( const SbxObject& );
	virtual SbxDataType GetType() const;
	virtual SbxClassType GetClass() const;
	virtual void Clear();

	virtual BOOL  IsClass( const String& ) const;
	const String& GetClassName() const { return aClassName; }
	void		  SetClassName( const String &rNew ) { aClassName = rNew; }

	SbxProperty* GetDfltProperty() const { return pDfltProp; }
	void SetDfltProperty( const String& r );
	void SetDfltProperty( SbxProperty* );

	virtual SbxVariable* FindUserData( UINT32 nUserData );
	virtual SbxVariable* Find( const String&, SbxClassType );
	SbxVariable* FindQualified( const String&, SbxClassType );

	virtual BOOL Call( const String&, SbxArray* = NULL );

	SbxVariable* Execute( const String& );

	virtual BOOL GetAll( SbxClassType ) { return TRUE; }
	SbxVariable* Make( const String&, SbxClassType, SbxDataType );
	virtual SbxObject* MakeObject( const String&, const String& );
	virtual void Insert( SbxVariable* );

	void QuickInsert( SbxVariable* );

	void VCPtrInsert( SbxVariable* );
	virtual void Remove( const String&, SbxClassType );
	virtual void Remove( SbxVariable* );

	void VCPtrRemove( SbxVariable* );
	void SetPos( SbxVariable*, USHORT );

	virtual String GenerateSource( const String &rLinePrefix,
					   const SbxObject *pRelativeTo );

	SbxArray* GetMethods()		{ return pMethods;	}
	SbxArray* GetProperties()	{ return pProps; 	}
	SbxArray* GetObjects()		{ return pObjs; 	}

	virtual SvDispatch* GetSvDispatch();

	void Dump( SvStream&, BOOL bDumpAll=FALSE );

	static void GarbageCollection( ULONG nObjects = 0  );
};

#ifndef __SBX_SBXOBJECTREF_HXX

#ifndef SBX_OBJECT_DECL_DEFINED
#define SBX_OBJECT_DECL_DEFINED
SV_DECL_REF(SbxObject)
#endif
SV_IMPL_REF(SbxObject)

#endif
#endif
#endif
