
#ifndef _SVT_SDBPARSE_HXX
#define _SVT_SDBPARSE_HXX

#ifndef _SVT_SDBSQL_HXX
#include "svtools/sdbsql.hxx"
#endif

#ifndef _LANG_HXX
#include <tools/lang.hxx>
#endif

#ifndef _COM_SUN_STAR_UNO_REFERENCE_H_
#include <com/sun/star/uno/Reference.h>
#endif

namespace com
{
	namespace sun
	{
		namespace star
		{
			namespace beans
			{
	class XPropertySet;
			}
			namespace util
			{
	class XNumberFormatter;
			}
		}
	}
}

class SdbSqlScanner;
class International;

#ifndef YYBISON
#ifndef FLEX_SCANNER
#include "svtools/sdbbison.hxx"
#endif
#endif

class SdbSqlParser 
{
	friend class SdbSqlParseNode;

private:
	static sal_uInt32 nRuleIDs[SdbSqlParseNode::rule_count + 1];
	SdbSqlParseNode*	m_pParseTree;	
	SdbSqlScanner*		m_pScanner;
	International*		m_pIntl;
	String				m_aFieldName;
	String				m_aErrorMessage;
	::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet >     m_xField;
	::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter >  m_xFormatter;
	sal_Int32			m_nFormatKey;
	LanguageType		m_eLanguage;

public:
	SdbSqlParser(LanguageType);
	~SdbSqlParser();

	SdbSqlParseNode* ParseTree(String& rErrorMessage,
	   const String& rStatement,
	   sal_Bool bInternational = sal_False);

	SdbSqlParseNode* PredicateTree(String& rErrorMessage, const String& rStatement,
	       const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > & xFormatter,
	       const International& rIntl,
	       const ::com::sun::star::uno::Reference< ::com::sun::star::beans::XPropertySet > & xField);

	void SetLanguage(LanguageType eType);
	LanguageType GetLanguageType() const {return m_eLanguage;}

	static String TokenIDToStr(sal_uInt32 nTokenID, sal_Bool bInternational);

	static String TokenIDToStr(sal_uInt32 nTokenID);

	static sal_uInt32 StrToTokenID(const ByteString & rName);

	static String RuleIDToStr(sal_uInt32 nRuleID);

	static sal_uInt32 StrToRuleID(const ByteString & rValue);

	static sal_uInt32 RuleID(SdbSqlParseNode::Rule eRule);

	static String GetInternationalKeywordString(sal_uInt32 nTokenID);

	void error(char *fmt);
	int SQLlex();
#ifdef YYBISON
	inline void SetParseTree(SdbSqlParseNode * pNewParseTree) {m_pParseTree = pNewParseTree;}

	sal_Bool InPredicateCheck() const {return m_xField.is();}
	const String& GetFieldName() const {return m_aFieldName;}

	void ReduceLiteral(SdbSqlParseNode*& pLiteral, sal_Bool bAppendBlank);

	sal_Int16 BuildNode(SdbSqlParseNode*& pAppend,SdbSqlParseNode* pLiteral,SdbSqlParseNode*& pCompare);

	sal_Int16 BuildNode_STR_NUM(SdbSqlParseNode*& pAppend,SdbSqlParseNode*& pLiteral,SdbSqlParseNode*& pCompare);
	sal_Int16 BuildNode_Date(const double& fValue, sal_Int16 nType, SdbSqlParseNode*& pAppend,SdbSqlParseNode* pLiteral,SdbSqlParseNode*& pCompare);

	sal_Int16 BuildComparsionRule(SdbSqlParseNode*& pAppend,SdbSqlParseNode* pLiteral);

	sal_Int16 BuildComparsionRule(SdbSqlParseNode*& pAppend,SdbSqlParseNode* pLiteral,SdbSqlParseNode*& pCompare);

	sal_Int16 BuildLikeRule(SdbSqlParseNode*& pAppend,SdbSqlParseNode*& pLiteral,const SdbSqlParseNode* pEscape);
	sal_Int16 BuildStringNodes(SdbSqlParseNode*& pLiteral);
#else
#endif
};

#endif

