

#ifndef _SFXSTYLE_HXX
#define _SFXSTYLE_HXX

#include <rsc/rscsfx.hxx>

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _LIST_HXX 
#include <tools/list.hxx>
#endif
#ifndef _SFXHINT_HXX 
#include <svtools/hint.hxx>
#endif
#ifndef _SFXLSTNER_HXX 
#include <svtools/lstner.hxx>
#endif
#ifndef _SFXBRDCST_HXX 
#include <svtools/brdcst.hxx>
#endif

#ifndef _SFXPOOLITEM_HXX 
#include <svtools/poolitem.hxx>
#endif
#include <svtools/style.hrc>

class SfxItemSet;
class SfxItemPool;

class SfxStyleSheetBasePool;
class SvStream;

#if SUPD > 509
#define VIRTUAL510 virtual
#else
#define VIRTUAL510
#endif

class SfxStyleSheetBase
{
	friend class SfxStyleSheetBasePool;

protected:
	SfxStyleSheetBasePool& 	rPool;			
	SfxStyleFamily 			nFamily;		

	UniString					aName, aParent, aFollow;
	String 					aHelpFile;		
	SfxItemSet* 			pSet;           
	USHORT  				nMask;			

	ULONG 					nHelpId;		

	BOOL   					bMySet;			

	SfxStyleSheetBase( const UniString&, SfxStyleSheetBasePool&,
			SfxStyleFamily eFam, USHORT mask );
	SfxStyleSheetBase( const SfxStyleSheetBase& );
	virtual ~SfxStyleSheetBase();
	virtual void Load( SvStream&, USHORT );
	virtual void Store( SvStream& );

public:
	TYPEINFO();
	virtual const UniString& GetName() const;
	virtual BOOL SetName( const UniString& );
	virtual const UniString& GetParent() const;
	virtual BOOL SetParent( const UniString& );
	virtual const UniString& GetFollow() const;
	virtual BOOL SetFollow( const UniString& );
	virtual BOOL HasFollowSupport() const;		
	virtual BOOL HasParentSupport() const;		
	virtual BOOL HasClearParentSupport() const;	
	virtual BOOL IsUsed() const;				

	virtual UniString GetDescription();
	virtual	UniString GetDescription( SfxMapUnit eMetric );

	SfxStyleSheetBasePool& GetPool() { return rPool;   }
	SfxStyleFamily GetFamily() const     { return nFamily; }
	USHORT 	 GetMask() const     { return nMask; }
	void	 SetMask( USHORT mask) { nMask = mask; }
	BOOL   IsUserDefined() const
	 { return BOOL( ( nMask & SFXSTYLEBIT_USERDEF) != 0 ); }

	virtual ULONG GetHelpId( String& rFile );
	virtual void   SetHelpId( const String& r, ULONG nId );

	virtual SfxItemSet& GetItemSet();
	virtual USHORT GetVersion() const;
};

DECLARE_LIST( SfxStyles, SfxStyleSheetBase* );

class SfxStyleSheetIterator

{
public:
	SfxStyleSheetIterator(SfxStyleSheetBasePool *pBase,
			  SfxStyleFamily eFam, USHORT n=0xFFFF );
	virtual USHORT GetSearchMask() const;
	virtual SfxStyleFamily GetSearchFamily() const;
	virtual USHORT Count();
	virtual SfxStyleSheetBase *operator[](USHORT nIdx);
	virtual SfxStyleSheetBase* First();
	virtual SfxStyleSheetBase* Next();
	virtual SfxStyleSheetBase* Find(const UniString& rStr);
	virtual ~SfxStyleSheetIterator();

protected:

	SfxStyleSheetBasePool*	pBasePool;
	SfxStyleFamily 			nSearchFamily;
	USHORT 					nMask;
	BOOL                    SearchUsed() const { return bSearchUsed; }

private:
	USHORT 					GetPos(){return nAktPosition;}
	BOOL 					IsTrivialSearch();
	BOOL 					DoesStyleMatch(SfxStyleSheetBase *pStyle);

	void*					pImp;
	SfxStyleSheetBase*		pAktStyle;
	USHORT 					nAktPosition;
	BOOL 					bSearchUsed;

friend class SfxStyleSheetBasePool;
};

class SfxStyleSheetBasePool_Impl;

class SfxStyleSheetBasePool: public SfxBroadcaster
{
friend class SfxStyleSheetIterator;
friend class SfxStyleSheetBase;

	SfxStyleSheetBasePool_Impl *pImp;

private:
	BOOL 						Load1_Impl( SvStream& );
	SfxStyleSheetIterator&      GetIterator_Impl();
protected:
	String 						aAppName;
	SfxItemPool& 				rPool;
	SfxStyles	 				aStyles;
	SfxStyleFamily 				nSearchFamily;
	USHORT 		 				nMask;

	SfxStyleSheetBase& 			Add( SfxStyleSheetBase& );
	void 						ChangeParent( const UniString&, const UniString&, BOOL bVirtual = TRUE );
	virtual SfxStyleSheetBase* 	Create( const UniString&, SfxStyleFamily, USHORT );
	virtual SfxStyleSheetBase* 	Create( const SfxStyleSheetBase& );

public:
					SfxStyleSheetBasePool( SfxItemPool& );
					SfxStyleSheetBasePool( const SfxStyleSheetBasePool& );
					~SfxStyleSheetBasePool();

	static String 				GetStreamName();

	const String& 				GetAppName() const { return aAppName; 	}

	SfxItemPool& 				GetPool();
	const SfxItemPool& 			GetPool() const;

#if SUPD > 363
	virtual SfxStyleSheetIterator* CreateIterator(SfxStyleFamily, USHORT nMask);
#endif
	virtual USHORT 				Count();
	virtual SfxStyleSheetBase*	operator[](USHORT nIdx);

	virtual SfxStyleSheetBase& 	Make(const UniString&,
						 SfxStyleFamily eFam,
						 USHORT nMask = 0xffff ,
						 USHORT nPos = 0xffff);

	VIRTUAL510 void             Replace(
		SfxStyleSheetBase& rSource, SfxStyleSheetBase& rTarget );
	virtual SfxStyleSheetBase * Remove( SfxStyleSheetBase* );
	virtual void 				Insert( SfxStyleSheetBase* );

	virtual void 				Erase( SfxStyleSheetBase* );
	virtual void 				Clear();

	SfxStyleSheetBasePool& 		operator=( const SfxStyleSheetBasePool& );
	SfxStyleSheetBasePool& 		operator+=( const SfxStyleSheetBasePool& );

	SfxStyles&	 				GetStyles() { return aStyles; }
	virtual SfxStyleSheetBase* 	First();
	virtual SfxStyleSheetBase* 	Next();
	virtual SfxStyleSheetBase* 	Find( const UniString&,
						  SfxStyleFamily eFam, USHORT n=0xFFFF );
	virtual BOOL                SetParent(SfxStyleFamily eFam,
							  const UniString &rStyle,
							  const UniString &rParent);

	SfxStyleSheetBase* 			Find(const UniString& rStr)
					{ return Find(rStr, nSearchFamily, nMask); }

	void   						SetSearchMask(SfxStyleFamily eFam, USHORT n=0xFFFF );
	USHORT 						GetSearchMask() const;
	SfxStyleFamily 				GetSearchFamily() const  { return nSearchFamily; }

	BOOL 						Load( SvStream& );
	BOOL 						Store( SvStream&, BOOL bUsed = TRUE );
};

class SfxStyleSheet: public SfxStyleSheetBase,
		 public SfxListener, public SfxBroadcaster
{
public:
			TYPEINFO();
				SfxStyleSheet( const UniString&, SfxStyleSheetBasePool&, SfxStyleFamily, USHORT );
			SfxStyleSheet( const SfxStyleSheet& );

	virtual 			~SfxStyleSheet();

	virtual void 		SFX_NOTIFY( SfxBroadcaster& rBC, const TypeId& rBCType,
					const SfxHint& rHint, const TypeId& rHintType );
	virtual BOOL 		SetParent( const UniString& );
};

class SfxStyleSheetPool: public SfxStyleSheetBasePool
{
protected:
	virtual SfxStyleSheetBase* Create(const UniString&, SfxStyleFamily, USHORT mask);
	virtual SfxStyleSheetBase* Create(const SfxStyleSheet &);

public:
	SfxStyleSheetPool( SfxItemPool& );

};

#define SFX_STYLESHEET_CREATED		 1	
#define SFX_STYLESHEET_MODIFIED      2	
#define SFX_STYLESHEET_CHANGED       3	
#define SFX_STYLESHEET_ERASED        4	
#define SFX_STYLESHEET_INDESTRUCTION 5	

#define SFX_STYLESHEETPOOL_CHANGES  1  

class SfxStyleSheetPoolHint : public SfxHint
{
	USHORT nHint;

public:
	TYPEINFO();
				SfxStyleSheetPoolHint(USHORT nArgHint) :  nHint(nArgHint){}
	USHORT				GetHint() const
			{ return nHint; }
};

class SfxStyleSheetHint: public SfxHint
{
	SfxStyleSheetBase*	pStyleSh;
	USHORT              nHint;

public:
			TYPEINFO();
				SfxStyleSheetHint( USHORT, SfxStyleSheetBase& = *((SfxStyleSheetBase*)0) );
	SfxStyleSheetBase*	GetStyleSheet() const
			{ return pStyleSh; }
	USHORT				GetHint() const
			{ return nHint; }
};

class SfxStyleSheetHintExtended: public SfxStyleSheetHint
{
	String              aName;

public:
			TYPEINFO();
				SfxStyleSheetHintExtended(
				USHORT, const String& rOld,
				SfxStyleSheetBase& =
				*((SfxStyleSheetBase*)0) );
	const String&       GetOldName() { return aName; }
};

#endif

