

#ifndef _SVEDIT_HXX
#define _SVEDIT_HXX

#ifndef _SV_WINTYPES_HXX
#include <vcl/wintypes.hxx>
#endif
#ifndef _EDIT_HXX 
#include <vcl/edit.hxx>
#endif

class ImpSvMEdit;
class Timer;
class ExtTextEngine;
class ExtTextView;

class MultiLineEdit : public Edit
{
private:
	ImpSvMEdit*		pImpSvMEdit;

	XubString		aSaveValue;
	Link			aModifyHdlLink;

	Timer*			pUpdateDataTimer;
	Link			aUpdateDataHdlLink;

protected:

	DECL_LINK( 		ImpUpdateDataHdl, Timer* );
	void 			StateChanged( StateChangedType nType );
	void 			DataChanged( const DataChangedEvent& rDCEvt );
	long 			PreNotify( NotifyEvent& rNEvt );
	long 			Notify( NotifyEvent& rNEvt );
	void 			ImplInitSettings( BOOL bFont, BOOL bForeground, BOOL bBackground );
	WinBits 		ImplInitStyle( WinBits nStyle );

	ExtTextEngine*	GetTextEngine() const;
	ExtTextView*	GetTextView() const;
	ScrollBar*		GetHScrollBar() const;
	ScrollBar*		GetVScrollBar() const;

#ifndef VCL
		MultiLineEdit( RESOURCE_TYPE nTyp, Window* pParent, WinBits nWinStyle );
#endif

public:
		MultiLineEdit( Window* pParent, WinBits nWinStyle = WB_LEFT | WB_BORDER );
		MultiLineEdit( Window* pParent, const ResId& rResId );
		~MultiLineEdit();

	virtual void	Modify();
	virtual void	UpdateData();

	void			SetModifyFlag();
	void			ClearModifyFlag();
	BOOL			IsModified() const;

	void			EnableUpdateData( ULONG nTimeout = EDIT_UPDATEDATA_TIMEOUT );
	void			DisableUpdateData() { delete pUpdateDataTimer; pUpdateDataTimer = NULL; }
	ULONG			IsUpdateDataEnabled() const;

	void			SetReadOnly( BOOL bReadOnly = TRUE );
	BOOL			IsReadOnly() const;

	void			EnableFocusSelectionHide( BOOL bHide );
	BOOL			IsFocusSelectionHideEnabled() const;

	void			SetMaxTextLen( ULONG nMaxLen = 0 );
	ULONG			GetMaxTextLen() const;

	void			SetSelection( const Selection& rSelection );
	Selection		GetSelection() const;

	void			ReplaceSelected( const XubString& rStr );
	void			DeleteSelected();
	XubString		GetSelected() const;

	void			Cut();
	void			Copy();
	void			Paste();

	void			SetText( const String& rStr );
	String			GetText() const;
	String			GetTextLines() const;

	void			SaveValue() 						{ aSaveValue = GetText(); }
	const XubString&	GetSavedValue() const 				{ return aSaveValue; }

	void			SetModifyHdl( const Link& rLink ) 	{ aModifyHdlLink = rLink; }
	const Link&		GetModifyHdl() const 				{ return aModifyHdlLink; }

	void			SetUpdateDataHdl( const Link& rLink ) { aUpdateDataHdlLink = rLink; }
	const Link&		GetUpdateDataHdl() const { return aUpdateDataHdlLink; }

	virtual void	Resize();
	virtual void	GetFocus();

	Size			CalcMinimumSize() const;
	Size			CalcAdjustedSize( const Size& rPrefSize ) const;
	Size			CalcSize( USHORT nColumns, USHORT nLines ) const;
	void			GetMaxVisColumnsAndLines( USHORT& rnCols, USHORT& rnLines ) const;

	void 			Draw( OutputDevice* pDev, const Point& rPos, const Size& rSize, ULONG nFlags );
};

inline ULONG MultiLineEdit::IsUpdateDataEnabled() const
{
	return pUpdateDataTimer ? pUpdateDataTimer->GetTimeout() : 0;
}

#endif

