

#ifndef _TABBAR_HXX
#define _TABBAR_HXX

#ifndef _LINK_HXX
#include <tools/link.hxx>
#endif
#ifndef _WINDOW_HXX
#include <vcl/window.hxx>
#endif

class MouseEvent;
class TrackingEvent;
class DataChangedEvent;
class ImplTabBarList;
class ImplTabButton;
class ImplTabSizer;
class TabBarEdit;

#define WB_RANGESELECT      ((WinBits)0x00200000)
#define WB_MULTISELECT      ((WinBits)0x00400000)
#define WB_TOPBORDER        ((WinBits)0x04000000)
#define WB_3DTAB            ((WinBits)0x08000000)
#define WB_MINSCROLL        ((WinBits)0x20000000)
#define WB_STDTABBAR        WB_BORDER

typedef USHORT TabBarPageBits;

#define TPB_SPECIAL         ((TabBarPageBits)0x0001)

#define TAB_APPEND          ((USHORT)0xFFFF)
#define TAB_PAGE_NOTFOUND   ((USHORT)0xFFFF)

#define TAB_RENAMING_YES    ((long)TRUE)
#define TAB_RENAMING_NO     ((long)FALSE)
#define TAB_RENAMING_CANCEL ((long)2)

class TabBar : public Window
{
	friend class    ImplTabButton;
	friend class    ImplTabSizer;

private:
	ImplTabBarList* mpItemList;
	ImplTabButton*  mpFirstBtn;
	ImplTabButton*  mpPrevBtn;
	ImplTabButton*  mpNextBtn;
	ImplTabButton*  mpLastBtn;
	ImplTabSizer*   mpSizer;
	TabBarEdit*     mpEdit;
	XubString       maEditText;
	Color           maSelColor;
	Color           maSelTextColor;
	Size            maWinSize;
	long            mnMaxPageWidth;
	long            mnCurMaxWidth;
	long            mnOffX;
	long            mnOffY;
	long            mnOutWidth;
	long            mnSplitSize;
	ULONG           mnSwitchTime;
	WinBits         mnWinStyle;
	USHORT          mnCurPageId;
	USHORT          mnFirstPos;
	USHORT          mnDropPos;
	USHORT          mnSwitchId;
	USHORT          mnEditId;
	BOOL            mbFormat;
	BOOL            mbFirstFormat;
	BOOL            mbSizeFormat;
	BOOL            mbAutoMaxWidth;
	BOOL            mbInSwitching;
	BOOL            mbAutoEditMode;
	BOOL            mbEditCanceled;
	BOOL            mbDropPos;
	BOOL            mbInSelect;
	BOOL            mbSelColor;
	BOOL            mbSelTextColor;
	Link            maSelectHdl;
	Link            maDoubleClickHdl;
	Link            maSplitHdl;
	Link            maActivatePageHdl;
	Link            maDeactivatePageHdl;
	Link            maStartRenamingHdl;
	Link            maAllowRenamingHdl;
	Link            maEndRenamingHdl;

#ifdef _SV_TABBAR_CXX
	void            ImplInit( WinBits nWinStyle );
	void            ImplInitSettings( BOOL bFont, BOOL bBackground );
	void            ImplGetColors( Color& rFaceColor, Color& rFaceTextColor,
	                               Color& rSelectColor, Color& rSelectTextColor );
	void            ImplShowPage( USHORT nPos );
	BOOL            ImplCalcWidth();
	void            ImplFormat();
	USHORT          ImplGetLastFirstPos();
	void            ImplInitControls();
	void            ImplEnableControls();
	                DECL_LINK( ImplClickHdl, ImplTabButton* );
#endif

public:
	                TabBar( Window* pParent, WinBits nWinStyle = WB_STDTABBAR );
	virtual         ~TabBar();
	 virtual void    MouseMove( const MouseEvent& rMEvt );
	virtual void    MouseButtonDown( const MouseEvent& rMEvt );
	virtual void    MouseButtonUp( const MouseEvent& rMEvt );
	virtual void    Paint( const Rectangle& rRect );
	virtual void    Resize();
	virtual void    RequestHelp( const HelpEvent& rHEvt );
	virtual void    StateChanged( StateChangedType nStateChange );
	virtual void    DataChanged( const DataChangedEvent& rDCEvt );
	 virtual void    Select();
	virtual void    DoubleClick();
	virtual void    Split();
	virtual void    ActivatePage();
	virtual long    DeactivatePage();
	virtual long    StartRenaming();
	virtual long    AllowRenaming();
	virtual void    EndRenaming();
	 void            InsertPage( USHORT nPageId, const XubString& rText,
	                            TabBarPageBits nBits = 0,
	                            USHORT nPos = TAB_APPEND );
	void            RemovePage( USHORT nPageId );
	void            MovePage( USHORT nPageId, USHORT nNewPos );
	void            Clear();
	 void            EnablePage( USHORT nPageId, BOOL bEnable = TRUE );
	BOOL            IsPageEnabled( USHORT nPageId ) const;
	 void            SetPageBits( USHORT nPageId, TabBarPageBits nBits = 0 );
	TabBarPageBits  GetPageBits( USHORT nPageId ) const;
	 USHORT          GetPageCount() const;
	USHORT          GetPageId( USHORT nPos ) const;
	USHORT          GetPagePos( USHORT nPageId ) const;
	USHORT          GetPageId( const Point& rPos ) const;
	Rectangle       GetPageRect( USHORT nPageId ) const;
	 void            SetCurPageId( USHORT nPageId );
	USHORT          GetCurPageId() const { return mnCurPageId; }
	 void            SetFirstPageId( USHORT nPageId );
	USHORT          GetFirstPageId() const { return GetPageId( mnFirstPos ); }
	void            MakeVisible( USHORT nPageId );
	 void            SelectPage( USHORT nPageId, BOOL bSelect = TRUE );
	void            SelectPageRange( BOOL bSelect = FALSE,
	                                 USHORT nStartPos = 0,
	                                 USHORT nEndPos = TAB_APPEND );
	USHORT          GetSelectPage( USHORT nSelIndex = 0 ) const;
	USHORT          GetSelectPageCount() const;
	BOOL            IsPageSelected( USHORT nPageId ) const;
	 void            EnableAutoMaxPageWidth( BOOL bEnable = TRUE ) { mbAutoMaxWidth = bEnable; }
	BOOL            IsAutoMaxPageWidthEnabled() const { return mbAutoMaxWidth; }
	void            SetMaxPageWidth( long nMaxWidth );
	long            GetMaxPageWidth() const { return mnMaxPageWidth; }
	void            ResetMaxPageWidth() { SetMaxPageWidth( 0 ); }
	BOOL            IsMaxPageWidth() const { return mnMaxPageWidth != 0; }
	 void            EnableEditMode( BOOL bEnable = TRUE ) { mbAutoEditMode = bEnable; }
	BOOL            IsEditModeEnabled() const { return mbAutoEditMode; }
	BOOL            StartEditMode( USHORT nPageId );
	void            EndEditMode( BOOL bCancel = FALSE );
	void            SetEditText( const XubString& rText ) { maEditText = rText; }
	const XubString& GetEditText() const { return maEditText; }
	BOOL            IsInEditMode() const { return (mpEdit != NULL); }
	BOOL            IsEditModeCanceled() const { return mbEditCanceled; }
	USHORT          GetEditPageId() const { return mnEditId; }
	 BOOL            StartDrag( const CommandEvent& rCEvt, Region& rRegion );
	USHORT          ShowDropPos( const Point& rPos );
	void            HideDropPos();
	BOOL            SwitchPage( const Point& rPos );
	void            EndSwitchPage();
	BOOL            IsInSwitching() { return mbInSwitching; }
	 void            SetSelectColor();
	void            SetSelectColor( const Color& rColor );
	const Color&    GetSelectColor() const { return maSelColor; }
	BOOL            IsSelectColor() const { return mbSelColor; }
	void            SetSelectTextColor();
	void            SetSelectTextColor( const Color& rColor );
	const Color&    GetSelectTextColor() const { return maSelTextColor; }
	BOOL            IsSelectTextColor() const { return mbSelTextColor; }
	 void            SetPageText( USHORT nPageId, const XubString& rText );
	XubString       GetPageText( USHORT nPageId ) const;
	void            SetHelpText( USHORT nPageId, const XubString& rText );
	XubString       GetHelpText( USHORT nPageId ) const;
	void            SetHelpId( USHORT nPageId, ULONG nHelpId );
	ULONG           GetHelpId( USHORT nPageId ) const;
	 long            GetSplitSize() const { return mnSplitSize; }
	long            GetMinSize() const;
	 void            SetHelpText( const XubString& rText )
	                    { Window::SetHelpText( rText ); }
	XubString       GetHelpText() const
	                    { return Window::GetHelpText(); };
	void            SetHelpId( ULONG nId )
	                    { Window::SetHelpId( nId ); }
	ULONG           GetHelpId() const
	                    { return Window::GetHelpId(); }
	 void            SetStyle( WinBits nStyle );
	WinBits         GetStyle() const { return mnWinStyle; }
	 Size            CalcWindowSizePixel() const;
	 void            SetSelectHdl( const Link& rLink ) { maSelectHdl = rLink; }
	const Link&     GetSelectHdl() const { return maSelectHdl; }
	void            SetDoubleClickHdl( const Link& rLink ) { maDoubleClickHdl = rLink; }
	const Link&     GetDoubleClickHdl() const { return maDoubleClickHdl; }
	void            SetSplitHdl( const Link& rLink ) { maSplitHdl = rLink; }
	const Link&     GetSplitHdl() const { return maSplitHdl; }
	void            SetActivatePageHdl( const Link& rLink ) { maActivatePageHdl = rLink; }
	const Link&     GetActivatePageHdl() const { return maActivatePageHdl; }
	void            SetDeactivatePageHdl( const Link& rLink ) { maDeactivatePageHdl = rLink; }
	const Link&     GetDeactivatePageHdl() const { return maDeactivatePageHdl; }
	void            SetStartRenamingHdl( const Link& rLink ) { maStartRenamingHdl = rLink; }
	const Link&     GetStartRenamingHdl() const { return maStartRenamingHdl; }
	void            SetAllowRenamingHdl( const Link& rLink ) { maAllowRenamingHdl = rLink; }
	const Link&     GetAllowRenamingHdl() const { return maAllowRenamingHdl; }
	void            SetEndRenamingHdl( const Link& rLink ) { maEndRenamingHdl = rLink; }
	const Link&     GetEndRenamingHdl() const { return maEndRenamingHdl; }
};

#endif  
