

#ifndef _TASKBAR_HXX
#define _TASKBAR_HXX

#ifndef _TOOLS_TIME_HXX
#include <tools/time.hxx>
#endif
#ifndef _TOOLS_INTN_HXX
#include <tools/intn.hxx>
#endif
#ifndef _TOOLS_LIST_HXX
#include <tools/list.hxx>
#endif

#ifndef _SV_TIMER_HXX
#include <vcl/timer.hxx>
#endif
#ifndef _SV_TOOLBOX_HXX
#include <vcl/toolbox.hxx>
#endif
#ifndef _SV_STATUS_HXX
#include <vcl/status.hxx>
#endif

class TaskBar;
class TaskStatusFieldItem;
class ImplTaskItemList;
class ImplTaskSBItemList;
class ImplTaskBarFloat;
struct ImplTaskSBFldItem;

class TaskButtonBar : public ToolBox
{
	friend class TaskBar;

private:
	TaskBar*            mpNotifyTaskBar;
	void*               mpDummy1;
	void*               mpDummy2;
	void*               mpDummy3;
	void*               mpDummy4;

public:
	                    TaskButtonBar( Window* pParent, WinBits nWinStyle = 0 );
	                    ~TaskButtonBar();
	 virtual void        RequestHelp( const HelpEvent& rHEvt );
	 void                InsertButton( USHORT nItemId,
	                                  const Image& rImage, const String& rText,
	                                  USHORT nPos = TOOLBOX_APPEND )
	                        { InsertItem( nItemId, rImage, rText, TIB_LEFT | TIB_AUTOSIZE, nPos ); }
	void                RemoveButton( USHORT nItemId )
	                        { RemoveItem( nItemId ); }
};

#define TASKTOOLBOX_TASK_NOTFOUND       ((USHORT)0xFFFF)

class TaskToolBox : public ToolBox
{
	friend class TaskBar;

private:
	ImplTaskItemList*   mpItemList;
	TaskBar*            mpNotifyTaskBar;
	Point               maContextMenuPos;
	ULONG               mnOldItemCount;
	long                mnMaxTextWidth;
	long                mnDummy1;
	USHORT              mnUpdatePos;
	USHORT              mnUpdateNewPos;
	USHORT              mnActiveItemId;
	USHORT              mnNewActivePos;
	USHORT              mnTaskItem;
	USHORT              mnSmallItem;
	USHORT              mnDummy2;
	BOOL                mbMinActivate;
	BOOL                mbDummy1;
	Link                maActivateTaskHdl;
	Link                maContextMenuHdl;

#ifdef _TASKBAR_CXX
	void                ImplFormat();
#endif

public:
	                    TaskToolBox( Window* pParent, WinBits nWinStyle = 0 );
	                    ~TaskToolBox();
	 void                ActivateTaskItem( USHORT nItemId,
	                                      BOOL bMinActivate = FALSE );
	USHORT              GetTaskItem( const Point& rPos ) const;
	 virtual void        ActivateTask();
	virtual void        ContextMenu();
	 virtual void        Select();
	 virtual void        MouseButtonDown( const MouseEvent& rMEvt );
	virtual void        Resize();
	virtual void        Command( const CommandEvent& rCEvt );
	virtual void        RequestHelp( const HelpEvent& rHEvt );
	 void                StartUpdateTask();
	void                UpdateTask( const Image& rImage, const String& rText,
	                                BOOL bActive = FALSE );
	void                EndUpdateTask();
	 const Point&        GetContextMenuPos() const { return maContextMenuPos; }
	USHORT              GetTaskItem() const { return mnTaskItem; }
	BOOL                IsMinActivate() const { return mbMinActivate; }
	 void                SetActivateTaskHdl( const Link& rLink ) { maActivateTaskHdl = rLink; }
	const Link&         GetActivateTaskHdl() const { return maActivateTaskHdl; }
	void                SetContextMenuHdl( const Link& rLink ) { maContextMenuHdl = rLink; }
	const Link&         GetContextMenuHdl() const { return maContextMenuHdl; }
};

inline USHORT TaskToolBox::GetTaskItem( const Point& rPos ) const
{
	USHORT nId = GetItemId( rPos );
	if ( nId )
	    return nId-1;
	else
	    return TASKTOOLBOX_TASK_NOTFOUND;
}

class ITaskStatusNotify
{
public:
	virtual BOOL        MouseButtonDown( USHORT nItemd, const MouseEvent& rMEvt );
	virtual BOOL        MouseButtonUp( USHORT nItemd, const MouseEvent& rMEvt );
	virtual BOOL        MouseMove( USHORT nItemd, const MouseEvent& rMEvt );
	virtual BOOL        Command( USHORT nItemd, const CommandEvent& rCEvt );
	virtual BOOL        UpdateHelp( USHORT nItemd );
};

#define TASKSTATUSFIELDITEM_FLASH           ((USHORT)0x0001)

class TaskStatusFieldItem
{
private:
	ITaskStatusNotify*  mpNotify;
	Image               maImage;
	XubString           maQuickHelpText;
	XubString           maHelpText;
	ULONG               mnHelpId;
	USHORT              mnFlags;

public:
	                    TaskStatusFieldItem();
	                    TaskStatusFieldItem( const TaskStatusFieldItem& rItem );
	                    TaskStatusFieldItem( ITaskStatusNotify* pNotify,
	                                         const Image& rImage,
	                                         const XubString& rQuickHelpText,
	                                         const XubString& rHelpText,
	                                         USHORT nFlags );
	                    ~TaskStatusFieldItem();
	 void                SetNotifyObject( ITaskStatusNotify* pNotify ) { mpNotify = pNotify; }
	ITaskStatusNotify*  GetNotifyObject() const { return mpNotify; }
	void                SetImage( const Image& rImage ) { maImage = rImage; }
	const Image&        GetImage() const { return maImage; }
	void                SetQuickHelpText( const XubString& rStr ) { maQuickHelpText = rStr; }
	const XubString&    GetQuickHelpText() const { return maQuickHelpText; }
	void                SetHelpText( const XubString& rStr ) { maHelpText = rStr; }
	const XubString&    GetHelpText() const { return maHelpText; }
	void                SetHelpId( ULONG nHelpId ) { mnHelpId = nHelpId; }
	ULONG               GetHelpId() const { return mnHelpId; }
	void                SetFlags( USHORT nFlags ) { mnFlags = nFlags; }
	USHORT              GetFlags() const { return mnFlags; }
	 const TaskStatusFieldItem& operator=( const TaskStatusFieldItem& rItem );
};

#define TASKSTATUSBAR_STATUSFIELDID         ((USHORT)61000)

#define TASKSTATUSBAR_CLOCKID               ((USHORT)61000)
#define TASKSTATUSFIELD_CLOCK               ((USHORT)0x0001)

class TaskStatusBar : public StatusBar
{
	friend class TaskBar;

private:
	ImplTaskSBItemList* mpFieldItemList;
	TaskBar*            mpNotifyTaskBar;
	ITaskStatusNotify*  mpNotify;
	Time                maTime;
	XubString           maTimeText;
	AutoTimer           maTimer;
	International       maIntn;
	long                mnClockWidth;
	long                mnItemWidth;
	long                mnFieldWidth;
	USHORT              mnFieldFlags;
	USHORT              mnDummy1;
	BOOL                mbFlashItems;
	BOOL                mbOutInterval;
	BOOL                mbDummy1;
	BOOL                mbDummy2;

#ifdef _TASKBAR_CXX
	ImplTaskSBFldItem*  ImplGetFieldItem( USHORT nItemId ) const;
	ImplTaskSBFldItem*  ImplGetFieldItem( const Point& rPos, BOOL& rFieldRect ) const;
	BOOL                ImplUpdateClock();
	BOOL                ImplUpdateFlashItems();
	void                ImplUpdateField( BOOL bItems );
	                    DECL_LINK( ImplTimerHdl, Timer* );
#endif

public:
	                    TaskStatusBar( Window* pParent, WinBits nWinStyle = WB_LEFT );
	                    ~TaskStatusBar();
	 virtual void        MouseButtonDown( const MouseEvent& rMEvt );
	virtual void        MouseButtonUp( const MouseEvent& rMEvt );
	virtual void        MouseMove( const MouseEvent& rMEvt );
	virtual void        Command( const CommandEvent& rCEvt );
	virtual void        RequestHelp( const HelpEvent& rHEvt );
	virtual void        UserDraw( const UserDrawEvent& rUDEvt );
	 void                InsertStatusField( long nOffset = STATUSBAR_OFFSET,
	                                       USHORT nPos = STATUSBAR_APPEND,
	                                       USHORT nFlags = TASKSTATUSFIELD_CLOCK );
	void                RemoveStatusField()
	                        { maTimer.Stop(); RemoveItem( TASKSTATUSBAR_STATUSFIELDID ); }
	void                SetFieldFlags( USHORT nFlags );
	USHORT              GetFieldFlags() const { return mnFieldFlags; }
	void                SetNotifyObject( ITaskStatusNotify* pNotify ) { mpNotify = pNotify; }
	ITaskStatusNotify*  GetNotifyObject() const { return mpNotify; }
	 void                AddStatusFieldItem( USHORT nItemId, const TaskStatusFieldItem& rItem,
	                                        USHORT nPos = 0xFFFF );
	void                ModifyStatusFieldItem( USHORT nItemId, const TaskStatusFieldItem& rItem );
	void                RemoveStatusFieldItem( USHORT nItemId );
	BOOL                GetStatusFieldItem( USHORT nItemId, TaskStatusFieldItem& rItem ) const;
};

class TaskBar : public Window
{
private:
	ImplTaskBarFloat*       mpAutoHideBar;
	TaskButtonBar*          mpButtonBar;
	TaskToolBox*            mpTaskToolBox;
	TaskStatusBar*          mpStatusBar;
	void*                   mpDummy1;
	void*                   mpDummy2;
	void*                   mpDummy3;
	void*                   mpDummy4;
	String                  maOldText;
	long                    mnStatusWidth;
	long                    mnMouseOff;
	long                    mnOldStatusWidth;
	long                    mnDummy1;
	long                    mnDummy2;
	long                    mnDummy3;
	long                    mnDummy4;
	WinBits                 mnWinBits;
	USHORT                  mnLines;
	BOOL                    mbStatusText;
	BOOL                    mbShowItems;
	BOOL                    mbAutoHide;
	BOOL                    mbAlignDummy1;
	BOOL                    mbDummy1;
	BOOL                    mbDummy2;
	BOOL                    mbDummy3;
	BOOL                    mbDummy4;
	Link                    maTaskResizeHdl;

#ifdef _TASKBAR_CXX
	void                    ImplInitSettings();
	void                    ImplNewHeight( long nNewHeight );
#endif

public:
	                        TaskBar( Window* pParent, WinBits nWinStyle = WB_BORDER | WB_SIZEABLE );
	                        ~TaskBar();
	 virtual void            TaskResize();
	 virtual TaskButtonBar*  CreateButtonBar();
	virtual TaskToolBox*    CreateTaskToolBox();
	virtual TaskStatusBar*  CreateTaskStatusBar();
	 virtual void            MouseMove( const MouseEvent& rMEvt );
	virtual void            MouseButtonDown( const MouseEvent& rMEvt );
	virtual void            Tracking( const TrackingEvent& rMEvt );
	virtual void            Paint( const Rectangle& rRect );
	virtual void            Resize();
	virtual void            StateChanged( StateChangedType nType );
	virtual void            DataChanged( const DataChangedEvent& rDCEvt );
	 void                    Format();
	 void                    SetLines( USHORT nLines );
	USHORT                  GetLines() const { return mnLines; }
	void                    EnableAutoHide( BOOL bAutoHide = TRUE );
	BOOL                    IsAutoHideEnabled() const { return mbAutoHide; }
	 void                    ShowStatusText( const String& rText );
	void                    HideStatusText();
	 void                    SetStatusSize( long nNewSize )
	                            { mnStatusWidth=nNewSize; Resize(); }
	long                    GetStatusSize() const { return mnStatusWidth; }
	 Size                    CalcWindowSizePixel() const;
	 TaskButtonBar*          GetButtonBar() const;
	TaskToolBox*            GetTaskToolBox() const;
	TaskStatusBar*          GetStatusBar() const;
	 void                    SetTaskResizeHdl( const Link& rLink ) { maTaskResizeHdl = rLink; }
	const Link&             GetTaskResizeHdl() const { return maTaskResizeHdl; }
};

#define WINDOWARRANGE_TILE      1
#define WINDOWARRANGE_HORZ      2
#define WINDOWARRANGE_VERT      3
#define WINDOWARRANGE_CASCADE   4

class ImplWindowArrangeList;

class WindowArrange
{
private:
	List*                   mpWinList;
	void*                   mpDummy;
	ULONG                   mnDummy;

#ifdef _TASKBAR_CXX
	void                    ImplTile( const Rectangle& rRect );
	void                    ImplHorz( const Rectangle& rRect );
	void                    ImplVert( const Rectangle& rRect );
	void                    ImplCascade( const Rectangle& rRect );
#endif

public:
	                        WindowArrange();
	                        ~WindowArrange();
	 void                    AddWindow( Window* pWindow, ULONG nPos = LIST_APPEND )
	                            { mpWinList->Insert( (void*)pWindow, nPos ); }
	void                    RemoveAllWindows()
	                            { mpWinList->Clear(); }
	 void                    Arrange( USHORT nType, const Rectangle& rRect );
};

#endif  
