
#ifndef _ZFORMAT_HXX
#define _ZFORMAT_HXX

#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _LANG_HXX 
#include <tools/lang.hxx>
#endif
#ifndef _INTN_HXX 
#include <tools/intn.hxx>
#endif

#include <svtools/zforlist.hxx>
#include <svtools/nfversi.hxx>

enum NfKeywordIndex
{
	NF_KEY_NONE = 0,
	NF_KEY_E,			
	NF_KEY_AMPM,		
	NF_KEY_AP,			
	NF_KEY_MI,			
	NF_KEY_MMI,			
	NF_KEY_M,			
	NF_KEY_MM,			
	NF_KEY_MMM,			
	NF_KEY_MMMM,		
	NF_KEY_H,			
	NF_KEY_HH,			
	NF_KEY_S,			
	NF_KEY_SS,			
	NF_KEY_Q,			
	NF_KEY_QQ,			
	NF_KEY_D,			
	NF_KEY_DD,			
	NF_KEY_DDD,			
	NF_KEY_DDDD,		
	NF_KEY_YY,			
	NF_KEY_YYYY,		
	NF_KEY_NN,			
	NF_KEY_NNNN,		
	NF_KEY_CCC,			
	NF_KEY_GENERAL,		
	NF_KEY_LASTOLDKEYWORD = NF_KEY_GENERAL,
	NF_KEY_NNN,			
	NF_KEY_WW,			
	NF_KEY_MMMMM,		
	NF_KEY_LASTKEYWORD = NF_KEY_MMMMM,
	NF_KEY_UNUSED4,
	NF_KEY_QUARTER,     
	NF_KEY_TRUE,		
	NF_KEY_FALSE,		
	NF_KEY_BOOLEAN,		
	NF_KEY_COLOR,		
	NF_KEY_FIRSTCOLOR,
	NF_KEY_BLACK = NF_KEY_FIRSTCOLOR,	
	NF_KEY_BLUE,
	NF_KEY_GREEN,
	NF_KEY_CYAN,
	NF_KEY_RED,
	NF_KEY_MAGENTA,
	NF_KEY_BROWN,
	NF_KEY_GREY,
	NF_KEY_YELLOW,
	NF_KEY_WHITE,
	NF_KEY_LASTCOLOR = NF_KEY_WHITE,
	NF_KEY_LASTKEYWORD_SO5 = NF_KEY_LASTCOLOR,

	NF_KEY_AAA,			
	NF_KEY_AAAA,		
	NF_KEY_EC,			
	NF_KEY_EEC,			
	NF_KEY_G,			
	NF_KEY_GG,			
	NF_KEY_GGG,			
	NF_KEY_R,			
	NF_KEY_RR,			
	NF_KEYWORD_ENTRIES_COUNT
};

typedef String NfKeywordTable [NF_KEYWORD_ENTRIES_COUNT];

#ifdef _ZFORMAT_CXX
#include "zforscan.hxx"
#endif

#if SUPD < 372
#define NF_COMMENT_IN_FORMATSTRING 1
#else
#define NF_COMMENT_IN_FORMATSTRING 0
#endif

class SvStream;
class Color;

class ImpSvNumberformatScan;			
class ImpSvNumberInputScan;				
class ImpSvNumMultipleWriteHeader;		
class ImpSvNumMultipleReadHeader;		
class SvNumberFormatter;

enum SvNumberformatLimitOps
{
	NUMBERFORMAT_OP_NO  = 0,			
	NUMBERFORMAT_OP_EQ  = 1, 			
	NUMBERFORMAT_OP_NE  = 2,			
	NUMBERFORMAT_OP_LT  = 3,			
	NUMBERFORMAT_OP_LE  = 4,			
	NUMBERFORMAT_OP_GT  = 5,			
	NUMBERFORMAT_OP_GE  = 6				
};

enum NfHackConversion
{
	NF_CONVERT_NONE,
	NF_CONVERT_GERMAN_ENGLISH,
	NF_CONVERT_ENGLISH_GERMAN
};

struct ImpSvNumberformatInfo           	
{
	String* sStrArray;					
	short* nTypeArray;					
	USHORT nThousand;					
	USHORT nCntPre;						
	USHORT nCntPost;					
	USHORT nCntExp;					   	
	short eScannedType;					
	BOOL bThousand;						

	void Copy( const ImpSvNumberformatInfo& rNumFor, USHORT nAnz );
	void Load(SvStream& rStream, USHORT nAnz);
	void Save(SvStream& rStream, USHORT nAnz) const;
};

struct SvNumberDBNum
{
	LanguageType	eLang;
	BYTE			nNum;
			SvNumberDBNum() : eLang( LANGUAGE_DONTKNOW ), nNum(0) {}
	BOOL			IsComplete() const { return nNum && eLang != LANGUAGE_DONTKNOW; }
};

class CharClass;

class ImpSvNumFor						
{
public:
	ImpSvNumFor();						
	~ImpSvNumFor();

	void Enlarge(USHORT nAnz);			
	void Load( SvStream& rStream, ImpSvNumberformatScan& rSc,
	String& rLoadedColorName);
	void Save( SvStream& rStream ) const;

	void Copy( const ImpSvNumFor& rNumFor );

	ImpSvNumberformatInfo& Info() { return aI;}
	const ImpSvNumberformatInfo& Info() const { return aI; }

	USHORT GetnAnz() const { return nAnzStrings;}

	Color* GetColor() const { return pColor; }
	void SetColor( Color* pCol, String& rName )
	 { pColor = pCol; sColorName = rName; }
	const String& GetColorName() const { return sColorName; }

	BOOL HasNewCurrency() const;
	BOOL GetNewCurrencySymbol( String& rSymbol, String& rExtension ) const;
	void SaveNewCurrencyMap( SvStream& rStream ) const;
	void LoadNewCurrencyMap( SvStream& rStream );

	void SetDBNumNum( BYTE nNum ) { aDBNum.nNum = nNum; }
	void SetDBNumLang( LanguageType eLang ) { aDBNum.eLang = eLang; }
	const SvNumberDBNum& GetDBNum() const { return aDBNum; }

private:
	ImpSvNumberformatInfo aI;			
	String sColorName;					
	Color* pColor;						
	USHORT nAnzStrings;					
	SvNumberDBNum aDBNum;				

};

class SvNumberformat
{
public:

	SvNumberformat( ImpSvNumberformatScan& rSc, LanguageType eLge );

	SvNumberformat( String& rString,
	   ImpSvNumberformatScan* pSc,
	   ImpSvNumberInputScan* pISc,
	   xub_StrLen& nCheckPos,
	   LanguageType& eLan,
	   BOOL bStand = FALSE );

	SvNumberformat( SvNumberformat& rFormat );

	SvNumberformat( SvNumberformat& rFormat, ImpSvNumberformatScan& rSc );

	~SvNumberformat();

	short GetType() const
		{ return (nNewStandardDefined &&
			(nNewStandardDefined <= SV_NUMBERFORMATTER_VERSION)) ?
			(eType & ~NUMBERFORMAT_DEFINED) : eType; }

	void SetType(const short eSetType)          { eType = eSetType; }

	void SetStandard()                          { bStandard = TRUE; }
	BOOL IsStandard() const						{ return bStandard; }

	void SetNewStandardDefined( USHORT nVer )
		{ nNewStandardDefined = nVer; eType |= NUMBERFORMAT_DEFINED; }

	USHORT GetNewStandardDefined() const		{ return nNewStandardDefined; }
	BOOL IsAdditionalStandardDefined() const
		{ return nNewStandardDefined == SV_NUMBERFORMATTER_VERSION_ADDITIONAL_I18N_FORMATS; }

	LanguageType GetLanguage() const            { return eLnge;}

	const String& GetFormatstring()	const	{ return sFormatstring; }

	String GetMappedFormatstring( const NfKeywordTable& rKeywords,
						const LocaleDataWrapper& rLoc,
						BOOL bDontQuote = FALSE ) const;

	void SetUsed(const BOOL b)					{ bIsUsed = b; }
	BOOL GetUsed() const						{ return bIsUsed; }
	BOOL IsStarFormatSupported() const			{ return bStarFlag; }
	void SetStarFormatSupport( BOOL b )			{ bStarFlag = b; }

	NfHackConversion Load( SvStream& rStream, ImpSvNumMultipleReadHeader& rHdr,
		SvNumberFormatter* pConverter, ImpSvNumberInputScan& rISc );
	void Save( SvStream& rStream, ImpSvNumMultipleWriteHeader& rHdr  ) const;

	static void LoadString( SvStream& rStream, String& rStr );

	BOOL GetOutputString( double fNumber, String& OutString, Color** ppColor );
	BOOL GetOutputString( String& sString, String& OutString, Color** ppColor );

	BOOL IsTextFormat() const { return (eType & NUMBERFORMAT_TEXT) != 0; }

	BOOL HasTextFormat() const
		{
			return (NumFor[3].GetnAnz() > 0) ||
	(NumFor[3].Info().eScannedType == NUMBERFORMAT_TEXT);
		}

	void GetFormatSpecialInfo(BOOL& bThousand,
				  BOOL& IsRed,
				  USHORT& nPrecision,
				  USHORT& nAnzLeading) const;

	USHORT GetFormatPrecision() const	{ return NumFor[0].Info().nCntPost; }

	USHORT GetLastInsertKey() const
		{ return NumFor[0].Info().nThousand; }
	void SetLastInsertKey(USHORT nKey)
		{ NumFor[0].Info().nThousand = nKey; }

	void ConvertLanguage( SvNumberFormatter& rConverter,
		LanguageType eConvertFrom, LanguageType eConvertTo, BOOL bSystem = FALSE );

	const String* GetNumForString( USHORT nNumFor, USHORT nPos,
			BOOL bString = FALSE ) const;

	short GetNumForType( USHORT nNumFor, USHORT nPos, BOOL bString = FALSE ) const;

	BOOL IsNegativeRealNegative() const
		{
			return fLimit1 == 0.0 && fLimit2 == 0.0 &&
			( (eOp1 == NUMBERFORMAT_OP_GE && eOp2 == NUMBERFORMAT_OP_NO) ||
	 (eOp1 == NUMBERFORMAT_OP_GT && eOp2 == NUMBERFORMAT_OP_LT) ||
	 (eOp1 == NUMBERFORMAT_OP_NO && eOp2 == NUMBERFORMAT_OP_NO) );
		}

	BOOL IsNegativeWithoutSign() const;

	BOOL HasNewCurrency() const;

	void Build50Formatstring( String& rStr ) const;

	static String StripNewCurrencyDelimiters( const String& rStr,
		BOOL bQuoteSymbol );

	BOOL GetNewCurrencySymbol( String& rSymbol, String& rExtension ) const;

	static BOOL HasStringNegativeSign( const String& rStr );

	static BOOL IsInQuote( const String& rString, xub_StrLen nPos,
			sal_Unicode cQuote = '"',
			sal_Unicode cEscIn = '\0', sal_Unicode cEscOut = '\\' );

	static xub_StrLen GetQuoteEnd( const String& rString, xub_StrLen nPos,
	sal_Unicode cQuote = '"',
	sal_Unicode cEscIn = '\0', sal_Unicode cEscOut = '\\' );

	void SetComment( const String& rStr )
#if NF_COMMENT_IN_FORMATSTRING
		{ SetComment( rStr, sFormatstring, sComment ); }
#else
		{ sComment = rStr; }
#endif
	const String& GetComment() const { return sComment; }

	static void EraseCommentBraces( String& rStr );

	static void SetComment( const String& rStr, String& rFormat, String& rComment );

	static void EraseComment( String& rStr );

	DateFormat GetDateOrder() const;	

	ImpSvNumberformatScan& ImpGetScan() const { return rScan; }

	void GetConditions( SvNumberformatLimitOps& rOper1, double& rVal1,
			SvNumberformatLimitOps& rOper2, double& rVal2 ) const;
	Color* GetColor( USHORT nNumFor ) const;
	void GetNumForInfo( USHORT nNumFor, short& rScannedType,
		BOOL& bThousand, USHORT& nPrecision, USHORT& nAnzLeading ) const;

	BYTE GetDBNumNum( USHORT nNumFor ) const
		{ return nNumFor <= 3 ? NumFor[nNumFor].GetDBNum().nNum : 0; }
	LanguageType GetDBNumLang( USHORT nNumFor ) const
		{ return nNumFor <= 3 ? NumFor[nNumFor].GetDBNum().eLang : LANGUAGE_DONTKNOW; }

private:
	static ::rtl::OUString sGregorian;	

	ImpSvNumFor NumFor[4];			
	String sFormatstring;			
	String sComment;				
	double fLimit1;					
	double fLimit2;					
	ImpSvNumberformatScan& rScan;	
	LanguageType eLnge;				
	SvNumberformatLimitOps eOp1;	
	SvNumberformatLimitOps eOp2;	
	USHORT nNewStandardDefined;		
	short eType;					
	BOOL bStarFlag;					
	BOOL bStandard;					
	BOOL bIsUsed;					

#ifdef _ZFORMAT_CXX		

	const CharClass& rChrCls() const		{ return rScan.GetChrCls(); }
	const LocaleDataWrapper& rLoc() const	{ return rScan.GetLoc(); }
	CalendarWrapper& GetCal() const			{ return rScan.GetCal(); }
	const SvNumberFormatter& GetFormatter() const   { return *rScan.GetNumberformatter(); }

	short ImpNextSymbol( String& rString,
		 xub_StrLen& nPos,
		 String& sSymbol );

	static xub_StrLen ImpGetNumber( String& rString,
	   xub_StrLen& nPos,
	   String& sSymbol );

	static LanguageType ImpGetLanguageType( const String& rString, xub_StrLen& nPos );

	void ImpGetOutputStandard( double& fNumber, String& OutString );

	void ImpGetOutputInputLine( double fNumber, String& OutString );

	short ImpCheckCondition(double& fNumber,
			 double& fLimit,
			 SvNumberformatLimitOps eOp);

	ULONG ImpGGT(ULONG x, ULONG y);
	ULONG ImpGGTRound(ULONG x, ULONG y);

	BOOL ImpNumberFill( String& sStr,
		double& rNumber,
		xub_StrLen& k,
		USHORT& j,
		USHORT nIx,
		short eSymbolType );

	BOOL ImpNumberFillWithThousands( String& sStr,
					 double& rNumber,
					 xub_StrLen k,
					 USHORT j,
					 USHORT nIx,
					 USHORT nDigCnt );

	void ImpDigitFill( String& sStr,
		xub_StrLen nStart,
		xub_StrLen& k,
		USHORT nIx,
		USHORT nThousandCnt );

	BOOL ImpGetDateOutput( double fNumber,
		   USHORT nIx,
		   String& OutString );
	BOOL ImpGetTimeOutput( double fNumber,
		   USHORT nIx,
		   String& OutString );
	BOOL ImpGetDateTimeOutput( double fNumber,
			   USHORT nIx,
			   String& OutString );

	BOOL IsOtherCalendar( const ImpSvNumFor& rNumFor ) const;

	void SwitchToOtherCalendar( String& rOrgCalendar, double& fOrgDateTime );

	void SwitchToGregorianCalendar( const String& rOrgCalendar, double fOrgDateTime );

	BOOL ImpGetNumberOutput( double fNumber,
			 USHORT nIx,
			 String& OutString );

	void CopyNumberformat( const SvNumberformat& rFormat );

	String GetDBNumString( const SvNumberDBNum& rNum, sal_Int32 nVal, USHORT nMinDigits = 0  ) const;
	String IntToString( USHORT nIx, sal_Int32 nVal, USHORT nMinDigits = 0 ) const
		{
			const SvNumberDBNum& rNum = NumFor[nIx].GetDBNum();
			if ( nMinDigits || rNum.IsComplete() )
	return GetDBNumString( rNum, nVal, nMinDigits );
			return String::CreateFromInt32( nVal );
		}

#endif 
};

#endif	
