

#ifndef _SVX_DLGCTL3D_HXX
#define _SVX_DLGCTL3D_HXX

#ifndef _SV_POLY_HXX
#include <vcl/poly.hxx>
#endif

#ifndef _SV_CTRL_HXX
#include <vcl/ctrl.hxx>
#endif

#ifndef _B3D_B3DGEOM_HXX
#include <goodies/b3dgeom.hxx>
#endif

#ifndef _B3D_B3DTRANS_HXX
#include <goodies/b3dtrans.hxx>
#endif

#ifndef _B3D_MATRIL3D_HXX
#include <goodies/matril3d.hxx>
#endif

#ifndef _B3D_B3DLIGHT_HXX
#include <goodies/b3dlight.hxx>
#endif

#ifndef _SV_SCRBAR_HXX
#include <vcl/scrbar.hxx>
#endif

#ifndef _SV_BUTTON_HXX
#include <vcl/button.hxx>
#endif

#ifndef _SFXITEMSET_HXX 
#include <svtools/itemset.hxx>
#endif

class Vector3D;
class FmFormModel;
class FmFormPage;
class E3dView;
class E3dPolyScene;
class E3dObject;

#define	PREVIEW_OBJECTTYPE_SPHERE			0x0000
#define	PREVIEW_OBJECTTYPE_CUBE  			0x0001

class Svx3DPreviewControl : public Control
{
protected:
	FmFormModel*		pModel;
	FmFormPage*			pFmPage;
	E3dView*			p3DView;
	E3dPolyScene*		pScene;
	E3dObject*			p3DObj;
	UINT16				nObjectType;

	void				Construct();

public:
		Svx3DPreviewControl( Window* pParent, const ResId& rResId );
		Svx3DPreviewControl( Window* pParent, WinBits nStyle = 0 );
		~Svx3DPreviewControl();

	virtual void 	Paint( const Rectangle& rRect );
	virtual void 	MouseButtonDown( const MouseEvent& rMEvt );
	virtual void	Resize();

	void			Reset();

	void			SetObjectType( UINT16 nType );
	UINT16			GetObjectType() const { return( nObjectType ); }

	SfxItemSet		Get3DAttributes() const;
	void			Set3DAttributes( const SfxItemSet& rAttr );

	void			Set3DObject( const E3dObject* pObj );
};

class SvxRectCtl3D : public Control
{
protected:
	long		nBW; 
	long		nBH; 
	USHORT		nRadius;
	Size		aSize;
	short		nActPoint;
	short		nOldPoint;
	short 		nDist;
	Point		aPointArray[20];
	Point 		aPolyPoints1[4];
	Point 		aPolyPoints2[4];
	Point 		aPolyPoints3[4];
	Polygon 	aPoly1;
	Polygon 	aPoly2;
	Polygon 	aPoly3;

	Vector3D	PointNumToVector( short nPoint );
	short 		VectorToPointNum( Vector3D aVect );

public:
	SvxRectCtl3D( Window* pParent,
	  const ResId& rResId,
	  USHORT nBorderW = 200,
	  USHORT nBorderH = 200,
	  USHORT nCircle = 80 );
	~SvxRectCtl3D();

	virtual void Paint( const Rectangle& rRect );
	virtual void MouseButtonDown( const MouseEvent& rMEvt );

	void		Reset();
	Vector3D	GetVector();
	void		SetPoint( Vector3D aVect );
};

#define	PREVIEW_NORMAL_MODE_OBJECT			0x0000
#define	PREVIEW_NORMAL_MODE_FLAT			0x0001
#define	PREVIEW_NORMAL_MODE_SPHERE			0x0002

#define	PREVIEW_SHADEMODE_FLAT				0x0000
#define	PREVIEW_SHADEMODE_PHONG				0x0001
#define	PREVIEW_SHADEMODE_GOURAUD			0x0002
#define	PREVIEW_SHADEMODE_DRAFT				0x0003

class SvxPreviewCtl3D : public Control
{
protected:

	B3dGeometry				aGeometry;

	B3dCamera				aCameraSet;
	double					fDistance;
	double					fDeviceSize;

	double					fRotateX;
	double					fRotateY;
	double					fRotateZ;

	B3dMaterial				aObjectMaterial;

	B3dLightGroup			aLights;

	UINT16					nHorSegs;
	UINT16					nVerSegs;

	UINT16					nNormalMode;

	UINT16					nShadeMode;

	BOOL					bGeometryCube;

public:
	SvxPreviewCtl3D( Window* pParent, const ResId& rResId);
	SvxPreviewCtl3D( Window* pParent, WinBits nStyle = 0);
	~SvxPreviewCtl3D();

	virtual void Paint( const Rectangle& rRect );
	void DrawGeometryClip(Base3D* pBase3D);
	virtual void DrawGeometry(Base3D* pBase3D);

	void SetGeometry(BOOL bGeomCube);

	void SetRotation(double fRotX, double fRotY, double fRotZ);
	void GetRotation(double& rRotX, double& rRotY, double& rRotZ);

	void SetMaterial(Color rNew,
		Base3DMaterialValue=Base3DMaterialAmbient);
	Color GetMaterial(Base3DMaterialValue=Base3DMaterialAmbient);
	void SetShininess(UINT16 nNew);
	UINT16 GetShininess();

	void SetLightGroup(B3dLightGroup* pNew=0L);
	B3dLightGroup* GetLightGroup() { return &aLights; }

	void SetUserDistance(double fNew);
	double GetUserDistance() { return fDistance; }
	void SetDeviceSize(double fNew);
	double GetDeviceSize() { return fDeviceSize; }

	UINT16 GetHorizontalSegments() { return nHorSegs; }
	UINT16 GetVerticalSegments() { return nVerSegs; }
	void SetHorizontalSegments(UINT16 nNew);
	void SetVerticalSegments(UINT16 nNew);
	void SetSegments(UINT16 nNewHor, UINT16 nNewVer);

	UINT16 GetNormalMode() { return nNormalMode; }
	void SetNormalMode(UINT16 nNew);

	UINT16 GetShadeMode() { return nShadeMode; }
	void SetShadeMode(UINT16 nNew);

protected:

	void CreateGeometry();

	void Init();
};

class SvxLightPrevievCtl3D : public SvxPreviewCtl3D
{
private:

	B3dGeometry			aLightGeometry;

	Base3DLightNumber	eSelectedLight;

	double				fObjectRadius;
	double				fDistanceToObject;
	double				fScaleSizeSelected;
	double				fLampSize;

	Link				aChangeCallback;
	Link				aSelectionChangeCallback;

	double				fSaveActionStartHor;
	double				fSaveActionStartVer;
	double				fSaveActionStartRotZ;
	Point				aActionStartPoint;

	INT32				nInteractionStartDistance;

	unsigned			bMouseMoved						: 1;
	unsigned			bGeometrySelected				: 1;

public:
	SvxLightPrevievCtl3D( Window* pParent, const ResId& rResId);
	SvxLightPrevievCtl3D( Window* pParent, WinBits nStyle = 0);
	~SvxLightPrevievCtl3D();

	void SelectLight(Base3DLightNumber=Base3DLightNone);
	Base3DLightNumber GetSelectedLight() { return eSelectedLight; }

	void SelectGeometry();
	BOOL IsGeometrySelected() { return bGeometrySelected; }

	void SetObjectRadius(double fNew);
	double GetObjectRadius() { return fObjectRadius; }

	void SetDistanceToObject(double fNew);
	double GetDistanceToObject() { return fDistanceToObject; }

	void SetScaleSizeSelected(double fNew);
	double GetScaleSizeSelected() { return fScaleSizeSelected; }

	void SetLampSize(double fNew);
	double GetLampSize() { return fLampSize; }

	virtual void DrawGeometry(Base3D* pBase3D);
	void DrawLightGeometry(Base3DLightNumber eLightNum, Base3D* pBase3D);

	BOOL IsSelectionValid();

	void GetPosition(double& rHor, double& rVer);
	void SetPosition(double fHor, double fVer);

	void SetChangeCallback(Link aNew) { aChangeCallback = aNew; }
	void SetSelectionChangeCallback(Link aNew) { aSelectionChangeCallback = aNew; }

	virtual void		MouseButtonDown( const MouseEvent& rMEvt );
	virtual void		Tracking( const TrackingEvent& rTEvt );

protected:

	void CreateLightGeometry();

	void TrySelection(Point aPosPixel);

	void Init();
};

class SvxLightCtl3D : public Control
{
private:

	SvxLightPrevievCtl3D	aLightControl;
	ScrollBar				aHorScroller;
	ScrollBar				aVerScroller;
	PushButton				aSwitcher;
	Vector3D				aVector;

	Link					aUserInteractiveChangeCallback;
	Link					aUserSelectionChangeCallback;

	unsigned				bVectorValid					: 1;
	unsigned				bSphereUsed						: 1;

public:
	SvxLightCtl3D( Window* pParent, const ResId& rResId);
	SvxLightCtl3D( Window* pParent, WinBits nStyle = 0);
	~SvxLightCtl3D();

	void SetVector(const Vector3D& rNew);
	const Vector3D& GetVector();
	BOOL GetVectorValid() { return bVectorValid; }

	virtual void		Resize();
	void NewLayout();

	void CheckSelection();

	SvxLightPrevievCtl3D& GetPreviewControl() { return aLightControl; }

	void SetUserInteractiveChangeCallback(Link aNew)
		{ aUserInteractiveChangeCallback = aNew; }
	void SetUserSelectionChangeCallback(Link aNew)
		{ aUserSelectionChangeCallback = aNew; }

protected:

	DECL_LINK( InternalInteractiveChange, void*);
	DECL_LINK( InternalSelectionChange, void*);
	DECL_LINK( ScrollBarMove, void*);
	DECL_LINK( ButtonPress, void*);

	void Init();
};

#endif 

