

#ifndef _MyEDITVIEW_HXX
#define _MyEDITVIEW_HXX

#ifndef _RSCSFX_HXX 
#include <rsc/rscsfx.hxx>
#endif
#ifndef _LANG_HXX 
#include <tools/lang.hxx>
#endif

#ifndef _TOOLS_COLOR_HXX
#include <tools/color.hxx>
#endif

class EditEngine;
class ImpEditEngine;
class ImpEditView;
class SvxSearchItem;
class SvxFieldItem;
class Window;
class Pointer;
class Cursor;
class KeyEvent;
class MouseEvent;
class DropEvent;
class CommandEvent;
class Rectangle;
class Link;
class Pair;
class Point;
class Range;
class SvStream;
class SvKeyValueIterator;
class SfxStyleSheet; 

#ifndef _EDITDATA_HXX
#include <svx/editdata.hxx>
#endif

class EditView
{
	friend class EditEngine;
	friend class ImpEditEngine;
	friend class EditSelFunctionSet;

public:	
	ImpEditView*	GetImpEditView() const 		{ return pImpEditView; }
	ImpEditEngine*	GetImpEditEngine() const;

private:
	ImpEditView*	pImpEditView;
			EditView( const EditView& );
	EditView&		operator=( const EditView& );

public:
		EditView( EditEngine* pEng, Window* pWindow );
	virtual			~EditView();

	void			SetEditEngine( EditEngine* pEditEngine );
	EditEngine*		GetEditEngine() const;

	void			SetWindow( Window* pWin );
	Window*			GetWindow() const;

	void			Paint( const Rectangle& rRect );
	void			Invalidate();
	Pair			Scroll( long nHorzScroll, long nVertScroll, BYTE nRangeCheck = RGCHK_NEG );

	void			ShowCursor( BOOL bGotoCursor = TRUE, BOOL bForceVisCursor = TRUE );
	void			HideCursor();

	EESelectionMode	GetSelectionMode() const;
	void			SetSelectionMode( EESelectionMode eMode );

	void			SetReadOnly( BOOL bReadOnly );
	BOOL			IsReadOnly() const;

	BOOL			HasSelection() const;
	ESelection		GetSelection() const;
	void			SetSelection( const ESelection& rNewSel );
	BOOL			SelectCurrentWord();

	void			IndentBlock();
	void			UnindentBlock();

	BOOL			IsInsertMode() const;
	void			SetInsertMode( BOOL bInsert );

	void			ReplaceSelected( const String& rStr );
	String			GetSelected();
	void			DeleteSelected();

	USHORT			GetSelectedScriptType() const;

	void				SetOutputArea( const Rectangle& rRec );
	const Rectangle&	GetOutputArea() const;

	void				SetVisArea( const Rectangle& rRec );
	const Rectangle&	GetVisArea() const;

	void			SetPointer( const Pointer& rPointer );
	const Pointer&	GetPointer() const;

	void			SetCursor( const Cursor& rCursor );
	Cursor*			GetCursor() const;

	void			InsertText( const String& rNew, BOOL bSelect = FALSE );

	BOOL			PostKeyEvent( const KeyEvent& rKeyEvent );

	BOOL			MouseButtonUp( const MouseEvent& rMouseEvent );
	BOOL			MouseButtonDown( const MouseEvent& rMouseEvent );
	BOOL			MouseMove( const MouseEvent& rMouseEvent );
	void			Command( const CommandEvent& rCEvt );

	BOOL    		Drop( const DropEvent& rEvt );
	BOOL    		QueryDrop( DropEvent& rEvt );
	ESelection		GetDropPos();

	void			Cut();
	void			Copy();
	void			Paste();
	void			PasteSpecial();

	void			EnablePaste( BOOL bEnable );
	BOOL			IsPasteEnabled() const;

	void			Undo();
	void			Redo();

	USHORT			GetParagraph( const Point& rMousePosPixel );
	Point			GetWindowPosTopLeft( USHORT nParagraph );
	void			MoveParagraphs( Range aParagraphs, USHORT nNewPos );

	const SfxItemSet& 	GetEmptyItemSet();
	SfxItemSet			GetAttribs();
	void				SetAttribs( const SfxItemSet& rSet );
	void				SetParaAttribs( const SfxItemSet& rSet, USHORT nPara );
	void				RemoveAttribs( BOOL bRemoveParaAttribs = FALSE, USHORT nWhich = 0 );
	void 				RemoveCharAttribs( USHORT nPara, USHORT nWhich = 0 );

	ULONG			Read( SvStream& rInput, EETextFormat eFormat, BOOL bSelect = FALSE, SvKeyValueIterator* pHTTPHeaderAttrs = NULL );
	ULONG			Write( SvStream& rOutput, EETextFormat eFormat );

	void 			SetBackgroundColor( const Color& rColor );
	Color			GetBackgroundColor() const;

	void			SetControlWord( ULONG nWord );
	ULONG			GetControlWord() const;

	EditTextObject*	CreateTextObject();
	void			InsertText( const EditTextObject& rTextObject );

	void			SetEditEngineUpdateMode( BOOL bUpdate );
	void			ForceUpdate();

	SfxStyleSheet*	GetStyleSheet() const;
	void			SetStyleSheet( SfxStyleSheet* pStyle );

	void			SetAnchorMode( EVAnchorMode eMode );
	EVAnchorMode	GetAnchorMode() const;

	BOOL			MatchGroup();

	void			CompleteAutoCorrect();

	EESpellState	StartSpeller( BOOL bMultipleDoc = FALSE );
	EESpellState	StartSpeller( LanguageType eLang, BOOL bMultipleDoc = FALSE );
	EESpellState	StartThesaurus();
	EESpellState	StartThesaurus( LanguageType eLang );
	USHORT			StartSearchAndReplace( const SvxSearchItem& rSearchItem );

	void			TransliterateText( sal_Int32 nTransliterationMode );

	BOOL			IsCursorAtWrongSpelledWord( BOOL bMarkIfWrong = FALSE );
	BOOL			IsWrongSpelledWordAtPos( const Point& rPosPixel, BOOL bMarkIfWrong = FALSE );
	void			SpellIgnoreWord();
	void			ExecuteSpellPopup( const Point& rPosPixel, Link* pCallBack = 0 );

	void 				InsertField( const SvxFieldItem& rFld );
	const SvxFieldItem*	GetFieldUnderMousePointer() const;
	const SvxFieldItem*	GetFieldUnderMousePointer( USHORT& nPara, xub_StrLen& nPos ) const;
	const SvxFieldItem*	GetField( const Point& rPos, USHORT* pnPara = NULL, xub_StrLen* pnPos = NULL ) const;

	const SvxFieldItem*	GetFieldAtSelection() const;

	String			GetWordUnderMousePointer() const;
	String			GetWordUnderMousePointer( Rectangle& rWordRect ) const;

	void			SetInvalidateMore( USHORT nPixel );
	USHORT			GetInvalidateMore() const;
};

#endif 
