

#ifndef _E3D_EXTRUD3D_HXX
#define _E3D_EXTRUD3D_HXX

#ifndef _E3D_OBJ3D_HXX
#include <svx/obj3d.hxx>
#endif

#ifndef _POLY3D_HXX
#include <svx/poly3d.hxx>
#endif

class E3dExtrudeObj : public E3dCompoundObject
{
private:

	PolyPolygon3D			aExtrudePolygon;
	double					fExtrudeScale;

	unsigned				bExtrudeSmoothed			: 1;
	unsigned				bExtrudeSmoothFrontBack		: 1;
	unsigned				bExtrudeCharacterMode		: 1;
	unsigned				bExtrudeCloseFront			: 1;
	unsigned				bExtrudeCloseBack			: 1;

protected:
	void SetDefaultAttributes(E3dDefaultAttributes& rDefault);
	PolyPolygon3D GetFrontSide();
	PolyPolygon3D GetBackSide(const PolyPolygon3D& rFrontSide);

public:
	TYPEINFO();

	E3dExtrudeObj(E3dDefaultAttributes& rDefault, const PolyPolygon& rPP, double fDepth);

	E3dExtrudeObj(E3dDefaultAttributes& rDefault, const XPolyPolygon& rXPP, double fDepth);
	E3dExtrudeObj();

	sal_uInt16 GetPercentDiagonal() const 
		{ return ((const Svx3DPercentDiagonalItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_PERCENT_DIAGONAL)).GetValue(); }

	sal_uInt16 GetPercentBackScale() const 
		{ return ((const Svx3DBackscaleItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_BACKSCALE)).GetValue(); }

	sal_uInt32 GetExtrudeDepth() const 
		{ return ((const Svx3DDepthItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_DEPTH)).GetValue(); }

	virtual UINT16 GetObjIdentifier() const;

	virtual void CreateWireframe(Polygon3D& rWirePoly, const Matrix4D* pTf = NULL,
		E3dDragDetail eDetail = E3DDETAIL_DEFAULT);

	virtual void operator=(const SdrObject&);
	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);

	virtual void TakeObjNameSingul(String& rName) const;
	virtual void TakeObjNamePlural(String& rName) const;

	virtual void CreateGeometry();

	virtual void GetLineGeometry(PolyPolygon3D& rLinePolyPolygon) const;

	void SetExtrudePolygon(const PolyPolygon3D &rNew);
	const PolyPolygon3D &GetExtrudePolygon() { return aExtrudePolygon; }

	void SetExtrudeScale(double fNew);
	double GetExtrudeScale() const { return fExtrudeScale; }

	void SetExtrudeSmoothed(BOOL bNew);
	BOOL GetExtrudeSmoothed() const { return bExtrudeSmoothed; }

	void SetExtrudeSmoothFrontBack(BOOL bNew);
	BOOL GetExtrudeSmoothFrontBack() const { return bExtrudeSmoothFrontBack; }

	void SetExtrudeCharacterMode(BOOL bNew);
	BOOL GetExtrudeCharacterMode() const { return bExtrudeCharacterMode; }

	void SetExtrudeCloseFront(BOOL bNew);
	BOOL GetExtrudeCloseFront() const { return bExtrudeCloseFront; }

	void SetExtrudeCloseBack(BOOL bNew);
	BOOL GetExtrudeCloseBack() const { return bExtrudeCloseBack; }

	virtual void PostItemChange(const sal_uInt16 nWhich);

	virtual BOOL IsBreakObjPossible();
	virtual SdrAttrObj* GetBreakObj();
};

#endif			

