
#ifndef _SVX_FMSHELL_HXX
#define _SVX_FMSHELL_HXX

#ifndef _SFX_SHELL_HXX 
#include <sfx2/shell.hxx>
#endif

#ifndef _SFXMODULE_HXX 
#include <sfx2/module.hxx>
#endif

#ifndef _SV_EVENT_HXX 
#include <vcl/event.hxx>
#endif

#ifndef _SVX_SVXIDS_HRC
#include <svx/svxids.hrc>
#endif

#ifndef _SVX_FMVIEW_HXX
#include <svx/fmview.hxx>
#endif

#include <svx/ifaceids.hxx>

class FmFormModel;
class FmFormPage;
class SvxFmTabWin;
class FmXFormShell;
class FmFormView;

class FmDesignModeChangedHint : public SfxHint
{
	sal_Bool m_bDesignMode;

public:
	TYPEINFO();
	FmDesignModeChangedHint( sal_Bool bDesMode );
	virtual ~FmDesignModeChangedHint();

	sal_Bool GetDesignMode() const { return m_bDesignMode; }
};

class FmFormShell : public SfxShell
{
	friend class FmFormView;
	friend class FmXFormShell;

	sal_uInt16	m_nLastSlot;
	sal_Bool	m_bDesignMode : 1;
	sal_Bool	m_bHasForms	: 1;	

	FmXFormShell*	m_pImpl;
	FmFormView*		m_pFormView;
	FmFormModel*	m_pFormModel;
	SfxViewShell*	m_pParentShell;

	void NotifyMarkListChanged(FmFormView*);

#ifdef NOOLDSV
	class FormShellWaitObject
	{
		Window*	m_pWindow;
	public:
		FormShellWaitObject(const FmFormShell* _pShell);
		~FormShellWaitObject();
	};
	friend class FormShellWaitObject;

	const OutputDevice* GetCurrentViewDevice() const { return m_pFormView ? m_pFormView->GetActualOutDev() : NULL; }
#endif

public:
	SFX_DECL_INTERFACE(SVX_INTERFACE_FORM_SH);
	TYPEINFO();

	FmFormShell(SfxViewShell* pParent, FmFormView* pView = NULL);
	virtual ~FmFormShell();

	virtual void Execute( SfxRequest& );
	virtual void GetState( SfxItemSet& );
	virtual sal_Bool HasUIFeature( sal_uInt32 nFeature );

	sal_Bool GetY2KState(sal_uInt16& nReturn);
	void SetY2KState(sal_uInt16 n);

	void SetView(FmFormView* pView);

	FmFormView*  GetFormView() const { return m_pFormView; }
	FmFormModel* GetFormModel() const { return m_pFormModel; }
	FmFormPage*  GetCurPage() const;
	FmXFormShell* GetImpl() const {return m_pImpl;};

	sal_uInt16 PrepareClose(sal_Bool bUI = sal_True, sal_Bool bForBrowsing = sal_False);
	sal_Bool IsDesignMode() const {return m_bDesignMode;}

	virtual void	Activate(sal_Bool bMDI);
	virtual void	Deactivate(sal_Bool bMDI);

protected:
	void GetFormState(SfxItemSet &rSet, sal_uInt16 nWhich);

	void DetermineForms(sal_Bool bInvalidate);
	void SetDesignMode( sal_Bool bDesign);
};

#endif          
