
#ifndef _SVX_FONTWORK_HXX
#define _SVX_FONTWORK_HXX

#ifndef _TOOLBOX_HXX 
#include <vcl/toolbox.hxx>
#endif
#ifndef _FIXED_HXX 
#include <vcl/fixed.hxx>
#endif
#ifndef _VALUESET_HXX 
#include <svtools/valueset.hxx>
#endif
#ifndef _SFXDOCKWIN_HXX 
#include <sfx2/dockwin.hxx>
#endif
#ifndef _SFXCTRLITEM_HXX 
#include <sfx2/ctrlitem.hxx>
#endif
#ifndef _XENUM_HXX
#include <svx/xenum.hxx>
#endif
#ifndef _SVX_DLG_CTRL_HXX
#include <svx/dlgctrl.hxx>
#endif

class SdrView;
class SdrPageView;
class SdrObject;

class XFormTextAdjustItem;
class XFormTextDistanceItem;
class XFormTextStartItem;
class XFormTextMirrorItem;
class XFormTextStdFormItem;
class XFormTextHideFormItem;
class XFormTextOutlineItem;
class XFormTextShadowItem;
class XFormTextShadowColorItem;
class XFormTextShadowXValItem;
class XFormTextShadowYValItem;

class SvxFontWorkDialog;

class SvxFontWorkControllerItem : public SfxControllerItem
{
	SvxFontWorkDialog  &rFontWorkDlg;

protected:
	virtual void StateChanged(USHORT nSID, SfxItemState eState,
				  const SfxPoolItem* pState);

public:
	SvxFontWorkControllerItem(USHORT nId, SvxFontWorkDialog&, SfxBindings&);
};

class SvxFontWorkChildWindow : public SfxChildWindow
{
 public:
	SvxFontWorkChildWindow(Window*, USHORT, SfxBindings*, SfxChildWinInfo*);
	SFX_DECL_CHILDWINDOW(SvxFontWorkChildWindow);
};

class SvxFontWorkDialog : public SfxDockingWindow
{
 #define CONTROLLER_COUNT 12

	SvxFontWorkControllerItem* pCtrlItems[CONTROLLER_COUNT];

	ValueSet		aFormSet;

	ToolBox			aTbxStyle;
	ToolBox			aTbxAdjust;

	FixedBitmap		aFbDistance;
	MetricField		aMtrFldDistance;
	FixedBitmap		aFbTextStart;
	MetricField		aMtrFldTextStart;

	ToolBox			aTbxShadow;

	FixedBitmap		aFbShadowX;
	MetricField		aMtrFldShadowX;
	FixedBitmap		aFbShadowY;
	MetricField		aMtrFldShadowY;

	ColorLB			aShadowColorLB;

	SfxBindings&	rBindings;
	Timer			aInputTimer;
	BOOL			bUserZoomedIn;

	USHORT			nLastStyleTbxId;
	USHORT			nLastAdjustTbxId;
	USHORT			nLastShadowTbxId;
	long			nSaveShadowX;
	long			nSaveShadowY;
	long			nSaveShadowAngle;
	long			nSaveShadowSize;

	const XColorTable* pColorTable;

#ifdef _SVX_FONTWORK_CXX
 friend class SvxFontWorkChildWindow;
 friend class SvxFontWorkControllerItem;

	DECL_LINK( SelectStyleHdl_Impl, void * );
	DECL_LINK( SelectAdjustHdl_Impl, void * );
	DECL_LINK( SelectShadowHdl_Impl, void * );

	DECL_LINK( ModifyInputHdl_Impl, void * );
	DECL_LINK( InputTimoutHdl_Impl, void * );

	DECL_LINK( FormSelectHdl_Impl, void * );
	DECL_LINK( ColorSelectHdl_Impl, void * );

	void SetStyle_Impl(const XFormTextStyleItem*);
	void SetAdjust_Impl(const XFormTextAdjustItem*);
	void SetDistance_Impl(const XFormTextDistanceItem*);
	void SetStart_Impl(const XFormTextStartItem*);
	void SetMirror_Impl(const XFormTextMirrorItem*);
	void SetStdForm_Impl(const XFormTextStdFormItem*);
	void SetShowForm_Impl(const XFormTextHideFormItem*);
	void SetOutline_Impl(const XFormTextOutlineItem*);
	void SetShadow_Impl(const XFormTextShadowItem*,
			BOOL bRestoreValues = FALSE);
	void SetShadowColor_Impl(const XFormTextShadowColorItem*);
	void SetShadowXVal_Impl(const XFormTextShadowXValItem*);
	void SetShadowYVal_Impl(const XFormTextShadowYValItem*);
#endif

 protected:
	virtual void	Zoom();
	virtual SfxChildAlignment CheckAlignment( SfxChildAlignment eActAlign,
								  SfxChildAlignment eAlign );

 public:
	SvxFontWorkDialog(	SfxBindings *pBindinx,
			SfxChildWindow *pCW,
			Window* pParent,
			const ResId& rResId );
	~SvxFontWorkDialog();

	void SetColorTable(const XColorTable* pTable);
	void SetActive(BOOL bActivate = TRUE);

	void CreateStdFormObj(SdrView& rView, SdrPageView& rPV,
			  const SfxItemSet& rAttr, SdrObject& rOldObj,
			  XFormTextStdForm eForm);
};

#endif		

