
#ifndef _SVX_GRIDCTRL_HXX
#define _SVX_GRIDCTRL_HXX

#ifndef _LIST_HXX 
#include <tools/list.hxx>
#endif

#ifndef _COM_SUN_STAR_SDBC_XROWSET_HPP_
#include <com/sun/star/sdbc/XRowSet.hpp>
#endif
#ifndef _COM_SUN_STAR_BEANS_PROPERTYCHANGEEVENT_HPP_
#include <com/sun/star/beans/PropertyChangeEvent.hpp>
#endif
#ifndef _COM_SUN_STAR_UTIL_XNUMBERFORMATTER_HPP_
#include <com/sun/star/util/XNumberFormatter.hpp>
#endif
#ifndef _COM_SUN_STAR_UTIL_DATE_HPP_
#include <com/sun/star/util/Date.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XINDEXACCESS_HPP_
#include <com/sun/star/container/XIndexAccess.hpp>
#endif
#ifndef _SV_FIXED_HXX 
#include <vcl/fixed.hxx>
#endif

#ifndef _SV_FIELD_HXX 
#include <vcl/field.hxx>
#endif

#ifndef _SV_BUTTON_HXX 
#include <vcl/button.hxx>
#endif

#ifndef _DATE_HXX
#include <tools/date.hxx>
#endif

#ifndef _REF_HXX 
#include <tools/ref.hxx>
#endif

#ifndef _SVTOOLS_EDITBROWSEBOX_HXX_
#include <svtools/editbrowsebox.hxx>
#endif

#ifndef _OSL_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif

#ifndef _COMPHELPER_PROPERTY_MULTIPLEX_HXX_
#include <comphelper/propmultiplex.hxx>
#endif

#ifndef _TRANSFER_HXX
#include <svtools/transfer.hxx>
#endif

extern XubString INVALIDTEXT;
extern XubString OBJECTTEXT;

class DbGridControl;
class CursorWrapper;

sal_Bool CompareBookmark(const ::com::sun::star::uno::Any& aLeft, const ::com::sun::star::uno::Any& aRight);

class DataColumn;
DECLARE_LIST(DbDataColumns, DataColumn*);

enum GridRowStatus
{
	GRS_CLEAN,
	GRS_MODIFIED,
	GRS_DELETED,
	GRS_INVALID
};

class DbGridRow : public SvRefBase
{
	::com::sun::star::uno::Any						m_aBookmark;		
	DbDataColumns				m_aVariants;
	GridRowStatus				m_eStatus;
	sal_Bool						m_bIsNew;

public:
	DbGridRow():m_eStatus(GRS_CLEAN), m_bIsNew(sal_True) { }
	DbGridRow(CursorWrapper* pCur, sal_Bool bPaintCursor);
	void SetState(CursorWrapper* pCur, sal_Bool bPaintCursor);

	~DbGridRow();	

	sal_Bool HasField(sal_uInt32 nPos) const {return nPos < m_aVariants.Count();}
	const DataColumn& GetField(sal_uInt32 nPos) const { return *m_aVariants.GetObject(nPos); }

	void			SetStatus(GridRowStatus _eStat) { m_eStatus = _eStat; }
	GridRowStatus	GetStatus() const				{ return m_eStatus; }
	void			SetNew(sal_Bool _bNew)				{ m_bIsNew = _bNew; }
	sal_Bool			IsNew() const					{ return m_bIsNew; }

	const ::com::sun::star::uno::Any& GetBookmark() const { return m_aBookmark; }

	sal_Bool	IsValid() const { return m_eStatus == GRS_CLEAN || m_eStatus == GRS_MODIFIED; }
	sal_Bool	IsModified() const { return m_eStatus == GRS_MODIFIED; }
};

SV_DECL_REF(DbGridRow);

class DbGridColumn;
DECLARE_LIST(DbGridColumns, DbGridColumn*);

class FmGridSelectionListener
{
public:
	virtual void selectionChanged() = 0;
};

class FmXGridSourcePropListener;
class DisposeListenerGridBridge;
typedef ::svt::EditBrowseBox	DbGridControl_Base;
class DbGridControl : public DbGridControl_Base
{
	friend class FmXGridSourcePropListener;
	friend class GridFieldValueListener;
	friend class DisposeListenerGridBridge;

public:

	class NavigationBar: public Control
	{
		class AbsolutePos : public NumericField
		{
		public:
			AbsolutePos(Window* pParent, WinBits nStyle = 0);
	virtual void KeyInput(const KeyEvent& rEvt);
			virtual void LoseFocus();
		};

		friend class NavigationBar::AbsolutePos;

		FixedText		m_aRecordText;
		AbsolutePos		m_aAbsolute;			
		FixedText		m_aRecordOf;
		FixedText		m_aRecordCount;

		ImageButton		m_aFirstBtn;			
		ImageButton		m_aPrevBtn;			
		ImageButton		m_aNextBtn;			
		ImageButton		m_aLastBtn;			
		ImageButton		m_aNewBtn;			
		sal_uInt16			m_nDefaultWidth;
		sal_uInt32			m_nCurrentPos;

		sal_Bool			m_bPositioning;		

	public:

		enum State
		{
			RECORD_TEXT	= 1,
			RECORD_ABSOLUTE,
			RECORD_OF,
			RECORD_COUNT,
			RECORD_FIRST,
			RECORD_NEXT,
			RECORD_PREV,
			RECORD_LAST,
			RECORD_NEW
		};

		NavigationBar(Window* pParent, WinBits nStyle = 0);

		void InvalidateAll(sal_uInt32 nCurrentPos = -1, sal_Bool bAll = sal_False);
		void InvalidateState(sal_uInt16 nWhich) {SetState(nWhich);}
		void SetState(sal_uInt16 nWhich);
		sal_Bool GetState(sal_uInt16 nWhich) const;
		sal_uInt16 GetDefaultWidth() const {return m_nDefaultWidth;}

	protected:
		virtual void Resize();
		virtual void Paint(const Rectangle& rRect);
		virtual void StateChanged( StateChangedType nType );

	private:
		DECL_LINK(OnClick, Button*);
		sal_uInt16 ArrangeControls();

		void PositionDataSource(sal_Int32 nRecord);
	};

	friend class DbGridControl::NavigationBar;

public:

	enum Option
	{
		OPT_READONLY	= 0x00,
		OPT_INSERT		= 0x01,
		OPT_UPDATE		= 0x02,
		OPT_DELETE		= 0x04
	};

private:
	Font			m_aDefaultFont;		  
	Link			m_aMasterStateProvider;
	Link			m_aMasterSlotExecutor;

	::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter > 		m_xFormatter;
	::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >	m_xServiceFactory;

	DbGridColumns	m_aColumns;			
	NavigationBar	m_aBar;
	DbGridRowRef	m_xDataRow;			

	DbGridRowRef	m_xSeekRow,			
			m_xEmptyRow;		

	sal_uInt32			m_nAsynAdjustEvent;

	::comphelper::OPropertyChangeMultiplexer*		m_pDataSourcePropMultiplexer;
	FmXGridSourcePropListener*						m_pDataSourcePropListener;

	void*											m_pFieldListeners;

	DisposeListenerGridBridge*						m_pCursorDisposeListener;

	FmGridSelectionListener*						m_pSelectionListener;

protected:
	CursorWrapper*	m_pDataCursor;		
	CursorWrapper*  m_pSeekCursor;		

private:  

	DbGridRowRef		m_xCurrentRow;		
	DbGridRowRef		m_xPaintRow;		
	sal_Int32			m_nSeekPos;			
	sal_Int32			m_nTotalCount;		

	osl::Mutex			m_aDestructionSafety;
	osl::Mutex			m_aAdjustSafety;

	com::sun::star::util::Date
			m_aNullDate;		

	BrowserMode			m_nMode;
	sal_Int32			m_nCurrentPos;		
	sal_uInt32			m_nDeleteEvent;		
	sal_uInt16			m_nOptions;			

	sal_uInt16			m_nOptionMask;		

	sal_Bool			m_bFrozen : 1;			
	sal_Bool			m_bDesignMode : 1;		
	sal_Bool			m_bRecordCountFinal : 1;
	sal_Bool			m_bMultiSelection   : 1;
	sal_Bool			m_bNavigationBar	  : 1;

	sal_Bool			m_bSynchDisplay : 1;
	sal_Bool			m_bForceROController : 1;
	sal_Bool			m_bHandle : 1;
	sal_Bool			m_bFilterMode : 1;
	sal_Bool			m_bWantDestruction : 1;
	sal_Bool			m_bInAdjustDataSource : 1;
	sal_Bool			m_bPendingAdjustRows : 1;	

protected:
	sal_Bool			m_bUpdating : 1;			

protected:
	virtual sal_Bool SeekRow(long nRow);
	virtual void VisibleRowsChanged( long nNewTopRow, sal_uInt16 nNumRows);
	virtual void PaintStatusCell(OutputDevice& rDev, const Rectangle& rRect) const;
	virtual void PaintCell(OutputDevice& rDev, const Rectangle& rRect, sal_uInt16 nColId) const;
	virtual RowStatus GetRowStatus(long nRow) const;
	virtual sal_Bool CursorMoving(long nNewRow, sal_uInt16 nNewCol);
	virtual void CursorMoved();
	virtual void ArrangeControls(sal_uInt16& nX, sal_uInt16 nY);
	virtual sal_uInt32 GetTotalCellWidth(long nRow, sal_uInt16 nColId);
	virtual void Command(const CommandEvent& rEvt);
	virtual long PreNotify(NotifyEvent& rEvt);
	virtual void KeyInput(const KeyEvent& rEvt);
	virtual void StateChanged( StateChangedType nType );
	virtual void DataChanged( const DataChangedEvent& rDCEvt );
	virtual void Select();

	virtual ::svt::CellController* GetController(long nRow, sal_uInt16 nCol);

	virtual void CellModified();
	virtual sal_Bool SaveModified();
	virtual sal_Bool IsModified() const;

	virtual sal_uInt16 AppendColumn(const String& rName, sal_uInt16 nWidth = 0, sal_uInt16 nPos = HEADERBAR_APPEND, sal_uInt16 nId = (sal_uInt16)-1);
	virtual void RemoveColumn(sal_uInt16 nId);
	virtual DbGridColumn* CreateColumn(sal_uInt16 nId, const DbGridRowRef& _xRow) const;
	virtual void ColumnMoved(sal_uInt16 nId);
	virtual sal_Bool SaveRow();
	virtual sal_Bool IsTabAllowed(sal_Bool bForward) const;

	virtual void	HideColumn(sal_uInt16 nId);

	virtual void	ShowColumn(sal_uInt16 nId);

	virtual void PreExecuteRowContextMenu(sal_uInt16 nRow, PopupMenu& rMenu);

	virtual void PostExecuteRowContextMenu(sal_uInt16 nRow, const PopupMenu& rMenu, sal_uInt16 nExecutionResult);

	virtual void DataSourcePropertyChanged(const ::com::sun::star::beans::PropertyChangeEvent& evt) throw(::com::sun::star::uno::RuntimeException);

	virtual void FieldValueChanged(sal_uInt16 _nId, const ::com::sun::star::beans::PropertyChangeEvent& _evt);
	virtual void FieldListenerDisposing(sal_uInt16 _nId);

	virtual void disposing(sal_uInt16 _nId, const ::com::sun::star::lang::EventObject& _rEvt);

	virtual void StartDrag( sal_Int8 nAction, const Point& rPosPixel );

public:
	DbGridControl(
		::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >,
		Window* pParent,
		WinBits nBits = WB_BORDER);
	DbGridControl(
		::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >,
		Window* pParent,
		const ResId& rId);

	virtual ~DbGridControl();

	virtual void Init();
	virtual void InitColumnsByFields(const ::com::sun::star::uno::Reference< ::com::sun::star::container::XIndexAccess >& xFields) = 0;
	virtual void RemoveRows();

	void RemoveRows(sal_Bool bNewCursor);

	void InvalidateStatus();

	const ::com::sun::star::uno::Reference< ::com::sun::star::util::XNumberFormatter >& getNumberFormatter() const {return m_xFormatter;}

	virtual void setDataSource(const ::com::sun::star::uno::Reference< ::com::sun::star::sdbc::XRowSet >& rCursor,
		sal_uInt16 nOpts = OPT_INSERT | OPT_UPDATE | OPT_DELETE);
	virtual void Dispatch(sal_uInt16 nId);

	CursorWrapper* getDataSource() const {return m_pDataCursor;}
	const DbGridColumns& GetColumns() const {return m_aColumns;}

	void EnableHandle(sal_Bool bEnable);
	sal_Bool HasHandle() const {return m_bHandle;}
	void InsertHandleColumn();

	sal_uInt16 GetViewColumnPos( sal_uInt16 nId ) const { sal_uInt16 nPos = GetColumnPos(nId); return (nPos==(sal_uInt16)-1) ? -1 : nPos-1; }

	sal_uInt16 GetModelColumnPos( sal_uInt16 nId ) const;

	sal_uInt16 GetViewColCount() const { return ColCount() - 1; }
	sal_uInt16 GetModelColCount() const { return (sal_uInt16)m_aColumns.Count(); }

	sal_uInt16 GetColumnIdFromViewPos( sal_uInt16 nPos ) const { return GetColumnId(nPos + 1); }
	sal_uInt16 GetColumnIdFromModelPos( sal_uInt16 nPos ) const;

	virtual void SetDesignMode(sal_Bool bMode);
	sal_Bool IsDesignMode() const {return m_bDesignMode;}
	sal_Bool IsOpen() const {return m_pSeekCursor != NULL;}

	virtual void SetFilterMode(sal_Bool bMode);
	sal_Bool IsFilterMode() const {return m_bFilterMode;}
	sal_Bool IsFilterRow(long nRow) const {return m_bFilterMode && nRow == 0;}

	void EnableNavigationBar(sal_Bool bEnable);
	sal_Bool HasNavigationBar() const {return m_bNavigationBar;}

	sal_uInt16 GetOptions() const {return m_nOptions;}
	NavigationBar& GetNavigationBar() {return m_aBar;}
	sal_uInt16 SetOptions(sal_uInt16 nOpt);

	void SetMultiSelection(sal_Bool bMulti);
	sal_Bool GetMultiSelection() const {return m_bMultiSelection;}

	const com::sun::star::util::Date&	getNullDate() const {return m_aNullDate;}

	void MoveToPosition(sal_uInt32 nPos);
	void MoveToFirst();
	void MoveToNext();
	void MoveToPrev();
	void MoveToLast();
	void AppendNew();

	void AdjustDataSource(sal_Bool bFull = sal_False);
	void Undo();

	virtual void BeginCursorAction();
	virtual void EndCursorAction();

	sal_Bool IsUpdating() const {return m_bUpdating;}

	virtual void RowRemoved( long nRow, long nNumRows = 1, sal_Bool bDoPaint = sal_True );
	virtual void RowInserted( long nRow, long nNumRows = 1, sal_Bool bDoPaint = sal_True );
	virtual void RowModified( long nRow, sal_uInt16 nColId = USHRT_MAX );

	void resetCurrentRow();

	sal_Bool getDisplaySynchron() const { return m_bSynchDisplay; }
	void setDisplaySynchron(sal_Bool bSync);
	void forceSyncDisplay();

	sal_Bool isForcedROController() const { return m_bForceROController; }
	void forceROController(sal_Bool bForce);

	const DbGridRowRef& GetCurrentRow() const {return m_xCurrentRow;}

	void SetStateProvider(const Link& rProvider) { m_aMasterStateProvider = rProvider; }

	void SetSlotExecutor(const Link& rExecutor) { m_aMasterSlotExecutor = rExecutor; }

	void EnablePermanentCursor(sal_Bool bEnable);
	sal_Bool IsPermanentCursorEnabled() const;

	::com::sun::star::uno::Reference< ::com::sun::star::lang::XMultiServiceFactory >
		getServiceManager() const { return m_xServiceFactory; }

	sal_Bool	canCopyCellText(sal_Int32 _nRow, sal_Int16 _nColId);

	void		copyCellText(sal_Int32 _nRow, sal_Int16 _nColId);

	FmGridSelectionListener*	getSelectionListener() const { return m_pSelectionListener; }
	void						setSelectionListener(FmGridSelectionListener* _pListener) { m_pSelectionListener = _pListener; }

	struct GrantCellControlAccess
	{
		friend class DbCellControl;
	protected:
		GrantCellControlAccess() { }
	};

	void refreshController(sal_uInt16 _nColId, GrantCellControlAccess _aAccess);

protected:
	void RecalcRows(long nNewTopRow, sal_uInt16 nLinesOnScreen, sal_Bool bUpdateCursor);
	sal_Bool SeekCursor(long nRow, sal_Bool bAbsolute = sal_False);
	void RemoveColumns();		
	void AdjustRows();
	sal_Int32 AlignSeekCursor();
	sal_Bool SetCurrent(long nNewRow, sal_Bool bCancelInsert);

	String GetCellText(DbGridColumn* pCol) const;
	virtual void DeleteSelectedRows();
	sal_Bool IsValid(const DbGridRowRef& _xRow) const {return _xRow && _xRow->IsValid();}

	sal_Bool IsCurrentAppending() const;

	sal_Bool IsEmptyRow(long nRow) const;

	void  SetSeekPos(sal_Int32 nPos) {m_nSeekPos = nPos;}
	sal_Int32 GetCurrentPos() const {return m_nCurrentPos;}
	sal_Int32 GetSeekPos() const {return m_nSeekPos;}
	sal_Int32 GetTotalCount() const {return m_nTotalCount;}

	const DbGridRowRef& GetEmptyRow() const {return m_xEmptyRow;}
	const DbGridRowRef& GetSeekRow() const {return m_xSeekRow;}
	CursorWrapper* GetSeekCursor() const {return m_pSeekCursor;}

	void ConnectToFields();
	void DisconnectFromFields();

	void implAdjustInSolarThread(sal_Bool _bRows);

protected:
	virtual void InitController(::svt::CellControllerRef& rController, long nRow, sal_uInt16 nCol);
	void Construct();
	void ImplInitSettings( sal_Bool bFont, sal_Bool bForeground, sal_Bool bBackground );
	DECL_LINK(OnDelete, void*);

	DECL_LINK(OnAsyncAdjust, void*);

};

SV_IMPL_REF(DbGridRow);

#endif 

