

#ifndef _E3D_LATHE3D_HXX
#define _E3D_LATHE3D_HXX

#ifndef _E3D_OBJ3D_HXX
#include <svx/obj3d.hxx>
#endif

#ifndef _E3D_POLY3D_HXX
#include <svx/poly3d.hxx>
#endif

class E3dLatheObj : public E3dCompoundObject
{

	enum { LATHE_PART_STD = 1, LATHE_PART_COVER = 2 };
	PolyPolygon3D	aPolyPoly3D;
	double			fLatheScale;

	unsigned		bLatheSmoothed				: 1;
	unsigned		bLatheSmoothFrontBack		: 1;
	unsigned		bLatheCharacterMode			: 1;
	unsigned		bLatheCloseFront			: 1;
	unsigned		bLatheCloseBack				: 1;

 protected:
	void SetDefaultAttributes(E3dDefaultAttributes& rDefault);

	PolyPolygon3D CreateLathePolyPoly(PolyPolygon3D& rPoly3D, long nVSegs);
	Polygon3D CreateLathePoly(Polygon3D& rPoly3D, long nVSegs);

 public:
	TYPEINFO();
	E3dLatheObj(E3dDefaultAttributes& rDefault, const PolyPolygon& rPoly);

	E3dLatheObj(E3dDefaultAttributes& rDefault, const XPolyPolygon& rXPoly);
	E3dLatheObj(E3dDefaultAttributes& rDefault, const XPolygon& rXPoly);
	E3dLatheObj(E3dDefaultAttributes& rDefault, const PolyPolygon3D rPoly3D);
	E3dLatheObj();

	sal_Int32 GetHorizontalSegments() const 
		{ return ((const Svx3DHorizontalSegmentsItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_HORZ_SEGS)).GetValue(); }

	sal_Int32 GetVerticalSegments() const 
		{ return ((const Svx3DVerticalSegmentsItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_VERT_SEGS)).GetValue(); }

	sal_uInt16 GetPercentDiagonal() const 
		{ return ((const Svx3DPercentDiagonalItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_PERCENT_DIAGONAL)).GetValue(); }

	sal_uInt16 GetBackScale() const 
		{ return ((const Svx3DBackscaleItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_BACKSCALE)).GetValue(); }

	sal_uInt32 GetEndAngle() const 
		{ return ((const Svx3DEndAngleItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_END_ANGLE)).GetValue(); }

	virtual UINT16 GetObjIdentifier() const;

	virtual void CreateWireframe(Polygon3D& rWirePoly, const Matrix4D* pTf = NULL,
		E3dDragDetail eDetail = E3DDETAIL_DEFAULT);

	const   PolyPolygon3D& GetPolyPolygon() const { return aPolyPoly3D; }
	void    ReSegment(long nHSegs, long nVSegs);

	virtual void operator=(const SdrObject&);
	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);

	virtual SdrObject* DoConvertToPolyObj(BOOL bBezier) const;

	virtual void CreateGeometry();

	virtual void GetLineGeometry(PolyPolygon3D& rLinePolyPolygon) const;

	virtual void TakeObjNameSingul(String& rName) const;
	virtual void TakeObjNamePlural(String& rName) const;

	void SetPolyPoly3D(const PolyPolygon3D& rNew);
	const PolyPolygon3D& GetPolyPoly3D() { return aPolyPoly3D; }

	void SetLatheScale(double fNew);
	double GetLatheScale() const { return fLatheScale; }

	void SetLatheSmoothed(BOOL bNew);
	BOOL GetLatheSmoothed() const { return bLatheSmoothed; }

	void SetLatheSmoothFrontBack(BOOL bNew);
	BOOL GetLatheSmoothFrontBack() const { return bLatheSmoothFrontBack; }

	void SetLatheCharacterMode(BOOL bNew);
	BOOL GetLatheCharacterMode() const { return bLatheCharacterMode; }

	void SetLatheCloseFront(BOOL bNew);
	BOOL GetLatheCloseFront() const { return bLatheCloseFront; }

	void SetLatheCloseBack(BOOL bNew);
	BOOL GetLatheCloseBack() const { return bLatheCloseBack; }

	virtual void PostItemChange(const sal_uInt16 nWhich);

	virtual BOOL IsBreakObjPossible();
	virtual SdrAttrObj* GetBreakObj();
};

#endif			

