
#ifndef _SVX_LRSPITEM_HXX
#define _SVX_LRSPITEM_HXX

#ifndef _SFXPOOLITEM_HXX 
#include <svtools/poolitem.hxx>
#endif
#ifndef _SVX_SVXIDS_HRC
#include <svx/svxids.hrc>
#endif

#ifdef _USE_NAMESPACE
namespace rtl
{
#endif
	class OUString;
#ifdef _USE_NAMESPACE
}
#endif

#define	LRSPACE_16_VERSION		((USHORT)0x0001)
#define	LRSPACE_TXTLEFT_VERSION	((USHORT)0x0002)
#define	LRSPACE_AUTOFIRST_VERSION ((USHORT)0x0003)
#define	LRSPACE_NEGATIVE_VERSION ((USHORT)0x0004)

class SvxLRSpaceItem : public SfxPoolItem
{
	short	nFirstLineOfst;     
	long	nTxtLeft;           
	long	nLeftMargin;        
	long	nRightMargin;		

	USHORT	nPropFirstLineOfst, nPropLeftMargin, nPropRightMargin;
	BOOL 	bAutoFirst	: 1;	
	BOOL 	bBulletFI 	: 1;	

	void   AdjustLeft();		

public:
	TYPEINFO();

	SvxLRSpaceItem( const USHORT nId = ITEMID_LRSPACE );
	SvxLRSpaceItem( const long nLeft, const long nRight,
		const long nTLeft = 0, const short nOfset = 0,
		const USHORT nId = ITEMID_LRSPACE );
	inline SvxLRSpaceItem& operator=( const SvxLRSpaceItem &rCpy );

	virtual int 			 operator==( const SfxPoolItem& ) const;

	virtual	sal_Bool        	 QueryValue( com::sun::star::uno::Any& rVal, BYTE nMemberId = 0 ) const;
	virtual	sal_Bool			 PutValue( const com::sun::star::uno::Any& rVal, BYTE nMemberId = 0 );

	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
	                                String &rText, const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*	 Clone( SfxItemPool *pPool = 0 ) const;
	virtual SfxPoolItem*	 Create(SvStream &, USHORT) const;
	virtual SvStream&		 Store(SvStream &, USHORT nItemVersion ) const;
	virtual USHORT			 GetVersion( USHORT nFileVersion ) const;
	virtual int				 ScaleMetrics( long nMult, long nDiv );
	virtual	int				 HasMetrics() const;

	inline void   SetLeft ( const long nL, const USHORT nProp = 100 );
	inline void   SetRight( const long nR, const USHORT nProp = 100 );

	inline long GetLeft()  const { return nLeftMargin; }
	inline long GetRight() const { return nRightMargin;}
	inline void SetLeftValue( const long nL ) { nTxtLeft = nLeftMargin = nL; }
	inline void SetRightValue( const long nR ) { nRightMargin = nR; }
	inline BOOL IsAutoFirst()  const { return bAutoFirst; }
	inline void SetAutoFirst( const BOOL bNew ) { bAutoFirst = bNew; }

	inline void SetPropLeft( const USHORT nProp = 100 )
		{ nPropLeftMargin = nProp; }
	inline void SetPropRight( const USHORT nProp = 100 )
		{ nPropRightMargin = nProp;}
	inline USHORT GetPropLeft()  const { return nPropLeftMargin; }
	inline USHORT GetPropRight() const { return nPropRightMargin;}

	inline void SetTxtLeft( const long nL, const USHORT nProp = 100 );
	inline long GetTxtLeft() const { return nTxtLeft; }

	inline void   SetTxtFirstLineOfst( const short nF, const USHORT nProp = 100 );
	inline short  GetTxtFirstLineOfst() const { return nFirstLineOfst; }
	inline void SetPropTxtFirstLineOfst( const USHORT nProp = 100 )
		{ nPropFirstLineOfst = nProp; }
	inline USHORT GetPropTxtFirstLineOfst() const
		{ return nPropFirstLineOfst; }
	inline void SetTxtFirstLineOfstValue( const short nValue )
		{ nFirstLineOfst = nValue; }

	void		SetBulletFI( BOOL b ) { bBulletFI = b; }
	BOOL		IsBulletFI() const { return bBulletFI; }
};

inline SvxLRSpaceItem &SvxLRSpaceItem::operator=( const SvxLRSpaceItem &rCpy )
{
	nFirstLineOfst = rCpy.nFirstLineOfst;
	nTxtLeft = rCpy.nTxtLeft;
	nLeftMargin = rCpy.nLeftMargin;
	nRightMargin = rCpy.nRightMargin;
	nPropFirstLineOfst = rCpy.nPropFirstLineOfst;
	nPropLeftMargin = rCpy.nPropLeftMargin;
	nPropRightMargin = rCpy.nPropRightMargin;
	bBulletFI = rCpy.bBulletFI;
	bAutoFirst = rCpy.bAutoFirst;
	return *this;
}

inline void SvxLRSpaceItem::SetLeft( const long nL, const USHORT nProp )
{
	nLeftMargin = (nL * nProp) / 100;
	nTxtLeft = nLeftMargin;
	nPropLeftMargin = nProp;
}
inline void SvxLRSpaceItem::SetRight( const long nR, const USHORT nProp )
{
	nRightMargin = (nR * nProp) / 100;
	nPropRightMargin = nProp;
}
inline void SvxLRSpaceItem::SetTxtFirstLineOfst( const short nF,
									 const USHORT nProp )
{
	nFirstLineOfst = short((long(nF) * nProp ) / 100);
	nPropFirstLineOfst = nProp;
	AdjustLeft();
}

inline void SvxLRSpaceItem::SetTxtLeft( const long nL, const USHORT nProp )
{
	nTxtLeft = (nL * nProp) / 100;
	nPropLeftMargin = nProp;
	AdjustLeft();
}

#endif

