

#ifndef _E3D_OBJ3D_HXX
#define _E3D_OBJ3D_HXX

#ifndef _SVDOATTR_HXX 
#include <svx/svdoattr.hxx>
#endif

#ifndef _SVDOBJ_HXX 
#include <svx/svdobj.hxx>
#endif

#ifndef _VOLUME3D_HXX
#include <svx/volume3d.hxx>
#endif

#ifndef _SVX_MATRIX3D_HXX
#include <svx/matrix3d.hxx>
#endif

#ifndef _DEF3D_HXX
#include <svx/def3d.hxx>
#endif

#ifndef _SVDPAGE_HXX
#include <svx/svdpage.hxx>
#endif

#ifndef _E3D_DEFLT3D_HXX
#include <svx/deflt3d.hxx>
#endif

#ifndef _SV_BITMAP_HXX
#include <vcl/bitmap.hxx>
#endif

#ifndef _B3D_B3DGEOM_HXX
#include <goodies/b3dgeom.hxx>
#endif

#ifndef _B3D_MATRIL3D_HXX
#include <goodies/matril3d.hxx>
#endif

#ifndef _B3D_B3DTEX_HXX
#include <goodies/b3dtex.hxx>
#endif

#ifndef _B3D_B3DLIGHT_HXX
#include <goodies/b3dlight.hxx>
#endif

#ifndef _B3D_BASE3D_HXX
#include <goodies/base3d.hxx>
#endif

#ifndef _SVX3DITEMS_HXX
#include <svx/svx3ditems.hxx>
#endif

#ifndef _SVX_XFLCLIT_HXX
#include <svx/xflclit.hxx>
#endif

class SfxPoolItem;
class Viewport3D;
class E3dLightList;
class E3dScene;
class E3dPolyScene;
class Base3D;
class PolyPolygon3D;

#define	E3D_DRAWFLAG_FILLED						0x0001
#define	E3D_DRAWFLAG_OUTLINE					0x0002
#define	E3D_DRAWFLAG_TRANSPARENT				0x0004
#define	E3D_DRAWFLAG_GHOSTED					0x0008

class E3DObjGeoData : public SdrObjGeoData
{
public:
	Volume3D					aLocalBoundVol;		
	Matrix4D					aTfMatrix;			

	E3DObjGeoData() {}
};

class E3dObjList : public SdrObjList
{
public:
	TYPEINFO();
	E3dObjList(SdrModel* pNewModel, SdrPage* pNewPage, E3dObjList* pNewUpList=NULL);
	E3dObjList(const E3dObjList& rSrcList);
	virtual ~E3dObjList();

	virtual void NbcInsertObject(SdrObject* pObj, ULONG nPos=CONTAINER_APPEND,
		const SdrInsertReason* pReason=NULL);
	virtual SdrObject* NbcRemoveObject(ULONG nObjNum);
	virtual SdrObject* RemoveObject(ULONG nObjNum);
};

class E3dObject : public SdrAttrObj
{

	friend class E3dObjList;
	friend class E3dDragMethod;

 protected:
	E3dObjList*		pSub;				

	Volume3D		aBoundVol;			
	Volume3D		aLocalBoundVol;		
	Matrix4D		aTfMatrix;			
	Matrix4D		aFullTfMatrix;		

	UINT16			nLogicalGroup;		
	UINT16			nObjTreeLevel;		
	UINT16			nPartOfParent;		

	E3dDragDetail	eDragDetail;		

	unsigned		bTfHasChanged			: 1;
	unsigned		bBoundVolValid			: 1;
	unsigned		bIsSelected				: 1;

 public:
	virtual void SetBoundVolInvalid();

 protected:
	virtual void SetTransformChanged();
	virtual void NewObjectInserted(const E3dObject* p3DObj);
	virtual void StructureChanged(const E3dObject* p3DObj);
	virtual void ForceDefaultAttr();
	virtual void RecalcBoundVolume();

	BOOL    ImpCheckSubRecords (const SdrObjIOHeader& rHead,
					SvStream&             rIn);
	void	SetObjTreeLevel(USHORT nNewLevel);
	void	ImpCreateWireframePoly(XPolyPolygon& rXPP,
					   E3dDragDetail eDetail = E3DDETAIL_DEFAULT);

	SfxItemPool* ImpGetItemPool(const SfxPoolItem* pAlternateItem1,
					const SfxPoolItem* pAlternateItem2) const;
	SfxItemPool* ImpGetItemPool() const;
	const SfxPoolItem* ImpSetNewAttr(const SfxPoolItem* pAkt,
						 const SfxPoolItem* pNew,
						 FASTBOOL bChg = TRUE);

 public:
	TYPEINFO();
	E3dObject();
	E3dObject(BOOL bIsFromChart);

	virtual void RecalcSnapRect();
	virtual void RecalcBoundRect();
	virtual void SetRectsDirty(FASTBOOL bNotMyself=FALSE);

	virtual ~E3dObject();

	virtual UINT32	GetObjInventor() const;
	virtual UINT16	GetObjIdentifier() const;

	virtual void	TakeObjInfo(SdrObjTransformInfoRec& rInfo) const;

	virtual SdrLayerID	GetLayer() const;
	virtual void		NbcSetLayer(SdrLayerID nLayer);

	virtual void		SetObjList(SdrObjList* pNewObjList);
	virtual void		SetPage(SdrPage* pNewPage);
	virtual void		SetModel(SdrModel* pNewModel);
	virtual void		NbcMove(const Size& rSize);
	virtual void NbcResize(const Point& rRef, const Fraction& xFact, const Fraction& yFact);

	virtual SdrObjList* GetSubList() const;

	virtual void	TakeXorPoly(XPolyPolygon& rPoly, FASTBOOL bDetail) const;
	virtual USHORT	GetHdlCount() const;
	virtual void	AddToHdlList(SdrHdlList& rHdlList) const;
	virtual FASTBOOL HasSpecialDrag() const;

	virtual void Paint3D(ExtOutputDevice& rOut, Base3D* pBase3D,
		const SdrPaintInfoRec& rInfoRec, UINT16 nDrawFlags=0);

	virtual void TakeContour3D(XPolyPolygon& rPoly);

	virtual void DrawShadows(Base3D *pBase3D, ExtOutputDevice& rXOut,
		const Rectangle& rBound, const Volume3D& rVolume,
		const SdrPaintInfoRec& rInfoRec);

	virtual void Insert3DObj(E3dObject* p3DObj);
	void Remove3DObj(E3dObject* p3DObj);

	E3dObject* GetParentObj() const;
	virtual E3dScene* GetScene() const;

	const Volume3D& GetLocalBoundVolume() { return aLocalBoundVol; }
	virtual const Volume3D& GetBoundVolume();
	Vector3D GetCenter();

	virtual void GetFullTransform(Matrix4D& rMatrix) const;
	virtual const Matrix4D& GetFullTransform();

	virtual const Matrix4D& GetTransform() const;
	virtual void NbcSetTransform(const Matrix4D& rMatrix);
	virtual void NbcResetTransform();
	virtual void SetTransform(const Matrix4D& rMatrix);
	virtual void ResetTransform();

	virtual void NbcTranslate(const Vector3D& rTrans);
	virtual void Translate(const Vector3D& rTrans);

	virtual void NbcScaleX	(double fSx);
	virtual void NbcScaleY	(double fSy);
	virtual void NbcScaleZ	(double fSz);
	virtual void NbcScale	(double fSx, double fSy, double fSz);
	virtual void NbcScale	(double fS);

	virtual void ScaleX	(double fSx);
	virtual void ScaleY	(double fSy);
	virtual void ScaleZ	(double fSz);
	virtual void Scale	(double fSx, double fSy, double fSz);
	virtual void Scale	(double fS);

	virtual void NbcRotateX(double fAng);
	virtual void NbcRotateY(double fAng);
	virtual void NbcRotateZ(double fAng);

	virtual void RotateX(double fAng);
	virtual void RotateY(double fAng);
	virtual void RotateZ(double fAng);

	virtual void NbcRotate(const Point& rRef, long nWink, double sn, double cs);

	virtual void CreateWireframe(Polygon3D& rWirePoly, const Matrix4D* pTf = NULL,
		E3dDragDetail eDetail = E3DDETAIL_DEFAULT);

	virtual void TakeObjNameSingul(String& rName) const;
	virtual void TakeObjNamePlural(String& rName) const;

	USHORT GetObjTreeLevel() const { return nObjTreeLevel; }

	FASTBOOL HasLogicalGroup() { return ( nLogicalGroup > 0 ); }
	USHORT GetLogicalGroup() { return nLogicalGroup; }
	void SetLogicalGroup(USHORT nGroup);

	void SetDragDetail(E3dDragDetail eDetail)	{ eDragDetail = eDetail; }
	E3dDragDetail GetDragDetail()				{ return eDragDetail; }

	void	SetPartOfParent(UINT16 nPartCode = 1) { nPartOfParent = nPartCode; }
	UINT16	GetPartCode()	{ return nPartOfParent; }
	FASTBOOL IsPartOfParent() const { return (nPartOfParent != 0); }

	virtual void operator=(const SdrObject&);

	virtual const SfxItemSet& GetItemSet() const;
	virtual SfxItemSet* CreateNewItemSet(SfxItemPool& rPool);

	virtual void ItemChange(const sal_uInt16 nWhich, const SfxPoolItem* pNewItem = 0);
	virtual void ItemSetChanged(const SfxItemSet& rSet);

	virtual void NbcSetStyleSheet(SfxStyleSheet* pNewStyleSheet, FASTBOOL bDontRemoveHardAttr);

	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);
	virtual void AfterRead();

	virtual SdrObjGeoData *NewGeoData() const;
	virtual void          SaveGeoData(SdrObjGeoData& rGeo) const;
	virtual void          RestGeoData(const SdrObjGeoData& rGeo);

	void ForceDefaultAttrAgain ();

	void ReadOnlyOwnMembers(const SdrObjIOHeader& rHead, SvStream& rIn);
	void WriteOnlyOwnMembers(SvStream& rOut) const;

	BOOL GetSelected() { return bIsSelected; }
	void SetSelected(BOOL bNew);

	virtual BOOL IsBreakObjPossible();
	virtual SdrAttrObj* GetBreakObj();
};

class E3dCompoundObject : public E3dObject
{
	virtual void ForceDefaultAttr();

protected:

	B3dGeometry				aDisplayGeometry;

	Color					aMaterialAmbientColor;
	B3dMaterial				aBackMaterial;

	unsigned				bCreateNormals				: 1;
	unsigned				bCreateTexture				: 1;

	unsigned				bUseDifferentBackMaterial	: 1;

	unsigned				bBytesLeft					: 1;

	unsigned				bCreateE3dPolyObj			: 1;

	unsigned				bGeometryValid				: 1;

protected:

	void GrowPoly(
		PolyPolygon3D& rPolyPolyGrow,
		PolyPolygon3D& rPolyPolyNormals,
		double fFactor);

	void RotatePoly(
		PolyPolygon3D& rPolyPolyRotate,
		Matrix4D& rRotMat);

	void ScalePoly(
		PolyPolygon3D& rPolyPolyScale,
		double fFactor);

	void AddNormals(
		PolyPolygon3D& rPolyPolyDest,
		const PolyPolygon3D& rPolyPolySource);

	void CreateFront(
		const PolyPolygon3D& rPolyPoly3D,
		const PolyPolygon3D& rFrontNormals,
		BOOL bCreateNormals = TRUE,
		BOOL bCreateTexture = TRUE);
	void AddFrontNormals(
		const PolyPolygon3D& rPolyPoly3D,
		PolyPolygon3D& rNormalsFront,
		Vector3D& rOffset);

	void CreateBack(
		const PolyPolygon3D& rPolyPoly3D,
		const PolyPolygon3D& rBackNormals,
		BOOL bCreateNormals = TRUE,
		BOOL bCreateTexture = TRUE);
	void AddBackNormals(
		const PolyPolygon3D& rPolyPoly3D,
		PolyPolygon3D& rNormalsBack,
		Vector3D& rOffset);

	void CreateInBetween(
		const PolyPolygon3D& rPolyPolyFront,
		const PolyPolygon3D& rPolyPolyBack,
		const PolyPolygon3D& rFrontNormals,
		const PolyPolygon3D& rBackNormals,
		BOOL bCreateNormals = TRUE,
		double fSurroundFactor=1.0,
		double fTextureStart=0.0,
		double fTextureDepth=1.0,
		BOOL bRotateTexture90=FALSE);
	void AddInBetweenNormals(
		const PolyPolygon3D& rPolyPolyFront,
		const PolyPolygon3D& rPolyPolyBack,
		PolyPolygon3D& rNormals,
		BOOL bSmoothed = TRUE);

	void AddGeometry(
		const PolyPolygon3D& rPoly3D,
		BOOL bHintIsComplex=TRUE,
		BOOL bOutline=FALSE);
	void AddGeometry(
		const PolyPolygon3D& rPoly3D,
		const PolyPolygon3D& rVector3D,
		BOOL bHintIsComplex=TRUE,
		BOOL bOutline=FALSE);
	void AddGeometry(
		const PolyPolygon3D& rPoly3D,
		const PolyPolygon3D& rVector3D,
		const PolyPolygon3D& rNormal3D,
		BOOL bHintIsComplex=TRUE,
		BOOL bOutline=FALSE);
	void StartCreateGeometry();

	void CreateSegment(
		const PolyPolygon3D& rFront,		
		const PolyPolygon3D& rBack,			
		const PolyPolygon3D* pPrev = 0L,	
		const PolyPolygon3D* pNext = 0L,	
		BOOL bCreateFront = TRUE,			
		BOOL bCreateBack = TRUE,			
		double fPercentDiag = 0.05,			
		BOOL bSmoothLeft = TRUE,			
		BOOL bSmoothRight = TRUE,			
		BOOL bSmoothFrontBack = FALSE,		
		double fSurroundFactor = 1.0,		
		double fTextureStart = 0.0,			
		double fTextureDepth = 1.0,			
		BOOL bCreateTexture = TRUE,
		BOOL bCreateNormals = TRUE,
		BOOL bCharacterExtrude = FALSE,		
		BOOL bRotateTexture90 = FALSE		
		);

	void SetDefaultAttributes(E3dDefaultAttributes& rDefault);

	XPolyPolygon TransformToScreenCoor(const PolyPolygon3D &rExtrudePoly);

public :
	TYPEINFO();

	E3dCompoundObject();
	E3dCompoundObject(E3dDefaultAttributes& rDefault);
	virtual ~E3dCompoundObject();

	BOOL GetDoubleSided() const
		{ return ((const Svx3DDoubleSidedItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_DOUBLE_SIDED)).GetValue(); }

	sal_uInt16 GetNormalsKind() const
		{ return ((const Svx3DNormalsKindItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_NORMALS_KIND)).GetValue(); }

	BOOL GetNormalsInvert() const
		{ return ((const Svx3DNormalsInvertItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_NORMALS_INVERT)).GetValue(); }

	sal_uInt16 GetTextureProjectionX() const
		{ return ((const Svx3DTextureProjectionXItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_TEXTURE_PROJ_X)).GetValue(); }

	sal_uInt16 GetTextureProjectionY() const
		{ return ((const Svx3DTextureProjectionYItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_TEXTURE_PROJ_Y)).GetValue(); }

	BOOL GetShadow3D() const
		{ return ((const Svx3DShadow3DItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_SHADOW_3D)).GetValue(); }

	Color GetMaterialColor() const
		{ return ((const Svx3DMaterialColorItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_MAT_COLOR)).GetValue(); }

	Color GetMaterialEmission() const
		{ return ((const Svx3DMaterialEmissionItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_MAT_EMISSION)).GetValue(); }

	Color GetMaterialSpecular() const
		{ return ((const Svx3DMaterialSpecularItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_MAT_SPECULAR)).GetValue(); }

	sal_uInt16 GetMaterialSpecularIntensity() const
		{ return ((const Svx3DMaterialSpecularIntensityItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_MAT_SPECULAR_INTENSITY)).GetValue(); }

	Base3DTextureKind GetTextureKind() const
		{ return (Base3DTextureKind)((const Svx3DTextureKindItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_TEXTURE_KIND)).GetValue(); }

	Base3DTextureMode GetTextureMode() const
		{ return (Base3DTextureMode)((const Svx3DTextureModeItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_TEXTURE_MODE)).GetValue(); }

	BOOL GetTextureFilter() const
		{ return ((const Svx3DTextureFilterItem&)GetUnmergedItemSet().Get(SDRATTR_3DOBJ_TEXTURE_FILTER)).GetValue(); }

	virtual UINT16 GetObjIdentifier() const;
	virtual void RecalcSnapRect();
	virtual void RecalcBoundRect();
	virtual const Volume3D& GetBoundVolume();

	virtual void PostItemChange(const sal_uInt16 nWhich);

	virtual SdrObject* CheckHit(const Point& rPnt, USHORT nTol,
		const SetOfByte* pVisiLayer) const;

	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);

	virtual void Paint3D(ExtOutputDevice& rOut, Base3D* pBase3D,
		const SdrPaintInfoRec& rInfoRec, UINT16 nDrawFlags=0);

	virtual void TakeContour3D(XPolyPolygon& rPoly);

	virtual void DrawShadows(Base3D *pBase3D, ExtOutputDevice& rXOut,
		const Rectangle& rBound, const Volume3D& rVolume,
		const SdrPaintInfoRec& rInfoRec);
	void GetShadowPolygon(PolyPolygon& rPoly);
	void DrawShadowPolygon(PolyPolygon& rPoly, ExtOutputDevice& rXOut);

	AlphaMask GetAlphaMask(const SfxItemSet& rSet, const Size& rSizePixel);
	Bitmap GetGradientBitmap(const SfxItemSet&);
	Bitmap GetHatchBitmap(const SfxItemSet&);

	void DestroyGeometry();
	virtual void CreateGeometry();
	void ReCreateGeometry(BOOL bCreateOldGeometry=FALSE);

	virtual void GetLineGeometry(PolyPolygon3D& rLinePolyPolygon) const;

	BOOL GetCreateNormals() const { return bCreateNormals; }
	void SetCreateNormals(BOOL bNew);

	BOOL GetCreateTexture() const { return bCreateTexture; }
	void SetCreateTexture(BOOL bNew);

	BOOL AreBytesLeft() const { return bBytesLeft; }

	virtual void operator=(const SdrObject&);

	void SetBase3DParams(ExtOutputDevice& rOut, Base3D*,
		BOOL& bDrawObject, BOOL& bDrawOutline, UINT16 nDrawFlags, BOOL bGhosted,
		BOOL bIsLineDraft = FALSE, BOOL bIsFillDraft = FALSE);
private:
	void ImpSet3DParForFill(ExtOutputDevice& rOut, Base3D* pBase3D,
		BOOL& bDrawObject, UINT16 nDrawFlags, BOOL bGhosted, BOOL bIsFillDraft);
	void ImpSet3DParForLine(ExtOutputDevice& rOut, Base3D* pBase3D,
		BOOL& bDrawOutline, UINT16 nDrawFlags, BOOL bGhosted, BOOL bIsLineDraft);
public:

	B3dGeometry& GetDisplayGeometry();

	void CenterObject(const Vector3D& rCenter = Vector3D());

	Color GetShadowColor();
	BOOL DrawShadowAsOutline();
	INT32 GetShadowXDistance();
	INT32 GetShadowYDistance();
	UINT16 GetShadowTransparence();
	BOOL DoDrawShadow();

	void DrawObjectWireframe(ExtOutputDevice& rOut);

private:
	void SetInvertNormals(BOOL bNew);
public:

	const Color& GetMaterialAmbientColor() const { return aMaterialAmbientColor; }
	void SetMaterialAmbientColor(const Color& rColor);

	const B3dMaterial& GetBackMaterial() const { return aBackMaterial; }
	void SetBackMaterial(const B3dMaterial& rNew);
	BOOL GetUseDifferentBackMaterial() const { return bUseDifferentBackMaterial; }
	void SetUseDifferentBackMaterial(BOOL bNew);
};

#endif			
