
#ifndef _SVX_RULRITEM_HXX
#define _SVX_RULRITEM_HXX

#ifndef _GEN_HXX 
#include <tools/gen.hxx>
#endif
#ifndef _SFXPOOLITEM_HXX 
#include <svtools/poolitem.hxx>
#endif

class SvxLongLRSpaceItem : public SfxPoolItem
{
	long	lLeft;         
	long	lRight;        

  protected:

	virtual int 			 operator==( const SfxPoolItem& ) const;

	virtual String			 GetValueText() const;
	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
	                                String &rText, const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*	 Clone( SfxItemPool *pPool = 0 ) const;

private:
	const SvxLongLRSpaceItem& operator=(const SvxLongLRSpaceItem &); 

public:
	TYPEINFO();
	SvxLongLRSpaceItem(long lLeft, long lRight, USHORT nId);
	SvxLongLRSpaceItem(const SvxLongLRSpaceItem &);

	long    GetLeft() const { return lLeft; }
	long    GetRight() const { return lRight; }
	void    SetLeft(long lArgLeft) {lLeft=lArgLeft;}
	void    SetRight(long lArgRight) {lRight=lArgRight;}
};

class SvxLongULSpaceItem : public SfxPoolItem
{
	long	lLeft;         
	long	lRight;        

  protected:

	virtual int 			 operator==( const SfxPoolItem& ) const;

	virtual String			 GetValueText() const;
	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
	                                String &rText, const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*	 Clone( SfxItemPool *pPool = 0 ) const;

private:
	const SvxLongULSpaceItem& operator=(const SvxLongULSpaceItem &); 

public:
	TYPEINFO();
	SvxLongULSpaceItem(long lUpper, long lLower, USHORT nId);
	SvxLongULSpaceItem(const SvxLongULSpaceItem &);

	long    GetUpper() const { return lLeft; }
	long    GetLower() const { return lRight; }
	void    SetUpper(long lArgLeft) {lLeft=lArgLeft;}
	void    SetLower(long lArgRight) {lRight=lArgRight;}
};

class SvxPagePosSizeItem : public SfxPoolItem
{
	Point aPos;
	long lWidth;
	long lHeight;
protected:
	virtual int 			 operator==( const SfxPoolItem& ) const;

	virtual String			 GetValueText() const;
	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
	                                String &rText, const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*	 Clone( SfxItemPool *pPool = 0 ) const;

private:
	const SvxPagePosSizeItem& operator=(const SvxPagePosSizeItem &); 
public:
	TYPEINFO();
	SvxPagePosSizeItem(const Point &rPos, long lWidth, long lHeight);
	SvxPagePosSizeItem(const SvxPagePosSizeItem &);

	const Point &GetPos() const { return aPos; }
	long    GetWidth() const { return lWidth; }
	long    GetHeight() const { return lHeight; }
};

struct SvxColumnDescription
{
	USHORT nStart;					   
	USHORT nEnd;					   
	BOOL   bVisible;				   
	SvxColumnDescription():
		nStart(0), nEnd(0), bVisible(TRUE) {}
	SvxColumnDescription(const SvxColumnDescription &rCopy) :
		nStart(rCopy.nStart), nEnd(rCopy.nEnd), bVisible(rCopy.bVisible) {}
	SvxColumnDescription(USHORT start, USHORT end, BOOL bVis = TRUE):
		nStart(start), nEnd(end), bVisible(bVis) {}
	int operator==(const SvxColumnDescription &rCmp) const {
		return nStart == rCmp.nStart &&
			bVisible == rCmp.bVisible &&
			nEnd == rCmp.nEnd;
	}
	int operator!=(const SvxColumnDescription &rCmp) const {
		return !operator==(rCmp);
	}
	USHORT GetWidth() const { return nEnd - nStart; }
};

typedef SvPtrarr SvxColumns;

class SvxColumnItem : public SfxPoolItem
{
	SvxColumns aColumns;
	long	nLeft,		
	 nRight;		

	USHORT nActColumn;	
	BOOL    bTable;		
	BOOL	bOrtho;     

	void DeleteAndDestroyColumns();

protected:
	virtual int 			 operator==( const SfxPoolItem& ) const;

	virtual String			 GetValueText() const;
	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
	                                String &rText, const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*	 Clone( SfxItemPool *pPool = 0 ) const;
public:
	TYPEINFO();

	SvxColumnItem(USHORT nAct = 0); 
	SvxColumnItem(USHORT nActCol,
	  USHORT nLeft, USHORT nRight = 0);	
	SvxColumnItem(const	SvxColumnItem &);
	~SvxColumnItem();

	const SvxColumnItem &operator=(const SvxColumnItem &);

	USHORT Count() const { return aColumns.Count(); }
	SvxColumnDescription &operator[](USHORT i)
		{ return *(SvxColumnDescription*)aColumns[i]; }
	const SvxColumnDescription &operator[](USHORT i) const
		{ return *(SvxColumnDescription*)aColumns[i]; }
	void Insert(const SvxColumnDescription &rDesc, USHORT nPos) {
		SvxColumnDescription* pDesc = new SvxColumnDescription(rDesc);
		aColumns.Insert(pDesc, nPos);
	}
	void   Append(const SvxColumnDescription &rDesc) { Insert(rDesc, Count()); }
	void   SetLeft(long left) { nLeft = left; }
	void   SetRight(long right) { nRight = right; }
	void   SetActColumn(USHORT nCol) { nActColumn = nCol; }

	USHORT GetActColumn() const { return nActColumn; }
	BOOL   IsFirstAct() const { return nActColumn == 0; }
	BOOL   IsLastAct() const { return nActColumn == Count()-1; }
	long GetLeft() { return nLeft; }
	long GetRight() { return nRight; }

	BOOL   IsTable() const { return bTable; }

	BOOL   CalcOrtho() const;
	void   SetOrtho(BOOL bVal) { bOrtho = bVal; }
	BOOL   IsOrtho () const { return FALSE ; }

	BOOL IsConsistent() const  { return nActColumn < aColumns.Count(); }
	long   GetVisibleRight() const;
};

class SvxObjectItem : public SfxPoolItem
{
private:
	long   nStartX;					   
	long   nEndX;					   
	long   nStartY;                    
	long   nEndY;                      
	BOOL   bLimits;					   
protected:
	virtual int 			 operator==( const SfxPoolItem& ) const;

	virtual String			 GetValueText() const;
	virtual SfxItemPresentation GetPresentation( SfxItemPresentation ePres,
						SfxMapUnit eCoreMetric,
						SfxMapUnit ePresMetric,
	                                String &rText, const IntlWrapper * = 0 ) const;

	virtual SfxPoolItem*	 Clone( SfxItemPool *pPool = 0 ) const;
private:
	const SvxObjectItem &operator=(const SvxObjectItem &); 
public:
	TYPEINFO();
	SvxObjectItem(long nStartX, long nEndX,
	  long nStartY, long nEndY,
	  BOOL bLimits = FALSE);
	SvxObjectItem(const SvxObjectItem &);

	BOOL   HasLimits() const { return bLimits; }

	long   GetStartX() const { return nStartX; }
	long   GetEndX() const { return nEndX; }
	long   GetStartY() const { return nStartY; }
	long   GetEndY() const { return nEndY; }

	void   SetStartX(long l) { nStartX = l; }
	void   SetEndX(long l) { nEndX = l; }
	void   SetStartY(long l) { nStartY = l; }
	void   SetEndY(long l) { nEndY = l; }
};

#endif

