

#ifndef _SVDCRTV_HXX
#define _SVDCRTV_HXX

#ifndef _SVDHDL_HXX
#include <svx/svdhdl.hxx>
#endif

#include <svx/svddrgv.hxx>

class XLineAttrSetItem;
class XFillAttrSetItem;
class SdrEdgeObj;
class SdrObjConnection;
class ImpSdrConnectMarker;

class SdrCreateView: public SdrDragView
{
	friend class				SdrPageView;
	friend class				ImpSdrCreateLibObjMove;
	friend class				ImpSdrCreateLibObjResize;

protected:
	SdrObject*					pAktCreate;   
	SdrPageView*				pCreatePV;    
	SdrObject*					pCurrentLibObj;
	SdrDragMethod*				pLibObjDragMeth;
	ImpSdrConnectMarker*		pConnectMarker;

	Pointer						aAktCreatePointer;

	INT32						nAutoCloseDistPix;
	INT32						nFreeHandMinDistPix;
	UINT32						nAktInvent;     
	UINT16						nAktIdent;      

	unsigned					bAutoTextEdit : 1; 
	unsigned					b1stPointAsCenter : 1;
	unsigned					bUseIncompatiblePathCreateInterface : 1;
	unsigned					bAutoClosePolys : 1;
	unsigned					bCurrentLibObjMoveNoResize : 1;
	unsigned					bCurrentLibObjSetDefAttr : 1;
	unsigned					bCurrentLibObjSetDefLayer : 1;

private:
	void ImpClearVars();
	void ImpMakeCreateAttr();
	void ImpDelCreateAttr();

protected:
	BOOL ImpBegCreateObj(UINT32 nInvent, UINT16 nIdent, const Point& rPnt, OutputDevice* pOut,
		short nMinMov, SdrPageView* pPV, const Rectangle& rLogRect);

	void ShowCreateObj(OutputDevice* pOut, BOOL bFull);
	void HideCreateObj(OutputDevice* pOut, BOOL bFull);
	void DrawCreateObj(OutputDevice* pOut, BOOL bFull) const;
	void DrawCreateObjDiff(XPolyPolygon& rXPP0, XPolyPolygon& rXPP1);
	virtual void WriteRecords(SvStream& rOut) const;
	virtual BOOL ReadRecord(const SdrIOHeader& rViewHead, const SdrNamedSubRecord& rSubHead, SvStream& rIn);
	BOOL CheckEdgeMode();

public:
	SdrCreateView(SdrModel* pModel1, OutputDevice* pOut=NULL);
	SdrCreateView(SdrModel* pModel1, ExtOutputDevice* pXOut);
	virtual ~SdrCreateView();

	virtual void ToggleShownXor(OutputDevice* pOut, const Region* pRegion) const;
	virtual BOOL IsAction() const;
	virtual void MovAction(const Point& rPnt);
	virtual void EndAction();
	virtual void BckAction();
	virtual void BrkAction();
	virtual void TakeActionRect(Rectangle& rRect) const;

	BOOL MouseMove(const MouseEvent& rMEvt, Window* pWin);

	void SetActiveLayer(const String& rName) { aAktLayer=rName; }
	const String& GetActiveLayer() const { return aAktLayer; }
	void SetMeasureLayer(const String& rName) { aMeasureLayer=rName; }
	const String& GetMeasureLayer() const { return aMeasureLayer; }

	void SetEditMode(SdrViewEditMode eMode) { SdrDragView::SetEditMode(eMode); CheckEdgeMode(); }
	void SetEditMode(BOOL bOn=TRUE) { SdrDragView::SetEditMode(bOn); CheckEdgeMode(); }
	void SetCreateMode(BOOL bOn=TRUE) { SdrDragView::SetCreateMode(bOn); CheckEdgeMode(); }
	void SetGluePointEditMode(BOOL bOn=TRUE) { SdrDragView::SetGluePointEditMode(bOn); CheckEdgeMode(); }

	BOOL IsTextTool() const;

	BOOL IsEdgeTool() const;

	BOOL IsMeasureTool() const;

	void SetCurrentObj(UINT16 nIdent, UINT32 nInvent=SdrInventor);
	void TakeCurrentObj(UINT16& nIdent, UINT32& nInvent) const  { nInvent=nAktInvent; nIdent=nAktIdent; }
	UINT32 GetCurrentObjInventor() const { return nAktInvent; }
	UINT16 GetCurrentObjIdentifier() const { return nAktIdent; }

	void SetCurrentLibObj(SdrObject* pObj, BOOL bMoveNoResize=FALSE, BOOL bSetDefAttr=TRUE, BOOL bSetDefLayer=TRUE);
	const SdrObject* GetCurrentLibObj() const { return pCurrentLibObj; }

	BOOL BegCreateObj(const Point& rPnt, OutputDevice* pOut=NULL, short nMinMov=-3, SdrPageView* pPV=NULL);
	void MovCreateObj(const Point& rPnt);
	BOOL EndCreateObj(SdrCreateCmd eCmd);
	void BckCreateObj();  
	void BrkCreateObj();
	BOOL IsCreateObj() const { return pAktCreate!=NULL; }
	SdrObject* GetCreateObj() const { return pAktCreate; }

	BOOL BegCreateCaptionObj(const Point& rPnt, const Size& rObjSiz, OutputDevice* pOut=NULL, short nMinMov=-3, SdrPageView* pPV=NULL);

	BOOL BegCreateLibObj(const Point& rPnt, SdrObject* pObj, BOOL bMoveNoResize=TRUE, BOOL bSetDefAttr=FALSE,
		BOOL bSetDefLayer=FALSE, OutputDevice* pOut=NULL, short nMinMov=-3, SdrPageView* pPV=NULL);

	BOOL IsTextEditAfterCreate() const { return bAutoTextEdit; }
	void SetTextEditAfterCreate(BOOL bOn) { bAutoTextEdit = bOn; }

	BOOL IsCreate1stPointAsCenter() const { return b1stPointAsCenter; }
	void SetCreate1stPointAsCenter(BOOL bOn) { b1stPointAsCenter = bOn; SnapMove(); }

	BOOL IsAutoClosePolys() const { return bAutoClosePolys; }
	void SetAutoClosePolys(BOOL bOn) { bAutoClosePolys=bOn; }

	USHORT GetAutoCloseDistPix() const { return USHORT(nAutoCloseDistPix); }
	void SetAutoCloseDistPix(USHORT nVal) { nAutoCloseDistPix=nVal; }

	USHORT GetFreeHandMinDistPix() const { return USHORT(nFreeHandMinDistPix); }
	void SetFreeHandMinDistPix(USHORT nVal) { nFreeHandMinDistPix=nVal; }

	BOOL IsUseIncompatiblePathCreateInterface() const { return bUseIncompatiblePathCreateInterface; }
	void SetUseIncompatiblePathCreateInterface(BOOL bOn) { bUseIncompatiblePathCreateInterface = bOn; }
	void SetConnectMarker(const SdrObjConnection& rCon, const SdrPageView& rPV);
	void HideConnectMarker();

	BOOL GetAttributes(SfxItemSet& rTargetSet, BOOL bOnlyHardAttr=FALSE) const;

	BOOL SetAttributes(const SfxItemSet& rSet, BOOL bReplaceAll);
	SfxStyleSheet* GetStyleSheet(BOOL& rOk) const;
	BOOL SetStyleSheet(SfxStyleSheet* pStyleSheet, BOOL bDontRemoveHardAttr);
};

#endif 

