

#ifndef _SVDEDTV_HXX
#define _SVDEDTV_HXX

#ifndef _SVDHDL_HXX
#include <svx/svdhdl.hxx>
#endif

#include <svx/svdmrkv.hxx>

#ifndef _XPOLY_HXX
#include <svx/xpoly.hxx>
#endif

#ifndef _SVDMODEL_HXX
#include <svx/svdmodel.hxx>
#endif

class SfxUndoAction;
class SdrUndoAction;
class SdrUndoGroup;
class SfxStyleSheet;
class SdrLayer;
class SvdProgressInfo;

enum SdrHorAlign  {
	SDRHALIGN_NONE,
	SDRHALIGN_LEFT,
	SDRHALIGN_RIGHT,
	SDRHALIGN_CENTER
};

enum SdrVertAlign {
	SDRVALIGN_NONE,
	SDRVALIGN_TOP,
	SDRVALIGN_BOTTOM,
	SDRVALIGN_CENTER
};

enum SdrMergeMode {
	SDR_MERGE_MERGE,
	SDR_MERGE_SUBSTRACT,
	SDR_MERGE_INTERSECT
};

#define SDRINSERT_DONTMARK    0x0001 
#define SDRINSERT_ADDMARK     0x0002 
#define SDRINSERT_SETDEFATTR  0x0004 
#define SDRINSERT_SETDEFLAYER 0x0008 
#define SDRINSERT_NOBROADCAST 0x0010 

class SdrEditView: public SdrMarkView
{
	friend class				SdrPageView;

protected:

	unsigned					bPossibilitiesDirty : 1;
	unsigned					bReadOnly : 1;
	unsigned					bGroupPossible : 1;
	unsigned					bUnGroupPossible : 1;
	unsigned					bGrpEnterPossible : 1;
	unsigned					bDeletePossible : 1;
	unsigned					bToTopPossible : 1;
	unsigned					bToBtmPossible : 1;
	unsigned					bReverseOrderPossible : 1;
	unsigned					bImportMtfPossible : 1;
	unsigned					bCombinePossible : 1;
	unsigned					bDismantlePossible : 1;
	unsigned					bCombineNoPolyPolyPossible : 1;
	unsigned					bDismantleMakeLinesPossible : 1;
	unsigned					bOrthoDesiredOnMarked : 1;
	unsigned					bMoreThanOneNotMovable : 1;   
	unsigned					bOneOrMoreMovable : 1;        
	unsigned					bMoreThanOneNoMovRot : 1;     
	unsigned					bContortionPossible : 1;      
	unsigned					bAllPolys : 1;                
	unsigned					bOneOrMorePolys : 1;          
	unsigned					bMoveAllowed : 1;
	unsigned					bResizeFreeAllowed : 1;
	unsigned					bResizePropAllowed : 1;
	unsigned					bRotateFreeAllowed : 1;
	unsigned					bRotate90Allowed : 1;
	unsigned					bMirrorFreeAllowed : 1;
	unsigned					bMirror45Allowed : 1;
	unsigned					bMirror90Allowed : 1;
	unsigned					bShearAllowed : 1;
	unsigned					bEdgeRadiusAllowed : 1;
	unsigned					bTransparenceAllowed : 1;
	unsigned					bGradientAllowed : 1;
	unsigned					bCanConvToPath : 1;
	unsigned					bCanConvToPoly : 1;
	unsigned					bCanConvToContour : 1;
	unsigned					bCanConvToPathLineToArea : 1;
	unsigned					bCanConvToPolyLineToArea : 1;
	unsigned					bMoveProtect : 1;
	unsigned					bResizeProtect : 1;
	unsigned					bCombineError : 1;

	unsigned					bBundleVirtObj : 1;

private:
	void ImpClearVars();
	void ImpResetPossibilityFlags();

protected:
	void ImpBroadcastEdgesOfMarkedNodes();

	void ImpConvertTo(BOOL bPath, BOOL bLineToArea);

	SdrObject* ImpConvertOneObj(SdrObject* pObj, BOOL bPath, BOOL bLineToArea);

	void ImpCheckToTopBtmPossible();

	void ImpBundleVirtObjOfMarkList();

	void ImpCopyAttributes(const SdrObject* pSource, SdrObject* pDest) const;

	BOOL ImpCanConvertForCombine1(const SdrObject* pObj) const;
	BOOL ImpCanConvertForCombine(const SdrObject* pObj) const;
	XPolyPolygon ImpGetXPolyPoly1(const SdrObject* pObj, BOOL bCombine) const;
	XPolyPolygon ImpGetXPolyPoly(const SdrObject* pObj, BOOL bCombine) const;
	void ImpCombineToSinglePoly(XPolyPolygon& rXPP, long nJoinTol) const;

	BOOL ImpCanDismantle(const XPolyPolygon& rXPP, BOOL bMakeLines) const;
	BOOL ImpCanDismantle(const SdrObject* pObj, BOOL bMakeLines) const;
	void ImpDismantleOneObject(const SdrObject* pObj, SdrObjList& rOL, ULONG& rPos, SdrPageView* pPV, BOOL bMakeLines);
	void ImpCrookObj(SdrObject* pO, const Point& rRef, const Point& rRad, SdrCrookMode eMode,
		BOOL bVertical, BOOL bNoContortion, BOOL bRotate, const Rectangle& rMarkRect);
	void ImpDistortObj(SdrObject* pO, const Rectangle& rRef, const XPolygon& rDistortedRect, BOOL bNoContortion);
	BOOL ImpDelLayerCheck(SdrObjList* pOL, SdrLayerID nDelID) const;
	void ImpDelLayerDelObjs(SdrObjList* pOL, SdrLayerID nDelID);

	void DeleteMarked(SdrMarkList& rMark);

	virtual void CheckPossibilities();
	void ForcePossibilities() const { if (bPossibilitiesDirty || bSomeObjChgdFlag) ((SdrEditView*)this)->CheckPossibilities(); }

public:
	SdrEditView(SdrModel* pModel1, OutputDevice* pOut=NULL);
	SdrEditView(SdrModel* pModel1, ExtOutputDevice* pXOut);
	virtual ~SdrEditView();

#ifndef WIN
	void BegUndo()                       { pMod->BegUndo();         } 
	void BegUndo(const String& rComment) { pMod->BegUndo(rComment); } 
	void BegUndo(const String& rComment, const String& rObjDescr, SdrRepeatFunc eFunc=SDRREPFUNC_OBJ_NONE) { pMod->BegUndo(rComment,rObjDescr,eFunc); } 
	void BegUndo(SdrUndoGroup* pUndoGrp) { pMod->BegUndo(pUndoGrp); } 
	void EndUndo();                                                   
	void AddUndo(SdrUndoAction* pUndo)   { pMod->AddUndo(pUndo);    } 

	void SetUndoComment(const String& rComment) { pMod->SetUndoComment(rComment); }
	void SetUndoComment(const String& rComment, const String& rObjDescr) { pMod->SetUndoComment(rComment,rObjDescr); }
#else  
	void BegUndo();
	void BegUndo(const String& rComment);
	void BegUndo(const String& rComment, const String& rObjDescr, SdrRepeatFunc eFunc=SDRREPFUNC_OBJ_NONE);
	void BegUndo(SdrUndoGroup* pUndoGrp);
	void EndUndo();                                                   
	void AddUndo(SdrUndoAction* pUndo);

	void SetUndoComment(const String& rComment);
	void SetUndoComment(const String& rComment, const String& rObjDescr);
#endif

	SdrLayer* InsertNewLayer(const String& rName, USHORT nPos=0xFFFF);

	void      DeleteLayer(const String& rName);

	void      MoveLayer(const String& rName, USHORT nNewPos);

	void ForceMarkedObjToAnotherPage();
	void ForceMarkedToAnotherPage()   { ForceMarkedObjToAnotherPage(); }

	BOOL IsReadOnly() const { ForcePossibilities(); return bReadOnly; }

	void DeleteMarkedObj();
	BOOL IsDeleteMarkedObjPossible() const { ForcePossibilities(); return bDeletePossible; }

	void SetMarkedObjRect(const Rectangle& rRect, BOOL bCopy=FALSE);
	void MoveMarkedObj(const Size& rSiz, BOOL bCopy=FALSE);
	void ResizeMarkedObj(const Point& rRef, const Fraction& xFact, const Fraction& yFact, BOOL bCopy=FALSE);
	long GetMarkedObjRotate() const;
	void RotateMarkedObj(const Point& rRef, long nWink, BOOL bCopy=FALSE);
	void MirrorMarkedObj(const Point& rRef1, const Point& rRef2, BOOL bCopy=FALSE);
	void MirrorMarkedObjHorizontal(BOOL bCopy=FALSE);
	void MirrorMarkedObjVertical(BOOL bCopy=FALSE);
	long GetMarkedObjShear() const;
	void ShearMarkedObj(const Point& rRef, long nWink, BOOL bVShear=FALSE, BOOL bCopy=FALSE);
	void CrookMarkedObj(const Point& rRef, const Point& rRad, SdrCrookMode eMode, BOOL bVertical=FALSE, BOOL bNoContortion=FALSE, BOOL bCopy=FALSE);
	void DistortMarkedObj(const Rectangle& rRef, const XPolygon& rDistortedRect, BOOL bNoContortion=FALSE, BOOL bCopy=FALSE);

	void CopyMarkedObj();
	void SetAllMarkedRect(const Rectangle& rRect, BOOL bCopy=FALSE) { SetMarkedObjRect(rRect,bCopy); }
	void MoveAllMarked(const Size& rSiz, BOOL bCopy=FALSE) { MoveMarkedObj   (rSiz,bCopy); }
	void ResizeAllMarked(const Point& rRef, const Fraction& xFact, const Fraction& yFact, BOOL bCopy=FALSE) { ResizeMarkedObj (rRef,xFact,yFact,bCopy); }
	long GetAllMarkedRotate() const { return GetMarkedObjRotate(); }
	void RotateAllMarked(const Point& rRef, long nWink, BOOL bCopy=FALSE) { RotateMarkedObj(rRef,nWink,bCopy); }
	void MirrorAllMarked(const Point& rRef1, const Point& rRef2, BOOL bCopy=FALSE) { MirrorMarkedObj(rRef1,rRef2,bCopy); }
	void MirrorAllMarkedHorizontal(BOOL bCopy=FALSE) { MirrorMarkedObjHorizontal(bCopy); }
	void MirrorAllMarkedVertical(BOOL bCopy=FALSE) { MirrorMarkedObjVertical(bCopy); }
	long GetAllMarkedShear() const { return GetMarkedObjShear(); }
	void ShearAllMarked(const Point& rRef, long nWink, BOOL bVShear=FALSE, BOOL bCopy=FALSE) { ShearMarkedObj(rRef,nWink,bVShear,bCopy); }
	void CrookAllMarked(const Point& rRef, const Point& rRad, SdrCrookMode eMode, BOOL bVertical=FALSE, BOOL bNoContortion=FALSE, BOOL bCopy=FALSE) { CrookMarkedObj(rRef,rRad,eMode,bVertical,bNoContortion,bCopy); }
	void DistortAllMarked(const Rectangle& rRef, const XPolygon& rDistortedRect, BOOL bNoContortion=FALSE, BOOL bCopy=FALSE) { DistortMarkedObj(rRef,rDistortedRect,bNoContortion,bCopy); }
	void CopyMarked() { CopyMarkedObj(); }
	BOOL IsMoveAllowed() const { ForcePossibilities(); return bMoveAllowed && !bMoveProtect; }
	BOOL IsResizeAllowed(BOOL bProp=FALSE) const;
	BOOL IsRotateAllowed(BOOL b90Deg=FALSE) const;
	BOOL IsMirrorAllowed(BOOL b45Deg=FALSE, BOOL b90Deg=FALSE) const;
	BOOL IsTransparenceAllowed() const;
	BOOL IsGradientAllowed() const;
	BOOL IsShearAllowed() const;
	BOOL IsEdgeRadiusAllowed() const;
	BOOL IsCrookAllowed(BOOL bNoContortion=FALSE) const;
	BOOL IsDistortAllowed(BOOL bNoContortion=FALSE) const;

	BOOL CombineMarkedObjects(BOOL bNoPolyPoly = FALSE);

	void MergeMarkedObjects(SdrMergeMode eMode);

	void DistributeMarkedObjects();

	void DismantleMarkedObjects(BOOL bMakeLines=FALSE);
	BOOL IsCombinePossible(BOOL bNoPolyPoly=FALSE) const;
	BOOL IsDismantlePossible(BOOL bMakeLines=FALSE) const;

	BOOL InsertObject(SdrObject* pObj, SdrPageView& rPV, ULONG nOptions=0);

	void ReplaceObject(SdrObject* pOldObj, SdrPageView& rPV, SdrObject* pNewObj, BOOL bMark=TRUE);

	void SetNotPersistAttrToMarked(const SfxItemSet& rAttr, BOOL bReplaceAll);
	void MergeNotPersistAttrFromMarked(SfxItemSet& rAttr, BOOL bOnlyHardAttr) const;
	void MergeAttrFromMarked(SfxItemSet& rAttr, BOOL bOnlyHardAttr) const;
	SfxItemSet GetAttrFromMarked(BOOL bOnlyHardAttr) const;
	void SetAttrToMarked(const SfxItemSet& rAttr, BOOL bReplaceAll);

	SfxItemSet GetGeoAttrFromMarked() const;
	void SetGeoAttrToMarked(const SfxItemSet& rAttr);

	SfxStyleSheet* GetStyleSheetFromMarked() const;

	void SetStyleSheetToMarked(SfxStyleSheet* pStyleSheet, BOOL bDontRemoveHardAttr);

	BOOL GetAttributes(SfxItemSet& rTargetSet, BOOL bOnlyHardAttr) const;

	BOOL SetAttributes(const SfxItemSet& rSet, BOOL bReplaceAll);
	SfxStyleSheet* GetStyleSheet(BOOL& rOk) const;
	BOOL SetStyleSheet(SfxStyleSheet* pStyleSheet, BOOL bDontRemoveHardAttr);

	void GroupMarked(const SdrObject* pUserGrp=NULL);

	void UnGroupMarked();

	BOOL IsGroupPossible() const { ForcePossibilities(); return bGroupPossible; }
	BOOL IsUnGroupPossible() const { ForcePossibilities(); return bUnGroupPossible; }
	BOOL IsGroupEnterPossible() const { ForcePossibilities(); return bGrpEnterPossible; }

	BOOL IsConvertToPathObjPossible(BOOL bLineToArea) const { ForcePossibilities(); return bLineToArea ? bCanConvToPathLineToArea : bCanConvToPath; }
	BOOL IsConvertToPolyObjPossible(BOOL bLineToArea) const { ForcePossibilities(); return bLineToArea ? bCanConvToPolyLineToArea : bCanConvToPoly; }
	BOOL IsConvertToContourPossible() const { ForcePossibilities(); return bCanConvToContour; }
	void ConvertMarkedToPathObj(BOOL bLineToArea);
	void ConvertMarkedToPolyObj(BOOL bLineToArea);

	void AlignMarkedObjects(SdrHorAlign eHor, SdrVertAlign eVert, BOOL bBoundRects=FALSE);
	BOOL IsAlignPossible() const;

	void MovMarkedToTop();

	void MovMarkedToBtm();

	void PutMarkedToTop();

	void PutMarkedToBtm();

	void PutMarkedInFrontOfObj(const SdrObject* pRefObj);

	void PutMarkedBehindObj(const SdrObject* pRefObj);

	void ReverseOrderOfMarked();

	BOOL IsToTopPossible() const { ForcePossibilities(); return bToTopPossible; }
	BOOL IsToBtmPossible() const { ForcePossibilities(); return bToBtmPossible; }
	BOOL IsReverseOrderPossible() const { ForcePossibilities(); return bReverseOrderPossible; }

	virtual SdrObject* GetMaxToTopObj(SdrObject* pObj) const;
	virtual SdrObject* GetMaxToBtmObj(SdrObject* pObj) const;

	virtual void ObjOrderChanged(SdrObject* pObj, ULONG nOldPos, ULONG nNewPos);

	void DoImportMarkedMtf(SvdProgressInfo *pProgrInfo=NULL);
	BOOL IsImportMtfPossible() const { ForcePossibilities(); return bImportMtfPossible; }

	void SetVirtualObjectBundling(BOOL bOn) { bBundleVirtObj=bOn; }
	BOOL IsVirtualObjectBundling() const { return bBundleVirtObj; }

	virtual void MarkListHasChanged();
	virtual void ModelHasChanged();
};

#endif 

