

#ifndef _SVDEDXV_HXX
#define _SVDEDXV_HXX

#ifndef _SVDHDL_HXX
#include <svx/svdhdl.hxx>
#endif

#include <svx/svdglev.hxx>

class SdrOutliner;
class OutlinerView;
class EditStatus;
class EditFieldInfo;

enum SdrEndTextEditKind {SDRENDTEXTEDIT_UNCHANGED, 
			 SDRENDTEXTEDIT_CHANGED,   
			 SDRENDTEXTEDIT_DELETED,   
			 SDRENDTEXTEDIT_SHOULDBEDELETED}; 

class SdrObjEditView: public SdrGlueEditView
{
	friend class				SdrPageView;
	friend class				ImpSdrEditPara;

protected:

	SdrObject*					pTextEditObj;          
	SdrPageView*				pTextEditPV;
	SdrOutliner*				pTextEditOutliner;     
	OutlinerView*				pTextEditOutlinerView; 
	Window*						pTextEditWin;          
	Cursor*						pTextEditCursorMerker; 
	ImpSdrEditPara*				pEditPara; 
	SdrObject*					pMacroObj;
	SdrPageView*				pMacroPV;
	Window*						pMacroWin;

	Rectangle					aTextEditArea;
	Rectangle					aMinTextEditArea;
	Link						aOldCalcFieldValueLink; 
	Point						aMacroDownPos;

	USHORT						nMacroTol;

	unsigned					bTextEditDontDelete : 1;   
	unsigned					bTextEditOnlyOneView : 1;  
	unsigned					bTextEditNewObj : 1;       
	unsigned					bQuickTextEditMode : 1;    
	unsigned					bMacroMode : 1;            
	unsigned					bMacroDown : 1;

private:
	void ImpClearVars();

protected:
	OutlinerView* ImpFindOutlinerView(Window* pWin) const;

	OutlinerView* ImpMakeOutlinerView(Window* pWin, BOOL bNoPaint, OutlinerView* pGivenView) const;
	void ImpPaintOutlinerView(OutlinerView& rOutlView, const Rectangle* pRect=NULL, BOOL bDrawButPaint=FALSE) const;

	Color ImpGetTextEditBackgroundColor() const;

	BOOL ImpIsTextEditAllSelected() const;
	void ImpMakeTextCursorAreaVisible();

	DECL_LINK(ImpOutlinerStatusEventHdl,EditStatus*);
	DECL_LINK(ImpOutlinerCalcFieldValueHdl,EditFieldInfo*);

	void ImpMacroUp(const Point& rUpPos);
	void ImpMacroDown(const Point& rDownPos);

public:
	SdrObjEditView(SdrModel* pModel1, OutputDevice* pOut=NULL);
	SdrObjEditView(SdrModel* pModel1, ExtOutputDevice* pXOut);
	virtual ~SdrObjEditView();

	virtual BOOL IsAction() const;
	virtual void MovAction(const Point& rPnt);
	virtual void EndAction();
	virtual void BrkAction();
	virtual void BckAction();
	virtual void TakeActionRect(Rectangle& rRect) const;

	virtual void SFX_NOTIFY(SfxBroadcaster& rBC, const TypeId& rBCType, const SfxHint& rHint, const TypeId& rHintType);
	virtual void ModelHasChanged();

	void SetQuickTextEditMode(BOOL bOn) { bQuickTextEditMode=bOn; }
	BOOL IsQuickTextEditMode() const { return bQuickTextEditMode; }

	BOOL BegTextEdit(SdrObject* pObj, SdrPageView* pPV=NULL, Window* pWin=NULL,
		SdrOutliner* pGivenOutliner=NULL, OutlinerView* pGivenOutlinerView=NULL,
		BOOL bDontDeleteOutliner=FALSE, BOOL bOnlyOneView=FALSE);
	BOOL BegTextEdit(SdrObject* pObj, SdrPageView* pPV=NULL, Window* pWin=NULL, BOOL bIsNewObj=FALSE,
		SdrOutliner* pGivenOutliner=NULL, OutlinerView* pGivenOutlinerView=NULL,
		BOOL bDontDeleteOutliner=FALSE, BOOL bOnlyOneView=FALSE);

	SdrEndTextEditKind EndTextEdit(BOOL bDontDeleteReally=FALSE);
	BOOL IsTextEdit() const { return pTextEditObj!=NULL; }

	BOOL IsTextEditFrame() const;

	BOOL IsTextEditHit(const Point& rHit, short nTol) const;

	BOOL IsTextEditFrameHit(const Point& rHit) const;

	BOOL IsTextEditInSelectionMode() const;

	void AddTextEditOfs(MouseEvent& rMEvt) const;

	SdrObject* GetTextEditObject() const { return pTextEditObj; }
	SdrPageView* GetTextEditPageView() const { return pTextEditPV; }

	Window* GetTextEditWin() const { return pTextEditWin; }
	void SetTextEditWin(Window* pWin);

	const SdrOutliner* GetTextEditOutliner() const { return pTextEditOutliner; }
	SdrOutliner* GetTextEditOutliner() { return pTextEditOutliner; }
	const OutlinerView* GetTextEditOutlinerView() const { return pTextEditOutlinerView; }
	OutlinerView* GetTextEditOutlinerView() { return pTextEditOutlinerView; }

	BOOL KeyInput(const KeyEvent& rKEvt, Window* pWin);
	BOOL MouseButtonDown(const MouseEvent& rMEvt, Window* pWin);
	BOOL MouseButtonUp(const MouseEvent& rMEvt, Window* pWin);
	BOOL MouseMove(const MouseEvent& rMEvt, Window* pWin);
	BOOL Command(const CommandEvent& rCEvt, Window* pWin);
	BOOL Cut(ULONG nFormat=SDR_ANYFORMAT);
	BOOL Yank(ULONG nFormat=SDR_ANYFORMAT);
	BOOL Paste(Window* pWin=NULL, ULONG nFormat=SDR_ANYFORMAT);

	USHORT GetScriptType() const;

	BOOL GetAttributes(SfxItemSet& rTargetSet, BOOL bOnlyHardAttr=FALSE) const;

	BOOL SetAttributes(const SfxItemSet& rSet, BOOL bReplaceAll);
	SfxStyleSheet* GetStyleSheet(BOOL& rOk) const;
	BOOL SetStyleSheet(SfxStyleSheet* pStyleSheet, BOOL bDontRemoveHardAttr);

	virtual void AddWin(OutputDevice* pWin1);
	virtual void DelWin(OutputDevice* pWin1);

	void SetMacroMode(BOOL bOn) { bMacroMode=bOn; }
	BOOL IsMacroMode() const { return bMacroMode; }
	BOOL BegMacroObj(const Point& rPnt, short nTol, SdrObject* pObj, SdrPageView* pPV, Window* pWin);
	BOOL BegMacroObj(const Point& rPnt, SdrObject* pObj, SdrPageView* pPV, Window* pWin) { return BegMacroObj(rPnt,-2,pObj,pPV,pWin); }
	void MovMacroObj(const Point& rPnt);
	void BrkMacroObj();
	BOOL EndMacroObj();
	BOOL IsMacroObj() const { return pMacroObj!=NULL; }
	BOOL IsMacroObjDown() const { return bMacroDown; }
};

#endif 

