

#ifndef _SVDMRKV_HXX
#define _SVDMRKV_HXX

#ifndef _SVDMARK_HXX
#include <svx/svdmark.hxx>
#endif

#ifndef _SVDHDL_HXX
#include <svx/svdhdl.hxx>
#endif

#include <svx/svdsnpv.hxx>

#ifndef _SVDTYPES_HXX
#include <svx/svdtypes.hxx>
#endif

#define SDRSEARCH_DEEP         0x0001 
#define SDRSEARCH_ALSOONMASTER 0x0002 
#define SDRSEARCH_WHOLEPAGE    0x0004 
#define SDRSEARCH_TESTMARKABLE 0x0008 
#define SDRSEARCH_TESTMACRO    0x0010 
#define SDRSEARCH_TESTTEXTEDIT 0x0020 
#define SDRSEARCH_WITHTEXT     0x0040 
#define SDRSEARCH_TESTTEXTAREA 0x0080 
#define SDRSEARCH_BACKWARD     0x0100 
#define SDRSEARCH_NEXT         0x0200 
#define SDRSEARCH_MARKED       0x0400 
#define SDRSEARCH_PASS2BOUND   0x0800 
#define SDRSEARCH_PASS3NEAREST 0x1000 

#define SDRSEARCH_PICKMARKABLE (SDRSEARCH_TESTMARKABLE)
#define SDRSEARCH_PICKTEXTEDIT (SDRSEARCH_DEEP|SDRSEARCH_TESTMARKABLE|SDRSEARCH_TESTTEXTEDIT)
#define SDRSEARCH_PICKMACRO    (SDRSEARCH_DEEP|SDRSEARCH_ALSOONMASTER|SDRSEARCH_WHOLEPAGE|SDRSEARCH_TESTMACRO)

#define SDRSEARCHPASS_DIRECT       0x0000 
#define SDRSEARCHPASS_INACTIVELIST 0x0001 
#define SDRSEARCHPASS_MASTERPAGE   0x0002 
#define SDRSEARCHPASS_BOUND        0x0010 
#define SDRSEARCHPASS_NEAREST      0x0020 

enum SdrHitKind {SDRHIT_NONE,      
	             SDRHIT_OBJECT,    
	             SDRHIT_BOUNDRECT, 
	             SDRHIT_BOUNDTL,   
	             SDRHIT_BOUNDTC,   
	             SDRHIT_BOUNDTR,   
	             SDRHIT_BOUNDCL,   
	             SDRHIT_BOUNDCR,   
	             SDRHIT_BOUNDBL,   
	             SDRHIT_BOUNDBC,   
	             SDRHIT_BOUNDBR, 
	              SDRHIT_HANDLE,          
	             SDRHIT_HELPLINE,        
	             SDRHIT_GLUEPOINT,       
	             SDRHIT_TEXTEDIT,        
	             SDRHIT_TEXTEDITOBJ,     
	             SDRHIT_URLFIELD,        
	             SDRHIT_MACRO,           
	             SDRHIT_MARKEDOBJECT,    
	             SDRHIT_UNMARKEDOBJECT}; 

enum SdrViewEditMode {SDREDITMODE_EDIT,           
	                  SDREDITMODE_CREATE,         
	                  SDREDITMODE_GLUEPOINTEDIT}; 

#define IMPSDR_MARKOBJDESCRIPTION    0x0000
#define IMPSDR_POINTSDESCRIPTION     0x0001
#define IMPSDR_GLUEPOINTSDESCRIPTION 0x0002

class SdrMarkView: public SdrSnapView 
{
	friend class				SdrPageView;

protected:
	SdrObject*					pMarkedObj;       
	SdrPageView*				pMarkedPV;        
	 Point						aRef1;            
	Point						aRef2;            
	Point						aLastCrookCenter; 
	SdrHdlList					aHdl;
	SdrMarkList					aMark;            
	SdrMarkList					aEdgesOfMarkedNodes;        
	SdrMarkList					aMarkedEdgesOfMarkedNodes;  
	SdrMarkList					aMarkedEdges;               
	Rectangle					aMarkedObjRect;
	Rectangle					aMarkedPointsRect;
	Rectangle					aMarkedGluePointsRect;
	 USHORT						nFrameHandlesLimit;
	USHORT						nSpecialCnt;
	USHORT						nInsPointNum;      
	ULONG						nMarkableObjCount;
	 SdrDragMode					eDragMode;        
	SdrViewEditMode				eEditMode;      
	SdrViewEditMode				eEditMode0;     
	 unsigned					bHdlShown : 1;
	unsigned					bRefHdlShownOnly : 1; 
	unsigned					bDesignMode : 1;      
	unsigned					bMarking : 1;         
	unsigned					bMarkingPoints : 1;   
	unsigned					bMarkingGluePoints : 1;
	unsigned					bUnmarking : 1;
	unsigned					bForceFrameHandles : 1; 
	unsigned					bPlusHdlAlways : 1;   
	unsigned					bMarkHdlWhenTextEdit : 1; 
	 unsigned					bInsPolyPoint : 1;     
	unsigned					bEdgesOfMarkedNodesDirty : 1;
	unsigned					bMarkedObjRectDirty : 1;
	unsigned					bMrkPntDirty : 1;
	unsigned					bMarkedPointsRectsDirty : 1;
	unsigned					bMarkableObjCountDirty : 1;
	unsigned					bHdlHidden : 1;

private:
	void ImpClearVars();
	void ImpSetPointsRects() const;
	void UndirtyMrkPnt() const;

protected:
	virtual void SFX_NOTIFY(SfxBroadcaster& rBC, const TypeId& rBCType, const SfxHint& rHint, const TypeId& rHintType);
	virtual void ModelHasChanged(); 
	 void ShowMarkObjOrPoints(OutputDevice* pOut);
	void HideMarkObjOrPoints(OutputDevice* pOut);
	void DrawMarkObjOrPoints(OutputDevice* pOut) const;
	 virtual void SetMarkHandles();                                           
	 virtual void SetMarkRects();                                             
	virtual void CheckMarked();                                              
	 virtual void AddDragModeHdl(SdrDragMode eMode);
	 virtual void AddCustomHdl();
	 void ForceRefToMarked();
	void ForceEdgesOfMarkedNodes();
	void ForceUndirtyMrkPnt() const                                       { if (bMrkPntDirty) UndirtyMrkPnt(); }
	 void ImpShowMarkHdl(OutputDevice* pOut, const Region* pRegion, BOOL bNoRefHdl);
	SdrObject* ImpCheckObjHit(const Point& rPnt, USHORT nTol, SdrObject* pObj, SdrPageView* pPV, ULONG nOptions, const SetOfByte* pMVisLay) const;
	SdrObject* ImpCheckObjHit(const Point& rPnt, USHORT nTol, SdrObjList* pOL, SdrPageView* pPV, ULONG nOptions, const SetOfByte* pMVisLay, SdrObject*& rpRootObj) const;
	BOOL ImpIsObjHit(const Point& rPnt, USHORT nTol, SdrObject* pObj, SdrPageView* pPV, ULONG nOptions) const { return ImpCheckObjHit(rPnt,nTol,pObj,pPV,nOptions,NULL)!=NULL; }
	BOOL ImpIsFrameHandles() const;
	 void ImpTakeDescriptionStr(const SdrMarkList& rML, USHORT nStrCacheID, String& rStr, USHORT nVal=0, USHORT nOpt=0) const;
	void ImpTakeDescriptionStr(USHORT nStrCacheID, String& rStr, USHORT nVal=0, USHORT nOpt=0) const;
	 BOOL ImpMarkPoint(SdrHdl* pHdl, SdrMark* pMark, BOOL bUnmark);
	BOOL MarkPoints(const Rectangle* pRect, BOOL bUnmark);
	BOOL MarkGluePoints(const Rectangle* pRect, BOOL bUnmark);
	 virtual void WriteRecords(SvStream& rOut) const;
	virtual BOOL ReadRecord(const SdrIOHeader& rViewHead, const SdrNamedSubRecord& rSubHead, SvStream& rIn);

public:
	void ImpAfterPaint(); 
	void AfterInitRedraw(USHORT nWinNum); 
	 SdrMarkView(SdrModel* pModel1, OutputDevice* pOut=NULL);
	SdrMarkView(SdrModel* pModel1, ExtOutputDevice* pXOut);
	 virtual void ToggleShownXor(OutputDevice* pOut, const Region* pRegion) const;
	virtual BOOL IsAction() const;
	virtual void MovAction(const Point& rPnt);
	virtual void EndAction();
	virtual void BckAction();
	virtual void BrkAction();
	virtual void TakeActionRect(Rectangle& rRect) const;
	 virtual void ClearPageViews();
	virtual void HidePage(SdrPageView* pPV);
	virtual void SetPagePos(SdrPageView* pPV, const Point& rOffs);
	 virtual BOOL IsObjMarkable(SdrObject* pObj, SdrPageView* pPV) const;

	void ForceInvalidateMarkHandles();
	 BOOL IsMarking() const { return bMarking || bMarkingPoints || bMarkingGluePoints; }
	 BOOL IsUnmarking() const { return bUnmarking; }
	void SetUnmarking(BOOL bOn) { bUnmarking=bOn; }
	 BOOL BegMarkObj(const Point& rPnt, OutputDevice* pOut);
	BOOL BegMarkObj(const Point& rPnt, BOOL bUnmark=FALSE);
	void MovMarkObj(const Point& rPnt);
	BOOL EndMarkObj();
	void BrkMarkObj();
	BOOL IsMarkObj() const { return bMarking; }
	 void SetDragMode(SdrDragMode eMode);
	SdrDragMode GetDragMode() const { return eDragMode; }
	BOOL ChkDragMode(SdrDragMode eMode) const;
	void SetFrameHandles(BOOL bOn);
	BOOL IsFrameHandles() const { return bForceFrameHandles; }
	 void SetFrameHandlesLimit(USHORT nAnz) { nFrameHandlesLimit=nAnz; }
	USHORT GetFrameHandlesLimit() const { return nFrameHandlesLimit; }
	 void SetEditMode(SdrViewEditMode eMode);
	SdrViewEditMode GetEditMode() const { return eEditMode; }
	 void SetEditMode(BOOL bOn=TRUE) { SetEditMode(bOn?SDREDITMODE_EDIT:SDREDITMODE_CREATE); }
	BOOL IsEditMode() const { return eEditMode==SDREDITMODE_EDIT; }
	void SetCreateMode(BOOL bOn=TRUE) { SetEditMode(bOn?SDREDITMODE_CREATE:SDREDITMODE_EDIT); }
	BOOL IsCreateMode() const { return eEditMode==SDREDITMODE_CREATE; }
	void SetGluePointEditMode(BOOL bOn=TRUE) { SetEditMode(bOn?SDREDITMODE_GLUEPOINTEDIT:eEditMode0); }
	BOOL IsGluePointEditMode() const { return eEditMode==SDREDITMODE_GLUEPOINTEDIT;; }
	 void SetDesignMode(BOOL bOn=TRUE);
	BOOL IsDesignMode() const { return bDesignMode; }
	 void SetFrameDragSingles(BOOL bOn=TRUE) { SetFrameHandles(bOn); }
	BOOL IsFrameDragSingles() const { return IsFrameHandles(); }
	 SdrHitKind PickSomething(const Point& rPnt, short nTol=-2) const;
	BOOL HasMarkableObj() const;
	ULONG GetMarkableObjCount() const;
	BOOL HasMarkedObj() const { return aMark.GetMarkCount()!=0; }
	ULONG GetMarkedObjCount() const { return aMark.GetMarkCount(); }
	virtual void ShowMarkHdl(OutputDevice* pOut, BOOL bNoRefHdl=FALSE);
	virtual void HideMarkHdl(OutputDevice* pOut, BOOL bNoRefHdl=FALSE);
	BOOL IsMarkHdlShown() const { return bHdlShown; }
	 void SetMarkHdlHidden(BOOL bOn);
	BOOL IsMarkHdlHidden() const { return bHdlHidden; }
	 BOOL IsMarkedHit(const Point& rPnt, short nTol=-2) const { return IsMarkedObjHit(rPnt,nTol); }
	BOOL IsMarkedObjHit(const Point& rPnt, short nTol=-2) const;
	SdrHdl* HitHandle(const Point& rPnt, const OutputDevice& rOut) const { return PickHandle(rPnt,rOut,0,NULL); }
	 SdrHdl* PickHandle(const Point& rPnt, const OutputDevice& rOut, ULONG nOptions=0, SdrHdl* pHdl0=NULL) const;
	SdrHdl* PickHandle(const Point& rPnt, ULONG nOptions=0, SdrHdl* pHdl0=NULL) const { return PickHandle(rPnt,*(const OutputDevice*)pActualOutDev,nOptions,pHdl0); }
	 BOOL PickObj(const Point& rPnt, short nTol, SdrObject*& rpObj, SdrPageView*& rpPV, ULONG nOptions, SdrObject** ppRootObj, ULONG* pnMarkNum=NULL, USHORT* pnPassNum=NULL) const;
	BOOL PickObj(const Point& rPnt, short nTol, SdrObject*& rpObj, SdrPageView*& rpPV, ULONG nOptions=0) const;
	BOOL PickObj(const Point& rPnt, SdrObject*& rpObj, SdrPageView*& rpPV, ULONG nOptions=0) const { return PickObj(rPnt,nHitTolLog,rpObj,rpPV,nOptions); }
	BOOL MarkObj(const Point& rPnt, short nTol=-2, BOOL bToggle=FALSE, BOOL bDeep=FALSE);
	 BOOL PickMarkedObj(const Point& rPnt, SdrObject*& rpObj, SdrPageView*& rpPV, ULONG* pnMarkNum=NULL, ULONG nOptions=0) const;
	 BOOL MarkNextObj(BOOL bPrev=FALSE);
	 BOOL MarkNextObj(const Point& rPnt, short nTol=-2, BOOL bPrev=FALSE);
	 BOOL MarkObj(const Rectangle& rRect, BOOL bUnmark=FALSE);
	void MarkObj(SdrObject* pObj, SdrPageView* pPV, BOOL bUnmark=FALSE, BOOL bImpNoSetMarkHdl=FALSE);
	void MarkAllObj(SdrPageView* pPV=NULL); 
	void UnmarkAllObj(SdrPageView* pPV=NULL); 
	 BOOL IsObjMarked(SdrObject* pObj) const;
	void MarkAll(SdrPageView* pPV=NULL) { MarkAllObj(pPV); }
	void UnMarkAll(SdrPageView* pPV=NULL) { UnmarkAllObj(pPV); }
	const SdrMarkList& GetMarkList() const { return aMark; }
	void SortMarkList() { aMark.ForceSort(); }
	 const SdrMarkList& GetEdgesOfMarkedNodes() { ForceEdgesOfMarkedNodes(); return aEdgesOfMarkedNodes; }
	 String GetMarkDescription() const { return aMark.GetMarkDescription(); }
	 USHORT GetMarkHdlSizePixel() const;
	void SetMarkHdlSizePixel(USHORT nSiz);
	 BOOL IsSolidMarkHdl() const { return aHdl.IsFineHdl(); }
	void SetSolidMarkHdl(BOOL bOn);
	 BOOL HasMarkablePoints() const;
	ULONG GetMarkablePointCount() const;
	BOOL HasMarkedPoints() const;
	ULONG GetMarkedPointCount() const;
	 BOOL IsPointMarkable(const SdrHdl& rHdl) const;
	BOOL MarkPoint(SdrHdl& rHdl, BOOL bUnmark=FALSE);
	 BOOL MarkPoints(const Rectangle& rRect, BOOL bUnmark=FALSE) { return MarkPoints(&rRect,bUnmark); }
	BOOL UnmarkPoint(SdrHdl& rHdl) { return MarkPoint(rHdl,TRUE); }
	BOOL UnMarkPoint(SdrHdl& rHdl) { return MarkPoint(rHdl,TRUE); }
	BOOL IsPointMarked(const SdrHdl& rHdl) const { ForceUndirtyMrkPnt(); return &rHdl!=NULL && rHdl.IsSelected(); }
	BOOL MarkAllPoints() { return MarkPoints(NULL,FALSE); }
	BOOL UnmarkAllPoints() { return MarkPoints(NULL,TRUE); }
	BOOL UnMarkAllPoints() { return MarkPoints(NULL,TRUE); }
	 BOOL MarkNextPoint(BOOL bPrev=FALSE);
	 BOOL MarkNextPoint(const Point& rPnt, BOOL bPrev=FALSE);
	 ULONG GetHdlNum(SdrHdl* pHdl) const { return aHdl.GetHdlNum(pHdl); }
	SdrHdl* GetHdl(ULONG nHdlNum)  const { return aHdl.GetHdl(nHdlNum); }
	const SdrHdlList& GetHdlList() const { return aHdl; }
	 BOOL BegMarkPoints(const Point& rPnt, OutputDevice* pOut);
	BOOL BegMarkPoints(const Point& rPnt, BOOL bUnmark=FALSE);
	void MovMarkPoints(const Point& rPnt);
	BOOL EndMarkPoints();
	void BrkMarkPoints();
	BOOL IsMarkPoints() const { return bMarkingPoints; }
	 void SetPlusHandlesAlwaysVisible(BOOL bOn);
	BOOL IsPlusHandlesAlwaysVisible() const { return bPlusHdlAlways; }
	 void SetMarkHdlWhenTextEdit(BOOL bOn) { bMarkHdlWhenTextEdit=bOn; }
	BOOL IsMarkHdlWhenTextEdit() const { return bMarkHdlWhenTextEdit; }
	 BOOL HasMarkableGluePoints() const;
	ULONG GetMarkableGluePointCount() const;
	BOOL HasMarkedGluePoints() const;
	ULONG GetMarkedGluePointCount() const;
	 BOOL PickGluePoint(const Point& rPnt, SdrObject*& rpObj, USHORT& rnId, SdrPageView*& rpPV, ULONG nOptions=0) const;
	BOOL MarkGluePoint(const SdrObject* pObj, USHORT nId, const SdrPageView* pPV, BOOL bUnmark=FALSE);
	BOOL UnmarkGluePoint(const SdrObject* pObj, USHORT nId, const SdrPageView* pPV) { return MarkGluePoint(pObj,nId,pPV,TRUE); }
	BOOL IsGluePointMarked(const SdrObject* pObj, USHORT nId) const;
	BOOL UnmarkGluePoint(const SdrHdl& rHdl);
	 SdrHdl* GetGluePointHdl(const SdrObject* pObj, USHORT nId) const;
	BOOL IsGluePoint(const SdrHdl& rHdl) const { return &rHdl!=NULL && rHdl.GetKind()==HDL_GLUE; }
	 BOOL MarkGluePoints(const Rectangle& rRect) { return MarkGluePoints(&rRect,FALSE); }
	BOOL UnmarkGluePoints(const Rectangle& rRect) { return MarkGluePoints(&rRect,TRUE); }
	BOOL MarkAllGluePoints() { return MarkGluePoints(NULL,FALSE); }
	BOOL UnmarkAllGluePoints() { return MarkGluePoints(NULL,TRUE); }
	 BOOL MarkNextGluePoint(BOOL bPrev=FALSE);
	 BOOL MarkNextGluePoint(const Point& rPnt, BOOL bPrev=FALSE);
	 BOOL BegMarkGluePoints(const Point& rPnt, OutputDevice* pOut);
	BOOL BegMarkGluePoints(const Point& rPnt, BOOL bUnmark=FALSE);
	void MovMarkGluePoints(const Point& rPnt);
	BOOL EndMarkGluePoints();
	void BrkMarkGluePoints();
	BOOL IsMarkGluePoints() const { return bMarkingGluePoints; }
	 void AdjustMarkHdl(BOOL bRestraintPaint=TRUE);
	 const Rectangle& GetMarkedObjRect() const; 
	Rectangle GetMarkedObjBoundRect() const;   
	const Rectangle& GetMarkedPointsRect() const;     
	const Rectangle& GetMarkedGluePointsRect() const; 
	const Rectangle& GetAllMarkedRect() const { return GetMarkedObjRect(); }
	Rectangle GetAllMarkedBoundRect() const { return GetMarkedObjBoundRect(); }
	const Rectangle& GetAllMarkedPointsRect() const  { return GetMarkedPointsRect(); }
	 virtual void MarkListHasChanged();
	 BOOL EnterMarkedGroup();
	 const Point& GetLastCrookCenter() const { return aLastCrookCenter; }
	 void SetLastCrookCenter(const Point& rPt) { aLastCrookCenter=rPt; }
	 const Point& GetRef1() const { return aRef1; }
	void SetRef1(const Point& rPt);
	 const Point& GetRef2() const { return aRef1; }
	void SetRef2(const Point& rPt);
};

#endif 

