

#ifndef _SVDOGRP_HXX
#define _SVDOGRP_HXX

#ifndef _DATETIME_HXX 
#include <tools/datetime.hxx>
#endif

#ifndef _SVDOBJ_HXX
#include <svx/svdobj.hxx>
#endif

class ImpSdrObjGroupLink;
class SdrObjList;
class SdrObjListIter;
class SfxItemSet;

#define SDRUSERDATA_OBJGROUPLINK	0

class ImpSdrObjGroupLinkUserData : public SdrObjUserData
{
	friend class				SdrObjGroup;
	friend class				ImpSdrObjGroupLink;

	SdrObject*					pObj;
	String						aFileName;   
	XubString					aObjName;    
	DateTime					aFileDate0;  
	Rectangle					aSnapRect0;  
	long						nDrehWink0;  
	long						nShearWink0; 

	ImpSdrObjGroupLink*			pLink;
	ULONG						nObjNum;     
	USHORT						nPageNum;    
	FASTBOOL					bMasterPage; 

	FASTBOOL					bOrigPos;    
	FASTBOOL					bOrigSize;   
	FASTBOOL					bOrigRotate; 
	FASTBOOL					bOrigShear;  

public:
	TYPEINFO();

	ImpSdrObjGroupLinkUserData(SdrObject* pObj1);
	virtual ~ImpSdrObjGroupLinkUserData();

	virtual SdrObjUserData* Clone(SdrObject* pObj1) const;
	virtual void WriteData(SvStream& rOut);
	virtual void ReadData(SvStream& rIn);
	virtual void AfterRead();
};

class SdrObjGroup : public SdrObject
{
	friend class				ImpSdrObjGroupLink;
	friend class				ImpSdrObjGroupLinkUserData;

protected:
	SfxItemSet*					mpGroupItemSet;
	SdrObjList*					pSub;    
	long						nDrehWink;
	long						nShearWink;

	String						aName;

	Point						aRefPoint; 
	FASTBOOL					bRefPoint; 

private:
	ImpSdrObjGroupLinkUserData* GetLinkUserData() const;
	void ImpLinkAnmeldung();
	void ImpLinkAbmeldung();

public:
	TYPEINFO();
	SdrObjGroup();
	virtual ~SdrObjGroup();

	void SetGroupLink(const String& rFileName, const String& rObjName);
	void ReleaseGroupLink();
	FASTBOOL IsLinkedGroup() const { return pPlusData!=NULL && GetLinkUserData()!=NULL; }
	FASTBOOL ReloadLinkedGroup(FASTBOOL bForceLoad=FALSE);

	FASTBOOL LoadGroup(const String& rFileName, const String& rObjName, USHORT* pnPgNum=NULL, FASTBOOL* pbMasterPg=NULL, ULONG* pnObjNum=NULL);

	virtual UINT16 GetObjIdentifier() const;
	virtual void TakeObjInfo(SdrObjTransformInfoRec& rInfo) const;
	virtual SdrLayerID GetLayer() const;
	virtual void NbcSetLayer(SdrLayerID nLayer);
	virtual void SetObjList(SdrObjList* pNewObjList);
	virtual void SetPage(SdrPage* pNewPage);
	virtual void SetModel(SdrModel* pNewModel);
	virtual FASTBOOL HasRefPoint() const;
	virtual Point GetRefPoint() const;
	virtual void SetRefPoint(const Point& rPnt);
	virtual SdrObjList* GetSubList() const;

	virtual const Rectangle& GetBoundRect() const;
	virtual const Rectangle& GetSnapRect() const;
	virtual FASTBOOL Paint(ExtOutputDevice& rOut, const SdrPaintInfoRec& rInfoRec) const;
	virtual SdrObject* CheckHit(const Point& rPnt, USHORT nTol, const SetOfByte* pVisiLayer) const;
	virtual void operator=(const SdrObject& rObj);

	virtual void TakeObjNameSingul(String& rName) const;
	virtual void TakeObjNamePlural(String& rName) const;

	virtual FASTBOOL HasSetName() const;

	virtual void SetName(const String& rStr);
	virtual String GetName() const;

	virtual void RecalcSnapRect();
	virtual void TakeXorPoly(XPolyPolygon& rPoly, FASTBOOL bDetail) const;
	virtual void TakeContour(XPolyPolygon& rXPoly, SdrContourType eType) const;

	virtual FASTBOOL BegDrag(SdrDragStat& rDrag)  const;
	virtual FASTBOOL BegCreate(SdrDragStat& rStat);

	virtual long GetRotateAngle() const;
	virtual long GetShearAngle(FASTBOOL bVertical=FALSE) const;

	virtual void Move(const Size& rSiz);
	virtual void Resize(const Point& rRef, const Fraction& xFact, const Fraction& yFact);
	virtual void Rotate(const Point& rRef, long nWink, double sn, double cs);
	virtual void Mirror(const Point& rRef1, const Point& rRef2);
	virtual void Shear(const Point& rRef, long nWink, double tn, FASTBOOL bVShear);
	virtual void SetAnchorPos(const Point& rPnt);
	virtual void SetRelativePos(const Point& rPnt);
	virtual void SetSnapRect(const Rectangle& rRect);
	virtual void SetLogicRect(const Rectangle& rRect);

	virtual void NbcMove(const Size& rSiz);
	virtual void NbcResize(const Point& rRef, const Fraction& xFact, const Fraction& yFact);
	virtual void NbcRotate(const Point& rRef, long nWink, double sn, double cs);
	virtual void NbcMirror(const Point& rRef1, const Point& rRef2);
	virtual void NbcShear(const Point& rRef, long nWink, double tn, FASTBOOL bVShear);
	virtual void NbcSetAnchorPos(const Point& rPnt);
	virtual void NbcSetRelativePos(const Point& rPnt);
	virtual void NbcSetSnapRect(const Rectangle& rRect);
	virtual void NbcSetLogicRect(const Rectangle& rRect);

	virtual const SfxItemSet& GetItemSet() const;
	virtual SfxItemSet* CreateNewItemSet(SfxItemPool& rPool);
	virtual void ItemSetChanged(const SfxItemSet& rSet);

	virtual void ItemChange(const sal_uInt16 nWhich, const SfxPoolItem* pNewItem = 0);

	virtual void PreSave();
	virtual void PostSave();

	virtual void NbcSetStyleSheet(SfxStyleSheet* pNewStyleSheet, FASTBOOL bDontRemoveHardAttr);
	virtual void SetStyleSheet(SfxStyleSheet* pNewStyleSheet, FASTBOOL bDontRemoveHardAttr);
	virtual SfxStyleSheet* GetStyleSheet() const;

	virtual void NbcReformatText();
	virtual void ReformatText();
	virtual void BurnInStyleSheetAttributes( BOOL bPseudoSheetsOnly=FALSE );

	virtual void RestartAnimation(SdrPageView* pPageView) const;

	virtual SdrObject* DoConvertToPolyObj(BOOL bBezier) const;

	virtual void WriteData(SvStream& rOut) const;
	virtual void ReadData(const SdrObjIOHeader& rHead, SvStream& rIn);
	virtual void AfterRead();

	virtual void MigrateItemPool(SfxItemPool* pSrcPool, SfxItemPool* pDestPool);
};

#endif 

