

#ifndef _SVDUNDO_HXX
#define _SVDUNDO_HXX

#ifndef _SOLAR_HRC
#include <svtools/solar.hrc>
#endif

#ifndef _UNDO_HXX 
#include <svtools/undo.hxx>
#endif

#ifndef _CONTNR_HXX 
#include <tools/contnr.hxx>
#endif

#ifndef _GEN_HXX 
#include <tools/gen.hxx>
#endif

#ifndef _SVDTYPES_HXX
#include <svx/svdtypes.hxx> 
#endif

class SfxItemSet;
class SfxStyleSheet;
class SdrView;
class SdrPageView;
class SdrModel;
class SdrObject;
class SdrPage;
class SdrObjList;
class SdrLayer;
class SdrLayerAdmin;
class SdrObjGeoData;
class OutlinerParaObject;
class SdrMasterPageDescriptor;

class SdrUndoAction : public SfxUndoAction
{
protected:
	SdrModel&					rMod;

protected:
	SdrUndoAction(SdrModel& rNewMod)
	:	rMod(rNewMod)
	{}

public:
	TYPEINFO();
	virtual BOOL CanRepeat(SfxRepeatTarget& rView) const;
	virtual void Repeat(SfxRepeatTarget& rView);

	virtual String GetRepeatComment(SfxRepeatTarget& rView) const;
	virtual String GetSdrRepeatComment(SdrView& rView) const;

	virtual FASTBOOL CanSdrRepeat(SdrView& rView) const;
	virtual void SdrRepeat(SdrView& rView);
};

class SdrUndoGroup : public SdrUndoAction
{
protected:
	Container					aBuf;

	String						aComment;
	String						aObjDescription;

	SdrRepeatFunc				eFunction;

public:
	SdrUndoGroup(SdrModel& rNewMod);
	SdrUndoGroup(SdrModel& rNewMod,const String& rStr);
	virtual ~SdrUndoGroup();

	void Clear();
	ULONG GetActionCount() const { return aBuf.Count(); }
	SdrUndoAction* GetAction(ULONG nNum) const { return (SdrUndoAction*)(aBuf.GetObject(nNum)); }
	void AddAction(SdrUndoAction* pAct);

	void SetComment(const String& rStr) { aComment=rStr; }
	void SetObjDescription(const String& rStr) { aObjDescription=rStr; }
	virtual String GetComment() const;
	virtual String GetSdrRepeatComment(SdrView& rView) const;

	virtual void Undo();
	virtual void Redo();

	virtual FASTBOOL CanSdrRepeat(SdrView& rView) const;
	virtual void SdrRepeat(SdrView& rView);
	void SetRepeatFunction(SdrRepeatFunc eFunc) { eFunction=eFunc; }
	SdrRepeatFunc GetRepeatFunction() const { return eFunction; }
};

class SdrUndoObj : public SdrUndoAction
{
protected:
	SdrObject*					pObj;

protected:
	SdrUndoObj(SdrObject& rNewObj);

	void ImpTakeDescriptionStr(USHORT nStrCacheID, String& rStr, FASTBOOL bRepeat=FALSE) const;
};

class SdrUndoAttrObj : public SdrUndoObj
{
protected:
	SfxItemSet*					pUndoSet;
	SfxItemSet*					pRedoSet;
	SfxItemSet*					pRepeatSet;

	SfxStyleSheet*				pUndoStyleSheet;
	SfxStyleSheet*				pRedoStyleSheet;
	SfxStyleSheet*				pRepeatStyleSheet;
	FASTBOOL					bStyleSheet;
	FASTBOOL					bHaveToTakeRedoSet;

	OutlinerParaObject*			pTextUndo;

	SdrUndoGroup*				pUndoGroup;

public:
	SdrUndoAttrObj(SdrObject& rNewObj, FASTBOOL bStyleSheet1=FALSE, FASTBOOL bSaveText=FALSE);
	virtual ~SdrUndoAttrObj();
	void SetRepeatAttr(const SfxItemSet& rSet);
	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
	virtual String GetSdrRepeatComment(SdrView& rView) const;

	virtual void SdrRepeat(SdrView& rView);
	virtual FASTBOOL CanSdrRepeat(SdrView& rView) const;
};

class SdrUndoMoveObj : public SdrUndoObj
{
protected:
	Size						aDistance;     

public:
	SdrUndoMoveObj(SdrObject& rNewObj): SdrUndoObj(rNewObj) {}
	SdrUndoMoveObj(SdrObject& rNewObj, const Size& rDist): SdrUndoObj(rNewObj),aDistance(rDist) {}

	void SetDistance(const Size& rDist) { aDistance=rDist; }
	const Size& GetDistance() const { return aDistance; }

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
	virtual String GetSdrRepeatComment(SdrView& rView) const;

	virtual void SdrRepeat(SdrView& rView);
	virtual FASTBOOL CanSdrRepeat(SdrView& rView) const;
};

class SdrUndoGeoObj : public SdrUndoObj
{
protected:
	SdrObjGeoData*				pUndoGeo;
	SdrObjGeoData*				pRedoGeo;

	SdrUndoGroup*				pUndoGroup;

public:
	SdrUndoGeoObj(SdrObject& rNewObj);
	virtual ~SdrUndoGeoObj();

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoObjList : public SdrUndoObj {
	FASTBOOL					bOwner;

protected:
	SdrObjList*					pObjList;
	SdrView*					pView;      
	SdrPageView*				pPageView;  
	UINT32						nOrdNum;

protected:
	SdrUndoObjList(SdrObject& rNewObj, FASTBOOL bOrdNumDirect=FALSE);
	virtual ~SdrUndoObjList();

	void SetView(SdrView* pView1, SdrPageView* pPageView1) { pView=pView1; pPageView=pPageView1; }
	BOOL IsOwner() { return bOwner; }
	void SetOwner(BOOL bNew);
};

class SdrUndoRemoveObj : public SdrUndoObjList
{
public:
	SdrUndoRemoveObj(SdrObject& rNewObj, FASTBOOL bOrdNumDirect=FALSE)
	: SdrUndoObjList(rNewObj,bOrdNumDirect) {}

	virtual void Undo();
	virtual void Redo();
};

class SdrUndoInsertObj : public SdrUndoObjList
{
public:
	SdrUndoInsertObj(SdrObject& rNewObj, FASTBOOL bOrdNumDirect=FALSE)
	:	SdrUndoObjList(rNewObj,bOrdNumDirect) {}

	virtual void Undo();
	virtual void Redo();
};

class SdrUndoDelObj : public SdrUndoRemoveObj
{
public:
	SdrUndoDelObj(SdrObject& rNewObj, FASTBOOL bOrdNumDirect=FALSE)
	:	SdrUndoRemoveObj(rNewObj,bOrdNumDirect) { SetOwner(TRUE); }

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
	virtual String GetSdrRepeatComment(SdrView& rView) const;

	virtual void SdrRepeat(SdrView& rView);
	virtual FASTBOOL CanSdrRepeat(SdrView& rView) const;
};

class SdrUndoNewObj : public SdrUndoInsertObj
{
public:
	SdrUndoNewObj(SdrObject& rNewObj, FASTBOOL bOrdNumDirect=FALSE)
	:	SdrUndoInsertObj(rNewObj,bOrdNumDirect) {}

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoReplaceObj : public SdrUndoObj
{
	FASTBOOL					bOldOwner;
	FASTBOOL					bNewOwner;

protected:
	SdrObjList*					pObjList;
	UINT32						nOrdNum;
	SdrObject*					pNewObj;

public:
	SdrUndoReplaceObj(SdrObject& rOldObj1, SdrObject& rNewObj1, FASTBOOL bOrdNumDirect=FALSE);
	virtual ~SdrUndoReplaceObj();

	virtual void Undo();
	virtual void Redo();

	BOOL IsNewOwner() { return bNewOwner; }
	void SetNewOwner(BOOL bNew);

	BOOL IsOldOwner() { return bOldOwner; }
	void SetOldOwner(BOOL bNew);
};

class SdrUndoCopyObj : public SdrUndoNewObj
{
public:
	SdrUndoCopyObj(SdrObject& rNewObj, FASTBOOL bOrdNumDirect=FALSE)
	:	SdrUndoNewObj(rNewObj,bOrdNumDirect) {}

	virtual String GetComment() const;
};

class SdrUndoObjOrdNum : public SdrUndoObj
{
protected:
	UINT32						nOldOrdNum;
	UINT32						nNewOrdNum;

public:
	SdrUndoObjOrdNum(SdrObject& rNewObj, UINT32 nOldOrdNum1, UINT32 nNewOrdNum1);

	virtual void Undo();
	virtual void Redo();

	virtual String   GetComment() const;
};

class SdrUndoObjSetText : public SdrUndoObj
{
protected:
	OutlinerParaObject*			pOldText;
	OutlinerParaObject*			pNewText;
	FASTBOOL					bNewTextAvailable;
	BOOL						bEmptyPresObj;

public:
	SdrUndoObjSetText(SdrObject& rNewObj);
	virtual ~SdrUndoObjSetText();

	FASTBOOL IsDifferent() const { return pOldText!=pNewText; }
	void AfterSetText();

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
	virtual String GetSdrRepeatComment(SdrView& rView) const;

	virtual void SdrRepeat(SdrView& rView);
	virtual FASTBOOL CanSdrRepeat(SdrView& rView) const;
};

class SdrUndoLayer : public SdrUndoAction
{
protected:
	SdrLayer*					pLayer;
	SdrLayerAdmin*				pLayerAdmin;
	USHORT						nNum;
	FASTBOOL					bItsMine;

protected:
	SdrUndoLayer(USHORT nLayerNum, SdrLayerAdmin& rNewLayerAdmin, SdrModel& rNewModel);
	virtual ~SdrUndoLayer();
};

class SdrUndoNewLayer : public SdrUndoLayer
{
public:
	SdrUndoNewLayer(USHORT nLayerNum, SdrLayerAdmin& rNewLayerAdmin, SdrModel& rNewModel)
	: SdrUndoLayer(nLayerNum,rNewLayerAdmin,rNewModel) {}

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoDelLayer : public SdrUndoLayer
{
public:
	SdrUndoDelLayer(USHORT nLayerNum, SdrLayerAdmin& rNewLayerAdmin, SdrModel& rNewModel)
	: SdrUndoLayer(nLayerNum,rNewLayerAdmin,rNewModel) { bItsMine=TRUE; }

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoMoveLayer : public SdrUndoLayer
{
	USHORT						nNeuPos;

public:
	SdrUndoMoveLayer(USHORT nLayerNum, SdrLayerAdmin& rNewLayerAdmin, SdrModel& rNewModel, USHORT nNeuPos1)
	:	SdrUndoLayer(nLayerNum,rNewLayerAdmin,rNewModel), nNeuPos(nNeuPos1) {}

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoPage : public SdrUndoAction
{
protected:
	SdrPage*					pPage;

protected:
	void ImpInsertPage(USHORT nNum);
	void ImpRemovePage(USHORT nNum);
	void ImpMovePage(USHORT nOldNum, USHORT nNewNum);

protected:
	SdrUndoPage(SdrPage& rNewPg);

	void ImpTakeDescriptionStr(USHORT nStrCacheID, String& rStr, USHORT n=0, FASTBOOL bRepeat=FALSE) const;
};

class SdrUndoPageList : public SdrUndoPage
{
protected:
	USHORT						nPageNum;

	FASTBOOL					bItsMine;

protected:
	SdrUndoPageList(SdrPage& rNewPg);
	virtual ~SdrUndoPageList();
};

class SdrUndoDelPage : public SdrUndoPageList
{

	SdrUndoGroup*				pUndoGroup;

public:
	SdrUndoDelPage(SdrPage& rNewPg);
	virtual ~SdrUndoDelPage();

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
	virtual String GetSdrRepeatComment(SdrView& rView) const;

	virtual void SdrRepeat(SdrView& rView);
	virtual FASTBOOL CanSdrRepeat(SdrView& rView) const;
};

class SdrUndoNewPage : public SdrUndoPageList
{
public:
	SdrUndoNewPage(SdrPage& rNewPg): SdrUndoPageList(rNewPg) {}

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoCopyPage : public SdrUndoNewPage
{
public:
	SdrUndoCopyPage(SdrPage& rNewPg): SdrUndoNewPage(rNewPg) {}

	virtual String GetComment() const;
	virtual String GetSdrRepeatComment(SdrView& rView) const;

	virtual void SdrRepeat(SdrView& rView);
	virtual FASTBOOL CanSdrRepeat(SdrView& rView) const;
};

class SdrUndoSetPageNum : public SdrUndoPage
{
protected:
	USHORT						nOldPageNum;
	USHORT						nNewPageNum;

public:
	SdrUndoSetPageNum(SdrPage& rNewPg, USHORT nOldPageNum1, USHORT nNewPageNum1)
	:	SdrUndoPage(rNewPg),nOldPageNum(nOldPageNum1),nNewPageNum(nNewPageNum1) {}

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoPageMasterPage : public SdrUndoPage
{
protected:
	SdrMasterPageDescriptor*	pMasterDescriptor;
	SdrMasterPageDescriptor*	pNewMasterDescriptor;
	USHORT						nMasterNum;
	USHORT						nNewMasterNum;

protected:

	SdrUndoPageMasterPage(SdrPage& rNewPg, USHORT nMasterDescriptorNum);

public:
	virtual ~SdrUndoPageMasterPage();
};

class SdrUndoPageInsertMasterPage : public SdrUndoPageMasterPage
{
public:
	SdrUndoPageInsertMasterPage(SdrPage& rNewPg, USHORT nMasterDescriptorNum)
	:	SdrUndoPageMasterPage(rNewPg,nMasterDescriptorNum) {}

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoPageRemoveMasterPage : public SdrUndoPageMasterPage
{
public:
	SdrUndoPageRemoveMasterPage(SdrPage& rNewPg, USHORT nMasterDescriptorNum)
	:	SdrUndoPageMasterPage(rNewPg,nMasterDescriptorNum) {}

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoPageMoveMasterPage : public SdrUndoPageMasterPage
{
public:
	SdrUndoPageMoveMasterPage(SdrPage& rNewPg, USHORT nMasterDescriptorNum, USHORT nNewPos)
	:	SdrUndoPageMasterPage(rNewPg,nMasterDescriptorNum) { nNewMasterNum=nNewPos; }

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

class SdrUndoPageChangeMasterPage : public SdrUndoPageMasterPage
{
public:
	SdrUndoPageChangeMasterPage(SdrPage& rNewPg, USHORT nMasterDescriptorNum)
	:	SdrUndoPageMasterPage(rNewPg,nMasterDescriptorNum) {}

	virtual void Undo();
	virtual void Redo();

	virtual String GetComment() const;
};

#endif 

