

#ifndef _SVIMBASE_HXX
#define _SVIMBASE_HXX

#ifdef VCL

#ifndef _SV_BITMAP_HXX 
#include <vcl/bitmap.hxx>
#endif
#ifndef _SV_SALBTYPE_HXX 
#include <vcl/salbtype.hxx>
#endif

#ifdef WIN
#define _SVHUGE huge
#else
#define _SVHUGE
#endif

enum SimDepthType
{
	SIM_DEPTH_1,
	SIM_DEPTH_4,
	SIM_DEPTH_8,
	SIM_DEPTH_24
};

struct SimColor
{
	BYTE cBlue;
	BYTE cGreen;
	BYTE cRed;
};

inline BOOL operator==( const SimColor& rCol1, const SimColor& rCol2 )
{
	return ( ( rCol1.cRed == rCol2.cRed ) &&
	( rCol1.cGreen == rCol2.cGreen ) &&
	( rCol1.cBlue == rCol2.cBlue ) );
}

struct SimPalette
{
	ULONG		nColors;
	SimColor	aColorArray[ 256 ];
};

class SfxViewFrame;
class SfxProgress;

class SvImageBase
{
private:

	Bitmap				aOutBitmap;
	HPBYTE				pArray1;
	HPBYTE				pArray2;
	HPBYTE				pArray3;
	HPBYTE				pArray4;
	HPBYTE				pOrgArray;
	HPBYTE				pDestArray;
	SimPalette*			pPal1;
	SimPalette*			pPal2;
	SimPalette*			pPal3;
	SimPalette*			pPal4;
	ULONG				nWidth1;
	ULONG				nWidth2;
	ULONG				nWidth3;
	ULONG				nWidth4;
	ULONG				nHeight1;
	ULONG				nHeight2;
	ULONG				nHeight3;
	ULONG				nHeight4;
	ULONG				nAlignedWidth1;
	ULONG				nAlignedWidth2;
	ULONG				nAlignedWidth3;
	ULONG				nAlignedWidth4;
	ULONG				nWhichOrg;
	SimDepthType		eOrgDepth;
	SimDepthType		eUndoDepth;
	SimDepthType		eRedoDepth;
	BOOL				bIsUndo;
	BOOL				bIsRedo;
	BOOL				bCreateUndo;
	BOOL				bValid;
	BOOL				bDitherAll;

	HPBYTE				BitmapToArray24( const Bitmap& rBitmap, ULONG* pWidth,
							 ULONG* pHeight, ULONG* pAlignedWidth,
							 SfxViewFrame *pFrame = NULL );
	BOOL				Array24ToBitmap( HPBYTE pArray, Bitmap &rBitmap,
							 const ULONG nWidth, const ULONG nHeight,
							 const ULONG nColorCount = 256,
							 ULONG nLast = 0, SfxProgress* pProgress = NULL );

	Bitmap				CreateSaveBitmap( const SimDepthType eDepth, SfxViewFrame *pFrame = NULL );

	HPBYTE				CreateArray24( ULONG nWidth, ULONG nHeight );
	void				DeleteArray( HPBYTE pArray );
				SvImageBase(const SvImageBase& rSvImageBase);
	const SvImageBase&  operator=(const SvImageBase& rSvImageBase);

public:
				SvImageBase();
			SvImageBase( const Bitmap& rBitmap,
						 const ULONG nColorCount = 256,
						 ULONG nLast = 0, SfxProgress* pProgress = NULL );
			~SvImageBase();

	BOOL				IsValid() { return bValid; }

	ULONG				GetOrgWidth() const { return nWhichOrg == 1 ? nWidth1 : nWidth2; }
	ULONG				GetDestWidth() const { return nWhichOrg == 1 ? nWidth2 : nWidth1; }

	ULONG				GetOrgHeight() const { return nWhichOrg == 1 ? nHeight1 : nHeight2; }
	ULONG				GetDestHeight() const { return nWhichOrg == 1 ? nHeight2 : nHeight1; }

	ULONG				GetOrgAlignedWidth() const { return nWhichOrg == 1 ? nAlignedWidth1 : nAlignedWidth2; }
	ULONG				GetDestAlignedWidth() const { return nWhichOrg == 1 ? nAlignedWidth2 : nAlignedWidth1; }

	ULONG				GetOrgAlignedSize() const { return GetOrgAlignedWidth() * GetOrgHeight(); }
	ULONG				GetDestAlignedSize() const { return GetDestAlignedWidth() * GetDestHeight(); }

	SimDepthType		GetDepth() const { return eOrgDepth; }
	void				SetDepth( const SimDepthType eDepth ) { eOrgDepth = eDepth; }

	SimDepthType		GetUndoDepth() const { return eUndoDepth; }
	void				SetUndoDepth(const SimDepthType eDepth) { eUndoDepth = eDepth; }

	SimDepthType		GetRedoDepth() const { return eRedoDepth; }
	void				SetRedoDepth(const SimDepthType eDepth) { eRedoDepth = eDepth; }

	BOOL				BeginProcessing( BOOL bUndo = TRUE );
	void				EndProcessing();

	BOOL				BeginProcessingExt(ULONG nWidth, ULONG nHeight, BOOL bUndo = TRUE);
	void				EndProcessingExt() { EndProcessing(); }

	HPBYTE				GetOrgPointer() { return pOrgArray; }
	HPBYTE				GetDestPointer() { return pDestArray; }

	BOOL				CreateOutBitmap( const ULONG nColorCount = 256, ULONG nLast = 0,
							 SfxProgress* pProgress = NULL );

	BOOL				DoUndo( SfxProgress* pProgress = NULL );
	BOOL				DoRedo( SfxProgress* pProgress = NULL );

	const Bitmap&		GetOutBitmap() const;

	Bitmap				GetSaveBitmap();

	SimPalette*			GetOrgPalette() const { return nWhichOrg == 1 ? pPal1 : pPal2; }
	SimPalette*			GetDestPalette() const { return nWhichOrg == 1 ? pPal2 : pPal1; }
};

BOOL DitherBitmap( Bitmap& rBitmap, BOOL bDitherAlways = FALSE );

#else 

#ifndef _BITMAP_HXX 
#include <vcl/bitmap.hxx>
#endif
#ifndef _PAL_HXX
#include <vcl/pal.hxx>
#endif
#ifndef _SVWIN_H
#include <tools/svwin.h>
#endif

#undef GetPrinter
#undef SetPrinter

#ifndef _SVHUGE
#ifdef WIN
#define _SVHUGE huge
#else
#define _SVHUGE
#endif 
#endif 

class SfxViewFrame;
class SfxProgress;

enum SimDepthType
{
	SIM_DEPTH_1,
	SIM_DEPTH_4,
	SIM_DEPTH_8,
	SIM_DEPTH_24
};

struct SimColor
{
	BYTE cBlue;
	BYTE cGreen;
	BYTE cRed;
};

BOOL operator==( const SimColor& rCol1, const SimColor& rCol2 );

struct SimPalette
{
	ULONG		nColors;
	SimColor	aColorArray[ 256 ];
};

class SvImageBase
{
#if defined(WIN) || defined(WNT)

	HGLOBAL         hArray1;
	HGLOBAL         hArray2;
	HGLOBAL         hArray3;
	HGLOBAL         hArray4;

protected:

	HGLOBAL         BitmapToArray24(const Bitmap& rBitmap, ULONG* pWidth,
						ULONG* pHeight, ULONG* pAlignedWidth,
						SfxViewFrame* pFrame = NULL);
	BOOL            Array24ToBitmap(HGLOBAL hArray, Bitmap &rBitmap,
						const ULONG nWidth, const ULONG nHeight,
						const ULONG nColorCount = 256,
						ULONG nLast = 0,
						SfxProgress* pProgress = NULL);

	HGLOBAL         CreateArray24(ULONG nWidth, ULONG nHeight);
	void            DeleteArray(HGLOBAL hArray);

	HBITMAP         DDBtoDIB(HBITMAP hBitmap, HPALETTE hPalette);
	ULONG           PaletteSize(void* p);

	BOOL            DecompressRLE4(BYTE _SVHUGE* pSrc, BYTE _SVHUGE* pDst,
					   ULONG nWidth, BITMAPINFO* pBMI);

	BOOL            DecompressRLE8(BYTE _SVHUGE* pSrc, BYTE _SVHUGE* pDst,
					   ULONG nWidth, BITMAPINFO* pBMI);

#endif 

#if defined(OS2) || defined(UNX) || defined(MAC)

	BYTE _SVHUGE*   pArray1;
	BYTE _SVHUGE*   pArray2;
	BYTE _SVHUGE*   pArray3;
	BYTE _SVHUGE*   pArray4;

protected:

	BYTE _SVHUGE*   BitmapToArray24(const Bitmap& rBitmap, ULONG* pWidth,
						ULONG* pHeight, ULONG* pAlignedWidth,
						SfxViewFrame *pFrame = NULL);
	BOOL            Array24ToBitmap(BYTE _SVHUGE*, Bitmap &rBitmap,
						const ULONG nWidth, const ULONG nHeight,
						const ULONG nColorCount = 256,
						ULONG nLast = 0,
						SfxProgress* pProgress = NULL);

	BYTE _SVHUGE*   CreateArray24(ULONG nWidth, ULONG nHeight);
	void            DeleteArray(BYTE _SVHUGE* pArray);

#if defined OS2

	BOOL            DecompressRLE4(BYTE* pSrc, BYTE* pDst,
					   ULONG nWidth, BmpPaletteEntryOS2* pPal);

	BOOL            DecompressRLE8(BYTE* pSrc, BYTE* pDst,
					   ULONG nWidth, BmpPaletteEntryOS2* pPal);

#endif 

#endif 

private:
	Palette             aDithPal;

	BYTE _SVHUGE*       pOrgArray;
	BYTE _SVHUGE*       pDestArray;
	BYTE _SVHUGE*       pUndoArray;
	BYTE _SVHUGE*       pMergeArray;

	Bitmap*             pOutBitmap;

	SimPalette*			pPal1;
	SimPalette*			pPal2;
	SimPalette*			pPal3;
	SimPalette*			pPal4;

	long				nDummy1;
	long				nDummy2;
	long				nDummy3;
	long				nDummy4;

	ULONG               nWidth1;
	ULONG               nWidth2;
	ULONG               nWidth3;
	ULONG               nWidth4;

	ULONG               nHeight1;
	ULONG               nHeight2;
	ULONG               nHeight3;
	ULONG               nHeight4;

	ULONG               nAlignedWidth1;
	ULONG               nAlignedWidth2;
	ULONG               nAlignedWidth3;
	ULONG               nAlignedWidth4;

	USHORT              nWhichOrg;
	BOOL                bIsUndo;
	BOOL                bIsRedo;
	BOOL                bCreateUndo;

	SimDepthType        eOrgDepth;
	SimDepthType        eUndoDepth;
	SimDepthType        eRedoDepth;

	BOOL                bValid;

	BOOL                bDitherAll;
				SvImageBase(const SvImageBase& rSvImageBase);
	const SvImageBase&  operator=(const SvImageBase& rSvImageBase);

public:

	SvImageBase();
	SvImageBase(const Bitmap& rBitmap, const ULONG nColorCount = 256, ULONG nLast = 0, SfxProgress* pProgress = NULL);
	~SvImageBase();

	BOOL        IsValid() { return bValid; }

	ULONG       GetOrgWidth() const     { return nWhichOrg == 1 ? nWidth1 : nWidth2; }
	ULONG       GetDestWidth() const    { return nWhichOrg == 1 ? nWidth2 : nWidth1; }
	ULONG       GetUndoWidth() const    { return nWidth3; }
	ULONG       GetMergeWidth() const   { return nWidth4; }

	ULONG       GetOrgHeight() const    { return nWhichOrg == 1 ? nHeight1 : nHeight2; }
	ULONG       GetDestHeight() const   { return nWhichOrg == 1 ? nHeight2 : nHeight1; }
	ULONG       GetUndoHeight() const   { return nHeight3; }
	ULONG       GetMergeHeight() const  { return nHeight4; }

	ULONG       GetOrgAlignedWidth() const   { return nWhichOrg == 1 ? nAlignedWidth1 : nAlignedWidth2; }
	ULONG       GetDestAlignedWidth() const  { return nWhichOrg == 1 ? nAlignedWidth2 : nAlignedWidth1; }
	ULONG       GetUndoAlignedWidth() const  { return nAlignedWidth3; }
	ULONG       GetMergeAlignedWidth() const { return nAlignedWidth4; }

	ULONG       GetOrgAlignedSize() const   { return GetOrgAlignedWidth() * GetOrgHeight(); }
	ULONG       GetDestAlignedSize() const  { return GetDestAlignedWidth() * GetDestHeight(); }
	ULONG       GetUndoAlignedSize() const  { return GetUndoAlignedWidth() * GetUndoHeight(); }
	ULONG       GetMergeAlignedSize() const { return GetMergeAlignedWidth() * GetMergeHeight(); }

	SimDepthType    GetDepth() const { return eOrgDepth; }
	void            SetDepth(const SimDepthType eDepth) { eOrgDepth = eDepth; }

	SimDepthType    GetUndoDepth() const { return eUndoDepth; }
	SimDepthType    GetRedoDepth() const { return eRedoDepth; }
	void            SetUndoDepth(const SimDepthType eDepth) { eUndoDepth = eDepth; }
	void            SetRedoDepth(const SimDepthType eDepth) { eRedoDepth = eDepth; }

	BOOL        BeginProcessing(BOOL bUndo = TRUE);
	void        EndProcessing();

	BOOL        BeginProcessingExt(ULONG nWidth, ULONG nHeight, BOOL bUndo = TRUE);
	void        EndProcessingExt();

	BOOL        BeginProcessingMerge(BOOL bUndo = TRUE);
	void        EndProcessingMerge();

	BYTE _SVHUGE*   GetOrgPointer() { return pOrgArray; }
	BYTE _SVHUGE*   GetDestPointer() { return pDestArray; }
	BYTE _SVHUGE*   GetUndoPointer() { return pUndoArray; }
	BYTE _SVHUGE*   GetMergePointer() { return pMergeArray; }

#if defined(WIN) || defined(WNT)
	HGLOBAL			GetOrgPointerUnchanged() { return (nWhichOrg == 1 ? hArray1 : hArray2); }
#endif 

#if defined(OS2) || defined(UNX) || defined(MAC)
	BYTE _SVHUGE*	GetOrgPointerUnchanged() { return (nWhichOrg == 1 ? pArray1 : pArray2); }
#endif 

	void			SetOrgBitmap(const Bitmap& rBitmap) { pOutBitmap = new Bitmap(rBitmap); }

	BOOL            CreateOutBitmap(const ULONG nColorCount = 256, ULONG nLast = 0, SfxProgress* pProgress = NULL );

	Bitmap          CreateSaveBitmap(const SimDepthType eDepth, SfxViewFrame *pFrame = NULL );

	BOOL            DoUndo( SfxProgress* pProgress = NULL );
	BOOL            DoRedo( SfxProgress* pProgress = NULL );

	Bitmap&         GetOutBitmap() const;

	Bitmap          GetSaveBitmap() { return CreateSaveBitmap( eOrgDepth ); }

	SimPalette*		GetOrgPalette() const { return nWhichOrg == 1 ? pPal1 : pPal2; }
	SimPalette*		GetDestPalette() const { return nWhichOrg == 1 ? pPal2 : pPal1; }
};

BOOL DitherBitmap( Bitmap& rBitmap );

#endif 
#endif 
