
#ifndef _SVX_TBCONTRL_HXX
#define _SVX_TBCONTRL_HXX

#ifndef _RSCSFX_HXX 
#include <rsc/rscsfx.hxx>
#endif
#ifndef _SFXLSTNER_HXX 
#include <svtools/lstner.hxx>
#endif
#ifndef _SFXTBXCTRL_HXX 
#include <sfx2/tbxctrl.hxx>
#endif

class SvxLineItem;
class SvxBoxInfoItem;
class SvxFontItem;
class SvxFontHeightItem;
class SfxStyleControllerItem;
class SfxStyleSheetBasePool;
class SfxTemplateItem;

class SvxStyleToolBoxControl : public SfxToolBoxControl,
				   public SfxListener
{
public:
	SFX_DECL_TOOLBOX_CONTROL();

	SvxStyleToolBoxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBindings );
	~SvxStyleToolBoxControl();

	virtual Window*	CreateItemWindow( Window *pParent );

	virtual void	StateChanged( USHORT nSID, SfxItemState eState,
					  const SfxPoolItem* pState );
	virtual void	SFX_NOTIFY( SfxBroadcaster& rBC,   const TypeId& rBCType,
				const SfxHint&  rHint, const TypeId& rHintType );

private:

#define MAX_FAMILIES 5

	SfxStyleSheetBasePool*		pStyleSheetPool;
	SfxStyleControllerItem* 	pBoundItems [MAX_FAMILIES];
	SfxTemplateItem*			pFamilyState[MAX_FAMILIES];
	USHORT					 	nActFamily;	
	USHORT						nActFilter;	
	USHORT						nAppFilter;	
	String						aCurSel;
	BOOL						bListening;

	void	Update();
	void	FillStyleBox();
	void 	SelectStyle( const String& rStyleName );

friend class SfxStyleControllerItem;

	void			SetFamilyState	( USHORT nIdx, const SfxTemplateItem* pItem );
	SfxStyleFamily	GetActFamily	();
};

#ifdef ITEMID_FONT

class SvxFontNameToolBoxControl : public SfxToolBoxControl
{
public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxFontNameToolBoxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBindings );

	virtual void	StateChanged		( USHORT nSID, SfxItemState eState,
							  const SfxPoolItem* pState );
	virtual Window*	CreateItemWindow	( Window *pParent );
};

#endif 

#ifdef ITEMID_FONTHEIGHT

class SvxFontSizeItem;
class SvxFontItem;
class SvxFontSizeBox;

class SvxFontHeightToolBoxControl : public SfxToolBoxControl
{
	SfxStatusForwarder	aFontNameForwarder;	
	SvxFontSizeBox* 	pBox;				
	SvxFontItem*		pFontItem;			

public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxFontHeightToolBoxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBindings );
	~SvxFontHeightToolBoxControl();

	virtual void 		StateChanged( USHORT nSID, SfxItemState eState,
						   const SfxPoolItem* pState );
	virtual Window*		CreateItemWindow( Window *pParent );

#if 0 // _SOLAR__PRIVATE
	const SvxFontItem*	GetFontItem_Impl()
			{
				GetBindings().Update(SID_ATTR_CHAR_FONT);
				return pFontItem;
			}
#endif
};

#endif 

#ifdef ITEMID_COLOR

class SvxTbxButtonColorUpdater;

class SvxFontColorToolBoxControl : public SfxToolBoxControl
{
	SvxTbxButtonColorUpdater*	pBtnUpdater;

public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxFontColorToolBoxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBindings );
	~SvxFontColorToolBoxControl();

	virtual void				StateChanged( USHORT nSID, SfxItemState eState,
								  const SfxPoolItem* pState );
	virtual SfxPopupWindowType	GetPopupWindowType() const;
	virtual SfxPopupWindow*		CreatePopupWindow();
};

#endif 

#ifdef ITEMID_COLOR

class SvxFontColorExtToolBoxControl : public SfxToolBoxControl
{
	SfxStatusForwarder 			aForward;
	SvxTbxButtonColorUpdater*	pBtnUpdater;

public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxFontColorExtToolBoxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBindings );
	~SvxFontColorExtToolBoxControl();

	virtual void				StateChanged( USHORT nSID, SfxItemState eState,
								  const SfxPoolItem* pState );
	virtual SfxPopupWindowType	GetPopupWindowType() const;
	virtual SfxPopupWindow*		CreatePopupWindow();
	virtual void				Select( BOOL bMod1 = FALSE );

};

#endif 

#ifdef ITEMID_BRUSH

class SvxColorToolBoxControl : public SfxToolBoxControl
{
	SvxTbxButtonColorUpdater*	pBtnUpdater;
public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxColorToolBoxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBindings );
	~SvxColorToolBoxControl();

	virtual void				StateChanged( USHORT nSID, SfxItemState eState,
								  const SfxPoolItem* pState );
	virtual SfxPopupWindowType	GetPopupWindowType() const;
	virtual SfxPopupWindow*		CreatePopupWindow();
};

#endif 

#ifdef ITEMID_BOX

class SvxFrameToolBoxControl : public SfxToolBoxControl
{
public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxFrameToolBoxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBindings );

	virtual SfxPopupWindowType	GetPopupWindowType() const;
	virtual SfxPopupWindow*		CreatePopupWindow();
	virtual void				StateChanged( USHORT nSID, SfxItemState eState,
								  const SfxPoolItem* pState );

private:
	BOOL bParagraphMode;
};

#endif 

#ifdef ITEMID_LINE

class SvxFrameLineStyleToolBoxControl : public SfxToolBoxControl
{
public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxFrameLineStyleToolBoxControl( USHORT nId, ToolBox& rTbx, SfxBindings& rBindings );

	virtual SfxPopupWindowType	GetPopupWindowType() const;
	virtual SfxPopupWindow*		CreatePopupWindow();
	virtual void				StateChanged( USHORT nSID, SfxItemState eState,
								  const SfxPoolItem* pState );
};

#endif 

#ifdef ITEMID_BRUSH

class SvxFrameLineColorToolBoxControl : public SfxToolBoxControl
{
public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxFrameLineColorToolBoxControl( USHORT nId,
						 ToolBox& rTbx,
						 SfxBindings& rBindings );
	~SvxFrameLineColorToolBoxControl();

	virtual void				StateChanged( USHORT nSID, SfxItemState eState,
								  const SfxPoolItem* pState );
	virtual SfxPopupWindowType	GetPopupWindowType() const;
	virtual SfxPopupWindow*		CreatePopupWindow();
};

#endif 

class SvxReloadControllerItem_Impl;
class SvxReloadControllerItem : public SfxToolBoxControl
{
private:
	SvxReloadControllerItem_Impl* pImpl;

public:
	SFX_DECL_TOOLBOX_CONTROL();
	SvxReloadControllerItem( USHORT nId,
					  ToolBox& rTbx,
					  SfxBindings& rBindings );
	~SvxReloadControllerItem();

protected:
	virtual void StateChanged( USHORT nSID, SfxItemState eState,
				   const SfxPoolItem* pState );
};

#endif 

