
#ifndef _XPOLY_HXX
#define _XPOLY_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

class Point;
class Rectangle;
class SvStream;
class Polygon;
class PolyPolygon;
class OutputDevice;

#define XPOLYPOLY_APPEND     0xFFFF
#define XPOLY_APPEND         0xFFFF

#ifdef WIN 
#define XPOLY_MAXPOINTS      8160   
#else
#define XPOLY_MAXPOINTS      0xFFF0 
#endif

enum XPolyFlags { XPOLY_NORMAL, XPOLY_SMOOTH, XPOLY_CONTROL, XPOLY_SYMMTR };

class ImpXPolygon;

class XPolygon
{
protected:
	ImpXPolygon*    pImpXPolygon;

	void    CheckReference();

	void    SubdivideBezier(USHORT nPos, BOOL bCalcFirst, double fT);
	void    GenBezArc(const Point& rCenter, long nRx, long nRy,
		  long nXHdl, long nYHdl, USHORT nStart, USHORT nEnd,
		  USHORT nQuad, USHORT nFirst);
	BOOL    CheckAngles(USHORT& nStart, USHORT nEnd, USHORT& nA1, USHORT& nA2);

public:
	XPolygon( USHORT nSize=16, USHORT nResize=16 );
	XPolygon( const XPolygon& rXPoly );
	XPolygon( const Polygon& rPoly );
	XPolygon( const Rectangle& rRect, long nRx = 0, long nRy = 0 );
	XPolygon( const Point& rCenter, long nRx, long nRy,
	 USHORT nStartAngle = 0, USHORT nEndAngle = 3600,
	 BOOL bClose = TRUE );

	~XPolygon();

	void        SetSize( USHORT nSize );
	USHORT      GetSize() const;

	void        SetPointCount( USHORT nPoints );
	USHORT      GetPointCount() const;

	void        Insert( USHORT nPos, const Point& rPt, XPolyFlags eFlags );
	void        Insert( USHORT nPos, const XPolygon& rXPoly );
	void        Insert( USHORT nPos, const Polygon& rPoly );
	void        Remove( USHORT nPos, USHORT nCount );
	void        Move( long nHorzMove, long nVertMove );
	Rectangle   GetBoundRect(OutputDevice* pOut = NULL) const;

	const Point&    operator[]( USHORT nPos ) const;
	Point&    operator[]( USHORT nPos );
	XPolygon&       operator=( const XPolygon& rXPoly );
	BOOL            operator==( const XPolygon& rXPoly ) const;
	BOOL            operator!=( const XPolygon& rXPoly ) const;

	XPolyFlags  GetFlags( USHORT nPos ) const;
	void        SetFlags( USHORT nPos, XPolyFlags eFlags );
	BOOL        IsControl(USHORT nPos) const;
	BOOL        IsSmooth(USHORT nPos) const;

	double  CalcDistance(USHORT nP1, USHORT nP2);

	void CalcSmoothJoin(USHORT nCenter, USHORT nDrag, USHORT nPnt);
	void CalcTangent(USHORT nCenter, USHORT nPrev, USHORT nNext);
	void PointsToBezier(USHORT nFirst);

	void Translate(const Point& rTrans);
	void Rotate(const Point& rCenter, double fSin, double fCos);
	void Rotate(const Point& rCenter, USHORT nAngle);
	void Scale(double fSx, double fSy);
	void SlantX(long nYRef, double fSin, double fCos);
	void SlantY(long nXRef, double fSin, double fCos);
	void Distort(const Rectangle& rRefRect, const XPolygon& rDistortedRect);
	void Rotate20 ();

	friend SvStream& operator>>( SvStream& rIStream, XPolygon& rXPoly );
	friend SvStream& operator<<( SvStream& rOStream, const XPolygon& rXPoly );
};

class ImpXPolyPolygon;

class XPolyPolygon
{
protected:
	ImpXPolyPolygon* pImpXPolyPolygon;

	void    CheckReference();

public:
		XPolyPolygon( USHORT nInitSize = 16, USHORT nResize = 16 );
		XPolyPolygon( const XPolygon& rXPoly );
		XPolyPolygon( const XPolyPolygon& rXPolyPoly );
		XPolyPolygon( const PolyPolygon& rPolyPoly);
			~XPolyPolygon();

	void            Insert( const XPolygon& rXPoly,
				USHORT nPos = XPOLYPOLY_APPEND );
	void            Insert( const XPolyPolygon& rXPoly,
				USHORT nPos=XPOLYPOLY_APPEND );
	XPolygon        Remove( USHORT nPos );
	XPolygon        Replace( const XPolygon& rXPoly, USHORT nPos );
	const XPolygon& GetObject( USHORT nPos ) const;

	void            Clear();
	USHORT          Count() const;

	void            Move( long nHorzMove, long nVertMove );
	Rectangle       GetBoundRect(OutputDevice* pOut = NULL) const;

	const XPolygon& operator[]( USHORT nPos ) const
			{ return GetObject( nPos ); }
	XPolygon&       operator[]( USHORT nPos );

	XPolyPolygon&   operator=( const XPolyPolygon& rXPolyPoly );
	BOOL            operator==( const XPolyPolygon& rXPolyPoly ) const;
	BOOL            operator!=( const XPolyPolygon& rXPolyPoly ) const;

	void Translate(const Point& rTrans);
	void Rotate(const Point& rCenter, double fSin, double fCos);
	void Rotate(const Point& rCenter, USHORT nAngle);
	void Scale(double fSx, double fSy);
	void SlantX(long nYRef, double fSin, double fCos);
	void SlantY(long nXRef, double fSin, double fCos);
	void Distort(const Rectangle& rRefRect, const XPolygon& rDistortedRect);
	void Rotate20 ();

	friend SvStream& operator>>( SvStream& rIStream, XPolyPolygon& rXPolyPoly );
	friend SvStream& operator<<( SvStream& rOStream, const XPolyPolygon& rXPolyPoly );
};

#endif      

