

#ifndef _TOOLS_BCST_HXX
#define _TOOLS_BCST_HXX

#include <tools/simplecm.hxx>

typedef USHORT BroadcastCategory;
typedef USHORT BroadcastCommand;

#define BCST_FIRST_COMMAND	0
#define BCST_FIRST_CATEGORY	0

#define BCST_CMD_ADD_CATEGORY		((BroadcastCommand)( BCST_FIRST_COMMAND +  1 ))
#define BCST_CMD_REMOVE_CATEGORY	((BroadcastCommand)( BCST_FIRST_COMMAND +  2 ))
#define BCST_CMD_BROADCAST			((BroadcastCommand)( BCST_FIRST_COMMAND +  3 ))
#define BCST_CMD_MAKE_PERSISTENT	((BroadcastCommand)( BCST_FIRST_COMMAND +  4 ))
#define BCST_CMD_CLIENT_ID			((BroadcastCommand)( BCST_FIRST_COMMAND +  5 ))

#define BCST_CAT_INVALID   			((BroadcastCategory)( 0xffff ))

#define BCST_CAT_ALL				((BroadcastCategory)( BCST_FIRST_CATEGORY      ))
#define BCST_CAT_MISC				((BroadcastCategory)( BCST_FIRST_CATEGORY +  1 ))
#define BCST_CAT_PL2X				((BroadcastCategory)( BCST_FIRST_CATEGORY +  2 ))
#define BCST_CAT_MINORCOPY			((BroadcastCategory)( BCST_FIRST_CATEGORY +  3 ))
#define BCST_CAT_DELIVER			((BroadcastCategory)( BCST_FIRST_CATEGORY +  4 ))
#define BCST_CAT_CHAT				((BroadcastCategory)( BCST_FIRST_CATEGORY +  5 ))
#define BCST_CAT_BSTRIGGER			((BroadcastCategory)( BCST_FIRST_CATEGORY +  6 ))
#define BCST_CAT_BSCHAT				((BroadcastCategory)( BCST_FIRST_CATEGORY +  7 ))
#define BCST_CAT_SOURCEACCEPTANCE	((BroadcastCategory)( BCST_FIRST_CATEGORY +  8 ))
#define BCST_CAT_ZIPSTATUS			((BroadcastCategory)( BCST_FIRST_CATEGORY +  9 ))
#define BCST_CAT_ZIPCONFIG			((BroadcastCategory)( BCST_FIRST_CATEGORY + 10 ))
#define BCST_CAT_ZIPPRJ				((BroadcastCategory)( BCST_FIRST_CATEGORY + 11 ))
#define BCST_CAT_BSBUILD			((BroadcastCategory)( BCST_FIRST_CATEGORY + 12 ))
#define BCST_CAT_BUILDMINOR			((BroadcastCategory)( BCST_FIRST_CATEGORY + 13 ))
#define BCST_CAT_PREMINOR			((BroadcastCategory)( BCST_FIRST_CATEGORY + 14 ))
#define BCST_CAT_KILLZIPJOB			((BroadcastCategory)( BCST_FIRST_CATEGORY + 15 ))
#define BCST_CAT_PACKAGERENAMED		((BroadcastCategory)( BCST_FIRST_CATEGORY + 16 ))
#define BCST_CAT_UPD_EVENTS			((BroadcastCategory)( BCST_FIRST_CATEGORY + 17 ))
#define BCST_LAST_CATEGORY			((BroadcastCategory)( BCST_FIRST_CATEGORY + 18 ))

#define BCST_CAT_BSBUILD_MODE_PRIO				"PRIO"

#define BCST_CAT_BSBUILD_MODE_DEPENDENCIES		"DEPENDENCIES"

#define BCST_CAT_BSBUILD_MODE_BUILDANDDELIVER	"DEPENDENCIES_AND_DELIVER"

#define BCST_CAT_BSBUILD_MODE_DELIVER			"DELIVER"

typedef ULONG ClientID;
#define BCST_CLIENT_UNKNOWN		((ClientID)0)
#define BCST_CLIENT_BROOKER		((ClientID)0xFFFFFFFF)

ByteString GetBroadcastHost();
ULONG GetBroadcastPort();

typedef ULONG BroadcastFilterFlag;
#define BCST_FILTER_NOFILTER			((BroadcastFilterFlag) 0x0000)
#define BCST_FILTER_COUNTER			((BroadcastFilterFlag) 0x0001)

class BroadcastMessage;

DECLARE_LIST( LinkList, Link * );
class InformationBroadcaster
{
	LinkList aMessageArrivedLnkList;
	LinkList aConnectionTerminatedLnkList;

	DECL_LINK( ManagerData, CommunicationLink* );
	DECL_LINK( ManagerClose, CommunicationLink* );
protected:

	virtual void IMessageArrived( const BroadcastMessage &aMsg );
	CommunicationManager *mpManager;
private:
	ClientID aThisClientID;				

public:

	InformationBroadcaster( CommunicationManager *pManager = NULL );
	virtual ~InformationBroadcaster();
	BOOL StartCommunicationIfNecessary();

	BOOL AddCategory( const ByteString &aType );
	BOOL AddCategory( BroadcastCategory nType );
	BOOL RemoveCategory( const ByteString &aType );
	BOOL RemoveCategory( BroadcastCategory nType );

	BOOL WaitForData();		

	virtual void MessageArrived( const BroadcastMessage &aMsg );
	virtual void ConnectionTerminated();

	void SetMessageArrivedHdl( Link aLink ) { AddMessageArrivedHdl( aLink ); }
	void AddMessageArrivedHdl( Link aLink ) { aMessageArrivedLnkList.Insert( new Link( aLink ), LIST_APPEND ); }
	void RemoveMessageArrivedHdl( Link aLink );

	void SetConnectionTerminatedHdl( Link aLink ) { AddConnectionTerminatedHdl( aLink ); }
	void AddConnectionTerminatedHdl( Link aLink ) { aConnectionTerminatedLnkList.Insert( new Link( aLink ), LIST_APPEND ); }
	void RemoveConnectionTerminatedHdl( Link aLink );

	BOOL Broadcast( const ByteString &aType, const ByteString &aMsg, BroadcastFilterFlag nFlags = BCST_FILTER_NOFILTER );
	BOOL Broadcast( BroadcastCategory nType, const ByteString &aMsg, BroadcastFilterFlag nFlags = BCST_FILTER_NOFILTER );

	BroadcastCategory Name2Id( const ByteString &aType );
	ByteString Id2Name( BroadcastCategory aType );
	ByteString Description( BroadcastCategory aType );
	ClientID GetClientID() { return aThisClientID; }

	void SetApplication( const ByteString& aApp ) { mpManager->SetApplication( aApp ); }
};

class BroadcastMessage : public ByteString
{
public:
	BroadcastMessage( ByteString nMsg, BroadcastCategory nBC, ClientID aCI = BCST_CLIENT_UNKNOWN ): ByteString( nMsg ), nBroadcastCategory( nBC ), aClientID( aCI ){;}
	BroadcastCategory GetBroadcastCategory() const { return nBroadcastCategory; }
	ClientID GetClientID() { return aClientID; }
	BOOL IsSameClient( BroadcastMessage *pBM ) { return pBM->GetClientID() == GetClientID(); }
	BOOL IsSameClient( InformationBroadcaster *pIB ) { return pIB->GetClientID() == GetClientID(); }
private:
	BroadcastCategory nBroadcastCategory;
	ClientID aClientID;
};

#endif
