

#ifndef _CONFIG_HXX
#define _CONFIG_HXX

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

struct ImplConfigData;
struct ImplGroupData;

class Config
{
private:
	XubString			maFileName;
	ByteString			maGroupName;
	ImplConfigData* 	mpData;
	ImplGroupData*		mpActGroup;
	ULONG				mnDataUpdateId;
	USHORT				mnLockCount;
	BOOL				mbPersistence;
	BOOL				mbDummy1;

#ifdef _CONFIG_CXX
	BOOL				ImplUpdateConfig() const;
	ImplGroupData*		ImplGetGroup() const;
#endif

public:
			Config();
			Config( const XubString& rFileName );
			~Config();

	virtual void		Modified();

	const XubString&	GetPathName() const { return maFileName; }
	static XubString	GetDefDirectory();
	static XubString	GetConfigName( const XubString& rPath, const XubString& rBaseName );

	void				SetGroup( const ByteString& rGroup );
	const ByteString&	GetGroup() const { return maGroupName; }
	void				DeleteGroup( const ByteString& rGroup );
	ByteString			GetGroupName( USHORT nGroup ) const;
	USHORT				GetGroupCount() const;
	BOOL				HasGroup( const ByteString& rGroup ) const;

	ByteString			ReadKey( const ByteString& rKey ) const;
	UniString			ReadKey( const ByteString& rKey, rtl_TextEncoding eEncoding ) const;
	ByteString			ReadKey( const ByteString& rKey, const ByteString& rDefault ) const;
	void				WriteKey( const ByteString& rKey, const ByteString& rValue );
	void				WriteKey( const ByteString& rKey, const UniString& rValue, rtl_TextEncoding eEncoding );
	void				DeleteKey( const ByteString& rKey );
	ByteString			GetKeyName( USHORT nKey ) const;
	ByteString			ReadKey( USHORT nKey ) const;
	USHORT				GetKeyCount() const;

	void				EnterLock();
	void				LeaveLock();
	BOOL				IsLocked() const { return (mnLockCount != 0); }
	BOOL				Update();
	void				Flush();

	void				EnablePersistence( BOOL bPersistence = TRUE )
				{ mbPersistence = bPersistence; }
	BOOL				IsPersistenceEnabled() const { return mbPersistence; }

	void				SetLineEnd( LineEnd eLineEnd );
	LineEnd 			GetLineEnd() const;

private:
			Config( const Config& rConfig );
	Config& 			operator = ( const Config& rConfig );
};

#endif 
