

#ifndef _DATE_HXX
#define _DATE_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

class ResId;

enum WeekCountStart { WEEKCOUNT_FIRSTDAY,
		  WEEKCOUNT_FIRST4DAYWEEK,
		  WEEKCOUNT_FIRSTFULLWEEK };

enum DayOfWeek { MONDAY, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY,
	 SATURDAY, SUNDAY };

class Date
{
private:
	ULONG			nDate;

public:
		Date();
		Date( const ResId & rResId );
		Date( ULONG _nDate ) { Date::nDate = _nDate; }
		Date( const Date& rDate )
			{ nDate = rDate.nDate; }
		Date( USHORT nDay, USHORT nMonth, USHORT nYear )
			{ nDate = ((ULONG)(nDay%100)) +
					  (((ULONG)(nMonth%100))*100) +
					  (((ULONG)nYear%10000)*10000); }

	void			SetDate( ULONG nNewDate ) { nDate = nNewDate; }
	ULONG			GetDate() const { return nDate; }

	void			SetDay( USHORT nNewDay );
	void			SetMonth( USHORT nNewMonth );
	void			SetYear( USHORT nNewYear );
	USHORT			GetDay() const { return (USHORT)(nDate % 100); }
	USHORT			GetMonth() const { return (USHORT)((nDate / 100) % 100); }
	USHORT			GetYear() const { return (USHORT)(nDate / 10000); }

	DayOfWeek		GetDayOfWeek() const;
	USHORT			GetDayOfYear() const;
	USHORT			GetWeekOfYear( DayOfWeek eStartDay = MONDAY,
					   WeekCountStart eWeekStart = WEEKCOUNT_FIRST4DAYWEEK ) const;

	USHORT			GetDaysInMonth() const;
	USHORT			GetDaysInYear() const { return (IsLeapYear()) ? 366 : 365; }
	BOOL			IsLeapYear() const;
	BOOL			IsValid() const;

	BOOL			IsBetween( const Date& rFrom, const Date& rTo ) const
			{ return ((nDate >= rFrom.nDate) &&
					 (nDate <= rTo.nDate)); }

	BOOL			operator ==( const Date& rDate ) const
			{ return (nDate == rDate.nDate); }
	BOOL			operator !=( const Date& rDate ) const
			{ return (nDate != rDate.nDate); }
	BOOL			operator  >( const Date& rDate ) const
			{ return (nDate > rDate.nDate); }
	BOOL			operator  <( const Date& rDate ) const
			{ return (nDate < rDate.nDate); }
	BOOL			operator >=( const Date& rDate ) const
			{ return (nDate >= rDate.nDate); }
	BOOL			operator <=( const Date& rDate ) const
			{ return (nDate <= rDate.nDate); }

	Date&			operator =( const Date& rDate )
			{ nDate = rDate.nDate; return *this; }
	Date&			operator +=( long nDays );
	Date&			operator -=( long nDays );
	Date&			operator ++();
	Date&			operator --();
#ifndef MPW33
	Date			operator ++( int );
	Date			operator --( int );
#endif

	friend Date 	operator +( const Date& rDate, long nDays );
	friend Date 	operator -( const Date& rDate, long nDays );
	friend long 	operator -( const Date& rDate1, const Date& rDate2 );

#ifdef SMART_UNO_GENERATION
	UINT16			getDay() const { return GetDay(); }
	UINT16			getMonth() const { return GetMonth(); }
	UINT16			getYear() const { return GetYear(); }

	void			setDay( UINT16 nDay ) { SetDay( nDay ); }
	void			setMonth( UINT16 nMonth ) { SetMonth( nMonth ); }
	void			setYear( UINT16 nYear ) { SetYear( nYear ); }

	void			setDate( UINT32 nDate ) { SetDate( nDate ); }
	UINT32			getDate() const { return GetDate(); }
#endif
};

#endif 
