

#ifndef __RSC

#ifndef _EINF_HXX
#define _EINF_HXX

#include <limits.h>
#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif
#ifndef _ERRCODE_HXX
#include <tools/errcode.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

class EDcr_Impl;
class ErrHdl_Impl;
class Window;

class ErrorInfo
{
private:
	ULONG                   lUserId;

public:
	                        TYPEINFO();
	                         ErrorInfo( ULONG lArgUserId ) :
	                            lUserId( lArgUserId ){}
	virtual                 ~ErrorInfo(){}
	 ULONG                   GetErrorCode() const { return lUserId; }
	 static ErrorInfo*       GetErrorInfo(ULONG);
};

class DynamicErrorInfo : public ErrorInfo
{
	friend class EDcr_Impl;

private:
	EDcr_Impl*              pImpl;

public:
	                        TYPEINFO();
	                         DynamicErrorInfo(ULONG lUserId, USHORT nMask);
	virtual                 ~DynamicErrorInfo();
	 operator                ULONG() const;
	USHORT                  GetDialogMask() const;
};

class StandardErrorInfo : public DynamicErrorInfo
{
private:
	ULONG                   lExtId;

public:
	                        TYPEINFO();
	                         StandardErrorInfo( ULONG lUserId, ULONG lExtId,
								  USHORT nFlags = 0);
	ULONG                   GetExtendedErrorCode() const { return lExtId; }

};

class StringErrorInfo : public DynamicErrorInfo
{
private:
	String                  aString;

public:
	                        TYPEINFO();
	                         StringErrorInfo( ULONG lUserId,
								const String& aStringP,
								USHORT nFlags = 0);
	const String&           GetErrorString() const { return aString; }
};

class TwoStringErrorInfo: public DynamicErrorInfo
{
private:
	String aArg1;
	String aArg2;

public:
	TYPEINFO();

	TwoStringErrorInfo(ULONG nUserID, const String & rTheArg1,
		   const String & rTheArg2, USHORT nFlags = 0):
	 DynamicErrorInfo(nUserID, nFlags), aArg1(rTheArg1), aArg2(rTheArg2) {}
	virtual ~TwoStringErrorInfo() {}

	String GetArg1() const { return aArg1; }
	String GetArg2() const { return aArg2; }
};

class MessageInfo : public DynamicErrorInfo
{
  public:
	                      TYPEINFO();
	                     MessageInfo(ULONG lUserId, USHORT nFlags = 0) :
				    DynamicErrorInfo(lUserId, nFlags){}
		        MessageInfo(ULONG lUserId, const String &rArg,
							USHORT nFlags = 0 ) :
	                         DynamicErrorInfo(lUserId, nFlags), aArg(rArg) {}
	const String&           GetMessageArg() const { return aArg; }

  private:

	String           aArg;
};

class ErrorContext
{
	friend class ErrorHandler;

private:
	ErrorContext*           pNext;
	Window*                 pWin;

public:
	                        ErrorContext(Window *pWin=0);
	                        ~ErrorContext();
	 virtual BOOL            GetString( ULONG nErrId, String& rCtxStr ) = 0;
	Window*                 GetParent() { return pWin; }
	 static ErrorContext*    GetContext();
};

typedef USHORT WindowDisplayErrorFunc(
	Window *, USHORT nMask, const String &rErr, const String &rAction);

typedef void BasicDisplayErrorFunc(
	const String &rErr, const String &rAction);

class ErrorHandler
{
	friend class ErrHdl_Impl;

private:
	ErrHdl_Impl*        pImpl;

protected:
	virtual BOOL        CreateString( const ErrorInfo *, 
						 String &, USHORT& nMask ) const = 0;
	        BOOL        ForwCreateString( const ErrorInfo*, 
							 String&, USHORT& nMask ) const;

public:
	                    ErrorHandler();
	virtual             ~ErrorHandler();
	 static USHORT       HandleError( ULONG lId, USHORT nMask = USHRT_MAX );
	 static void         RegisterDisplay( BasicDisplayErrorFunc* );
	static void         RegisterDisplay( WindowDisplayErrorFunc* );
};

class SimpleErrorHandler : private ErrorHandler
{
protected:
	virtual BOOL        CreateString( const ErrorInfo*, String &, 
						  USHORT &nMask ) const;

public:
	                    SimpleErrorHandler();

};

#endif
#endif
