

#ifndef _IICLIENT_HXX
#define _IICLIENT_HXX

#ifndef _STCPCON_HXX
#include <tools/stcpcon.hxx>
#endif

#include <tools/sasyncfn.hxx>

class SvMemoryStream;
class InfoDataPack;
class GenericInformation;
class TcpConBase;

const ULONG nSvMemoryInitSize = 32 * 1024;
const ULONG nSvMemoryReSize   = 32 * 1024;

class InformationClient

{
private:
  enum SendMode { request, request_list, request_whole_list, infosend, infosend_new, command, good_handshake, bad_handshake, user };

	Link aConnectionClosedHdl;

protected:
InformationClient(
	 const ByteString &rRequest,
	 BOOL bRequestList,
	 BOOL bRequestWholeList );

public:
virtual ~InformationClient();

  enum Error_Code { none,  server_shutdown, no_info };

  BOOL CnctIsValid();	 

  void SetRequest( const ByteString &rRequest,
	 BOOL bListRequest = FALSE,
	 BOOL bWholeList = FALSE )
  { sRequest = rRequest;  eSendMode = (bListRequest?(bWholeList?request_whole_list:request_list):request); }

  void SetCommand( const ByteString &rCommand,
	 const ByteString &rOptions = ByteString() )
  { sRequest = rCommand; sRequestOptions = rOptions; eSendMode = command; }
  void SetInfo( const ByteString &rKey,
		const ByteString &rValue,
		BOOL bNewPath )
  { sRequest = rKey; sRequestOptions = rValue;  eSendMode = (bNewPath?infosend_new:infosend); }

  void SetData( InfoDataPack *pData );
  long int SendData(); 

  GenericInformation *GetInfo( BOOL bAquireOwnership = FALSE )
	{
		if ( bAquireOwnership )
		{
			GenericInformation *pTemp = pGenInfo;
			pGenInfo = NULL;
			return pTemp;
		}
		else
			return pGenInfo;
	}

  BOOL InfoIsValid() const { return bValid; }
  BOOL CommandAcknowledged() const { return bValid; }
  BOOL CommandAknowledge() const { DBG_ERROR("Command misspelled: deprecated! Use CommandAcknowledged") return bValid; }
  Error_Code GetError() const { return eErrorCode; }

  virtual const GenericInformation *Send_n_Wait()=0;

  virtual BOOL Command_n_Wait()=0;

  void SetGotInfoHdl ( Link &rLink ) { aGotInfoHdl = rLink; }

  virtual BOOL Notify_Me( const ByteString &rFileName )=0; 
  virtual BOOL Denotify_Me( const ByteString &rFileName )=0;

  virtual void SetNotifyHdl ( Link &rLink )=0;

  virtual const ByteString GetNotifyFile() const=0;

  void SetConnectionClosedHdl( const Link &rLink ) { aConnectionClosedHdl = rLink; }

	void StartCommunication() { pTcpCon->StartCommunication(); }
	ClientContext *GetClientContext() { return pTcpCon->GetClientContext(); }

	long int SendData( DataPack* pDPack ) { return pTcpCon->SendData( pDPack ); }

	ByteString&	GetAddress() { return pTcpCon->GetAddress(); }
	ByteString& GetTcpNr() { return pTcpCon->GetTcpNr(); }

	BOOL	IsServer() { return pTcpCon->IsServer(); }
	BOOL	ClientHasCon() { return pTcpCon->ClientHasCon(); }

	void 	SetStatusHdl( const Link &rLink ) { pTcpCon->SetStatusHdl( rLink ); }

	AsyncFuncsBase *GetAsyncFuncs() { return pAsyncFuncs; }

	void SetApplication( const ByteString& aApp ) { pTcpCon->SetApplication( aApp ); }

protected:

	void LateInit();

	AsyncFuncsBase *pAsyncFuncs;

  virtual DECL_LINK( PacketRecieved, DataPack* );
  virtual DECL_LINK( ConnectionOpened, ClientContext* );
  virtual DECL_LINK( ConnectionClosed, ClientContext* );
  void SendHandShake( BOOL bSuccess = TRUE );

  TcpConBase *pTcpCon;

  InfoDataPack *pInfoPack;

  Link   aGotInfoHdl;
  ByteString sRequest, sRequestOptions;
  GenericInformation *pGenInfo;
  SvMemoryStream *pWholeListStream;
  int    nRetries;
  BOOL   bSend, bRecieved, bValid, bConnectionBad; 
  UINT16 nLastPaket; 
  Error_Code eErrorCode;
  SendMode   eSendMode;

};

#endif
