

#ifndef TOOLS_INETMIME_HXX
#define TOOLS_INETMIME_HXX

#ifndef _RTL_ALLOC_H_
#include <rtl/alloc.h>
#endif
#ifndef _RTL_STRING_H_
#include <rtl/string.h>
#endif

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _ERRCODE_HXX
#include <tools/errcode.hxx>
#endif
#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

class DateTime;
class INetContentTypeParameterList;
class INetMIMECharsetList_Impl;
class INetMIMEOutputSink;

class INetMIME
{
public:
	enum { SOFT_LINE_LENGTH_LIMIT = 76,
	 HARD_LINE_LENGTH_LIMIT = 998 };

	enum HeaderFieldType
	{
		HEADER_FIELD_TEXT,
		HEADER_FIELD_STRUCTURED,
		HEADER_FIELD_PHRASE,
		HEADER_FIELD_MESSAGE_ID,
		HEADER_FIELD_ADDRESS
	};

	static inline bool isUSASCII(sal_uInt32 nChar);

	static inline bool isISO88591(sal_uInt32 nChar);

	static inline bool isControl(sal_uInt32 nChar);

	static inline bool isWhiteSpace(sal_uInt32 nChar);

	static inline bool isVisible(sal_uInt32 nChar);

	static inline bool isDigit(sal_uInt32 nChar);

	static inline bool isCanonicHexDigit(sal_uInt32 nChar);

	static inline bool isHexDigit(sal_uInt32 nChar);

	static inline bool isUpperCase(sal_uInt32 nChar);

	static inline bool isLowerCase(sal_uInt32 nChar);

	static inline bool isAlpha(sal_uInt32 nChar);

	static inline bool isAlphanumeric(sal_uInt32 nChar);

	static inline bool isBase64Digit(sal_uInt32 nChar);

	static bool isAtomChar(sal_uInt32 nChar);

	static bool isTokenChar(sal_uInt32 nChar);

	static bool isEncodedWordTokenChar(sal_uInt32 nChar);

	static bool isIMAPAtomChar(sal_uInt32 nChar);

	static inline sal_uInt32 toUpperCase(sal_uInt32 nChar);

	static inline sal_uInt32 toLowerCase(sal_uInt32 nChar);

	static inline int getWeight(sal_uInt32 nChar);

	static inline int getHexWeight(sal_uInt32 nChar);

	static inline int getBase64Weight(sal_uInt32 nChar);

	static sal_uInt32 getDigit(int nWeight);

	static sal_uInt32 getHexDigit(int nWeight);

	static sal_uInt32 getBase64Digit(int nWeight);

	static inline bool isHighSurrogate(sal_uInt32 nUTF16);

	static inline bool isLowSurrogate(sal_uInt32 nUTF16);

	static inline sal_uInt32 toUTF32(sal_Unicode cHighSurrogate,
						 sal_Unicode cLowSurrogate);

	static bool equalIgnoreCase(const sal_Char * pBegin1,
					const sal_Char * pEnd1,
					const sal_Char * pBegin2,
					const sal_Char * pEnd2);

	static bool equalIgnoreCase(const sal_Char * pBegin1,
					const sal_Char * pEnd1,
					const sal_Char * pString2);

	static bool equalIgnoreCase(const sal_Unicode * pBegin1,
					const sal_Unicode * pEnd1,
					const sal_Char * pString2);

	static inline bool equalIgnoreCase(const ByteString & rString1,
						   const sal_Char * pString2);

	static inline bool startsWithLineBreak(const sal_Char * pBegin,
							   const sal_Char * pEnd);

	static inline bool startsWithLineBreak(const sal_Unicode * pBegin,
							   const sal_Unicode * pEnd);

	static inline bool startsWithLineFolding(const sal_Char * pBegin,
								 const sal_Char * pEnd);

	static inline bool startsWithLineFolding(const sal_Unicode * pBegin,
								 const sal_Unicode * pEnd);

	static bool startsWithLinearWhiteSpace(const sal_Char * pBegin,
							   const sal_Char * pEnd);

	static const sal_Char * skipLinearWhiteSpace(const sal_Char * pBegin,
									 const sal_Char * pEnd);

	static const sal_Unicode * skipLinearWhiteSpace(const sal_Unicode *
										    pBegin,
										const sal_Unicode * pEnd);

	static const sal_Char * skipComment(const sal_Char * pBegin,
							const sal_Char * pEnd);

	static const sal_Unicode * skipComment(const sal_Unicode * pBegin,
							   const sal_Unicode * pEnd);

	static const sal_Char * skipLinearWhiteSpaceComment(const sal_Char *
											    pBegin,
											const sal_Char *
											    pEnd);

	static const sal_Unicode * skipLinearWhiteSpaceComment(const sal_Unicode *
											       pBegin,
											   const sal_Unicode *
											       pEnd);

	static inline bool needsQuotedStringEscape(sal_uInt32 nChar);

	static const sal_Char * skipQuotedString(const sal_Char * pBegin,
								 const sal_Char * pEnd);

	static const sal_Unicode * skipQuotedString(const sal_Unicode * pBegin,
									const sal_Unicode * pEnd);

	static const sal_Char * scanAtom(const sal_Char * pBegin,
						 const sal_Char * pEnd);

	static const sal_Unicode * scanAtom(const sal_Unicode * pBegin,
							const sal_Unicode * pEnd);

	static bool scanUnsigned(const sal_Char *& rBegin, const sal_Char * pEnd,
				 bool bLeadingZeroes, sal_uInt32 & rValue);

	static bool scanUnsigned(const sal_Unicode *& rBegin,
				 const sal_Unicode * pEnd, bool bLeadingZeroes,
				 sal_uInt32 & rValue);

	static bool scanUnsignedHex(const sal_Char *& rBegin,
					const sal_Char * pEnd, bool bLeadingZeroes,
					sal_uInt32 & rValue);

	static bool scanUnsignedHex(const sal_Unicode *& rBegin,
					const sal_Unicode * pEnd, bool bLeadingZeroes,
					sal_uInt32 & rValue);

	static const sal_Char * scanQuotedBlock(const sal_Char * pBegin,
								const sal_Char * pEnd,
								sal_uInt32 nOpening,
								sal_uInt32 nClosing,
								sal_Size & rLength,
								bool & rModify);

	static const sal_Unicode * scanQuotedBlock(const sal_Unicode * pBegin,
								   const sal_Unicode * pEnd,
								   sal_uInt32 nOpening,
								   sal_uInt32 nClosing,
								   sal_Size & rLength,
								   bool & rModify);

	static sal_Char const * scanParameters(sal_Char const * pBegin,
							   sal_Char const * pEnd,
							   INetContentTypeParameterList *
							       pParameters);

	static sal_Unicode const * scanParameters(sal_Unicode const * pBegin,
								  sal_Unicode const * pEnd,
								  INetContentTypeParameterList *
								      pParameters);

	static inline rtl_TextEncoding translateToMIME(rtl_TextEncoding
									       eEncoding);

	static inline rtl_TextEncoding translateFromMIME(rtl_TextEncoding
										     eEncoding);

	static const sal_Char * getCharsetName(rtl_TextEncoding eEncoding);

	static rtl_TextEncoding getCharsetEncoding(const sal_Char * pBegin,
								   const sal_Char * pEnd);

	static rtl_TextEncoding getCharsetEncoding(const sal_Unicode * pBegin,
								   const sal_Unicode * pEnd);

	static inline bool isMIMECharsetEncoding(rtl_TextEncoding eEncoding);

	static INetMIMECharsetList_Impl *
	createPreferredCharsetList(rtl_TextEncoding eEncoding);

	static sal_Unicode * convertToUnicode(const sal_Char * pBegin,
							  const sal_Char * pEnd,
							  rtl_TextEncoding eEncoding,
							  sal_Size & rSize);

	static sal_Char * convertFromUnicode(const sal_Unicode * pBegin,
							 const sal_Unicode * pEnd,
							 rtl_TextEncoding eEncoding,
							 sal_Size & rSize);

	static inline int getUTF8OctetCount(sal_uInt32 nChar);

	static inline void writeEscapeSequence(INetMIMEOutputSink & rSink,
							   sal_uInt32 nChar);

	static void writeUTF8(INetMIMEOutputSink & rSink, sal_uInt32 nChar);

	static void writeUnsigned(INetMIMEOutputSink & rSink, sal_uInt32 nValue,
				  int nMinDigits = 1);

	static void writeDateTime(INetMIMEOutputSink & rSink,
				  const DateTime & rUTC);

	static void writeHeaderFieldBody(INetMIMEOutputSink & rSink,
						 HeaderFieldType eType,
						 const ByteString & rBody,
						 rtl_TextEncoding ePreferredEncoding,
						 bool bInitialSpace = true);

	static void writeHeaderFieldBody(INetMIMEOutputSink & rSink,
						 HeaderFieldType eType,
						 const UniString & rBody,
						 rtl_TextEncoding ePreferredEncoding,
						 bool bInitialSpace = true);

	static bool translateUTF8Char(const sal_Char *& rBegin,
					  const sal_Char * pEnd,
					  rtl_TextEncoding eEncoding,
					  sal_uInt32 & rCharacter);

	static ByteString decodeUTF8(const ByteString & rText,
					 rtl_TextEncoding eEncoding);

	static UniString decodeHeaderFieldBody(HeaderFieldType eType,
							   const ByteString & rBody);

	static inline sal_uInt32 getUTF32Character(const sal_Unicode *& rBegin,
								   const sal_Unicode * pEnd);

	static inline sal_Unicode * putUTF32Character(sal_Unicode * pBuffer,
									  sal_uInt32 nUTF32);
};

inline bool INetMIME::isUSASCII(sal_uInt32 nChar)
{
	return nChar <= 0x7F;
}

inline bool INetMIME::isISO88591(sal_uInt32 nChar)
{
	return nChar <= 0xFF;
}

inline bool INetMIME::isControl(sal_uInt32 nChar)
{
	return nChar <= 0x1F || nChar == 0x7F;
}

inline bool INetMIME::isWhiteSpace(sal_uInt32 nChar)
{
	return nChar == '\t' || nChar == ' ';
}

inline bool INetMIME::isVisible(sal_uInt32 nChar)
{
	return nChar >= '!' && nChar <= '~';
}

inline bool INetMIME::isDigit(sal_uInt32 nChar)
{
	return nChar >= '0' && nChar <= '9';
}

inline bool INetMIME::isCanonicHexDigit(sal_uInt32 nChar)
{
	return isDigit(nChar) || nChar >= 'A' && nChar <= 'F';
}

inline bool INetMIME::isHexDigit(sal_uInt32 nChar)
{
	return isCanonicHexDigit(nChar) || nChar >= 'a' && nChar <= 'f';
}

inline bool INetMIME::isUpperCase(sal_uInt32 nChar)
{
	return nChar >= 'A' && nChar <= 'Z';
}

inline bool INetMIME::isLowerCase(sal_uInt32 nChar)
{
	return nChar >= 'a' && nChar <= 'z';
}

inline bool INetMIME::isAlpha(sal_uInt32 nChar)
{
	return isUpperCase(nChar) || isLowerCase(nChar);
}

inline bool INetMIME::isAlphanumeric(sal_uInt32 nChar)
{
	return isAlpha(nChar) || isDigit(nChar);
}

inline bool INetMIME::isBase64Digit(sal_uInt32 nChar)
{
	return isUpperCase(nChar) || isLowerCase(nChar) || isDigit(nChar)
	    || nChar == '+' || nChar == '/';
}

inline sal_uInt32 INetMIME::toUpperCase(sal_uInt32 nChar)
{
	return isLowerCase(nChar) ? nChar - ('a' - 'A') : nChar;
}

inline sal_uInt32 INetMIME::toLowerCase(sal_uInt32 nChar)
{
	return isUpperCase(nChar) ? nChar + ('a' - 'A') : nChar;
}

inline int INetMIME::getWeight(sal_uInt32 nChar)
{
	return isDigit(nChar) ? int(nChar - '0') : -1;
}

inline int INetMIME::getHexWeight(sal_uInt32 nChar)
{
	return isDigit(nChar) ? int(nChar - '0') :
	    nChar >= 'A' && nChar <= 'F' ? int(nChar - 'A' + 10) :
	    nChar >= 'a' && nChar <= 'f' ? int(nChar - 'a' + 10) : -1;
}

inline int INetMIME::getBase64Weight(sal_uInt32 nChar)
{
	return isUpperCase(nChar) ? int(nChar - 'A') :
	    isLowerCase(nChar) ? int(nChar - 'a' + 26) :
	    isDigit(nChar) ? int(nChar - '0' + 52) :
	    nChar == '+' ? 62 :
	    nChar == '/' ? 63 :
	    nChar == '=' ? -1 : -2;
}

inline bool INetMIME::isHighSurrogate(sal_uInt32 nUTF16)
{
	return nUTF16 >= 0xD800 && nUTF16 <= 0xDBFF;
}

inline bool INetMIME::isLowSurrogate(sal_uInt32 nUTF16)
{
	return nUTF16 >= 0xDC00 && nUTF16 <= 0xDFFF;
}

inline sal_uInt32 INetMIME::toUTF32(sal_Unicode cHighSurrogate,
						sal_Unicode cLowSurrogate)
{
	DBG_ASSERT(isHighSurrogate(cHighSurrogate)
	  && isLowSurrogate(cLowSurrogate),
	  "INetMIME::toUTF32(): Bad chars");
	return (sal_uInt32(cHighSurrogate) & 0x3FF) << 10
	     | sal_uInt32(cLowSurrogate) & 0x3FF;
}

inline bool INetMIME::equalIgnoreCase(const ByteString & rString1,
						  const sal_Char * pString2)
{
	return equalIgnoreCase(rString1.GetBuffer(),
			   rString1.GetBuffer() + rString1.Len(), pString2);
}

inline bool INetMIME::startsWithLineBreak(const sal_Char * pBegin,
							  const sal_Char * pEnd)
{
	DBG_ASSERT(pBegin && pBegin <= pEnd,
	  "INetMIME::startsWithLineBreak(): Bad sequence");

	return pEnd - pBegin >= 2 && pBegin[0] == 0x0D && pBegin[1] == 0x0A;

}

inline bool INetMIME::startsWithLineBreak(const sal_Unicode * pBegin,
								  const sal_Unicode * pEnd)
{
	DBG_ASSERT(pBegin && pBegin <= pEnd,
	  "INetMIME::startsWithLineBreak(): Bad sequence");

	return pEnd - pBegin >= 2 && pBegin[0] == 0x0D && pBegin[1] == 0x0A;

}

inline bool INetMIME::startsWithLineFolding(const sal_Char * pBegin,
								const sal_Char * pEnd)
{
	DBG_ASSERT(pBegin && pBegin <= pEnd,
	  "INetMIME::startsWithLineFolding(): Bad sequence");

	return pEnd - pBegin >= 3 && pBegin[0] == 0x0D && pBegin[1] == 0x0A
	    && isWhiteSpace(pBegin[2]); 
}

inline bool INetMIME::startsWithLineFolding(const sal_Unicode * pBegin,
								const sal_Unicode * pEnd)
{
	DBG_ASSERT(pBegin && pBegin <= pEnd,
	  "INetMIME::startsWithLineFolding(): Bad sequence");

	return pEnd - pBegin >= 3 && pBegin[0] == 0x0D && pBegin[1] == 0x0A
	    && isWhiteSpace(pBegin[2]); 
}

inline bool INetMIME::startsWithLinearWhiteSpace(const sal_Char * pBegin,
									 const sal_Char * pEnd)
{
	DBG_ASSERT(pBegin && pBegin <= pEnd,
	  "INetMIME::startsWithLinearWhiteSpace(): Bad sequence");

	return pBegin != pEnd
	    && (isWhiteSpace(*pBegin) || startsWithLineFolding(pBegin, pEnd));
}

inline bool INetMIME::needsQuotedStringEscape(sal_uInt32 nChar)
{
	return nChar == '"' || nChar == '\\';
}

inline rtl_TextEncoding INetMIME::translateToMIME(rtl_TextEncoding eEncoding)
{
#if defined WNT
	return eEncoding == RTL_TEXTENCODING_MS_1252 ?
	     RTL_TEXTENCODING_ISO_8859_1 : eEncoding;
#else 
	return eEncoding;
#endif 
}

inline rtl_TextEncoding INetMIME::translateFromMIME(rtl_TextEncoding
										    eEncoding)
{
#if defined WNT
	return eEncoding == RTL_TEXTENCODING_ISO_8859_1 ?
	     RTL_TEXTENCODING_MS_1252 : eEncoding;
#else 
	return eEncoding;
#endif 
}

inline bool INetMIME::isMIMECharsetEncoding(rtl_TextEncoding eEncoding)
{
	return eEncoding > RTL_TEXTENCODING_DONTKNOW
	    && eEncoding < RTL_TEXTENCODING_STD_COUNT;
}

inline int INetMIME::getUTF8OctetCount(sal_uInt32 nChar)
{
	DBG_ASSERT(nChar < 0x80000000, "INetMIME::getUTF8OctetCount(): Bad char");

	return nChar < 0x80 ? 1 :
	    nChar < 0x800 ? 2 :
	    nChar <= 0x10000 ? 3 :
	    nChar <= 0x200000 ? 4 :
	    nChar <= 0x4000000 ? 5 : 6;
}

inline sal_uInt32 INetMIME::getUTF32Character(const sal_Unicode *& rBegin,
								  const sal_Unicode * pEnd)
{
	DBG_ASSERT(rBegin && rBegin < pEnd,
	  "INetMIME::getUTF32Character(): Bad sequence");
	if (rBegin + 1 < pEnd && rBegin[0] >= 0xD800 && rBegin[0] <= 0xDBFF
		&& rBegin[1] >= 0xDC00 && rBegin[1] <= 0xDFFF)
	{
		sal_uInt32 nUTF32 = sal_uInt32(*rBegin++ & 0x3FF) << 10;
		return (nUTF32 | (*rBegin++ & 0x3FF)) + 0x10000;
	}
	else
		return *rBegin++;
}

inline sal_Unicode * INetMIME::putUTF32Character(sal_Unicode * pBuffer,
									 sal_uInt32 nUTF32)
{
	DBG_ASSERT(nUTF32 <= 0x10FFFF, "INetMIME::putUTF32Character(): Bad char");
	if (nUTF32 < 0x10000)
		*pBuffer++ = sal_Unicode(nUTF32);
	else
	{
		nUTF32 -= 0x10000;
		*pBuffer++ = sal_Unicode(0xD800 | nUTF32 >> 10);
		*pBuffer++ = sal_Unicode(0xDC00 | nUTF32 & 0x3FF);
	}
	return pBuffer;
}

class INetMIMEOutputSink
{
public:
	enum { NO_LINE_LENGTH_LIMIT = 0xFFFFFFFF };

private:
	sal_uInt32 m_nColumn;
	sal_uInt32 m_nLineLengthLimit;

protected:

	virtual void writeSequence(const sal_Char * pBegin,
				   const sal_Char * pEnd) = 0;

	virtual sal_Size writeSequence(const sal_Char * pSequence);

	virtual void writeSequence(const sal_uInt32 * pBegin,
				   const sal_uInt32 * pEnd);

	virtual void writeSequence(const sal_Unicode * pBegin,
				   const sal_Unicode * pEnd);

public:
	INetMIMEOutputSink(sal_uInt32 nTheColumn = 0,
		   sal_uInt32 nTheLineLengthLimit
		       = INetMIME::SOFT_LINE_LENGTH_LIMIT):
		m_nColumn(nTheColumn), m_nLineLengthLimit(nTheLineLengthLimit) {}

	virtual ~INetMIMEOutputSink() {}

	sal_uInt32 getColumn() const { return m_nColumn; }

	sal_uInt32 getLineLengthLimit() const { return m_nLineLengthLimit; }

	void setLineLengthLimit(sal_uInt32 nTheLineLengthLimit)
	{ m_nLineLengthLimit = nTheLineLengthLimit; }

	virtual ErrCode getError() const;

	inline void write(const sal_Char * pBegin, const sal_Char * pEnd);

	void write(const sal_Char * pBegin, sal_Size nLength)
	{ write(pBegin, pBegin + nLength); }

	inline void write(const sal_uInt32 * pBegin, const sal_uInt32 * pEnd);

	inline void write(const sal_Unicode * pBegin, const sal_Unicode * pEnd);

	inline void write(const ByteString & rString, xub_StrLen nBegin,
		  xub_StrLen nEnd);

	inline INetMIMEOutputSink & operator <<(sal_Char nOctet);

	inline INetMIMEOutputSink & operator <<(const sal_Char * pOctets);

	inline INetMIMEOutputSink & operator <<(const ByteString & rOctets);

	INetMIMEOutputSink &
	operator <<(INetMIMEOutputSink & (* pManipulator)(INetMIMEOutputSink &))
	{ return pManipulator(*this); }

	void writeLineEnd();

	static inline INetMIMEOutputSink & endl(INetMIMEOutputSink & rSink);
};

inline void INetMIMEOutputSink::write(const sal_Char * pBegin,
						  const sal_Char * pEnd)
{
	writeSequence(pBegin, pEnd);
	m_nColumn += pEnd - pBegin;
}

inline void INetMIMEOutputSink::write(const sal_uInt32 * pBegin,
						  const sal_uInt32 * pEnd)
{
	writeSequence(pBegin, pEnd);
	m_nColumn += pEnd - pBegin;
}

inline void INetMIMEOutputSink::write(const sal_Unicode * pBegin,
						  const sal_Unicode * pEnd)
{
	writeSequence(pBegin, pEnd);
	m_nColumn += pEnd - pBegin;
}

inline void INetMIMEOutputSink::write(const ByteString & rOctets,
						  xub_StrLen nBegin, xub_StrLen nEnd)
{
	writeSequence(rOctets.GetBuffer() + nBegin, rOctets.GetBuffer() + nEnd);
	m_nColumn += nEnd - nBegin;
}

inline INetMIMEOutputSink & INetMIMEOutputSink::operator <<(sal_Char nOctet)
{
	writeSequence(&nOctet, &nOctet + 1);
	++m_nColumn;
	return *this;
}

inline INetMIMEOutputSink & INetMIMEOutputSink::operator <<(const sal_Char *
												    pOctets)
{
	m_nColumn += writeSequence(pOctets);
	return *this;
}

inline INetMIMEOutputSink & INetMIMEOutputSink::operator <<(const ByteString &
												    rOctets)
{
	writeSequence(rOctets.GetBuffer(), rOctets.GetBuffer() + rOctets.Len());
	m_nColumn += rOctets.Len();
	return *this;
}

inline INetMIMEOutputSink & INetMIMEOutputSink::endl(INetMIMEOutputSink &
										     rSink)
{
	rSink.writeLineEnd();
	return rSink;
}

inline void INetMIME::writeEscapeSequence(INetMIMEOutputSink & rSink,
							  sal_uInt32 nChar)
{
	DBG_ASSERT(nChar <= 0xFF, "INetMIME::writeEscapeSequence(): Bad char");
	rSink << '=' << sal_uInt8(getHexDigit(nChar >> 4))
	<< sal_uInt8(getHexDigit(nChar & 15));
}

class INetMIMEStringOutputSink: public INetMIMEOutputSink
{
	ByteString m_aBuffer;
	bool m_bOverflow;

	virtual void writeSequence(const sal_Char * pBegin,
				   const sal_Char * pEnd);

public:
	inline INetMIMEStringOutputSink(sal_uInt32 nColumn = 0,
						sal_uInt32 nLineLengthLimit
						    = INetMIME::SOFT_LINE_LENGTH_LIMIT):
		INetMIMEOutputSink(nColumn, nLineLengthLimit), m_bOverflow(false) {}

	virtual ErrCode getError() const;

	inline ByteString takeBuffer();
};

inline ByteString INetMIMEStringOutputSink::takeBuffer()
{
	ByteString aTheBuffer = m_aBuffer;
	m_aBuffer.Erase();
	m_bOverflow = false;
	return aTheBuffer;
}

class INetMIMEUnicodeOutputSink: public INetMIMEOutputSink
{
	UniString m_aBuffer;
	bool m_bOverflow;

	virtual void writeSequence(const sal_Char * pBegin,
				   const sal_Char * pEnd);

	virtual void writeSequence(const sal_uInt32 * pBegin,
				   const sal_uInt32 * pEnd);

	virtual void writeSequence(const sal_Unicode * pBegin,
				   const sal_Unicode * pEnd);

public:
	inline INetMIMEUnicodeOutputSink(sal_uInt32 nColumn = 0,
						 sal_uInt32 nLineLengthLimit
						     = INetMIME::SOFT_LINE_LENGTH_LIMIT):
		INetMIMEOutputSink(nColumn, nLineLengthLimit), m_bOverflow(false) {}

	virtual ErrCode getError() const;

	inline UniString takeBuffer();
};

inline UniString INetMIMEUnicodeOutputSink::takeBuffer()
{
	UniString aTheBuffer = m_aBuffer;
	m_aBuffer.Erase();
	m_bOverflow = false;
	return aTheBuffer;
}

class INetMIMEEncodedWordOutputSink
{
public:
	enum Context { CONTEXT_TEXT = 1,
	   CONTEXT_COMMENT = 2,
	   CONTEXT_PHRASE = 4 };

	enum Space { SPACE_NO, SPACE_ENCODED, SPACE_ALWAYS };

private:
	enum { BUFFER_SIZE = 256 };

	enum Coding { CODING_NONE, CODING_QUOTED, CODING_ENCODED,
	  CODING_ENCODED_TERMINATED };

	enum EncodedWordState { STATE_INITIAL, STATE_FIRST_EQUALS,
				STATE_FIRST_QUESTION, STATE_CHARSET,
				STATE_SECOND_QUESTION, STATE_ENCODING,
				STATE_THIRD_QUESTION, STATE_ENCODED_TEXT,
				STATE_FOURTH_QUESTION, STATE_SECOND_EQUALS,
				STATE_BAD };

	INetMIMEOutputSink & m_rSink;
	Context m_eContext;
	Space m_eInitialSpace;
	sal_uInt32 m_nExtraSpaces;
	INetMIMECharsetList_Impl * m_pEncodingList;
	sal_Unicode * m_pBuffer;
	sal_uInt32 m_nBufferSize;
	sal_Unicode * m_pBufferEnd;
	Coding m_ePrevCoding;
	rtl_TextEncoding m_ePrevMIMEEncoding;
	Coding m_eCoding;
	sal_uInt32 m_nQuotedEscaped;
	EncodedWordState m_eEncodedWordState;

	inline bool needsEncodedWordEscape(sal_uInt32 nChar) const;

	void finish(bool bWriteTrailer);

public:
	inline INetMIMEEncodedWordOutputSink(INetMIMEOutputSink & rTheSink,
							 Context eTheContext,
							 Space eTheInitialSpace,
							 rtl_TextEncoding ePreferredEncoding);

	~INetMIMEEncodedWordOutputSink();

	INetMIMEEncodedWordOutputSink & operator <<(sal_uInt32 nChar);

	inline void write(const sal_Char * pBegin, const sal_Char * pEnd);

	inline void write(const sal_Unicode * pBegin, const sal_Unicode * pEnd);

	inline bool flush();
};

inline INetMIMEEncodedWordOutputSink::INetMIMEEncodedWordOutputSink(
	    INetMIMEOutputSink & rTheSink, Context eTheContext,
	 Space eTheInitialSpace, rtl_TextEncoding ePreferredEncoding):
	m_rSink(rTheSink),
	m_eContext(eTheContext),
	m_eInitialSpace(eTheInitialSpace),
	m_nExtraSpaces(0),
	m_pEncodingList(INetMIME::createPreferredCharsetList(ePreferredEncoding)),
	m_ePrevCoding(CODING_NONE),
	m_eCoding(CODING_NONE),
	m_nQuotedEscaped(0),
	m_eEncodedWordState(STATE_INITIAL)
{
	m_nBufferSize = BUFFER_SIZE;
	m_pBuffer = static_cast< sal_Unicode * >(rtl_allocateMemory(
	                                       m_nBufferSize
									     * sizeof (sal_Unicode)));
	m_pBufferEnd = m_pBuffer;
}

inline void INetMIMEEncodedWordOutputSink::write(const sal_Char * pBegin,
									 const sal_Char * pEnd)
{
	DBG_ASSERT(pBegin && pBegin <= pEnd,
	  "INetMIMEEncodedWordOutputSink::write(): Bad sequence");

	while (pBegin != pEnd)
		operator <<(*pBegin++);
}

inline void INetMIMEEncodedWordOutputSink::write(const sal_Unicode * pBegin,
									 const sal_Unicode * pEnd)
{
	DBG_ASSERT(pBegin && pBegin <= pEnd,
	  "INetMIMEEncodedWordOutputSink::write(): Bad sequence");

	while (pBegin != pEnd)
		operator <<(*pBegin++);
}

inline bool INetMIMEEncodedWordOutputSink::flush()
{
	finish(true);
	return m_ePrevCoding != CODING_NONE;
}

struct INetContentTypeParameter
{

	const ByteString m_sAttribute;

	const ByteString m_sCharset;

	const ByteString m_sLanguage;

	const UniString m_sValue;

	const bool m_bConverted;

	inline INetContentTypeParameter(const ByteString & rTheAttribute,
						const ByteString & rTheCharset,
						const ByteString & rTheLanguage,
						const UniString & rTheValue,
						bool bTheConverted);
};

inline INetContentTypeParameter::INetContentTypeParameter(const ByteString &
											      rTheAttribute,
											  const ByteString &
											      rTheCharset,
											  const ByteString &
											      rTheLanguage,
											  const UniString &
											      rTheValue,
											  bool bTheConverted):
	m_sAttribute(rTheAttribute),
	m_sCharset(rTheCharset),
	m_sLanguage(rTheLanguage),
	m_sValue(rTheValue),
	m_bConverted(bTheConverted)
{}

class INetContentTypeParameterList: private List
{
public:
	~INetContentTypeParameterList() { Clear(); }

	using List::Count;

	void Clear();

	void Insert(INetContentTypeParameter * pParameter, ULONG nIndex)
	{ List::Insert(pParameter, nIndex); }

	inline const INetContentTypeParameter * GetObject(ULONG nIndex) const;

	const INetContentTypeParameter * find(const ByteString & rAttribute)
		const;
};

inline const INetContentTypeParameter *
INetContentTypeParameterList::GetObject(ULONG nIndex) const
{
	return static_cast< INetContentTypeParameter * >(List::GetObject(nIndex));
}

#endif 

