

#ifndef _TOOLS_INETSTRM_HXX
#define _TOOLS_INETSTRM_HXX

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

class INetMessage;
class INetMIMEMessage;
class INetHTTPMessage;
class SvMemoryStream;
class SvStream;

enum INetStreamStatus
{
	INETSTREAM_STATUS_LOADED     = -4,
	INETSTREAM_STATUS_WOULDBLOCK = -3,
	INETSTREAM_STATUS_OK         = -2,
	INETSTREAM_STATUS_ERROR      = -1
};

class INetIStream
{

	INetIStream (const INetIStream& rStrm);
	INetIStream& operator= (const INetIStream& rStrm);

protected:
	virtual int GetData (
		sal_Char *pData, ULONG nSize, void *pCtx = NULL) = 0;

public:
	INetIStream (ULONG nIBufferSize = 0);
	virtual ~INetIStream (void);

	int Read (sal_Char *pData, ULONG nSize, void *pCtx = NULL);

	static void Decode64 (SvStream& rIn, SvStream& rOut);
	static void Encode64 (SvStream& rIn, SvStream& rOut);
};

class INetOStream
{

	INetOStream (const INetOStream& rStrm);
	INetOStream& operator= (const INetOStream& rStrm);

protected:
	virtual int PutData (
		const sal_Char *pData, ULONG nSize, void *pCtx = NULL) = 0;

public:
	INetOStream (ULONG nOBufferSize = 0);
	virtual ~INetOStream (void);

	int Write (const sal_Char *pData, ULONG nSize, void *pCtx = NULL);
};

class INetIOStream : public INetIStream, public INetOStream
{

	INetIOStream (const INetIOStream& rStrm);
	INetIOStream& operator= (const INetIOStream& rStrm);

public:
	INetIOStream (ULONG nIBufferSize = 0, ULONG nOBufferSize = 0);
	virtual ~INetIOStream (void);
};

enum INetMessageStreamState
{
	INETMSG_EOL_BEGIN,
	INETMSG_EOL_DONE,
	INETMSG_EOL_SCR,
	INETMSG_EOL_FCR,
	INETMSG_EOL_FLF,
	INETMSG_EOL_FSP,
	INETMSG_EOL_FESC
};

class INetMessageIStream : public INetIStream
{
	INetMessage    *pSourceMsg;
	BOOL            bHeaderGenerated;

	ULONG           nBufSiz;
	sal_Char       *pBuffer;
	sal_Char       *pRead;
	sal_Char       *pWrite;

	SvStream       *pMsgStrm;
	SvMemoryStream *pMsgBuffer;
	sal_Char       *pMsgRead;
	sal_Char       *pMsgWrite;

	virtual int GetData (
		sal_Char *pData, ULONG nSize, void *pCtx = NULL);

	INetMessageIStream (const INetMessageIStream& rStrm);
	INetMessageIStream& operator= (const INetMessageIStream& rStrm);

protected:
	virtual int GetMsgLine (
		sal_Char *pData, ULONG nSize, void *pCtx = NULL);

public:
	INetMessageIStream (ULONG nBufferSize = 2048);
	virtual ~INetMessageIStream (void);

	INetMessage *GetSourceMessage (void) const { return pSourceMsg; }
	void SetSourceMessage (INetMessage *pMsg) { pSourceMsg = pMsg; }

	void GenerateHeader (BOOL bGen = TRUE) { bHeaderGenerated = !bGen; }
	BOOL IsHeaderGenerated (void) const { return bHeaderGenerated; }
};

class INetMessageOStream : public INetOStream
{
	INetMessage            *pTargetMsg;
	BOOL                    bHeaderParsed;

	INetMessageStreamState  eOState;

	SvMemoryStream         *pMsgBuffer;

	virtual int PutData (
		const sal_Char *pData, ULONG nSize, void *pCtx = NULL);

	INetMessageOStream (const INetMessageOStream& rStrm);
	INetMessageOStream& operator= (const INetMessageOStream& rStrm);

protected:
	virtual int PutMsgLine (
		const sal_Char *pData, ULONG nSize, void *pCtx = NULL);

public:
	INetMessageOStream (void);
	virtual ~INetMessageOStream (void);

	INetMessage *GetTargetMessage (void) const { return pTargetMsg; }
	void SetTargetMessage (INetMessage *pMsg) { pTargetMsg = pMsg; }

	void ParseHeader (BOOL bParse = TRUE) { bHeaderParsed = !bParse; }
	BOOL IsHeaderParsed (void) const { return bHeaderParsed; }
};

class INetMessageIOStream
	: public INetMessageIStream,
	  public INetMessageOStream
{

	INetMessageIOStream (const INetMessageIOStream& rStrm);
	INetMessageIOStream& operator= (const INetMessageIOStream& rStrm);

public:
	INetMessageIOStream (ULONG nBufferSize = 2048);
	virtual ~INetMessageIOStream (void);
};

enum INetMessageEncoding
{
	INETMSG_ENCODING_7BIT,
	INETMSG_ENCODING_8BIT,
	INETMSG_ENCODING_BINARY,
	INETMSG_ENCODING_QUOTED,
	INETMSG_ENCODING_BASE64
};

class INetMIMEMessageStream : public INetMessageIOStream
{
	int                    eState;

	ULONG                  nChildIndex;
	INetMIMEMessageStream *pChildStrm;

	INetMessageEncoding    eEncoding;
	INetMessageIStream    *pEncodeStrm;
	INetMessageOStream    *pDecodeStrm;

	SvMemoryStream        *pMsgBuffer;

	static INetMessageEncoding GetMsgEncoding (
		const String& rContentType);

	INetMIMEMessageStream (const INetMIMEMessageStream& rStrm);
	INetMIMEMessageStream& operator= (const INetMIMEMessageStream& rStrm);

protected:
	virtual int GetMsgLine (
		sal_Char *pData, ULONG nSize, void *pCtx = NULL);
	virtual int PutMsgLine (
		const sal_Char *pData, ULONG nSize, void *pCtx = NULL);

public:
	INetMIMEMessageStream (ULONG nBufferSize = 2048);
	virtual ~INetMIMEMessageStream (void);

	void SetSourceMessage (INetMIMEMessage *pMsg)
	{
		INetMessageIStream::SetSourceMessage ((INetMessage *)pMsg);
	}
	INetMIMEMessage *GetSourceMessage (void) const
	{
		return ((INetMIMEMessage *)INetMessageIStream::GetSourceMessage());
	}

	void SetTargetMessage (INetMIMEMessage *pMsg)
	{
		INetMessageOStream::SetTargetMessage ((INetMessage *)pMsg);
	}
	INetMIMEMessage *GetTargetMessage (void) const
	{
		return ((INetMIMEMessage *)INetMessageOStream::GetTargetMessage());
	}
};

class INetHTTPMessageStream : public INetMIMEMessageStream
{

	INetMessageStreamState  m_eState;
	INetMessageOStream     *m_pDecodeStrm;

	INetHTTPMessageStream (const INetHTTPMessageStream& rStrm);
	INetHTTPMessageStream& operator= (const INetHTTPMessageStream& rStrm);

protected:
	virtual int GetMsgLine (
		sal_Char *pData, ULONG nSize, void *pCtx = NULL);
	virtual int PutMsgLine (
		const sal_Char *pData, ULONG nSize, void *pCtx = NULL);

public:
	INetHTTPMessageStream (ULONG nBufferSize = 2048);
	virtual ~INetHTTPMessageStream (void);

	void SetSourceMessage (INetHTTPMessage *pMsg)
	{
		INetMessageIStream::SetSourceMessage ((INetMessage *)pMsg);
	}
	INetHTTPMessage *GetSourceMessage (void) const
	{
		return ((INetHTTPMessage *)INetMessageIStream::GetSourceMessage());
	}

	void SetTargetMessage (INetHTTPMessage *pMsg)
	{
		INetMessageOStream::SetTargetMessage ((INetMessage *)pMsg);
	}
	INetHTTPMessage *GetTargetMessage (void) const
	{
		return ((INetHTTPMessage *)INetMessageOStream::GetTargetMessage());
	}
};

#endif 

