
#ifndef IPARSER_HXX
#define IPARSER_HXX

#include <tools/string.hxx>
#include <tools/link.hxx>
#include <tools/stream.hxx>

class Dir;
class GenericInformation;
class GenericInformationList;

#define DEF_STAND_LST  "\\\\dev4\\data1\\b_server\\config\\stand.lst"

#define IP_NO_ERROR			0x0000
#define IP_UNEXPECTED_EOF	0x0001

#define REPLACE_VARIABLES	TRUE

class InformationParser
{
private:
  static const char cKeyLevelChars;
	BOOL bRecover;
	ByteString sOldLine;

	ByteString sCurrentComment;

	BOOL bReplaceVariables; 
	USHORT nLevel;
	ByteString sUPD;
	ByteString sVersion;

	Link aStatusLink;
	SvStream *pActStream;
	UniString sStreamName;

	USHORT nErrorCode;
	ULONG nErrorLine;
	ByteString sErrorText;
	ULONG nActLine;

	ByteString &ReadLine();
	GenericInformation *ReadKey( GenericInformationList *pExistingList );
	void Recover();

protected:
  BOOL Save( SvStream &rOutStream,
	  const GenericInformationList *pSaveList, USHORT nLevel );
	GenericInformationList *Execute( SvStream &rSourceStream,
					GenericInformationList *pExistingList );
	virtual void PrintStatus( ByteString &rStatus )
		{ if ( aStatusLink.IsSet()) aStatusLink.Call( &rStatus ); }

public:
	InformationParser( BOOL bReplace = FALSE );
	~InformationParser();

	GenericInformationList *Execute( SvMemoryStream &rSourceStream,
					GenericInformationList *pExistingList = NULL );
	GenericInformationList *Execute( SvFileStream &rSourceStream,
					GenericInformationList *pExistingList = NULL );
	GenericInformationList *Execute( UniString &rSourceFile,
					GenericInformationList *pExistingList = NULL );

	GenericInformationList *Execute( Dir &rDir,
					GenericInformationList *pExistingList = NULL );

  BOOL Save( SvFileStream &rSourceStream,
	  const GenericInformationList *pSaveList );
  BOOL Save( SvMemoryStream &rSourceStream,
	  const GenericInformationList *pSaveList );
  BOOL Save( const UniString &rSourceFile,
	  const GenericInformationList *pSaveList );

	USHORT GetErrorCode();
	ByteString &GetErrorText();

	void SetStatusHdl( const Link &rHdl ) { aStatusLink = rHdl; }
};

#endif

