

#ifndef _SV_MULTISEL_HXX
#define _SV_MULTISEL_HXX

#ifndef _GEN_HXX
#include <tools/gen.hxx>
#endif
#ifndef _LIST_HXX
#include <tools/list.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifdef _SV_MULTISEL_CXX
DECLARE_LIST( ImpSelList, Range* );
#else
#define ImpSelList List
#endif

#define SFX_ENDOFSELECTION		CONTAINER_ENTRY_NOTFOUND

class MultiSelection
{
private:
	ImpSelList		aSels;		
	Range			aTotRange;	
	ULONG			nCurSubSel; 
	long			nCurIndex;	
	ULONG			nSelCount;	
	BOOL			bInverseCur;
	BOOL			bCurValid;	
	BOOL			bSelectNew; 

#ifdef _SV_MULTISEL_CXX
	void			ImplClear();
	ULONG			ImplFindSubSelection( long nIndex ) const;
	BOOL			ImplMergeSubSelections( ULONG nPos1, ULONG nPos2 );
	long			ImplFwdUnselected();
	long			ImplBwdUnselected();
#endif

public:
		MultiSelection();
		MultiSelection( const MultiSelection& rOrig );
		MultiSelection( const Range& rRange );
		MultiSelection( const UniString& rString,
						sal_Unicode cRange = '-',
						sal_Unicode cSep = ';' );
		~MultiSelection();

	MultiSelection& operator= ( const MultiSelection& rOrig );
	BOOL			operator== ( MultiSelection& rOrig );
	BOOL			operator!= ( MultiSelection& rOrig )
			{ return !operator==( rOrig ); }
	BOOL			operator !() const
			{ return nSelCount == 0; }

	void			SelectAll( BOOL bSelect = TRUE );
	BOOL			Select( long nIndex, BOOL bSelect = TRUE );
	void			Select( const Range& rIndexRange, BOOL bSelect = TRUE );
	BOOL			IsSelected( long nIndex ) const;
	BOOL			IsAllSelected() const
			{ return nSelCount == ULONG(aTotRange.Len()); }
	long			GetSelectCount() const { return nSelCount; }

	void			SetTotalRange( const Range& rTotRange );
	void			Insert( long nIndex, long nCount = 1 );
	void			Remove( long nIndex );
	void			Append( long nCount = 1 );

	const Range&	GetTotalRange() const { return aTotRange; }
	BOOL			IsCurValid() const { return bCurValid; }
	long			GetCurSelected() const { return nCurIndex; }
	long			FirstSelected( BOOL bInverse = FALSE );
	long			LastSelected( BOOL bInverse = FALSE );
	long			NextSelected();
	long			PrevSelected();

	ULONG			GetRangeCount() const { return aSels.Count(); }
	const Range&	GetRange( ULONG nRange ) const { return *(const Range*)aSels.GetObject(nRange); }
};

#endif	
