

#ifndef _TOOLS_RESMGR_HXX
#define _TOOLS_RESMGR_HXX

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _LANG_HXX
#include <tools/lang.hxx>
#endif
#ifndef _REF_HXX
#include <tools/ref.hxx>
#endif

#ifndef _TOOLS_RESID_HXX
#include <tools/resid.hxx>
#endif

#define CREATEVERSIONRESMGR_NAME( Name )   #Name MAKE_NUMSTR( SUPD )
#define CREATEVERSIONRESMGR( Name ) 	   ResMgr::CreateResMgr( CREATEVERSIONRESMGR_NAME( Name ) )

class SvStream;
class List;
class InternalResMgr;
class ResMgr;
struct ImplSVResourceData
{
	List*				pInternalResMgrList;
	ResMgr* 			pAppResMgr;
	ImplSVResourceData()
		: pInternalResMgrList( NULL )
		, pAppResMgr( NULL )
		{}
};

struct RSHEADER_TYPE
{
private:
	USHORT					nId;		
	RESOURCE_TYPE			nRT;		
	USHORT					nGlobOff;	
	USHORT					nLocalOff;	

public:
	inline USHORT		   GetId(); 	   
	inline RESOURCE_TYPE   GetRT(); 	   
	inline USHORT		   GetGlobOff();   
	inline USHORT		   GetLocalOff();  
};

struct ImpContent;
class InternalResMgr : public SvRefBase
{
	friend class ResMgr;
	friend class SimpleResMgr;

	ImpContent *			pContent;
	UINT32					nOffCorrection;
	BYTE *					pStringBlock;
	SvStream *				pStm;
	BOOL					bEqual2Content;
	UINT32					nEntries;
	UniString				aFileName;
	UniString				aShortFileName;
	class Table *			pResUseDump;
					InternalResMgr();
				~InternalResMgr();
	BOOL					Create();

	static void 			GetResMgrPath( InternalResMgr* pThis,
							   const UniString& rFileName,
							   const UniString* pAppName = NULL,
							   const UniString* pResPath = NULL );
	static InternalResMgr*	Create( const UniString& rName,
						const UniString* pAppName = NULL,
						const UniString* pResPath = NULL );
	static InternalResMgr*	GetInternalResMgr( const String & rFileName, const String * pAppName = NULL, const String * pResPath = NULL );
	static void 			FreeInternalResMgr( InternalResMgr* pFreeInternalResMgr );

	BOOL					IsGlobalAvailable( RESOURCE_TYPE nRT, USHORT nId ) const;
	void *					LoadGlobalRes( RESOURCE_TYPE nRT, USHORT nId,
							   void **pResHandle );
public:
	void					FreeGlobalRes( void *, void * );

	SvStream *				GetBitmapStream( USHORT nResId );
};

typedef void (*ResHookProc)( UniString& rStr );

#define RC_NOTYPE				0x0

#define RC_GLOBAL				0x1
#define RC_AUTORELEASE			0x2

class Resource;
class ResMgr;
struct ImpRCStack
{

	RSHEADER_TYPE * pResource;		
	void		  * pClassRes;		
	short			Flags;			
	void *			aResHandle; 	
	const Resource* pResObj;		
	USHORT			nId;			
	ResMgr *		pResMgr;		

	void			Clear();
	void			Init( ResMgr * pMgr, const Resource * pObj, USHORT nId );
};

#define MAX_RC_STACK	32

class ResMgr
{

public:
	InternalResMgr* 	pImpRes;
	ImpRCStack			aStack[MAX_RC_STACK];	
	short				nTopRes;				

	static void 		DestroyAllResMgr();  
	void				Init( const UniString& rFileName );

public:
				ResMgr( InternalResMgr * pImp );
			ResMgr( const UniString* pFileName = NULL );	
			ResMgr( const UniString& rFileName,
					const UniString* pAppName = NULL,
					const UniString* pResPath = NULL );
			~ResMgr();

	static const sal_Char*	GetLang( USHORT nPrio = 0 ); 
#ifdef WTC
	static const sal_Char*	GetLang( LanguageType& eLanguage, USHORT nPrio );
#else
	static const sal_Char*	GetLang( LanguageType& eLanguage, USHORT nPrio = 0 );
#endif
	static ResMgr*		CreateResMgr( const sal_Char* pPrefixName,
						  LanguageType eLanguage,
						  const UniString* pAppName,
						  const UniString* pResPath );
	static ResMgr*		CreateResMgr( const sal_Char* pPrefixName,
						  LanguageType eLanguage = LANGUAGE_DONTKNOW ); 
	static ResMgr*		SearchCreateResMgr( const sal_Char* pPrefixName,
								LanguageType& rLanguageReturn );

	void				TestStack( const Resource * );
	const ImpRCStack *	StackTop( USHORT nOff = 0 ) const
			{
					return (((short)nOff >= nTopRes) ? NULL : &aStack[nTopRes-nOff]);
			}

	BOOL				IsAvailable( const ResId& rId,
						 const Resource* = NULL) const;

	BOOL				GetResource( const ResId& rId, const Resource * = NULL );
	static void *		GetResourceSkipHeader( const ResId& rResId, ResMgr ** ppResMgr );

	void				PopContext( const Resource* = NULL );

	void*				Increment( USHORT nSize );

	static USHORT		GetObjSize( RSHEADER_TYPE* pHT )
				{ return( pHT->GetGlobOff() ); }

	static USHORT		GetString( UniString& rStr, const BYTE* pStr );

	static USHORT		GetStringSize( USHORT nLen )
				{ nLen++; return (nLen + nLen%2); }
	static USHORT		GetStringSize( const BYTE* pStr );

	static INT32		GetLong( void * pLong );

	static INT16		GetShort( void * pShort );

	void *				GetClass()
				{ return( aStack[nTopRes].pClassRes ); }

	RSHEADER_TYPE * 	CreateBlock( const ResId & rId );

	USHORT				GetRemainSize();

	const UniString&	GetFileName() const { return pImpRes->aFileName; }

	INT16				ReadShort();
	INT32				ReadLong();
	UniString			ReadString();

	static void 		SetReadStringHook( ResHookProc pProc );
	static ResHookProc	GetReadStringHook();
};

inline USHORT RSHEADER_TYPE::GetId()
{
	return (USHORT)ResMgr::GetShort( &nId );
}
inline RESOURCE_TYPE RSHEADER_TYPE::GetRT()
{
	return (RESOURCE_TYPE)ResMgr::GetShort( &nRT );
}
inline USHORT RSHEADER_TYPE::GetGlobOff()
{
	return (USHORT)ResMgr::GetShort( &nGlobOff );
}
inline USHORT RSHEADER_TYPE::GetLocalOff()
{
	return (USHORT)ResMgr::GetShort( &nLocalOff );
}

#endif 
