

#ifndef _SIMPLECM_HXX
#define _SIMPLECM_HXX

#ifndef _LINK_HXX 
#include <tools/link.hxx>
#endif
#ifndef _STRING_HXX 
#include <tools/string.hxx>
#endif
#ifndef _STREAM_HXX 
#include <tools/stream.hxx>
#endif
#ifndef _VOS_SOCKET_HXX_ 
#include <vos/socket.hxx>
#endif
#ifndef _TOOLS_DEBUG_HXX 
#include <tools/debug.hxx>
#endif
#ifndef _DATETIME_HXX 
#include <tools/datetime.hxx>
#endif

#include <tools/commdefines.hxx>

#define CM_UNLIMITED_CONNECTIONS	0xffff

typedef USHORT CM_NameType;
#define CM_DOTTED	( (CM_NameType) 01 )
#define CM_FQDN		( (CM_NameType) 02 )

typedef USHORT CM_InfoType;

#define CM_NO_TEXT		( (CM_InfoType) 01 )
#define CM_SHORT_TEXT	( (CM_InfoType) 02 )
#define CM_VERBOSE_TEXT	( (CM_InfoType) 03 )

#define CM_OPEN			( (CM_InfoType) 0x0004 )
#define CM_CLOSE		( (CM_InfoType) 0x0008 )
#define CM_RECEIVE		( (CM_InfoType) 0x0010 )
#define CM_SEND			( (CM_InfoType) 0x0020 )
#define CM_ERROR		( (CM_InfoType) 0x0040 )
#define CM_MISC			( (CM_InfoType) 0x0080 )

#define CM_USER_1		( (CM_InfoType) 0x0100 )
#define CM_USER_2		( (CM_InfoType) 0x0200 )
#define CM_USER_3		( (CM_InfoType) 0x0400 )
#define CM_USER_4		( (CM_InfoType) 0x0800 )

#define CM_ALL			( CM_OPEN | CM_CLOSE | CM_RECEIVE | CM_SEND | CM_ERROR | CM_MISC )
#define CM_NONE			( 0 )

#define CByteString( constAsciiStr ) ByteString( RTL_CONSTASCII_STRINGPARAM ( constAsciiStr ) )

#define INFO_MSG( Short, Long, Type, CLink ) \
{ \
	if ( (Type & GetInfoType()) > 0 ) \
	{ \
		switch ( GetInfoType() & 03 ) \
		{ \
		case CM_NO_TEXT: \
	CallInfoMsg( InfoString( ByteString(), Type, CLink ) ); \
	break; \
		case CM_SHORT_TEXT: \
	CallInfoMsg( InfoString( Short, Type, CLink ) ); \
	break; \
		case CM_VERBOSE_TEXT: \
	CallInfoMsg( InfoString( Long, Type, CLink ) ); \
	break; \
		default: \
	break; \
		} \
	} \
}\

class CommunicationLink;

SV_DECL_REF( CommunicationLink );

class InfoString : public ByteString
{
public:
	InfoString( ByteString &nMsg, CM_InfoType nIT, CommunicationLink *pCL = NULL ): ByteString( nMsg ), nInfoType( nIT ), pCommLink( pCL ) {;}
	CM_InfoType GetInfoType(){ return nInfoType; }
	CommunicationLinkRef GetCommunicationLink(){ return pCommLink; }
private:
	CM_InfoType nInfoType;
	CommunicationLinkRef pCommLink;
};

class PacketHandler;
class CommunicationManager;
class SingleCommunicationManager;
class MultiCommunicationManager;
class CommunicationManagerServerAcceptThread;
class CommunicationLink : public SvRefBase
{
	friend CommunicationManager;
	friend SingleCommunicationManager;
	friend MultiCommunicationManager;
	friend CommunicationManagerServerAcceptThread;

protected:	
	virtual ~CommunicationLink();
	void InvalidateManager() { pMyManager = NULL; }

	PacketHandler* pPacketHandler;

public:
	CommunicationLink( CommunicationManager *pMan );

	virtual BOOL StopCommunication()=0;
	virtual BOOL IsCommunicationError()=0;
	CommunicationManager* GetCommunicationManager(){ return pMyManager; }

	virtual ByteString GetCommunicationPartner( CM_NameType eType )=0;

	virtual ByteString GetMyName( CM_NameType eType )=0;

	virtual SvStream* GetBestCommunicationStream()=0;

	BOOL TransferDataStream( SvStream *pDataStream, CMProtocol nProtocol = CM_PROTOCOL_OLDSTYLE );

	UINT16 GetProtocol(){ return nServiceProtocol; }

	SvStream* GetServiceData(){ SvStream *pTemp = pServiceData; pServiceData = NULL; return pTemp; }

	void FinishCallback(){ bIsInsideCallback = FALSE; }

	virtual BOOL ReceiveDataStream(){ return FALSE; }

	DateTime GetStart() { return aStart; }
	ULONG GetTotalBytes() { return nTotalBytes; }
	DateTime GetLastAccess() { return aLastAccess; }
	const ByteString& GetApplication() { return aApplication; }
	virtual void SetApplication( const ByteString& aApp );

protected:
	void CallInfoMsg( InfoString aMsg );
	CM_InfoType GetInfoType();
	CommunicationManager *pMyManager;

	virtual DECL_LINK( ConnectionClosed, void* = NULL );
	virtual DECL_LINK( DataReceived, void* = NULL );

	virtual BOOL DoTransferDataStream( SvStream *pDataStream, CMProtocol nProtocol = CM_PROTOCOL_OLDSTYLE );

	SvStream *pServiceData;
	UINT16 nServiceProtocol;
	UINT16 nServiceHeaderType;

	void StartCallback(){ bIsInsideCallback = TRUE; }
	BOOL bIsInsideCallback;

	virtual BOOL SendHandshake( HandshakeType aHandshakeType, SvStream* pData = NULL)=0;

	virtual BOOL ShutdownCommunication() = 0;	

	DateTime aStart;
	ULONG nTotalBytes;
	DateTime aLastAccess;
	ByteString aApplication;
};

SV_IMPL_REF( CommunicationLink );

class CommonSocketFunctions;
class CommunicationManager
{
	friend CommunicationLink;
	friend CommonSocketFunctions;
public:
	CommunicationManager( BOOL bUseMultiChannel = FALSE );
	virtual ~CommunicationManager();

	virtual BOOL StartCommunication()=0;
	virtual BOOL StartCommunication( String aApp, String aParams ) { return FALSE; }
	virtual BOOL StartCommunication( ByteString aHost, ULONG nPort ){ return FALSE;}
	virtual BOOL StopCommunication()=0;		
	virtual BOOL IsCommunicationRunning() { return bIsCommunicationRunning; }

	virtual ByteString GetMyName( CM_NameType eType );

	virtual BOOL IsLinkValid( CommunicationLink* pCL )=0;	

	virtual USHORT GetCommunicationLinkCount()=0;
	virtual CommunicationLinkRef GetCommunicationLink( USHORT nNr )=0;

	CommunicationLinkRef GetLastNewLink() { return xLastNewLink; }

	void SetConnectionOpenedHdl( Link lConnectionOpened ){ mlConnectionOpened = lConnectionOpened; }
	void SetConnectionClosedHdl( Link lConnectionClosed ){ mlConnectionClosed = lConnectionClosed; }
	void SetDataReceivedHdl( Link lDataReceived ){ mlDataReceived = lDataReceived; }
	void SetInfoMsgHdl( Link lInfoMsg ){ mlInfoMsg = lInfoMsg; }

	void SetInfoType( CM_InfoType nIT ){ nInfoType = nIT; }
	CM_InfoType GetInfoType(){ return nInfoType; }

	BOOL IsMultiChannel(){ return bIsMultiChannel; }
	void SetApplication( const ByteString& aApp, BOOL bRunningLinks = FALSE );

protected:

	virtual void CallConnectionOpened( CommunicationLink* pCL );
	virtual void CallConnectionClosed( CommunicationLink* pCL );
	void CallDataReceived( CommunicationLink* pCL );
	void CallInfoMsg( InfoString aMsg );

	CM_InfoType nInfoType;

	virtual void ConnectionOpened( CommunicationLink* pCL ){ mlConnectionOpened.Call( pCL ); }
	virtual void ConnectionClosed( CommunicationLink* pCL ){ mlConnectionClosed.Call( pCL ); }
	virtual void DataReceived( CommunicationLink* pCL ){ mlDataReceived.Call( pCL ); }
	virtual void InfoMsg( InfoString aMsg ){ mlInfoMsg.Call( &aMsg ); }

	BOOL bIsCommunicationRunning;

	virtual void DestroyingLink( CommunicationLink *pCL )=0;	
	ByteString aApplication;

private:
	Link mlConnectionOpened;
	Link mlConnectionClosed;
	Link mlDataReceived;
	Link mlInfoMsg;
	CommunicationLinkRef xLastNewLink;

	BOOL bIsMultiChannel;
};

class SingleCommunicationManager : public CommunicationManager
{
public:
	SingleCommunicationManager( BOOL bUseMultiChannel = FALSE );
	virtual ~SingleCommunicationManager();
	virtual BOOL StopCommunication();		
	virtual BOOL IsLinkValid( CommunicationLink* pCL );
	virtual USHORT GetCommunicationLinkCount();
	virtual CommunicationLinkRef GetCommunicationLink( USHORT nNr );

protected:
	virtual void CallConnectionOpened( CommunicationLink* pCL );
	virtual void CallConnectionClosed( CommunicationLink* pCL );
	CommunicationLinkRef xActiveLink;
	CommunicationLink *pInactiveLink;
	virtual void DestroyingLink( CommunicationLink *pCL );	
};

class ICommunicationManagerClient
{
	friend CommonSocketFunctions;
protected:
	virtual BOOL RetryConnect() { return FALSE; }	
};

class TCPIO;
class SimpleCommunicationLinkViaSocket : public CommunicationLink
{
public:
	virtual BOOL IsCommunicationError();
	virtual BOOL StopCommunication();

	virtual ByteString GetCommunicationPartner( CM_NameType eType );
	virtual ByteString GetMyName( CM_NameType eType );
	virtual SvStream* GetBestCommunicationStream();
	virtual void SetApplication( const ByteString& aApp );

private:
	ByteString aCommunicationPartner;
	ByteString aMyName;

	TCPIO* pTCPIO;

protected:
	SimpleCommunicationLinkViaSocket( CommunicationManager *pMan, NAMESPACE_VOS(OStreamSocket) *pSocket );
	virtual ~SimpleCommunicationLinkViaSocket();

	NAMESPACE_VOS(OStreamSocket) *pStreamSocket;
	SvStream *pReceiveStream;
	BOOL DoReceiveDataStream();				
	virtual BOOL SendHandshake( HandshakeType aHandshakeType, SvStream* pData = NULL);
	void SetFinalRecieveTimeout();
	BOOL bIsRequestShutdownPending;
	virtual void WaitForShutdown()=0;
	void SetNewPacketAsCurrent();
};

class SimpleCommunicationLinkViaSocketWithReceiveCallbacks : public SimpleCommunicationLinkViaSocket
{
public:
	SimpleCommunicationLinkViaSocketWithReceiveCallbacks( CommunicationManager *pMan, NAMESPACE_VOS(OStreamSocket) *pSocket );
	~SimpleCommunicationLinkViaSocketWithReceiveCallbacks();
	virtual BOOL ReceiveDataStream();
protected:
	virtual BOOL ShutdownCommunication();	
	virtual void WaitForShutdown();
};

class CommonSocketFunctions
{
public:
	BOOL DoStartCommunication( CommunicationManager *pCM, ICommunicationManagerClient *pCMC, ByteString aHost, ULONG nPort );
protected:
	virtual CommunicationLink *CreateCommunicationLink( CommunicationManager *pCM, NAMESPACE_VOS(OConnectorSocket) *pCS )=0;
};

class SingleCommunicationManagerClientViaSocket : public SingleCommunicationManager, public ICommunicationManagerClient, CommonSocketFunctions
{
public:
	SingleCommunicationManagerClientViaSocket( ByteString aHost, ULONG nPort, BOOL bUseMultiChannel = FALSE );
	SingleCommunicationManagerClientViaSocket( BOOL bUseMultiChannel = FALSE );
	virtual BOOL StartCommunication(){ return DoStartCommunication( this, (ICommunicationManagerClient*) this, aHostToTalk, nPortToTalk );}
	virtual BOOL StartCommunication( ByteString aHost, ULONG nPort ){ return DoStartCommunication( this, (ICommunicationManagerClient*) this, aHost, nPort );}
private:
	ByteString aHostToTalk;
	ULONG nPortToTalk;
protected:
	virtual CommunicationLink *CreateCommunicationLink( CommunicationManager *pCM, NAMESPACE_VOS(OConnectorSocket) *pCS ){ return new SimpleCommunicationLinkViaSocketWithReceiveCallbacks( pCM, pCS ); }
};

#endif
