

#ifndef _STREAM_HXX
#define _STREAM_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _EINF_HXX
#include <tools/errinf.hxx>
#endif
#ifndef _REF_HXX
#include <tools/ref.hxx>
#endif
#ifndef _RTTI_HXX
#include <tools/rtti.hxx>
#endif

class StreamData;

inline rtl_TextEncoding GetStoreCharSet( rtl_TextEncoding eEncoding, USHORT nVersion = SOFFICE_FILEFORMAT_50 )
{
	if ( eEncoding == RTL_TEXTENCODING_ISO_8859_1 )
		return RTL_TEXTENCODING_MS_1252;
	else
		return eEncoding;
}

typedef USHORT StreamMode;

#define STREAM_READ 					0x0001	
#define STREAM_WRITE					0x0002	

#define STREAM_NOCREATE 				0x0004	
#define STREAM_TRUNC					0x0008	
#define STREAM_COPY_ON_SYMLINK			0x0010	

#define STREAM_READWRITEBITS			(STREAM_READ | STREAM_WRITE | \
							 STREAM_NOCREATE | STREAM_TRUNC)

#define STREAM_SHARE_DENYNONE			0x0100
#define STREAM_SHARE_DENYREAD			0x0200	
#define STREAM_SHARE_DENYWRITE			0x0400	
#define STREAM_SHARE_DENYALL			0x0800	

#define STREAM_SHAREBITS				(STREAM_SHARE_DENYNONE | STREAM_SHARE_DENYREAD |\
							 STREAM_SHARE_DENYWRITE | STREAM_SHARE_DENYALL)

#define STREAM_READWRITE				(STREAM_READ | STREAM_WRITE)
#define STREAM_SHARE_DENYREADWRITE		(STREAM_SHARE_DENYREAD | STREAM_SHARE_DENYWRITE)

#define STREAM_STD_READ 				(STREAM_READ | STREAM_SHARE_DENYNONE | STREAM_NOCREATE)
#define STREAM_STD_WRITE				(STREAM_WRITE | STREAM_SHARE_DENYALL)
#define STREAM_STD_READWRITE			(STREAM_READWRITE | STREAM_SHARE_DENYALL)

#define STREAM_SEEK_TO_BEGIN			0L
#define STREAM_SEEK_TO_END				0xFFFFFFFFL

#define NUMBERFORMAT_INT_BIGENDIAN		(UINT16)0x0000
#define NUMBERFORMAT_INT_LITTLEENDIAN	(UINT16)0xFFFF

#define COMPRESSMODE_FULL				(UINT16)0xFFFF
#define COMPRESSMODE_NONE				(UINT16)0x0000
#define COMPRESSMODE_ZBITMAP			(UINT16)0x0001
#define COMPRESSMODE_NATIVE 			(UINT16)0x0010

#define JUSTIFY_RIGHT					0x00
#define JUSTIFY_LEFT					0x01

#define STREAM_IO_DONTKNOW				0
#define STREAM_IO_READ					1
#define STREAM_IO_WRITE 				2

#define ID_STREAM						1
#define ID_FILESTREAM					2
#define ID_MEMORYSTREAM 				3
#define ID_SHAREDMEMORYSTREAM			4
#define ID_STORAGESTREAM				5
#define ID_PERSISTSTREAM				6

class SvStream;
typedef SvStream& (*SvStrPtr)( SvStream& );

inline SvStream& operator<<( SvStream& rStr, SvStrPtr f );

enum LockType {};

struct SvLockBytesStat
{
	ULONG nSize;

	SvLockBytesStat(): nSize(0) {}
};

enum SvLockBytesStatFlag { SVSTATFLAG_DEFAULT };

class SvLockBytes: public virtual SvRefBase
{
	SvStream * m_pStream;
	BOOL m_bOwner;
	BOOL m_bSync;

protected:
	void close();

public:
	TYPEINFO();

	SvLockBytes(): m_pStream(0), m_bOwner(FALSE), m_bSync(FALSE) {}

	SvLockBytes(SvStream * pTheStream, BOOL bTheOwner = FALSE):
		m_pStream(pTheStream), m_bOwner(bTheOwner), m_bSync(FALSE) {}

	virtual ~SvLockBytes() { close(); }

	virtual const SvStream * GetStream() const { return m_pStream; }

	virtual void SetSynchronMode(BOOL bTheSync = TRUE) { m_bSync = bTheSync; }

	virtual BOOL IsSynchronMode() const { return m_bSync; }

	virtual ErrCode ReadAt(ULONG nPos, void * pBuffer, ULONG nCount,
			   ULONG * pRead) const;

	virtual ErrCode WriteAt(ULONG nPos, const void * pBuffer, ULONG nCount,
				ULONG * pWritten);

	virtual ErrCode Flush() const;

	virtual ErrCode SetSize(ULONG nSize);

	virtual ErrCode LockRegion(ULONG, ULONG, LockType);

	virtual ErrCode UnlockRegion(ULONG, ULONG, LockType);

	virtual ErrCode Stat(SvLockBytesStat * pStat, SvLockBytesStatFlag) const;
};

SV_DECL_IMPL_REF(SvLockBytes);

class SvOpenLockBytes: public SvLockBytes
{
public:
	TYPEINFO();

	SvOpenLockBytes(): SvLockBytes(0, FALSE) {}

	SvOpenLockBytes(SvStream * pStream, BOOL bOwner):
		SvLockBytes(pStream, bOwner) {}

	virtual ErrCode FillAppend(const void * pBuffer, ULONG nCount,
				   ULONG * pWritten) = 0;

	virtual ULONG Tell() const = 0;

	virtual ULONG Seek(ULONG nPos) = 0;

	virtual void Terminate() = 0;
};

SV_DECL_IMPL_REF(SvOpenLockBytes);

class SvAsyncLockBytes: public SvOpenLockBytes
{
	ULONG m_nSize;
	BOOL m_bTerminated;

public:
	TYPEINFO();

	SvAsyncLockBytes(SvStream * pStream, BOOL bOwner):
		SvOpenLockBytes(pStream, bOwner), m_nSize(0), m_bTerminated(FALSE) {}

	virtual ErrCode ReadAt(ULONG nPos, void * pBuffer, ULONG nCount,
			   ULONG * pRead) const;

	virtual ErrCode WriteAt(ULONG nPos, const void * pBuffer, ULONG nCount,
				ULONG * pWritten);

	virtual ErrCode FillAppend(const void * pBuffer, ULONG nCount,
				   ULONG * pWritten);

	virtual ULONG Tell() const { return m_nSize; }

	virtual ULONG Seek(ULONG nPos);

	virtual void Terminate() { m_bTerminated = TRUE; }
};

SV_DECL_IMPL_REF(SvAsyncLockBytes);

class SvStream
{
private:

	void*			pImp;			
	SvLockBytesRef	xLockBytes; 	
	ULONG			nActPos;		

	BYTE*			pRWBuf; 		
	BYTE*			pBufPos;		
	USHORT			nBufSize;		
	USHORT			nBufActualLen;	

	USHORT			nBufActualPos;	
	USHORT			nBufFree;		
	int 			eIOMode:2;		

	int 			bIsDirty:1; 	
	int 			bIsConsistent:1;

	int 			bSwap:1;
	int 			bIsEof:1;
	ULONG			nError;
	UINT16			nNumberFormatInt;
	UINT16			nCompressMode;
	LineEnd 		eLineDelimiter;
	CharSet 		eStreamCharSet;

	ByteString		aKey;			
	unsigned char	nCryptMask;

	char			cFiller;
	BYTE			nRadix;
	BYTE			nPrecision;
	BYTE			nWidth;
	BYTE			nPrintfParams;
	BYTE			nJustification;
	ByteString		aFormatString;

	long			nVersion;		

	void			CreateFormatString();
	void			ImpInit();
		SvStream ( const SvStream& rStream ); 
	SvStream&		operator=( const SvStream& rStream ); 

protected:
	ULONG			nBufFilePos;	
	USHORT			eStreamMode;
	BOOL			bIsWritable;

	virtual ULONG	GetData( void* pData, ULONG nSize );
	virtual ULONG	PutData( const void* pData, ULONG nSize );
	virtual ULONG	SeekPos( ULONG nPos );
	virtual void	FlushData();
	virtual void	SetSize( ULONG nSize );

	void			ClearError();
	void			ClearBuffer();

	ULONG			CryptAndWriteBuffer( const void* pStart, ULONG nLen );
	BOOL			EncryptBuffer( void* pStart, ULONG nLen );

	void			SyncSvStream( ULONG nNewStreamPos ); 
	void			SyncSysStream(); 

public:
		SvStream();
		SvStream( SvLockBytes *pLockBytes);
	virtual 		~SvStream();

	ErrCode 		SetLockBytes( SvLockBytesRef& rBytes );
	SvLockBytes*	GetLockBytes() const { return xLockBytes; }

	ULONG			GetError() const { return ERRCODE_TOERROR(nError); }
	ULONG			GetErrorCode() const { return nError; }

	void			SetError( ULONG nErrorCode );
	virtual void	ResetError();

	void			SetNumberFormatInt( UINT16 nNewFormat );
	UINT16			GetNumberFormatInt() const { return nNumberFormatInt; }

	inline void		SetEndianSwap( BOOL bVal );

	BOOL			IsEndianSwap() const { return 0 != bSwap; }

	void			SetCompressMode( UINT16 nNewMode )
			{ nCompressMode = nNewMode; }
	UINT16			GetCompressMode() const { return nCompressMode; }

	void			SetKey( const ByteString& rKey );
	const ByteString&	GetKey() const { return aKey; }

	void			SetStreamCharSet( CharSet eCharSet )
			{ eStreamCharSet = eCharSet; }
	CharSet 		GetStreamCharSet() const { return eStreamCharSet; }

	void			SetLineDelimiter( LineEnd eLineEnd )
			{ eLineDelimiter = eLineEnd; }
	LineEnd 		GetLineDelimiter() const { return eLineDelimiter; }

	SvStream&		operator>>( USHORT& rUShort );
	SvStream&		operator>>( ULONG& rULong );
	SvStream&		operator>>( long& rLong );
	SvStream&		operator>>( short& rShort );
	SvStream&		operator>>( int& rInt );
	SvStream&		operator>>( unsigned int& rUInt );
	SvStream&		operator>>( signed char& rChar );
	SvStream&		operator>>( char& rChar );
	SvStream&		operator>>( unsigned char& rChar );
	SvStream&		operator>>( float& rFloat );
	SvStream&		operator>>( double& rDouble );
#ifdef ENABLE_BYTESTRING_STREAM_OPERATORS
	SvStream&		operator>>( ByteString& rString ) { return ReadByteString(rString); }
#endif
#ifdef ENABLE_STRING_STREAM_OPERATORS
	SvStream&		operator>>( UniString& rString ) { return ReadByteString(rString); }
#endif
	SvStream&		operator>>( SvStream& rStream );

	SvStream&		operator<<( USHORT nUShort );
	SvStream&		operator<<( ULONG nULong );
	SvStream&		operator<<( long nLong );
	SvStream&		operator<<( short nShort );
	SvStream&		operator<<( int nInt );
	SvStream&		operator<<( unsigned int nUInt );
	SvStream&		operator<<( signed char nChar );
	SvStream&		operator<<( char nChar );
	SvStream&		operator<<( unsigned char nChar );
	SvStream&		operator<<( float nFloat );
	SvStream&		operator<<( const double& rDouble );
	SvStream&		operator<<( const char* pBuf );
	SvStream&		operator<<( const unsigned char* pBuf );
#ifdef ENABLE_BYTESTRING_STREAM_OPERATORS
	SvStream&		operator<<( const ByteString& rString ) { return WriteByteString( rString ); }
#endif
#ifdef ENABLE_STRING_STREAM_OPERATORS
	SvStream&		operator<<( const UniString& rString ) { return WriteByteString(rString); }
#endif
	SvStream&		operator<<( SvStream& rStream );

	SvStream&		ReadByteString( UniString& rStr, rtl_TextEncoding eSrcCharSet );
	SvStream&		ReadByteString( UniString& rStr ) { return ReadByteString( rStr, GetStreamCharSet() ); }
	SvStream&       ReadByteString( ByteString& rStr );
	SvStream&		WriteByteString( const UniString& rStr, rtl_TextEncoding eDestCharSet );
	SvStream&		WriteByteString( const UniString& rStr ) { return WriteByteString( rStr, GetStreamCharSet() ); }
	SvStream&       WriteByteString( const ByteString& rStr );

	void			SetRadix( BYTE nRad )
			{ nRadix = nRad; CreateFormatString(); }
	BYTE			GetRadix() const { return nRadix; }
	void			SetPrecision( BYTE nPrec )
			{ nPrecision = nPrec; CreateFormatString(); }
	BYTE			GetPrecision() const { return nPrecision; }
	void			SetWidth( BYTE nWid)
			{ nWidth = nWid; CreateFormatString(); }
	BYTE			GetWidth() const { return nWidth; }
	void			SetFiller( char cFil )
			{ cFiller = cFil; CreateFormatString(); }
	char			GetFiller() const { return cFiller; }
	void			SetJustification( BYTE nJus )
			 { nJustification = nJus; CreateFormatString(); }
	BYTE			GetJustification() const { return nJustification; }

	SvStream&		ReadNumber( short& rShort );
	SvStream&		ReadNumber( USHORT& rUShort );
	SvStream&		ReadNumber( long& rLong );
	SvStream&		ReadNumber( ULONG& rULong );
	SvStream&		ReadNumber( int& rInt );
	SvStream&		ReadNumber( unsigned int& rUInt );
	SvStream&		ReadNumber( float& rFloat );
	SvStream&		ReadNumber( double& rDouble );

	SvStream&		WriteNumber( short nShort );
	SvStream&		WriteNumber( USHORT nUShort );
	SvStream&		WriteNumber( long nLong );
	SvStream&		WriteNumber( ULONG nULong );
	SvStream&		WriteNumber( int nInt );
	SvStream&		WriteNumber( unsigned int nUInt );
	SvStream&		WriteNumber( float nFloat );
	SvStream&		WriteNumber( const double& rDouble );

	ULONG			Read( void* pData, ULONG nSize );
	ULONG			Write( const void* pData, ULONG nSize );
	ULONG			Seek( ULONG nPos );
	ULONG			SeekRel( long nPos );
	ULONG			Tell() const { return nBufFilePos+nBufActualPos;  }
	void			Flush();
	BOOL			IsEof() const { return bIsEof; }

	BOOL			SetStreamSize( ULONG nSize );

	BOOL			ReadCString( ByteString& rStr );
	BOOL 			ReadCString( String& rStr, rtl_TextEncoding eToEncode );
	BOOL 			ReadCString( String& rStr ) { return ReadCString( rStr, GetStreamCharSet()); }

	BOOL			ReadLine( ByteString& rStr );
	BOOL			WriteLine( const ByteString& rStr );
	BOOL			WriteLines( const ByteString& rStr );

	BOOL			ReadByteStringLine( String& rStr, rtl_TextEncoding eSrcCharSet );
	BOOL			ReadByteStringLine( String& rStr ) { return ReadByteStringLine( rStr, GetStreamCharSet()); }
	BOOL			WriteByteStringLine( const String& rStr, rtl_TextEncoding eDestCharSet );
	BOOL			WriteByteStringLine( const String& rStr ) { return WriteByteStringLine( rStr, GetStreamCharSet()); }
	BOOL			WriteByteStringLines( const String& rStr, rtl_TextEncoding eDestCharSet );
	BOOL			WriteByteStringLines( const String& rStr ) { return WriteByteStringLine( rStr, GetStreamCharSet()); }

	BOOL			StartWritingUnicodeText();

	BOOL			StartReadingUnicodeText();

	BOOL			ReadUniStringLine( String& rStr );

	BOOL			ReadUniOrByteStringLine( String& rStr, rtl_TextEncoding eSrcCharSet );
	BOOL			ReadUniOrByteStringLine( String& rStr )
			{ return ReadUniOrByteStringLine( rStr, GetStreamCharSet() ); }

	BOOL			WriteUnicodeText( const String& rStr );

	BOOL			WriteUnicodeOrByteText( const String& rStr, rtl_TextEncoding eDestCharSet );
	BOOL			WriteUnicodeOrByteText( const String& rStr )
			{ return WriteUnicodeOrByteText( rStr, GetStreamCharSet() ); }

	BOOL			WriteUniStringLine( const String& rStr );

	BOOL			WriteUniStringLines( const String& rStr );

	BOOL			WriteUniOrByteStringLine( const String& rStr, rtl_TextEncoding eDestCharSet );
	BOOL			WriteUniOrByteStringLine( const String& rStr )
			{ return WriteUniOrByteStringLine( rStr, GetStreamCharSet() ); }

	BOOL			WriteUniOrByteStringLines( const String& rStr, rtl_TextEncoding eDestCharSet );
	BOOL			WriteUniOrByteStringLines( const String& rStr )
			{ return WriteUniOrByteStringLines( rStr, GetStreamCharSet() ); }

	BOOL			WriteUniOrByteChar( sal_Unicode ch, rtl_TextEncoding eDestCharSet );
	BOOL			WriteUniOrByteChar( sal_Unicode ch )
			{ return WriteUniOrByteChar( ch, GetStreamCharSet() ); }

	void			SetBufferSize( USHORT nBufSize );
	USHORT			GetBufferSize() const { return nBufSize; }

	void			RefreshBuffer();
	SvStream&		PutBack( char aCh );
	void			EatWhite();

	BOOL			IsWritable() const { return bIsWritable; }
	StreamMode		GetStreamMode() const { return eStreamMode; }
	virtual USHORT	IsA() const;

	long			GetVersion() { return nVersion; }
	void			SetVersion( long n ) { nVersion = n; }

	virtual void	AddMark(ULONG nPos);

	virtual void	RemoveMark(ULONG nPos);

	friend SvStream& operator<<( SvStream& rStr, SvStrPtr f ); 
};

inline SvStream& operator<<( SvStream& rStr, SvStrPtr f )
{
	(*f)(rStr);
	return rStr;
}

inline SvStream& SvStream::ReadNumber( short& rShort )
{
	long nTmp;
	ReadNumber( nTmp );
	rShort = (short)nTmp;
	return *this;
}

inline SvStream& SvStream::ReadNumber( USHORT& rUShort )
{
	ULONG nTmp;
	ReadNumber( nTmp );
	rUShort = (USHORT)nTmp;
	return *this;
}

inline SvStream& SvStream::ReadNumber( int& rInt )
{
	long nTmp;
	ReadNumber( nTmp );
	rInt = (int)nTmp;
	return *this;
}

inline SvStream& SvStream::ReadNumber( unsigned int& rUInt )
{
	ULONG nTmp;
	ReadNumber( nTmp );
	rUInt = (unsigned int)nTmp;
	return *this;
}

inline SvStream& SvStream::ReadNumber( float& rFloat )
{
	double nTmp;
	ReadNumber( nTmp );
	rFloat = (float)nTmp;
	return *this;
}

inline SvStream& SvStream::WriteNumber( short nShort )
{
	WriteNumber( (long)nShort );
	return *this;
}

inline SvStream& SvStream::WriteNumber( USHORT nUShort )
{
	WriteNumber( (ULONG)nUShort );
	return *this;
}

inline SvStream& SvStream::WriteNumber( int nInt )
{
	WriteNumber( (long)nInt );
	return *this;
}

inline SvStream& SvStream::WriteNumber( unsigned int nUInt )
{
	WriteNumber( (ULONG)nUInt );
	return *this;
}

inline SvStream& SvStream::WriteNumber( float nFloat )
{
	double nTemp = nFloat;
	WriteNumber( nTemp );
	return *this;
}

inline void SvStream::SetEndianSwap( BOOL bVal )
{
#ifdef __BIGENDIAN
	SetNumberFormatInt( bVal ? NUMBERFORMAT_INT_LITTLEENDIAN : NUMBERFORMAT_INT_BIGENDIAN );
#else
	SetNumberFormatInt( bVal ? NUMBERFORMAT_INT_BIGENDIAN : NUMBERFORMAT_INT_LITTLEENDIAN );
#endif
}

SvStream& endl( SvStream& rStr );

SvStream& endlu( SvStream& rStr );

SvStream& endlub( SvStream& rStr );

class SvFileStream : public SvStream
{
	friend class ImpEaMgr;
	friend class CORmFileStream;

private:
	StreamData* 		pInstanceData;
	String				aFilename;
	USHORT				nLockCounter;
	BOOL				bIsOpen;
	ULONG				GetFileHandle() const;

protected:

	virtual ULONG	GetData( void* pData, ULONG nSize );
	virtual ULONG	PutData( const void* pData, ULONG nSize );
	virtual ULONG	SeekPos( ULONG nPos );
	virtual void	SetSize( ULONG nSize );
	virtual void	FlushData();

public:
			SvFileStream( const String& rFileName, StreamMode eOpenMode );
		SvFileStream();
		~SvFileStream();

	virtual void	ResetError();

	BOOL			LockRange( ULONG nByteOffset, ULONG nBytes );
	BOOL			UnlockRange( ULONG nByteOffset, ULONG nBytes );
	BOOL			LockFile();
	BOOL			UnlockFile();

	void			Open( const String& rFileName, StreamMode eOpenMode );
	void			Close();
	void			ReOpen(); 
	BOOL			IsOpen() const { return bIsOpen; }
	BOOL			IsLocked() const { return ( nLockCounter!=0 ); }
	virtual USHORT	IsA() const;

	const String&	GetFileName() const { return aFilename; }
};

class SvMemoryStream : public SvStream
{
protected:
	ULONG			nSize;
	ULONG			nResize;
	ULONG			nPos;
	ULONG			nEndOfData;
	BYTE*			pBuf;
	BOOL			bOwnsData;

	virtual ULONG	GetData( void* pData, ULONG nSize );
	virtual ULONG	PutData( const void* pData, ULONG nSize );
	virtual ULONG	SeekPos( ULONG nPos );
	virtual void	SetSize( ULONG nSize );
	virtual void	FlushData();

	virtual BOOL	AllocateMemory( ULONG nSize );

	virtual BOOL	ReAllocateMemory( long nDiff );

	virtual void	FreeMemory();
			SvMemoryStream(void*) { }	

public:
		SvMemoryStream( void* pBuf, ULONG nSize, StreamMode eMode);
		SvMemoryStream( ULONG nInitSize=512, ULONG nResize=64 );
		~SvMemoryStream();

	virtual void	ResetError();

	ULONG			GetSize() const { return nSize; }
	const void* 	GetData() { Flush(); return pBuf; }
	operator const	void*() { Flush(); return pBuf; }
	virtual USHORT	IsA() const;

	void*			SwitchBuffer( ULONG nInitSize=512, ULONG nResize=64 );
	void*			SetBuffer( void* pBuf, ULONG nSize,
				   BOOL bOwnsData=TRUE, ULONG nEOF=0 );

	void			ObjectOwnsMemory( BOOL bOwn ) { bOwnsData = bOwn; }
	BOOL			IsObjectMemoryOwner() { return bOwnsData; }
	void			SetResizeOffset( ULONG nNewResize ) { nResize = nNewResize; }
	ULONG			GetResizeOffset() const { return nResize; }
};

class SvSharedMemoryStream : public SvMemoryStream
{
	void*			aHandle;
protected:

	virtual BOOL	AllocateMemory( ULONG nSize );
	virtual BOOL	ReAllocateMemory( long nDiff );
	virtual void	FreeMemory();

	virtual ULONG	GetData( void* pData, ULONG nSize );
	virtual ULONG	PutData( const void* pData, ULONG nSize );

public:
		SvSharedMemoryStream( void* pBuf, ULONG nSize, StreamMode eMode );
		SvSharedMemoryStream( ULONG nInitSize=4000, ULONG nResize=4000 );
		~SvSharedMemoryStream();

	const void* 	GetData();
	operator const	void*();
	virtual USHORT	IsA() const;

	void*			SwitchBuffer( ULONG nInitSize=4000, ULONG nResize=4000 );

	void*			SetBuffer( void* pBuf, ULONG nSize,
				   BOOL bOwnsMem = TRUE, ULONG nEOF = 0 );

	void*			SetHandle( void* aHandle, ULONG nSize,
				   BOOL bOwnsHandle = TRUE, ULONG nEOF = 0);

	void*			GetHandle() const { return aHandle; }
};

class SvDataCopyStream
{
public:
			TYPEINFO();
	virtual 		~SvDataCopyStream(){}
	virtual void	Load( SvStream & ) = 0;
	virtual void	Save( SvStream & ) = 0;
	virtual void	Assign( const SvDataCopyStream & );
};

#endif 
