

#ifndef _URLOBJ_HXX
#define _URLOBJ_HXX

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#define INET_PASS_TOKEN '@'
#define INET_DELIM_TOKEN ':'
#define INET_DELIM_IMAPID ';'
#define INET_DELIM_NEWSID '<'
#define INET_ENC_DELIM_TOKEN '|'
#define INET_DELIM_HOST_TOKEN '.'
#define INET_PATH_TOKEN '/'
#define INET_DOSPATH_TOKEN '\\'
#define INET_MARK_TOKEN '#'
#define INET_PARAM_TOKEN '?'
#define INET_HEX_ESCAPE '%'

#define	INET_FTP_SCHEME "ftp://"
#define INET_HTTP_SCHEME "http://"
#define INET_HTTPS_SCHEME "https://"
#define INET_FILE_SCHEME "file://"
#define INET_GOPHER_SCHEME "gopher://"
#define INET_MAILTO_SCHEME "mailto:"
#define INET_CID_SCHEME "cid:"
#define INET_NEWS_SCHEME "news:"
#define INET_NEWS_SRV_SCHEME "news://"
#define INET_POP3_SCHEME "pop3://"
#define INET_LDAP_SCHEME "ldap://"
#define INET_PRIVATE_SCHEME "private:"
#define INET_BUGDOC_SCHEME "bugdoc:"
#define INET_SLOT_SCHEME "slot:"
#define INET_MACRO_SCHEME "macro:"
#define INET_JAVASCRIPT_SCHEME "javascript:"
#define INET_IMAP_SCHEME "imap://"
#define INET_DOWNPASS_SCHEME ".."
#define INET_DATA_SCHEME "data:"
#define INET_OUT_SCHEME "out://"
#define INET_FIF_SCHEME "fif://"
#define INET_CEPT_SCHEME "cept://"
#define INET_VIM_SCHEME "vim://"
#define INET_UNO_SCHEME ".uno:"
#define INET_COMPONENT_SCHEME ".component:"
#define INET_DB_SCHEME "db:"
#define INET_BUGID_SCHEME "bugid:"

#define URL_PREFIX_PRIV_SOFFICE "private:"
enum
{
	URL_PREFIX_PRIV_SOFFICE_LEN
	 = RTL_CONSTASCII_LENGTH(URL_PREFIX_PRIV_SOFFICE)
};

#define URL_PREFIX_PRIV_OBSOLETE URL_PREFIX_PRIV_SOFFICE
enum
{
	URL_PREFIX_PRIV_OBSOLETE_LEN
	 = RTL_CONSTASCII_LENGTH(URL_PREFIX_PRIV_OBSOLETE)
};

#define URL_PREFIX_PRIV_EXTERN "staroffice:"
enum
{
	URL_PREFIX_PRIV_EXTERN_LEN = RTL_CONSTASCII_LENGTH(URL_PREFIX_PRIV_EXTERN)
};

enum INetProtocol
{
	INET_PROT_NOT_VALID = 0,
	INET_PROT_FTP = 1,
	INET_PROT_HTTP = 2,
	INET_PROT_FILE = 3,
	INET_PROT_MAILTO = 4,
	INET_PROT_VND_SUN_STAR_WEBDAV = 5,
	INET_PROT_NEWS = 6,
	INET_PROT_PRIV_SOFFICE = 7,
	INET_PROT_PRIVATE = INET_PROT_PRIV_SOFFICE, 
	INET_PROT_VND_SUN_STAR_HELP = 8,
	INET_PROT_HTTPS = 9,
	INET_PROT_SLOT = 10,
	INET_PROT_MACRO = 11,
	INET_PROT_JAVASCRIPT = 12,
	INET_PROT_IMAP = 13,
	INET_PROT_POP3 = 14,
	INET_PROT_DATA = 15,
	INET_PROT_CID = 16,
	INET_PROT_OUT = 17,
	INET_PROT_VND_SUN_STAR_WFS = 18,
	INET_PROT_VND_SUN_STAR_HIER = 19,
	INET_PROT_VIM = 20,
	INET_PROT_UNO = 21,
	INET_PROT_COMPONENT = 22,
	INET_PROT_VND_SUN_STAR_PKG = 23,
	INET_PROT_LDAP = 24,
	INET_PROT_DB = 25,
	INET_PROT_VND_SUN_STAR_CMD = 26,
	INET_PROT_VND_SUN_STAR_SCRIPT = 27,
	INET_PROT_END = 28
};

class INetURLObject
{
public:

	enum EncodeMechanism
	{

		ENCODE_ALL,

		WAS_ENCODED,

		NOT_CANONIC
	};

	enum DecodeMechanism
	{

		NO_DECODE,

		DECODE_TO_IURI,

		DECODE_WITH_CHARSET,

		DECODE_UNAMBIGUOUS
	};

	inline INetURLObject():
		m_eScheme(INET_PROT_NOT_VALID), m_eSmartScheme(INET_PROT_HTTP) {}

	inline bool HasError() const { return m_eScheme == INET_PROT_NOT_VALID; }

	inline UniString GetMainURL(DecodeMechanism eMechanism = DECODE_TO_IURI,
					rtl_TextEncoding eCharset
					    = RTL_TEXTENCODING_UTF8) const
	{ return decode(m_aAbsURIRef, getEscapePrefix(), eMechanism, eCharset); }

	UniString GetURLNoPass(DecodeMechanism eMechanism = DECODE_TO_IURI,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
		const;

	UniString GetURLNoMark(DecodeMechanism eMechanism = DECODE_TO_IURI,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
		const;

	bool operator ==(INetURLObject const & rObject) const;

	inline bool operator !=(INetURLObject const & rObject) const
	{ return !(*this == rObject); }

	bool operator <(INetURLObject const & rObject) const;

	inline bool operator >(INetURLObject const & rObject) const
	{ return rObject < *this; }

	inline INetURLObject(ByteString const & rTheAbsURIRef,
			 EncodeMechanism eMechanism = WAS_ENCODED,
			 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline INetURLObject(UniString const & rTheAbsURIRef,
			 EncodeMechanism eMechanism = WAS_ENCODED,
			 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline bool SetURL(ByteString const & rTheAbsURIRef,
		   EncodeMechanism eMechanism = WAS_ENCODED,
		   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline bool SetURL(UniString const & rTheAbsURIRef,
		   EncodeMechanism eMechanism = WAS_ENCODED,
		   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	bool ConcatData(INetProtocol eTheScheme, UniString const & rTheUser,
		UniString const & rThePassword,
		UniString const & rTheHost, sal_uInt32 nThePort,
		UniString const & rThePath,
		EncodeMechanism eMechanism = WAS_ENCODED,
		rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	enum FSysStyle
	{

		FSYS_VOS = 0x1,

		FSYS_UNX = 0x2,

		FSYS_DOS = 0x4,

		FSYS_MAC = 0x8,

		FSYS_DETECT = FSYS_VOS | FSYS_UNX | FSYS_DOS | FSYS_MAC
	};

	inline INetURLObject(UniString const & rTheAbsURIRef,
			 INetProtocol eTheSmartScheme,
			 EncodeMechanism eMechanism = WAS_ENCODED,
			 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
			 FSysStyle eStyle = FSYS_DETECT);

	inline void SetSmartProtocol(INetProtocol eTheSmartScheme)
	{ m_eSmartScheme = eTheSmartScheme; }

	inline bool
	SetSmartURL(ByteString const & rTheAbsURIRef,
	EncodeMechanism eMechanism = WAS_ENCODED,
	rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	FSysStyle eStyle = FSYS_DETECT);

	inline bool
	SetSmartURL(UniString const & rTheAbsURIRef,
	EncodeMechanism eMechanism = WAS_ENCODED,
	rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	FSysStyle eStyle = FSYS_DETECT);

	inline INetURLObject
	smartRel2Abs(ByteString const & rTheRelURIRef,
	 bool & rWasAbsolute,
	 bool bIgnoreFragment = false,
	 EncodeMechanism eMechanism = WAS_ENCODED,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	 bool bRelativeNonURIs = false,
	 FSysStyle eStyle = FSYS_DETECT) const;

	inline INetURLObject
	smartRel2Abs(UniString const & rTheRelURIRef,
	 bool & rWasAbsolute,
	 bool bIgnoreFragment = false,
	 EncodeMechanism eMechanism = WAS_ENCODED,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	 bool bRelativeNonURIs = false,
	 FSysStyle eStyle = FSYS_DETECT) const;

	inline bool
	GetNewAbsURL(ByteString const & rTheRelURIRef,
	 INetURLObject * pTheAbsURIRef,
	 EncodeMechanism eMechanism = WAS_ENCODED,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	 FSysStyle eStyle = FSYS_DETECT) const;

	inline bool
	GetNewAbsURL(UniString const & rTheRelURIRef,
	 INetURLObject * pTheAbsURIRef,
	 EncodeMechanism eMechanism = WAS_ENCODED,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	 FSysStyle eStyle = FSYS_DETECT) const;

	static inline UniString
	GetRelURL(ByteString const & rTheBaseURIRef,
	 ByteString const & rTheAbsURIRef,
	 EncodeMechanism eEncodeMechanism = WAS_ENCODED,
	 DecodeMechanism eDecodeMechanism = DECODE_TO_IURI,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	 FSysStyle eStyle = FSYS_DETECT);

	static inline UniString
	GetRelURL(UniString const & rTheBaseURIRef,
	 UniString const & rTheAbsURIRef,
	 EncodeMechanism eEncodeMechanism = WAS_ENCODED,
	 DecodeMechanism eDecodeMechanism = DECODE_TO_IURI,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	 FSysStyle eStyle = FSYS_DETECT);

	UniString getExternalURL(DecodeMechanism eMechanism = DECODE_TO_IURI,
				 rtl_TextEncoding eCharset
				     = RTL_TEXTENCODING_UTF8) const;

	static inline bool translateToExternal(ByteString const & rTheIntURIRef,
							   UniString & rTheExtURIRef,
							   EncodeMechanism eEncodeMechanism
							       = WAS_ENCODED,
							   DecodeMechanism eDecodeMechanism
							       = DECODE_TO_IURI,
							   rtl_TextEncoding eCharset
							       = RTL_TEXTENCODING_UTF8);

	static inline bool translateToExternal(UniString const & rTheIntURIRef,
							   UniString & rTheExtURIRef,
							   EncodeMechanism eEncodeMechanism
							       = WAS_ENCODED,
							   DecodeMechanism eDecodeMechanism
							       = DECODE_TO_IURI,
							   rtl_TextEncoding eCharset
							       = RTL_TEXTENCODING_UTF8);

	static inline bool translateToInternal(ByteString const & rTheExtURIRef,
							   UniString & rTheIntURIRef,
							   EncodeMechanism eEncodeMechanism
							       = WAS_ENCODED,
							   DecodeMechanism eDecodeMechanism
							       = DECODE_TO_IURI,
							   rtl_TextEncoding eCharset
							       = RTL_TEXTENCODING_UTF8);

	static inline bool translateToInternal(UniString const & rTheExtURIRef,
							   UniString & rTheIntURIRef,
							   EncodeMechanism eEncodeMechanism
							       = WAS_ENCODED,
							   DecodeMechanism eDecodeMechanism
							       = DECODE_TO_IURI,
							   rtl_TextEncoding eCharset
							       = RTL_TEXTENCODING_UTF8);

	struct SchemeInfo;

	inline INetProtocol GetProtocol() const { return m_eScheme; }

	static UniString GetScheme(INetProtocol eTheScheme);

	static inline INetProtocol CompareProtocolScheme(ByteString const &
										     rTheAbsURIRef)
	{ return CompareProtocolScheme(extend(rTheAbsURIRef)); }

	static INetProtocol CompareProtocolScheme(UniString const &
								      rTheAbsURIRef);

	inline bool HasUserData() const { return m_aUser.isPresent(); }

	inline bool IsEmptyUser() const
	{ return m_aUser.isPresent() && m_aUser.isEmpty(); }

	bool hasPassword() const;

	inline bool IsEmptyPass() const
	{ return hasPassword() && m_aAuth.isEmpty(); }

	inline UniString GetUser(DecodeMechanism eMechanism = DECODE_TO_IURI,
				 rtl_TextEncoding eCharset
				     = RTL_TEXTENCODING_UTF8) const
	{ return decode(m_aUser, getEscapePrefix(), eMechanism, eCharset); }

	inline UniString GetPass(DecodeMechanism eMechanism = DECODE_TO_IURI,
				 rtl_TextEncoding eCharset
				     = RTL_TEXTENCODING_UTF8) const
	{ return decode(m_aAuth, getEscapePrefix(), eMechanism, eCharset); }

	inline bool SetUser(ByteString const & rTheUser,
			EncodeMechanism eMechanism = WAS_ENCODED,
			rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
	{ return setUser(extend(rTheUser), true, eMechanism, eCharset); }

	inline bool SetUser(UniString const & rTheUser,
			EncodeMechanism eMechanism = WAS_ENCODED,
			rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
	{ return setUser(rTheUser, false, eMechanism, eCharset); }

	void makeAuthCanonic();

	inline bool SetPass(ByteString const & rThePassword,
			EncodeMechanism eMechanism = WAS_ENCODED,
			rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline bool SetPass(UniString const & rThePassword,
			EncodeMechanism eMechanism = WAS_ENCODED,
			rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline bool SetUserAndPass(ByteString const & rTheUser,
				   ByteString const & rThePassword,
				   EncodeMechanism eMechanism = WAS_ENCODED,
				   rtl_TextEncoding eCharset
				       = RTL_TEXTENCODING_UTF8);

	inline bool SetUserAndPass(UniString const & rTheUser,
				   UniString const & rThePassword,
				   EncodeMechanism eMechanism = WAS_ENCODED,
				   rtl_TextEncoding eCharset
				       = RTL_TEXTENCODING_UTF8);

	inline bool HasPort() const { return m_aPort.isPresent(); }

	inline UniString GetHost(DecodeMechanism eMechanism = DECODE_TO_IURI,
				 rtl_TextEncoding eCharset
				     = RTL_TEXTENCODING_UTF8) const
	{ return decode(m_aHost, getEscapePrefix(), eMechanism, eCharset); }

	UniString GetHostPort(DecodeMechanism eMechanism = DECODE_TO_IURI,
			  rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	sal_uInt32 GetPort() const;

	inline bool SetHost(ByteString const & rTheHost,
			EncodeMechanism eMechanism = WAS_ENCODED,
			rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
	{ return setHost(extend(rTheHost), true, eMechanism, eCharset); }

	inline bool SetHost(UniString const & rTheHost,
			EncodeMechanism eMechanism = WAS_ENCODED,
			rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
	{ return setHost(rTheHost, false, eMechanism, eCharset); }

	bool SetPort(sal_uInt32 nThePort);

	void makePortCanonic();

	static inline UniString encodeHostPort(ByteString const & rTheHostPort,
							   EncodeMechanism eMechanism,
							   rtl_TextEncoding eCharset
							       = RTL_TEXTENCODING_UTF8);

	static inline UniString encodeHostPort(UniString const & rTheHostPort,
							   EncodeMechanism eMechanism,
							   rtl_TextEncoding eCharset
							       = RTL_TEXTENCODING_UTF8)
	{ return encodeHostPort(rTheHostPort, false, eMechanism, eCharset); }

	static bool parseHost(sal_Unicode const *& rBegin,
			  sal_Unicode const * pEnd, bool bOctets,
			  EncodeMechanism eMechanism,
			  rtl_TextEncoding eCharset, UniString & rCanonic);

	inline bool HasURLPath() const { return !m_aPath.isEmpty(); }

	inline UniString GetURLPath(DecodeMechanism eMechanism = DECODE_TO_IURI,
					rtl_TextEncoding eCharset
					    = RTL_TEXTENCODING_UTF8) const
	{ return decode(m_aPath, getEscapePrefix(), eMechanism, eCharset); }

	inline bool SetURLPath(ByteString const & rThePath,
			   EncodeMechanism eMechanism = WAS_ENCODED,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
	{ return setPath(extend(rThePath), true, eMechanism, eCharset); }

	inline bool SetURLPath(UniString const & rThePath,
			   EncodeMechanism eMechanism = WAS_ENCODED,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
	{ return setPath(rThePath, false, eMechanism, eCharset); }

	enum { LAST_SEGMENT = -1 };

	sal_Int32 getSegmentCount(bool bIgnoreFinalSlash = true) const;

	bool removeSegment(sal_Int32 nIndex = LAST_SEGMENT,
		   bool bIgnoreFinalSlash = true);

	inline bool insertName(UniString const & rTheName,
			   bool bAppendFinalSlash = false,
			   sal_Int32 nIndex = LAST_SEGMENT,
			   bool bIgnoreFinalSlash = true,
			   EncodeMechanism eMechanism = WAS_ENCODED,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	UniString getName(sal_Int32 nIndex = LAST_SEGMENT,
		  bool bIgnoreFinalSlash = true,
		  DecodeMechanism eMechanism = DECODE_TO_IURI,
		  rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
		const;

	bool setName(UniString const & rTheName, sal_Int32 nIndex = LAST_SEGMENT,
	 bool bIgnoreFinalSlash = true,
	 EncodeMechanism eMechanism = WAS_ENCODED,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	UniString getBase(sal_Int32 nIndex = LAST_SEGMENT,
		  bool bIgnoreFinalSlash = true,
		  DecodeMechanism eMechanism = DECODE_TO_IURI,
		  rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
		const;

	bool setBase(UniString const & rTheBase, sal_Int32 nIndex = LAST_SEGMENT,
	 bool bIgnoreFinalSlash = true,
	 EncodeMechanism eMechanism = WAS_ENCODED,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	bool hasExtension(sal_Int32 nIndex = LAST_SEGMENT,
		  bool bIgnoreFinalSlash = true) const;

	UniString getExtension(sal_Int32 nIndex = LAST_SEGMENT,
			   bool bIgnoreFinalSlash = true,
			   DecodeMechanism eMechanism = DECODE_TO_IURI,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
		const;

	bool setExtension(UniString const & rTheExtension,
		  sal_Int32 nIndex = LAST_SEGMENT,
		  bool bIgnoreFinalSlash = true,
		  EncodeMechanism eMechanism = WAS_ENCODED,
		  rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	bool removeExtension(sal_Int32 nIndex = LAST_SEGMENT,
			 bool bIgnoreFinalSlash = true);

	bool hasFinalSlash() const;

	bool setFinalSlash();

	bool removeFinalSlash();

	inline bool HasParam() const { return m_aQuery.isPresent(); }

	inline UniString GetParam(DecodeMechanism eMechanism = DECODE_TO_IURI,
				  rtl_TextEncoding eCharset
				      = RTL_TEXTENCODING_UTF8) const
	{ return decode(m_aQuery, getEscapePrefix(), eMechanism, eCharset); }

	inline bool SetParam(ByteString const & rTheQuery,
			 EncodeMechanism eMechanism = WAS_ENCODED,
			 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline bool SetParam(UniString const & rTheQuery,
			 EncodeMechanism eMechanism = WAS_ENCODED,
			 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline bool HasMark() const { return m_aFragment.isPresent(); }

	inline UniString GetMark(DecodeMechanism eMechanism = DECODE_TO_IURI,
				 rtl_TextEncoding eCharset
				     = RTL_TEXTENCODING_UTF8) const
	{ return decode(m_aFragment, getEscapePrefix(), eMechanism, eCharset); }

	inline bool SetMark(ByteString const & rTheFragment,
			EncodeMechanism eMechanism = WAS_ENCODED,
			rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline bool SetMark(UniString const & rTheFragment,
			EncodeMechanism eMechanism = WAS_ENCODED,
			rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	static UniString createFragment(UniString const & rText);

	inline INetURLObject(UniString const & rFSysPath, FSysStyle eStyle);

	bool setFSysPath(UniString const & rFSysPath, FSysStyle eStyle);

	UniString getFSysPath(FSysStyle eStyle, sal_Unicode * pDelimiter = 0)
		const;

	bool HasMsgId() const;

	UniString GetMsgId(DecodeMechanism eMechanism = DECODE_TO_IURI,
		   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
		const;
	 bool getParameter(UniString const & rKey, UniString * pValue);

	enum Part
	{
		PART_OBSOLETE_NORMAL = 0x001, 
		PART_OBSOLETE_FILE = 0x002, 
		PART_OBSOLETE_PARAM = 0x004, 
		PART_USER_PASSWORD = 0x008,
		PART_IMAP_ACHAR = 0x010,
		PART_VIM = 0x020,
		PART_HOST_EXTRA = 0x040,
		PART_FPATH = 0x080,
		PART_AUTHORITY = 0x100,
		PART_PATH_SEGMENTS_EXTRA = 0x200,
		PART_REL_SEGMENT_EXTRA = 0x400,
		PART_URIC = 0x800,
		PART_HTTP_PATH = 0x1000,
		PART_FILE_SEGMENT_EXTRA = 0x2000, 
		PART_MESSAGE_ID = 0x4000,
		PART_MESSAGE_ID_PATH = 0x8000,
		PART_MAILTO = 0x10000,
		PART_PATH_BEFORE_QUERY = 0x20000,
		PART_PCHAR = 0x40000,
		PART_FRAGMENT = 0x80000, 
		PART_VISIBLE = 0x100000,
		PART_VISIBLE_NONSPECIAL = 0x200000,
		PART_CREATEFRAGMENT = 0x400000,
		PART_UNO_PARAM_VALUE = 0x800000,
		PART_UNAMBIGUOUS = 0x1000000,
		PART_URIC_NO_SLASH = 0x2000000,
	    PART_HTTP_QUERY = 0x4000000,
		max_part = 0x80000000

	};

	enum EscapeType
	{
		ESCAPE_NO,
		ESCAPE_OCTET,
		ESCAPE_UTF32
	};

	static inline UniString encode(ByteString const & rText, Part ePart,
					   sal_Char cEscapePrefix,
					   EncodeMechanism eMechanism,
					   rtl_TextEncoding eCharset
					       = RTL_TEXTENCODING_UTF8);

	static inline UniString encode(UniString const & rText, Part ePart,
					   sal_Char cEscapePrefix,
					   EncodeMechanism eMechanism,
					   rtl_TextEncoding eCharset
					       = RTL_TEXTENCODING_UTF8);

	static inline UniString decode(UniString const & rText,
					   sal_Char cEscapePrefix,
					   DecodeMechanism eMechanism,
					   rtl_TextEncoding eCharset
					       = RTL_TEXTENCODING_UTF8);

	static void appendUCS4Escape(UniString & rTheText, sal_Char cEscapePrefix,
					 sal_uInt32 nUCS4);

	static void appendUCS4(UniString & rTheText, sal_uInt32 nUCS4,
			   EscapeType eEscapeType, bool bOctets, Part ePart,
			   sal_Char cEscapePrefix, rtl_TextEncoding eCharset,
			   bool bKeepVisibleEscapes);

	static sal_uInt32 getUTF32(sal_Unicode const *& rBegin,
				   sal_Unicode const * pEnd, bool bOctets,
				   sal_Char cEscapePrefix,
				   EncodeMechanism eMechanism,
				   rtl_TextEncoding eCharset,
				   EscapeType & rEscapeType);

	static sal_uInt32 scanDomain(sal_Unicode const *& rBegin,
					 sal_Unicode const * pEnd,
					 bool bEager = true);

	static bool scanIPv6reference(sal_Unicode const *& rBegin,
					  sal_Unicode const * pEnd,
					  bool bEager = true);

	static UniString
	RelToAbs(ByteString const & rTheRelURIRef,
	bool bIgnoreFragment = false,
	EncodeMechanism eEncodeMechanism = WAS_ENCODED,
	DecodeMechanism eDecodeMechanism = DECODE_TO_IURI,
	rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	FSysStyle eStyle = FSYS_DETECT);

	static UniString
	RelToAbs(UniString const & rTheRelURIRef,
	bool bIgnoreFragment = false,
	EncodeMechanism eEncodeMechanism = WAS_ENCODED,
	DecodeMechanism eDecodeMechanism = DECODE_TO_IURI,
	rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	FSysStyle eStyle = FSYS_DETECT);

	static UniString
	AbsToRel(ByteString const & rTheAbsURIRef,
	EncodeMechanism eEncodeMechanism = WAS_ENCODED,
	DecodeMechanism eDecodeMechanism = DECODE_TO_IURI,
	rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	FSysStyle eStyle = FSYS_DETECT);

	static UniString
	AbsToRel(UniString const & rTheAbsURIRef,
	EncodeMechanism eEncodeMechanism = WAS_ENCODED,
	DecodeMechanism eDecodeMechanism = DECODE_TO_IURI,
	rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8,
	FSysStyle eStyle = FSYS_DETECT);

	static bool SetBaseURL(ByteString const & rTheBaseURIRef,
			   EncodeMechanism eMechanism = WAS_ENCODED,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	static bool SetBaseURL(UniString const & rTheBaseURIRef,
			   EncodeMechanism eMechanism = WAS_ENCODED,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	static UniString GetBaseURL(DecodeMechanism eMechanism = DECODE_TO_IURI,
					rtl_TextEncoding eCharset
					    = RTL_TEXTENCODING_UTF8);

	UniString GetPartBeforeLastName(DecodeMechanism eMechanism
						    = DECODE_TO_IURI,
						rtl_TextEncoding eCharset
						    = RTL_TEXTENCODING_UTF8) const;

	UniString GetLastName(DecodeMechanism eMechanism = DECODE_TO_IURI,
			  rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
		const;

	UniString GetFileExtension(DecodeMechanism eMechanism = DECODE_TO_IURI,
				   rtl_TextEncoding eCharset
				       = RTL_TEXTENCODING_UTF8) const;

	inline bool Append(ByteString const & rTheSegment,
		   EncodeMechanism eMechanism = WAS_ENCODED,
		   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
	{ return appendSegment(extend(rTheSegment), true, eMechanism, eCharset); }

	inline bool Append(UniString const & rTheSegment,
		   EncodeMechanism eMechanism = WAS_ENCODED,
		   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8)
	{ return appendSegment(rTheSegment, false, eMechanism, eCharset); }

	bool CutLastName();

	UniString PathToFileName() const;

	UniString GetFull() const;

	UniString GetPath() const;

	void SetBase(UniString const & rTheBase);

	UniString GetBase() const;

	void SetName(UniString const & rTheName,
	 EncodeMechanism eMechanism = WAS_ENCODED,
	 rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline UniString GetName(DecodeMechanism eMechanism = DECODE_TO_IURI,
				 rtl_TextEncoding eCharset
				     = RTL_TEXTENCODING_UTF8) const
	{ return GetLastName(eMechanism, eCharset); }

	UniString CutName(DecodeMechanism eMechanism = DECODE_TO_IURI,
		  rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	void SetExtension(UniString const & rTheExtension,
		  EncodeMechanism eMechanism = WAS_ENCODED,
		  rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	inline UniString GetExtension(DecodeMechanism eMechanism = DECODE_TO_IURI,
					  rtl_TextEncoding eCharset
					      = RTL_TEXTENCODING_UTF8) const
	{ return GetFileExtension(eMechanism, eCharset); }

	UniString CutExtension(DecodeMechanism eMechanism = DECODE_TO_IURI,
			   rtl_TextEncoding eCharset = RTL_TEXTENCODING_UTF8);

	bool IsCaseSensitive() const;

private:

	class SubString
	{
		xub_StrLen m_nBegin;
		xub_StrLen m_nLength;

	public:
		explicit inline SubString(xub_StrLen nTheBegin = STRING_NOTFOUND,
					  xub_StrLen nTheLength = 0):
			m_nBegin(nTheBegin), m_nLength(nTheLength) {}

		inline bool isPresent() const { return m_nBegin != STRING_NOTFOUND; }

		inline bool isEmpty() const { return m_nLength == 0; }

		inline xub_StrLen getBegin() const { return m_nBegin; }

		inline xub_StrLen getLength() const { return m_nLength; }

		inline xub_StrLen getEnd() const { return m_nBegin + m_nLength; }

		inline sal_Int32 clear();

		inline sal_Int32 set(UniString & rString,
				 UniString const & rSubString);

		inline sal_Int32 set(UniString & rString,
				 UniString const & rSubString,
				 xub_StrLen nTheBegin);

		inline void operator +=(sal_Int32 nDelta);

		bool equals(SubString const & rOther, UniString const & rThisString,
		UniString const & rOtherString) const;
	};

	UniString m_aAbsURIRef;
	SubString m_aUser;
	SubString m_aAuth;
	SubString m_aHost;
	SubString m_aPort;
	SubString m_aPath;
	SubString m_aQuery;
	SubString m_aFragment;
	INetProtocol m_eScheme;
	INetProtocol m_eSmartScheme;

	void setInvalid();

	bool setAbsURIRef(UniString const & rTheAbsURIRef, bool bOctets,
		  EncodeMechanism eMechanism, rtl_TextEncoding eCharset,
		  bool bSmart, FSysStyle eStyle);

	static INetURLObject m_aBaseURIRef;

	bool convertRelToAbs(UniString const & rTheRelURIRef, bool bOctets,
			 INetURLObject & rTheAbsURIRef, bool & rWasAbsolute,
			 EncodeMechanism eMechanism,
			 rtl_TextEncoding eCharset, bool bIgnoreFragment,
			 bool bSmart, bool bRelativeNonURIs, FSysStyle eStyle)
		const;

	bool convertAbsToRel(UniString const & rTheAbsURIRef, bool bOctets,
			 UniString & rTheRelURIRef,
			 EncodeMechanism eEncodeMechanism,
			 DecodeMechanism eDecodeMechanism,
			 rtl_TextEncoding eCharset, FSysStyle eStyle) const;

	static bool convertIntToExt(UniString const & rTheIntURIRef, bool bOctets,
					UniString & rTheExtURIRef,
					EncodeMechanism eEncodeMechanism,
					DecodeMechanism eDecodeMechanism,
					rtl_TextEncoding eCharset);

	static bool convertExtToInt(UniString const & rTheExtURIRef, bool bOctets,
					UniString & rTheIntURIRef,
					EncodeMechanism eEncodeMechanism,
					DecodeMechanism eDecodeMechanism,
					rtl_TextEncoding eCharset);

	struct PrefixInfo;

	static inline SchemeInfo const & getSchemeInfo(INetProtocol eTheScheme);

	inline SchemeInfo const & getSchemeInfo() const;

	static PrefixInfo const * getPrefix(sal_Unicode const *& rBegin,
							sal_Unicode const * pEnd);

	SubString getAuthority() const;

	bool setUser(UniString const & rTheUser, bool bOctets,
	 EncodeMechanism eMechanism, rtl_TextEncoding eCharset);

	bool clearPassword();

	bool setPassword(UniString const & rThePassword, bool bOctets,
		 EncodeMechanism eMechanism, rtl_TextEncoding eCharset);

	static UniString encodeHostPort(UniString const & rTheHostPort,
						bool bOctets, EncodeMechanism eMechanism,
						rtl_TextEncoding eCharset);

	bool setHost(UniString const & rTheHost, bool bOctets,
	 EncodeMechanism eMechanism, rtl_TextEncoding eCharset);

	bool parsePath(sal_Unicode const ** pBegin,
	   sal_Unicode const * pEnd,
	   bool bOctets,
	   EncodeMechanism eMechanism,
	   rtl_TextEncoding eCharset,
	   bool bSkippedInitialSlash,
	   sal_uInt32 nSegmentDelimiter,
	   sal_uInt32 nAltSegmentDelimiter,
	   sal_uInt32 nQueryDelimiter,
	   sal_uInt32 nFragmentDelimiter,
	   UniString * pSynPath);

	bool setPath(UniString const & rThePath, bool bOctets,
	 EncodeMechanism eMechanism, rtl_TextEncoding eCharset);

	bool appendSegment(UniString const & rTheSegment, bool bOctets,
		   EncodeMechanism eMechanism, rtl_TextEncoding eCharset);

	SubString getSegment(sal_Int32 nIndex, bool bIgnoreFinalSlash) const;

	bool insertName(UniString const & rTheName, bool bOctets,
		bool bAppendFinalSlash, sal_Int32 nIndex,
		bool bIgnoreFinalSlash, EncodeMechanism eMechanism,
		rtl_TextEncoding eCharset);

	bool clearQuery();

	bool setQuery(UniString const & rTheQuery, bool bOctets,
	  EncodeMechanism eMechanism, rtl_TextEncoding eCharset);

	bool clearFragment();

	bool setFragment(UniString const & rTheMark, bool bOctets,
		 EncodeMechanism eMechanism, rtl_TextEncoding eCharset);

	enum FTPType { FTP_TYPE_NONE, FTP_TYPE_A, FTP_TYPE_I, FTP_TYPE_D };

	FTPType getFTPType() const;

	sal_uInt32 getIMAPUID() const;

	static inline UniString extend(ByteString const & rOctets)
	{ return UniString(rOctets, RTL_TEXTENCODING_ISO_8859_1); }

	static inline sal_Char getEscapePrefix(INetProtocol eTheScheme)
	{ return eTheScheme == INET_PROT_VIM ? '=' : '%'; }

	inline sal_Char getEscapePrefix() const
	{ return getEscapePrefix(m_eScheme); }

	static inline void appendEscape(UniString & rTheText,
						sal_Char cEscapePrefix,
						sal_uInt32 nOctet);

	static inline void appendUTF32(UniString & rTheText, sal_uInt32 nUTF32);

	static UniString encodeText(sal_Unicode const * pBegin,
					sal_Unicode const * pEnd, bool bOctets,
					Part ePart, sal_Char cEscapePrefix,
					EncodeMechanism eMechanism,
					rtl_TextEncoding eCharset,
					bool bKeepVisibleEscapes);

	static inline UniString encodeText(UniString const & rTheText,
						   bool bOctets, Part ePart,
						   sal_Char cEscapePrefix,
						   EncodeMechanism eMechanism,
						   rtl_TextEncoding eCharset,
						   bool bKeepVisibleEscapes);

	static UniString decode(sal_Unicode const * pBegin,
				sal_Unicode const * pEnd, sal_Char cEscapePrefix,
				DecodeMechanism, rtl_TextEncoding eCharset);

	inline UniString decode(SubString const & rSubString,
				sal_Char cEscapePrefix,
				DecodeMechanism eMechanism,
				rtl_TextEncoding eCharset) const;
};

inline UniString INetURLObject::encodeText(UniString const & rTheText,
							   bool bOctets, Part ePart,
							   sal_Char cEscapePrefix,
							   EncodeMechanism eMechanism,
							   rtl_TextEncoding eCharset,
							   bool bKeepVisibleEscapes)
{
	return encodeText(rTheText.GetBuffer(),
		  rTheText.GetBuffer() + rTheText.Len(), bOctets, ePart,
		  cEscapePrefix, eMechanism, eCharset,
		  bKeepVisibleEscapes);
}

inline UniString INetURLObject::decode(SubString const & rSubString,
						   sal_Char cEscapePrefix,
						   DecodeMechanism eMechanism,
						   rtl_TextEncoding eCharset) const
{
	return decode(m_aAbsURIRef.GetBuffer() + rSubString.getBegin(),
	  m_aAbsURIRef.GetBuffer() + rSubString.getEnd(),
	  cEscapePrefix, eMechanism, eCharset);
}

inline INetURLObject::INetURLObject(ByteString const & rTheAbsURIRef,
						EncodeMechanism eMechanism,
						rtl_TextEncoding eCharset):
	m_eScheme(INET_PROT_NOT_VALID), m_eSmartScheme(INET_PROT_HTTP)
{
	setAbsURIRef(extend(rTheAbsURIRef), true, eMechanism, eCharset, false,
	 FSysStyle(0));
}

inline INetURLObject::INetURLObject(UniString const & rTheAbsURIRef,
						EncodeMechanism eMechanism,
						rtl_TextEncoding eCharset):
	m_eScheme(INET_PROT_NOT_VALID), m_eSmartScheme(INET_PROT_HTTP)
{
	setAbsURIRef(rTheAbsURIRef, false, eMechanism, eCharset, false,
	 FSysStyle(0));
}

inline bool INetURLObject::SetURL(ByteString const & rTheAbsURIRef,
					  EncodeMechanism eMechanism,
					  rtl_TextEncoding eCharset)
{
	return setAbsURIRef(extend(rTheAbsURIRef), true, eMechanism, eCharset,
			false, FSysStyle(0));
}

inline bool INetURLObject::SetURL(UniString const & rTheAbsURIRef,
					  EncodeMechanism eMechanism,
					  rtl_TextEncoding eCharset)
{
	return setAbsURIRef(rTheAbsURIRef, false, eMechanism, eCharset, false,
			FSysStyle(0));
}

inline INetURLObject::INetURLObject(UniString const & rTheAbsURIRef,
						INetProtocol eTheSmartScheme,
						EncodeMechanism eMechanism,
						rtl_TextEncoding eCharset,
						FSysStyle eStyle):
	m_eScheme(INET_PROT_NOT_VALID), m_eSmartScheme(eTheSmartScheme)
{
	setAbsURIRef(rTheAbsURIRef, false, eMechanism, eCharset, true, eStyle);
}

inline bool INetURLObject::SetSmartURL(ByteString const & rTheAbsURIRef,
						   EncodeMechanism eMechanism,
						   rtl_TextEncoding eCharset,
						   FSysStyle eStyle)
{
	return setAbsURIRef(extend(rTheAbsURIRef), true, eMechanism, eCharset,
			true, eStyle);
}

inline bool INetURLObject::SetSmartURL(UniString const & rTheAbsURIRef,
						   EncodeMechanism eMechanism,
						   rtl_TextEncoding eCharset,
						   FSysStyle eStyle)
{
	return setAbsURIRef(rTheAbsURIRef, false, eMechanism, eCharset, true,
			eStyle);
}

inline INetURLObject
INetURLObject::smartRel2Abs(ByteString const & rTheRelURIRef,
				bool & rWasAbsolute,
				bool bIgnoreFragment,
				EncodeMechanism eMechanism,
				rtl_TextEncoding eCharset,
				bool bRelativeNonURIs,
				FSysStyle eStyle) const
{
	INetURLObject aTheAbsURIRef;
	convertRelToAbs(extend(rTheRelURIRef), true, aTheAbsURIRef, rWasAbsolute,
		eMechanism, eCharset, bIgnoreFragment, true,
		bRelativeNonURIs, eStyle);
	return aTheAbsURIRef;
}

inline INetURLObject
INetURLObject::smartRel2Abs(UniString const & rTheRelURIRef,
				bool & rWasAbsolute,
				bool bIgnoreFragment,
				EncodeMechanism eMechanism,
				rtl_TextEncoding eCharset,
				bool bRelativeNonURIs,
				FSysStyle eStyle) const
{
	INetURLObject aTheAbsURIRef;
	convertRelToAbs(rTheRelURIRef, false, aTheAbsURIRef, rWasAbsolute,
		eMechanism, eCharset, bIgnoreFragment, true,
		bRelativeNonURIs, eStyle);
	return aTheAbsURIRef;
}

inline bool INetURLObject::GetNewAbsURL(ByteString const & rTheRelURIRef,
							INetURLObject * pTheAbsURIRef,
							EncodeMechanism eMechanism,
							rtl_TextEncoding eCharset,
							FSysStyle eStyle) const
{
	INetURLObject aTheAbsURIRef;
	bool bWasAbsolute;
	if (!convertRelToAbs(extend(rTheRelURIRef), true, aTheAbsURIRef,
			 bWasAbsolute, eMechanism, eCharset, false, false,
			 false, eStyle))
		return false;
	if (pTheAbsURIRef)
		*pTheAbsURIRef = aTheAbsURIRef;
	return true;
}

inline bool INetURLObject::GetNewAbsURL(UniString const & rTheRelURIRef,
							INetURLObject * pTheAbsURIRef,
							EncodeMechanism eMechanism,
							rtl_TextEncoding eCharset,
							FSysStyle eStyle) const
{
	INetURLObject aTheAbsURIRef;
	bool bWasAbsolute;
	if (!convertRelToAbs(rTheRelURIRef, false, aTheAbsURIRef, bWasAbsolute,
			 eMechanism, eCharset, false, false, false, eStyle))
		return false;
	if (pTheAbsURIRef)
		*pTheAbsURIRef = aTheAbsURIRef;
	return true;
}

inline UniString INetURLObject::GetRelURL(ByteString const & rTheBaseURIRef,
							  ByteString const & rTheAbsURIRef,
							  EncodeMechanism eEncodeMechanism,
							  DecodeMechanism eDecodeMechanism,
							  rtl_TextEncoding eCharset,
							  FSysStyle eStyle)
{
	UniString aTheRelURIRef;
	INetURLObject(rTheBaseURIRef, eEncodeMechanism, eCharset).
		convertAbsToRel(extend(rTheAbsURIRef), true, aTheRelURIRef,
			eEncodeMechanism, eDecodeMechanism, eCharset, eStyle);
	return aTheRelURIRef;
}

inline UniString INetURLObject::GetRelURL(UniString const & rTheBaseURIRef,
							  UniString const & rTheAbsURIRef,
							  EncodeMechanism eEncodeMechanism,
							  DecodeMechanism eDecodeMechanism,
							  rtl_TextEncoding eCharset,
							  FSysStyle eStyle)
{
	UniString aTheRelURIRef;
	INetURLObject(rTheBaseURIRef, eEncodeMechanism, eCharset).
		convertAbsToRel(rTheAbsURIRef, false, aTheRelURIRef, eEncodeMechanism,
			eDecodeMechanism, eCharset, eStyle);
	return aTheRelURIRef;
}

inline bool INetURLObject::translateToExternal(ByteString const &
								       rTheIntURIRef,
								   UniString & rTheExtURIRef,
								   EncodeMechanism
								       eEncodeMechanism,
								   DecodeMechanism
								       eDecodeMechanism,
								   rtl_TextEncoding eCharset)
{
	return convertIntToExt(extend(rTheIntURIRef), true, rTheExtURIRef,
			   eEncodeMechanism, eDecodeMechanism, eCharset);
}

inline bool INetURLObject::translateToExternal(UniString const &
								       rTheIntURIRef,
								   UniString & rTheExtURIRef,
								   EncodeMechanism
								       eEncodeMechanism,
								   DecodeMechanism
								       eDecodeMechanism,
								   rtl_TextEncoding eCharset)
{
	return convertIntToExt(rTheIntURIRef, false, rTheExtURIRef,
			   eEncodeMechanism, eDecodeMechanism, eCharset);
}

inline bool INetURLObject::translateToInternal(ByteString const &
								       rTheExtURIRef,
								   UniString & rTheIntURIRef,
								   EncodeMechanism
								       eEncodeMechanism,
								   DecodeMechanism
								       eDecodeMechanism,
								   rtl_TextEncoding eCharset)
{
	return convertExtToInt(extend(rTheExtURIRef), true, rTheIntURIRef,
			   eEncodeMechanism, eDecodeMechanism, eCharset);
}

inline bool INetURLObject::translateToInternal(UniString const &
								       rTheExtURIRef,
								   UniString & rTheIntURIRef,
								   EncodeMechanism
								       eEncodeMechanism,
								   DecodeMechanism
								       eDecodeMechanism,
								   rtl_TextEncoding eCharset)
{
	return convertExtToInt(rTheExtURIRef, false, rTheIntURIRef,
			   eEncodeMechanism, eDecodeMechanism, eCharset);
}

inline bool INetURLObject::SetPass(ByteString const & rThePassword,
					   EncodeMechanism eMechanism,
					   rtl_TextEncoding eCharset)
{
	return rThePassword.Len() == 0 ?
	     clearPassword() :
	     setPassword(extend(rThePassword), true, eMechanism, eCharset);
}

inline bool INetURLObject::SetPass(UniString const & rThePassword,
					   EncodeMechanism eMechanism,
					   rtl_TextEncoding eCharset)
{
	return rThePassword.Len() == 0 ?
	     clearPassword() :
	     setPassword(rThePassword, false, eMechanism, eCharset);
}

inline bool INetURLObject::SetUserAndPass(ByteString const & rTheUser,
							  ByteString const & rThePassword,
							  EncodeMechanism eMechanism,
							  rtl_TextEncoding eCharset)
{
	return setUser(extend(rTheUser), true, eMechanism, eCharset)
	 && (rThePassword.Len() == 0 ?
	      clearPassword() :
	      setPassword(extend(rThePassword), true, eMechanism,
				   eCharset));
}

inline bool INetURLObject::SetUserAndPass(UniString const & rTheUser,
							  UniString const & rThePassword,
							  EncodeMechanism eMechanism,
							  rtl_TextEncoding eCharset)
{
	return setUser(rTheUser, false, eMechanism, eCharset)
	 && (rThePassword.Len() == 0 ?
	      clearPassword() :
	      setPassword(rThePassword, false, eMechanism, eCharset));
}

inline UniString INetURLObject::encodeHostPort(ByteString const &
								       rTheHostPort,
								   EncodeMechanism eMechanism,
								   rtl_TextEncoding eCharset)
{
	return encodeHostPort(extend(rTheHostPort), true, eMechanism, eCharset);
}

inline bool INetURLObject::insertName(UniString const & rTheName,
						  bool bAppendFinalSlash,
						  sal_Int32 nIndex,
						  bool bIgnoreFinalSlash,
						  EncodeMechanism eMechanism,
						  rtl_TextEncoding eCharset)
{
	return insertName(rTheName, false, bAppendFinalSlash, nIndex,
		  bIgnoreFinalSlash, eMechanism, eCharset);
}

inline bool INetURLObject::SetParam(ByteString const & rTheQuery,
						EncodeMechanism eMechanism,
						rtl_TextEncoding eCharset)
{
	return rTheQuery.Len() == 0 ?
	     clearQuery() :
	     setQuery(extend(rTheQuery), true, eMechanism, eCharset);
}

inline bool INetURLObject::SetParam(UniString const & rTheQuery,
						EncodeMechanism eMechanism,
						rtl_TextEncoding eCharset)
{
	return rTheQuery.Len() == 0 ?
	     clearQuery() :
	     setQuery(rTheQuery, false, eMechanism, eCharset);
}

inline bool INetURLObject::SetMark(ByteString const & rTheFragment,
					   EncodeMechanism eMechanism,
					   rtl_TextEncoding eCharset)
{
	return rTheFragment.Len() == 0 ?
	     clearFragment() :
	     setFragment(extend(rTheFragment), true, eMechanism, eCharset);
}

inline bool INetURLObject::SetMark(UniString const & rTheFragment,
					   EncodeMechanism eMechanism,
					   rtl_TextEncoding eCharset)
{
	return rTheFragment.Len() == 0 ?
	     clearFragment() :
	     setFragment(rTheFragment, false, eMechanism, eCharset);
}

inline INetURLObject::INetURLObject(UniString const & rFSysPath,
						FSysStyle eStyle):
	m_eScheme(INET_PROT_NOT_VALID), m_eSmartScheme(INET_PROT_HTTP)
{
	setFSysPath(rFSysPath, eStyle);
}

inline UniString INetURLObject::encode(ByteString const & rText, Part ePart,
						   sal_Char cEscapePrefix,
						   EncodeMechanism eMechanism,
						   rtl_TextEncoding eCharset)
{
	return encodeText(extend(rText), true, ePart, cEscapePrefix, eMechanism,
		  eCharset, false);
}

inline UniString INetURLObject::encode(UniString const & rText, Part ePart,
						   sal_Char cEscapePrefix,
						   EncodeMechanism eMechanism,
						   rtl_TextEncoding eCharset)
{
	return encodeText(rText, false, ePart, cEscapePrefix, eMechanism,
		  eCharset, false);
}

inline UniString INetURLObject::decode(UniString const & rText,
						   sal_Char cEscapePrefix,
						   DecodeMechanism eMechanism,
						   rtl_TextEncoding eCharset)
{
	return decode(rText.GetBuffer(), rText.GetBuffer() + rText.Len(),
	  cEscapePrefix, eMechanism, eCharset);
}

#endif 
