

#ifndef _WINSHELL_HXX
#define _WINSHELL_HXX

#include <tools/prewin.h>
#include <tools/shutil.h>
#include <tools/postwin.h>

#include <tools/string.hxx>

class CItemIDList
{
public:
	CItemIDList();
	CItemIDList( const CItemIDList & );
	CItemIDList( LPCITEMIDLIST );
	CItemIDList( const String & );
	CItemIDList( int nFolder );
	~CItemIDList();

	CItemIDList & operator = ( const CItemIDList & );
	CItemIDList & operator += ( const CItemIDList & );
	CItemIDList operator + ( const CItemIDList & ) const;

	int operator == ( const CItemIDList & ) const;
	int operator != ( const CItemIDList & rIDList ) const
		{ return ! operator == ( rIDList ); };

	operator LPCITEMIDLIST() const
		{ return m_pidl; };

	int GetTokenCount() const;
	CItemIDList GetToken( int nIndex ) const;
	CItemIDList GetParent( int nLevelFromBottom = 1) const;
	CItemIDList GetChild( int nLevelFromBottom = 1 ) const;
	void Split( CItemIDList & rParent, CItemIDList & rChild, int nLevelFromBottom = 1 ) const;

	CItemIDList operator []( int nIndex ) const
		{ return GetToken( nIndex ); };

	String GetFilePath() const;

	BOOL IsValid() const
		{ return m_pidl != NULL && m_pidl->mkid.cb != sizeof(USHORT); }

	int GetRootID() const
		{ return m_nFolder; }

protected:
	ITEMIDLIST	*m_pidl;
	int			m_nFolder;
};

inline CItemIDList CItemIDList::operator + ( const CItemIDList & rIDList ) const
{
	CItemIDList	aCopy( *this );
	aCopy += rIDList;
	return aCopy;
}

enum NotificationEvent
{
	NotificationEvent_Error,
	NotificationEvent_Signaled,
	NotificationEvent_Canceled
};

typedef struct _WIN32_VOLUME_DATA
{
	TCHAR	cDeviceName[MAX_PATH];
	TCHAR	cVolumeName[MAX_PATH];
	TCHAR	cFileSystemName[MAX_PATH];
	DWORD	dwSerialNumber;
	DWORD	dwFileSystemFlags;
	DWORD	nMaxComponentLength;
} WIN32_VOLUME_DATA;

static const GUID CLSID_IE4 = 
{ 0x871C5380, 0x42A0, 0x1069, { 0xA2, 0xEA, 0x08, 0x00, 0x2B, 0x30, 0x30, 0x9D } };

static const GUID CLSID_Network = 
{ 0x208D2C60, 0x3AEA, 0x1069, { 0xA2, 0xD7, 0x08, 0x00, 0x2B, 0x30, 0x30, 0x9D } };

static const GUID CLSID_RecycleBin = 
{ 0x645FF040, 0x5081, 0x101B, { 0x9F, 0x08, 0x00, 0xAA, 0x00, 0x2F, 0x95, 0x4E } };

static const GUID CLSID_MyComputer = 
{ 0x20D04FE0, 0x3AEA, 0x1069, { 0xA2, 0xD8, 0x08, 0x00, 0x2B, 0x30, 0x30, 0x9D } };

static const GUID CLSID_Tasks = 
{ 0xD6277990, 0x4C6A, 0x11CF, { 0x8D, 0x87, 0x00, 0xAA, 0x00, 0x60, 0xF5, 0xBF } };

#define SHGDN_INCLUDE_NONFILESYS	0x2000

#define CSIDL_UNKNOWN	-1
#define CSIDL_ROOT		-2
#define CSIDL_SYSTEM	-3

#define SHCONTF_ALL	(SHCONTF_FOLDERS | SHCONTF_NONFOLDERS | SHCONTF_INCLUDEHIDDEN)

#define SHGII_CONTAINER_MASK		0x70

#define SHGII_COMPUTER				0x20

#define SHGII_COMPUTER_REMOVABLE	0x22
#define SHGII_COMPUTER_FIXED		0x23
#define SHGII_COMPUTER_REMOTE		0x24
#define SHGII_COMPUTER_CDROM		0x25
#define SHGII_COMPUTER_RAMDISK		0x26
#define SHGII_COMPUTER_FLOPPY525	0x28
#define SHGII_COMPUTER_FLOPPY35		0x29
#define SHGII_COMPUTER_NETWORK		0x2A
#define SHGII_COMPUTER_REGITEM		0x2E

#define SHGII_ROOT					0x10
#define SHGII_ROOT_REGITEM			0x1F

#define SHGII_NETWORK				0x40
#define SHGII_NETWORK_TREE			0x47
#define SHGII_NETWORK_SERVER		0x42
#define SHGII_NETWORK_DIRECTORY		0x43
#define SHGII_NETWORK_PRINTER		0x41

#define SHGII_FILESYSTEM			0x30
#define SHGII_FILESYSTEM_FILE		0x31
#define SHGII_FILESYSTEM_DIRECTORY	0x32

#define SHGII_ANCESTOR				0x80

#define SHITEMCONTAINER( pidl )		((pidl)->mkid.abID[0] & SHGII_CONTAINER_MASK)
#define SHITEMKIND( pidl )			((pidl)->mkid.abID[0] & 0x7F)

class CShellFolderData
{
protected:
	CShellFolderData();
	virtual ~CShellFolderData();

	IShellFolder	*m_pShellFolder;
	IEnumIDList		*m_pEnumIDList;
	IShellIcon		*m_pShellIcon;
	BOOL			m_bIsOpen;
	HANDLE			m_hCancelEvent;
	DWORD			m_dwContentFlags;
};

class CShellFolder : public CShellFolderData
{
public:
	CShellFolder( const CItemIDList & );
	CShellFolder( const CShellFolder & );
	CShellFolder( IShellFolder * );

	virtual BOOL Reset();

	BOOL GetNextValidID( CItemIDList & );

	virtual BOOL GetNextID( CItemIDList & );
	virtual BOOL ValidateID( const CItemIDList & );

	virtual BOOL GetAttributesOf( const CItemIDList &, LPDWORD pdwInOut );
	virtual BOOL GetNameOf( const CItemIDList &, String & );

	virtual BOOL GetFileInfo( const CItemIDList &, WIN32_FIND_DATA * );
	virtual BOOL GetVolumeInfo( const CItemIDList & rIDList, WIN32_VOLUME_DATA * );

	virtual BOOL SetNameOf( const CItemIDList &, const String &, CItemIDList & );
	virtual BOOL DeleteItem( const CItemIDList & );

	virtual int CompareIDs( const CItemIDList &, const CItemIDList & );

	virtual IContextMenu *GetContextMenu( int nItems, const CItemIDList * );
	virtual String GetIconLocation( const CItemIDList & );

	virtual NotificationEvent WaitForChanges();
	virtual void CancelWaitNotifications();

protected:
	CShellFolder() : CShellFolderData() {};

	void Initialize( IShellFolder *pShellFolder );
	void Initialize( LPCITEMIDLIST );
};

class CFileSystemFolder : public CShellFolder
{
public:
	CFileSystemFolder( LPCSTR pszPath );

	virtual NotificationEvent WaitForChanges();

protected:
	CFileSystemFolder() : CShellFolder() {};

	void Initialize( LPCTSTR pszPath );

	TCHAR	m_szPath[MAX_PATH];
};

class CSpecialFolder : public CShellFolder
{
public:
	CSpecialFolder( int nFolder );

protected:
	CSpecialFolder() : CShellFolder() {};

	void Initialize( int nFolder );

	int	m_nFolder;
};

class CMyComputerFolder : public CSpecialFolder
{
public:
	CMyComputerFolder() : CSpecialFolder( CSIDL_DRIVES ) {};
};

class CVolumesFolder : public CMyComputerFolder
{
public:
	CVolumesFolder() : CMyComputerFolder() {};

	virtual BOOL ValidateID( const CItemIDList & );
	virtual NotificationEvent WaitForChanges();
};

class CWorkplaceFolder : public CMyComputerFolder
{
	CWorkplaceFolder() : CMyComputerFolder() {};

	virtual BOOL ValidateID( const CItemIDList & );
	virtual NotificationEvent WaitForChanges();
};

class CDesktopFolder : public CSpecialFolder
{
public:
	CDesktopFolder() : CSpecialFolder( CSIDL_DESKTOP ) {};
};

class CDesktopAncestorsFolder : public CDesktopFolder
{
public:
	CDesktopAncestorsFolder() : CDesktopFolder() {};

	virtual BOOL ValidateID( const CItemIDList & );
	virtual NotificationEvent WaitForChanges();
};

class CDesktopContentsFolder : public CDesktopFolder
{
public:
	CDesktopContentsFolder() : CDesktopFolder() {};

	virtual BOOL ValidateID( const CItemIDList & );
	virtual NotificationEvent WaitForChanges();
};

#endif 
