

#ifndef _ZCODEC_HXX
#define _ZCODEC_HXX

#ifndef _SOLAR_H
#include <tools/solar.h>
#endif

#define DEFAULT_IN_BUFSIZE			(0x00008000UL)
#define DEFAULT_OUT_BUFSIZE			(0x00008000UL)

#define MAX_MEM_USAGE 8

#define ZCODEC_NO_COMPRESSION		(0x00000000UL)
#define ZCODEC_BEST_SPEED			(0x00000001UL)
#define	ZCODEC_DEFAULT_COMPRESSION	(0x00000006UL)
#define ZCODEC_BEST_COMPRESSION		(0x00000009UL)

#define ZCODEC_DEFAULT_STRATEGY		(0x00000000UL)
#define ZCODEC_ZFILTERED			(0x00000100UL)
#define ZCODEC_ZHUFFMAN_ONLY		(0x00000200UL)

#define ZCODEC_UPDATE_CRC			(0x00010000UL)
#define ZCODEC_GZ_LIB				(0x00020000UL)

#define ZCODEC_PNG_DEFAULT ( ZCODEC_NO_COMPRESSION | ZCODEC_DEFAULT_STRATEGY | ZCODEC_UPDATE_CRC )
#define ZCODEC_DEFAULT	( ZCODEC_DEFAULT_COMPRESSION | ZCODEC_DEFAULT_STRATEGY )

class SvStream;

class ZCodec
{
private:

	ULONG			mbInit;
	BOOL			mbStatus;
	BOOL			mbFinish;
	ULONG			mnMemUsage;
	SvStream*		mpIStm;
	BYTE*			mpInBuf;
	ULONG			mnInBufSize;
	ULONG			mnInToRead;
	SvStream*		mpOStm;
	BYTE*			mpOutBuf;
	ULONG			mnOutBufSize;

	ULONG			mnCRC;
	ULONG			mnCompressMethod;
	void*			mpsC_Stream;

	void			ImplInitBuf( BOOL nIOFlag );
	void			ImplWriteBack( void );

public:	
		ZCodec( ULONG nInBuf, ULONG nOutBuf, ULONG nMemUsage = MAX_MEM_USAGE );
		ZCodec( void );	
		~ZCodec();

	virtual void	BeginCompression( ULONG nCompressMethod = ZCODEC_DEFAULT );
	virtual long	EndCompression();
	BOOL            IsFinished () const { return mbFinish; }

	long			Compress( SvStream& rIStm, SvStream& rOStm );
	long			Decompress( SvStream& rIStm, SvStream& rOStm );

	long			Write( SvStream& rOStm, const BYTE* pData, ULONG nSize );
	long			Read( SvStream& rIStm, BYTE* pData, ULONG nSize );
	long			ReadAsynchron( SvStream& rIStm, BYTE* pData, ULONG nSize );

	void			SetBreak( ULONG );
	ULONG			GetBreak( void );
	void			SetCRC( ULONG nCurrentCRC );
	ULONG			UpdateCRC( ULONG nLatestCRC, ULONG nSource );	
	ULONG			UpdateCRC( ULONG nLatestCRC, BYTE* pSource, long nDatSize );
	ULONG			GetCRC();
};

class GZCodec : public ZCodec
{

public:
		GZCodec(){};
		~GZCodec(){};
	virtual void	BeginCompression( ULONG nCompressMethod = ZCODEC_DEFAULT );
};

#endif 
