

#ifndef _SV_GFXLINK_HXX
#define _SV_GFXLINK_HXX

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#if 0 // _SOLAR__PRIVATE

#ifndef _URLOBJ_HXX
#include <tools/urlobj.hxx>
#endif

struct ImpBuffer
{
	ULONG		mnRefCount;
	BYTE*		mpBuffer;
		ImpBuffer( ULONG nSize )
	{
		mnRefCount = 1UL;
		mpBuffer = nSize ? new BYTE[ nSize ] : NULL;
	}
		ImpBuffer( BYTE* pBuf ) { mnRefCount = 1UL; mpBuffer = pBuf; }
		~ImpBuffer() { delete[] mpBuffer; }
};

struct ImpSwap
{
	INetURLObject   maURL;
	ULONG			mnDataSize;
	ULONG			mnRefCount;
			ImpSwap( BYTE* pData, ULONG nDataSize );
		~ImpSwap();

	BYTE*			GetData() const;

	BOOL			IsSwapped() const { return maURL.GetMainURL( INetURLObject::NO_DECODE ).Len() > 0; }

	void			WriteTo( SvStream& rOStm ) const;
};

#endif 

enum GfxLinkType
{
	GFX_LINK_TYPE_NONE			= 0,
	GFX_LINK_TYPE_EPS_BUFFER	= 1,
	GFX_LINK_TYPE_NATIVE_GIF	= 2,	
	GFX_LINK_TYPE_NATIVE_JPG	= 3,	
	GFX_LINK_TYPE_NATIVE_PNG	= 4,	
	GFX_LINK_TYPE_NATIVE_TIF	= 5,	
	GFX_LINK_TYPE_NATIVE_WMF	= 6,	
	GFX_LINK_TYPE_NATIVE_MET	= 7,	
	GFX_LINK_TYPE_NATIVE_PCT	= 8,	
	GFX_LINK_TYPE_USER			= 0xffff
};

#define GFX_LINK_FIRST_NATIVE_ID	GFX_LINK_TYPE_NATIVE_GIF
#define GFX_LINK_LAST_NATIVE_ID		GFX_LINK_TYPE_NATIVE_PCT

struct ImpBuffer;
struct ImpSwap;
class Graphic;

class GfxLink
{
private:

	GfxLinkType			meType;
	ImpBuffer*			mpBuf;
	ImpSwap*			mpSwap;
	ULONG				mnBufSize;
	ULONG				mnUserId;
	ULONG				mnExtra1;
	ULONG				mnExtra2;

	void				ImplCopy( const GfxLink& rGfxLink );

public:					
			GfxLink();
			GfxLink( const GfxLink& );
			GfxLink( const String& rPath, GfxLinkType nType );
			GfxLink( BYTE* pBuf, ULONG nBufSize, GfxLinkType nType, BOOL bOwns );
			~GfxLink();

	GfxLink&			operator=( const GfxLink& );

	GfxLinkType			GetType() const;

	void				SetUserId( ULONG nUserId ) { mnUserId = nUserId; }
	ULONG				GetUserId() const { return mnUserId; }

	ULONG				GetDataSize() const;	
	void				SetData( BYTE* pBuf, ULONG nSize, GfxLinkType nType, BOOL bOwns );
	const BYTE*			GetData() const;

	BOOL				IsNative() const;
	BOOL				IsUser() const { return( GFX_LINK_TYPE_USER == meType ); }

	BOOL				LoadNative( Graphic& rGraphic );

	void				SwapOut();
	void				SwapIn();
	BOOL				IsSwappedOut() const { return( mpSwap != NULL ); }

public:

	friend SvStream&	operator<<( SvStream& rOStream, const GfxLink& rGfxLink );
	friend SvStream&	operator>>( SvStream& rIStream, GfxLink& rGfxLink );
};

#endif
