

#ifndef _SV_HELP_HXX
#define _SV_HELP_HXX

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif

class Point;
class Rectangle;
class Window;

#define QUICKHELP_LEFT		((USHORT)0x0001)
#define QUICKHELP_CENTER	((USHORT)0x0002)
#define QUICKHELP_RIGHT 	((USHORT)0x0004)
#define QUICKHELP_TOP		((USHORT)0x0008)
#define QUICKHELP_VCENTER	((USHORT)0x0010)
#define QUICKHELP_BOTTOM	((USHORT)0x0020)
#define QUICKHELP_NOAUTOPOS (QUICKHELP_LEFT | QUICKHELP_CENTER | QUICKHELP_RIGHT | QUICKHELP_TOP | QUICKHELP_VCENTER | QUICKHELP_BOTTOM)
#define QUICKHELP_CTRLTEXT	((USHORT)0x0040)

#define HELP_INDEX			((ULONG)0xFFFFFFFF)
#define HELP_HELPONHELP 	((ULONG)0xFFFFFFFE)

class Help
{
private:
	String				maHelpFile;

public:
			Help();

	void				SetHelpFile( const String& rFileName ) { maHelpFile = rFileName; }
	const String&		GetHelpFile() const { return maHelpFile; }

	virtual BOOL		Start( ULONG nHelpId, const Window* pWindow  );
	virtual BOOL		Start( const XubString& rKeyWord, const Window* pWindow );
	virtual XubString	GetHelpText( ULONG nHelpId, const Window* pWindow );

	static void 		EnableContextHelp();
	static void 		DisableContextHelp();
	static BOOL 		IsContextHelpEnabled();
	static BOOL 		StartContextHelp();

	static void 		EnableExtHelp();
	static void 		DisableExtHelp();
	static BOOL 		IsExtHelpEnabled();
	static BOOL 		StartExtHelp();
	static BOOL 		EndExtHelp();
	static BOOL 		IsExtHelpActive();

	static void 		EnableBalloonHelp();
	static void 		DisableBalloonHelp();
	static BOOL 		IsBalloonHelpEnabled();
	static BOOL 		ShowBalloon( Window* pParent,
						 const Point& rScreenPos,
						 const XubString& rHelpText );
	static BOOL 		ShowBalloon( Window* pParent,
						 const Point& rScreenPos,
						 const Rectangle&,
						 const XubString& rHelpText );

	static void 		EnableQuickHelp();
	static void 		DisableQuickHelp();
	static BOOL 		IsQuickHelpEnabled();
	static BOOL 		ShowQuickHelp( Window* pParent,
						   const Rectangle& rScreenRect,
						   const XubString& rHelpText,
						   const XubString& rLongHelpText,
						   USHORT nStyle = 0 );
	static BOOL 		ShowQuickHelp( Window* pParent,
						   const Rectangle& rScreenRect,
						   const XubString& rHelpText,
						   USHORT nStyle = 0 )
				{ return Help::ShowQuickHelp( pParent, rScreenRect, rHelpText, XubString(), nStyle ); }

	static ULONG		ShowTip( Window* pParent,
					 const Rectangle& rScreenRect,
					 const XubString& rText, USHORT nStyle = 0 );
	static void 		HideTip( ULONG nId );
};

#endif	
