

#ifndef _SV_IMAGE_HXX
#define _SV_IMAGE_HXX

#ifndef _GEN_HXX
#include <tools/gen.hxx>
#endif

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_RESID_HXX
#include <vcl/resid.hxx>
#endif

class ImplImage;
class ImplImageList;
class Bitmap;
class BitmapEx;
class Color;

#ifdef MAC
#ifdef IMAGE_STDBTN_COLOR
#undef IMAGE_STDBTN_COLOR
#endif
#endif
#define IMAGE_STDBTN_COLOR		Color( 0xC0, 0xC0, 0xC0 )

class Image
{
	friend class ImageList;

private:
	ImplImage*		mpImplData;

public:
		Image();
		Image( const ResId& rResId );
		Image( const Image& rImage );
		Image( const Bitmap& rBitmap );
		Image( const Bitmap& rBitmap, const Bitmap& rMaskBitmap );
		Image( const Bitmap& rBitmap, const Color& rColor );
		Image( const BitmapEx& rBitmapEx );
		~Image();

	Size			GetSizePixel() const;

	BOOL			operator!() const;

	Image&			operator=( const Image& rImage );
	BOOL			operator==( const Image& rImage ) const;
	BOOL			operator!=( const Image& rImage ) const
			{ return !(Image::operator==( rImage )); }
};

inline BOOL Image::operator!() const
{
	if ( !mpImplData )
		return TRUE;
	else
		return FALSE;
}

#define IMAGELIST_IMAGE_NOTFOUND	((USHORT)0xFFFF)

class ImageList
{
private:
	ImplImageList*	mpImplData;
	USHORT			mnInitSize;
	USHORT			mnGrowSize;

public:
		ImageList( USHORT nInit = 8, USHORT nGrow = 4 );
		ImageList( const ResId& rResId );
		ImageList( const ImageList& rImageList );
		ImageList( const Bitmap& rBitmap,
				   USHORT nInitSize, USHORT* pIdAry = NULL,
				   USHORT nGrow = 4 );
		ImageList( const Bitmap& rBitmap, const Bitmap& rMaskBmp,
				   USHORT nInitSize, USHORT* pIdAry = NULL,
				   USHORT nGrow = 4 );
		ImageList( const Bitmap& rBitmap, const Color& rColor,
				   USHORT nInitSize, USHORT* pIdAry = NULL,
				   USHORT nGrow = 4 );
		~ImageList();

	void			AddImage( USHORT nNewId, const Image& rImage );
	void			CopyImage( USHORT nNewId, USHORT nCopyId );
	void			ReplaceImage( USHORT nId, const Image& rImage );
	void			ReplaceImage( USHORT nId, USHORT nReplaceId );
	void			MergeImage( USHORT nId, USHORT nMergeId );
	void			RemoveImage( USHORT nId );
	Image			GetImage( USHORT nId ) const;
	void			Clear();

	USHORT			GetImageCount() const;
	USHORT			GetImagePos( USHORT nId ) const;
	USHORT			GetImageId( USHORT nPos ) const;
	Size			GetImageSize() const;

	Bitmap			GetBitmap() const;
	Bitmap			GetMaskBitmap() const;
	BOOL			HasMaskBitmap() const;
	Color			GetMaskColor() const;
	BOOL			HasMaskColor() const;

	ImageList&		operator=( const ImageList& rImageList );
	BOOL			operator==( const ImageList& rImageList ) const;
	BOOL			operator!=( const ImageList& rImageList ) const
			{ return !(ImageList::operator==( rImageList )); }

	friend SvStream& operator>>( SvStream& rIStream, ImageList& rImageList );
	friend SvStream& operator<<( SvStream& rOStream, const ImageList& rImageList );
};

#endif	
