

#ifndef _SV_KEYCODE_HXX
#define _SV_KEYCODE_HXX

#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_RESID_HXX
#include <vcl/resid.hxx>
#endif
#ifndef _SV_KEYCODES_HXX
#include <vcl/keycodes.hxx>
#endif

#include <vcl/vclenum.hxx>

class Window;

class KeyCode
{
private:
	USHORT		nCode;
	KeyFuncType eFunc;

public:
	KeyCode() { nCode = 0; eFunc = KEYFUNC_DONTKNOW; }
	KeyCode( const ResId& rResId );
	KeyCode( USHORT nKey, USHORT nModifier = 0 )
		{ nCode = nKey | nModifier; eFunc = KEYFUNC_DONTKNOW; }
	KeyCode( USHORT nKey, BOOL bShift, BOOL bMod1, BOOL bMod2 );
	KeyCode( KeyFuncType eFunction );

	USHORT		GetFullCode() const { return nCode; }
	USHORT		GetFullKeyCode() const { return (nCode & ~KEY_CONTROLMOD); }
	KeyFuncType GetFullFunction() const { return eFunc; }
	BOOL		IsDefinedKeyCodeEqual( const KeyCode& rKeyCode ) const;

	USHORT		GetCode() const
		{ return (nCode & KEY_CODE); }

	USHORT		GetModifier() const
		{ return (nCode & KEY_MODTYPE); }
	USHORT		GetAllModifier() const
		{ return (nCode & KEY_ALLMODTYPE); }
	BOOL		IsShift() const
		{ return ((nCode & KEY_SHIFT) != 0); }
	BOOL		IsMod1() const
		{ return ((nCode & KEY_MOD1) != 0); }
	BOOL		IsMod2() const
		{ return ((nCode & KEY_MOD2) != 0); }
	BOOL		IsControlMod() const
		{ return ((nCode & KEY_CONTROLMOD) != 0); }

	USHORT		GetGroup() const
		{ return (nCode & KEYGROUP_TYPE); }

	XubString	GetName( Window* pWindow = NULL ) const;
	XubString	GetSymbolName( const XubString& rFontName, Window* pWindow = NULL ) const;

	BOOL		IsFunction() const
		{ return ((eFunc != KEYFUNC_DONTKNOW) ? TRUE : FALSE); }

	KeyFuncType GetFunction() const;

	BOOL		operator ==( const KeyCode& rKeyCode ) const;
	BOOL		operator !=( const KeyCode& rKeyCode ) const;
};

inline KeyCode::KeyCode( USHORT nKey, BOOL bShift, BOOL bMod1, BOOL bMod2 )
{
	nCode = nKey;
	if( bShift )
		nCode |= KEY_SHIFT;
	if( bMod1 )
		nCode |= KEY_MOD1;
	if( bMod2 )
		nCode |= KEY_MOD2;
	eFunc = KEYFUNC_DONTKNOW;
}

inline BOOL KeyCode::operator ==( const KeyCode& rKeyCode ) const
{
	if ( (eFunc == KEYFUNC_DONTKNOW) && (rKeyCode.eFunc == KEYFUNC_DONTKNOW) )
		return (nCode == rKeyCode.nCode);
	else
		return (GetFunction() == rKeyCode.GetFunction());
}

inline BOOL KeyCode::operator !=( const KeyCode& rKeyCode ) const
{
	if ( (eFunc == KEYFUNC_DONTKNOW) && (rKeyCode.eFunc == KEYFUNC_DONTKNOW) )
		return (nCode != rKeyCode.nCode);
	else
		return (GetFunction() != rKeyCode.GetFunction());
}

inline BOOL KeyCode::IsDefinedKeyCodeEqual( const KeyCode& rKeyCode ) const
{
	if ( (eFunc == KEYFUNC_DONTKNOW) && (rKeyCode.eFunc == KEYFUNC_DONTKNOW) )
		return (GetFullKeyCode() == rKeyCode.GetFullKeyCode());
	else
		return (GetFunction() == rKeyCode.GetFunction());
}

#endif 
