

#ifndef _SV_OCTREE_HXX
#define _SV_OCTREE_HXX

#ifndef _SV_SALBTYPE_HXX
#include <vcl/salbtype.hxx>
#endif

#define OCTREE_BITS		5
#define OCTREE_BITS_1	10

typedef struct OctreeNode
{
	ULONG		nCount;
	ULONG		nRed;
	ULONG		nGreen;
	ULONG		nBlue;
	OctreeNode* pChild[ 8 ];
	OctreeNode*	pNext;
	OctreeNode*	pNextInCache;
	USHORT		nPalIndex;
	BOOL		bLeaf;
} NODE;

typedef NODE*		PNODE;
typedef PNODE*		PPNODE;

class ImpNodeCache;
class BitmapReadAccess;

class Octree
{
private:

	BitmapPalette				aPal;
	ULONG						nMax;
	ULONG						nLeafCount;
	ULONG						nLevel;
	PNODE						pTree;
	PNODE						pReduce[ OCTREE_BITS + 1 ];
	BitmapColor*				pColor;
	ImpNodeCache*				pNodeCache;
	const BitmapReadAccess*		pAcc;
	USHORT						nPalIndex;
						Octree() {};

	void						CreatePalette( PNODE pNode );
	void						GetPalIndex( PNODE pNode );

	void						ImplCreateOctree();
	void						ImplDeleteOctree( PPNODE ppNode );
	void						ImplAdd( PPNODE ppNode );
	void						ImplReduce();

public:							
						Octree( const BitmapReadAccess& rReadAcc, ULONG nColors );
					Octree( ULONG nColors );
					~Octree();

	void						AddColor( const BitmapColor& rColor );

	inline const BitmapPalette&	GetPalette();
	inline USHORT				GetBestPaletteIndex( const BitmapColor& rColor );
};

inline const BitmapPalette& Octree::GetPalette()
{
	aPal.SetEntryCount( (USHORT) nLeafCount );
	nPalIndex = 0;
	CreatePalette( pTree );
	return aPal;
}

inline USHORT Octree::GetBestPaletteIndex( const BitmapColor& rColor )
{
	pColor = &(BitmapColor&) rColor;
	nPalIndex = 65535;
	nLevel = 0L;
	GetPalIndex( pTree );
	return nPalIndex;
}

class InverseColorMap
{
private:

	BYTE*				pBuffer;
	BYTE*				pMap;
	const ULONG			nBits;

#if 0 // _SOLAR__PRIVATE 

	void				ImplCreateBuffers( const ULONG nMax );

#endif 

public:
				InverseColorMap( const BitmapPalette& rPal );
			~InverseColorMap();

	inline USHORT		GetBestPaletteIndex( const BitmapColor& rColor );
};

inline USHORT InverseColorMap::GetBestPaletteIndex( const BitmapColor& rColor )
{
	return pMap[ ( ( (ULONG) rColor.GetRed() >> nBits ) << OCTREE_BITS_1 ) | 
	 ( ( (ULONG) rColor.GetGreen() >> nBits ) << OCTREE_BITS ) | 
	 ( (ULONG) rColor.GetBlue() >> nBits ) ];
}

#endif 
