

#ifndef _SV_REGION_HXX
#define _SV_REGION_HXX

#ifndef _GEN_HXX
#include <tools/gen.hxx>
#endif

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif

class ImplRegion;
class ImplRegionBand;
class Polygon;
class PolyPolygon;
struct ImplRegionInfo;

enum RegionType { REGION_NULL, REGION_EMPTY, REGION_RECTANGLE, REGION_COMPLEX };
enum RegionOverlapType { REGION_INSIDE, REGION_OVER, REGION_OUTSIDE };

typedef long RegionHandle;

class Region
{
private:
	ImplRegion* 	mpImplRegion;

#if 0 // _SOLAR__PRIVATE
#include <region.imp>
#endif

public:
		Region();
		Region( RegionType eType );
		Region( const Rectangle& rRect );
		Region( const Polygon& rPolygon );
		Region( const PolyPolygon& rPolyPoly );
		Region( const Region& rRegion );
		~Region();

	void			Move( long nHorzMove, long nVertMove );
	void			Scale( double fScaleX, double fScaleY );
	BOOL			Union( const Rectangle& rRegion );
	BOOL			Intersect( const Rectangle& rRegion );
	BOOL			Exclude( const Rectangle& rRegion );
	BOOL			XOr( const Rectangle& rRegion );
	BOOL			Union( const Region& rRegion );
	BOOL			Intersect( const Region& rRegion );
	BOOL			Exclude( const Region& rRegion );
	BOOL			XOr( const Region& rRegion );

	RegionType		GetType() const;
	BOOL			IsEmpty() const { return GetType() == REGION_EMPTY; };
	BOOL			IsNull() const { return GetType() == REGION_NULL; };

	void			SetEmpty();
	void			SetNull();

	Rectangle		GetBoundRect() const;
	 BOOL            HasPolyPolygon() const;
	PolyPolygon     GetPolyPolygon() const;

	ULONG			GetRectCount() const;
	RegionHandle	BeginEnumRects();
	BOOL			GetEnumRects( RegionHandle hRegionHandle, Rectangle& rRect );
	BOOL			GetNextEnumRect( RegionHandle hRegionHandle, Rectangle& rRect )
			{ return GetEnumRects( hRegionHandle, rRect ); }
	void			EndEnumRects( RegionHandle hRegionHandle );

	BOOL			IsInside( const Point& rPoint ) const;
	BOOL			IsInside( const Rectangle& rRect ) const;
	BOOL			IsOver( const Rectangle& rRect ) const;

	Region& 		operator=( const Region& rRegion );
	Region& 		operator=( const Rectangle& rRect );

	BOOL			operator==( const Region& rRegion ) const;
	BOOL			operator!=( const Region& rRegion ) const
			{ return !(Region::operator==( rRegion )); }

	friend SvStream& operator>>( SvStream& rIStm, Region& rRegion );
	friend SvStream& operator<<( SvStream& rOStm, const Region& rRegion );

	RegionOverlapType  GetOverlapType( const Rectangle& rRect ) const;
};

#endif	
