

#ifndef _SV_SCRBAR_HXX
#define _SV_SCRBAR_HXX

#ifndef _SV_SV_H
#include <vcl/sv.h>
#endif
#ifndef _SV_CTRL_HXX
#include <vcl/ctrl.hxx>
#endif

enum ScrollType { SCROLL_DONTKNOW, SCROLL_LINEUP, SCROLL_LINEDOWN,
	              SCROLL_PAGEUP, SCROLL_PAGEDOWN, SCROLL_DRAG };

class ScrollBar : public Control
{
private:
	Rectangle       maBtn1Rect;
	Rectangle       maBtn2Rect;
	Rectangle       maPage1Rect;
	Rectangle       maPage2Rect;
	Rectangle       maThumbRect;
	void*           mpDummy;
	long            mnStartPos;
	long            mnMouseOff;
	long            mnThumbPixRange;
	long            mnThumbPixPos;
	long            mnThumbPixSize;
	long            mnMinRange;
	long            mnMaxRange;
	long            mnThumbPos;
	long            mnVisibleSize;
	long            mnLineSize;
	long            mnPageSize;
	long            mnDelta;
	USHORT          mnDragDraw;
	USHORT          mnStateFlags;
	ScrollType      meScrollType;
	ScrollType      meDDScrollType;
	BOOL            mbCalcSize;
	BOOL            mbFullDrag;
	BOOL            mbDummy1;
	BOOL            mbDummy2;
	Link            maScrollHdl;
	Link            maEndScrollHdl;

#ifdef _SV_SCRBAR_CXX
#include <scrbar.imp>
#endif

public:
	                ScrollBar( Window* pParent, WinBits nStyle = WB_VERT );
	                ScrollBar( Window* pParent, const ResId& rResId );
	                ~ScrollBar();
	 virtual void    MouseButtonDown( const MouseEvent& rMEvt );
	virtual void    Tracking( const TrackingEvent& rTEvt );
	virtual void    KeyInput( const KeyEvent& rKEvt );
	virtual void    Paint( const Rectangle& rRect );
	virtual void    Resize();
	virtual void    StateChanged( StateChangedType nType );
	virtual void    DataChanged( const DataChangedEvent& rDCEvt );
	 virtual void    Scroll();
	virtual void    EndScroll();
	 long            DoScroll( long nNewPos );
	long            DoScrollAction( ScrollType eScrollType );
	 void            EnableDrag( BOOL bEnable = TRUE )
	                    { mbFullDrag = bEnable; }
	BOOL            IsDragEnabled() const { return mbFullDrag; }
	 void            SetRangeMin( long nNewRange );
	long            GetRangeMin() const { return mnMinRange; }
	void            SetRangeMax( long nNewRange );
	long            GetRangeMax() const { return mnMaxRange; }
	void            SetRange( const Range& rRange );
	Range           GetRange() const { return Range( GetRangeMin(), GetRangeMax() ); }
	void            SetThumbPos( long nThumbPos );
	long            GetThumbPos() const { return mnThumbPos; }
	void            SetLineSize( long nNewSize ) { mnLineSize = nNewSize; }
	long            GetLineSize() const { return mnLineSize; }
	void            SetPageSize( long nNewSize ) { mnPageSize = nNewSize; }
	long            GetPageSize() const { return mnPageSize; }
	void            SetVisibleSize( long nNewSize );
	long            GetVisibleSize() const { return mnVisibleSize; }
	 long            GetDelta() const { return mnDelta; }
	ScrollType      GetType() const { return meScrollType; }
	 void            SetScrollHdl( const Link& rLink ) { maScrollHdl = rLink; }
	const Link&     GetScrollHdl() const { return maScrollHdl;    }
	void            SetEndScrollHdl( const Link& rLink ) { maEndScrollHdl = rLink; }
	const Link&     GetEndScrollHdl() const { return maEndScrollHdl; }
};

class ScrollBarBox : public Window
{
#ifdef _SV_SCRBAR_CXX
private:
	void            ImplInit( Window* pParent, WinBits nStyle );
	void            ImplInitSettings();
#endif

public:
	                ScrollBarBox( Window* pParent, WinBits nStyle = 0 );
	                ScrollBarBox( Window* pParent, const ResId& rResId );
	 virtual void    StateChanged( StateChangedType nType );
	virtual void    DataChanged( const DataChangedEvent& rDCEvt );
};

#endif 
