

#ifndef _SV_SELENG_HXX
#define _SV_SELENG_HXX

#ifndef _SV_TIMER_HXX
#include <vcl/timer.hxx>
#endif

#ifndef _SV_EVENT_HXX
#include <vcl/event.hxx>
#endif

class Window;
class CommandEvent;

#define SELENG_DRAGDROP_TIMEOUT     400
#define SELENG_AUTOREPEAT_INTERVAL  50

enum SelectionMode { NO_SELECTION, SINGLE_SELECTION, RANGE_SELECTION, MULTIPLE_SELECTION };

class FunctionSet
{
public:
	virtual void    BeginDrag() = 0;
	 virtual void    CreateAnchor() = 0;  
	virtual void	DestroyAnchor() = 0;
	 virtual BOOL    SetCursorAtPoint( const Point& rPointPixel,
	                                  BOOL bDontSelectAtCursor = FALSE ) = 0;
	 virtual BOOL    IsSelectionAtPoint( const Point& rPointPixel ) = 0;
	virtual void    DeselectAtPoint( const Point& rPointPixel ) = 0;
	 virtual void    DeselectAll() = 0;
};

#define SELENG_DRG_ENAB     0x0001
#define SELENG_IN_SEL       0x0002
#define SELENG_IN_ADD       0x0004
#define SELENG_ADD_ALW      0x0008
#define SELENG_IN_DRG       0x0010
#define SELENG_HAS_ANCH     0x0020
#define SELENG_CMDEVT       0x0040
#define SELENG_WAIT_UPEVT   0x0080
#define SELENG_EXPANDONMOVE	0x0100

class SelectionEngine
{
private:
	FunctionSet*        pFunctionSet;
	Window*             pWin;
	Rectangle           aArea;
	Timer               aWTimer; 
	MouseEvent          aLastMove;
	SelectionMode       eSelMode;
	 USHORT              nMouseSensitivity;
	USHORT              nLockedMods;
	USHORT              nFlags;
#if 0 // _SOLAR__PRIVATE
	                    DECL_LINK( ImpWatchDog, Timer * );
#endif

public:
	                     SelectionEngine( Window* pWindow,
	                                     FunctionSet* pFunctions = NULL );
	                    ~SelectionEngine();
	 BOOL                SelMouseButtonDown( const MouseEvent& rMEvt );
	BOOL                SelMouseButtonUp( const MouseEvent& rMEvt );
	BOOL                SelMouseMove( const MouseEvent& rMEvt );
	 void                CursorPosChanging( BOOL bShift, BOOL bMod1 );
	 void                SetVisibleArea( const Rectangle rNewArea )
	                        { aArea = rNewArea; }
	const Rectangle&    GetVisibleArea() const { return aArea; }
	 void                SetAddMode( BOOL);
	BOOL                IsAddMode() const;
	 void                AddAlways( BOOL bOn );
	BOOL                IsAlwaysAdding() const;
	 void                EnableDrag( BOOL bOn );
	BOOL                IsDragEnabled() const;
	void                ActivateDragMode();
	BOOL                IsInDragMode() const;
	 void                SetSelectionMode( SelectionMode eMode );
	SelectionMode       GetSelectionMode() const { return eSelMode; }
	 void                SetFunctionSet( FunctionSet* pFuncs )
	                        { pFunctionSet = pFuncs; }
	const FunctionSet*  GetFunctionSet() const { return pFunctionSet; }
	 void                SetMouseSensitivity( USHORT nSensitivity )
	                        { nMouseSensitivity = nSensitivity; }
	USHORT              GetMouseSensitivity() const
	                        { return nMouseSensitivity; }
	 const Point&        GetMousePosPixel() const
	                        { return aLastMove.GetPosPixel(); }
	const MouseEvent&   GetMouseEvent() const { return aLastMove; }
	 void                SetWindow( Window*);
	Window*             GetWindow() const { return pWin; }
	 void                LockModifiers( USHORT nModifiers )
	                        { nLockedMods = nModifiers; }
	USHORT              GetLockedModifiers() const { return nLockedMods; }
	 BOOL                IsInSelection() const;
	void                Reset();
	 void                Command( const CommandEvent& rCEvt );
	 BOOL                HasAnchor() const;
	void                SetAnchor( BOOL bAnchor );

	void				ExpandSelectionOnMouseMove( BOOL bExpand = TRUE )
			{
				if( bExpand )
					nFlags |= SELENG_EXPANDONMOVE;
				else
					nFlags &= ~SELENG_EXPANDONMOVE;
			}
};

inline BOOL SelectionEngine::IsDragEnabled() const
{
	if ( nFlags & SELENG_DRG_ENAB )
	    return TRUE;
	else
	    return FALSE;
}

inline BOOL SelectionEngine::IsAddMode()  const
{
	if ( nFlags & (SELENG_IN_ADD | SELENG_ADD_ALW) )
	    return TRUE;
	else
	    return FALSE;
}

inline void SelectionEngine::SetAddMode( BOOL bNewMode )
{
	if ( bNewMode )
	    nFlags |= SELENG_IN_ADD;
	else
	    nFlags &= (~SELENG_IN_ADD);
}

inline void SelectionEngine::EnableDrag( BOOL bOn )
{
	if ( bOn )
	    nFlags |= SELENG_DRG_ENAB;
	else
	    nFlags &= (~SELENG_DRG_ENAB);
}

inline void SelectionEngine::AddAlways( BOOL bOn )
{
	if( bOn )
	    nFlags |= SELENG_ADD_ALW;
	else
	    nFlags &= (~SELENG_ADD_ALW);
}

inline BOOL SelectionEngine::IsAlwaysAdding() const
{
	if ( nFlags & SELENG_ADD_ALW )
	    return TRUE;
	else
	    return FALSE;
}

inline BOOL SelectionEngine::IsInDragMode() const
{
	if ( nFlags & SELENG_IN_DRG )
	    return TRUE;
	else
	    return FALSE;
}

inline BOOL SelectionEngine::IsInSelection() const
{
	if ( nFlags & SELENG_IN_SEL )
	    return TRUE;
	else
	    return FALSE;
}

inline BOOL SelectionEngine::HasAnchor() const
{
	if ( nFlags & SELENG_HAS_ANCH )
	    return TRUE;
	else
	    return FALSE;
}

inline void SelectionEngine::SetAnchor( BOOL bAnchor )
{
	if ( bAnchor )
	    nFlags |= SELENG_HAS_ANCH;
	else
	    nFlags &= (~SELENG_HAS_ANCH);
}

#endif	

