
#ifndef _XMLOFF_SHAPEEXPORT_HXX_
#define _XMLOFF_SHAPEEXPORT_HXX_

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif

#ifndef _RTL_USTRBUF_HXX_ 
#include <rtl/ustrbuf.hxx>
#endif

#ifndef _UNIVERSALL_REFERENCE_HXX
#include <xmloff/uniref.hxx>
#endif

#ifndef _COM_SUN_STAR_DRAWING_XSHAPE_HPP_
#include <com/sun/star/drawing/XShape.hpp>
#endif

#ifndef _COM_SUN_STAR_DRAWING_XSHAPES_HPP_
#include <com/sun/star/drawing/XShapes.hpp>
#endif

#ifndef _COM_SUN_STAR_AWT_POINT_HPP_
#include <com/sun/star/awt/Point.hpp>
#endif

#include <map>

#ifndef _XMLOFF_PROPERTYSETMAPPER_HXX
#include <xmloff/xmlprmap.hxx>
#endif

#ifndef _XMLOFF_XMLEXPPR_HXX
#include <xmloff/xmlexppr.hxx>
#endif

#ifndef _XMLOFF_ANIMEXP_HXX
#include <xmloff/animexp.hxx>
#endif

#ifndef _XMLOFF_FAMILIES_HXX_
#include <xmloff/families.hxx>
#endif

#define SEF_EXPORT_X		0x0001
#define SEF_EXPORT_Y		0x0002
#define SEF_EXPORT_POSITION	0x0003

#define SEF_EXPORT_WIDTH	0x0004
#define SEF_EXPORT_HEIGHT	0x0008
#define SEF_EXPORT_SIZE		0x000c

#define SEF_EXPORT_NO_CHART_DATA	0x0010

#define SEF_DEFAULT			SEF_EXPORT_POSITION|SEF_EXPORT_SIZE

enum XmlShapeType
{
	XmlShapeTypeUnknown,							

	XmlShapeTypeDrawRectangleShape,					
	XmlShapeTypeDrawEllipseShape,					
	XmlShapeTypeDrawControlShape,					
	XmlShapeTypeDrawConnectorShape,					
	XmlShapeTypeDrawMeasureShape,					
	XmlShapeTypeDrawLineShape,						
	XmlShapeTypeDrawPolyPolygonShape,				
	XmlShapeTypeDrawPolyLineShape,					
	XmlShapeTypeDrawOpenBezierShape,				
	XmlShapeTypeDrawClosedBezierShape,				
	XmlShapeTypeDrawGraphicObjectShape,				
	XmlShapeTypeDrawGroupShape,						
	XmlShapeTypeDrawTextShape,						
	XmlShapeTypeDrawOLE2Shape,						
	XmlShapeTypeDrawChartShape,						
	XmlShapeTypeDrawTableShape,						
	XmlShapeTypeDrawPageShape,						
	XmlShapeTypeDrawFrameShape,						
	XmlShapeTypeDrawCaptionShape,					
	XmlShapeTypeDrawAppletShape,					
	XmlShapeTypeDrawPluginShape,					

	XmlShapeTypeDraw3DSceneObject,					
	XmlShapeTypeDraw3DCubeObject,					
	XmlShapeTypeDraw3DSphereObject,					
	XmlShapeTypeDraw3DLatheObject,					
	XmlShapeTypeDraw3DExtrudeObject,				

	XmlShapeTypePresTitleTextShape,					
	XmlShapeTypePresOutlinerShape,					
	XmlShapeTypePresSubtitleShape,					
	XmlShapeTypePresGraphicObjectShape,				
	XmlShapeTypePresPageShape,						
	XmlShapeTypePresOLE2Shape,						
	XmlShapeTypePresChartShape,						
	XmlShapeTypePresTableShape,						
	XmlShapeTypePresOrgChartShape,					
	XmlShapeTypePresNotesShape,						
	XmlShapeTypeHandoutShape,						

	XmlShapeTypeNotYetSet
};

struct XShapeCompareHelper
{
  bool operator()(com::sun::star::uno::Reference < com::sun::star::drawing::XShape > x1, 
	  com::sun::star::uno::Reference < com::sun::star::drawing::XShape > x2 ) const
  {
	return x1.get() < x2.get();
  }
};

struct XShapesCompareHelper
{
  bool operator()(com::sun::star::uno::Reference < com::sun::star::drawing::XShapes > x1, 
	  com::sun::star::uno::Reference < com::sun::star::drawing::XShapes > x2 ) const
  {
	return x1.get() < x2.get();
  }
};

typedef std::map< com::sun::star::uno::Reference < com::sun::star::drawing::XShape >, sal_Int32, XShapeCompareHelper > ShapeIdsMap;

struct ImplXMLShapeExportInfo
{
	rtl::OUString	msStyleName;
	rtl::OUString	msTextStyleName;
	sal_Int32 mnFamily;
	XmlShapeType meShapeType;

	ImplXMLShapeExportInfo() : meShapeType( XmlShapeTypeNotYetSet ), mnFamily( XML_STYLE_FAMILY_SD_GRAPHICS_ID ) {}
};

typedef std::vector< ImplXMLShapeExportInfo > ImplXMLShapeExportInfoVector;

typedef std::map< com::sun::star::uno::Reference < com::sun::star::drawing::XShapes >, ImplXMLShapeExportInfoVector, XShapesCompareHelper > ShapesInfos;

class SvXMLExport;
class SvXMLExportPropertyMapper;
class Matrix3D;
class Vector2D;

class XMLShapeExport : public UniRefBase
{
private:

	SvXMLExport&								rExport;
	UniReference< XMLPropertyHandlerFactory >	xSdPropHdlFactory;
	UniReference< SvXMLExportPropertyMapper >	xPropertySetMapper;
	UniReference< XMLAnimationsExporter >		mxAnimationsExporter;
	sal_Int32									mnNextUniqueShapeId;
	ShapeIdsMap									maShapeIds;
	ShapesInfos									maShapesInfos;
	ShapesInfos::iterator						maCurrentShapesIter;
	sal_Bool									mbExportLayer;
	ImplXMLShapeExportInfoVector				maShapeInfos;
	ImplXMLShapeExportInfoVector::iterator		maCurrentInfo;
	rtl::OUString								msPresentationStylePrefix;

	sal_Bool									mbHandleProgressBar;

	SvXMLExport& GetExport() { return rExport; }
	const SvXMLExport& GetExport() const  { return rExport; }

	UniReference< SvXMLExportPropertyMapper > GetPropertySetMapper() const { return xPropertySetMapper;	}

	const rtl::OUString							msZIndex;
	const rtl::OUString							msEmptyPres;
	const rtl::OUString							msModel;
	const rtl::OUString							msStartShape;
	const rtl::OUString							msEndShape;
	const rtl::OUString							msOnClick;
	const rtl::OUString							msEventType;
	const rtl::OUString							msPresentation;
	const rtl::OUString							msMacroName;
	const rtl::OUString							msLibrary;
	const rtl::OUString							msClickAction;
	const rtl::OUString							msBookmark;
	const rtl::OUString							msEffect;
	const rtl::OUString							msPlayFull;
	const rtl::OUString							msVerb;
	const rtl::OUString							msSoundURL;
	const rtl::OUString							msSpeed;
	const rtl::OUString							msStarBasic;

	rtl::OUStringBuffer	msBuffer;

	void ImpCalcShapeType(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType& eShapeType);

	void ImpExportNewTrans(const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xPropSet, sal_Int32 nFeatures, com::sun::star::awt::Point* pRefPoint);
	void ImpExportNewTrans_GetMatrix3D(Matrix3D& rMat, const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xPropSet);
	void ImpExportNewTrans_DecomposeAndRefPoint(const Matrix3D& rMat, Vector2D& rTRScale, double& fTRShear, double& fTRRotate, Vector2D& rTRTranslate, com::sun::star::awt::Point* pRefPoint);
	void ImpExportNewTrans_FeaturesAndWrite(Vector2D& rTRScale, double fTRShear, double fTRRotate, Vector2D& rTRTranslate, const sal_Int32 nFeatures);
	sal_Bool ImpExportPresentationAttributes( const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xPropSet, const rtl::OUString& rClass );
	void ImpExportText( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape );
	void ImpExportEvents( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape );
	void ImpExportGluePoints( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape );

	void ImpExportGroupShape( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExport3DSceneShape( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportRectangleShape( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportLineShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportEllipseShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportPolygonShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportTextBoxShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportGraphicObjectShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportChartShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportSpreadsheetShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT,	com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportControlShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT,	com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportConnectorShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportMeasureShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT,	com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportOLE2Shape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportPageShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportCaptionShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExport3DShape(const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportFrameShape( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportPluginShape( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );
	void ImpExportAppletShape( const com::sun::star::uno::Reference< com::sun::star::drawing::XShape >& xShape, XmlShapeType eShapeType, sal_Int32 nFeatures = SEF_DEFAULT, com::sun::star::awt::Point* pRefPoint = NULL );

public:
	XMLShapeExport(SvXMLExport& rExp, SvXMLExportPropertyMapper *pExtMapper=0 );
	virtual ~XMLShapeExport();

	void collectShapeAutoStyles( 
		const com::sun::star::uno::Reference < com::sun::star::drawing::XShape >& xShape);

	void exportShape( 
		const com::sun::star::uno::Reference < com::sun::star::drawing::XShape >& xShape,
		sal_Int32 nFeatures = SEF_DEFAULT,
		com::sun::star::awt::Point* pRefPoint = NULL
		);

	void collectShapesAutoStyles( 
		const com::sun::star::uno::Reference < com::sun::star::drawing::XShapes >& xShapes);

	void exportShapes( 
		const com::sun::star::uno::Reference < com::sun::star::drawing::XShapes >& xShapes,
		sal_Int32 nFeatures = SEF_DEFAULT,
		com::sun::star::awt::Point* pRefPoint = NULL
		);

	void seekShapes(
		const ::com::sun::star::uno::Reference< ::com::sun::star::drawing::XShapes >& xShapes ) throw();

	void exportAutoStyles();

	void createShapeId( const com::sun::star::uno::Reference < com::sun::star::drawing::XShape >& xShape );

	sal_Int32 getShapeId( const com::sun::star::uno::Reference < com::sun::star::drawing::XShape >& xShape );

	void setAnimationsExporter( UniReference< XMLAnimationsExporter > xAnimExport ) { mxAnimationsExporter = xAnimExport; }

	UniReference< XMLAnimationsExporter > getAnimationsExporter() const { return mxAnimationsExporter; }

	static SvXMLExportPropertyMapper* CreateShapePropMapper( SvXMLExport& rExport );

	void enableLayerExport( sal_Bool bEnable = sal_True ) { mbExportLayer = bEnable; }
	sal_Bool IsLayerExportEnabled() const { return mbExportLayer; }

	void enableHandleProgressBar( sal_Bool bEnable = sal_True ) { mbHandleProgressBar = bEnable; }
	sal_Bool IsHandleProgressBarEnabled() const { return mbHandleProgressBar; }

	void setPresentationStylePrefix( const rtl::OUString& rPrefix ) { msPresentationStylePrefix = rPrefix; }

	void export3DSceneAttributes( const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xPropSet );

	void export3DLamps( const com::sun::star::uno::Reference< com::sun::star::beans::XPropertySet >& xPropSet );
};

#endif
