

#ifndef _UNIVERSALL_REFERENCE_HXX
#define _UNIVERSALL_REFERENCE_HXX

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif
#ifndef _OSL_INTERLOCK_H_
#include <osl/interlck.h>
#endif

template< class T > class UniReference
{
private:
	T*	mpElement;

public:

	UniReference()
	: mpElement( NULL )
	{}

	inline ~UniReference();

	inline UniReference( const UniReference< T > & rRef );

	inline UniReference( T * pElement );
	 inline UniReference< T > & operator = ( T * pElement );
	 inline UniReference< T > & operator = ( const UniReference< T > & rRef );
	 inline T* operator -> () const;

	inline sal_Bool is() const;

	inline sal_Bool operator == ( const UniReference & rRef ) const;

	inline sal_Bool operator != ( const UniReference & rRef ) const;
	 inline T* get() const;
};

class UniRefBase
{
private:
	 oslInterlockedCount			m_refCount;

public:
	UniRefBase() : m_refCount( 0 )
	{}
	virtual ~UniRefBase();
	 void acquire() { osl_incrementInterlockedCount( &m_refCount ); }
	void release();
};

template< class T >
inline UniReference< T >::UniReference( const UniReference< T > & rRef )
	: mpElement( rRef.mpElement )
{
	if( mpElement )
		mpElement->acquire();
}

template< class T >
inline UniReference< T >::~UniReference()
{
	if( mpElement )
		mpElement->release();
}

template< class T >
inline UniReference< T >::UniReference( T * pElement )
	: mpElement( pElement )
{
	if( mpElement )
		mpElement->acquire();
}

template< class T >
inline UniReference< T > & UniReference< T >::operator = ( T * pElement )
{
	if( pElement )
		pElement->acquire();
	if( mpElement )
		mpElement->release();

	mpElement = pElement;

	return *this;
}

template< class T >
inline UniReference< T > & UniReference< T >::operator = ( const UniReference< T > & rRef )
{
	return operator = ( rRef.mpElement );
}

template< class T >
inline T* UniReference< T >::operator -> () const 
{ 
	return get(); 
}

template< class T >
inline T* UniReference< T >::get () const 
{ 
	return static_cast< T * >( mpElement ); 
}

template< class T >
inline sal_Bool UniReference< T >::is() const
{ 
	return (mpElement != 0); 
}

template< class T >
inline sal_Bool UniReference< T >::operator == ( const UniReference & rRef ) const
{
	return ( mpElement == rRef.mpElement );
}

template< class T >
inline sal_Bool UniReference< T >::operator != ( const UniReference & rRef ) const
{ 
	return ( ! operator == ( rRef ) ); 
}

#endif	
