

#ifndef _XMLOFF_PROPERTYSETMAPPER_HXX
#define _XMLOFF_PROPERTYSETMAPPER_HXX

#ifndef _COM_SUN_STAR_UNO_SEQUENCE_HXX_
#include <com/sun/star/uno/Sequence.hxx>
#endif

#ifndef _COM_SUN_STAR_UNO_ANY_HXX_
#include <com/sun/star/uno/Any.hxx>
#endif

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

#ifndef _UNIVERSALL_REFERENCE_HXX
#include <xmloff/uniref.hxx>
#endif

#ifndef _XMLOFF_PROPMAPPINGTYPES_HXX
#include <xmloff/maptype.hxx>
#endif

#ifndef _XMLOFF_XMLTYPES_HXX
#include <xmloff/xmltypes.hxx>
#endif

#ifndef _XMLOFF_PROPERTYHANDLERFACTORY_HXX
#include <xmloff/prhdlfac.hxx>
#endif

class SvXMLUnitConverter;
class XMLPropertyHandler;

struct XMLPropertySetMapperEntry_Impl
{
	::rtl::OUString sXMLAttributeName;
	::rtl::OUString sAPIPropertyName;
	sal_uInt16 nXMLNameSpace;
	sal_Int32  nType;
	sal_Int16  nContextId;
	const XMLPropertyHandler *pHdl;

	XMLPropertySetMapperEntry_Impl(
		const XMLPropertyMapEntry& rMapEntry,
		const UniReference< XMLPropertyHandlerFactory >& rFactory );

	XMLPropertySetMapperEntry_Impl(
		const XMLPropertySetMapperEntry_Impl& rEntry );
};

class XMLPropertySetMapper : public UniRefBase
{
	::std::vector< XMLPropertySetMapperEntry_Impl > aMapEntries;
	::std::vector< UniReference < XMLPropertyHandlerFactory > > aHdlFactories;

public:

	XMLPropertySetMapper(
			const XMLPropertyMapEntry* pEntries,
			const UniReference< XMLPropertyHandlerFactory >& rFactory );
	virtual ~XMLPropertySetMapper();

	void AddMapperEntry( const UniReference < XMLPropertySetMapper >& rMapper );

	const sal_Int32	GetEntryCount() const { return aMapEntries.size(); }

	const sal_uInt32 GetEntryFlags( sal_Int32 nIndex ) const
	{
		return aMapEntries[nIndex].nType & ~MID_FLAG_MASK;
	}

	const sal_uInt32 GetEntryType( sal_Int32 nIndex,
					   sal_Bool bWithFlags = sal_True ) const
	{
		sal_uInt32 nType = aMapEntries[nIndex].nType;
		if( !bWithFlags )
			nType = nType & MID_FLAG_MASK;
		return nType;
	}

	const sal_uInt16 GetEntryNameSpace( sal_Int32 nIndex ) const
	{
		return aMapEntries[nIndex].nXMLNameSpace;
	}

	const ::rtl::OUString& GetEntryXMLName( sal_Int32 nIndex ) const
	{
		return aMapEntries[nIndex].sXMLAttributeName;
	}

	const ::rtl::OUString& GetEntryAPIName( sal_Int32 nIndex ) const
	{
		return aMapEntries[nIndex].sAPIPropertyName;
	}

	const sal_Int16 GetEntryContextId( sal_Int32 nIndex ) const
	{
		return aMapEntries[nIndex].nContextId;
	}

	const sal_Int32 GetEntryIndex( sal_uInt16 nNamespace,
					   const ::rtl::OUString& rStrName,
					   sal_Int32 nStartAt = -1 ) const;

	const XMLPropertyHandler* GetPropertyHandler( sal_Int32 nIndex ) const
	{
		return aMapEntries[nIndex].pHdl;
	}

	virtual sal_Bool exportXML( ::rtl::OUString& rStrExpValue,
					const XMLPropertyState& rProperty,
					const SvXMLUnitConverter& rUnitConverter ) const;
	virtual sal_Bool importXML( const ::rtl::OUString& rStrImpValue,
					XMLPropertyState& rProperty,
					const SvXMLUnitConverter& rUnitConverter ) const;

	sal_Int32 FindEntryIndex( const sal_Char* sApiName,
				  sal_uInt16 nNameSpace,
				  const ::rtl::OUString& sXMLName ) const;

	sal_Int32 FindEntryIndex( const sal_Int16 nContextId ) const;
};

#endif 
